*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                            PROGRAM CETRAN
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* S. Lohrengel (Mars 1998) / D.Martin (Juin 1998)
*
* Calcul Elements Finis de la solution de
*          Rot_{-Beta}(Rot_Beta(E)) -  Grad_Beta(Div_Beta(eps^2*E) = F  
* avec des condition naturelle et essentielle homogenes au bord:
*          Div_Beta(eps*E) = 0  et  E ^ nu = 0  
* et conditions de transmission a l'interface
*          [ E ^ nu ] =0     et  [ eps E ]. nu = 0
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*     Declarations des super-tableaux
*
      CHARACTER AST
      INTEGER   MOTMXR,NBCHMX
      REAL      RST
      PARAMETER (MOTMXR=5000000,NBCHMX=80000)
      COMMON/     /RST(MOTMXR) /STCHAR/AST(NBCHMX)
*
*     Variables locales pour les constantes
      CHARACTER TYPE*10,CARAC
      INTEGER   IBID
      REAL      EPSP,EPSP2,EPSM,EPSM2,BETA,BETA2,SHIFT
      COMPLEX   CBID
*-----------------------------------------------------------------------
*     Initialisation de l'allocation dynamique et
*     creation des structures de donnees.
      CALL INITIE (NBCHMX,MOTMXR)
*
*     Lecture du fichier de geometrie (maillage)
      CALL LCGEOM
*
*     Lecture des directives de definition des inconnues variationnelles
*     et de construction des termes elements-finis
      CALL LCDIRE
*
*     Lecture des donnees scalaires et tableaux
      CALL LCDONN
*
*     Numerotations globales en noeuds (pour toutes les interpolations)
      CALL NUMNEU
*
*     Distribution des coordonnees sur les domaines
      CALL CPTDOM
*-----------------------------------------------------------------------
*     Activation des constantes du probleme
*
      CALL GETCST ('EPSIL+',TYPE,IBID,EPSP,CBID,CARAC)
      CALL GETCST ('EPSIL-',TYPE,IBID,EPSM,CBID,CARAC)
      CALL GETCST ('BETA',TYPE,IBID,BETA,CBID,CARAC)
      CALL GETCST ('SHIFT',TYPE,IBID,SHIFT,CBID,CARAC)
      BETA2=BETA*BETA
      EPSP2=EPSP*EPSP
      EPSM2=EPSM*EPSM
*
      CALL PUTCST ('-BETA','REEL',IBID,-BETA,CBID,CARAC)
      CALL PUTCST ('BETA2','REEL',IBID,BETA2,CBID,CARAC)
      CALL PUTCST ('-EPBTA','REEL',IBID,-EPSP2*BETA,CBID,CARAC)
      CALL PUTCST ('-EMBTA','REEL',IBID,-EPSM2*BETA,CBID,CARAC)
      CALL PUTCST ('EPBTA2','REEL',IBID,EPSP2*BETA2,CBID,CARAC)
      CALL PUTCST ('EMBTA2','REEL',IBID,EPSM2*BETA2,CBID,CARAC)
      CALL PUTCST ('EPSP2','REEL',IBID,EPSP2,CBID,CARAC)
      CALL PUTCST ('EPSM2','REEL',IBID,EPSM2,CBID,CARAC)
*
*     Remplissage du tableau associe (pour FCTRM)
*
      CALL PUTTAB ('TABMAT',1,1,'REEL',IBID,BETA,CBID,CARAC)
      CALL PUTTAB ('TABMAT',1,2,'REEL',IBID,SHIFT,CBID,CARAC)
      CALL PUTTAB ('TABMAT',1,3,'REEL',IBID,EPSP,CBID,CARAC)
      CALL PUTTAB ('TABMAT',1,4,'REEL',IBID,EPSM,CBID,CARAC)
*
*-----------------------------------------------------------------------
*     Calcul des termes elements-finis sur les domaines
*
      CALL CALKEF
*-----------------------------------------------------------------------
*     Assemblage des matrices sur Omega+ et Omega-
*     Somme(Omega) ( Rot_beta(u).Rot_beta(v) 
*                  + Eps^2 Div_beta(u).Div_beta(v) 
*                  + SHIFT  u.v ) dx
      CALL ASMTRM ('ADEUV+',1,0)
      CALL ASMTRM ('ADEUV-',1,0)
      CALL DSMTRM ('ADEUVV',10,0)

*-----------------------------------------------------------------------
*     Calcul du 2nd second membre sur Omega+ et Omega-, puis assemblage 
*     Somme(Omega)  F.V = MASSE*DONNEE 
*     solution reguliere
*
      CALL MATVEC ('MASSE+',1,'DONNE+',1,'SCMBE+',1,0)
      CALL MATVEC ('MASSE-',1,'DONNE-',1,'SCMBE-',1,0)
      CALL DSVTRM ('SCMBRE',10,0)
*
*     Prise en compte des conditions essentielles de Dirichlet
*
      CALL CDESSE ('ELEC+E',1,'ADEUVV',10,'SCMBRE',10)
      CALL CDESSE ('ELEC+O',1,'ADEUVV',10,'SCMBRE',10)
      CALL CDESSE ('ELEC+N',1,'ADEUVV',10,'SCMBRE',10)
      CALL CDESSE ('ELEC-E',1,'ADEUVV',10,'SCMBRE',10)
      CALL CDESSE ('ELEC-O',1,'ADEUVV',10,'SCMBRE',10)
      CALL CDESSE ('ELEC-S',1,'ADEUVV',10,'SCMBRE',10)
*
*     Prise en compte des conditions essentielles de Transmission
*
      CALL CDTRAN ('EVECTN',1,'ADEUVV',10,'SCMBRE',10)
      CALL CDTRAN ('ESCALN',1,'ADEUVV',10,'SCMBRE',10)
*
*-----------------------------------------------------------------------
*     Factorisation et resolution
*
      CALL FALU   ('ADEUVV',10,' ','ADEUVV',20,0)
      CALL FASV   ('ADEUVV',20,'SCMBRE',10,'ELFSOL',10,0)
      CALL TBTUER ('ADEUVV',10)
      CALL TBTUER ('ADEUVV',20)
*  
*     Comparaison avec la solution exacte sur chaque sous-domaine
*
      CALL SDEKAM ('ELFSOL',10,'E+','ELFSO+',1,0)
      CALL DFTERM ('ELFSO+',1,'SOLEX+',1,10)
      CALL SDEKAM ('ELFSOL',10,'E-','ELFSO-',1,0)
      CALL DFTERM ('ELFSO-',1,'SOLEX-',1,10)
*
*-----------------------------------------------------------------------
*
*     Sorties pour post-traitement graphique Grame ou Matlab
*
      CALL M2UOUV ('Dessin1','2D')
      CALL M2UENR ('Dessin1','Terme E1+','1')
      CALL M2URSD ('Dessin1','ELFSO+',1,'OMEGA+')
      CALL M2UOUV ('Dessin2','2D')
      CALL M2UENR ('Dessin2','Terme E1-','1')
      CALL M2URSD ('Dessin2','ELFSO-',1,'OMEGA-')
      CALL M2UOUV ('Dessin3','2D')
      CALL M2UENR ('Dessin3','Terme E2+','2')
      CALL M2URSD ('Dessin3','ELFSO+',1,'OMEGA+')
      CALL M2UOUV ('Dessin4','2D')
      CALL M2UENR ('Dessin4','Terme E2-','2')
      CALL M2URSD ('Dessin4','ELFSO-',1,'OMEGA-')
      CALL M2UGP1 ()
*      CALL M2UGEO ()
      CALL STIMPR (9)
*
      CALL BYE('C''est fini!')
      END
