*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      PROGRAM DELTAU
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Decembre 2001)
* Derniere modification : D.Martin (07 Decembre 2001)
*
* Resolution de \Delta U = F avec conditions de Dirichlet ou Neumann
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
*     Declarations globales des super-tableaux
*
      CHARACTER     AST
      INTEGER       MOTMXR,NBCHMX
      REAL          RST
      PARAMETER    (MOTMXR=134217727,NBCHMX=80000)
      COMMON/      /RST(MOTMXR) /STCHAR/AST(NBCHMX)
*
*     Variables locales
      INTEGER    EBID,DOMAIN,DOMAIX,IMPASS,IMPSYS,IMPRES,IMPSOL,IDO
     &          ,MXITER
      REAL       RBID,EPSIL
      COMPLEX    CBID
      CHARACTER  ABID
      PARAMETER  (DOMAIX=6)
      CHARACTER  TYPE*1,SOLEXA*6,S2MEMB*11
      CHARACTER*3  CL(DOMAIX)
      CHARACTER*10 CONLIM(DOMAIX),RESOL*10
*
*-----------------------------------------------------------------------
*
*     Initialisation de l'allocation dynamique et
*     creation des structures de donnees.
      CALL INITIE (NBCHMX,MOTMXR)
*
*     Lecture du fichier de geometrie (maillage)
      CALL LCGEOM
*
*     Lecture des directives de definition des inconnues variationnelles
*     et de construction des termes elements-finis
      CALL LCDIRE
*
*     Lecture des donnees scalaires
      CALL LCDONN
      CALL CHRONO (' Lecture des donnees',6)
*
*     Numerotations globales en noeuds
      CALL NUMNEU
      CALL CHRONO (' Renumerotation',6)
*
*     Distribution des coordonnees sur les domaines
      CALL CPTDOM
*
*     Calcul des termes elements-finis sur les domaines
      CALL CALKEF
      CALL CHRONO (' Calculs Elements Finis',6)
*
      CALL T2TERM ('Gragra',0,'+','Masse',0,'Matris',0,0)
*
*     Constantes pour impression
      CALL GETCST ('Imp_ass',TYPE,IMPASS,RBID,CBID,ABID)
      CALL GETCST ('Imp_sys',TYPE,IMPSYS,RBID,CBID,ABID)
      CALL GETCST ('Imp_res',TYPE,IMPRES,RBID,CBID,ABID)
      CALL GETCST ('Imp_sol',TYPE,IMPSOL,RBID,CBID,ABID)
*
*     Nombre de bords du domaine et conditions aux limites
*
      CALL GETCST ('Domaines',TYPE,DOMAIN,RBID,CBID,ABID)
      IF (DOMAIN.GT.DOMAIX)
     &    CALL BAISE ('Nombre de domaines non prevu.')
      DO 1 IDO=1,DOMAIN
         CL(IDO)='CL'
         CALL ENCLER (IDO,CL(IDO)(3:3))
         CALL GETCST (CL(IDO),TYPE,EBID,RBID,CBID,CONLIM(IDO))
1     CONTINUE
*
*     Second Membre de l'equation
*
      CALL GETCST ('Calcul_2membre',TYPE,EBID,RBID,CBID,S2MEMB)
      IF (S2MEMB(1:6).EQ.'Interp')
     &   CALL MATVEC ('Masse',0,'Fval',0,'SecmbF',0,IMPASS)
      CALL ASMAKE ('SecmbF',0,'Secmbr',0)
*
      CALL TBTUER ('Gragra',0)
      CALL TBTUER ('Masse',0)
*
*     Second Membre issu de conditions de Neuman
*
      DO 10 IDO=1,DOMAIN
         IF (CONLIM(IDO)(1:8).EQ.'NeumanNH') THEN
            IF (S2MEMB(1:6).EQ.'Interp') THEN
            CALL MATVEC ('Masse',IDO,'dudn',IDO,'SecmbN',IDO,IMPASS)
            CALL TBTUER ('Masse',IDO)
            ENDIF
            CALL ASMAKE ('SecmbN',IDO,'Secmbr',0)
         ENDIF
10    CONTINUE
*
*     Assemblage du second membre
      CALL ASVTRM ('Secmbr',0,IMPASS)
*
*     Conditions de Dirichlet
*
      DO 20 IDO=1,DOMAIN
         IF (CONLIM(IDO)(1:9).EQ.'Dirichlet') THEN
            CALL CDESSE ('Dirich',IDO,'Matris',0,'Secmbr',0)
         ENDIF
20    CONTINUE
*
      CALL CHRONO (' Constitution du Systeme',6)
*
*     Methode de resolution
*
      CALL GETCST ('Resolution',TYPE,EBID,RBID,CBID,RESOL)
      IF (RESOL(1:8).EQ.'Choleski') THEN
         CALL FALLT  ('Matris',0,'Matfac',0,IMPRES)
         CALL FASV   ('Matfac',0,'Secmbr',0,'Soluti',0,IMPRES)
      ELSEIF (RESOL(1:5).EQ.'Gauss') THEN
         CALL FALU   ('Matris',0,'S','Matfac',0,IMPRES)
         CALL FASV   ('Matfac',0,'Secmbr',0,'Soluti',0,IMPRES)
      ELSEIF (RESOL(1:7).EQ.'Gradient') THEN
         CALL GETCST ('Epsilon_GC',TYPE,EBID,EPSIL,CBID,ABID)
         CALL GETCST ('Max_Iter_GC',TYPE,MXITER,RBID,CBID,ABID)
         CALL GCJ    ('Matris',0,'Secmbr',0,' ',0,EPSIL,MXITER
     &               ,'Soluti',0,IMPRES,*99991)
      ELSE
         CALL BAISE  ('Methode de resolution non prevue.')
      ENDIF
      CALL CHRONO (' Resolution Systeme',6)
      CALL STIMPR (9)
*
*     Comparaison avec une solution exacte
*
      CALL GETCST ('Solution_exacte',TYPE,EBID,RBID,CBID,SOLEXA)
      IF (SOLEXA(1:1).NE.' ')
     &   CALL DFTERM (SOLEXA,0,'Soluti',0,IMPSOL)
      CALL BYE (' That''s all Folks')
*
99991 CALL BAISE('Le Gradient Conjugue n''a pas converge')
      END

