      SUBROUTINE DESFIC (NBFIMX,NBLMAX,IFICH,NUMREP,MAILLA,ARETES,
     &                   LARTRA,X,Y,Z,ZMIN,N,NBEL,NBCOL,PASCOL,VALPRJ)

C- Fonction : Dessin du fichier IFICH en representation NUMREP
C-
C- Entree :
C-    NBFIMX : Nombre maximum de fichiers   
C-    NBLMAX : Nombre maximum d'elements
C-    IFICH  : Indice de fichiers
C-    NUMREP : Type de representation
C-    MAILLA : Maillage ou non
C-    ARETES : Tableau contenant le debut et la fin des aretes.
C-    LARTRA : Largeur du trace en NUMREP=2
C-    X,Y,Z  : Tableaux des coordonnees des sommets
C-    ZMIN   : Min des Z
C-    N      : Tableau contenant les noeuds
C-    NBEL   : Nombre d'elements
C-    NBCOL  : Nombre de couleurs possibles. 0:noir, NBCOL+1:blanc
C-    PASCOL : DeltaZ correspondant a un changement de couleur
C-    VALPRJ : Tableau contenant les parametres de projection 3D en 2D
C-
C- Sortie : -
C-
C- Local :
C-    XP,YP,ZP : Coordonnees des triangles
C-    NBARET   : Nombre d'aretes
C-    TRIARE   : Tableau contenant les 3 aretes d'un triangle
C-
C- Derniere modif : 29 NOV 2000

      INTEGER NBFIMX,NBLMAX,IFICH,NUMREP,LARTRA,N(NBFIMX,NBLMAX,*)
      INTEGER NBEL(*),NBCOL,ARETES(*),NBARET,TRIARE(6),I,J,K,L
      REAL X(NBFIMX,*),Y(NBFIMX,*),Z(NBFIMX,*),ZMIN,PASCOL
      REAL VALPRJ(*),XP(3),YP(3),ZP(3)
      LOGICAL MAILLA

C- Initialisation du nombre d'aretes
      NBARET=0

C- Ouverture du buffer 
      CALL PGBBUF ()      
      
C- Creation des triangles. 
      DO 50 I=1,NBEL(IFICH)
         DO 10 J=1,3
            XP(J) = X(IFICH,N(IFICH,I,J))
            YP(J) = Y(IFICH,N(IFICH,I,J))
            ZP(J) = Z(IFICH,N(IFICH,I,J))
10       CONTINUE

C- On dessine les triangles en couleur en NUMREP>1
         IF (NUMREP.GT.1) 
     &      CALL TRINIV (NUMREP,LARTRA,XP,YP,ZP,ZMIN,PASCOL,VALPRJ)

C- On trace le maillage ou les bords des isovaleurs planes        
C- en blanc pour NUMREP=1 et 2, en noir pour NUMREP 3 e t 4
C- On trace toujours en NUMREP=1 sinon ya rien a voir 
         IF (MAILLA.OR.NUMREP.EQ.1) THEN
            IF (NUMREP.LT.3) THEN
               CALL PGSCI (NBCOL+1)
            ELSE
               CALL PGSCI (0)
            ENDIF
C- On passe en pointille en NUMREP>1
            IF (NUMREP.GT.1) CALL PGSLS (4)
            CALL PGSFS (2)
            CALL PGSLW (1)
            CALL DESCAS (NUMREP,3,XP,YP,ZP,VALPRJ)
            IF (NUMREP.GT.1) CALL PGSLS (1) 
         ENDIF

C- On cherche les aretes 
         IF (.NOT.MAILLA.AND.NUMREP.EQ.2) THEN
            CALL CREARE (N(IFICH,I,1),N(IFICH,I,2),N(IFICH,I,3),TRIARE)
            DO 40 J=1,3
C- On teste chaque nouvelle arete. 
               DO 30 K=1,NBARET/2
                  IF (TRIARE(2*J-1).EQ.ARETES(2*K-1).AND.TRIARE(2*J)
     &                             .EQ.ARETES(2*K)) THEN
C- L'arete ARETES(2*K-1,2*K) existe deja. On la supprime puisque
C- une arete ne peut etre utilisee que 1 ou 2 fois.
                     DO 20 L=K,NBARET/2
                        ARETES(2*L-1)=ARETES(2*L+1)
                        ARETES(2*L)=ARETES(2*L+2)
20                   CONTINUE
                     NBARET=NBARET-2             
                     GOTO 40
                  ENDIF
30             CONTINUE
C- Si on est arrive la, l'arete est nouvelle. Donc on l'ajoute
               ARETES(NBARET+1)=TRIARE(2*J-1)
               ARETES(NBARET+2)=TRIARE(2*J)
               NBARET=NBARET+2
40          CONTINUE
         ENDIF
50    CONTINUE

C- On repasse en epaisseur 1 sinon le repcol est en LARTRA
      CALL PGSLW (1)

C- On trace les aretes
      IF (.NOT.MAILLA.AND.NUMREP.EQ.2) THEN
         CALL PGSCI (NBCOL+1)
         DO 60 I=1,NBARET/2
            J=ARETES(2*I-1)
            K=ARETES(2*I)
            CALL PGMOVE (X(IFICH,J),Y(IFICH,J))
            CALL PGDRAW (X(IFICH,K),Y(IFICH,K))
60       CONTINUE
      ENDIF

C- Fermeture du buffer (i.e affichage)
      CALL PGEBUF ()

      END
