      SUBROUTINE TRINIV (NUMREP,LARTRA,XP,YP,ZP,ZMIN,PASCOL,VALPRJ)

C- Fonction : Calcul et tracage des courbes de niveaux  
C-
C- Entree :
C-    NUMREP : Type de representation
C-    LARTRA : Largeur du trace en NUMREP=2
C-    XP,YP,ZP : Coordonnees des sommets d'un triangle
C-    ZMIN : Min et max des Z
C-    PASCOL : DeltaZ correspondant a un changement de couleur
C-    VALPRJ : Tableau contenant les parametres de projection 3D en 2D
C-
C- Sortie : -
C-
C- Local :
C-    NCOLMX : 3 * Nombre maximum de couleurs representables (3 cotes)
C-    NBPTS   : Nombre de points trouves
C-    X,Y,Z,C : Coordonnees et couleurs des points trouves a l'interieur 
C-              du triangle
C-    XT,YT,ZT,CT : Variables de tri
C-    XPL,YPL,ZPL : Coordonnees des sommets du polygone
C-    S : Couleurs correspondants aux 3 sommets
C-    S1,S2 : Couleurs des sommets du cote considere
C-    NBPL : Nombre de points d'un polygone dans le triangle.
C-           Maximum= 2 points trouves +  1 sommet + 2 points du polygone
C-           precedent = 5
C- 
C- Derniere modif : 21 JUN 2000
      
      REAL XP(*),YP(*),ZP(*),ZMIN,PASCOL,VALPRJ(*)
      INTEGER NUMREP,LARTRA,NCOLMX,I,J,K,NBPTS
      PARAMETER (NCOLMX = 768)
      REAL X(NCOLMX),Y(NCOLMX),Z(NCOLMX)
      REAL XT,YT,ZT,XPL(5),YPL(5),ZPL(5)
      INTEGER S(3),S1,S2,C(NCOLMX),CT,NBPL
 
C- Epaisseur de la ligne en 200eme/inch
      CALL PGSLW (LARTRA)
C- On cherche les couleurs correspondants aux 3 sommets : S
      DO 20 I=1,3
20    S(I)= INT((ZP(I) - ZMIN) / PASCOL) + 1
      
C- On cherche les coordonnnes Z des points se trouvant entres S1 et S2
C- On prend successivement les cotes 1-2, 1-3 et 2-3
      NBPTS=0
      DO 40 I=1,2
      DO 40 J=I+1,3
         IF (S(I).EQ.S(J)) GOTO 40
C- On prend S1 < S2
         S1=MIN(S(I),S(J))
         S2=MAX(S(I),S(J))
         DO 30 K=S1,S2-1
            NBPTS=NBPTS+1
            IF (NBPTS.GT.1900) STOP 'NBPTS > NBPMAX dans TRINIV'
            Z(NBPTS)=K*PASCOL + ZMIN
            C(NBPTS)=K
            X(NBPTS)=(XP(J)*(ZP(I)-Z(NBPTS)) + XP(I)*(Z(NBPTS)-ZP(J))) 
     &             /(ZP(I)-ZP(J))   
            Y(NBPTS)=(YP(J)*(ZP(I)-Z(NBPTS)) + YP(I)*(Z(NBPTS)-ZP(J)))
     &            /(ZP(I)-ZP(J))
30       CONTINUE
40    CONTINUE 

      IF (NUMREP.GT.2) GOTO 60
C- Isovaleurs
C- Si C(J)=C(I) on trace une ligne
      DO 50 I=1,NBPTS-1
      DO 50 J=I+1,NBPTS
         IF (C(I).EQ.C(J)) THEN
            CALL PGSCI (C(I))
            CALL PGMOVE (X(I),Y(I))
            CALL PGDRAW (X(J),Y(J))
         ENDIF
50    CONTINUE
      RETURN

C- Isotriangles
60    CALL PGSFS (1)
C- Si on a pas trouve de points on trace directement le triangle
      IF (NBPTS.EQ.0) THEN
         CALL PGSCI (S(1))
         CALL DESCAS (NUMREP,3,XP,YP,ZP,VALPRJ)
         RETURN
      ENDIF

C- On ajoute les 3 sommets aux points trouves
      DO 70 I=1,3
         X(NBPTS+I)=XP(I)
         Y(NBPTS+I)=YP(I)
         Z(NBPTS+I)=ZP(I)
         C(NBPTS+I)=S(I)   
70    CONTINUE
      NBPTS=NBPTS+3

C- On trie les points par C croissants puis Z croissants
80    CONTINUE  
      DO 90 I=1,NBPTS-1
         IF (C(I+1).LT.C(I).OR.(C(I+1).EQ.C(I).AND.Z(I+1).LT.Z(I))) THEN
            XT=X(I)
            YT=Y(I)
            ZT=Z(I)
            CT=C(I) 
            X(I)=X(I+1)
            Y(I)=Y(I+1)
            Z(I)=Z(I+1)
            C(I)=C(I+1)
            X(I+1)=XT
            Y(I+1)=YT
            Z(I+1)=ZT
            C(I+1)=CT
            GOTO 80
         ENDIF
90    CONTINUE

C- On part du premier point
      NBPL=1
      XPL(1)=X(1)
      YPL(1)=Y(1)
      ZPL(1)=Z(1)
      DO 100 I=2,NBPTS
         IF (C(I).EQ.C(I-1)) THEN
            NBPL=NBPL+1
            XPL(NBPL)=X(I)   
            YPL(NBPL)=Y(I)
            ZPL(NBPL)=Z(I)
         ELSE
C- Quand on change de couleur on dessine le polygone precedent
C- puis on initialise le nouveau polygone par les 2 dernieres
C- valeurs du polygone precedent
            CALL DESPOL(NUMREP,C(I-1),XPL,YPL,ZPL,NBPL,VALPRJ)
            NBPL=3
            XPL(1)=X(I-2)
            YPL(1)=Y(I-2)
            ZPL(1)=Z(I-2)
            XPL(2)=X(I-1)
            YPL(2)=Y(I-1)
            ZPL(2)=Z(I-1)
            XPL(3)=X(I)
            YPL(3)=Y(I)
            ZPL(3)=Z(I)
         ENDIF
100     CONTINUE
C- On trace le dernier polygone de la couleur du dernier point
        CALL DESPOL(NUMREP,C(NBPTS),XPL,YPL,ZPL,NBPL,VALPRJ)

        END 
