*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
       SUBROUTINE GCANAL (TYNOYO,K,B,CORM,CORP,E,D1EM,D1EP,D2EMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : O.DeBayser (Septembre 1995)
* Derniere modification : D.Martin (7 Mai 2002)
* Version 1.0.1
*  
* Solution Elementaire au point 'CORM' de l'Equation de HELMHOLTZ
* dans une bande de R2 au facteur (i/4) pres, par serie de Fourier
* 
* -Bande d'axe Ox et de largeur: [-B,+B] en y.
*  
*-- Arguments d'entree -- 
* TYNOYO: type du noyau de Green a calculer
* K     : constante "nombre d'ondes"
* B     : demi-largeur du domaine "bande"  
* CORM  : coordonnees du point source  
* CORP  : coordonnees du point image
*-- Arguments de sortie --
* E     : E(M,P) 
* D1EM  : derivees de E par rapport aux coordonnees de M
* D1EP  : derivees de E par rapport aux coordonnees de P
* D2EMP : derivees secondes croisees dans l'ordre suivant  
*         d2E/dxmdxp, d2E/dxmdyp, d2E/dymdxp, d2E/dymdyp
*  
*-- Remarques: La multiplication par i/4 n'est pas effectuee dans ce S-P 
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*
*     RE-affecter toute les instructions concernees en DOUBLE PREC ..
*     Cf >ftnchek gcanal.f sur sun
*
      CHARACTER*(*) TYNOYO 
      REAL K,B
      COMPLEX E,D1EM,D1EP,D2EMP  
      DIMENSION D1EM(*),D1EP(*),D2EMP(*),CORP(*),CORM(*)
*
*>>>>>>>>>>>>
      COMPLEX TOTO,F
*>>>>>>>>>>>>
      DOUBLE PRECISION SRV,SRVP1,TET,TETM,TETP,CM,CP,SM,SP
      DOUBLE PRECISION BP,BPP,BPM,BPS
      REAL V,RR,RPP,RPM,RPMP,PPIS2B
      COMPLEX EP,EE  
      PARAMETER  ( NMAX = 150 ) 
      DATA EPSIL,PI /1.E-8,3.141549/
*
      E=0.
      D1EP(1) = 0.
      D1EP(2) = 0.
      D1EM(1) = 0. 
      D1EM(2) = 0.
      D2EMP(1)= 0.
      D2EMP(2)= 0.
      D2EMP(3)= 0.
      D2EMP(4)= 0.
*
      X =CORP(1)-CORM(1) 
      YJ=CORP(2)-CORM(2)
      RJ=X*X+YJ*YJ
      IF (RJ.LT.1.E-06) GO TO 999
      COUPUR=2.*K*B/PI
*>>>>>>>>>>>>
      WRITE (*,*) '*Gcanal* Coupure=',COUPUR
      TOTO=0.
*>>>>>>>>>>>>
*
      DO 10 IP=1,INT(COUPUR)
         PPIS2B=(REAL(IP)*PI)/(2.*B)
         V=K*K-PPIS2B*PPIS2B
         V=SQRT(V)
         TETM=PPIS2B*(CORM(2)+B)
         TETP=PPIS2B*(CORP(2)+B)
         CM=DCOS(TETM)
         CP=DCOS(TETP)
         RR=CM*CP/B
         SM=PPIS2B*DSIN(TETM)
         SP=PPIS2B*DSIN(TETP)
         RPP=-CM*SP/B
         RPM=-SM*CP/B
         RPMP=SM*SP/B
         IF (X.GT.0.) THEN
            EE=(1./V)*EXP(CMPLX(0.,+V*X))/CMPLX(0.,2.)
            EP=+EXP(CMPLX(0.,+V*X))/2.
         ELSE
            EE=(1./V)*EXP(CMPLX(0.,-V*X))/CMPLX(0.,2.)
            EP=-EXP(CMPLX(0.,-V*X))/2.
         ENDIF
         E=E+RR*EE
         D1EP(1)=D1EP(1)+EP*RR
         D1EP(2)=D1EP(2)+EE*RPP
         D1EM(1)=D1EM(1)-EP*RR
         D1EM(2)=D1EM(2)+EE*RPM
         IF (TYNOYO(:3).EQ.'FOU'.OR.TYNOYO(:3).EQ.'NEU') THEN  
            D2EMP(1)=D2EMP(1)+V*V*EE*RR
            D2EMP(2)=D2EMP(2)-EP*RPP
            D2EMP(3)=D2EMP(3)+EP*RPM
            D2EMP(4)=D2EMP(4)+EE*RPMP
         END IF
*>>>>>>>>>>>>
      TOTO=TOTO-PPIS2B*PPIS2B*EE*RR
*>>>>>>>>>>>>
10    CONTINUE
*
      IP=INT(COUPUR)+1
      PIS2B =PI/(2.*B)
      PPIS2B=REAL(IP)*PIS2B
      V=PPIS2B*PPIS2B-K*K
      V=SQRT(V)
      XTOL=0.1/V
      IF (X.GT.-XTOL.AND.X.LT.XTOL) THEN
*     Calcul avec le developpement en serie  transforme au voisinage de "0"
         SRV=1./(4.*V*B)
         TETM=PIS2B*(CORM(2)+B)
         TETP=PIS2B*(CORP(2)+B)
         TET =REAL(IP)*(TETM+TETP)
         CM  =DCOS(TET)
         SM  =PPIS2B*DSIN(TET)
         TET =REAL(IP)*(TETM-TETP)
         CP  =DCOS(TET)
         SP  =PPIS2B*DSIN(TET)
*>>>>>>>>>>>>
         F=E
      WRITE (*,*) '*Gcanal* 1)',F
         F=F-SRV*(CM+CP)
      WRITE (*,*) '*Gcanal* 2)',F
*>>>>>>>>>>>>
*        E=E-SRV*(CM+CP)
*        D1EP(2)=D1EP(2)-SRV*(-SM+SP)
*        D1EM(2)=D1EM(2)-SRV*(-SM-SP)
*        IF (TYNOYO(:3).EQ.'FOU'.OR.TYNOYO(:3).EQ.'NEU') THEN  
*           D2EMP(4)=D2EMP(4)-SRV*(-CM+CP)*PPIS2B*PPIS2B
*        END IF
         CALL QCALC(IP,TETM,TETP,BP,BPP,BPM,BPS)
         PPIS2B=REAL(IP+1)*PIS2B
         V=PPIS2B*PPIS2B-K*K
         V=SQRT(V)
         SRVP1=1./(2.*V*B)
*>>>>>>>>>>>>
         F=F+SRVP1*BP
      WRITE (*,*) '*Gcanal* 3)',F
*>>>>>>>>>>>>
*        E=E+SRVP1*BP
*        D1EP(2)=D1EP(2)+SRVP1*BPP*PIS2B
*        D1EM(2)=D1EM(2)+SRVP1*BPM*PIS2B
*        IF (TYNOYO(:3).EQ.'FOU'.OR.TYNOYO(:3).EQ.'NEU') THEN  
*           D2EMP(4)=D2EMP(4)+SRVP1*BPS*PIS2B*PIS2B
*        END IF
      DO 15 IP=INT(COUPUR)+2,NMAX
         PPIS2B=REAL(IP)*PIS2B
         V=PPIS2B*PPIS2B-K*K
         V=SQRT(V)
         SRV=1./(2.*V*B)
         CALL QCALC(IP,TETM,TETP,BP,BPP,BPM,BPS)
         PPIS2B=REAL(IP+1)*PIS2B
         V=PPIS2B*PPIS2B-K*K
         V=SQRT(V)
         SRVP1=1./(2.*V*B)
*>>>>>>>>>>>>
         F=F+BP*(SRVP1-SRV)
      WRITE (*,*) '            3)',F,(SRVP1-SRV),1./(IP*(IP+1))
     &           ,(SRVP1-SRV)*(IP*(IP+1))
*        on remarque que (SRVP1-SRV)*(IP*(IP+1)) tend vers 1/pi.
*>>>>>>>>>>>>
*        E=E+BP*(SRVP1-SRV)
*        D1EP(2)=D1EP(2)+BPP*(SRVP1-SRV)*PIS2B
*        D1EM(2)=D1EM(2)+BPM*(SRVP1-SRV)*PIS2B
*        IF (TYNOYO(:3).EQ.'FOU'.OR.TYNOYO(:3).EQ.'NEU') THEN  
*           D2EMP(4)=D2EMP(4)+BPS*(SRVP1-SRV)*PIS2B*PIS2B
*        END IF
         IF (ABS(BP*(SRVP1-SRV)).LE.EPSIL) GOTO 16
15    CONTINUE
*     ENDIF
      WRITE (*,*) '*Gcanal* NON CONVERGENCE de la serie transforme!'
16    CONTINUE
      WRITE (*,*) '*Gcanal* 4)',F
      ENDIF
*>>>>>>>>>>>>
*     ELSE
*>>>>>>>>>>>>
*     Calcul avec le developpement en serie de Fourier
      DO 20 IP=INT(COUPUR)+1,NMAX
         PPIS2B=(REAL(IP)*PI)/(2.*B)
         V=PPIS2B*PPIS2B-K*K
         V=SQRT(V)
         TETM=PPIS2B*(CORM(2)+B)
         TETP=PPIS2B*(CORP(2)+B)
         CM=COS(TETM)
         CP=COS(TETP)
         RR=CM*CP/B
         SM=PPIS2B*SIN(TETM)
         SP=PPIS2B*SIN(TETP)
         RPP=-CM*SP/B
         RPM=-SM*CP/B
         RPMP=SM*SP/B
         IF (X.GT.0.) THEN
            EE=-(1./(2.*V))*EXP(-V*X)
            EP=-EXP(-V*X)/2.
         ELSE
            EE=-(1./(2.*V))*EXP(+V*X)
            EP=-EXP(+V*X)/2.
         ENDIF
         E=E+RR*EE
         D1EP(1)=D1EP(1)+EP*RR
         D1EP(2)=D1EP(2)+EE*RPP
         D1EM(1)=D1EM(1)-EP*RR
         D1EM(2)=D1EM(2)+EE*RPM
         IF (TYNOYO(:3).EQ.'FOU'.OR.TYNOYO(:3).EQ.'NEU') THEN  
            D2EMP(1)=D2EMP(1)-V*V*EE*RR
            D2EMP(2)=D2EMP(2)-EP*RPP
            D2EMP(3)=D2EMP(3)+EP*RPM
            D2EMP(4)=D2EMP(4)+EE*RPMP
         END IF
*>>>>>>>>>>>>
      TOTO=TOTO-PPIS2B*PPIS2B*EE*RR
*>>>>>>>>>>>>
         IF (ABS(RR*EE).LE.EPSIL) GOTO 60
20    CONTINUE
*     ENDIF
*
      WRITE (*,*) '*Gcanal* NON CONVERGENCE de la serie !'
      WRITE (*,*) '         point CORP=',CORP(1),CORP(2)
      WRITE (*,*) '         point CORM=',CORM(1),CORM(2)
*     STOP
*
60    CONTINUE
*>>>>>>>>>>>>
      WRITE (*,*) '*Gcanal* Verification:',-D2EMP(1)+TOTO+K*K*E
      WRITE (*,*) ' '
*>>>>>>>>>>>>
*>>>>>>>>>>>>
      IF (X.GT.-XTOL.AND.X.LT.XTOL) THEN
      WRITE (*,*) '*Gcanal* diff des deux series',X,XTOL,E,F,E-F
      WRITE (*,*) 
      ENDIF
*>>>>>>>>>>>>
      RETURN
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
999   WRITE (*,1000) (CORP(I),I=1,2),(CORM(I),I=1,2)
      STOP 
1000  FORMAT('*Gcanal* On essaie de calculer la fonction de Green pour'
     &,' une distance < 10**-6'/10X,'XP=',E12.4,' YP=',E12.4
     &,'      XM=',E12.4,' YM=',E12.4/)
                                                                    END !Gcanal
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE QCALC  (IP,TETM,TETP,BP,BPP,BPM,BPS)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      INTEGER          IP
      DOUBLE PRECISION TETM,TETP,BP,BPP,BPM,BPS
      DOUBLE PRECISION TET,QP,QPP,QPS,SINTS2,SI2TS2,SI4TS2
*
      TET=TETM+TETP
      SINTS2=DSIN(TET*0.5)
      SI2TS2=SINTS2*SINTS2
      SI4TS2=SI2TS2*SI2TS2
      QP =(DSIN((FLOAT(IP)+0.5)*TET))/SINTS2
      QPP=(IP*DSIN((IP-1.)*TET)+(IP+1)*DSIN(IP*TET))/SI2TS2
      QPS=   (IP+1)*(IP+1)*DCOS((IP+1)*TET)+IP*(3*IP+2)*DCOS(IP*TET)
     &    +(3*IP+1)*(IP-1)*DCOS((IP-1)*TET)+IP*(IP-2)  *DCOS((IP-2)*TET)
      QPS=QPS/SI4TS2
      BP=QP/4.
      BPP=-QPP/8.
      BPM=-QPP/8.
      BPS=-QPS/32.
      TET=TETM-TETP
      SINTS2=DSIN(TET*0.5)
      SI2TS2=SINTS2*SINTS2
      SI4TS2=SI2TS2*SI2TS2
      QP=(DSIN((FLOAT(IP)+0.5)*TET))/SINTS2
      QPP=(IP*DSIN((IP-1.)*TET)+(IP+1)*DSIN(IP*TET))/SI2TS2
      QPS=   (IP+1)*(IP+1)*DCOS((IP+1)*TET)+IP*(3*IP+2)*DCOS(IP*TET)
     &    +(3*IP+1)*(IP-1)*DCOS((IP-1)*TET)+IP*(IP-2)  *DCOS((IP-2)*TET)
      QPS=QPS/SI4TS2
      BP=BP+QP/4.+0.5
      BPP=BPP+QPP/8.
      BPM=BPM-QPP/8.
      BPS=BPS+QPS/32.
      RETURN
                                                                    END !Qcalc
