*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE GREEN (TYNOYO,TABCSG,CORP,CORM,E,TABASS)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Octobre 1989)
* Derniere modification : D.Martin (7 Mai 2002)
* Version 1.0.1
*
* Calcul de la fonction de Green en un couple de point (P,M)
*
*-- Arguments d'entree --
* TYNOYO type de calcul du noyau
*        'DIRICHLET' calcul de G(M,P), dG/dxm(M,P)
*        'NEUMANN'   calcul de dG/dxp(M,P), d2G/dxpdxm(M,P)
*        'FOURIER'   calcul de G(M,P), dG/dxm(M,P), dG/dxp(M,P), d2G/dxpdxm(M,P)
* TABCSG tableau contenant les valeurs necessaires au calcul (constante(s), ..)
* CORP   coordonnees du point P
* CORM   coordonnees du point M (source)
* E      tableau recepteur de la valeur de la fonction de Green,
*        de ses derivees par aux coordonnees de P et M
*        et dans le cas 'FOURIER' ou 'NEUMANN' de ses derivees secondes croisees
* TABASS tableau associe servant a vehiculer n'importe qu'elle information
*        son contenu est a l'entiere discretion de l'utilisateur.
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYNOYO
      COMPLEX TABCSG(*),E(*),TABASS(*)
      REAL    CHAMP,K,B,CORP(*),CORM(*)
      INTEGER ICHAMP
      COMPLEX AK
*-----------------------------------------------------------------------
      AK=TABASS(1)
*
*      CHAMP =REAL (TABCSG(2))
      ICHAMP =INT (TABCSG(2)+.1)
      IF (ICHAMP.LE.0) THEN
*        Cas classique: "helmholtz 2d"
         K=REAL(TABCSG(1))
         CALL GHLMZ2 (TYNOYO,K,CORM,CORP,E(1),E(2),E(4),E(6))
      ELSEIF (ICHAMP.EQ.1) THEN
*        Champ lointain: "helmholtz 2d"
         K=REAL(TABCSG(1))
         CALL GHLOIN (TYNOYO,K,CORM,CORP,E(1),E(2),E(4),E(6))
      ELSEIF (ICHAMP.EQ.2) THEN
*        Canal infini de largeur: +b,-b
         K=REAL(TABCSG(1))
         B=REAL(TABCSG(3))
         CALL GHBAND (TYNOYO,K,B,CORM,CORP,E(1),E(2),E(4),E(6))
      ELSEIF (ICHAMP.EQ.3) THEN
*        Cas "helmholtz 2D" avec une condition de symetrie sur l'axe Ox
         K=REAL(TABCSG(1))
         CALL GHELMS (TYNOYO,K,CORM,CORP,E(1),E(2),E(4),E(6))
      ELSE
         CALL BAISE
     &   ('*Green* La variable "Champ" doit etre 0.,1.,2. ou 3.')
      ENDIF
                                                                    END !Green
