*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
       SUBROUTINE GHELMZ (TYNOYO,K,CORM,CORP,E,D1EM,D1EP,D2EMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Octobre 1989)
* Derniere modification : D.Martin (12 Mars 1993)
* Version 1.0.0
*
* Solution Elementaire au point 'M' de l'Equation de Helmholtz dans R3  :
*
*                   E = [-1/4Pi] EXP(ikR)/R, avec R = || MP ||
*
* Attention : La multiplication par -1./4*PI n'est pas effectuee dans ce s.p.
*
*-- Arguments d'entree -- 
* TYNOYO type du noyau de Green a calculer
* K      constante K de EXP(I*K*R)/R
* CORM   coordonnees du point source
* CORP   coordonnees du point image
*-- Arguments de sortie --
* E     EXP(I*K*R)/R
* D1EM  dE/dxm, dE/dym, dE/dzm
* D1EP  dE/dxp, dE/dyp, dE/dzp
* D2EMP derivees secondes croisees dans l'ordre suivant
*       d2E/dxmdxp, d2E/dxmdyp, d2E/dxmdzp
*       d2E/dymdxp, d2E/dymdyp, d2E/dymdzp
*       d2E/dzmdxp, d2E/dzmdyp, d2E/dzmdzp
*  
*-- Si on note E la Solution Elementaire du Laplacien dans R3 , et H celle
*     de l'Operateur de Helmholtz on a :
*    H      = exp(iK/E) * E
*   dH/dx   = (1-iK/E) * exp(iK/E) * dE/dx 
*  d2H/dxdy = {(1-iK/E) * d2E/dxdy  -  K**2/E**3 * dE/dx * dE/dy} * exp(iK/E)  
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYNOYO 
      REAL          CORP(*),CORM(*)
      COMPLEX       K,E,D1EP(*),D1EM(*),D2EMP(*)
*
      COMPLEX       K2R3,IKR,EXPIKR,UNPEIK 
*-----------------------------------------------------------------------
*
      CALL GLAP3C (TYNOYO,CORM,CORP,R,E,D1EM,D1EP,D2EMP)
*
      IKR=(0.,1.)*K*R  
      EXPIKR=EXP(IKR)  
      UNPEIK=(1.-IKR)*EXPIKR
      K2R3=(IKR*IKR*R)*EXPIKR  
      E=EXPIKR*E
*
      IF (TYNOYO(1:3).EQ.'FOU'.OR.TYNOYO(1:3).EQ.'NEU') THEN
         IRANG2=1
         DO 3 I=1,3 
            DO 2 J=1,3 
               D2EMP(IRANG2)=UNPEIK*D2EMP(IRANG2)+K2R3*D1EP(J)*D1EM(I)  
               IRANG2=IRANG2+1
2           CONTINUE
3        CONTINUE
      ENDIF
*
      DO 1 I=1,3
         K2R3 = UNPEIK*D1EM(I)
         D1EM(I) = K2R3
         D1EP(I) =-K2R3
1     CONTINUE
                                                                 END    !Ghelmz
