*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      FUNCTION SLXHLM (K,COR)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Fevrier 1987)  
* Derniere modification : D.Martin (23 Mars 1995)
* Version 1.0.0
*  
* Champ de pression diffracte, d'une onde plane incidente Phi_w= exp^{(iKx}), 
* par une sphere rigide de rayon R.
* Representation sous forme d'une serie de fonctions de Bessel spheriques et 
* de Polynomes de Legendre :
*
*   Phi(r,Theta) = sum_0^infini  A_n h_n^{(1)}(Kr) P_n(cos Theta)
*
*   ou les coeff. du developpement en serie sont donnes par
*
*   A_n = - i^n (2*n+1) ( j'_n(KR) / h_n^{(1)}'(KR) )
*
*   avec
*      h_n = j_n + i y_n fct de Bessel spherique de 3eme espece d'ordre n,
*      j_n : fonction de Bessel spherique de 1ere esp\`ece = Real(h_n)
*      y_n : fonction de Bessel spherique de 2eme esp\`ece = Aimag(h_n)
*      P_n : polynome de Legendre de degre 1.
*      i   = (0.,1.)
*      R est ici choisi egal a 1.
*
* Bibliographie : ABRAMOVITZ et STEGUN - Handbook of Math. Functions
*                 (Bessel fonctions of fractional order , pp.437,439)  
*  
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      COMPLEX       SLXHLM 
      REAL          K,COR(*)
*  
      PARAMETER   ( NDIM = 3 , NMAX = 50 ) 
      LOGICAL       LASTCL 
      REAL          KKEEP,KRKEEP,AJYN(NMAX+1)  
      COMPLEX       CIN,H1PRIM,COEFAN,HNZERO
      COMMON/KEEPK/ COEFAN(NMAX),HNZERO(NMAX+1),KKEEP,KRKEEP,NBTRMH
      DATA EPSIL   /1.E-12/
*-----------------------------------------------------------------------
      R2=0.
      DO 1 IDIM=1,NDIM 
         R2=R2+COR(IDIM)*COR(IDIM)
1     CONTINUE
      R     =SQRT(R2)
      COSTET=COR(1)/R  
      IF (ABS(K-KKEEP).GT.1.E-10) THEN  
*  
*        Calcul des coefficients AN du developpement en serie  
*  
          CALL BESFYN (K,NMAX,AJYN)                                     !SpecFct
          DO 2 I=1,NMAX+1  
             HNZERO(I)=CMPLX(0.,AJYN(I)) 
2         CONTINUE 
          CALL BESFJN (K,NMAX,AJYN)                                     !SpecFct
          DO 3 I=1,NMAX+1  
             HNZERO(I)=HNZERO(I)+AJYN(I)  
3         CONTINUE
          COEFAN(1)=-REAL(HNZERO(2))/HNZERO(2) 
          CIN      =(0.,1.)  
          X1M2I    =-3.
          LASTCL   =.FALSE.
          DO 4 I=2,NMAX
             IM1      =I-1  
             H1PRIM   =I*HNZERO(I+1)-IM1*HNZERO(IM1)
             COEFAN(I)=CIN*X1M2I*REAL(H1PRIM)/H1PRIM  
             CIN      =(0.,1.)*CIN  
             X1M2I    =X1M2I-2.
             IF (LASTCL) GO TO 5
             IF (ABS(COEFAN(I)).LE.EPSIL) LASTCL=.TRUE.
4         CONTINUE 
          STOP ' *SLXHLM Convergence non atteinte : augmenter NMAX ?'  
5         NBTRMH=I-1
      ENDIF
*  
      R=K*R
      IF (ABS(K-KKEEP).GT.1.E-10.OR.ABS(R-KRKEEP).GT.1.E-10) THEN
          CALL BESFYN (R,NBTRMH,AJYN)                                   !SpecFct
          DO 6 I=1,NBTRMH+1
             HNZERO(I)=CMPLX(0.,AJYN(I))
6         CONTINUE
          CALL BESFJN (R,NBTRMH,AJYN)                                   !SpecFct
          DO 7 I=1,NBTRMH+1
             HNZERO(I)=(HNZERO(I)+AJYN(I))*COEFAN(I) 
7         CONTINUE 
      ENDIF
      KKEEP=K  
      KRKEEP=R 
*  
*    Calcul des polynomes de LEGENDRE et de la serie : 
*  
      SLXHLM=HNZERO(1) 
      P0=1.
      P1=COSTET
      DOSTET=COSTET+COSTET 
      DO 8 I=2,NBTRMH  
         COSTET=COSTET+DOSTET 
         P2    =(COSTET*P1-(I-1)*P0)/I
         P0    =P1
         P1    =P2
         SLXHLM=SLXHLM+P0*HNZERO(I)
8     CONTINUE
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*     IF(ABS(P1*HNZERO(NBTRMH+1)).GT.1.E-8)
*     &WRITE (*,20) NBTRMH+1,P1*HNZERO(NBTRMH+1),COEFAN(NBTRMH+1),R
20    FORMAT(' IMAX=',I3,' Terme neglige:',2E11.4,' Coef:',2E11.4,' KR='
     &,E11.4)  
                                                                    END !Slxhlm
