*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
       SUBROUTINE GHOUL2 (TYNOYO,ANU,CORM,CORP,E,D1EM,D1EP,D2EMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Octobre 1989)
* Derniere modification : D.Martin (15 Decembre 2000)
* Version 1.0.1
*  
*    Calcul de la Fonction de GREEN du Mouvement sur Houle Bidimensionnel  
*             (et de ses derivees) au point 'CORM' dans R2_ :  
*  
*              E = (1/2*PI) LOG(R/Rprim) - H   si nu > 0
*              E = (1/2*PI) LOG(R*Rprim)       si nu = 0
*  
*     ou H = Re (EXP(nu*Z)*E1(nu*Z)) - sign(Xp-Xm)* Im (EXP(nu*Z)) 
*                                              + i* Re (EXP(nu*Z)) 
*        E1 est la fonction exponentielle integrale complexe
*        Z = Ym + Yp + i* (Xp - Xm)
*        R = dist (M,P)  ,  Rprim = dist (Mprim,P) 
*        M a pour coordonnees (Xm,Ym), Mprim (Xm,-Ym) et P (Xp,Yp) 
*  
*-- Arguments d'entree --
* TYNOYO type de noyau ('DIRICHLET','NEUMANN','FOURIER')
* ANU    nombre d'onde ( nu = omega**2/g )
* CORM   coordonnees du point source
* CORP   coordonnees du point image
*-- Arguments de sortie --
* E      definie ci-dessus
* D1EM   tableaux des derivees de E par rapport aux coordonnees de M
* D1EP   tableaux des derivees de E par rapport aux coordonnees de P
* D2EMP  tableaux des derivees secondes croisees dans l'ordre suivant
*         d2E/dxmdxp, d2E/dxmdyp
*         d2E/dymdxp, d2E/dymdyp
*  
*-- sous-programme utilise : GLAP2C
* lequel retourne LOG(r) et ses derivees en cumulant les valeurs
* dans les variables de retour: E,D1EM,D1EP et D2EMP
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYNOYO
      REAL R,CORP(*),CORM(*)
      COMPLEX E,D1EP(*),D1EM(*),D2EMP(*)
*
      COMPLEX NUZ,CE1NUZ,ENUZ,ONPIZ,ONPIZ2,CE1,H,DXPH,DYPH,DXMXPH,DYMXPH
      DIMENSION RENUZ(2)
      EQUIVALENCE (ENUZ,RENUZ) 
      DATA PISR2,UNSRPI/.159154943091895,.318309886183790/
*-----------------------------------------------------------------------
*     Calcul de LOG(r)-LOG(rprim) si ANU > 0.  
*            de LOG(r)+LOG(rprim) si ANU=0.
*  
      DO 3 I=1,2
          CALL GLAP2C (TYNOYO,CORM,CORP,RPRIM,E,D1EM,D1EP,D2EMP)        !utilite
          D1EM(2)=-D1EM(2) 
          D2EMP(3)=-D2EMP(3)
          D2EMP(4)=-D2EMP(4)
          IF(ANU.GT.1.E-06) THEN 
             E=-E
             DO 1 J=1,2  
                D1EM(J)=-D1EM(J) 
1               D1EP(J)=-D1EP(J) 
             DO 2 J=1,4  
2               D2EMP(J)=-D2EMP(J)
          END IF 
3         CORM(2)=-CORM(2) 
*  
*     Multiplication par 1/2*pi
*  
      E = PISR2 * E 
      D1EM(1) = PISR2*D1EM(1)  
      D1EM(2) = PISR2*D1EM(2)  
      D1EP(1) = PISR2*D1EP(1)  
      D1EP(2) = PISR2*D1EP(2)  
      IF(TYNOYO(:3).NE.'DIR') THEN 
         D2EMP(1)= PISR2*D2EMP(1)  
         D2EMP(2)= PISR2*D2EMP(2)  
         D2EMP(3)= PISR2*D2EMP(3)  
         D2EMP(4)= PISR2*D2EMP(4)  
      END IF
*     WRITE (*,*) E,D1EM(1),D1EM(2),D1EP(1),D1EP(2)
*  
      IF(ANU.LE.1.E-06) RETURN 
*  
*     Calcul de la partie dite reguliere : H (m,p) 
*  
      XPMXM=CORP(1)-CORM(1)
      SGN=SIGN(1.,XPMXM)
*         ==== 
*     WRITE (*,*) XPMXM,SGN
      NUZ=CMPLX(CORP(2)+CORM(2),XPMXM) 
      ONPIZ=UNSRPI/NUZ
      ONPIZ2=ONPIZ/NUZ 
      NUZ=ANU*NUZ  
*  
*     Calcul de Exp(nu*z)  
      ENUZ=EXP(NUZ)
*          === 
*     Calcul de CE1(Z)=(1/PI) EXP(Z)*E1(Z) 
      CE1NUZ=UNSRPI*CE1(NUZ)                                            !Specfct
*  
      H = CMPLX(REAL(CE1NUZ) - SGN*RENUZ(2) ,RENUZ(1)) 
      E = E - H
*  
      DXPH=ANU*CMPLX(AIMAG(CE1NUZ)+SGN*RENUZ(1),RENUZ(2)) - AIMAG(ONPIZ)
      DYPH=-ANU*H + REAL(ONPIZ)
      D1EM(1) =D1EM(1) - DXPH  
      D1EM(2) =D1EM(2) + DYPH  
      D1EP(1) =D1EP(1) + DXPH  
      D1EP(2) =D1EP(2) + DYPH  
*  
      IF(TYNOYO(:3).EQ.'DIR') RETURN
      DXMXPH=ANU*DYPH - REAL(ONPIZ2)
      DYMXPH=ANU*DXPH + AIMAG(ONPIZ2)  
      D2EMP(1)=D2EMP(1) + DXMXPH
      D2EMP(2)=D2EMP(2) - DYMXPH
      D2EMP(3)=D2EMP(3) + DYMXPH
      D2EMP(4)=D2EMP(4) + DXMXPH
                                                                    END !Ghoul2
