*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
       SUBROUTINE GHOULE (TYNOYO,NU,CORM,CORP,E,D1EM,D1EP,D2EPM)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin & A.Clement
* Derniere modification : D.Martin (12 Avril 1996)
* version 1.0.0
*  
* Calcul de la Fonction de GREEN du Mouvement sur Houle Tridimensionnel 
*             (et de ses derivees) au point 'M' dans R3_
* Bibliographie : Rapport de recherche ENSTA Nos 124 (Mars 80) et 165 (Aout 82)
*  
*-- Arguments d'entree -- 
* TYNOYO autorise ou non le calcul des derivees secondes (si <> 'D'))
* NU     omega**2/g
* CORM   coordonnees du point source  
* CORP   coordonnees du point image
*-- Arguments de sortie --
* E      valeur de la fonction de Green en (m,p)  
* D1EM   derivees de E par rapport aux coordonnees de M
* D1EP   derivees de E par rapport aux coordonnees de P
* D2EMP  derivees secondes croisees dans l'ordre suivant  
*         d2E/dxmdxp, d2E/dxmdyp, d2E/dxmdzp
*         d2E/dymdxp, d2E/dymdyp, d2E/dymdzp
*         d2E/dzmdxp, d2E/dzmdyp, d2E/dzmdzp
*  
*!!! Remarque 
*    La multiplication par -1./4*PI n'est pas effectuee dans ce S-P
*  
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYNOYO
      REAL          NU,CORP(*),CORM(*)
      COMPLEX       E,D1EM(*),D1EP(*),D2EPM(*)
*
      INTEGER       INTPA,NPAS,I
      REAL          ZGARDE,RGARDE
      REAL          RPRIM,RPRIM2,RPRIM3,XMMXP,YMMYP,RADIAL,ZMPZP
     &             ,XSURR,YSURR,TAMP
      REAL          NUR,NUZ,PIEZ,G2,D2,G3,D3
      COMPLEX       G4,D4  
      COMPLEX       H,DRH,DRRH,DXH,DYH,DZH,D1XH,D1YH,D2YH,D3XH,D3YH,D3ZH  
      COMMON/GARDZR/ZGARDE,RGARDE,H,DRH
*-----------------------------------------------------------------------
*         Calcul de la partie correspondant a  1/r + 1/rprim
*  
      DO 1 I=1,2
         CALL GLAP3C (TYNOYO,CORM,CORP,RPRIM,E,D1EM,D1EP,D2EPM)         Utilite
         D1EM(3)=-D1EM(3)  
         IF (TYNOYO(1:1).NE.'D') THEN
            D2EPM(7)=-D2EPM(7) 
            D2EPM(8)=-D2EPM(8) 
            D2EPM(9)=-D2EPM(9) 
         ENDIF
         CORM(3)=-CORM(3)
1     CONTINUE 
      IF (NU.LE.1.E-05) RETURN 
*  
*     Calcul de la partie dite reguliere : H (m,p) 
*     et de sa derivee radiale horizontale : dH/dR (m,p)
*  
      XMMXP =CORP(1)-CORM(1)
      YMMYP =CORP(2)-CORM(2)
      RADIAL=SQRT (XMMXP*XMMXP+YMMYP*YMMYP) 
      ZMPZP =CORP(3)+CORM(3)
      NUR  =NU*RADIAL  
      NUZ  =NU*ZMPZP
*    On ne recalcule pas les Integrales lorsque, pour 2 couples consecutifs,
*    les valeurs de NUR et NUZ sont les memes (e.g. sym./plans verticaux)
      IF ((ABS (ZGARDE-NUZ)+ABS (RGARDE-NUR)).GT.1.E-06) THEN  
         ZGARDE=NUZ
         RGARDE=NUR
         INTPA =0
         NPAS  =0
         PIEZ  =EXP (NUZ)
*      1)Calcul de la fonction H et de sa derivee radiale horizontale DRH  
*       (On conseille la lecture du rapport CLEMENT ,rapport ENSTA 165,Aout 82)
*  
*        Choix de la formule de calcul de la partie 'integrale'
*  
         CALL DIXMS4 (NUR,NUZ,INTPA,NPAS)                               Appl-  
*  
*        Calcul de l'integrale (G2+G3) et de sa derivee (D2+D3)
*  
         CALL GD23 (INTPA,NPAS,NUR,NUZ,PIEZ,G3,G2,D3,D2)                Appl-  
*  
*        Calcul des termes G4 et D4 (Fonctions de STRUVE et de BESSEL) 
*  
         CALL GD4 (INTPA,NUR,G4,D4)                                     Appl-  
*  
         PIEZ=3.14159265*PIEZ  
         H   =NU*(G2+G3-PIEZ*G4) 
         DRH =NU*NU*(D2+D3-PIEZ*D4)
*        ---------------------------
      ENDIF
*  
*     2) Calcul des derivees de H (M,P)
*  
      DXH   =DRH  
      DYH   =DRH  
      RPRIM =1./RPRIM
      RPRIM2=RPRIM*RPRIM
      RPRIM3=NU*RPRIM*RPRIM2  
      RPRIM3=RPRIM3+RPRIM3 
      DZH   =NU*(H+RPRIM+RPRIM)  
      D3ZH  =NU*DZH-ZMPZP*RPRIM3
      D1XH  =.5*D3ZH 
      D2YH  =D1XH
      D1YH  =0.  
      IF (RADIAL.GT.2.E-07) THEN
         RADIAL=1./RADIAL  
         XSURR =XMMXP*RADIAL
         YSURR =YMMYP*RADIAL
         DXH   =DXH*XSURR 
         DYH   =DYH*YSURR 
         DRRH  =D1XH+DRH*RADIAL  
         TAMP  =XSURR*XSURR-YSURR*YSURR  
         D1XH  =D1XH+TAMP*DRRH
         D2YH  =D2YH-TAMP*DRRH
         D1YH  =XSURR*YSURR*DRRH 
         D1YH  =D1YH+D1YH
         D3XH  =NU*DXH  
         D3YH  =NU*DYH  
      ENDIF
      D3XH  =D3XH-XMMXP*RPRIM3
      D3YH  =D3YH-YMMYP*RPRIM3
*  
      E      =  E    +H
      D1EM(1)=D1EM(1)-DXH
      D1EM(2)=D1EM(2)-DYH
      D1EM(3)=D1EM(3)+DZH
      D1EP(1)=D1EP(1)+DXH
      D1EP(2)=D1EP(2)+DYH
      D1EP(3)=D1EP(3)+DZH
      IF (TYNOYO(1:1).NE.'D') THEN  
         D2EPM(1)=D2EPM(1)+D1XH
         D2EPM(2)=D2EPM(2)+D1YH
         D2EPM(3)=D2EPM(3)-D3XH
         D2EPM(4)=D2EPM(4)+D1YH
         D2EPM(5)=D2EPM(5)+D2YH
         D2EPM(6)=D2EPM(6)-D3YH
         D2EPM(7)=D2EPM(7)+D3XH
         D2EPM(8)=D2EPM(8)+D3YH
         D2EPM(9)=D2EPM(9)+D3ZH
      ENDIF
                                                                    END !Ghoule
