*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
       SUBROUTINE CGHOUL (TYNOYO,NU,CORM,CORP,E,D1EM,D1EP,D2EPM)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : C.Hazard & D.Martin (Decembre 1992)
* Derniere modification : D.Martin (23 Janvier 1997)
* Version 1.0.0
*  
* Calcul de la Fonction de GREEN du Mouvement sur Houle Tridimensionnel 
* (et derivees) au point 'CORM' dans R3_ de la cas d'une 'frequence' COMPLEXE
*
* Bibliographie : Rapport de recherche ENSTA Nos 124 (Mars 80) et 165 (Aout 82)
*                 pour le cas reel (voir aussi GHOULE (appl_houle3d))
*                 Article a definir ()
*                 pour le cas complexe
*  
*-- Arguments d'entree --
* TYNOYO autorise ou non le calcul des derivees secondes (si <> 'D'))
* NU     omega**2/g avec 'frequence complexe' omega
* CORM   coordonnees du point source  
* CORP   coordonnees du point image
*-- Arguments de sortie --
* E      valeur de la fonction de Green en (m,p)  
* D1EM   derivees de E par rapport aux coordonnees de M
* D1EP   derivees de E par rapport aux coordonnees de P
* D2EMP  derivees secondes croisees dans l'ordre suivant  
*            d2E/dxmdxp, d2E/dxmdyp, d2E/dxmdzp
*             d2E/dymdxp, d2E/dymdyp, d2E/dymdzp
*             d2E/dzmdxp, d2E/dzmdyp, d2E/dzmdzp
*  
*!!! Remarque
*    La multiplication par -1./4*PI n'est pas effectuee dans ce S-P
*  
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYNOYO
      REAL          CORP(*),CORM(*)
      COMPLEX       NU,E,D1EP(*),D1EM(*),D2EPM(*)
*
      INTEGER       NPAS,I
      REAL          RPRIM,RPRIM2,RPRIM3,RADIAL,XMMXP,YMMYP,ZMPZP
     &             ,XSURR,YSURR,TAMP
      COMPLEX       NUR,NUZ,PIEZ,INTG,INTD,RESD,FSPG,FSPD
      COMPLEX       H,DRH,DRRH,DXH,DYH,DZH,D1XH,D1YH,D2YH,D3XH,D3YH,D3ZH
      COMPLEX      JN(0:1),YN(0:1),HN(0:1),DEUXI
      PARAMETER   (DEUXI=(0.,2.))
*-----------------------------------------------------------------------
*     Calcul de la partie correspondant a  1/r + 1/rprim
*  
      DO 1 I=1,2
         CALL GLAP3C (TYNOYO,CORM,CORP,RPRIM,E,D1EM,D1EP,D2EPM)         Appl-
         D1EM(3)=-D1EM(3)  
         IF (TYNOYO(1:1).NE.'D') THEN
            D2EPM(7)=-D2EPM(7) 
            D2EPM(8)=-D2EPM(8) 
            D2EPM(9)=-D2EPM(9) 
         ENDIF
         CORM(3)=-CORM(3)
1     CONTINUE  
      IF (ABS(NU).LE.1.E-05) RETURN 
*  
*     Calcul de la partie dite reguliere : H (m,p) 
*     et de sa derivee radiale horizontale : dH/dR (m,p)
*  
      XMMXP=CORP(1)-CORM(1)
      YMMYP=CORP(2)-CORM(2)
      RADIAL=SQRT (XMMXP*XMMXP+YMMYP*YMMYP) 
      ZMPZP=CORP(3)+CORM(3)
      NUR=NU*RADIAL  
      NUZ=NU*ZMPZP
      NPAS=0
      IF (CORP(3).GE.1.E-04) GOTO 99991
      IF (CORM(3).GE.1.E-04) GOTO 99992
      IF (ABS(NUZ).LE.1.E-02.AND.ABS(NUR).LE.1.E-02) GOTO 99993
*
      IF (ABS(NUZ).GT.10.*ABS(NUR)) THEN
*
*        Integrale de l' exponentielle integrale
*
         NPAS=32
         CALL CIEXE1 (NUR,NUZ,NPAS,INTG,INTD)                           Appl-
*
*        Calcul de la partie integree analytiquement de la derivee (D3) :
*            -2i/Pi * Int_0^Pi) cos(Theta)/Q(Theta) dTheta
*                   avec Q(Theta)= Nu Z + i Nu R cos(Theta)
*          = -2*(NuZ+Sqrt(NuZ*NuZ+NuR*NuR))/(NuR*Sqrt(NuZ*NuZ+NuR*NuR)) 
*
         IF (ABS(NUR).GE.1.E-06) THEN
            RESD=SQRT (NUR*NUR+NUZ*NUZ) 
            RESD=(NUZ+RESD)/(NUR*RESD) 
         ENDIF
*
         H   =NU*INTG
         DRH =NU*NU*(INTD-(RESD+RESD))
*        -----------------------------
      ELSE
*
*        Integrale d'exponentielle simple et fonctions speciales
*
         NPAS=32
         CALL CIEXNU (NU,RADIAL,ZMPZP,NPAS,INTG,INTD)                   Appl-
*
*        Calcul de J0(NUR), J1(NUR), Y0(NUR) et Y1(NUR)
*
         CALL BCJY01 (NUR,JN(0),JN(1),YN(0),YN(1))                     !Specfct
*
*        Calcul de H0(NUR) et H1(NUR)
*        La  procedure retourne H0 et H1-2/Pi
*
         CALL STCH01 (NUR,YN(0),YN(1),HN(0),HN(1))                     !Specfct
*
         FSPG=HN(0)+YN(0)-DEUXI*JN(0)
         FSPD=HN(1)+YN(1)-DEUXI*JN(1)
*
         PIEZ=3.14159265*EXP(NUZ) 
         H   =NU*(INTG-PIEZ*FSPG) 
         DRH =NU*(INTD+NU*PIEZ*FSPD)
*        ---------------------------
      ENDIF
*  
*     2) Calcul des derivees de H (M,P)
*  
      DXH   =DRH  
      DYH   =DRH  
      RPRIM =1./RPRIM
      RPRIM2=RPRIM*RPRIM
      RPRIM3=RPRIM*RPRIM2
      RPRIM3=RPRIM3+RPRIM3 
      DZH   =NU*(H+RPRIM+RPRIM)  
      D3ZH  =NU*(DZH-ZMPZP*RPRIM3)
      D1XH  =.5*D3ZH 
      D2YH  =D1XH
      D1YH  =0.  
      IF (RADIAL.GT.2.E-07) THEN
         RADIAL=1./RADIAL  
         XSURR =XMMXP*RADIAL
         YSURR =YMMYP*RADIAL
         DXH   =DXH*XSURR 
         DYH   =DYH*YSURR 
         DRRH  =D1XH+DRH*RADIAL  
         TAMP  =XSURR*XSURR-YSURR*YSURR  
         D1XH  =D1XH+TAMP*DRRH
         D2YH  =D2YH-TAMP*DRRH
         D1YH  =XSURR*YSURR*DRRH 
         D1YH  =D1YH+D1YH
         D3XH  =NU*DXH  
         D3YH  =NU*DYH  
      ENDIF
      D3XH  =D3XH-NU*XMMXP*RPRIM3
      D3YH  =D3YH-NU*YMMYP*RPRIM3
*     ---------------------------  
      E      =  E    +H
      D1EM(1)=D1EM(1)-DXH
      D1EM(2)=D1EM(2)-DYH
      D1EM(3)=D1EM(3)+DZH
      D1EP(1)=D1EP(1)+DXH
      D1EP(2)=D1EP(2)+DYH
      D1EP(3)=D1EP(3)+DZH
      IF (TYNOYO(1:1).NE.'D') THEN  
         D2EPM(1)=D2EPM(1)+D1XH
         D2EPM(2)=D2EPM(2)+D1YH
         D2EPM(3)=D2EPM(3)-D3XH
         D2EPM(4)=D2EPM(4)+D1YH
         D2EPM(5)=D2EPM(5)+D2YH
         D2EPM(6)=D2EPM(6)-D3YH
         D2EPM(7)=D2EPM(7)+D3XH
         D2EPM(8)=D2EPM(8)+D3YH
         D2EPM(9)=D2EPM(9)+D3ZH
      ENDIF
      RETURN
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
99991 WRITE(*,99901) 'Point P',(CORP(I),I=1,3)
      CALL BAISE  (' *CGhoul* Probleme de maillage?')                   !Utilite
99992 WRITE(*,99901) 'Point M ',(CORM(I),I=1,3)
      CALL BAISE  (' *CGhoul* Probleme de maillage?')                   !Utilite
99901 FORMAT(//' Le point ',A,' de coordonnees :(',3E12.4,')'/
     &' n''est pas dans le demi-espace R3_')
99993 CALL WARNNG (' *CGhoul* Argument NUR ou NUZ trop petit,' 
     &//' calcul nonteste')
                                                                    END !CGhoul
