*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE CIEXE1 (AR,AZ,NP,G2,D2)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin (Janvier 1993)
* Derniere modification (D.Martin) le 23 Janvier 1997
* 
* Calcul des integrales
* 
*     2 /Pi * Int_0^Pi EXP(Q(Theta))*E1(Q(Theta)) dTheta
* 
*     2i/Pi * Int_0^Pi EXP(Q(Theta))*E1(Q(Theta))*cos(Theta) dTheta
*
* par la regle de Simpson des 3/8emes
*
*    ou E1(z) est la fonction exponentielle integrale complexe
*    et Q(Theta) = AZ + i * AR * cos(Theta)
*
* Bibliographie : HANDBOOK Of MATHEMATICAL FUNCTIONS 
* -------------   Ed. M.ABRAMOWITZ & I.A. STEGUN  
*                 Chap. 9 : Exponential Integral and Related Functions (p227)
*
* Rappel : Regle de Simpson des 3/8emes
* ------   Int(a,b) f(x) dx  = (b-a)/8n *
*              * Somme(i=0,n) [f(x[i])+3*f(x[i]+h)+3*f(x[i]+2h)+f(x[i]+3h)]
*          avec h=(b-a)/3n et x[i]=a+3ih
*
*-- Arguments d'entree -- 
* AR  (C) ANU*R     (R : distance horizontale entre deux points)  
* AZ  (C) ANU*(Z+C) (Z+C : somme des cotes des deux points)  
* ou ANU=Omega*Omega/G est le nombre d'onde  complexe
*     (Omega est la pulsation ici complexe , G la constante de gravite)
* NP  (I) nombre de pas d'integration de la formule de quadrature  
* 
*-- Arguments de sortie --
* G2  (C) valeur de l'integrale
* D2  (C) valeur de la derivee de l'integrale
*
*-- Procedure utilisee --
* CCEXP1 (C) exponentielle integrale complexe multipliee par l'exponentielle
*            CCEXP1 =  E1(Z)*EXP(Z)       si Im (Z)<0.
*                   = (E1(Z)+2iPi)*EXP(Z) sinon 
*            (i.e. la coupure est sur l'axe reel POSITIF)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      INTEGER NP
      COMPLEX AR,AZ,G2,D2
*
      INTEGER I
      REAL    UNSUNP,H,BMASN,COSH,SINH,COST,SINT,COSKIP
      COMPLEX CI,CIAR,CCEXP1,POINT,VALEUR,AUTIER,AUBOUT,AUTIED,AUBOUD
      PARAMETER ( CI = (0.,1.) ) 
*-----------------------------------------------------------------------
      IF (ABS(AR).LT.1.E-06.OR.NP.LE.0) THEN
*
*        Formule du point milieu (en Theta = Pi/2)
*
         G2=CCEXP1(AZ)                                                  !Appl-
         G2=G2+G2
         D2=0.
      ELSEIF (NP.GT.0) THEN
         CIAR=CI*AR
*
*        Regle de Simpson des 3/8emes ( h=(Pi/NP)/3 )
*
         UNSUNP=1./NP
         H=1.0471974*UNSUNP
         COSH=COS (H)  
         SINH=SIN (H)  
         COST=COSH
         SINT=SINH
*
*        Valeur en Theta=0.
         POINT=AZ+CIAR
         AUBOUT=CCEXP1 (POINT)                                          !Appl-
         AUBOUD=AUBOUT
         G2=0.
         D2=0.
         DO 1 I=1,NP
*
*           Formule de Simpson des 3/8emes
*           Calcul de [f(x[i]+h)+3*f(x[i]+h)+3*f(x[i]+2h)+f(x[i]+3h)]
*
*           Valeur a l'extremite gauche (x[i]) de l'intervalle
*           = Valeur a l'extremite droite de l'intervalle precedent
*
            G2=G2+AUBOUT
            D2=D2+AUBOUD
*
*           Valeur au tiers gauche (x[i]+h) de l'intervalle
*
            POINT=AZ+CIAR*COST
            VALEUR=CCEXP1 (POINT)                                       !Appl-
            AUTIER=VALEUR
            AUTIED=VALEUR*COST
*
*           Valeur au tiers droit (x[i]+2h) de l'intervalle
*
            COSKIP=COST
*           Cos(Theta+h)=Cos(Theta)*Cos(h)-Sin(Theta)*Sin(h) !
*           Sin(Theta+h)=Cos(Theta)*Sin(h)-Sin(Theta)*Cos(h) !
            COST=COSKIP*COSH-SINT*SINH 
            SINT=COSKIP*SINH+SINT*COSH
            POINT=AZ+CIAR*COST
            VALEUR=CCEXP1 (POINT)                                       !Appl-
            AUTIER=AUTIER+VALEUR
            AUTIED=AUTIED+VALEUR*COST
*
*           Valeur a l'extremite droite (x[i]+3h=x[i+1]) de l'intervalle
*
            COSKIP=COST
            COST=COSKIP*COSH-SINT*SINH 
            SINT=COSKIP*SINH+SINT*COSH
            POINT=AZ+CIAR*COST
            VALEUR=CCEXP1 (POINT)                                       !Appl-
            AUBOUT=VALEUR
            AUBOUD=VALEUR*COST
            COSKIP=COST
            COST=COSKIP*COSH-SINT*SINH 
            SINT=COSKIP*SINH+SINT*COSH
*
*           = + 3*f(x1) + 3*f(x2) + f(x3)
*
            G2=G2+AUTIER+AUTIER+AUTIER+AUBOUT
            D2=D2+AUTIED+AUTIED+AUTIED+AUBOUD
1        CONTINUE
*
*        Fin de la formule : Multiplication par (2/Pi)*(Pi/8*NP)
         BMASN=.25*UNSUNP
         G2   =BMASN*G2
         D2   =CI*BMASN*D2
      ENDIF 
                                                                    END !CIExE1
