*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE CIEXNU (NU,R,Z,NP,G2,D2)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin (Fevrier 1993)
* Derniere modification (D.Martin) le 23 Janvier 1997
* 
* Calcul numerique des integrales
* 
*    -2   *Int_0^{-Z} Exp(-nu*t) / ((t+Z)**2+R**2)**1/2 dt  pour R > 0.
* 
*     2*R *Int_0^{-Z} Exp(-nu*t) / ((t+Z)**2+R**2)**3/2 dt
*
* par la regle de Simpson des 3/8emes
*
* Rappel : Regle de Simpson des 3/8emes
* ------   Int(a,b) f(x) dx  = (b-a)/8n *
*              * Somme(i=0,n) [f(x[i])+3*f(x[i]+h)+3*f(x[i]+2h)+f(x[i]+3h)]
*          avec h=(b-a)/3n et x[i]=a+3ih
*
*-- Arguments d'entree --
* NU (C) =Omega*Omega/G est le nombre d'onde  complexe
*        (Omega est la pulsation ici complexe , G la constante de gravite)
* R  (R) distance horizontale entre deux points  
* Z  (R) somme des cotes des deux points
* NP (I) nombre de pas d'integration de la formule de quadrature    
* 
*-- Arguments de sortie --
* G2  (C) valeur de l'integrale
* D2  (C) valeur de la derivee (en R) de l'integrale
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      INTEGER NP
      REAL    R,Z
      COMPLEX NU,G2,D2
*
      INTEGER I
      REAL    UNSUNP,H,R2,Z2PR2,DENOM2,BMASN,TPZ,SQDENO
      COMPLEX EXPNUH,EXPNUT
      COMPLEX VALEUR,AUTIER,AUBOUT,AUTIED,AUBOUD
*-----------------------------------------------------------------------
      R2   =R*R
      Z2PR2=Z*Z+R2
*
      IF (NP.LE.0.OR.R.LE.1.E-06) THEN
*
*     Formule du point milieu (en t=-Z/2.)
*
         Z2PR2=Z2PR2+.25*Z*Z
         DENOM2=1./Z2PR2
         G2=-2.*Z*DENOM2*EXP (.5*NU*Z)
         D2=-R*G2*DENOM2
*
      ELSE
*
*        Regle de Simpson des 3/8emes ( h=(-Z/NP)/3 )
*
         UNSUNP=1./NP
         H     =-Z*UNSUNP/3.
         EXPNUH=EXP (-NU*H)
*        Valeur en t=0.
         DENOM2=1./Z2PR2
         AUBOUT=SQRT (DENOM2)
         AUBOUD=AUBOUT*DENOM2
*        Initialisation (en t=h)
         TPZ=Z+H
         EXPNUT=EXPNUH
         G2=0.
         D2=0.
         DO 1 I=1,NP
*
*           Calcul de [f(x[i]+h)+3*f(x[i]+h)+3*f(x[i]+2h)+f(x[i]+3h)]
*
*           Valeur a l'extremite gauche (xi) de l'intervalle
*           = Valeur a l'extremite droite de l'intervalle precedent
            G2=G2+AUBOUT
            D2=D2+AUBOUD
*
*           Valeur au tiers gauche (x[i]+h) de l'intervalle
*
            DENOM2=1./(R2+TPZ*TPZ)
            SQDENO=SQRT (DENOM2)
            VALEUR=EXPNUT*SQDENO
            AUTIER=VALEUR
            AUTIED=VALEUR*DENOM2
*
*           Valeur au tiers droit (x[i]+2h) de l'intervalle
*
            TPZ=TPZ+H
*           Exp(-nu*(t+h))=Exp(-nu*t)*Exp(-nu*h) !
            EXPNUT=EXPNUT*EXPNUH
            DENOM2=1./(R2+TPZ*TPZ)
            SQDENO=SQRT (DENOM2)
            VALEUR=EXPNUT*SQDENO
            AUTIER=AUTIER+VALEUR
            AUTIED=AUTIED+VALEUR*DENOM2
*
*           Valeur a l'extremite droite (x[i]+3h=x[i+1]) de l'intervalle
*
            TPZ=TPZ+H
            EXPNUT=EXPNUT*EXPNUH
            DENOM2=1./(R2+TPZ*TPZ)
            SQDENO=SQRT (DENOM2)
            VALEUR=EXPNUT*SQDENO
            AUBOUT=VALEUR
            AUBOUD=VALEUR*DENOM2
            TPZ=TPZ+H
            EXPNUT=EXPNUT*EXPNUH
*
            G2=G2+AUTIER+AUTIER+AUTIER+AUBOUT
            D2=D2+AUTIED+AUTIED+AUTIED+AUBOUD
1        CONTINUE
*
*        Fin de la formule : Multiplication par (-2)*(-Z/8*NP)
         BMASN=.25*Z*UNSUNP
         G2   =BMASN*G2
         D2   =-BMASN*R*D2
      ENDIF 
                                                                    END !CIExNu
