*----------------------------------------------------------------------- 
       SUBROUTINE GRW2A (TYNOYO,ANU,CORM,CORP,E,D1EM,D1EP,D2EMP)
*-----------------------------------------------------------------------
* Auteur: Chrysoula TSOGKA (juin 1994)
* Derniere modification (O.DeBayser) le 10 Avril 96
*
*    Calcul de la Fonction de GREEN du probleme auxilliaire de 
*    resistance de vagues, et de ses derivees, au point 'CORM'.  
*    Version "probleme auxiliaire - G1" - cas "nu" reel
*
*           E = (1/2*PI) LOG(R/Rprim) - H1 - H2 - H3   si nu > 0 
*           E = (1/2*PI) LOG(R*Rprim)                  si nu = 0 
*
*     ou H1 = 1/2PI(EXP(nu*Z)*E1(nu*Z) + EXP(nu*ZB)*E1(nu*ZB))
*        H2 =  (i/2) (1+sign(nu*(Xp-Xm))) EXP(nu*Z)
*        H3 =  (i/2) (1-sign(nu*(Xp-Xm))) EXP(nu*ZB)
*
*        E1 est la fonction exponentielle integrale complexe  
*        Z = Ym + Yp + i* (Xp - Xm) 
*        ZB= Ym + Yp - i* (Xp - Xm) 
*        R = dist (M,P)  ,  Rprim = dist (Mprim,P)  
*        M a pour coordonnees (Xm,Ym), Mprim (Xm,-Ym) et P (Xp,Yp)  
*
*-- Arguments d'entree --
* TYNOYO: type de noyau ('DIRICHLET','NEUMANN','FOURIER')  
* ANU   : nombre d'onde ( nu = g/uo**2 ) reel
* CORM  : coordonnees du point source (appartient a Sigma, la surface  
*         artificielle entourant le corps)  
* CORP  : coordonnees du point de travail (appartient a Gamma, la surface 
*         sur laquelle on calcule la formule de representation integrale) 
*-- Arguments de sortie --
* E     : la fonction de Green 
* D1EM  : tableaux des derivees de E par rapport aux coordonnees de M   
* D1EP  : tableaux des derivees de E par rapport aux coordonnees de P   
* D2EMP : tableaux des derivees secondes croisees dans l'ordre suivant 
*         d2E/dxmdxp, d2E/dxmdyp 
*         d2E/dymdxp, d2E/dymdyp 
*
*-- REMARQUE les initialisations de E,D1EP,D1EM,D2EMP doivent etre
*        realisees dans le programme appelant. En effet, des boucles
*        sont prevues en cas de symetrie de la geometrie pour ghoule
*-- CEE1  est la fonction complexe E1*EXP(nu*Z)
************************************************************************
	IMPLICIT  COMPLEX (Z)
	CHARACTER*(*) TYNOYO  
	COMPLEX   CEE1,E,D1EP,D1EM,D2EMP
	DIMENSION D1EP(*),D1EM(*),D2EMP(*),CORP(*),CORM(*)
        COMPLEX ONUZ ,ONUZ2 ,NUZ ,ENUZ 
        COMPLEX ONUZB,ONUZB2,NUZB,ENUZB
*
*------------------------------------------------------------------------  
*     Calcul de LOG(r)
*------------------------------------------------------------------------  
	X     = CORP(1)-CORM(1)
	Y     = CORP(2)-CORM(2)
	R2    = X*X+Y*Y
	R4    = R2*R2
	R     = SQRT(R2)
	EE    = ALOG(R)
	D1EM1 =-X/R2
	D1EM2 =-Y/R2
	D1EP1 = X/R2
	D1EP2 = Y/R2
	IF(TYNOYO(:3).NE.'DIR') THEN 
	 D2EMP1= (X*X-Y*Y)/R4
	 D2EMP2= 2*X*Y/R4
	 D2EMP3= 2*X*Y/R4
	 D2EMP4= (-X*X+Y*Y)/R4
	END IF
*-----------------------------------------------------------------------
*     Calcul de LOG(rprim)
*-----------------------------------------------------------------------
        Y      = CORP(2)+CORM(2)
	R2     = X*X+Y*Y
	R4     = R2*R2
	R      = SQRT(R2)
	RE     = ALOG(R)
	RD1EM1 =-X/R2
	RD1EM2 = Y/R2
	RD1EP1 = X/R2
	RD1EP2 = Y/R2
	IF(TYNOYO(:3).NE.'DIR') THEN 
	 RD2EMP1= (X*X-Y*Y)/R4
	 RD2EMP2= 2*X*Y/R4
	 RD2EMP3=-2*X*Y/R4
	 RD2EMP4= (X*X-Y*Y)/R4
	END IF
*-----------------------------------------------------------------------
*     Somme si nu=0, difference sinon et division par 2*PI
*-----------------------------------------------------------------------
        IF(ANU.LE.1.E-9) THEN
	 S=1
	ELSE
	 S=-1
	ENDIF
*
	dpi=2*3.14159265358979323846
*
	E = (EE + S*RE)/dpi + E
*
	D1EM(1) = (D1EM1+S*RD1EM1)/dpi + D1EM(1) 
	D1EM(2) = (D1EM2+S*RD1EM2)/dpi + D1EM(2) 
	D1EP(1) = (D1EP1+S*RD1EP1)/dpi + D1EP(1) 
	D1EP(2) = (D1EP2+S*RD1EP2)/dpi + D1EP(2) 
*
	IF(TYNOYO(:3).NE.'DIR') THEN 
	 D2EMP(1)= (D2EMP1+S*RD2EMP1)/dpi + D2EMP(1)
	 D2EMP(2)= (D2EMP2+S*RD2EMP2)/dpi + D2EMP(2)
	 D2EMP(3)= (D2EMP3+S*RD2EMP3)/dpi + D2EMP(3)
	 D2EMP(4)= (D2EMP4+S*RD2EMP4)/dpi + D2EMP(4)
	END IF
*
*-----------------------------------------------------------------------
*     FIN lorsque nu=0
*-----------------------------------------------------------------------
	IF(ANU.LE.1.E-9) RETURN
*-----------------------------------------------------------------------
*     Initialisation de variables 
*-----------------------------------------------------------------------
	Y     = CORP(2)+CORM(2)
	ZI    = CMPLX(0.,1.) 
	Z     = CMPLX(Y,X) 
	ZB    = CMPLX(Y,-X) 
	NUZ   = ANU*Z  
	NUZB  = ANU*ZB  
	ONUZ  = 1./NUZ
	ONUZB = 1./NUZB
	ONUZ2 = ONUZ*ONUZ 
	ONUZB2= ONUZB*ONUZB 
	ENUZ  = EXP(NUZ)
	ENUZB = EXP(NUZB)
*-----------------------------------------------------------------------
*     Soustraction de H1=1/2pi*(exp(nu*z)E1(nu*z)+exp(nu*zb)E1(nu*zb)) 
*-----------------------------------------------------------------------
*     Calcul de CEE1(nu*Z)= EXP(nu*Z)*E1(nu*Z) 
	ZH1=CEE1(NUZ) 
	ZH1B=CEE1(NUZB) 
*
	Z1    = ANU*(ZH1-ONUZ)
	Z1P1  =  ZI*Z1
	Z1P2  =     Z1
	Z1M1  = -ZI*Z1
	Z1M2  =     Z1
*
	IF(TYNOYO(:3).NE.'DIR') THEN 
	 Z2    = ANU*ANU*(ZH1-ONUZ+ONUZ2)
	 Z2MP1 =     Z2
	 Z2MP2 = -ZI*Z2
	 Z2MP3 =  ZI*Z2
	 Z2MP4 =     Z2
	END IF
*
	ZB1   = ANU*(ZH1B-ONUZB)
	ZB1P1 = -ZI*ZB1
	ZB1P2 =     ZB1
	ZB1M1 =  ZI*ZB1
	ZB1M2 =     ZB1
*
	IF(TYNOYO(:3).NE.'DIR') THEN 
	 ZB2   = ANU*ANU*(ZH1B-ONUZB+ONUZB2)
	 ZB2MP1=     ZB2
	 ZB2MP2=  ZI*ZB2
	 ZB2MP3= -ZI*ZB2
	 ZB2MP4=     ZB2
	END IF
*
	E      = E - (ZH1 + ZH1B)/dpi
	D1EM(1) = D1EM(1) - (Z1M1+ZB1M1)/dpi
	D1EM(2) = D1EM(2) - (Z1M2+ZB1M2)/dpi
	D1EP(1) = D1EP(1) - (Z1P1+ZB1P1)/dpi
	D1EP(2) = D1EP(2) - (Z1P2+ZB1P2)/dpi
*
	IF(TYNOYO(:3).NE.'DIR') THEN 
	 D2EMP(1)= D2EMP(1) - (Z2MP1+ZB2MP1)/dpi
	 D2EMP(2)= D2EMP(2) - (Z2MP2+ZB2MP2)/dpi
	 D2EMP(3)= D2EMP(3) - (Z2MP3+ZB2MP3)/dpi
	 D2EMP(4)= D2EMP(4) - (Z2MP4+ZB2MP4)/dpi 
	END IF
*
*-----------------------------------------------------------------------
*     Soustraction de H2=i/2*(1+sgn(nu*x))*exp(nu*z) 
*               et de H3=i/2*(1-sgn(nu*x))*exp(nu*zb) 
*-----------------------------------------------------------------------
	RNUZ=AIMAG(NUZ)
*
	IF (RNUZ.LT.0.) THEN
	 ZH2  = (0.,0.)
	 ZH3  = ZI*ENUZB
	ELSE IF (RNUZ.EQ.0.) THEN 
	 ZH2  = ZI*ENUZ
	 ZH3  = ZI*ENUZB
	ELSE
	 ZH2  = ZI*ENUZ
	 ZH3  = (0.,0.)
	ENDIF
*
	Z1   = ANU*ZH2
	Z1P1 =  ZI*Z1
	Z1P2 =     Z1
	Z1M1 = -ZI*Z1
	Z1M2 =     Z1
*
	IF(TYNOYO(:3).NE.'DIR') THEN 
	 Z2    = ANU*ANU*ZH2
	 Z2MP1 =     Z2
	 Z2MP2 = -ZI*Z2
	 Z2MP3 =  ZI*Z2
	 Z2MP4 =     Z2
	END IF
*
	ZB1  = ANU*ZH3
	ZB1P1= -ZI*ZB1
	ZB1P2=     ZB1
	ZB1M1=  ZI*ZB1
	ZB1M2=     ZB1
*
	IF(TYNOYO(:3).NE.'DIR') THEN 
	 ZB2   = ANU*ANU*ZH3
	 ZB2MP1=     ZB2
	 ZB2MP2=  ZI*ZB2
	 ZB2MP3= -ZI*ZB2
	 ZB2MP4=     ZB2
	END IF
*
	E      = E - (ZH2 + ZH3)
*
	D1EP(1) = D1EP(1) - (Z1P1+ZB1P1)
	D1EP(2) = D1EP(2) - (Z1P2+ZB1P2)
	D1EM(1) = D1EM(1) - (Z1M1+ZB1M1)
	D1EM(2) = D1EM(2) - (Z1M2+ZB1M2)
*
	IF(TYNOYO(:3).NE.'DIR') THEN 
	 D2EMP(1)= D2EMP(1) - (Z2MP1+ZB2MP1)
	 D2EMP(2)= D2EMP(2) - (Z2MP2+ZB2MP2)
	 D2EMP(3)= D2EMP(3) - (Z2MP3+ZB2MP3)
	 D2EMP(4)= D2EMP(4) - (Z2MP4+ZB2MP4) 
	END IF
	                                                            END !Grw2a
