*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE ASIDMO (OPERAT,TYPCST,CHCSTE,NUMCST
     &                  ,NMTRMB,NVTRMB,TYTRMB,ITYSYB,NMSTOK,NVSTOK
     &                  ,NMTRMR,NVTRMR,TYTRMR,ITYSYR
     &                  ,NCPINC,NCPINL)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Avril 2002)
* Derniere modification : D.Martin (24 Avril 2002)
* Version 1.0.1
*
* Assemblage d'une matrice stockee Bi-Morse, Morse-Ligne ou Morse-Colonne (B)
* dans une matrice (R) stockee selon des structures identiques.
*
*-- Arguments d'entree --
* OPERAT operateur de combinaison lineaire ('*', '+' ou '-')
* TYPCST type (entier, reel ou complexe) de la constante si OPERAT='*'
* CHCSTE le cas echeant, nom du tableau des constantes du type ad'hoc
* NUMCST son numero dans la liste des constantes du type ad'hoc
* NMTRMB nom du terme a combiner (B)
* NVTRMB son niveau
* TYTRMB type (reel ou complexe) du terme (B)
* ITYSYB indicateur de symetrie du terme (B)
* NMSTOK nom du tableau de stockage du terme (B) (et aussi de (S))
* NVSTOK niveau du tableau de stockage du terme (B)
* NMTRMR nom du terme Morse resultat (R)
* NVTRMR son niveau
* TYTRMR type (reel ou complexe) du terme resultat (R)
* ITYSYR indicateur de symetrie du terme resultat (R)
* NCPINC nombre de composante de l'inconnue (vectorielle) en colonne
* NCPINL nombre de composante de l'inconnue (vectorielle) en ligne
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) OPERAT,TYPCST,CHCSTE,NMTRMB,TYTRMB,NMSTOK
     &             ,NMTRMR,TYTRMR
      INTEGER       NUMCST,NVTRMB,ITYSYB
     &             ,NVSTOK,NVTRMR,ITYSYR,NVSTOR
     &             ,NCPINC,NCPINL,NIVIMP,IMPFCH
*
      INCLUDE 'ALLOC'
      INCLUDE 'CONTEX'
*
      INTEGER       MCSTOK,MCTRMR,MCTRMB,NBL,NBPL,MCPLAG,MCPLIG,LGLB
     &             ,NBC,NBPC,MCPLAU,MCPLIU,LGUB,LGDB,NCFSSD,MCCSTE
     &             ,LGTRMB
      CHARACTER     SYMIND*14,TYPINT*4
      CHARACTER     TYPSYM*14
      CHARACTER     ERCODE*120
      COMMON/FORMAH/ERCODE
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*Asidmo*')
*
      CALL TBAR3  (ERCODE,NMTRMR,NVTRMR,MCTRMR,NMSTOK,NVSTOK,MCSTOK
     &                   ,NMTRMB,NVTRMB,MCTRMB)
*
      IF (NMSTOK.EQ.'&BMORS') THEN
*
*        Assemblage de matrices Bi-Morse
*
         CALL EXBMOR (MCSTOK,IST(MCSTOK),NBL,NBPL,MCPLAG,MCPLIG,LGLB
     &                                  ,NBC,NBPC,MCPLAU,MCPLIU,LGUB)   !sdexplo
         LGDB=MIN (NBL*NCPINL,NBC*NCPINC)
         NCFSSD=MAX (NCPINL,NCPINC)
         IF (NCFSSD.LT.3) NCFSSD=NCFSSD-1
         LGLB=LGLB*NCPINC*NCPINL+MIN(NBL*NCFSSD,NBC*NCFSSD)
         LGUB=LGUB*NCPINC*NCPINL+MIN(NBC*NCFSSD,NBL*NCFSSD)
*
*        Longueur du tableau contenant la matrice B
*         (si la matrice B n'est pas symetrique, on ajoute
*          la longueur de sa partie triangulaire superieure)
         LGTRMB=LGDB+LGLB
         IF (ITYSYB.EQ.NDFSYM) LGTRMB=LGTRMB+LGUB            
*
*        Assemblage dans la partie triangulaire inferieure
*        (et la partie triangulaire superieure si la matrice n'est
*         ni symetrique, ni antisymetrique, ni autoadjointe, ni antiadjointe)
*
         CALL ASMCLI (OPERAT,CHCSTE,TYPCST,NUMCST
     &               ,TYTRMB,MCTRMB,LGTRMB,TYTRMR,MCTRMR,IST,RST,CST)   !assembl
*
         IF (ITYSYR.NE.ITYSYB) THEN
*
*           Assemblage de la partie triangulaire superieure
*
            IF (OPERAT(1:1).EQ.'*') CALL TBAR1  (ERCODE,CHCSTE,1,MCCSTE)
            TYPSYM=SYMIND (ITYSYB)                                      !utilite
*
            IF (TYPSYM(5:12).EQ.'ADJOINTE')
*           Matrice (B) autoadjointe ou antiadjointe
     &         CALL TCDANT (LGLB,TYTRMB,MCTRMB,RST,CST 
     &                          ,TYTRMB,MCTRMB,RST,CST)                 !utilite
*           Matrice (B) antisymetrique ou antiadjointe
            IF (TYPSYM(1:4).EQ.'ANTI')
     &         CALL ASMNSY (OPERAT,TYPCST,MCCSTE+NUMCST,IST,RST,CST)    !assembl
*
               CALL ASMCLI (OPERAT,CHCSTE,TYPCST,NUMCST,TYTRMB
     &                     ,MCTRMB+LGDB,LGLB,TYTRMR,MCTRMR+LGTRMB
     &                     ,IST,RST,CST)                                !assembl
*
               IF (TYPSYM(5:12).EQ.'ADJOINTE')
*              Matrice (B) autoadjointe ou antiadjointe
     &            CALL TCDANT (LGLB,TYTRMB,MCTRMB,RST,CST 
     &                             ,TYTRMB,MCTRMB,RST,CST)              !utilite
*              Matrice (B) antisymetrique ou antiadjointe
               IF (TYPSYM(1:4).EQ.'ANTI')
     &            CALL ASMNSY (OPERAT,TYPCST,MCCSTE+NUMCST,IST,RST,CST) !assembl
*
          ENDIF

      ELSEIF (NMSTOK.EQ.'&PLAGE') THEN
*        Assemblage de matrices Morse-L
         CALL EXMORS (MCSTOK,IST(MCSTOK)
     &               ,NBL,NBC,NBPL,MCPLAG,MCPLIG,LGLB)                  !sdexplo
         LGTRMB=LGLB*NCPINL*NCPINC
         CALL ASMCLI (OPERAT,CHCSTE,TYPCST,NUMCST
     &               ,TYTRMB,MCTRMB,LGTRMB,TYTRMR,MCTRMR,IST,RST,CST)   !assembl
      ELSEIF (NMSTOK.EQ.'&EGALP') THEN
*        Assemblage de matrices Morse-C
         CALL EXMORS (MCSTOK,IST(MCSTOK)
     &               ,NBC,NBL,NBPL,MCPLAG,MCPLIG,LGLB)                  !sdexplo
         LGTRMB=LGLB*NCPINL*NCPINC
         CALL ASMCLI (OPERAT,CHCSTE,TYPCST,NUMCST
     &               ,TYTRMB,MCTRMB,LGTRMB,TYTRMR,MCTRMR,IST,RST,CST)   !assembl
      ELSE
         CALL ERTERS (11,NMTRMR,NVTRMR,NMTRMB,NVTRMB)                   !utilite
      ENDIF
*
      CALL PRFXMJ (-1,'*Asidmo*')  
                                                                    END !Asidmo
