*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE ASMPVM (MOOUBM,NCPINL,NCPINC
     &                ,NBNLIB,NGNLIB,NBNCOB,NGNCOB,NULAPB,IPLAGB,NMBPTR
     &                ,NBNLIS,NGNLIS,NBNCOS,NGNCOS,NULAPS,IPLAGS,IDBPTR
     &                ,NULAPU,IPLAGU,IDBPTU,PTRB2S)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Janvier 1992)
* Derniere modification : D.Martin (5 novembre 2008)
* Version 2
*  
* Recherche dichotomique du tableau de pointeurs pour l'assemblage d'une matrice
* BiMorse, Morse-L ou Morse-C (B) dans le cas d'inconnues vectorielles
*  
*-- Arguments d'entree --
* MOOUBM =0 pour le cas Morse-Ligne ou Morse-Colonne,
*        =1 pour le cas BiMorse
* NCPINL nombre de composantes de l'inconnue en ligne
* NCPINC nombre de composantes de l'inconnue en colonne
*        =NCPINL dans le cas Bimorse
* NBNLIB nombre de lignes du terme matriciel a assembler (B)
* NGNLIB numerotation en ligne du terme matriciel a assembler (B)
* NBNCOB nombre de colonnes du terme matriciel a assembler (B)
* NGNCOB numerotation en colonne du terme matriciel a assembler (B)
* NULAPB tableau des numeros de derniere plage des lignes de (B)
* IPLAGB tableau de stockage Morse-Ligne de la matrice (B)
* NBNLIS nombre de lignes du terme matriciel resultat (S)
* NGNLIS numerotation en ligne du terme matriciel resultat (S)
* NBNCOS nombre de lignes du terme matriciel resultat (S)
* NGNCOS numerotation en colonne du terme matriciel resultat (S)
* NULAPS tableau des numeros de derniere plage des lignes de (S)
* IPLAGS tableau de stockage Morse-Ligne de la matrice (S)
* IDBPTR adresse relative des pointeurs dans le triangle inf. de (S)
* NULAPU tableau des numeros de derniere plage des colonnes de (S)
* IPLAGU tableau de stockage Morse-Colonne de la matrice (S)
* IDBPTU adresse relative des pointeurs dans le triangle sup. de (S)
*-- Arguments de sortie --
* NMBPTR nombre de coefficients de (B) a assembler dans (S)
* PRTB2S tableau de pointeurs d'assemblage d'une matrice dans une autre.
*        Sa taille est celle de la matrice (B) a assembler dans (S)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      INTEGER MOOUBM,NCPINL,NCPINC
     &       ,NBNLIB,NGNLIB(*),NBNCOB,NGNCOB(*),NULAPB(*),IPLAGB(2,*)
     &       ,NBNLIS,NGNLIS(*),NBNCOS,NGNCOS(*),NULAPS(*),IPLAGS(2,*)
     &       ,NMBPTR,IDBPTR,NULAPU(*),IPLAGU(2,*),IDBPTU,PTRB2S(*)
*
      INTEGER NCFSSD,MOOUB1,LS,NEXTLS,KS,NEXTKS,NSNEMI,NBNEMI,LGCUMS
     &       ,NUPRPS,NUDRPS,NUPLS,LGCUAS,NUPRUS,NUDRUS,NUPLU,LGLIGS
     &       ,LGCUVS,LGCUSV,LGCUWS,LGCUUS,LGCOGS,LGCUVU,LGCUWU
     &       ,NUPRPB,NUDRPB,NUPLB,LGCUAB,LGCUMB,LGLIGB
     &       ,KOLDEB,KOLFIN,NXTPTR,LB,KB,IDLL,IDLC
*-----------------------------------------------------------------------
*     Nombre de CoeFficients SouS-Diagonaux d'un bloc diagonal
*     cree pour les inconnues vectorielles. Cas Bimorse seulement
      NCFSSD=0
      IF (MOOUBM.EQ.1) NCFSSD=((NCPINC-1)*NCPINC)/2
      MOOUB1=1+MOOUBM
*
      NEXTLS=1
      LGCUAS=0
      LGCUMS=0
      NUDRPS=0
      NUDRUS=0
      NSNEMI=MIN (NBNLIS,NBNCOS)
      LGCUAB=0
      LGCUMB=0
      NUDRPB=0
      NBNEMI=MIN (NBNLIB,NBNCOB)
*
*     Parcours des lignes LB de la matrice B a assembler dans S
      DO 50 LB=1,NBNLIB
*        Premiere et derniere plage de la ligne courante LB de B
         NUPRPB=NUDRPB+1
         IF (LB.GT.MOOUBM) NUDRPB=NULAPB(LB-MOOUBM)
         IF (NUDRPB.GT.0) LGCUMB=IPLAGB(2,NUDRPB)
         LGLIGB=LGCUMB-LGCUAB
*        Recherche de la ligne LS de S 'contenant' la ligne LB de B
         CALL DICOTO (NEXTLS,NBNLIS,NGNLIS,NGNLIB(LB),LS,*1175)         !utilite
         NEXTLS=LS+1
*        Premiere, derniere plage de la ligne LS correspondant dans S
         NUPRPS=NUDRPS+1
         IF (LS.GT.MOOUB1) NUPRPS=1+NULAPS(LS-MOOUB1)
         IF (NUPRPS.GT.1) LGCUAS=IPLAGS(2,NUPRPS-1)
         IF (LS.GT.MOOUBM) NUDRPS=NULAPS(LS-MOOUBM)
         IF (NUDRPS.GT.0) LGCUMS=IPLAGS(2,NUDRPS)
         LGLIGS=LGCUMS-LGCUAS
         LGCUVS=IDBPTR+LGCUAS*NCPINC*NCPINL+NCFSSD*MIN(LS-1,NSNEMI)
         LGCUSV=LGCUVS
*        Parcours des plages et des colonnes KB de la ligne LB de B
         NEXTKS=1
         DO 26 NUPLB=NUPRPB,NUDRPB
            KOLDEB=IPLAGB(1,NUPLB)
            LGCUMB=IPLAGB(2,NUPLB)
            KOLFIN=KOLDEB+LGCUMB-LGCUAB-1
            LGCUAB=LGCUMB
            DO 25 KB=KOLDEB,KOLFIN
*              Recherche de la colonne KS de la ligne LS de S contenant
*              la colonne KB de B
               CALL DICOTO (NEXTKS,NBNCOS,NGNCOS,NGNCOB(KB),KS,*1176)   !utilite
               NEXTKS=KS+1
*              Recherche de la colonne KS dans le triangle inf. de S
               CALL DIKLPL (KS,NUPRPS,NUDRPS,LGCUAS,IPLAGS,NUPLS,*16)  !morse
               IF (NUPLS.GT.NUPRPS) THEN
*                 Changement de plage sur la ligne LS de S
                  LGCUMS=IPLAGS(2,NUPLS-1)
                  LGCUVS=LGCUVS+(LGCUMS-LGCUAS)*NCPINC
                  LGCUAS=LGCUMS
                  NUPRPS=NUPLS
               ENDIF
*
               LGCUWS=LGCUVS+(KS-IPLAGS(1,NUPLS))*NCPINC
               NXTPTR=NMBPTR
               DO 14 IDLL=1,NCPINL
                  DO 12 IDLC=1,NCPINC
                     PTRB2S(NXTPTR+IDLC)=LGCUWS+IDLC
12                CONTINUE
                  NXTPTR=NXTPTR+LGLIGB*NCPINC
                  IF (LB.LE.NBNEMI.AND.MOOUBM.EQ.1) NXTPTR=NXTPTR+IDLL-1
                  LGCUWS=LGCUWS+LGLIGS*NCPINC
                  IF (LS.LE.NSNEMI.AND.MOOUBM.EQ.1) LGCUWS=LGCUWS+IDLL-1
14             CONTINUE
               GOTO 24
*              On est dans le triangle sup. de S
16             NUPRUS=1
               IF (KS.GT.MOOUB1) NUPRUS=1+NULAPU(KS-MOOUB1)
               LGCUUS=0
               IF (NUPRUS.GT.1) LGCUUS=IPLAGU(2,NUPRUS-1)
               IF (KS.GT.MOOUBM) NUDRUS=NULAPU(KS-MOOUBM)
               LGCOGS=IPLAGU(2,NUDRUS)-LGCUUS
               LGCUVU=IDBPTU+LGCUUS*NCPINC*NCPINL
     &                      +NCFSSD*MIN(KS-1,NSNEMI)
               CALL DIKLPL (LS,NUPRUS,NUDRUS,LGCUUS,IPLAGU,NUPLU,*187)  !morse
               IF (NUPLU.GT.NUPRUS) THEN
*                 Changement de plage sur la Colonne KS de S
                  LGCUMS=IPLAGU(2,NUPLU-1)
                  LGCUVU=LGCUVU+(LGCUMS-LGCUUS)*NCPINL
                  LGCUUS=LGCUMS
                  NUPRUS=NUPLU
               ENDIF
               LGCUVU=LGCUVU+(LS-IPLAGU(1,NUPLU))*NCPINL
               NXTPTR=NMBPTR
               DO 174 IDLL=1,NCPINL
                  LGCUWU=LGCUVU+IDLL
                  DO 172 IDLC=1,NCPINC
                     PTRB2S(NXTPTR+IDLC)=LGCUWU
                     LGCUWU=LGCUWU+LGCOGS*NCPINL
                     IF (KS.LE.NSNEMI.AND.MOOUBM.EQ.1)
     &                  LGCUWU=LGCUWU+IDLC-1
172               CONTINUE
                  NXTPTR=NXTPTR+LGLIGB*NCPINC
                  IF (LB.LE.NBNEMI.AND.MOOUBM.EQ.1) NXTPTR=NXTPTR+IDLL-1
174            CONTINUE
               GOTO 24
*              On est sur un bloc de la diagonale de S
187            LGCUWS=LGCUSV+LGLIGS*NCPINC
               LGCUVU=LGCUVU+LGCOGS*NCPINL
*              sous-diagonale de S
               NXTPTR=NMBPTR
               DO 182 IDLL=2,NCPINL
                  NXTPTR=NXTPTR+LGLIGB*NCPINC
                  IF (LB.LE.NBNEMI.AND.MOOUBM.EQ.1) NXTPTR=NXTPTR+IDLL-2
                  LGCUWS=LGCUWS+LGLIGS*NCPINC
                  IF (LS.LE.NSNEMI.AND.MOOUBM.EQ.1) LGCUWS=LGCUWS+IDLL-2
                  DO 182 IDLC=1,IDLL-1
                     PTRB2S(NXTPTR+IDLC)=LGCUWS+IDLC
182            CONTINUE
*              diagonale de S
               NXTPTR=NMBPTR
               DO 184 IDLL=1,NCPINL
                  PTRB2S(NXTPTR+IDLL)=(LS-1)*NCPINL+IDLL
                  NXTPTR=NXTPTR+LGLIGB*NCPINC
                  IF (LB.LE.NBNEMI.AND.MOOUBM.EQ.1) NXTPTR=NXTPTR+IDLL-1
184            CONTINUE
*              sur-diagonale de S
               DO 186 IDLC=2,NCPINC
                  NXTPTR=NMBPTR
                  LGCUVU=LGCUVU+LGCOGS*NCPINL
                  IF (LS.LE.NSNEMI.AND.MOOUBM.EQ.1) LGCUVU=LGCUVU+IDLC-2
                  DO 186 IDLL=1,IDLC-1
                     PTRB2S(NXTPTR+IDLC)=LGCUVU+IDLL
                     NXTPTR=NXTPTR+LGLIGB*NCPINC
                     IF (LB.LE.NBNEMI.AND.MOOUBM.EQ.1)
     &                  NXTPTR=NXTPTR+IDLL-1
186            CONTINUE
24             NMBPTR=NMBPTR+NCPINC
25          CONTINUE
26       CONTINUE
*
         IF (MOOUBM.EQ.1.AND.LB.LE.NBNEMI) THEN
*
*           Cas de coefficients sous-diagonaux des blocs diagonaux de B
*
*           Recherche de la colonne KS de la ligne LS de S contenant
*           la colonne KB de B.
            KB=LB
            NEXTKS=1
            CALL DICOTO (NEXTKS,NBNCOS,NGNCOS,NGNCOB(KB),KS,*1176)      !utilite
            NEXTKS=KS+1
*           Recherche de la colonne KS dans le triangle inferieur de S
            CALL DIKLPL (KS,NUPRPS,NUDRPS,LGCUMS,IPLAGS,NUPLS,*197)     !morse
            IF (NUPLS.GT.NUPRPS) THEN
*              Changement de plage sur la ligne LS de S
               LGCUMS=IPLAGS(2,NUPLS-1)
               LGCUVS=LGCUVS+(LGCUMS-LGCUAS)*NCPINC
               LGCUAS=LGCUMS
               NUPRPS=NUPLS
            ENDIF
            LGCUWS=LGCUVS+(KS-IPLAGS(1,NUPLS))*NCPINC
*           Partie triangulaire inferieure de S
            DO 18 IDLL=2,NCPINC
               NMBPTR=NMBPTR+LGLIGB*NCPINC+IDLL-2
               LGCUWS=LGCUWS+LGLIGS*NCPINC
               IF (LS.LE.NSNEMI) LGCUWS=LGCUVS+IDLL-2
               DO 18 IDLC=1,IDLL-1
                  PTRB2S(NMBPTR+IDLC)=LGCUWS+IDLC
18             CONTINUE
            GOTO 19
*           Partie triangulaire superieure de S
197         NUPRUS=1
            IF (KS.GT.MOOUB1) NUPRUS=1+NULAPU(KS-MOOUB1)
            LGCUUS=0
            IF (NUPRUS.GT.1) LGCUUS=IPLAGU(2,NUPRUS-1)
            NUDRUS=0
            IF (KS.GT.MOOUBM) NUDRUS=NULAPU(KS-MOOUBM)
            IF (NUDRPS.GT.0) LGCUMS=IPLAGU(2,NUDRUS)
            LGCOGS=LGCUMS-LGCUUS
            LGCUVU=IDBPTU+LGCUUS*NCPINC*NCPINC+NCFSSD*MIN(KS-1,NSNEMI)
            CALL DIKLPL (LS,NUPRUS,NUDRUS,LGCUUS,IPLAGU,NUPLU,*207)     !morse
            IF (NUPLU.GT.NUPRUS) THEN
*              Changement de plage sur la Colonne KS de S
               LGCUMS=IPLAGU(2,NUPLU-1)
               LGCUVU=LGCUVU+(LGCUMS-LGCUUS)*NCPINC
               LGCUUS=LGCUMS
               NUPRUS=NUPLU
            ENDIF
            LGCUVU=LGCUVU+(LS-IPLAGU(1,NUPLU))*NCPINC
            DO 192 IDLL=2,NCPINC
               NMBPTR=NMBPTR+LGLIGB*NCPINC+IDLL-2
               LGCUWU=LGCUVU+IDLL
               DO 192 IDLC=1,IDLL-1
                  PTRB2S(NMBPTR+IDLC)=LGCUWU
                  LGCUWU=LGCUWU+LGCOGS*NCPINC
                  IF (KS.LE.NSNEMI) LGCUWU=LGCUWU+IDLC-1
192         CONTINUE
            GOTO 19
*           Bloc diagonaux de la matrice (S)
207         LGCUVS=LGCUSV+LGLIGS*NCPINC
*           Cas de coefficients sous-diagonaux d'un bloc diagonal
*           d'une matrice (S) BiMorse
            DO 202 IDLL=2,NCPINC
               NMBPTR=NMBPTR+LGLIGB*NCPINC+IDLL-2
               LGCUVS=LGCUVS+LGLIGS*NCPINC
               IF (LS.LE.NSNEMI) LGCUVS=LGCUVS+IDLL-2
               DO 202 IDLC=1,IDLL-1
                  PTRB2S(NMBPTR+IDLC)=LGCUVS+IDLC
202         CONTINUE
19          NMBPTR=NMBPTR+NCPINC-1
*
****** Lignes mises en commentaires pour bug (E. Luneville)
*****    ELSE
*****       NMBPTR=NMBPTR+2*LGLIGB*NCPINL
******
         ENDIF
50    CONTINUE
*
      RETURN
*-------------------------------------------------------------------------
1175  CALL BAISE('*Asmpvm* On ne retrouve pas la ligne   de B dans '
     &//'celles de S')
1176  CALL BAISE('*Asmpvm* On ne retrouve pas la colonne de B dans '
     &//'celles de S')
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 2 : D.Martin (5 novembre 2008)
*  Bug dans le cas non BiMorse ( NCFSSD = 0 dans ce cas )
* Version 1 : O.DeBayser (3 Avril 1996)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !AsMPvM
