*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE DSNUME (NMTERM,NVTERM,NVDSMB,NIVIMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin & O.DeBayser (Fevrier 1992)
* Refonte complete : D.Martin (Avril 1997)
* Derniere modification : D.Martin (8 Juin 2000)
* Version 1.0.1
*  
* Creation des numerotations en colonne et en ligne par inconnue pour 
* l'assemblage multi-inconnue du terme NMTERM/NVTERM. 
* Pour chaque colonne/ligne de termes de la matrice multi-inconnues, on cree
* une numerotation, qui est l'union des numerotations en colonne/ligne des
* termes de la colonne/ligne.
* La relative complexite de la procedure resulte du choix de ne creer aucun
* tableau de numerotation en trop.
*  
*-- Arguments d'entree --
* NMTERM nom du terme assemble multi-inconnues
* NMTERM niveau
* NVDSMB niveau de la structure $DSMBL sans l'entete
* NIVIMP niveau d'impression
*
*-- Structure $DSMBL
* (1)=LGEDSM (=4)
* (2)=NBLOCC nombre de termes en colonne
* (3)=INCDSM nb. de caracteristiques par terme de la structure
* (4)=NBLOCL nombre de termes en ligne
* puis pour chaque terme de la structure (I=1,NBLOCC*NBLOCL)
* (.)   numero du terme dans la structure $SDTRM (NUTERA)
* (.+1) numero de l'inconnue en colonne du terme (NUINCC)
* (.+2) numero de l'inconnue en ligne du terme   (NUINCL)
* (.+3) niveau de la numerotation en colonne du terme (NVNUMC/NVDUMC)
* (.+4) niveau de la numerotation en ligne du terme   (NVNUML/NVDUML)
* (.+5) inutilisee dans cette procedure
* ou  '.' designe LGEDSM+INCDSM*(I-1)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) NMTERM
      INTEGER       NVTERM,NVDSMB,NIVIMP
*
      INCLUDE 'ALLOC'  
      INCLUDE 'CONTEX' 
*
      INTEGER KLNIVE
      INTEGER IDLHRM,MCDTRM,ADDTRM,ADWTRM,MCDSMB,MCWOR1,NBDLMX
     &       ,MCCONU,MCNUMW,MCNUME
      INTEGER LGETRM,NBTERM,INCTRM,NCHTRM,LGEDSM,NBLOCC,INCDSM,NBLOCL
     &       ,NULOCC,NULOCL,NBDL,NBDLK,ITEMDB,ITEMFN,INTRP,NBT
     &       ,LGNUME,KOOULI,INCREX,LRIPEX,INCRIN,LRIPIN,LRIPNN
     &       ,NVTERA,ITYPTR,ITYCAL,NUKALE,NUDONN,ITYSYM,NVDSSS,NUINCC
     &       ,NVDUMC,NBNECO,NVNUMC,NUINCL,NVDUML,NBNELI,NVNUML,INSTOK
     &       ,NVSTOK,NVCOOR,IATRDO,NIVITR
     &       ,NWTERA,IWYPTR,IWYCAL,NWKALE,NWDONN,IWYSYM,NWDSSS,NWINCC
     &       ,NWDUMC,NWNECO,NWNUMC,NWINCL,NWDUML,NWNELI,NWNUML,IWDSTK
     &       ,NWSTOK,NWCOOR,IWTRDO,NWVITR,NWBLOC,NCPIN,NCATR1
     &       ,NVNUME,NVNUMK,NVNURA,NVNURE,NVDEJA,NUTERA,NWTERM,NBNURA
      CHARACTER ERCODE*120,MODTRM,CRENUM,MODADR
      COMMON/FORMAH/ERCODE 
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*DsNume*')
*
*     Cas d'elements d'Hermite supprime
*
      IDLHRM=0
      CALL TBAR1  (ERCODE,'&WORK$',1,MCWOR1)                     
*
*     Tableau de numerotation provisoire
*
      NBDLMX       =MAX (IST(MCWOR1),IST(MCWOR1+1))
      LGNUME       =1+(NBDLMX+1)*(IDLHRM+1)
      IST(MCWOR1)  =0
      IST(MCWOR1+1)=0
      CALL TBCREE ('&WORK$',2,1,LGNUME,'c')
*
*     Attributs de definition des structures $SDTRM et $DSMBL
*
      CALL TBAR2  (ERCODE,'$SDTRM',1,MCDTRM,'$DSMBL',NVDSMB,MCDSMB)
      CALL SDEXDB (IST(MCDTRM),LGETRM,NBTERM,INCTRM,NCHTRM)             !utilite
      CALL SDEXDB (IST(MCDSMB),LGEDSM,NBLOCC,INCDSM,NBLOCL)             !utilite
*
*
*     1) Numerotations en colonne pour chaque bloc de colonnes
*
      KOOULI=3
*     Increments de parcours par colonne de la structure $DSMBL
      INCRIN=INCDSM
      INCREX=INCDSM*NBLOCL
*
      CALL TBAR1  (ERCODE,'$DSMBL',NVDSMB,MCDSMB)
      MODADR='Y'
      LRIPEX=LGEDSM
*
*     Boucle sur les colonnes de termes de la matrice finale
*
      DO 120 NULOCC=1,NBLOCC
         LRIPIN=LRIPEX
*
*        Niveau de la numerotation en colonne de tous les termes
*        de la colonne de termes de la matrice
*
         NVNUME=IST(MCDSMB+LRIPEX+KOOULI)
         IF (NVNUME.EQ.NDFNUM) THEN
            NBDL  =0
            NBDLK =0
            NVNUMK=NDFNUM
*
*           Boucle sur tous les termes dans la colonne de la matrice
*
            DO 105 NULOCL=1,NBLOCL
               NUTERA=IST(MCDSMB+LRIPIN)
               IF (NUTERA.GT.0) THEN
                  IF (MODADR.EQ.'Y')
     &            CALL TBAR2  (ERCODE,'#NCONU',1,MCCONU
     &                               ,'$SDTRM',1,MCDTRM)
                  MODADR='N'
                  NVNURA=NDFNUM
                  NBNURA=0
                  MODTRM='N'
                  CRENUM='N'
                  NVDEJA=0
*
*                 Attributs du terme courant, correspondant au bloc
*                 (NULOCL,NULOCC) dans la matrice assemblee finale
*
                  ADDTRM=MCDTRM+LGETRM+INCTRM*(NUTERA-1)
                  CALL GETTRM (IST(ADDTRM),NVTERA,ITYPTR,ITYCAL,NUKALE
     &                        ,NUDONN,ITYSYM,NVDSSS,NUINCC,NVDUMC,NBNECO
     &                        ,NVNUMC,NUINCL,NVDUML,NBNELI,NVNUML,INSTOK
     &                        ,NVSTOK,NVCOOR,IATRDO,NIVITR)             !sdexplo
*
*                 Attributs de l'inconnue en colonne
*
                  CALL SDEXCO (NUINCC,IST(MCCONU),NCPIN,INTRP,NBT)      !utilite
                  NVNURE=NVNUMC
*
*                 Recherche dans la meme colonne d'un terme ayant
*                 la meme numerotation en colonne que le terme courant
*
                  LRIPNN=LRIPEX
                  DO 101 NWBLOC=1,NULOCL-1
                     NWTERM=IST(MCDSMB+LRIPNN)
                     IF (NWTERM.GT.0) THEN
                        ADWTRM=MCDTRM+LGETRM+INCTRM*(NWTERM-1)
                        CALL GETTRM (IST(ADWTRM),NWTERA,IWYPTR,IWYCAL
     &                              ,NWKALE,NWDONN,IWYSYM,NWDSSS
     &                              ,NWINCC,NWDUMC,NWNECO,NWNUMC
     &                              ,NWINCL,NWDUML,NWNELI,NWNUML
     &                              ,IWDSTK,NWSTOK,NWCOOR,IWTRDO,NWVITR)!sdexplo
                        IF (NVNURE.EQ.NWNUMC) THEN
                           NVDEJA=NWNUMC
                           IF (NCPIN.GT.1) THEN
                              NVDUMC=NWDUMC
                              NVDEJA=NWDUMC
                           ENDIF
                           MODTRM='Y'
                           GOTO 102
                        ENDIF
                     ENDIF
                     LRIPNN=LRIPNN+INCRIN
101               CONTINUE
*
*                 Dans le cas d'inconnue vectorielle, on creera tout d'abord
*                 la numerotation en colonne pour les d.l. vectoriels
*                 pour le terme courant
*
102               IF (NCPIN.GT.1) THEN
                     IF (NVDUMC.EQ.NDFDUM) THEN
                        NVDUMC=KLNIVE ()                                !sdexplo
                        CRENUM='Y'
                        MODTRM='Y'
                        NVNURA=NVNURE
                        NBNURA=NBNECO
                     ENDIF
                     NVNURE=NVDUMC
*                    Terme ayant la meme inconnue en ligne qu'en colonne
*                    (cas courant pour les termes diagonaux)
                     IF (NVDUMC.NE.NDFNUM.AND.NVNUMC.EQ.NVNUML
     &                                   .AND.NUINCC.EQ.NUINCL) THEN
                        NVDUML=NVDUMC
                        MODTRM='Y'
                     ENDIF
                  ENDIF
*
*                 Les attributs du terme courant ont ete modifies
*
                  IF (MODTRM.EQ.'Y')
     &               CALL PUTTRM (IST(ADDTRM),NVTERA,ITYPTR,ITYCAL
     &                           ,NUKALE,NUDONN,ITYSYM,NVDSSS
     &                           ,NUINCC,NVDUMC,NBNECO,NVNUMC
     &                           ,NUINCL,NVDUML,NBNELI,NVNUML
     &                           ,INSTOK,NVSTOK,NVCOOR,IATRDO,NIVITR)   !sdexplo
*
*                 Numerotation en colonne pour une inconnue vectorielle
*
                  IF (CRENUM.EQ.'Y') THEN
                     CALL NUM2DL (NCPIN,NVNURA,NVNURE,NBNURA*NCPIN
     &                           ,NIVITR)                               !sdexplo
                     MODADR='Y'
                  ENDIF
*
*                 Numerotation en colonne de la colonne de termes
*                 = Union des numerotations des termes de la colonne
*
                  IF (NVNURE.NE.NVDEJA) THEN
                     CALL TBRR1  (ERCODE,'#GNEDO',NVNURE,MCNUME)
                     CALL TBAR2  (ERCODE,'&WORK$',2,MCNUMW
     &                                  ,'#GNEDO',NVNURE,MCNUME)
                     IF (IST(MCNUME).NE.0) GOTO 99992
                     IF (NBDL.LE.0) THEN
*                       Premier terme non vide dans la colonne
                        NBDL=IST(MCNUME+1)
                        CALL TDANST (NBDL+2,'E',MCNUME,IST,RST,CST
     &                                     ,'E',MCNUMW,IST,RST,CST)     !utilite
                     ELSE
*                       Si '#GNEDO',NVNURE est incluse dans &WORK$/2
*                       on saute a la suivante (104)
                        CALL LISINC (IST(MCNUME+1),IST(MCNUME+2),NBDL
     &                              ,IST(MCNUMW+2),ITEMDB,ITEMFN,*104)  !utilite
                        CALL LISUNI (NBDL,IST(MCNUMW+2),IST(MCNUME+1)
     &                              ,IST(MCNUME+2),.FALSE.,IST(MCNUME+2)
     &                              ,.TRUE.,NBDLMX,*99991)              !utilite
                     ENDIF
                     IF (NBDLK.LT.IST(MCNUME+1)) THEN
*                       On retient la numerotation la plus grande
                        NBDLK=IST(MCNUME+1)
                        NVNUMK=NVNURE
                     ENDIF
104                  CALL TBSAVE('#GNEDO',NVNURE)
                     MODADR='Y'
                  ENDIF
                  IF (MODADR.EQ.'Y')
     &               CALL TBAR1  (ERCODE,'$DSMBL',NVDSMB,MCDSMB)
               ENDIF
               LRIPIN=LRIPIN+INCRIN
105         CONTINUE
*
*           Numerotation definitive en colonne de la colonne de termes
*           (dans le cas ou la plus grande numerotation rencontree est
*           la bonne, on a travaille pour rien..., ce cas est courant)
*
            IF (NBDL.EQ.NBDLK) THEN
               NVNUME=NVNUMK
            ELSE
               NVNUME=KLNIVE ()                                         !sdexplo
               LGNUME=1+(NBDL+1)*(IDLHRM+1)
               CALL TBCREE ('#GNEDO',NVNUME,1,LGNUME,'c')
               CALL TBAR2  (ERCODE,'#GNEDO',NVNUME,MCNUME
     &                            ,'&WORK$',2,MCNUMW)
               CALL TDANST (LGNUME,'E',MCNUMW,IST,RST,CST
     &                            ,'E',MCNUME,IST,RST,CST)
               IST(MCNUME)  =IDLHRM
               IST(MCNUME+1)=NBDL
               IF (NIVIMP.GE.3) THEN
                  CALL PRNEDO ('*Dsnume*','colonne (d.l.)',IST(MCNUME)
     &                        ,IMPSDR)                                  !PrSd
               ENDIF
               CALL TBSAVE ('#GNEDO',NVNUME)                     
               MODADR='Y'
            ENDIF
*
*           Mise a jour de la structure d'assemblage
*
            CALL TBAR2 (ERCODE,'$DSMBL',NVDSMB,MCDSMB,'&WORK$',1,MCWOR1)
            LRIPIN=LRIPEX
            DO 118 NULOCL=1,NBLOCL
               IST(MCDSMB+LRIPIN+KOOULI)=NVNUME
               LRIPIN=LRIPIN+INCRIN
118         CONTINUE
            IST(MCWOR1+1)=IST(MCWOR1+1)+NBDL
         ENDIF
         LRIPEX=LRIPEX+INCREX
120    CONTINUE
*
*
*     2) Numerotations en ligne pour chaque ligne de termes
*
      KOOULI=4
*     Increments de parcours par ligne de la structure $DSMBL
      INCRIN=INCDSM*NBLOCL
      INCREX=INCDSM
*
      CALL TBAR1  (ERCODE,'$DSMBL',NVDSMB,MCDSMB)
      MODADR='Y'
      LRIPEX=LGEDSM
      DO 220 NULOCL=1,NBLOCL
         NVNUME=IST(MCDSMB+LRIPEX+KOOULI)
         LRIPIN=LRIPEX
         IF (NVNUME.EQ.NDFNUM) THEN
            NBDL=0
            NBDLK=0
            NVNUMK=NDFNUM
*
            DO 205 NULOCC=1,NBLOCC
               NUTERA=IST(MCDSMB+LRIPIN)
               IF (NUTERA.GT.0) THEN
                  IF (MODADR.EQ.'Y')
     &            CALL TBAR2  (ERCODE,'#NCONU',1,MCCONU
     &                               ,'$SDTRM',1,MCDTRM)
                  MODADR='N'
                  NVNURA=NDFNUM
                  NBNURA=0
                  MODTRM='N'
                  CRENUM='N'
                  NVDEJA=0
                  ADDTRM=MCDTRM+LGETRM+INCTRM*(NUTERA-1)
                  CALL GETTRM (IST(ADDTRM),NVTERA,ITYPTR,ITYCAL,NUKALE
     &                        ,NUDONN,ITYSYM,NVDSSS,NUINCC,NVDUMC,NBNECO
     &                        ,NVNUMC,NUINCL,NVDUML,NBNELI,NVNUML,INSTOK
     &                        ,NVSTOK,NVCOOR,IATRDO,NIVITR)             !sdexplo
*
                  CALL SDEXCO (NUINCL,IST(MCCONU),NCPIN,INTRP,NBT)      !utilite
                  NVNURE=NVNUML
*
                  LRIPNN=LRIPEX
                  DO 201 NWBLOC=1,NULOCC-1
                     NWTERM=IST(MCDSMB+LRIPNN)
                     IF (NWTERM.GT.0) THEN
                        ADWTRM=MCDTRM+LGETRM+INCTRM*(NWTERM-1)
                        CALL GETTRM (IST(ADWTRM),NWTERA,IWYPTR,IWYCAL
     &                              ,NWKALE,NWDONN,IWYSYM,NWDSSS
     &                              ,NWINCC,NWDUMC,NWNECO,NWNUMC
     &                              ,NWINCL,NWDUML,NWNELI,NWNUML
     &                              ,IWDSTK,NWSTOK,NWCOOR,IWTRDO,NWVITR)!sdexplo
                        IF (NVNURE.EQ.NWNUML) THEN
                           NVDEJA=NWNUML
                           IF (NCPIN.GT.1) THEN
                              NVDUML=NWDUML
                              NVDEJA=NWDUML
                           ENDIF
                           MODTRM='Y'
                           GOTO 202
                        ENDIF
                     ENDIF
                     LRIPNN=LRIPNN+INCRIN
201               CONTINUE
202               IF (NCPIN.GT.1) THEN
                     IF (NVDUML.EQ.NDFDUM) THEN
                        NVDUML=KLNIVE()                                 !sdexplo
                        CRENUM='Y'
                        MODTRM='Y'
                        NVNURA=NVNURE
                        NBNURA=NBNELI
                     ENDIF
                     NVNURE=NVDUML
                  ENDIF
                  IF (MODTRM.EQ.'Y')
     &               CALL PUTTRM (IST(ADDTRM),NVTERA,ITYPTR,ITYCAL
     &                           ,NUKALE,NUDONN,ITYSYM,NVDSSS
     &                           ,NUINCC,NVDUMC,NBNECO,NVNUMC
     &                           ,NUINCL,NVDUML,NBNELI,NVNUML
     &                           ,INSTOK,NVSTOK,NVCOOR,IATRDO,NIVITR)   !sdexplo
*
                  IF (CRENUM.EQ.'Y') THEN
                     CALL NUM2DL (NCPIN,NVNURA,NVNURE,NBNURA*NCPIN
     &                           ,NIVITR)                               !sdexplo
                     MODADR='Y'
                  ENDIF
                  IF (NVNURE.NE.NVDEJA) THEN
                     CALL TBRR1  (ERCODE,'#GNEDO',NVNURE,MCNUME)
                     CALL TBAR2  (ERCODE,'&WORK$',2,MCNUMW
     &                                  ,'#GNEDO',NVNURE,MCNUME)
                     IF (IST(MCNUME).NE.0) GOTO 99992
                     IF (NBDL.LE.0) THEN
                        NBDL=IST(MCNUME+1)
                        CALL TDANST (IST(MCNUME+1),'E',MCNUME+2,IST,RST
     &                              ,CST,'E',MCNUMW+2,IST,RST,CST)      !utilite
                     ELSE
                        CALL LISINC (IST(MCNUME+1),IST(MCNUME+2),NBDL
     &                              ,IST(MCNUMW+2),ITEMDB,ITEMFN,*204)  !utilite
                        CALL LISUNI (NBDL,IST(MCNUMW+2),IST(MCNUME+1)
     &                              ,IST(MCNUME+2),.FALSE.,IST(MCNUME+2)
     &                              ,.TRUE.,NBDLMX,*99991)              !utilite
                     ENDIF
                     IF (NBDLK.LT.IST(MCNUME+1)) THEN
                        NBDLK=IST(MCNUME+1)
                        NVNUMK=NVNURE
                     ENDIF
204                  CALL TBSAVE('#GNEDO',NVNURE)
                     MODADR='Y'
                  ENDIF
                  IF (MODADR.EQ.'Y')
     &               CALL TBAR1  (ERCODE,'$DSMBL',NVDSMB,MCDSMB)
               ENDIF
               LRIPIN=LRIPIN+INCRIN
205         CONTINUE
*
            IF (NBDL.EQ.NBDLK) THEN
               NVNUME=NVNUMK
            ELSE
               NVNUME=KLNIVE ()                                         !sdexplo
               LGNUME=1+(NBDL+1)*(IDLHRM+1)
               CALL TBCREE ('#GNEDO',NVNUME,1,LGNUME,'c')
               CALL TBAR2  (ERCODE,'#GNEDO',NVNUME,MCNUME
     &                            ,'&WORK$',2,MCNUMW)
               CALL TDANST (LGNUME,'E',MCNUMW,IST,RST,CST
     &                            ,'E',MCNUME,IST,RST,CST)
               IST(MCNUME)  =IDLHRM
               IST(MCNUME+1)=NBDL
               IF (NIVIMP.GE.3) THEN
                  CALL PRNEDO ('*Dsnume*','ligne (d.l.)',IST(MCNUME)
     &                        ,IMPSDR)                                  !prsdd
               ENDIF
               CALL TBSAVE ('#GNEDO',NVNUME)                     
               MODADR='Y'
            ENDIF
*
*           Mise a jour de la structure d'assemblage
*
            CALL TBAR2 (ERCODE,'$DSMBL',NVDSMB,MCDSMB,'&WORK$',1,MCWOR1)
            LRIPIN=LRIPEX
            DO 218 NULOCC=1,NBLOCC
               IST(MCDSMB+LRIPIN+KOOULI)=NVNUME
               LRIPIN=LRIPIN+INCRIN
218         CONTINUE
            IST(MCWOR1)=IST(MCWOR1)+NBDL
         ENDIF
         LRIPEX=LRIPEX+INCREX
220    CONTINUE
*
      CALL TBTUER ('&WORK$',2)
      CALL PRFXMJ (-1,'*DsNume*')  
      RETURN
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
99991 CALL PRNTRM (NMTERM,NVTERM,ERCODE,NCATR1)
      CALL BAISE  ('Assemblage multi-inconnue de '//ERCODE(1:NCATR1)
     &//'.Taille sous estimee pour le tableau de numerotation $WORK$/2')
99992 CALL PRNTRM (NMTERM,NVTERM,ERCODE,NCATR1)
      CALL BAISE  ('Assemblage multi-inconnue de '//ERCODE(1:NCATR1)
     &//'. Le cas d''une interpolation de Hermite est exclus')
                                                                    END !Dsnume
