*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE MATVEC (NMMATR,NVMATR,NMVECT,NVVECT 
     &                  ,NMPROD,NVPROD,NIVIMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin & O.DeBayser (Juin 1989)
* Derniere modification : D.Martin (10 novembre 2009)
* Version 3
*  
* Calcul d'un vecteur produit d'une matrice par un vecteur A x V
*
* La matrice A uni-inconnue ou multi-inconnues, peut etre stockee sous forme  
*  'MORSE-Ligne', 'MORSE-Colonne', 'BIMORSE', 'PROFIL','PLEIN-L' ou 'PLEIN-C'
* et dans les cas 'BIMORSE' et 'PROFIL', elle peut etre
*  'SYMETRIQUE', 'AUTOADJOINTE', 'ANTISYMETRIQUE' ou 'ANTIADJOINTE'.
*  
*-- Arguments d'entree -- 
* NMMATR nom de la matrice (A)
* NVMATR son niveau
* NMVECT nom du terme vectoriel operande (V)
* NVVECT son niveau
* NMPROD nom du terme vectoriel resultat 
* NVPROD son niveau
* NIVIMP niveau d'impression du terme resultat
*
*-- Principales procedures utilisees --
* VCPREP /sdexplo
* MATMPC /multmat
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) NMMATR,NMVECT,NMPROD
      INTEGER       NVMATR,NVVECT,NVPROD,NIVIMP
*  
      INCLUDE 'CONTEX' 
      INCLUDE 'ALLOC'  
*
      CHARACTER     STOCHN*6,STODLN*6,TYPINT*9,PRFXAS*8
      CHARACTER     ERCODE*120,NMSTOK*6,NMSTOD*6,MULTIM,YADLVC,YADLVL
     &             ,KEEP,TYPROD*8,TYMATR*8,TYVECT*8,NMVAUX*16
      INTEGER       KLTERM
      INTEGER       MCMATR,MCVECT,MCPROD,MCSTOK,MCDTRM
     &             ,LGETRM,NBTERM,INCTRM,NCHTRM,MCMTRM,MCINCO
     &             ,NUTERM,NCPINC,NCPINL,NBTNEU,ADTRMA,ITMATR
     &             ,ITYCAL,NUKALE,NUDONN,ITYSYM,NVDSMA,NUINMC,NVDUMC
     &             ,NBNEMC,NVNUMC,NUINML,NVDUML,NBNEML,NVNUML,INSTOK
     &             ,NVSTOK,NVCOOR,IATRDO,NIVIMA,NVINUT,ADTRVE,ITVECT
     &             ,NVDSMV,KTYCAL,KUKALE,KUDONN,KTYSYM,KUINCV,NBVECT
     &             ,KVDUMV,KVNUMV,NUINCV,NVDUMV,NBNELV,NVNUMV
     &             ,KNSTOK,KVSTOK,KATRDO,NIVIVC
     &             ,NVVAUX,NBLIGN,NBCOLO,NCATR1,NCATR2,NCATR3,ITPROD
      COMMON/FORMAH/ERCODE 
      EQUIVALENCE (ERCODE(65:65),NMSTOD),(ERCODE(73:73),NMSTOK)
     &           ,(ERCODE(81:81),TYMATR),(ERCODE(89:89),TYVECT)
     &           ,(ERCODE(97:97),TYPROD),(ERCODE(105:105),NMVAUX)
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*MatVec*')
      ERCODE=' '
*
      CALL TBAR3  (ERCODE,'#OMTRM',1,MCMTRM,'$SDTRM',1,MCDTRM
     &                   ,'#NCONU',1,MCINCO)                     
      CALL SDEXDB (IST(MCDTRM),LGETRM,NBTERM,INCTRM,NCHTRM)             !sdexplo
      IF (NMVECT.EQ.NMPROD.AND.NVVECT.EQ.NVPROD)
     &   CALL ERTERM (9,NMVECT,NVVECT)                                  !utilite
*
      IF (NIVIMP.GT.0.AND.IMPSDR.GT.0) THEN
         CALL PRNTRM (NMPROD,NVPROD,ERCODE,NCATR1)
         CALL PRNTRM (NMMATR,NVMATR,ERCODE(NCATR1+1:),NCATR2)
         CALL PRNTRM (NMVECT,NVVECT,ERCODE(NCATR1+NCATR2+1:),NCATR3)
         WRITE (IMPSDR,10000) PRFXAS (0),ERCODE(1:NCATR1),' '
     &         ,ERCODE(NCATR1+1:NCATR1+NCATR2)
     &         ,ERCODE(NCATR1+NCATR2+1:NCATR1+NCATR2+NCATR3)
      ENDIF
*  
*--   Caracteristiques de la matrice
*
      NUTERM=KLTERM (NMMATR,NVMATR,AST(MCMTRM),IST(MCDTRM))             !sdexplo
      IF (NUTERM.LE.0) CALL ERTERM (1,NMMATR,NVMATR)                    !utilite
      ADTRMA=LGETRM+(NUTERM-1)*INCTRM
      CALL GETTRM (IST(MCDTRM+ADTRMA)  ,NVINUT,ITMATR,ITYCAL,NUKALE
     &            ,NUDONN,ITYSYM,NVDSMA,NUINMC,NVDUMC,NBNEMC,NVNUMC
     &            ,NUINML,NVDUML,NBNEML,NVNUML,INSTOK,NVSTOK,NVCOOR
     &            ,IATRDO,NIVIMA)                                       !sdexplo
*
*--   Caracteristiques du vecteur operande
*
      NUTERM=KLTERM (NMVECT,NVVECT,AST(MCMTRM),IST(MCDTRM))             !sdexplo
      IF (NUTERM.LE.0) CALL ERTERM (1,NMVECT,NVVECT)                    !utilite
      ADTRVE=LGETRM+(NUTERM-1)*INCTRM
      CALL GETTRM (IST(MCDTRM+ADTRVE)  ,NVINUT,ITVECT,KTYCAL,KUKALE
     &            ,KUDONN,KTYSYM,NVDSMV,KUINCV,KVDUMV,NBVECT,KVNUMV
     &            ,NUINCV,NVDUMV,NBNELV,NVNUMV,KNSTOK,KVSTOK,NVCOOR
     &            ,KATRDO,NIVIVC)                                       !sdexplo
*     Il s'agit a priori d'un vecteur colonne
      IF (KUINCV.NE.NDFINC) THEN
         NBVECT=NBNELV
         NUINCV=KUINCV
         NVDUMV=KVDUMV
         NVNUMV=KVNUMV
      ENDIF
*
*--   Detection du caractere multi-inconnue de la matrice
*
      NCPINC=1
      NCPINL=1
      MULTIM='N'
      IF (NVDSMA.NE.NDFDSM.AND.NUINMC.EQ.NDFINC.AND.NUINML.EQ.NDFINC) 
     &   MULTIM='Y'
*
*--   Type de stockage/structure de la matrice et nombre de lignes/colonnes
*
      NMSTOK=STOCHN (INSTOK)                                            !utilite
      NMSTOD=NMSTOK
*
      IF (MULTIM.EQ.'Y') THEN
*
*        Cas d'une matrice multi-inconnues
*
         IF (NMSTOK.NE.'&BMORS') CALL ERTERM (18,NMMATR,NVMATR)         !utilite
         CALL TBRR1  (ERCODE,'$DSMBL',NVDSMA,NBTNEU)
         NBCOLO=NBNEMC
         NBLIGN=NBNEML
      ELSE
*
         IF (NMSTOK.EQ.' '.OR.NMSTOK(1:4).EQ.'&NUM') 
     &      CALL ERTERM (18,NMMATR,NVMATR)                              !utilite
         CALL SDEXCO (NUINMC,IST(MCINCO),NCPINC,INTRMC,NBTNEU)          !sdexplo
         NCPINL=NCPINC
         IF (NUINML.NE.NUINMC) 
     &      CALL SDEXCO (NUINML,IST(MCINCO),NCPINL,INTRML,NBTNEU)       !sdexplo
*        Cas d'inconnues vectorielles
         IF (NMSTOK(1:2).NE.'PL') NMSTOD = STODLN (INSTOK,NCPINC,NCPINL)
*
         CALL FONUDL (NCPINC,NCPINL,NVNUMC,NVNUML,0,0,0,0
     &               ,NVDUMC,NVDUML,YADLVC,YADLVL)                      !sdexplo
         IF (YADLVC.NE.'N'.OR.YADLVL.NE.'N') 
     &      CALL PUTTRM (IST(MCDTRM+ADTRMA),NVMATR,ITMATR,ITYCAL,NUKALE
     &                  ,NUDONN,ITYSYM,NVDSMA,NUINMC,NVDUMC,NBNEMC
     &                  ,NVNUMC,NUINML,NVDUML,NBNEML,NVNUML
     &                  ,INSTOK,NVSTOK,NDFCOR,IATRDO,NIVIMA)            !sdexplo
         NBLIGN=NBNEML*NCPINL
         NBCOLO=NBNEMC*NCPINC
*
*--      Transformation des numerotations en d.l. ...
*        ... en colonne
         IF (YADLVC.EQ.'C')
     &      CALL NUM2DL (NCPINC,NVNUMC,NVDUMC,NBCOLO,NIVIMP)            !sdexplo
*        ... en ligne
         IF (YADLVL.EQ.'C')
     &      CALL NUM2DL (NCPINL,NVNUML,NVDUML,NBLIGN,NIVIMP)            !sdexplo
*        ... des pointeurs de stockage
         IF (NCPINC*NCPINL.GT.1) THEN
            IF (NMSTOD(1:4).EQ.'&BMO') THEN
               CALL BMO2DL (NVSTOK,NCPINL,NCPINC,ITYSYM,NMMATR,NVMATR
     &                     ,NIVIMP)                                     !sdexplo
               NMSTOK=NMSTOD
            ELSEIF (NMSTOD(1:4).EQ.'&PLA') THEN
               CALL MOL2DL (NVSTOK,NCPINL,NCPINC,NMMATR,NVMATR,NIVIMP)  !sdexplo
               NMSTOK=NMSTOD
            ELSEIF (NMSTOD(1:4).EQ.'&EGA') THEN
               CALL MOC2DL (NVSTOK,NCPINL,NCPINC,NMMATR,NVMATR,NIVIMP)  !sdexplo
               NMSTOK=NMSTOD
            ELSEIF (NMSTOD(1:5).NE.'PLEIN') THEN
               GOTO 99991
            ENDIF
         ENDIF
      ENDIF
*
      TYMATR=TYPINT (ITMATR)
      NBVECT=1
*  
*--   Vecteur resultat coherent avec la structure en ligne de la matrice
*
      ITPROD=ITVECT
      IF (TYMATR(1:2).EQ.'CO') ITPROD=ITMATR
      KEEP='n'
      CALL VCPREP (NMPROD,NVPROD,KEEP.EQ.'y',TYPINT (ITPROD)
     &            ,'Ligne',NMMATR,NVMATR,MULTIM,NVDSMA
     &            ,NUINML,NCPINL,NVDUML,NBNEML,NVNUML,'$WORK$',0
     &            ,NUTERM,NVDSMV,NBVECT,NMVAUX,NVVAUX,TYPROD,NIVIMP)    !sdexplo
*
      IF (MULTIM.EQ.'Y'.AND.NVDSMV.NE.NDFDSM)
     &   CALL TBSAVE ('$DSMBL',NVDSMV)
*  
*--   Vecteur operande mis en coherence avec la structure colonne
*--   de la matrice; s'il est modifie, il ressort dans le vecteur auxiliaire
*
      KEEP='y'
      CALL VCPREP (NMVECT,NVVECT,KEEP.EQ.'y',TYPINT (ITVECT)
     &            ,'Colonne',NMMATR,NVMATR,MULTIM,NVDSMA
     &            ,NUINMC,NCPINC,NVDUMC,NBNEMC,NVNUMC,'$WORK$',0
     &            ,NUTERM,NVDSMV,NBVECT,NMVAUX,NVVAUX,TYVECT,NIVIMP)    !sdexplo
*
*     Sauvegarde des structures multi-inconnues
*
      IF (MULTIM.EQ.'Y') THEN
         IF (NVDSMV.NE.NDFDSM) CALL TBSAVE ('$DSMBL',NVDSMV)
         CALL TBSAVE ('$DSMBL',NVDSMA)
      ENDIF
*
*--   Restauration en m.c. de la matrice et de son tableau de pointeurs
*
      CALL TBRR1  (ERCODE,NMMATR,NVMATR,MCMATR)
      IF (NMSTOK(1:2).NE.'PL') CALL TBRR1 (ERCODE,NMSTOK,NVSTOK,MCSTOK)
*
*--   Adresses des operandes et du resultat
*
      IF (NMSTOK(1:2).NE.'PL') CALL TBAR1 (ERCODE,NMSTOK,NVSTOK,MCSTOK)
      CALL TBAR3  (ERCODE,NMMATR,NVMATR,MCMATR,NMVAUX,NVVAUX,MCVECT
     &                   ,NMPROD,NVPROD,MCPROD)
*
*--   Multiplication effective
*
      CALL MATMPC (NBLIGN,NBCOLO
     &            ,NMSTOK,MCSTOK,ITYSYM,TYMATR,MCMATR
     &            ,NBVECT,TYVECT,MCVECT,TYPROD,MCPROD,IST,RST,CST)      !multmat
*
*--   Sauvegarde des tableaux
*
      IF (NIVIMP.GT.0) CALL PRTERM (NMPROD,NVPROD,NIVIMP,IMPSDR)        !prsd
      IF (NMSTOK(1:2).NE.'PL') CALL TBSAVE (NMSTOK,NVSTOK)
      CALL TBSAVE (NMMATR,NVMATR)
      CALL TBSAVE (NMVECT,NVVECT)
      CALL TBSAVE (NMPROD,NVPROD)
      IF (NMVAUX.EQ.'$WORK$') CALL TBTUER (NMVAUX,NVVAUX)
*
      IF (NIVIMP.GE.0.AND.IMPPAL.GT.0) THEN
         CALL PRNTRM (NMPROD,NVPROD,ERCODE,NCATR1)
         WRITE (IMPPAL,10000) PRFXAS (0),ERCODE(1:NCATR1),TYPROD
      ENDIF
      CALL PRFXMJ (-1,'*MatVec*')
      RETURN
*-----------------------------------------------------------------------
10000 FORMAT(/T2,A8,' Produit matrice x vecteur ',A,', ',A:
     &/T11,'matrice ',A,' et vecteur ',A)
*-----------------------------------------------------------------------
99991 CALL PRNTRM (NMMATR,NVMATR,ERCODE,NCATR1)
      CALL BAISE  (' Multiplication matrice x vecteur non prevue pour '
     &//'le stockage '//STOCHN (INSTOK)//' en d.l. vectoriels'
     &//': matrice '//ERCODE(1:NCATR1)//'.')
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 3 : C.Chambeyron (10 novembre 2009)
*  lig. 163 remplacer NMSTOD.NE.'' par NMSTOD(1:5).NE.'PLEIN'
* Version 2 : D.Martin (30 octobre 2008)
*  changement de nom des structures en DL (appel de STODLN)
* Version 1.0.4 D.Martin (13 Mars 2002)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      END
