*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE T1TERM (NMTRM1,NVTRM1,TRANS,NMTRM2,NVTRM2,NIVIMP)  
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (26 Juin 1997) 
* Derniere modification : D.Martin (31 janvier 2011)
* Version 6
*
* Creation d'un terme par transformation des coefficients d'un autre terme
* ou Modification des coefficients d'un terme
*         (pour des combinaisons de 2 termes, voir T2Term/assembl)
*  
*-- Arguments d'entree -- 
* NMTRM1 nom du terme origine (ou terme 1, ou terme d'entree)
* NVTRM1 niveau du terme origine
* TRANS  chaine de caractere definissant la transformation en minuscule ou 
*        majuscule sauf dans la cas ou elle contient le nom d'une donnee
* NMTRM2 nom du terme resultat (ou terme 2)
* NVTRM2 niveau du terme resultat
* NIVIMP niveau d'impression du terme resultat
*
*-- Transformation prevues --
*     TRANS           |                  transformation
*---------------------|---------------------------------------------------------
* '=', 'CP','COPIE'   | duplication du terme 1 reel ou complexe en un terme 2
* ou 'COPY'           | avec les memes attributs
* 
* '-'                 | terme 2 oppose du terme 1 reel ou complexe
* 'ABS', 'MODULE'     | module des coefficients du terme 1 reel ou complexe
* '^n' ou '**n'       | puissance n des coefficients du terme 1 reel ou complexe
*                     | ** actuellement 'n' est necessairement entier **
* 'CMPLX'             | transforme un terme 1 reel en terme 2 complexe
* 'RE', 'REAL'        | partie reelle des coefficients du terme 1 complexe
* 'IM', 'AIMAG'       | partie imaginaire des coefficients du terme 1 complexe
* 'CONJG'             | conjugue des coefficients du terme 1 complexe
* 'EXP'               | exponentation des coeffs du terme 1 reel ou complexe
* 'LOG'               | logarithme des coeffs du terme 1 reel ou complexe
* 'SQRT'              | racine carree des coeffs du terme 1 reel ou complexe
* 'ATAN2', 'PHASE'    | phase (=ATAN2(x,y)) des coeff. x+iy du terme 1 complexe
* 'MODULE ET PHASE'   | module et phase des coefficients du terme 1 complexe
* ou                  | retourne le module dans la partie reelle,
* 'MODULE ET ARGUMENT'| l'argument ou la phase dans la partie imaginaire
* 
* 'COL[ONNE/UMN] i'   | retourne le terme 2, i-eme colonne du terme 1 d'entree
*                     | ** le terme 1 en entree doit etre PLEIN **
* '>COL[ONNE/UMN] i'  | remplit la colonne i du terme 2 multi-colonne avec les
*                     | valeurs du terme 1
*                     | ** le terme 1 en entree doit etre PLEIN **
* 'MULTICOL[ONNE] m'  | cree le terme 2 multi-colonne a m colonnes NUL avec
*                     | avec les attributs du terme 1
* 
* 'COMPO[SANTE] i'    | retourne le terme 2 dont les coefficients sont ceux
*                     | du terme 1 associes a la composante i de l'inconnue
*                     | ** le terme en entree doit etre PLEIN et affecte
*                     | d' une inconnue vectorielle **
* '>COMPO[SANTE] i'   | reporte les coeff. du terme 1, a inconnue scalaire
*                     | dans les coeff. du terme 2, a inconnue vectorielle,
*                     | correspondant a le composante i de l'inconnue
*                     | ** les termes doivent etre PLEIN et le 2eme affecte
*                     | d' une inconnue vectorielle **
*
* './' ou 'INV[ERSE]' | retourne le terme 2 dont les coefficients sont les 
*                     | inverse des coefficients du terme 1
* 'NORM[E]'           | retourne le terme 2 dont les coefficients sont les 
*                     | normes des coefficients du terme 1
*                     | ** le terme en entree doit etre PLEIN et affecte
*                     | d' une inconnue vectorielle **
* 'NORME CARRE[E]'    | idem avec le carre de la norme
* 'NORM SQUARED'      | ** le terme en entree doit etre PLEIN et affecte
*                     | d' une inconnue vectorielle **
* 
* '0', 'ZERO[S]'      | retourne le terme nul avec les attributs du terme 1
* ou 'NUL[LE]'        | d'entree
* '1', 'ONE[S]'       | retourne le terme 2 ne contenant que des coefficients
* ou 'UN[S]'          | egaux a 1 avec les attributs du terme 1 d'entree.
*                     |
* 'IDENTIT[E/Y]'      | retourne la matrice identite avec les attributs du terme
*                     | d'entree 
*                     | ** le terme en entree doit etre une matrice carree **
*
* '*'                 | retourne le produit des coefficients du terme 1 par
*                     | la valeur de sa DONNEE CONSTANTE affectee.
*
* '*donnee'           | ou donnee est le nom d'une DONNEE CONSTANTE;
*                     | retourne le produit des valeurs du terme 1 par
*                     | la valeur de la DONNEE CONSTANTE de nom 'donnee'
*
* Cas particulier ou le 1er argument peut correspondre a un tableau
* '=VALEUR[S]         | retourne les valeurs du tableau 1 dans le terme 2 AVEC les
*   /VALUE[S]'        | attributs du terme 2 ** qui existe necessairement**
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) NMTRM1,NMTRM2,TRANS
      INTEGER       NVTRM1,NVTRM2,NIVIMP
*  
      INCLUDE 'CONTEX'
      INCLUDE 'ALLOC'
* 
      CHARACTER     STOIND*7,TYPINT*8,UPPERC*16
      CHARACTER*8   TYTRM1,TYTRM2,TYDONN
      INTEGER INDSTO,INTDON,INTTYP,INDASS,KELCHN,KLTERM
      INTEGER NIVEAU,ITYPTR,ITYCAL,NUKALE,NUDON1,ITYSYM,NVDSMB
     &       ,NUINCC,NVDUMC,NBNECO,NVNUMC,NUINCL,NVDUML,NBNELI,NVNUML
     &       ,INSTOK,NVSTOK,NVCOOR,IATRDO,NIVIMR,NVALNE
     &       ,MCINCO,MCMTRM,MCDTRM,LGETRM,NBTERM,INCTRM,NCHTRM,NUTERM
     &       ,LGTRM1,MCTRM1,ADTRM1,NCATR1,NCATR2,LGTRM2,MCTRM2,ADTRM2
     &       ,ITYPT2,ITYCA2,NUKAL2,NUDON2,ITYSY2,NVDSM2,NUINC2,NVDUC2
     &       ,NBNEC2,NVNUC2,NUINL2,NVDUL2,NBNEL2,NVNUL2,INSTO2,NUBLAN
     &       ,NUMERO,VALEUR,LENTRA,INTREL,INTCOM,NCPINC,NCPINL,NBLIGN
     &       ,MCMDON,MCDONN,LGEDON,NBDONN,INCDON,NCHDON,ITYDO1,NIVDO1
     &       ,ITYPD1,MCCSTE,ADTRMI,NUMERR,NCPIL2,IBIDO0,IBIDO1,I,IU
      LOGICAL CRETRM,SAMTRM,FORBID
      REAL    X,Y
      COMPLEX Z
      CHARACTER     ERCODE*120,UTRANS*16,NMSTOK*7
      COMMON/FORMAH/ERCODE
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*T1Term*')
*     Si les termes 1 et 2 coincident ...
      SAMTRM=NMTRM1.EQ.NMTRM2.AND.NVTRM1.EQ.NVTRM2
*     ... certaines transformations sont interdites
*        (type incongru, changement de type, de longueur)
      FORBID=.FALSE.
* 
      LENTRA=LEN (TRANS)
      IU=0
*     Suppression des blancs en entete de la chaine
      DO 1 I=1,LENTRA
         IF (IU.EQ.0.AND.TRANS(I:I).EQ.' ') GOTO 1
         IU=IU+1
         UTRANS(IU:IU)=TRANS(I:I)
1     CONTINUE
      LENTRA=IU
*     Suppression des blancs en fin de chaine
      DO 2 LENTRA=IU,1,-1
         IF (UTRANS(LENTRA:LENTRA).NE.' ') GOTO 3
2     CONTINUE
*     La chaine de transformation est transformee en majuscules
3     UTRANS=UPPERC (UTRANS)                                            !utilite
*
      INTREL=INTTYP ('REEL')                                            !utilite
      INTCOM=INTTYP ('COMPLEXE')                                        !utilite
      CRETRM=.FALSE.
* 
      VALEUR=0
      IF (LENTRA.GT.1) THEN
*        Transformation '=valeur'
         VALEUR=INDEX (UTRANS,'VALEUR')
         IF (VALEUR.EQ.0) VALEUR=INDEX (UTRANS,'VALUE')
         IF (VALEUR.EQ.0) THEN
*           multiplication par une donnee
            IF (UTRANS(1:1).EQ.'*'.AND.UTRANS(2:2).NE.'*') VALEUR=-1
         ENDIF
      ENDIF 
*  
*     Recherche des caracteristiques du terme source
* 
*     Adresses du tableau de noms de termes et de la structure
*     contenant les caracteristiques des termes
      CALL TBAR2  (ERCODE,'#OMTRM',1,MCMTRM,'$SDTRM',1,MCDTRM)
*     Parametres de la structures des caracteristiques des termes
      CALL SDEXDB (IST(MCDTRM),LGETRM,NBTERM,INCTRM,NCHTRM)             !utilite
*     Numero du terme
      NUTERM=KLTERM (NMTRM1,NVTRM1,AST(MCMTRM),IST(MCDTRM))             !sdexplo
      IF (NUTERM.GT.0) THEN
*        Caracteristiques du terme
         CALL GETTRM (IST(MCDTRM+LGETRM+INCTRM*(NUTERM-1))  
     &               ,NIVEAU,ITYPTR,ITYCAL,NUKALE,NUDON1,ITYSYM,NVDSMB
     &               ,NUINCC,NVDUMC,NBNECO,NVNUMC,NUINCL,NVDUML,NBNELI
     &               ,NVNUML,INSTOK,NVSTOK,NVCOOR,IATRDO,NIVIMR)        !sdexplo
*        Pour les termes valeurs nodales, ITYSYM contient le nombre
*        de valeurs par noeud (terme dit a valeurs vectorielles)
         NVALNE=ITYSYM
*        Recherche du terme (si necessaire retour en m.c.)
         CALL TBRR1  (ERCODE,NMTRM1,NVTRM1,LGTRM1)
*        Type (REEL ou COMPLEXE) du terme en entree
         TYTRM1=TYPINT (ITYPTR)
*        Initialisation des caracteristiques du second terme
*        (a priori celle du terme en entree)
         LGTRM2=LGTRM1
         ITYPT2=ITYPTR
         ITYSY2=ITYSYM
         NUKAL2=NUKALE
         NUDON2=NDFDON
         NVDSM2=NVDSMB
         NUINC2=NUINCC
         NVDUC2=NVDUMC
         NBNEC2=NBNECO
         NVNUC2=NVNUMC
         NUINL2=NUINCL
         NVDUL2=NVDUML
         NBNEL2=NBNELI
         NVNUL2=NVNUML
         NBLIGN=NBNELI
*        Pas de donnee affectee au terme resultat ...
         NUDON2=NDFDON
*        ... sauf ses 2 termes coincident.
         IF (SAMTRM) NUDON2=NUDON1
         NCPINC=1
         NCPINL=1
      ELSE
         INSTOK=0
         NVSTOK=NDFSTO
         CALL TBRR1  (ERCODE,NMTRM1,NVTRM1,LGTRM1)
         CALL TBTYPE (NMTRM1,NVTRM1,ITYPTR)
         TYTRM1=TYPINT (ITYPTR)
      ENDIF
* 
      IF (VALEUR.LT.0) THEN
* 
*        Transformation a l'aide d'une DONNEE CONSTANTE
* 
*        Adresses du tableau de noms de donnees et de la structure
*        contenant les caracteristiques des donnees
         CALL TBAR2  (ERCODE,'#OMDON',1,MCMDON,'$DONNE',1,MCDONN)
*        Parametres de la structures des caracteristiques des donnees
         CALL SDEXDB (IST(MCDONN),LGEDON,NBDONN,INCDON,NCHDON)          !utilite
*
         TYDONN=' '
         IF (LENTRA.GT.1.AND.TRANS(2:2).NE.' ') THEN
*           Recherche du numero de la donnee dont le nom figure dans TRANS
            NUDON1=KELCHN (TRANS(2:LENTRA),AST(MCMDON),NBDONN,NCHDON)   !sdexplo
            IF (NUDON1.LE.0) CALL ERDONN (1,TRANS(2:LENTRA))            !utilite
         ENDIF
         IF (NUDON1.GT.0) THEN
*           Caracteristiques de la donnee
            CALL SDEXCO (NUDON1,IST(MCDONN),ITYDO1,NIVDO1,ITYPD1)       !utilite
*           La donnee DOIT ici etre une constante
            IF (ITYDO1.NE.INTDON ('CONSTANTE')) 
     &         CALL ERDONN (2,TRANS(2:LENTRA))                          !utilite
*           Type de declaration de la donnee (ENTIER, REEL, COMPLEXE)
            TYDONN=TYPINT (ITYPD1)                                      !utilite
         ENDIF
*        Le 2nd terme herite de la combinaison du 1er et de la constante
         TYTRM2=TYTRM1
         IF (TYDONN(1:1).EQ.'C') TYTRM2=TYDONN
         ITYPTR=INTTYP (TYTRM2)                                         !utilite
         FORBID=SAMTRM.AND.TYTRM2.NE.TYTRM1
*
      ELSEIF (VALEUR.EQ.0) THEN
*
         IF (UTRANS(1:3).EQ.'COL') THEN
*
*           Extraction d'une colonne d'un terme multi-colonne
*
            NUBLAN=INDEX (UTRANS,' ')
            NUMERO=0
            IF (NUBLAN.LE.0) GOTO 99992
            CALL RELCNE (UTRANS(NUBLAN+1:LENTRA),NUMERO)                !utilite
            CALL TBAR1  (ERCODE,'#NCONU',1,MCINCO)
            IF (NUINCC.NE.NDFINC)
     &         CALL SDEXCO (NUINCC,IST(MCINCO),NCPINC,IBIDO0,IBIDO1)    !utilite
            IF (NUINCL.NE.NDFINC)
     &         CALL SDEXCO (NUINCL,IST(MCINCO),NCPINL,IBIDO0,IBIDO1)    !utilite
            LGTRM2=LGTRM2/(NBNECO*NCPINC)
            IF (NUMERO.LE.0.OR.NUMERO.GT.NBNECO) GOTO 99992
            NUINC2=NDFINC
            NVDUC2=NDFDUM
            NBNEC2=1
            NVNUC2=NDFNUM
            FORBID=SAMTRM
*
         ELSEIF (UTRANS(1:4).EQ.'>COL') THEN
*
*           Remplissage d'une colonne d'un terme multi-colonne
*
            NUBLAN=INDEX (UTRANS,' ')
            NUMERO=0
            IF (NUBLAN.LE.0) GOTO 99992
            CALL RELCNE (UTRANS(NUBLAN+1:LENTRA),NUMERO)                !utilite
            IF (NUMERO.LE.0) GOTO 99992
            FORBID=SAMTRM
*
         ELSEIF (UTRANS(1:8).EQ.'MULTICOL') THEN
*
*           Creation d'un terme multi-colonne a partir d'un terme uni-colonne
*
            IF (NBNEC2.GT.1) GOTO 99995
            NUMERR=37
            IF (INSTOK.NE.INDSTO ('PLEIN-C')) GOTO 99990
            NUBLAN=INDEX (UTRANS,' ')
            IF (NUBLAN.LE.0) GOTO 99996
            CALL RELCNE (UTRANS(NUBLAN+1:LENTRA),NUMERO)                !utilite
            LGTRM2=LGTRM2*NUMERO
            NUINC2=NDFINC
            NVDUC2=NDFDUM
            NBNEC2=NUMERO
            NVNUC2=NDFNUM
            UTRANS='ZEROS'
*
         ELSEIF (UTRANS(1:5).EQ.'COMPO') THEN
*
*           Extraction des coefficients correspondant a une composante
*           d'une inconnue vectorielle d'un terme a valeurs vectorielles 
*
            NUBLAN=INDEX (UTRANS,' ')
            NUMERO=0
            IF (NUBLAN.LE.0) GOTO 99993
            NUMERR=31
            IF (NVALNE.LE.1) GOTO 99990
            CALL TBAR1  (ERCODE,'#NCONU',1,MCINCO)
            IF (NUINCL.NE.NDFINC) THEN
               CALL SDEXCO (NUINCL,IST(MCINCO),NCPINL,IBIDO0,IBIDO1)    !utilite
            ELSEIF (NUINCC.NE.NDFINC) THEN
               CALL SDEXCO (NUINCC,IST(MCINCO),NCPINL,IBIDO0,IBIDO1)    !utilite
            ELSE
               GOTO 99990
            ENDIF
            CALL RELCNE (UTRANS(NUBLAN+1:LENTRA),NUMERO)                !utilite
            IF (NUMERO.LE.0.OR.NUMERO.GT.NCPINL) GOTO 99993
*           Longueur et nombre de valeurs par noeud du resultat
            LGTRM2=LGTRM2/NCPINL
            ITYSY2=1
            FORBID=SAMTRM.AND.LGTRM2.NE.LGTRM1
*
         ELSEIF (UTRANS(1:6).EQ.'>COMPO') THEN
*
*           Report des coefficients du terme 1 a inconnue scalaire
*           dans les coefficients du terme 2 correspondant a la composante 
*           NUMERO de l'inconnue vectorielle associee
*
            NUBLAN=INDEX (UTRANS,' ')
            NUMERO=0
            IF (NUBLAN.LE.0) GOTO 99994
            CALL RELCNE (UTRANS(NUBLAN+1:LENTRA),NUMERO)                !utilite
            IF (NUMERO.LE.0) GOTO 99994
            FORBID=SAMTRM
*
         ELSEIF (UTRANS(1:3).EQ.'INV') THEN
*
            LENTRA=3
            UTRANS='1./'
*
         ELSEIF (UTRANS(1:4).EQ.'NORM') THEN
*           Type, longueur et nombre de valeurs par noeud du resultat
            LGTRM2=LGTRM1/NVALNE
            ITYSY2=1
            FORBID=SAMTRM.AND.ITYPTR.NE.INTREL
            ITYPTR=INTREL
*
         ELSEIF (UTRANS(1:1).EQ.'^') THEN
*
*           Valeur de l'exposant
            CALL RELCNE (UTRANS(2:LENTRA),NUMERO)                       !utilite
*
         ELSEIF (UTRANS(1:2).EQ.'**') THEN
*
*           Valeur de l'exposant
            CALL RELCNE (UTRANS(3:LENTRA),NUMERO)                       !utilite
*
         ELSEIF (LENTRA.EQ.19) THEN
*
            IF (UTRANS(1:LENTRA).EQ.'MODULE ET ARGUMENT') THEN
*
               NUMERR=5
               IF (ITYPTR.NE.INTCOM) GOTO 99990
               LENTRA=15
               UTRANS='MODULE ET PHASE'
            ENDIF
*
         ELSEIF (LENTRA.EQ.15) THEN
*
            IF (UTRANS(1:LENTRA).EQ.'MODULE ET PHASE') THEN
*
               NUMERR=5
               IF (ITYPTR.NE.INTCOM) GOTO 99990
            ENDIF
*
         ELSEIF (LENTRA.EQ.8) THEN
*
            IF (UTRANS(1:4).EQ.'IDEN') THEN
*
               ITYPTR=INTREL
               NMSTOK=STOIND (INSTOK)                                   !utilite
               CALL TBAR1  (ERCODE,'#NCONU',1,MCINCO)
               IF (NUINCC.NE.NDFINC)
     &         CALL SDEXCO (NUINCC,IST(MCINCO),NCPINC,IBIDO0,IBIDO1)    !utilite
               IF (NUINCL.NE.NDFINC)
     &         CALL SDEXCO (NUINCL,IST(MCINCO),NCPINL,IBIDO0,IBIDO1)    !utilite
               NUMERR=11
               IF (NBNELI.NE.NBNECO) GOTO 99990
               IF (NCPINL.NE.NCPINC) GOTO 99990
               NBLIGN=NBLIGN*NCPINL
            ENDIF
*
         ELSEIF (LENTRA.GE.6) THEN
*
            IF (UTRANS(1:5).EQ.'MODUL') THEN
*
               LENTRA=3
               UTRANS='ABS'
               FORBID=SAMTRM.AND.ITYPTR.NE.INTREL
               ITYPTR=INTREL
            ENDIF
*
         ELSEIF (LENTRA.EQ.5) THEN
*
            IF (UTRANS(1:LENTRA).EQ.'COPIE') THEN
               LENTRA=1
               UTRANS='='
*
            ELSEIF (UTRANS(1:LENTRA).EQ.'AIMAG') THEN
*
               NUMERR=5
               IF (ITYPTR.NE.INTCOM) GOTO 99990
               ITYPTR=INTREL
               ITYPTR=INTREL
*
            ELSEIF (UTRANS(1:LENTRA).EQ.'CMPLX') THEN
*
               FORBID=SAMTRM.AND.ITYPTR.EQ.INTREL
               ITYPTR=INTCOM
               LENTRA=1
               UTRANS='='
*
            ELSEIF (UTRANS(1:LENTRA).EQ.'CONJG') THEN
*
               NUMERR=5
               IF (ITYPTR.NE.INTCOM) GOTO 99990
*
            ELSEIF (UTRANS(1:LENTRA).EQ.'PHASE'
     &          .OR.UTRANS(1:LENTRA).EQ.'ATAN2') THEN
*
               FORBID=SAMTRM.AND.ITYPTR.NE.INTCOM
               UTRANS='ATAN2'
               ITYPTR=INTREL
*
            ELSEIF (UTRANS(1:LENTRA).EQ.'NULLE') THEN
*
               UTRANS='ZEROS'
            ENDIF
*
         ELSEIF (LENTRA.EQ.4) THEN
*
            IF (UTRANS(1:LENTRA).EQ.'COPY') THEN
               LENTRA=1
               UTRANS='='
*
            ELSEIF (UTRANS(1:LENTRA).EQ.'ZERO') THEN
*
               LENTRA=5
               UTRANS='ZEROS'
*
            ELSEIF (UTRANS(1:LENTRA).EQ.'REAL') THEN
               IF (ITYPTR.EQ.INTREL) THEN
                  LENTRA=1
                  UTRANS="="
               ENDIF
               FORBID=SAMTRM.AND.ITYPTR.NE.INTREL
               ITYPTR=INTREL
            ENDIF
*
         ELSEIF (LENTRA.EQ.3) THEN
*
            IF (UTRANS(1:LENTRA).EQ.'ABS') THEN
*
               FORBID=SAMTRM.AND.ITYPTR.NE.INTREL
               ITYPTR=INTREL
*
            ELSEIF (UTRANS(1:LENTRA).EQ.'NUL') THEN
*
               LENTRA=5
               UTRANS='ZEROS'
*
            ELSEIF (UTRANS(1:LENTRA).EQ.'UNS'
     &          .OR.UTRANS(1:LENTRA).EQ.'ONE') THEN
*
               LENTRA=4
               UTRANS='ONES'
            ENDIF
*
         ELSEIF (LENTRA.EQ.2) THEN
*
            IF (UTRANS(1:LENTRA).EQ.'CP') THEN
               LENTRA=1
               UTRANS='='
*
            ELSEIF (UTRANS(1:LENTRA).EQ.'IM') THEN
*
               NUMERR=5
               IF (ITYPTR.NE.INTCOM) GOTO 99990
               FORBID=SAMTRM
               ITYPTR=INTREL
               LENTRA=5
               UTRANS='AIMAG'
*
            ELSEIF (UTRANS(1:LENTRA).EQ.'RE') THEN
*
               IF (ITYPTR.EQ.INTREL) THEN
                  LENTRA=1
                  UTRANS="="
               ELSE
                  FORBID=SAMTRM
                  ITYPTR=INTREL
                  LENTRA=4
                  UTRANS='REAL'
               ENDIF
* 
            ELSEIF (UTRANS(1:LENTRA).EQ.'UN') THEN
* 
               LENTRA=4
               UTRANS='ONES'
*
            ELSEIF (UTRANS(1:LENTRA).EQ.'./') THEN
* 
               LENTRA=3
               UTRANS='1./'

            ENDIF
* 
         ELSEIF (LENTRA.EQ.1) THEN
* 
            IF (UTRANS(1:1).EQ.'0') THEN
* 
               LENTRA=5
               UTRANS='ZEROS'
* 
            ELSEIF (UTRANS(1:1).EQ.'1') THEN
* 
               LENTRA=4
               UTRANS='ONES'
* 
            ENDIF
* 
         ENDIF
* 
      ENDIF

*-----------------------------------------------------------------------
*     Cas d'une transformation interdite
* 
      NUMERR=33
      IF (FORBID) GOTO 99990
*-----------------------------------------------------------------------
      IF (UTRANS(1:1).EQ.'>') THEN
*
         NUTERM=KLTERM (NMTRM2,NVTRM2,AST(MCMTRM),IST(MCDTRM))          !sdexplo
         IF (NUTERM.GT.0) THEN
            CALL GETTRM (IST(MCDTRM+LGETRM+INCTRM*(NUTERM-1))  
     &                 ,NIVEAU,ITYPT2,ITYCA2,NUKAL2,NUDON2,ITYSY2,NVDSM2
     &                 ,NUINC2,NVDUC2,NBNEC2,NVNUC2,NUINL2,NVDUL2,NBNEL2
     &                 ,NVNUL2,INSTO2,NVSTOK,NVCOOR,IATRDO,NIVIMR)      !sdexplo
            IF (NUINC2.NE.NDFINC) THEN
               CALL SDEXCO (NUINC2,IST(MCINCO),NCPIL2,IBIDO0,IBIDO1)    !utilite
            ELSEIF (NUINL2.NE.NDFINC) THEN
               CALL SDEXCO (NUINL2,IST(MCINCO),NCPIL2,IBIDO0,IBIDO1)    !utilite
            ELSE
               NUMERR=31
               GOTO 99990
            ENDIF
         ELSE
            INSTO2=INDSTO ('PLEIN-C')
            CALL TBTYPE (NMTRM2,NVTRM2,ITYPT2)
         ENDIF
         CALL TBRR1  (ERCODE,NMTRM2,NVTRM2,LGTRM2)
         TYTRM2=TYPINT (ITYPT2)
*
      ELSEIF (VALEUR.LE.0) THEN
* 
*        Les attributs du terme 1 sont affectes au terme 2
*        Les valeurs seront affectees plus bas selon la transformation
* 
         TYTRM2=TYPINT (ITYPTR)
         IF (.NOT.SAMTRM) NUKALE=-INDASS (' ')                         !utilite
         CALL CRTERM (NMTRM2,NVTRM2,LGTRM2,ITYPTR,ITYCAL,NUKALE,NUDON2
     &               ,ITYSY2,NVDSMB,NUINC2,NVDUC2,NBNEC2,NVNUC2,NUINL2
     &               ,NVDUL2,NBNEL2,NVNUL2,INSTOK,NVSTOK,NVCOOR,IATRDO
     &               ,NIVIMP,SAMTRM,CRETRM)                            !sdexplo
      ELSE
* 
*        Les valeurs (et non les attributs) du terme 1 seront copiees
*        dans le terme 2 (les attributs du terme 2 sont inchanges)
* 
         CALL TBRR1  (ERCODE,NMTRM2,NVTRM2,LGTRM2)
         CALL TBTYPE (NMTRM2,NVTRM2,ITYPTR)
         TYTRM2=TYPINT (ITYPTR)
         IF (VALEUR.GE.1.OR.UTRANS(1:1).EQ.'=') THEN
            LENTRA=1
            UTRANS='='
         ENDIF
         CALL WATERS (NMTRM1,NVTRM1,NUINCC,NUINCL,ITYSYM,INSTOK,LGTRM1
     &               ,NMTRM2,NVTRM2,NUINCC,NUINCL,ITYSYM,INSTOK,LGTRM2) !utilite
         NUMERR=3
         IF (TYTRM2.NE.TYTRM1) GOTO 99990
      ENDIF
* 
*     Adresses des tableaux '1' et '2' :
*

      CALL TBAR2  (ERCODE,NMTRM1,NVTRM1,MCTRM1,NMTRM2,NVTRM2,MCTRM2)

*
*-----------------------------------------------------------------------
*     Operations proprement dite
*-----------------------------------------------------------------------
*
      IF (VALEUR.LT.0) THEN
*
         IF (TYDONN(1:1).EQ.'R') THEN
            CALL TBAR1 (ERCODE,'$RCSTE',1,MCCSTE)
         ELSEIF (TYDONN(1:1).EQ.'C') THEN
            CALL TBAR1 (ERCODE,'$CCSTE',1,MCCSTE)
         ELSEIF (TYDONN(1:1).EQ.'E') THEN
            CALL TBAR1 (ERCODE,'$ECSTE',1,MCCSTE)
         ENDIF
*
         IF (VALEUR.EQ.-1) THEN
*
*        Multiplication par une DONNEE CONSTANTE
*
            CALL TEQAT (LGTRM1,TYTRM1,MCTRM1,RST,CST
     &                 ,TYDONN,MCCSTE+NIVDO1,IST,RST,CST
     &                 ,TYTRM2,MCTRM2,RST,CST)                          !utilite
         ELSE
            NUMERR=34
            GOTO 99990
         ENDIF
*    
      ELSE
*
         IF (UTRANS(1:3).EQ.'COL') THEN
*
*           Extraction de la colonne NUMERO
*
            IF (INSTOK.EQ.INDSTO ('PLEIN-C')) THEN
*              Matrice Pleine stockee 'a la Fortran'
               CALL TDANST (LGTRM2
     &                     ,TYTRM1,MCTRM1+(NUMERO-1)*LGTRM2,IST,RST,CST
     &                     ,TYTRM2,MCTRM2,IST,RST,CST)                  !utilite
            ELSEIF (INSTOK.EQ.INDSTO ('PLEIN-L')) THEN
*              Matrice Pleine stockee ligne par ligne
               IF (TYTRM1(1:1).EQ.'R') THEN
                  DO 10302 ADTRM1=MCTRM1,MCTRM1+LGTRM1-1,NBNELI*NCPINL
                     RST(MCTRM2)=RST(ADTRM1)
                     MCTRM2=MCTRM2+1
10302             CONTINUE
               ELSEIF (TYTRM1(1:1).EQ.'C') THEN
                  DO 10304 ADTRM1=MCTRM1,MCTRM1+LGTRM1-1,NBNELI*NCPINL
                     CST(MCTRM2)=CST(ADTRM1)
                     MCTRM2=MCTRM2+1
10304             CONTINUE
               ENDIF
            ENDIF
*
         ELSEIF (UTRANS(1:4).EQ.'>COL') THEN
*
*           Remplissage (a partir) de la colonne NUMERO
*
            IF (INSTO2.EQ.INDSTO ('PLEIN-C')) THEN
*              Matrice Pleine stockee 'a la Fortran'
               CALL TDANST (LGTRM1
     &                     ,TYTRM1,MCTRM1,IST,RST,CST
     &                     ,TYTRM2,MCTRM2+(NUMERO-1)*LGTRM1,IST,RST,CST)!utilite
            ELSE
               GOTO 99991
            ENDIF
*
         ELSEIF (UTRANS(1:5).EQ.'COMPO') THEN
*
*           Extraction des coefficients correspondant a la composante NUMERO
*           de l'inconnue associee au terme 
*
            IF (TYTRM1(1:1).EQ.'R') THEN
               DO 10512 ADTRM1=MCTRM1+NUMERO-1,MCTRM1+LGTRM1-1,NCPINL
                  RST(MCTRM2)=RST(ADTRM1)
                  MCTRM2=MCTRM2+1
10512           CONTINUE
            ELSEIF (TYTRM1(1:1).EQ.'C') THEN
               DO 10514 ADTRM1=MCTRM1+NUMERO-1,MCTRM1+LGTRM1-1,NCPINL
                  CST(MCTRM2)=CST(ADTRM1)
                  MCTRM2=MCTRM2+1
10514            CONTINUE
            ENDIF
            
         ELSEIF (UTRANS(1:6).EQ.'>COMPO') THEN
*
*           Report des coefficients du terme 1 a inconnue scalaire
*           dans les coefficients du terme 2 correspondant a la composante NUMERO 
*
            IF (LGTRM2.LT.NUMERO*LGTRM1) GOTO 99994
            IF (TYTRM1(1:1).EQ.'R') THEN
               DO 10513 ADTRM2=MCTRM2+NUMERO-1,MCTRM2+LGTRM2-1,NCPIL2
                  RST(ADTRM2)=RST(MCTRM1)
                  MCTRM1=MCTRM1+1
10513           CONTINUE
            ELSEIF (TYTRM1(1:1).EQ.'C') THEN
               DO 10515 ADTRM2=MCTRM2+NUMERO-1,MCTRM2+LGTRM2-1,NCPIL2
                  CST(ADTRM2)=CST(MCTRM1)
                  MCTRM1=MCTRM1+1
10515            CONTINUE
            ENDIF
* 
         ELSEIF (UTRANS(1:4).EQ.'NORM') THEN
* 
*        Norme en chaque noeud d'un terme a valeurs vectorielles par noeud
* 
            IF (TYTRM1(1:1).EQ.'R') THEN
               DO 10522 ADTRM1=MCTRM1,MCTRM1+LGTRM1-1,NVALNE
                  ADTRMI=ADTRM1
                  X=RST(ADTRMI)*RST(ADTRMI)
                  DO 10521 I=1,NVALNE-1
                     ADTRMI=ADTRMI+1
                     X=X+RST(ADTRMI)*RST(ADTRMI)
10521             CONTINUE
                  RST(MCTRM2)=X
                  MCTRM2=MCTRM2+1
10522          CONTINUE
            ELSEIF (TYTRM1(1:1).EQ.'C') THEN
               DO 10524 ADTRM1=MCTRM1,MCTRM1+LGTRM1-1,NVALNE
                  ADTRMI=ADTRM1
                  X=CST(ADTRMI)*CONJG(CST(ADTRMI))
                  DO 10523 I=1,NVALNE-1
                     ADTRMI=ADTRMI+1
                     X=X+CST(ADTRMI)*CONJG(CST(ADTRMI))
10523             CONTINUE
                  RST(MCTRM2)=X
                  MCTRM2=MCTRM2+1
10524          CONTINUE
            ENDIF
* 
            IF (LENTRA.LE.5) THEN
* 
*              Extraction de la racine carree pour le calcul de la norme
* 
               DO 10525 I=MCTRM2-LGTRM2,MCTRM2-1
                  RST(I)=SQRT(RST(I))
10525          CONTINUE
            ENDIF
* 
         ELSEIF (UTRANS(1:1).EQ.'^'.OR.UTRANS(1:2).EQ.'**') THEN
* 
*           Puissance NUMERO des coefficients du terme 1
* 
            IF (TYTRM1(1:1).EQ.'R') THEN
               DO 102 ADTRM1=MCTRM1,MCTRM1+LGTRM1-1
                  RST(MCTRM2)=RST(ADTRM1)**NUMERO
                  MCTRM2=MCTRM2+1
102            CONTINUE
            ELSEIF (TYTRM1(1:1).EQ.'C') THEN
               DO 104 ADTRM1=MCTRM1,MCTRM1+LGTRM1-1
                  CST(MCTRM2)=CST(ADTRM1)**NUMERO
                  MCTRM2=MCTRM2+1
104            CONTINUE
            ENDIF
*
         ELSEIF (LENTRA.EQ.15) THEN
*
            IF (UTRANS(1:LENTRA).EQ.'MODULE ET PHASE') THEN
*
               DO 1501 ADTRM1=MCTRM1,MCTRM1+LGTRM2-1
                  Z=CST(ADTRM1)
                  X=REAL (Z)
                  Y=AIMAG (Z)
                  CST(MCTRM2)=CMPLX (ABS (Z),ATAN2 (X,Y))
                  MCTRM2=MCTRM2+1
1501           CONTINUE
            ELSE
               NUMERR=34
               GOTO 99990
            ENDIF
*
         ELSEIF (LENTRA.EQ.8) THEN
*
            IF (UTRANS(1:4).EQ.'IDEN') THEN
*
*              Matrice identite avec les attributs du terme initial
*
               CALL TAZERO (LGTRM2,TYTRM2,MCTRM2,AST,IST,RST,CST)
               IF (NMSTOK.EQ.'BIMORSE'.OR.NMSTOK.EQ.'PROFIL ') THEN
                  IF (TYTRM2(1:1).EQ.'R') THEN
                     DO 821 ADTRM2=MCTRM2,MCTRM2+NBLIGN-1
                        RST(ADTRM2)=1.
821                  CONTINUE
                  ELSEIF (TYTRM2(1:1).EQ.'C') THEN
                     Z=CMPLX(1.,0.)
                     DO 841 ADTRM2=MCTRM2,MCTRM2+NBLIGN-1
                        CST(ADTRM2)=Z
841                  CONTINUE
                  ENDIF
               ELSEIF (NMSTOK.EQ.'PLEIN-L'.OR.NMSTOK.EQ.'PLEIN-C') THEN
                  IF (TYTRM2(1:1).EQ.'R') THEN
                     ADTRM2=MCTRM2
                     DO 822 I=1,NBLIGN
                        RST(ADTRM2)=1.
                        ADTRM2=ADTRM2+NBLIGN+1
822                  CONTINUE
                  ELSEIF (TYTRM2(1:1).EQ.'C') THEN
                     Z=CMPLX(1.,0.)
                     ADTRM2=MCTRM2
                     DO 842 I=1,NBLIGN
                        CST(ADTRM2)=Z
                        ADTRM2=ADTRM2+NBLIGN+1
842                  CONTINUE
                  ENDIF
               ELSE
                  NUMERR=18
                  GOTO 99990
               ENDIF
            ENDIF
*
         ELSEIF (LENTRA.EQ.5) THEN
*
            IF (UTRANS(1:LENTRA).EQ.'AIMAG') THEN
*
*              Partie imaginaire des coefficients du terme 1 complexe
*
               DO 501 ADTRM1=MCTRM1,MCTRM1+LGTRM2-1
                  RST(MCTRM2)=AIMAG(CST(ADTRM1))
                  MCTRM2=MCTRM2+1
501            CONTINUE
*
            ELSEIF (UTRANS(1:LENTRA).EQ.'ATAN2') THEN
*
               DO 502 ADTRM1=MCTRM1,MCTRM1+LGTRM2-1
                  Z=CST(ADTRM1)
                  X=REAL (Z)
                  Y=AIMAG (Z)
                  RST(MCTRM2)=ATAN2 (X,Y)
                  MCTRM2=MCTRM2+1
502             CONTINUE
*
            ELSEIF (UTRANS(1:LENTRA).EQ.'CONJG') THEN
*
*              Conjugue des coefficients du terme 1 complexe
*
               DO 503 ADTRM1=MCTRM1,MCTRM1+LGTRM2-1
                  CST(MCTRM2)=CONJG (CST(ADTRM1))
                  MCTRM2=MCTRM2+1
503            CONTINUE
*
            ELSEIF (UTRANS(1:LENTRA).EQ.'ZEROS') THEN
*
*              Creation d'un terme nul
*
               CALL TAZERO (LGTRM2,TYTRM1,MCTRM2,AST,IST,RST,CST)       !utilite
*
            ELSE
               NUMERR=34
               GOTO 99990
            ENDIF
*
         ELSEIF (LENTRA.EQ.4) THEN
*
            IF (UTRANS(1:LENTRA).EQ.'REAL') THEN
*
*              Partie reelle des coefficients du terme 1 complexe
*
               DO 410 ADTRM1=MCTRM1,MCTRM1+LGTRM2-1
                  RST(MCTRM2)=REAL (CST(ADTRM1))
                  MCTRM2=MCTRM2+1
410             CONTINUE
*
            ELSEIF (UTRANS(1:LENTRA).EQ.'ONES') THEN
*
*              Creation d'un terme dont tous les coefficients valent 1
*              avec les attributs du terme initial
*
               IF (TYTRM1(1:1).EQ.'R') THEN
                  DO 412 ADTRM1=MCTRM2,MCTRM2+LGTRM2-1
                     RST(ADTRM1)=1.
412               CONTINUE
               ELSEIF (TYTRM1(1:1).EQ.'C') THEN
                  Z=CMPLX(1.,0.)
                  DO 414 ADTRM1=MCTRM2,MCTRM2+LGTRM2-1
                     CST(ADTRM1)=Z
414               CONTINUE
               ENDIF
*
            ELSEIF (UTRANS(1:LENTRA).EQ.'SQRT') THEN
*
*              Racine carree des coefficients du terme 1 complexe
*
               IF (TYTRM1(1:1).EQ.'R') THEN
                  DO 422 ADTRM1=MCTRM1,MCTRM1+LGTRM2-1
                     RST(MCTRM2)=SQRT (RST(ADTRM1))
                     MCTRM2=MCTRM2+1
422               CONTINUE
               ELSEIF (TYTRM1(1:1).EQ.'C') THEN
                  DO 424 ADTRM1=MCTRM1,MCTRM1+LGTRM2-1
                     CST(MCTRM2)=SQRT (CST(ADTRM1))
                     MCTRM2=MCTRM2+1
424               CONTINUE
               ENDIF
            ELSE
               NUMERR=34
               GOTO 99990
            ENDIF
*
         ELSEIF (LENTRA.EQ.3) THEN
*
            IF (UTRANS(1:LENTRA).EQ.'ABS') THEN
*
               IF (TYTRM1(1:1).EQ.'R') THEN
                  DO 312 ADTRM1=MCTRM1,MCTRM1+LGTRM2-1
                     RST(MCTRM2)=ABS (RST(ADTRM1))
                     MCTRM2=MCTRM2+1
312               CONTINUE
               ELSEIF (TYTRM1(1:1).EQ.'C') THEN
                  DO 314 ADTRM1=MCTRM1,MCTRM1+LGTRM2-1
                     RST(MCTRM2)=ABS (CST(ADTRM1))
                     MCTRM2=MCTRM2+1
314               CONTINUE
               ENDIF
*
            ELSEIF (UTRANS(1:LENTRA).EQ.'EXP') THEN
*
               IF (TYTRM1(1:1).EQ.'R') THEN
                  DO 322 ADTRM1=MCTRM1,MCTRM1+LGTRM2-1
                     RST(MCTRM2)=EXP (RST(ADTRM1))
                     MCTRM2=MCTRM2+1
322               CONTINUE
               ELSEIF (TYTRM1(1:1).EQ.'C') THEN
                  DO 324 ADTRM1=MCTRM1,MCTRM1+LGTRM2-1
                     CST(MCTRM2)=EXP (CST(ADTRM1))
                     MCTRM2=MCTRM2+1
324               CONTINUE
               ENDIF
*
            ELSEIF (UTRANS(1:LENTRA).EQ.'LOG') THEN
*
               IF (TYTRM1(1:1).EQ.'R') THEN
                  DO 332 ADTRM1=MCTRM1,MCTRM1+LGTRM2-1
                     RST(MCTRM2)=LOG (RST(ADTRM1))
                     MCTRM2=MCTRM2+1
332               CONTINUE
               ELSEIF (TYTRM1(1:1).EQ.'C') THEN
                  DO 334 ADTRM1=MCTRM1,MCTRM1+LGTRM2-1
                     CST(MCTRM2)=LOG (CST(ADTRM1))
                     MCTRM2=MCTRM2+1
334               CONTINUE
               ENDIF
*
            ELSEIF (UTRANS(1:LENTRA).EQ.'1./') THEN
* 
*              Inverses des coefficients du terme 1
* 
               IF (TYTRM1(1:1).EQ.'R') THEN
                  DO 342 ADTRM1=MCTRM1,MCTRM1+LGTRM1-1
                     RST(MCTRM2)=1./RST(ADTRM1)
                     MCTRM2=MCTRM2+1
342               CONTINUE
               ELSEIF (TYTRM1(1:1).EQ.'C') THEN
                  DO 344 ADTRM1=MCTRM1,MCTRM1+LGTRM1-1
                     CST(MCTRM2)=1./CST(ADTRM1)
                     MCTRM2=MCTRM2+1
344               CONTINUE
               ENDIF
*
            ELSE
               NUMERR=34
               GOTO 99990
            ENDIF
*
         ELSEIF (LENTRA.EQ.1) THEN
*
            IF (UTRANS(1:1).EQ.'='.OR.UTRANS(1:1).EQ.'+') THEN
               CALL TDANST (LGTRM2,TYTRM1,MCTRM1,IST,RST,CST
     &                            ,TYTRM2,MCTRM2,IST,RST,CST)           !utilite
            ELSEIF (UTRANS(1:1).EQ.'-') THEN
               CALL TMDANT (LGTRM2,TYTRM1,MCTRM1,RST,CST
     &                            ,TYTRM2,MCTRM2,RST,CST)               !utilite
            ELSE
               NUMERR=34
               GOTO 99990
            ENDIF
         ELSE
            NUMERR=34
            GOTO 99990
         ENDIF
      ENDIF
*
      CALL TBSAVE (NMTRM1,NVTRM1)
      IF (.NOT.SAMTRM) CALL TBSAVE (NMTRM2,NVTRM2)
      CALL PRFXMJ (-1,'*T1Term*')
      RETURN
*-----------------------------------------------------------------------
99990 CALL PRNTRM (NMTRM1,NVTRM1,ERCODE,NCATR1)
      WRITE (*,*) 'Operation ',ERCODE(1:NCATR1),'<-',TRANS
      CALL ERTERM (NUMERR,NMTRM1,NVTRM1)
99991 CALL PRNTRM (NMTRM2,NVTRM2,ERCODE,NCATR1)
      CALL PRNTRM (NMTRM2,NVTRM2,ERCODE(NCATR1+1:),NCATR2)
      CALL ENCLER (NUMERO,ERCODE(NCATR1+NCATR2+1:NCATR1+NCATR2+4))
      CALL BAISE  ('Remplissage de la colonne No '
     &//ERCODE(NCATR1+NCATR2+1:NCATR1+NCATR2+4)//' du terme '
     &//ERCODE(1:NCATR1)//' a l''aide du terme '
     &//ERCODE(NCATR1+NCATR2+1:NCATR1+NCATR2+4)//' impossible.')
99992 CALL PRNTRM (NMTRM1,NVTRM1,ERCODE,NCATR1)
      CALL ENCLER (NUMERO,ERCODE(NCATR1+1:NCATR1+4))
      CALL BAISE  ('Extraction de la colonne No '
     &//ERCODE(NCATR1+1:NCATR1+4)//' du terme '//ERCODE(1:NCATR1)
     &//' impossible.')
99993 CALL PRNTRM (NMTRM1,NVTRM1,ERCODE,NCATR1)
      CALL ENCLER (NUMERO,ERCODE(NCATR1+1:NCATR1+4))
      CALL BAISE  ('Extraction des coefficients pour la composante No '
     &//ERCODE(NCATR1+1:NCATR1+4)//' du terme '//ERCODE(1:NCATR1)
     &//' impossible.')
99994 CALL PRNTRM (NMTRM2,NVTRM2,ERCODE,NCATR1)
      CALL ENCLER (NUMERO,ERCODE(NCATR1+1:NCATR1+4))
      CALL BAISE  ('Report des coefficients pour la composante No '
     &//ERCODE(NCATR1+1:NCATR1+4)//' du terme '//ERCODE(1:NCATR1)
     &//' impossible.')
99995 CALL PRNTRM (NMTRM1,NVTRM1,ERCODE,NCATR1)
      WRITE (*,*) 'Operation ',ERCODE(1:NCATR1),'<-',TRANS
      CALL BAISE  ('Creation d''un terme multicolonne impossible.'
     &//' Le terme n''est pas uni-colonne.')
99996 CALL PRNTRM (NMTRM1,NVTRM1,ERCODE,NCATR1)
      WRITE (*,*) 'Operation ',ERCODE(1:NCATR1),'<-',TRANS
      CALL BAISE  ('Creation d''un terme multicolonne impossible')
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 6 : D.Martin (31.janvier.2011)
* - superbe (et assez ancien) bug corrig dans l'appel de GETTRM situe dans le
* - bloc de IF (UTRANS(1:1).EQ.'>') THEN, le second argument ne peut s'appeler
* - NVTRM2 comme c'etait le cas ...
* Version 5 : D.Martin (28/30.janvier.2011)
* - ajout du cas '>COMPOSANTE' pour le les coeff. du terme 1, a inconnue scalaire
*   dans les coeff. du terme 2, a inco. vectorielle, correspondant a une compo.
* Version 4 : D.Martin (26.mars.2010)
* - ajour d'un message d'erreur en cas de stockage non reconnu pour le cas '>COL'
* Version 3 : D.Martin (21.oct.2006)
* - ajout de l'initialisation comme matrice identite pour les matrices pleines
* Version 2 : D.Martin (12.juin.2004)
* Version 1.9 : D.Martin (24.oct.2002) ajout 'Inverse'
* Version 1.8 : D.Martin (7.oct.2002) bug argument TAZERO
* Version 1.7 : D.Martin (2.sep.2002) bug pour TRANS='-'
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !T1Term

