*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE CALGEO (NDMDOM,NDMDOE,NUDOMG,NUDOMK,NCHDOM,NBELDO
     &                  ,NXLIST,LIOUCO,INTERP,INDEXS,NBNEFX,NBNEDO
     &                  ,IRPLOC,ICORNE,NVNORM,NVCORN,NIVIMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Novembre 1988)
* Derniere modification : D.Martin (6 Septembre 2001)
* Version 1.1.1
*  
* Calcul des coordonnees des noeuds et des normales unitaires aux noeuds
* sur un domaine pour une interpolation donnee
*  
*-- Arguments --  
* NDMDOM dimension (des constituants) du domaine
* NDMDOE dimension des elements supports des constituants du domaine geometrique
* NUDOMG numero du domaine geometrique
* NUDOMK numero du domaine de calcul
* NCHDOM nombre de caracteres par nom de domaine
* NBELDO nombre de constituants (aretes, faces ou elements) du domaine  
* NXLIST pointeur dans #ISTEL pour le 1er element
* LIOUCO chaine de caracteres indiquant 'Ligne' ou 'Colonne' 
* INTERP niveau de l'interpolation en ligne ou colonne selon LIOUCO
* INDEXS indicateur d'extension du domaine en ligne ou colonne
* NBNEFX nombre maximum de noeuds pour l'interpolation sur une face
*        de dimension NDMDOM d'un element du domaine
* NBNEDO nombre de noeuds sur le domaine selon l'interpolation selon LIOUCO
* IRPLOC indicateur de calcul des normales sur le domaine
* ICORNE     -            -       coordonnees des noeuds sur le domaine
* NVNORM niveau du tableau &NORMA des normales unitaires sur le domaine
* NVCORN le cas echeant niveau du tableau &CORNE des coordonnees des noeuds
* NIVIMP niveau d'impression du domaine
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) LIOUCO
      INTEGER       NDMDOM,NDMDOE,NUDOMG,NUDOMK,NCHDOM,NBELDO,NXLIST
     &             ,INTERP,INDEXS,NBNEFX,NBNEDO,IRPLOC,ICORNE,NVNOR
     &             M,NVCORN,NIVIMP
*
      INCLUDE 'ALLOC'  
      INCLUDE 'CONTEX' 
*
      CHARACTER     PRFXAF*8
      INTEGER       MCNORM,MCCORN,MCGOCO,MCCORL,MCREPR,MCNECG,MCDLCG
     &             ,MCDFCG,MCBLOK,MCMDOM,MCLIST,MCCORP,MCWORK,MCRPLC
     &             ,MCCORF,MCFCTG,MCNONO,LGWORK,ADMDOM
     &             ,LISRAN,NUTGAV,NUFAAV,NIVAMP,ELEMEN,LAG,KCL
     &             ,NUTGEO,NUELEM,NUTYFG,NBNEEC,NBDLEC,NUFACE,NUTYFC
     &             ,NBNEFC,NBDLFC,NUTELC,NUTYLG,NBNEEG,NBDLEG,NBNEFG
     &             ,NBDLFG,NBNEUL,IADDLL,ADCORF,ADCORP,IDLF,IDLL,NUTEAV
     &             ,NIVBMP,IDM,NCATAB,LG,NUTFAV
      REAL          NORME
      CHARACTER     ERCODE*120,THETAB*16
      COMMON/FORMAH/ERCODE 
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*CalGeo*')
*
      CALL ENCLER (NUDOMK,ERCODE(2:4))                                  !utilite
*
      IF (IRPLOC.NE.0) THEN
         ERCODE(1:1)='!'
         CALL TBRR1  (ERCODE,'&NORMA',NVNORM,MCNORM)
         IF (ERCODE(1:1).EQ.'C'.OR.MCNORM.LT.NDIM*NBNEDO)
     &   CALL TBCREE ('&NORMA',NVNORM,2,NDIM*NBNEDO,'c')
      ENDIF
*
      IF (ICORNE.NE.0) THEN
         ERCODE(1:1)='!'
         CALL TBRR1  (ERCODE,'&CORNE',NVCORN,MCCORN)
         IF (ERCODE(1:1).EQ.'C'.OR.MCCORN.LT.NDIM*NBNEDO)
     &   CALL TBCREE ('&CORNE',NVCORN,2,NDIM*NBNEDO,'c')
      ENDIF
*
*     Tableau des rangs des noeuds des elements du domaine
*     dans la numerotation des noeuds du domaine
*
      CALL TBRR1  (ERCODE,'#RGNDO',NUDOMK,MCGOCO)
*
*     Calcul des adresses relatives des tableaux de travail
*
*     Tableau des coordonnees des points dans (la face de) l'element courant 
*     longueur NDMDOM*NBPTFX si necessaire (domaine etendu)
*     coincide avec le tableau des coordonnees #ORPTL des points sinon
      MCCORF=0
      LG=0
      IF (INDEXS.GE.1) LG=NDMDOM*NBPTFX
*     Tableau dex coordonnees des noeuds sur la face de l'element de reference
*     pour l'interpolation en colonne, longueur NDMDOM*NBNEFX
      MCCORL=MCCORF+LG
      LG=NDMDOM*NBNEFX
*     Tableau des reperes locaux aux noeuds de la face de l'element courant
*     longueur NDIM*NBNEFX ou IRPLOC*NDIM*NBNEFX
      MCRPLC=MCCORL+LG
      LG=NDIM*NBNEFX
      IF (IRPLOC.GT.0) LG=IRPLOC*NDIM*NBNEFX
*     Repere local en un noeud (tableau de travail)
*     longueur NDIM*NDIM
      MCREPR=MCRPLC+LG
      LG=NDIM*NDIM
*     Tableau de travail pour NUNEFA
*     longueur MAX(NBNEFX,NBPTFX)
      MCNECG=MCREPR+LG
      LG=MAX (NBNEFX,NBPTFX)
*     Tableau de travail pour NUDLFA
*     longueur MAX(NBDLFX,NBPTFX)
      MCDLCG=MCNECG+LG
      LG=MAX (NBNEFX,NBPTFX)
*     Tableau des valeurs des fonctions de base geometriques
*     longueur NBPTFX si necessaire (coordonnees des noeuds)
      MCFCTG=MCDLCG+LG
      LG=0
      IF (ICORNE.NE.0) LG=NBPTFX
*     Tableau des derivees des fonctions de base geometriques
*     longueur NDMDOM*NBPTFX si necessaire (normales)
      MCDFCG=MCFCTG+LG
      LG=0
      IF (IRPLOC.GT.0) LG=NDMDOM*NBPTFX
*
*     Longueur puis creation du tableau de travail
      LGWORK=MCDFCG+LG 
      CALL TBCREE ('$WORK$',0,2,LGWORK,'c')
*
*     Adresse du tableau #LOKEL de description des elements par blocs
*     (inutile si tous les elements du maillage sont de meme type geometrique)
*
      MCBLOK=1
      IF (NBBBLK.GE.2) CALL TBAR1  (ERCODE,'#LOKEL',1,MCBLOK)
*
*     Adresses des rangs des noeuds du domaine de calcul, 
*              de la liste des elements du domaine geometrique,
*              des coordonnees des points du domaine de calcul
*              et du tableau de travail
*
      CALL TBAR4  (ERCODE,'#RGNDO',NUDOMK,MCGOCO,'#ORPTL',NUDOMK,MCCORP
     &                   ,'#ISTEL',NUDOMG,MCLIST,'$WORK$',0,MCWORK)
*
      IF (IRPLOC.NE.0) THEN
*
*        Adresse puis initialisation du tableau des normales
*
         CALL TBAR1  (ERCODE,'&NORMA',NVNORM,MCNORM)
         CALL TAZERO (NDIM*NBNEDO,'R',MCNORM,AST,IST,RST,CST)           !utilite
      ENDIF
*
      IF (ICORNE.NE.0) THEN
*
*        Adresse du tableau des coordonnees de noeuds du domaine
*
         CALL TBAR1  (ERCODE,'&CORNE',NVCORN,MCCORN)
      ENDIF
*
*    Selon que l'on travaille pour l'interpolation en colonne ou ligne,
*    adresse du tableau des rangs des noeuds des elements du domaine
*
      IF (LIOUCO(1:1).NE.'C') THEN
         MCGOCO=MCGOCO+IST(MCGOCO)-1
      ELSE
         MCGOCO=MCGOCO+1
      ENDIF
*
*     Adresses abslues des differents sous-tableaux du tableau de travail
*
      MCCORL=MCWORK+MCCORL
      MCRPLC=MCWORK+MCRPLC 
      MCREPR=MCWORK+MCREPR 
      MCDLCG=MCWORK+MCDLCG
      MCFCTG=MCWORK+MCFCTG
      MCDFCG=MCWORK+MCDFCG 
*
*     Initialisation pour le parcours des elements du domaine
*
      LISRAN=0 
      NUTGAV=0
      NUTEAV=0 
      NUFAAV=0 
      NIVAMP=NIVIMP
      NUTFAV=0
*
      DO 100 ELEMEN=1,NBELDO
*
*        Nombre de noeuds de (la face de) l'element courant
*
         NBNEUL=IST(MCGOCO)
*        Pour impression
         NIVIMP=0  
         IF (ELEMEN.EQ.1.OR.ELEMEN.EQ.NBELDO) NIVIMP=NIVAMP
         NIVBMP=NIVIMP
         IF (INTERP.NE.INTRPG) NIVBMP=0
         IF (NIVIMP.GT.0) WRITE (IMPSDR,10000) PRFXAF( ),ELEMEN
*  
*        Exploitation de la structure #ISTEL et recherche des donnees  
*        geometriques et pour l'interpolation
*
         CALL EXSTEL (NDMDOM,NDMDOE,LISRAN,IST(MCLIST+NXLIST)
     &               ,IST(MCBLOK),NUTGEO,INTRPG,NUELEM,NUTGAV,NUTYLG
     &               ,LAG,KCL,NBNEEG,NBDLEG,NUFACE,NUFAAV,NUTYFG
     &               ,NBNEFG,NBDLFG,IST(MCDLCG),IST(MCDLCG)
     &               ,NIVBMP,IMPSDR)                                    !sdexplo
         IF (INTERP.EQ.INTRPG) THEN 
            NUTYFC=NUTYFG
            NBNEFC=NBNEFG
         ELSE
            CALL EXSTEW (NDMDOM,INTERP,NUTGEO,NUELEM,NUTEAV,NUTELC
     &                  ,LAG,KCL,NBNEEC,NBDLEC,NUFACE,NUFAAV,NUTYFC
     &                  ,NBNEFC,NBDLFC,IST(MCDLCG),IST(MCDLCG)
     &                  ,NIVIMP,IMPSDR)                                 !sdexplo
         ENDIF
*
*        Coordonnees des noeuds de l'element de reference correspondant
*        a l'interpolation pour laquelle on travaille
*
         IF (NUTYFC.NE.NUTFAV) CALL ELEMCO (NUTYFC,NDMDOM,RST(MCCORL))  !ef3d
         NUTFAV=NUTYFC
*
         IF (INDEXS.GE.1) THEN
*
*           Si le domaine est etendu, transfert des coordonnees de points 
*           sur la face dans un tableau indice par la numerotation de la face
*
            MCCORF=MCWORK+MCCORF
            IADDLL=MCDLCG
            ADCORF=MCCORF
            DO 10 IDLF=1,NBDLFG
*              Numero local IDLL du noeud IDLF de la face
               IDLL=IST(IADDLL)
               ADCORP=MCCORP+2+NDIM*(IDLL-1)
               DO 11 IDM=1,NDIM
                  RST(ADCORF)=RST(ADCORP)
                  ADCORF=ADCORF+1
                  ADCORP=ADCORP+1
11             CONTINUE
               IADDLL=IADDLL+1
10          CONTINUE
         ELSE
*           Sinon les points sont ceux de la face:
            MCCORF=MCCORP+2
         ENDIF
*
         IF (IRPLOC.GT.0) THEN
*
*           Calcul des normales sur le domaine aux noeuds
*
            CALL REPLOC (NDIM,NUTYFG,NBDLFG,RST(MCCORF)
     &                  ,NBNEFC,RST(MCCORL),RST(MCDFCG),IRPLOC
     &                  ,RST(MCRPLC),RST(MCREPR)
     &                  ,NIVIMP,IMPSDR)                                 !calelem
*
*           Assemblage des normales
*
            CALL ASNORM (IST(MCCORP+1),NDIM,NBNEFC,IST(MCGOCO+1)
     &                  ,RST(MCRPLC),RST(MCNORM))                       !caldom
         ENDIF
*
         IF (ICORNE.NE.0) THEN
*
*           Calcul des coordonnees des noeuds du domaine
*
            IF (INTERP.NE.INTRPG) THEN
               CALL CORNEL (NDIM,NDMDOM,NUTYFG,NBDLFG,RST(MCCORF)
     &                     ,NBNEFC,RST(MCCORL),RST(MCFCTG),RST(MCRPLC)
     &                     ,NIVIMP,IMPSDR)                              !calelem
*
               CALL ASCORN (NDIM,IST(MCGOCO),IST(MCGOCO+1),RST(MCRPLC)
     &                     ,RST(MCCORN))                                !caldom
            ELSE
               CALL ASCORN (NDIM,IST(MCGOCO),IST(MCGOCO+1),RST(MCCORF)
     &                     ,RST(MCCORN))                                !caldom
            ENDIF
         ENDIF
         NUTGAV=NUTGEO 
         NUTEAV=NUTELC
         NUFAAV=NUFACE
         MCCORP=MCCORP+2+IST(MCCORP)*NDIM  
         MCGOCO=MCGOCO+NBNEUL+1
100   CONTINUE
*
*     Normalisation des vecteurs normaux assembles sur le domaine
*
      IF (IRPLOC.GT.0) THEN
         MCNONO=MCNORM
         DO 200 NBNEUL=1,NBNEDO
            CALL NORMIZ (NDIM,RST(MCNONO),NORME)                        !utilite
            MCNONO=MCNONO+NDIM
200      CONTINUE
      ENDIF
*
      IF (NIVIMP.GT.0.AND.IMPSDR.GT.0) THEN
*        Adresse des tableaux des noms de domaines
         CALL TBAR1  (ERCODE,'#OMDOM',1,MCMDOM)
*        Adresse absolue du nom du domaine
         ADMDOM=MCMDOM+NCHDOM*(NUDOMG-1)
      ENDIF
      IF (IRPLOC.NE.0) THEN
*
*        Impression du tableau des normales
*
         IF (NIVIMP.GT.0.AND.IMPSDR.GT.0) THEN
            WRITE (IMPSDR,10001) PRFXAF(),'Normales unitaires'
     &                          ,(AST(KCL),KCL=ADMDOM,ADMDOM+NCHDOM-1)
            CALL PRCOOR (NDIM,NBNEDO,RST(MCNORM),' Normal'
     &                  ,NIVIMP,IMPSDR)                                 !prsd
         ENDIF
         IF (NIVIMP.GE.0.AND.IMPPAL.GT.0) THEN
            CALL PRNTRM ('&NORMA',NVNORM,THETAB,NCATAB)                 !utilite
            WRITE (IMPPAL,10002) PRFXAF(),THETAB(1:NCATAB)
     &                          ,'Normales assemblees'
         ENDIF
      ENDIF
*
      IF (ICORNE.NE.0) THEN
*
*        Impression du tableau des coordonnees des noeuds
*
         IF (NIVIMP.GT.0.AND.IMPSDR.GT.0) THEN
            WRITE (IMPSDR,10001) PRFXAF(),'Coordonnes des noeuds'
     &                          ,(AST(KCL),KCL=ADMDOM,ADMDOM+NCHDOM-1)
            CALL PRCOOR (NDIM,NBNEDO,RST(MCCORN),' Coordo'
     &                  ,NIVIMP,IMPSDR)                                 !prsd
         ENDIF
         IF (NIVIMP.GE.0.AND.IMPPAL.GT.0) THEN
            CALL PRNTRM ('&CORNE',NVCORN,THETAB,NCATAB)                 !utilite
            WRITE (IMPPAL,10002) PRFXAF(),THETAB(1:NCATAB)
     &                          ,'Coordonnes des noeuds'
         ENDIF
      ENDIF
*     Sauvegarde des tableaux des normales et des coordonnees des noeuds
      IF (IRPLOC.NE.0) CALL TBSAVE ('&NORMA',NVNORM)
      IF (ICORNE.NE.0) CALL TBSAVE ('&CORNE',NVCORN)
*     Destruction du tableau de travail
      CALL TBTUER ('$WORK$',0)
      CALL PRFXMJ (-1,'*CalGeo*')
*----------------------------------------------------------------------- 
10000 FORMAT(/T2,A8,T30,'Calculs geometriques dans l''element',I6 
     &/T30,36('-'))
10001 FORMAT(/T2,A8,' ',A,' sur : ',50A1)
10002 FORMAT(T5,A8,' tableau ',A,' ',A)
                                                                    END !CalGeo
