*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE CALELB (ELEMEN,FACE  ,NDIM  ,NDMDOM,NBTRDO,INDTRD
     &                  ,NCHDON,NOMDON,NUTGFV,ISGDET,CORDLG
     &                  ,NBPTQX,NUSCHQ,NBPTQD,POIDQD,COORQD
     &                  ,NUTYFC,NBDLFC,KELFCC,FCTBC,DFCTBC
     &                  ,NUTYFL,NBDLFL,KELFCL,FCTBL,DFCTBL
     &                  ,NUTYFG,NBDLFG,KELFCG,FCTBG,DFCTBG
     &                  ,MCMATR,MCMATC,IST,RST,CST,NODONA,NOMTRA
     &                  ,NIVIMP,IMPFCH)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Novembre 1988)
* Derniere modification : D.Martin (23 juin 2005)
* Version 4
*  
* Calcul des integrales elementaires sur un domaine  de bord ne necessitant pas
* d'extension (integrales sans derivees ou avec derivees tangentielles)
* -> Seule la geometrie de la face/arete est prise en compte et les calculs
*    sont effectues sur l'element de reference associe a la face/arete
*  
*-- Arguments --
* ELEMEN numero de l'element en cours 
* FACE   numero de la face ou de l'arete
* NDIM   dimension d'espace
* NDMDOM dimension du domaine d'integration (face ou arete)
* NBTRDO nombre de termes a calculer sur le domaine
* INDTRD structure locale de description des termes sur le domaine
*     Contenu du tableau INDTRD
*     (NUINT          ) niveau (plus tard adresse absolue du terme)
*     (NUINT+   NBTRDO) adresses relatives des matrices elementaires
*                       dans le tableau des matrices elementaires du bon type
*     (NUINT+ 2*NBTRDO) adresses relatives des tableaux de valeurs
*                       nodales dans le tableau des matrices elementaires
*     (NUINT+ 3*NBTRDO) type REEL ou COMPLEXE du terme
*     (NUINT+ 4*NBTRDO) indice de type de calcul
*     (NUINT+ 5*NBTRDO) numero d'integrand
*     (NUINT+ 6*NBTRDO) nombre de composantes de l'inconnue en colonne
*     (NUINT+ 9*NBTRDO) nombre de composantes de l'inconnue en ligne
*     (NUINT+12*NBTRDO) numero de donnee associee si FONCTION ou TABLEAU
*     (NUINT+13*NBTRDO) type de la donnee associee (FONCTION,TABLEAU)
*     (NUINT+14*NBTRDO) niveau ou numero de la donnee associee
*     (NUINT+15*NBTRDO) type du declaration de la fonction ou du tableau
*     (NUINT+16*NBTRDO) adresse de la donnee TABLEAU associee
*                       ou adresse de la CONSTANTE ou du TABLEAU associe
*                       dans le cas d'une donnee FONCTION
* NCHDON nombre de caracteres par nom de donne
* NOMDON tableau #OMDON des noms de donnees
* NUTGFV numero de type geometrique de la face lors de l'appel precedent
* ISGDET signe indiquant l'orientation de l'element (pour la normale ext.)
* CORDLG coordonnees des points de la face/ arete de l'element
* NBPTQX nombre maximum de points de quadrature
* NUSCHQ numero du schema de quadrature 
* NBPTQD nombre de points de la formule de quadrature
* POIDQD tableau recepteur des poids de quadrature
* COORQD tableau recepteur des coordonnees de la formule de quadrature
* NUTYFC numero de type d'interpolation en colonne de la face FACE
* NBDLFC nombre de d.l. sur une face ou arete pour l'interpolation en colonne
* KELFCC indice de calcul des fonctions de base pour l'interp. en colonne
*      (1:1)='Y' ==> calcul des fonctions de base
*      (2:2)='Y' ==> calcul des derivees des fonctions de base
*      (3:3)='Y' ==> calcul des derivees secondes des fonctions de base
*      (4:4)='C' ==> calculs selon l'interpolation en 'colonne'
*            'G' ==> calculs selon l'interpolation geometrique
*      (5:5)='Y' ==> on conserve les calculs en tous les points
*                    de quadrature pour l'element suivant
*      (6:6)='Y' ==> inutilise ici
* FCTBC  tableau de calcul des valeurs des fonctions de base dans l'element
*        de reference pour l'interpolation en colonne (en tous les points de
*        quadrature)
* DFCTBC    -          -          -        derivees des fonctions dans l'element
*        de reference pour l'interpolation en colonne
* NUTYFL numero de type d'interpolation en ligne de la face FACE
* NBDLFL nombre de d.l. sur une face ou arete pour l'interpolation en ligne
* KELFCL indice de calcul des fonctions de base pour l'interp. en ligne
*        (cf. KELFCC)
* FCTBL  tableau de calcul des valeurs des fonctions de base dans l'element
*        de reference pour l'interpolation en ligne (en tous les points de
*        quadrature)
* DFCTBL    -          -          -        derivees des fonctions dans l'element
*        de reference pour l'interpolation en ligne
* NUTYFG numero de type geometrique de la face FACE
* NBDLFG nombre de points sur la face de l'element geometrique
* KELFCG indice de calcul des fonctions de base geometriques
*        (cf. KELFCL, KELFCC)
* FCTBG  valeurs des fonctions de base pour les calcul geometriques sur le bord
* DFCTBG derivees des fct. de base pour les calculs geometriques sur le bord  
* MCMATR adresse du tableau des matrices elementaires reelles 
* MCMATC adresse du tableau des matrices elementaires complexes
* IST    super-tableau entier 
* RST    super-tableau reel
* CST    super-tableau complexe
* NODONA chaine de caractere receptrice d'un nom de donnee (cf. CalinB)
* NOMTRA chaine de caractere receptrice d'un nom de fonction (cf. CalinB)
* NIVIMP niveau d'impression  
* IMPFCH numero de l'unite logique des impressions
*
* Remarques: L'argument NUTGFV permettent d'eviter de rappeler la procedure de
*        calcul de la formule de quadrature et surtout de refaire les calculs
*        des fonctions de base si la place necessaire pour leur conservation 
*        est disponible en M.C. 
*        (test sur la valeur de KELFCx(5:5))
*            Les 6 tableaux FCTBC,FCTBL,FCTBG,DFCTBC,DFCTBL,DFCTBG peuvent 
*        contenir les valeurs des fonctions de base et derivees pour 1 ou 
*        tous les points de quadrature selon la place en M.C.
*        (test sur KELFCx(5:5))
*            De plus 2 ou 3 des tableaux FCTBx ou DFCTBx peuvent coincider
*        (adresses d'entree egales) lorsque les interpolations correspondantes
*        sont les memes (Test sur KELFCx(4:4))
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) NODONA,NOMTRA,KELFCC,KELFCL,KELFCG
      CHARACTER*1   NOMDON(*)
      INTEGER       ELEMEN,FACE,NDIM,NDMDOM,NBTRDO,NCHDON,NUTGFV,ISGDET
     &             ,NBPTQX,NUSCHQ,NBPTQD,MCMATR,MCMATC,INDTRD(*)
     &             ,NUTYFG,NBDLFG,NUTYFC,NBDLFC,NUTYFL,NBDLFL
     &             ,NIVIMP,IMPFCH
     &             ,IST(*)
      REAL          CORDLG(NDIM,*),POIDQD(*),COORQD(NDMDOM,*)
     &             ,FCTBG(*),DFCTBG(NDMDOM,*)
     &             ,FCTBC(*),DFCTBC(NDMDOM,*)
     &             ,FCTBL(*),DFCTBL(NDMDOM,*)
     &             ,RST(*)
      COMPLEX       CST(*) 
*
* Variables locales
      CHARACTER     FORMAN*120
      INTEGER       IMPINT,IMPFCT,IMP,NUINT,MIXTE,NUPTQD
     &             ,IAFCG,IADFCG,IAFCC,IADFCC,IAFCL,IADFCL,I
* POIN   tableau de travail pour recevoir les coordonnees d'un point
*        la norme du vecteur normal et ses composantes
* JACOBF tableau de travail pour recevoir les vecteurs tangents
*        pour le calcul de l'element differentiel (matrice jacobienne)
      REAL          POIN(7),JACOBF(9),ELTDIF
      LOGICAL       KLAGEF 
*
      COMMON/ONSEPA/FORMAN
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*CalelB*')
*
      IMP=IMPFCH
      IF (NIVIMP.LE.0) IMP=0
*
*     Variable impliquant le necessaire recalcul des fonctions de base
*     (premier element ou changement de type de la face /arete de l'element) 
      KLAGEF=NUTYFG.NE.NUTGFV
*  
*     Formule de quadrature
*     ---------------------
      IF (KLAGEF) THEN
         CALL ELEMQD (NUTYFG,NDMDOM,NUSCHQ,NBPTQD,POIDQD,COORQD,IMP)    !Ef3d
         IF (NBPTQD.GT.NBPTQX) GOTO 99995  
      ENDIF
*
*     Indices pour impression de resultats partiels
      IMPINT=0 
      IMPFCT=0 
      IF (NIVIMP.GT.0) THEN 
         IF (NIVIMP.GT.5) THEN
            IMPINT=IMPFCH 
            IMPFCT=IMPFCH 
            WRITE (IMP,10000) ELEMEN,FACE,NUTYFC,NUTYFG,1,NBPTQD
         ELSE
            WRITE (IMP,10001) ELEMEN,FACE,NUTYFC,NUTYFG,NBPTQD
         ENDIF
         IF (.NOT.KLAGEF) IMPFCT=0  
         DO 10 NUINT=1,NBTRDO 
           IF (INDTRD(NUINT+5*NBTRDO).GT.0) 
     &        WRITE (IMP,10002) INDTRD(NUINT+5*NBTRDO)
     &       ,ABS(INDTRD(NUINT+12*NBTRDO)),INDTRD(NUINT+NBTRDO)
10       CONTINUE  
      ENDIF
*     Pour annuler les composantes du vecteur normal
      DO 11 I=NDIM+2,7
         POIN(I)=0.
11    CONTINUE
*-------------------------------------------------------------------------------
* Plan :
*   I--> Calculs geometriques sur la face
*     a) Derivees des fonctions de base geometriques
*     b) Calcul des vecteurs tangents et de l'element de surface
*     c) Le cas echeant, calcul de l'image par la transformation geometrique
*       du point de quadrature dans l'element
*       (Rappel : Les calculs geometriques (jacobien et element differentiel)
*        utilisent l'element geometrique (NUTYFG), les calculs d'integrands
*        utilisent les fonctions de base de l'element pour l'interpolation
*        en colonne (NUTYFC) et ligne (NUTYFL).
*
*  II--> Calcul des fonctions de base des interpolations en colonne puis ligne
*        dans le calcul des integrands dans le cas non isoparametrique ou mixte.
*        Dans le cas isoparametrique on utilise les fonctions de base deja
*        calculees pour la geometrie (cf. affectation des tableaux dans ADWKEF)
*     f) calcul des fonctions et derivees de fontions de base dans l'element
*        de reference (cas d'une interpolation non isoparametrique)
*
* III--> Calcul proprement dit des matrices elementaires
*-------------------------------------------------------------------------------
      DO 111 NUPTQD=1,NBPTQD
*
*
* --I) Calculs geometriques
*     a) Derivees des fonctions de base geometriques
         CALL ECOBAS (KLAGEF,NUPTQD,COORQD(1,NUPTQD),NDMDOM,NUTYFG
     &               ,NBDLFG,NBDLFG,KELFCG
     &               ,FCTBG,DFCTBG,IAFCG,IADFCG,IMPFCT)
*
*     b) Calcul des vecteurs tangents
         CALL DELAFO (NDIM,NDMDOM,NBDLFG,CORDLG,DFCTBG(1,IADFCG)
     &               ,IMPINT,JACOBF)
*
*        Calcul de l'element de surface et de la normale unitaire
         CALL ELDIFF (NDIM,NDMDOM,JACOBF,ISGDET,ELTDIF,POIN(NDIM+2)
     &               ,IMPINT)
*        Norme du vecteur normal = Element de surface ou de longueur
         POIN(NDIM+1)=ELTDIF
*
*     c) Image du point de quadrature dans l'element courant
*        (cas des integrands a coefficient variable)
         IF (KELFCG(6:6).EQ.'Y')
     &      CALL TRANSF (NDIM,NBDLFG,CORDLG,FCTBG(IAFCG),POIN)
*
*        Calculs differencies selon les interpolations en colonne et ligne
*        -----------------------------------------------------------------
         IF (KELFCC(4:4).EQ.'G') THEN
*
            IF (KELFCL(4:4).EQ.'G') THEN
*  
*      f)      Interpolation isoparametrique en colonne et ligne
* III)
               MIXTE=NUTYFC-NUTYFL
               CALL CALINB (MIXTE,NDIM,NDMDOM,NBTRDO,INDTRD,NCHDON
     &                     ,NOMDON,POIDQD(NUPTQD),ELTDIF,POIN,JACOBF
     &                     ,NBDLFG,FCTBG(IAFCG),DFCTBG(1,IADFCG)
     &                     ,NBDLFG,FCTBG(IAFCG),DFCTBG(1,IADFCG)
     &                     ,MCMATR,MCMATC,IST,RST,CST,NODONA,NOMTRA
     &                     ,IMPINT)
*
            ELSEIF (KELFCL(4:4).EQ.'L') THEN
*  
*              Interpolation isoparametrique en colonne et non en ligne
* II)  f)
               CALL ECOBAS (KLAGEF,NUPTQD,COORQD(1,NUPTQD),NDMDOM
     &                     ,NUTYFL,NBDLFL,NBDLFL,KELFCL
     &                     ,FCTBL,DFCTBL,IAFCL,IADFCL,IMPFCT)
* III)
               MIXTE=NUTYFC-NUTYFL
               CALL CALINB (MIXTE,NDIM,NDMDOM,NBTRDO,INDTRD,NCHDON
     &                     ,NOMDON,POIDQD(NUPTQD),ELTDIF,POIN,JACOBF
     &                     ,NBDLFG,FCTBG(IAFCG),DFCTBG(1,IADFCG)
     &                     ,NBDLFL,FCTBL(IAFCL),DFCTBL(1,IADFCL)
     &                     ,MCMATR,MCMATC,IST,RST,CST,NODONA,NOMTRA
     &                     ,IMPINT)                 
*
            ENDIF
*
         ELSEIF (KELFCC(4:4).EQ.'C') THEN
           
*
            IF (KELFCL(4:4).EQ.'C') THEN
*  
*             Interpolation non-isoparametrique identiques en colonne et ligne
* II)  f)
               CALL ECOBAS (KLAGEF,NUPTQD,COORQD(1,NUPTQD),NDMDOM
     &                     ,NUTYFC,NBDLFC,NBDLFC,KELFCC
     &                     ,FCTBC,DFCTBC,IAFCC,IADFCC,IMPFCT)
* III)
               MIXTE=NUTYFC-NUTYFL
               CALL CALINB (MIXTE,NDIM,NDMDOM,NBTRDO,INDTRD,NCHDON
     &                     ,NOMDON,POIDQD(NUPTQD),ELTDIF,POIN,JACOBF
     &                     ,NBDLFC,FCTBC(IAFCC),DFCTBC(1,IADFCC)
     &                     ,NBDLFC,FCTBC(IAFCC),DFCTBC(1,IADFCC)
     &                     ,MCMATR,MCMATC,IST,RST,CST,NODONA,NOMTRA
     &                     ,IMPINT)
*
            ELSEIF (KELFCL(4:4).EQ.'G') THEN
*  
*             Interpolation non-iso. en colonne, isoparametrique en ligne
* II)  f)
               CALL ECOBAS (KLAGEF,NUPTQD,COORQD(1,NUPTQD),NDMDOM
     &                     ,NUTYFC,NBDLFC,NBDLFC,KELFCC
     &                     ,FCTBC,DFCTBC,IAFCC,IADFCC,IMPFCT)
* III)
               MIXTE=NUTYFC-NUTYFL
               CALL CALINB (MIXTE,NDIM,NDMDOM,NBTRDO,INDTRD,NCHDON
     &                     ,NOMDON,POIDQD(NUPTQD),ELTDIF,POIN,JACOBF
     &                     ,NBDLFC,FCTBC(IAFCC),DFCTBC(1,IADFCC)
     &                     ,NBDLFG,FCTBG(IAFCG),DFCTBG(1,IADFCG)
     &                     ,MCMATR,MCMATC,IST,RST,CST,NODONA,NOMTRA
     &                     ,IMPINT)
*
            ELSEIF (KELFCL(4:4).EQ.'L') THEN
*
*              Interpolation non-isoparametriques differentes en colonne et ligne
* II)  f)
               CALL ECOBAS (KLAGEF,NUPTQD,COORQD(1,NUPTQD),NDMDOM
     &                     ,NUTYFC,NBDLFC,NBDLFC,KELFCC
     &                     ,FCTBC,DFCTBC,IAFCC,IADFCC,IMPFCT)
               CALL ECOBAS (KLAGEF,NUPTQD,COORQD(1,NUPTQD),NDMDOM
     &                     ,NUTYFL,NBDLFL,NBDLFL,KELFCL
     &                     ,FCTBL,DFCTBL,IAFCL,IADFCL,IMPFCT)
* III)
               MIXTE=NUTYFC-NUTYFL
               CALL CALINB (MIXTE,NDIM,NDMDOM,NBTRDO,INDTRD,NCHDON
     &                     ,NOMDON,POIDQD(NUPTQD),ELTDIF,POIN,JACOBF
     &                     ,NBDLFC,FCTBC(IAFCC),DFCTBC(1,IADFCC)
     &                     ,NBDLFL,FCTBL(IAFCL),DFCTBL(1,IADFCL)
     &                     ,MCMATR,MCMATC,IST,RST,CST,NODONA,NOMTRA
     &                     ,IMPINT)
            ENDIF
         ENDIF
*
*        Pour les impressions au dernier point de quadrature
*
         IMPINT = 0
         IMPFCT = 0
         IF (NIVIMP.GT.0.AND.NUPTQD.EQ.NBPTQD-1) THEN
            IMPINT = IMPFCH
            IMPFCT = IMPFCH
          ENDIF
*
111   CONTINUE
      CALL PRFXMJ (-1,'*CalelB*')  
      RETURN
*-----------------------------------------------------------------------
10000 FORMAT(/' *CalelB*  Element #',I5,' Face ',I1,' : Type ',I5,  
     &' ,  type geometrique ',I5,', aux points de quadrature #'
     &,5(I3:,', '))
10001 FORMAT(/' *CalelB*  Element #',I5,' Face ',I1,' : Type ',I5,
     &' ,  type geometrique ',I5,', au dernier point de quadrature #'
     &,(I3:,', '))
10002 FORMAT(T13,'Integrand #',I4,' Donnee associee # ',I3,' Adr.rel. :'
     &,I9)
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
99995 CALL ENCLER (NBPTQX,FORMAN(1:3))
      CALL ENCLER (NBPTQD,FORMAN(4:6))
      CALL BAISE  ('Nombre maximum ('//FORMAN(1:3)//
     &') de points de quadrature depasse : NBPTQD='//FORMAN(4:6))
                                                                    END !CalelB
