*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE CALELM (ELEMEN,NDIM  ,NBTRDO,INDTRD,NCHDON,NOMDON
     &                  ,EPSMAC,NUTGAV,CORDLG
     &                  ,NBPTQX,NUSCHQ,NBPTQD,POIDQD,COORQD
     &                  ,NUTYLC,NBDLEC,KELFCC,FCTBC,DFCTBC,DWC
     &                  ,NUTYLL,NBDLEL,KELFCL,FCTBL,DFCTBL,DWL
     &                  ,NUTYLG,NBDLEG,KELFCG,FCTBG,DFCTBG
     &                  ,MCMATR,MCMATC,IST,RST,CST,NODONA,NOMTRA
     &                  ,NIVIMP,IMPFCH)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Novembre 1988)
* Derniere modification : D.Martin (23 juin 2005)
* Version 4
*  
* Calcul des integrales elementaires de volume en dimension 3,
* de surface en dimension 2, de ligne en dimension 1
*
*-- Arguments -- 
* ELEMEN numero de l'element en cours
* NDIM   dimension d'espace
* NBTRDO nombre d'integrales elementaires sur le domaine + autres choses  
* INDTRD tableau de gestion des adresses pour le calcul des termes
*     Contenu du tableau INDTRD
*     (NUINT          ) niveau (plus tard adresse absolue du terme)
*     (NUINT+   NBTRDO) adresses relatives des matrices elementaires
*                       dans le tableau des matrices elementaires du bon type
*     (NUINT+ 2*NBTRDO) adresses relatives des tableaux de valeurs
*                       nodales dans le tableau des matrices elementaires
*     (NUINT+ 3*NBTRDO) type REEL ou COMPLEXE du terme
*     (NUINT+ 4*NBTRDO) indice de type de calcul
*     (NUINT+ 5*NBTRDO) numero d'integrand
*     (NUINT+ 6*NBTRDO) nombre de composantes de l'inconnue en colonne
*     (NUINT+ 9*NBTRDO) nombre de composantes de l'inconnue en ligne
*     (NUINT+12*NBTRDO) numero de donnee associee si FONCTION ou TABLEAU
*     (NUINT+13*NBTRDO) type de la donnee associee (FONCTION,TABLEAU)
*     (NUINT+14*NBTRDO) niveau ou numero de la donnee associee
*     (NUINT+15*NBTRDO) type du declaration de la fonction ou du tableau
*     (NUINT+16*NBTRDO) adresse de la donnee TABLEAU associee
*                       ou adresse de la CONSTANTE ou du TABLEAU associe
*                       dans le cas d'une donnee FONCTION
* NCHDON nombre de caracteres par nom de donnee
* NOMDON tableau #OMDON des noms de donnees
* NUTGAV numero de type de l'element lors de l'appel precedent
* CORDLG coordonnees des points (d.l. geometriques) de l'element courant
* NBPTQX nombre maximum de points de quadrature 
* NUSCHQ numero du schema de quadrature
* NBPTQD nombre de points de quadrature 
* POIDQD tableau des poids de quadrature dans l'element de reference
* COORQD tableau des coordonnees des points de quadrature   "
* NUTYLC numero de type d'interpolation en colonne de l'element
*        (type d'interpolation de la solution)
* NBDLEC nombre de degres de liberte en colonne de l'element
* KECFCC indice de calcul des fonctions de base pour l'interpolation en colonne
*       (1:1)='Y' ==> calcul des fonctions de base
*       (2:2)='Y' ==> calcul des derivees des fonctions de base
*       (3:3)='Y' ==> calcul des derivees secondes des fonctions de base
*       (4:4)='C' ==> calculs selon l'interpolation en 'colonne'
*             'G' ==> calculs selon l'interpolation geometrique
*       (5:5)='Y' ==> on conserve les calculs en tous les points
*                     de quadrature pour l'element suivant
*       (6:6)='Y' ==> inutilise pour les calculs en dimension d'espace
* FCTBC  tableau de calcul des valeurs des fonctions de base dans l'element
*        de reference pour l'interpolation en colonne (en tous les points de
*        quadrature)
* DFCTBC    -          -          derivees des fonctions de base dans l'element
*        de reference pour l'interpolation en colonne (en tous les points de
*        quadrature)
* DWC       -          -          -        derivees des fonctions dans l'element
*        courant pour l'interpolation en colonne (en un point de quadrature)
* NUTYLL numero de type d'interpolation en ligne de l'element
*        (type d'interpolation des fonctions tests)
* NBDLEL nombre de degres de liberte en ligne de l'element
* KELFCL indice de calcul des fonctions de base pour l'interp. en ligne
*        (cf. KECFCC ci-dessus)
* FCTBL  tableau de calcul des valeurs des fonctions de base dans l'element
*        de reference pour l'interpolation en ligne (en tous les points de
*        quadrature)
* DFCTBL    -          -          -        derivees des fonctions dans l'element
*        de reference pour l'interpolation en ligne (en tous les points de
*        quadrature)
* DWL       -          -          -        derivees des fonctions dans l'element
*        courant pour l'interpolation en ligne (en un point de quadrature)
* NUTYLG numero de type geometrique de l'element
* NBDLEG nombre de points de l'element geometrique 
* KELFCG indice de calcul des fonctions de base geometriques
*        (cf. KECFCC ci-dessus)
* FCTBG  tableau de calcul des fonctions de base geometriques 
* DFCTBG    -          -          -        derivees des fonctions geometriques 
* MCMATR adresse du tab. des matirces elementaires reelles 
* MCMATC adresse du tab. des matirces elementaires complexes
* IST,RST,CST super-tableau version entiere,reelle, complexe
* NODONA chaine de caractere receptrice d'un nom de donnee (cf. CalinT)
* NOMTRA chaine de caractere receptrice d'un nom de fonction (cf. CalinT) 
* NIVIMP numero de niveau d'impression
* IMPFCH numero de l'unite logique des impressions (pas d'impression si =0)
*
* Remarques: L'argument NUTGAV permet d'eviter de rappeler la procedure de 
*            calcul de la formule de quadrature et surtout de refaire les
*            calculs des fonctions de base si la place necessaire pour leur
*            conservation est disponible en M.C.
*            (test sur la valeur de KELFCx(5:5))
*            Les 6 tableaux FCTBC,FCTBL,FCTBG,DFCTBC,DFCTBL,DFCTBG peuvent 
*            contenir les valeurs des fonctions de base et derivees pour 1 ou
*            tous les points de quadrature selon la place en M.C.
*            (test sur KELFCx(5:5))
*            De plus 2 ou 3 des tableaux FCTBx ou DFCTBx peuvent coincider
*            (adresses egales) lorsque les interpolations correspondantes
*            sont les memes (test sur KELFCx(4:4))
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) NODONA,NOMTRA,KELFCC,KELFCL,KELFCG
      CHARACTER     NOMDON(*)
      INTEGER       ELEMEN,NDIM,NBTRDO,NCHDON,NUTGAV,NBPTQX,NUSCHQ
     &             ,NBPTQD,MCMATR,MCMATC,INDTRD(*),NIVIMP,IMPFCH
     &             ,NUTYLG,NBDLEG,NUTYLC,NBDLEC,NUTYLL,NBDLEL
     &             ,IST(*)
      REAL          EPSMAC,CORDLG(NDIM,*),POIDQD(*),COORQD(NDIM,*)
     &             ,FCTBG(*),DFCTBG(NDIM,*)
     &             ,FCTBC(*),DFCTBC(NDIM,*),DWC(NDIM,*)
     &             ,FCTBL(*),DFCTBL(NDIM,*),DWL(NDIM,*)
     &             ,RST(*)
      COMPLEX       CST(*)
*
* Variables locales
      CHARACTER LEYTUN*5,FORMAN*120
      INTEGER   IMPINT,IMPFCT,IMP,NUINT,MIXTE,NUPTQD
     &         ,IAFCG,IADFCG,IAFCC,IADFCC,IAFCL,IADFCL
     &         ,DIMPTX,DIMJAX
* POIN   tableau pour les coordonnees d'un point (images par F)
* JACOBF tableau pour le calcul de la matrice jacobienne
* JACOBM tableau pour le calcul de la matrice jacobienne inverse
      PARAMETER ( DIMPTX=3 , DIMJAX=9 )
      REAL      POIN(DIMPTX),JACOBF(DIMJAX),JACOBM(DIMJAX)
     &         ,DETDF,DETDFA,ADETDF,ELTDIF
      LOGICAL   KLAGEN
*
      COMMON/ONSEPA/FORMAN
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*CalelM*')
      IMP=IMPFCH
      IF (NIVIMP.LE.0) IMP=0
*
*     Variable impliquant le recalcul des fonctions de base
*     (premier element ou changement de type de l'element)
      KLAGEN=NUTYLG.NE.NUTGAV
*  
*     Formule de quadrature
*     ---------------------
      IF (KLAGEN) THEN
         CALL ELEMQD (NUTYLG,NDIM,NUSCHQ,NBPTQD,POIDQD,COORQD,IMP)      !Ef3d
         IF (NBPTQD.GT.NBPTQX) GOTO 99995
      ENDIF
*
      DETDFA=0.
*
*     Indices pour impression de resultats partiels
      IMPINT=0
      IMPFCT=0
      IF (NIVIMP.GT.0) THEN
         IF (NIVIMP.GE.5.AND.NDIM.LT.3) THEN
            IMPINT=IMPFCH
            IMPFCT=IMPFCH
            WRITE (IMP,10000) ELEMEN,NUTYLC,NUTYLG,1,NBPTQD
         ELSE
            WRITE (IMP,10001) ELEMEN,NUTYLC,NUTYLG,NBPTQD
         ENDIF
         IF (.NOT.KLAGEN) IMPFCT=0
         DO 10 NUINT=1,NBTRDO
            IF (INDTRD(NUINT+5*NBTRDO).GT.0)
     &         WRITE (IMP,10002) INDTRD(NUINT+5*NBTRDO)
     &        ,ABS(INDTRD(NUINT+12*NBTRDO)),INDTRD(NUINT+NBTRDO)
10       CONTINUE
      ENDIF
*-------------------------------------------------------------------------------
* Plan :
*  I--> Calculs geometriques (matrice jacobienne,...)
*    a) Derivees des fonctions de base definissant la transformation geometrique
*    b) Matrice jacobienne de la transformation geometrique
*    c) Determinant jacobien et matrice inverse
*    e) Le cas echeant, calcul de l'image par la transformation geometrique
*       du point de quadrature dans l'element
*    g) Le cas echeant (interpolation isoparametrique)
*       calcul des derivees de fontions de base dans l'element courant
*       (Rappel : Les calculs geometriques (jacobien et element differentiel)
*        utilisent l'element geometrique (NUTYLG), les calculs d'integrands
*        utilisent les fonctions de base de l'element pour l'interpolation
*        en colonne (NUTYLC) et ligne (NUTYLL)).
*  
* II--> Calcul des fonctions de base et/ou des derivees intervenant dans
*       dans le calcul des integrands dans le cas non isoparametrique ou mixte
*       Dans le cas isoparametrique on utilise les fonctions de base deja
*       calculees pour la geometrie
*    f) calcul des fonctions et derivees de fontions de base dans l'element
*       de reference (cas d'une interpolation non isoparametrique)
*    g) calcul des derivees de fontions de base dans l'element courant
*  
* III--> Calcul des matrices elementaires en un point de quadrature
*------------------------------------------------------------------
*
      DO 111 NUPTQD=1,NBPTQD
* --I) Calculs geometriques
*     a) Derivees des fct. de base definissant la transformation geometrique
         CALL ECOBAS (KLAGEN,NUPTQD,COORQD(1,NUPTQD),NDIM
     &               ,NUTYLG,NBDLEG,NBDLEG,KELFCG,FCTBG,DFCTBG
     &               ,IAFCG,IADFCG,IMPFCT)
*     b) Matrice jacobienne
         CALL DELAFO (NDIM,NDIM,NBDLEG,CORDLG,DFCTBG(1,IADFCG)
     &               ,IMPINT,JACOBF)
*     c) Determinant et Inverse de la matrice jacobienne
*        (OFALED retourne le determinant et l'inverse de la matrice jacobienne )
         CALL OFALED (NDIM,JACOBF,DETDF,JACOBM,EPSMAC,IMPINT)
*        Tests sur la nullite ou le changement de signe du determinant
         ADETDF=ABS(DETDF)
         IF (ADETDF.LE.EPSMAC) GOTO 99990
         IF (DETDF*DETDFA.LT.0) GOTO 99992
         DETDFA=DETDF
         ELTDIF=ADETDF
*     e) Image du point de quadrature dans l'element courant
*        (cas des integrands a coefficient variable)
         IF (KELFCG(6:6).EQ.'Y')
     &      CALL TRANSF (NDIM,NBDLEG,CORDLG,FCTBG(IAFCG),POIN)
*
*        Calculs differencies selon les interpolations en colonne et ligne
*        -----------------------------------------------------------------
         IF (KELFCC(4:4).EQ.'G') THEN
            IF (KELFCL(4:4).EQ.'G') THEN
*
*              Interpolation isoparametrique en colonne et en ligne
*        f)
*        g)    Calcul des derivees de fontions de base dans l'element courant
               IF (KELFCG(2:2).EQ.'Y')
     &            CALL DWIDXK (NDIM,NBDLEG,DFCTBG(1,IADFCG),JACOBM
     &                        ,DWC,IMPFCT)
* III)
               MIXTE=0
               CALL CALINT (MIXTE,NDIM,NBTRDO,INDTRD,NCHDON,NOMDON
     &                     ,POIDQD(NUPTQD),ELTDIF,POIN
     &                     ,NBDLEG,FCTBG(IAFCG),DWC
     &                     ,NBDLEG,FCTBG(IAFCG),DWC,MCMATR,MCMATC
     &                     ,IST,RST,CST,NODONA,NOMTRA,IMPINT)
*
            ELSEIF (KELFCL(4:4).EQ.'L') THEN
*
*              Interpolation isoparametrique en colonne et non en ligne
* II)    f)
               CALL ECOBAS (KLAGEN,NUPTQD,COORQD(1,NUPTQD),NDIM
     &                     ,NUTYLL,NBDLEL,NBDLEL,KELFCL,FCTBL,DFCTBL
     &                     ,IAFCL,IADFCL,IMPFCT)
*        g)
               IF (KELFCG(2:2).EQ.'Y')
     &            CALL DWIDXK (NDIM,NBDLEG,DFCTBG(1,IADFCG),JACOBM
     &                        ,DWC,IMPFCT)
               IF (KELFCL(2:2).EQ.'Y')
     &            CALL DWIDXK (NDIM,NBDLEL,DFCTBL(1,IADFCL),JACOBM
     &                        ,DWL,IMPFCT)
* III)
               MIXTE=NUTYLC-NUTYLL
               CALL CALINT (MIXTE,NDIM,NBTRDO,INDTRD,NCHDON,NOMDON
     &                     ,POIDQD(NUPTQD),ELTDIF,POIN
     &                     ,NBDLEG,FCTBG(IAFCG),DWC
     &                     ,NBDLEL,FCTBL(IAFCL),DWL,MCMATR,MCMATC
     &                     ,IST,RST,CST,NODONA,NOMTRA,IMPINT)
*
            ENDIF
*
         ELSEIF (KELFCC(4:4).EQ.'C') THEN
*
            IF (KELFCL(4:4).EQ.'C') THEN
*
*              Interpolation non-isoparametrique identique en colonne et ligne
* II)    f)
               CALL ECOBAS (KLAGEN,NUPTQD,COORQD(1,NUPTQD),NDIM
     &                     ,NUTYLC,NBDLEC,NBDLEC,KELFCC,FCTBC,DFCTBC
     &                     ,IAFCC,IADFCC,IMPFCT)
*        g)
               IF (KELFCC(2:2).EQ.'Y')
     &            CALL DWIDXK (NDIM,NBDLEC,DFCTBC(1,IADFCC),JACOBM
     &                        ,DWC,IMPFCT)
* III)
               MIXTE=0
               CALL CALINT (MIXTE,NDIM,NBTRDO,INDTRD,NCHDON,NOMDON
     &                     ,POIDQD(NUPTQD),ELTDIF,POIN
     &                     ,NBDLEC,FCTBC(IAFCC),DWC
     &                     ,NBDLEC,FCTBC(IAFCC),DWC,MCMATR,MCMATC
     &                     ,IST,RST,CST,NODONA,NOMTRA,IMPINT)
*
            ELSEIF (KELFCL(4:4).EQ.'G') THEN
*
*              Interpolation non-iso. en colonne/ isoparametrique en ligne
* II)    f)
               CALL ECOBAS (KLAGEN,NUPTQD,COORQD(1,NUPTQD),NDIM
     &                     ,NUTYLC,NBDLEC,NBDLEC,KELFCC,FCTBC,DFCTBC
     &                     ,IAFCC,IADFCC,IMPFCT)
*        g)
               IF (KELFCC(2:2).EQ.'Y')
     &            CALL DWIDXK (NDIM,NBDLEC,DFCTBC(1,IADFCC),JACOBM
     &                        ,DWC,IMPFCT)
               IF (KELFCG(2:2).EQ.'Y')
     &            CALL DWIDXK (NDIM,NBDLEG,DFCTBG(1,IADFCG),JACOBM
     &                        ,DWL,IMPFCT)
* III)
               MIXTE=NUTYLC-NUTYLL
               CALL CALINT (MIXTE,NDIM,NBTRDO,INDTRD,NCHDON,NOMDON
     &                     ,POIDQD(NUPTQD),ELTDIF,POIN
     &                     ,NBDLEC,FCTBC(IAFCC),DWC
     &                     ,NBDLEG,FCTBG(IAFCG),DWL,MCMATR,MCMATC
     &                     ,IST,RST,CST,NODONA,NOMTRA,IMPINT)
*
            ELSEIF (KELFCL(4:4).EQ.'L') THEN
*
*              Interpolation non-isoparam. differentes en colonne et ligne
* II)    f)
               CALL ECOBAS (KLAGEN,NUPTQD,COORQD(1,NUPTQD),NDIM
     &                     ,NUTYLC,NBDLEC,NBDLEC,KELFCC,FCTBC,DFCTBC
     &                     ,IAFCC,IADFCC,IMPFCT)
               CALL ECOBAS (KLAGEN,NUPTQD,COORQD(1,NUPTQD),NDIM
     &                     ,NUTYLL,NBDLEL,NBDLEL,KELFCL,FCTBL,DFCTBL
     &                     ,IAFCL,IADFCL,IMPFCT)
*        g)
               IF (KELFCC(2:2).EQ.'Y')
     &            CALL DWIDXK (NDIM,NBDLEC,DFCTBC(1,IADFCC),JACOBM
     &                        ,DWC,IMPFCT)
               IF (KELFCL(2:2).EQ.'Y')
     &            CALL DWIDXK (NDIM,NBDLEL,DFCTBL(1,IADFCL),JACOBM
     &                        ,DWL,IMPFCT)
* III)
               MIXTE=NUTYLC-NUTYLL
               CALL CALINT (MIXTE,NDIM,NBTRDO,INDTRD,NCHDON,NOMDON
     &                     ,POIDQD(NUPTQD),ELTDIF,POIN
     &                     ,NBDLEC,FCTBC(IAFCC),DWC
     &                     ,NBDLEL,FCTBL(IAFCL),DWL,MCMATR,MCMATC
     &                     ,IST,RST,CST,NODONA,NOMTRA,IMPINT)
*
            ENDIF
         ENDIF
*
*        Pour les impressions au dernier point de quadrature
*
         IMPINT = 0
         IMPFCT = 0
         IF (NIVIMP.GT.0.AND.NUPTQD.EQ.NBPTQD-1) THEN
            IMPINT = IMPFCH
            IMPFCT = IMPFCH
         ENDIF
111   CONTINUE
      CALL PRFXMJ (-1,'*CalelM*')
      RETURN
*-----------------------------------------------------------------------
10000 FORMAT(/' *CalelM* Element #',I5,' : Type ',I5,
     &', type geometrique ',I5,', aux points de quadrature #'
     &,5(I3:,', '))
10001 FORMAT(/' *CalelM* Element #',I5,' : Type ',I5,
     &', type geometrique ',I5,', au dernier point de quadrature #',I3)
10002 FORMAT(T13,'Integrand #',I4,' Donnee associee # ',I3,' Adr.rel. :'
     &,I9)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*    Impression en cas de nullite ou changement de signe du determinant
*
99990 WRITE (IMPFCH,99991) DETDF,ELEMEN,LEYTUN (NUTYLG),NUPTQD
      NUPTQD=1+(-1)**NDIM
      FORMAN='('//CHAR(NDIM+48+NUPTQD)//'(I5,TR3,'//CHAR(NDIM+48)
     &//'E9.3,'' *''))'
      WRITE (IMPFCH,FMT=FORMAN)
     &  (NUPTQD,(CORDLG(IAFCG,NUPTQD),IAFCG=1,NDIM),NUPTQD=1,NBDLEG)
      CALL BAISE  (' Determinant nul, probleme de maillage?')
99991 FORMAT(/' *CalelM*   ---> STOP'/'  Determinant nul'
     &/T20,' DETDF =',E12.4/'  dans l''element no',I5
     &,' de type geometrique ',A,' au point de quadrature no',I4
     &//'  Coordonnees des points :')
99992 WRITE (IMPFCH,99993) DETDF,DETDFA,ELEMEN,LEYTUN (NUTYLG),NUPTQD
      NUPTQD=1+(-1)**NDIM
      FORMAN='('//CHAR(NDIM+48+NUPTQD)//'(I5,TR3,'//CHAR(NDIM+48)
     &//'E9.3,'' *''))'
      WRITE (IMPFCH,FMT=FORMAN)
     &  (NUPTQD,(CORDLG(IAFCG,NUPTQD),IAFCG=1,NDIM),NUPTQD=1,NBDLEG)
      CALL BAISE  ('Changement de signe du determinant,'
     &// 'Probleme de maillage ?')
99993 FORMAT(/' *CalelM*   ---> STOP'/
     &'  Changement de signe du determinant jacobien'
     &/T20,' DETDF =',E12.4,' DETDFav =',E12.4/'  dans l''element no'
     &,I5,' de type geometrique ',A,' au point de quadrature no',I4
     &//'  Coordonnees des points :')
99995 CALL ENCLER (NBPTQX,FORMAN(1:3))
      CALL ENCLER (NBPTQD,FORMAN(4:6))
      CALL BAISE  ('Nombre maximum ('//FORMAN(1:3)//
     &') de points de quadrature depasse : NBPTQD='//FORMAN(4:6))
                                                                    END !CalelM
