*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE CALIND (MIXTE,NDIM,NBTRDO,INDTRD,NCHDON,NOMDON
     &                  ,POIDQ,ELTDIF,POIN
     &                  ,NBFC,NBEC,WC,DWC
     &                  ,NBFL,NBEL,WL,DWL,MCMATR,MCMATC
     &                  ,IST,RST,CST,NODONA,NOMFCT,IMPFCH)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Octobre 1988)
* Derniere modification : G.Legendre (4 juillet 2011)
* Version 5
*  
* Calcul des integrales elementaires en un point de quadrature sur un domaine
* necessitant une extension (integrales avec derivees non tangentielles)
*  
*-- Arguments (voir aussi Caleld) --
* MIXTE  indice de mixite des interpolations en ligne/colonne
* NDIM   dimension d'espace
* NBTRDO nombre de termes a calculer sur le domaine
* INDTRD structure locale de description des termes du domaine
*   Contenu du tableau INDTRD pour le terme numero NUTRDO
*   (NUTRDO)           niveau, puis adresse absolue du terme dans RST,CST
*   (NUTRDO+   NBTRDO) adresses relatives des matrices elementaires  
*                      dans le tableau des matrices elementaires du bon type
*   (NUTRDO+ 2*NBTRDO) adresses relatives des tableaux locaux de valeurs  
*                      nodales dans le tableau des matrices elementaires
*   (NUTRDO+ 3*NBTRDO) indice de type de declaration REEL ou COMPLEXE du terme
*   (NUTRDO+ 4*NBTRDO) indice de type de calcul (ITYCAL)
*   (NUTRDO+ 5*NBTRDO) numero d'integrand (NUKALE)
*   (NUTRDO+ 6*NBTRDO) nombre de composantes de l'inconnue en colonne
*   (NUTRDO+ 7*NBTRDO) type d'interplation de l'inconnue en colonne
*   (NUTRDO+ 8*NBTRDO) nombre de noeuds de l'inconnue en colonne
*                      ou nombre de fonctions propres (inconnue spectrale)
*   (NUTRDO+ 9*NBTRDO) nombre de composantes de l'inconnue en ligne
*   (NUTRDO+10*NBTRDO) type d'interplation de l'inconnue en ligne
*   (NUTRDO+11*NBTRDO) nombre de noeuds de l'inconnue en ligne
*                      ou nombre de fonctions propres (inconnue spectrale)
*   (NUTRDO+12*NBTRDO) numero de donnee affectee si cette donnee est de type
*                      FONCTION ou TABLEAU
*   (NUTRDO+13*NBTRDO) type de la donnee associee (CONSTANTE,FONCTION,TABLEAU) 
*   (NUTRDO+14*NBTRDO) niveau ou numero de la donnee affectee au terme (NIVDON)  
*   (NUTRDO+15*NBTRDO) type du resultat de la donnnee associee (REEL,COMPLEXE) 
*   (NUTRDO+20*NBTRDO) niveau d'impression du terme
*   de plus si la donnee affectee au terme est un tableau ou une fonction
*   (NUTRDO+16*NBTRDO) adresse du tableau associe au terme ou
*                      de la constante ou du tableau associe a la fonction
*   (NUTRDO+17*NBTRDO) type (CONSTANTE ou TABLEAU) associe a la fonction
*   (NUTRDO+18*NBTRDO) type du resultat (REEL, COMPLEXE) associe a la fonction
*   (NUTRDO+19*NBTRDO) nombre de composantes par noeud pour une donnee
*                      associee de type TABLEAU
* NCHDON nombre de caracteres par nom de donnee
* NOMDON tableau #OMDON des noms de donnees
* POIDQ  poids de quadrature au point de quadrature en cours
* ELTDIF element d'integration (element de surface ou de longueur)
* POIN   tableau de travail contenant les coordonnees de l'image du point
*        de quadrature dans l'element courant,
*        suivi de la norme et des composantes du vecteur normal unitaire
* NBFC   nombre de d.l. (de la face ou arete) de l'element en colonne
* NBEC   nombre de d.l. de l'element pour l'interpolation en colonne
* WC     valeurs des fonctions de base pour l'interpolation en colonne
* DWC    derivees des fonctions de base pour l'interpolation en colonne
* NBFL   nombre de d.l. (de la face ou arete) de l'element en ligne
* NBEL   nombre de d.l. de l'element pour l'interpolation en ligne
* WL     valeurs des fonctions de base pour l'interpolation en ligne
* DWL    derivees des fonctions de base pour l'interpolation en ligne
* MCMATR adresse du tableau des matrices elementaires reelles
* MCMATC adresse du tableau des matirces elementaires complexes
* IST    super-tableau entier 
* RST    super-tableau reel
* CST    super-tableau complexe
* NODONA chaine de caractere receptrice d'un nom de donnee
* NOMFCT chaine de caractere receptrice d'un nom de fonction
* IMPFCH unite logique d'impression si non nul (pas d'impression si nul)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) NODONA,NOMFCT
      CHARACTER     NOMDON(*)
      INTEGER       MIXTE,NDIM,NBTRDO,NCHDON,NBFC
     &             ,NBEC,NBFL,NBEL,MCMATR,MCMATC,IMPFCH
      INTEGER       INDTRD(*),IST(*)
      REAL          POIDQ,POIN(*),ELTDIF
     &             ,WL(*),WC(*),DWC(NDIM,*),DWL(NDIM,*)
     &             ,RST(*)
      COMPLEX       CST(*)
*
*     Variables locales
      CHARACTER     DONINT*9,TYPINT*12,TYPE*12
      INTEGER       NUTRDO,ITYPTR,KINTEG,ITYCAL
     &             ,NCPINC,NCPINL
     &             ,NIVIMP,IMPINT,NUDONN,MCADRR,MCADRC,ADRTRM,ALPHA,BETA
*     Pour dimensionner le tableau en cas de coefficient variable.
      INTEGER NDIMX,LGCOFX
      PARAMETER ( NDIMX = 3
     &          , LGCOFX = NDIMX*NDIMX )
      REAL    COFVAR(LGCOFX)
      COMPLEX COFVAC(LGCOFX)
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*CalInd*')
*  
*     Boucle sur les Integrands elementaires
*
      DO 3000 NUTRDO=1,NBTRDO
*        Type de calcul ('ELFINI') du terme courant
         ITYCAL=INDTRD(NUTRDO+ 4*NBTRDO)
*        Numero d'integrand
         KINTEG=INDTRD(NUTRDO+ 5*NBTRDO)
         IF (KINTEG.LE.0) GOTO 3000
*        Adresse relative pour l'integrand courant dans le tableau des
*        matrices elementaires
         ADRTRM=INDTRD(NUTRDO+   NBTRDO)
*        Indicateur de type de declaration de l'integrale elementaire
         ITYPTR=INDTRD(NUTRDO+ 3*NBTRDO)
*        Nombre de composantes pour l'inconnue en colonne
         NCPINC=INDTRD(NUTRDO+ 6*NBTRDO)
*        Nombre de composantes pour l'inconnue en ligne
         NCPINL=INDTRD(NUTRDO+ 9*NBTRDO)
*        Numero de donnee affectee a l'integrale elementaire
         NUDONN=INDTRD(NUTRDO+12*NBTRDO)
*        Niveau d'impression pour l'integrale elementaire
         NIVIMP=INDTRD(NUTRDO+20*NBTRDO)
*
*        Type REEL ou COMPLEXE de l'integrale elementaire
         TYPE  =TYPINT (ITYPTR)                                         !utilite
         IMPINT=0  
         IF (NIVIMP.GT.0) IMPINT=IMPFCH
*        Adresse absolue pour l'integrale elementaire
         MCADRR=MCMATR-1+ADRTRM
         MCADRC=MCMATC-1+ADRTRM
*  
*        Integrale contenant un coefficient variable ?
*-----------------------------------------------------
         COFVAR(1)=1.
         COFVAC(1)=1.
*
         IF (NUDONN.GT.0)
     &      CALL CALCOV (NDIM,NBTRDO,INDTRD,POIN,NBFC,WC,MCMATR,MCMATC
     &                  ,NUTRDO,TYPE,NUDONN,NCHDON,NOMDON,NODONA             
     &                  ,NOMFCT,IST,RST,CST,COFVAR,COFVAC)
*
         IF (NCPINC.LE.1.AND.NCPINL.LE.1) THEN
*
*        Integrands a inconnues scalaires
*-----------------------------------------------------------------------
         GOTO (1999,1002,1002,1002,1999,1999,1007,1007,1007,1010
     &        ,1010,1010) KINTEG
         GOTO (1013,1013,1013,1016,1016,1016,1019,1019,1019,1022)
     &        KINTEG-12
         GOTO (1999,1999,1303,1304,1305,1306,1307) KINTEG-300
         GOTO 1999 
*
*--      dV/dx_{alpha}     (2,3,4)
 1002    ALPHA=KINTEG-1  
         CALL DW     (ALPHA,NBEL,DWL,NDIM
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      U.dV/dx_{alpha}    (7,8,9)
 1007    ALPHA=KINTEG-6
         CALL WDW    (ALPHA,NBEL,DWL,NBFC,WC,NDIM
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      dU/dx_{alpha}.V    (10,11,12)
 1010    ALPHA=KINTEG-9
         CALL DWW    (ALPHA,NBFL,WL,NBEC,DWC,NDIM
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
* 
*--      dU/dx_{alpha}.dV/dx_{beta} (13,14,15,16,17,18,19,20,21)
 1013    ALPHA=1
         BETA=KINTEG-12
         GOTO 1020
 1016    ALPHA=2
         BETA=KINTEG-15
         GOTO 1020
 1019    ALPHA=3
         BETA=KINTEG-18
*
 1020    CALL DWDW   (MIXTE,ALPHA,BETA,NBEL,DWL,NBEC,DWC,NDIM
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      gradU.gradV (produit scalaire)
 1022    CALL GRAGRA (MIXTE,NBEL,DWL,NBEC,DWC,NDIM
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      dV/dn
 1303    CALL DNW    (NBEL,DWL,NDIM,POIN(NDIM+2)
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      U.dV/dn
 1304    CALL WDNW   (NBEL,DWL,NBFC,WC,NDIM,POIN(NDIM+2)
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      dU/dn.V
 1305    CALL DNWW   (NBFL,WL,NBEC,DWC,NDIM,POIN(NDIM+2)
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      dU/dn.dV/dn
 1306    CALL DNWDNW (NBEL,DWL,NBEC,DWC,NDIM,POIN(NDIM+2)
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      dU/dt.dV/dt
 1307    CALL DTWDTZ (NBEL,DWL,NBEC,DWC,NDIM,POIN(NDIM+2)
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
      ELSE
*
*        Integrands a inconnues vectorielles
*        -----------------------------------
*
         GOTO (2999,2999,2999,2999,2999,2999,2007,2007,2007) KINTEG
         GOTO (2022,2023,2024) KINTEG-21
         IF (KINTEG.EQ.41) GOTO 2041
         GOTO (2999,2999,2999,2999,2305,2999,2999,2309,2310) KINTEG-300
         GOTO 2999
*
*--      U.dV/dx_{alpha} (produit scalaire, inconnues vectorielles)
 2007    ALPHA=KINTEG-6
         CALL WDW    (ALPHA,NBEL,DWL,NBFC,WC,NDIM
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      gradU.gradV (inconnues vectorielles)
 2022    CALL GRVGRV (MIXTE,NBEL,DWL,NBEC,DWC,NDIM
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      rotU.rotV
 2023    CALL ROTROT (MIXTE,NBEL,DWL,NBEC,DWC,NDIM
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      divU.divV
 2024    CALL DIVDIV (MIXTE,NBEL,DWL,NBEC,DWC,NDIM
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      somme dV/dxl, l=1,ndim (i.e. divV, inc. vectorielle)
 2041    CALL TENDW  (NBEL,DWL,NDIM
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      U.dV/dn = somme UidVi/dn i=1,ndim
 2304    CALL WNGRAW (NBEL,DWL,NBFC,WC,NDIM,POIN(NDIM+2)
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      dU/dn.V = somme dUi/dn.Vi i=1,ndim
 2305    CALL NGRAWW (NBFL,WL,NBEC,DWC,NDIM,POIN(NDIM+2)
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      n^RotU.V
 2309    CALL NROTWW (NBFL,WL,NBEC,DWC,NDIM,POIN(NDIM+2)
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      nDivU.V
 2310    CALL NDIVWW (NBFL,WL,NBEC,DWC,NDIM,POIN(NDIM+2)
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
      ENDIF
 3000 CONTINUE
      CALL PRFXMJ (-1,'*CalInd*')
      RETURN
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 1999 CALL ERINTG (15,KINTEG)
 2999 CALL ERINTG (16,KINTEG)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 5 : G.Legendre (4 juillet 2011)
* - decalage oublie des integrands 309, 310 apres introduction du #308
*   (cf. intgroups.h du 6 juin)
* Version 4 : D.Martin (23 juin 2005)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !CalInd
