*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE CALINT (MIXTE,NDIM,NBTRDO,INDTRD,NCHDON,NOMDON
     &                  ,POIDQ,ELTDIF,POIN
     &                  ,NBC,WC,DWC,NBL,WL,DWL,MCMATR,MCMATC
     &                  ,IST,RST,CST,NODONA,NOMFCT,IMPFCH)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Octobre 1988)
* Derniere modification : C.Chambeyron & D.Martin (03 janvier 2011)
* Version 11
*  
* Calcul des integrales elementaires sur un domaine de dimension egale
* a la dimension d'espace, en un point de quadrature
*  
*-- Arguments (voir aussi Calelm) --
* MIXTE  indice de mixite des interpolations en ligne/colonne
* NDIM   dimension d'espace
* NBTRDO nombre de termes a calculer sur le domaine
* INDTRD structure locale de description des termes du domaine
*   Contenu du tableau INDTRD pour le terme numero NUTRDO
*   (NUTRDO)           niveau, puis adresse absolue du terme dans RST,CST
*   (NUTRDO+   NBTRDO) adresses relatives des matrices elementaires  
*                      dans le tableau des matrices elementaires du bon type
*   (NUTRDO+ 2*NBTRDO) adresses relatives des tableaux locaux de valeurs  
*                      nodales dans le tableau des matrices elementaires
*   (NUTRDO+ 3*NBTRDO) indice de type de declaration REEL ou COMPLEXE du terme
*   (NUTRDO+ 4*NBTRDO) indice de type de calcul (ITYCAL)
*   (NUTRDO+ 5*NBTRDO) numero d'integrand (NUKALE)
*   (NUTRDO+ 6*NBTRDO) nombre de composantes de l'inconnue en colonne
*   (NUTRDO+ 7*NBTRDO) type d'interplation de l'inconnue en colonne
*   (NUTRDO+ 8*NBTRDO) nombre de noeuds de l'inconnue en colonne
*                      ou nombre de fonctions propres (inconnue spectrale)
*   (NUTRDO+ 9*NBTRDO) nombre de composantes de l'inconnue en ligne
*   (NUTRDO+10*NBTRDO) type d'interplation de l'inconnue en ligne
*   (NUTRDO+11*NBTRDO) nombre de noeuds de l'inconnue en ligne
*                      ou nombre de fonctions propres (inconnue spectrale)
*   (NUTRDO+12*NBTRDO) numero de donnee affectee si cette donnee est de type
*                      FONCTION ou TABLEAU
*   (NUTRDO+13*NBTRDO) type de la donnee associee (CONSTANTE,FONCTION,TABLEAU) 
*   (NUTRDO+14*NBTRDO) niveau ou numero de la donnee affectee au terme (NIVDON)  
*   (NUTRDO+15*NBTRDO) type du resultat de la donnnee associee (REEL,COMPLEXE) 
*   (NUTRDO+20*NBTRDO) niveau d'impression du terme
*   de plus si la donnee affectee au terme est un tableau ou une fonction
*   (NUTRDO+16*NBTRDO) adresse du tableau associe au terme ou
*                      de la constante ou du tableau associe a la fonction
*   (NUTRDO+17*NBTRDO) type (CONSTANTE ou TABLEAU) associe a la fonction
*   (NUTRDO+18*NBTRDO) type du resultat (REEL, COMPLEXE) associe a la fonction
*   (NUTRDO+19*NBTRDO) nombre de composantes par noeud pour une donnee
*                      associee de type TABLEAU
* NCHDON nombre de caracteres par nom de donnee
* NOMDON tableau #OMDON des noms de donnees
* POIDQ  poids de quadrature au point de quadrature en cours
* ELTDIF element d'integration (| determinant jacobien | en dimension d'espace)
* POIN   tableau de travail contenant les coordonnees de l'image du point
*        de quadrature dans l'element courant ...
* NBC    nombre de d.l. scalaires de l'element pour l'interpolation en colonne
* WC     valeurs des fonctions de base pour l'interpolation en colonne
* DWC    derivees des fonctions de base pour l'interpolation en colonne
* NBL    nombre de d.l. scalaires de l'element pour l'interpolation en ligne
* WL     valeurs des fonctions de base pour l'interpolation en ligne
* DWL    derivees des fonctions de base pour l'interpolation en ligne
* MCMATR adresse du tableau des matrices elementaires reelles
* MCMATC adresse du tableau des matrices elementaires complexes
* IST    super-tableau entier 
* RST    super-tableau reel
* CST    super-tableau complexe
* NODONA chaine de caractere receptrice d'un nom de donnee
* NOMFCT chaine de caractere receptrice d'un nom de fonction
* IMPFCH unite logique d'impression si non nul (pas d'impression si nul)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) NODONA,NOMFCT  
      CHARACTER     NOMDON(*)
      INTEGER       MIXTE,NDIM,NBTRDO,NCHDON,NBC,NBL,MCMATR,MCMATC
      INTEGER       IMPFCH,INDTRD(*),IST(*)
      REAL          POIDQ,POIN(*),ELTDIF
     &             ,WL(*),WC(*),DWC(NDIM,*),DWL(NDIM,*)
     &             ,RST(*)
      COMPLEX       CST(*)
*
*     Variables locales
      CHARACTER     DONINT*9,TYPINT*12,TYPE*12
      INTEGER       NUTRDO,ITYPTR,KINTEG,ITYCAL,ALPHA,BETA,GAMMA,DELTA
     &             ,NCPINC,NCPINL
     &             ,NIVIMP,IMPINT,NUDONN,MCADRR,MCADRC,ADRTRM
*     Pour dimensionner le tableau en cas de coefficient variable.
      INTEGER NDIMX,LGCOFX
      PARAMETER ( NDIMX = 3
     &          , LGCOFX = NDIMX*NDIMX )
      INTEGER IDXPML(NDIMX)
      REAL    COFVAR(LGCOFX)
      COMPLEX COFVAC(LGCOFX)
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*CalInt*')
*  
*     Boucle sur les integrands elementaires
*
      DO 3000 NUTRDO=1,NBTRDO
*        Type de calcul ('ELFINI') du terme courant
         ITYCAL=INDTRD(NUTRDO+ 4*NBTRDO)
*        Numero d'integrand
         KINTEG=INDTRD(NUTRDO+ 5*NBTRDO)
         IF (KINTEG.LE.0) GOTO 3000
*        Adresse relative pour l'integrand courant dans le tableau des
*        matrices elementaires
         ADRTRM=INDTRD(NUTRDO+   NBTRDO)
*        Indicateur de type de declaration de l'integrand courant
         ITYPTR =INDTRD(NUTRDO+ 3*NBTRDO)
*        Nombre de composantes pour l'inconnue en colonne
         NCPINC=INDTRD(NUTRDO+ 6*NBTRDO)
*        Nombre de composantes pour l'inconnue en ligne
         NCPINL=INDTRD(NUTRDO+ 9*NBTRDO)
*        Numero de donnee affectee a l'integrand courant
         NUDONN=INDTRD(NUTRDO+12*NBTRDO)
*        Niveau d'impression pour l'integrand courant
         NIVIMP=INDTRD(NUTRDO+20*NBTRDO)
*
*        Type REEL ou COMPLEXE de l'integrand courant
         TYPE  =TYPINT (ITYPTR)                                         !utilite
         IMPINT=0
         IF (NIVIMP.GT.0) IMPINT=IMPFCH
*        Adresse absolue pour l'integrand courant
         MCADRR=MCMATR-1+ADRTRM
         MCADRC=MCMATC-1+ADRTRM
*  
*        Integrand contenant un coefficient variable ?
*-----------------------------------------------------
         COFVAR(1)=1.
         COFVAC(1)=1.
*
         IF (NUDONN.GT.0)
     &      CALL CALCOV (NDIM,NBTRDO,INDTRD,POIN,NBC,WC,MCMATR,MCMATC
     &                  ,NUTRDO,TYPE,NUDONN,NCHDON,NOMDON,NODONA             
     &                  ,NOMFCT,IST,RST,CST,COFVAR,COFVAC)
*
         IF (NCPINC.LE.1.AND.NCPINL.LE.1) THEN
*
*        Integrands a inconnues scalaires
*-----------------------------------------------------------------------
*
         GOTO (1001,1002,1002,1002,1005,1999,1007,1007,1007,1010
     &        ,1010,1010) KINTEG
         IF (KINTEG.LE.21) GOTO 1013
         GOTO (1022,1999,1999,1999,1026) KINTEG-21
         GOTO 1999
*
*--      Integrand V       (1)
 1001    CALL WISCA  (NBL,WL
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      dV/dx_{delta}     (2,3,4)
 1002    DELTA=KINTEG-1  
         CALL DW     (DELTA,NBL,DWL,NDIM
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      Integrand U.V (inconnues scalaires) (5)
 1005    CALL WWISCA (MIXTE,NBL,WL,NBC,WC
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      U.dV/dx_{delta}    (7,8,9)
 1007    DELTA=KINTEG-6
         CALL WDW    (DELTA,NBL,DWL,NBC,WC,NDIM
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      dU/dx_{gamma}.V    (10,11,12)
 1010    GAMMA=KINTEG-9
         CALL DWW    (GAMMA,NBL,WL,NBC,DWC,NDIM
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      dU/dx_{gamma}.dV/dx_{delta} (13,14,15,16,17,18,19,20,21)
 1013    CALL INTNOS (KINTEG,ALPHA,BETA,GAMMA,DELTA)
         CALL DWDW   (MIXTE,GAMMA,DELTA,NBL,DWL,NBC,DWC,NDIM
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      Integrand gradU.gradV (produit scalaire) (22)
 1022    CALL GRAGRA (MIXTE,NBL,DWL,NBC,DWC,NDIM
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      Integrand \sum_{ij} a_{ij}d_i u d_j v (26)
*
 1026    GOTO 1999
*
*-----------------------------------------------------------------------
      ELSE
*
*        Integrands a (au moins une) inconnue(s) vectorielle(s)
*-----------------------------------------------------------------------
*
         GOTO (2001,2999,2999,2999,2005,2006,2007,2007,2007,2010,2010
     &        ,2010,2013,2013,2013,2013,2013,2013,2013,2013,2013) KINTEG
         GOTO (2022,2023,2024,2025,2999,2027,2028,2029,2030,2031,2032 
     &        ,2033,2034,2035,2036,2037,2038,2039,2040,2041) KINTEG-21
         IF (KINTEG.LE.50) THEN
            GOTO 2042
         ELSEIF (KINTEG.LE.131) THEN
            GOTO 2051
         ELSEIF (KINTEG.LE.132) THEN
            GOTO 2132
         ELSEIF (KINTEG.LE.133) THEN
            GOTO 2133
         ELSEIF (KINTEG.LE.187) THEN
            GOTO 2134
         ELSEIF (KINTEG.LE.222) THEN
*           Integrands PML
            GOTO 2188
         ELSEIF (KINTEG.LE.228) THEN
            GOTO 2223
         ENDIF
         GOTO 2999
*
*        F.V
 2001    CALL WIVEC  (NBL,WL,NCPINL
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      U.V (produit scalaire, inconnues vectorielles)
 2005    CALL WWIVEC (MIXTE,NBL,WL,NCPINL,NBC,WC,NCPINC
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      U.AV (produit scalaire, inconnues vectorielles)
 2006    CALL WAWVEC (MIXTE,NBL,WL,NCPINL,NBC,WC,NCPINC
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      U.dV/dx_{delta} (produit scalaire, inconnues vectorielles)
 2007    DELTA=KINTEG-6
         CALL WDWVEC (MIXTE,DELTA,NBL,DWL,NCPINL,NBC,WC,NCPINC
     &               ,NDIM,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      dU/dx_{gamma}.V (produit scalaire, inconnues vectorielles)
 2010    GAMMA=KINTEG-9
         CALL DWWVEC (MIXTE,GAMMA,NBL,WL,NCPINL,NBC,DWC,NCPINC
     &               ,NDIM,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000

*--      dU/dx_{gamma}.dV/dx_{delta} 
*                    (produit scalaire, U V inconnues vectorielles)
 2013    CALL INTNOS (KINTEG,ALPHA,BETA,GAMMA,DELTA)
         CALL DWDWVE (MIXTE,GAMMA,DELTA,NBL,DWL,NCPINL,NBC,DWC,NCPINC
     &               ,NDIM,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      gradU.gradV (inconnues vectorielles, produit scalaire)
 2022    CALL GRVGRV (MIXTE,NBL,DWL,NBC,DWC
     &               ,NDIM,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      rotU.rotV   (inconnues vectorielles, produit scalaire en 3d)
 2023    CALL ROTROT (MIXTE,NBL,DWL,NBC,DWC
     &               ,NDIM,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      divU divV   (inconnues vectorielles)
 2024    CALL DIVDIV (MIXTE,NBL,DWL,NBC,DWC
     &               ,NDIM,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      Epsilon_ij(U) Epsilon_ij(V)   (inconnues vectorielles)
 2025    CALL EIJEIJ (MIXTE,NBL,DWL,NBC,DWC
     &               ,NDIM,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      gradU.V     (U scalaire, V vectorielle)
 2027    CALL GRAWW  (NBL,WL,NBC,DWC
     &               ,NDIM,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      divU q      (U vectorielle, q scalaire)
 2028    CALL DIVUQ  (NBL,WL,NBC,DWC
     &               ,NDIM,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
 2029    IF (NDIM.EQ.2) THEN
*--         rotU q      (U vectorielle, q scalaire, dimension 2)
            CALL ROTUQ  (NBL,WL,NBC,DWC
     &                  ,NDIM,POIDQ*ELTDIF,TYPE
     &                  ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         ELSEIF (NDIM.EQ.3) THEN
*           RotU.V      (U et V vectorielles, dimension 3)
            CALL ROTUV  (MIXTE,NBL,WL,NBC,DWC
     &                  ,NDIM,POIDQ*ELTDIF,TYPE
     &                  ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         ENDIF
         GOTO 3000
*
*--      pdivV       (p scalaire, V vectorielle)
 2030    CALL PDIVV  (NBL,DWL,NBC,WC
     &               ,NDIM,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      U.gradV     (U vectorielle, V scalaire)
 2031    CALL WGRAW  (NBL,DWL,NBC,WC
     &               ,NDIM,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
 2032    IF (NDIM.EQ.2) THEN
*--         p rotV      (p scalaire, V vectorielle, dimension 2)
            CALL PROTV  (NBL,DWL,NBC,WC
     &                  ,NDIM,POIDQ*ELTDIF,TYPE
     &                  ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         ELSEIF (NDIM.EQ.3) THEN
*           U.RotV      (U et V vectorielles, dimension 3)
            CALL UROTV  (MIXTE,NBL,DWL,NBC,WC
     &                  ,NDIM,POIDQ*ELTDIF,TYPE
     &                  ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         ENDIF
         GOTO 3000
*
*--      ROTROTBT0 (coeff. de Beta^0 dans rot_Beta(U).rot_Beta(V))
 2033    CALL ROTBT0 (MIXTE,NBL,DWL,NCPINL,NBC,DWC,NCPINC
     &               ,NDIM,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      ROTROTBT1 (coeff. de Beta^1 dans rot_Beta(U).rot_Beta(V))
 2034    CALL ROTBT1 (MIXTE,NBL,WL,NCPINL,NBC,DWC,NCPINC
     &               ,NDIM,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      ROTROTBT2 (coeff. de Beta^2 dans rot_Beta(U).rot_Beta(V))
 2035    CALL ROTBT2 (MIXTE,NBL,WL,NCPINL,NBC,WC,NCPINC
     &               ,NDIM,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      DIVDIVBT0 (coeff. de Beta^0 dans div_Beta(U) div_Beta(V))
 2036    CALL DIVBT0 (MIXTE,NBL,DWL,NCPINL,NBC,DWC,NCPINC
     &               ,NDIM,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      DIVDIVBT1 (coeff. de Beta^1 dans div_Beta(U) div_Beta(V))
 2037    CALL DIVBT1 (MIXTE,NBL,DWL,NCPINL,NBC,WC,NCPINC
     &               ,NDIM,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      DIVDIVBT2  (coeff. de Beta^2 dans div_Beta(U) div_Beta(V))
 2038    CALL DIVBT2 (MIXTE,NBL,WL,NCPINL,NBC,WC,NCPINC
     &               ,NDIM,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      EIJEIJBT0 (coeff. de Beta^0 dans eij_Beta(U) eij_Beta(V))
 2039    CALL EIJBT0 (MIXTE,NBL,DWL,NCPINL,NBC,DWC,NCPINC
     &               ,NDIM,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      U^V (produit vectoriel 2D, inconnues vectorielles)
 2040    CALL WVW    (MIXTE,NBL,WL,NCPINL,NBC,WC,NCPINC
     &               ,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      somme dV/dxl, l=1,ndim (i.e. divV, inc. vectorielle)
 2041    CALL TENDW  (NBL,DWL
     &               ,NDIM,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      UaVb
 2042    CALL INTNOS (KINTEG,ALPHA,BETA,GAMMA,DELTA)
         IF (ALPHA.EQ.BETA) THEN
            CALL WAWA   (MIXTE,ALPHA,NBL,WL,NCPINL,NBC,WC,NCPINC
     &                  ,POIDQ*ELTDIF,TYPE
     &                  ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         ELSE
            CALL WAWB   (MIXTE,BETA,NBL,WL,NCPINL,ALPHA,NBC,WC,NCPINC
     &                  ,POIDQ*ELTDIF,TYPE
     &                  ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         ENDIF
         GOTO 3000
*
*--      Ua,gVb,d
 2051    CALL INTNOS (KINTEG,ALPHA,BETA,GAMMA,DELTA)
         IF (ALPHA.EQ.BETA.AND.GAMMA.EQ.DELTA) THEN
            CALL WAGWAG (GAMMA,ALPHA,NBL,DWL,NCPINL,NBC,DWC,NCPINC
     &                  ,NDIM,POIDQ*ELTDIF,TYPE
     &                  ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         ELSE
            CALL WAGWBD (DELTA,BETA ,NBL,DWL,NCPINL
     &                  ,GAMMA,ALPHA,NBC,DWC,NCPINC
     &                  ,NDIM,POIDQ*ELTDIF,TYPE
     &                  ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         ENDIF
         GOTO 3000
*
*--      U1,1V2,2 + U2,2V1,1
 2132    CALL EIIEJJ (MIXTE,NBL,DWL,NBC,DWC
     &               ,NDIM,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
*--      (U1,2+U2,1)(V1,2+V2,1)
 2133    CALL EIJEJI (MIXTE,NBL,DWL,NBC,DWC
     &               ,NDIM,POIDQ*ELTDIF,TYPE
     &               ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         GOTO 3000
*
 2134    CALL INTNOS (KINTEG,ALPHA,BETA,GAMMA,DELTA)
         IF (DELTA.LE.0) THEN
*--      Ua,gVb
            CALL WAGWB  (BETA,NBL,WL,NCPINL,GAMMA,ALPHA,NBC,DWC,NCPINC
     &                  ,NDIM,POIDQ*ELTDIF,TYPE
     &                  ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         ELSE
*--      UaVb,d
            CALL WAWBD  (DELTA,BETA,NBL,DWL,NCPINL,ALPHA,NBC,WC,NCPINC
     &                  ,NDIM,POIDQ*ELTDIF,TYPE
     &                  ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         ENDIF
         GOTO 3000
*
*        Integrands PML (G. Legendre) (du numero 188 au numero 222)
*        ==============
*        (attention en cas de translation de numero d'integrand dans intgroups.h)
 2188    IDXPML(1)=0
         IDXPML(2)=0
         IDXPML(3)=0
         IF (MOD(KINTEG+1,7).EQ.0) THEN 
            IDXPML(1)=1
         ELSEIF (MOD(KINTEG+1,7).EQ.1) THEN
            IDXPML(2)=1
         ELSEIF (MOD(KINTEG+1,7).EQ.2) THEN
            IDXPML(3)=1
         ELSEIF (MOD(KINTEG+1,7).EQ.3) THEN 
            IDXPML(1)=1
            IDXPML(2)=1
         ELSEIF (MOD(KINTEG+1,7).EQ.4) THEN 
            IDXPML(2)=1
            IDXPML(3)=1
         ELSEIF (MOD(KINTEG+1,7).EQ.5) THEN 
            IDXPML(1)=1
            IDXPML(3)=1
         ELSE
            IDXPML(1)=1
            IDXPML(2)=1
            IDXPML(3)=1
         END IF
*
*--      RotU RotV   (U & V vectorielles, PML)
         IF ( KINTEG.LE.194 ) THEN
            CALL ROTPML (MIXTE,NBL,WL,DWL,NBC,WC,DWC
     &                  ,IDXPML,NDIM,POIDQ*ELTDIF,TYPE
     &                  ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
*
*--      divU divV   (U & V vectorielles, PML)
         ELSEIF ( KINTEG.LE.201 ) THEN
            CALL DIVPML (MIXTE,NBL,WL,DWL,NBC,WL,DWC
     &                  ,IDXPML,NDIM,POIDQ*ELTDIF,TYPE
     &                  ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
*
*--      Epsilon_ij(U) Epsilon_ij(V) (U & V vectorielles, PML)
         ELSEIF ( KINTEG.LE.208 ) THEN
            CALL EIJPML (MIXTE,NBL,DWL,NBC,DWC
     &                  ,IDXPML,NDIM,POIDQ*ELTDIF,TYPE
     &                  ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
*
         ELSEIF ( KINTEG.LE.215 ) THEN
           IF ( NDIM.EQ.2 ) THEN
*--           p rotV  (p scalaire, V vectorielle, PML, dimension 2)
              CALL PROTVP (NBL,WL,DWL,NBC,WC
     &                    ,IDXPML,NDIM,POIDQ*ELTDIF,TYPE
     &                    ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
           ELSEIF ( NDIM.EQ.3 ) THEN
*--           U.RotV  (U et V vectorielles, PML, dimension 3)
              CALL UROTVP (MIXTE,NBL,WL,DWL,NBC,WC
     &                    ,IDXPML,NDIM,POIDQ*ELTDIF,TYPE
     &                    ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
           ENDIF
         ELSE
           IF ( NDIM.EQ.2 ) THEN
*--           rotU q  (U vectorielle, q scalaire, PML, dimension 2)
              CALL ROTUQP (NBL,DWL,NBC,WC,DWC
     &                    ,IDXPML,NDIM,POIDQ*ELTDIF,TYPE
     &                    ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
           ELSEIF ( NDIM.EQ.3 ) THEN
*--           rotU V  (U et V vectorielles, PML, dimension 3)
              CALL ROTUVP (MIXTE,NBL,WL,NBC,WC,DWC
     &                    ,IDXPML,NDIM,POIDQ*ELTDIF,TYPE
     &                    ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
           ENDIF
         ENDIF
         GOTO 3000
*
*--      Ua.v ou u.Va (produit d'une composante d'une inconnue vectorielle
*        par un inconnue scalaire)
 2223    CALL INTNOS (KINTEG,ALPHA,BETA,GAMMA,DELTA)
         IF (ALPHA.GT.0) THEN
            CALL WAW(MIXTE,ALPHA,NBL,WL,NCPINL,NBC,WC,NCPINC
     &              ,POIDQ*ELTDIF,TYPE
     &              ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         ELSE
            CALL WWA(MIXTE,BETA,NBL,WL,NCPINL,NBC,WC,NCPINC
     &              ,POIDQ*ELTDIF,TYPE
     &              ,COFVAR,RST(MCADRR),COFVAC,CST(MCADRC),IMPINT)
         ENDIF
         GOTO 3000
      ENDIF
*
 3000 CONTINUE
*
      CALL PRFXMJ (-1,'*CalInt*')
      RETURN
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 1999 CALL ERINTG (13,KINTEG)
 2999 CALL ERINTG (14,KINTEG)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 11 : C.Chambeyron & D.Martin (03 janvier 2011)
* - mise en place des integrands PML 3d
* Version 10 : D.Martin (03 decembre 2010)
* - preparation pour l'introduction des integrands PML 3d
* Version 9 : D.Martin (13 mai 2010)
* - bug dans la direction PML (selon le numero d'integrand mod 3).
* Version 8 : D.Martin (09 novembre 2009)
* - ajout integrales a une inconnue vectorielle U_{alpha}V et UV_{alpha}
* Version 7 : D.Martin & L.Legendre (17 septembre 2009)
* - mise en place de l'integrands U.RotV 3d (G. Legendre)
* Version 6 : D.Martin & L.Legendre (30 octobre 2008)
* - mise en place des integrands PML generaux pour l'elasticite (EijPml)
* Version 5 : D.Martin (19 octobre 2006)
* - mise en place des integrands PML generaux (RotPml et DivPml)
* Version 4 : D.Martin (29 juin 2005)
* - mise a jour pour les modifications de ELDIFF et OFALED
* Version 3.2 : D.Martin (24 mars 2005)
* -
* Version 3.1 : D.Martin (05 mai 2004)
* - nouveaux integrands U_{alpha}V_{beta}
* Version 2.0.7 : D.Martin (20 juin 2003)
* - nouveaux integrands U_{alpha,gamma}V_{beta} et U_{alpha}V_{beta,delta}
* Version 2.0.6 : D.Martin (11 novembre 2002)
* - integrands elements finis localises par integration numerique
* Version 2.0.5 : D.Martin (10 juillet 2002)
* - nouveaux integrands ROTUV, ROTUQP
* Version 2.0.4 : D.Martin (14 juin 2002)
* - nouveaux integrands  U_{alpha,gamma}V_{beta,gamma}
* Version 2.0.1 : D.Martin (9 septembre 2001)
* -
* Version 0 : D.Martin (octobre 1988)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !CalInt
