*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE ECOBAS (RECALC,IQUAD,COOR,NDIM,NUTYLE,NBDLF,NBDLDF
     &                  ,KELFCT,FCTB,DFCTB,IAFCT,IADFCT,IMPFCT)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Juin 1989)
* Derniere modification : D.Martin (27 Juin 1997)
* Version 1.0.0
*  
* Calcul des fonctions de base en un point de quadrature
* ou de leur adresse lorsque le calcul a deja ete effectue pour un element
* de meme numero de type et que l'on dispose de suffisament de place
*  
*-- Arguments d'entree --
* RECALC indique un necessaire recalcul des fonctions de base ( insuffisance
*        de place pour leur stockage ou changement de type d'element) 
* IQUAD  numero du point de quadrature ou l'on effectue le calcul
*        (permet le cas echeant de recuperer l'adresse des valeurs idoines)
* COOR   coordonnees du point de calcul des fonctions de base
* NDIM   dimension d'espace (nb de derivees des fonctions de base)
* NUTYLE numero de type de l'element pour lequel les calculs sont effectuees
* NBDLF  nombre de d.l. de l' element ou la face de l'element en question
* NBDLDF nombre de d.l. de l'element en question
* KELFCT variable character impliquant le calcul des fonctions ou des derivees
*        KELFCT(5:5).EQ.'Y' les valeurs des fonctions de base et derivees sont
*                           conservees pour tous les points de quadrature lors
*                           du passage d'un element au suivant
* FCTB   tableau recepteur des valeurs des fonctions de base
* DFCTB  tableau recepteur des valeurs des derivees des fonctions de base 
* IAFCT  adresse du tableau FCTB ou sont stockees les fonctions de base
*        pour le point de quadrature numero IQUAD 
* IADFCT idem pour les derivees
* IMPFCT numero d'unite logique d'impression ( pas d'impression si nul)
*
*  Remarque : Dans le cas ou les calculs des fonctions ont deja ete effectues 
*             pour un element precedent, le sous-programme se contente de  
*             retourner la bonne adresse (IAFCT ou IADFCT) ou se trouve les
*             valeurs concernant le point de quadrature courant.
*             On n'effectue ainsi dans les cas les plus courants (un seul type 
*             sur un domaine) qu'une seule fois le calcul des fonctions de base
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      LOGICAL RECALC
      CHARACTER*(*) KELFCT
      INTEGER       IQUAD,NDIM,NUTYLE,NBDLF,NBDLDF,IAFCT,IADFCT
     &             ,IMPFCT
      REAL          COOR(*),FCTB(*),DFCTB(NDIM,*)
*
      REAL          PAUTIL
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*Ecobas*')
*
      IF (KELFCT(5:5).EQ.'Y') THEN
*
*        La place est suffisante pour le stockage ==>
*        Calcul de l'adresse pour stocker ou recuperer les valeurs idoines
         IAFCT=1+NBDLF*(IQUAD-1)
         IADFCT=1+NBDLDF*(IQUAD-1) 
         IF (RECALC) CALL ELEMFB (NUTYLE,KELFCT(1:3),COOR,FCTB(IAFCT)
     &                           ,DFCTB(1,IADFCT),PAUTIL,IMPFCT)
      ELSE
*
*        Pas de place disponible en M.C. ==>
*        Recalcul des fonctions de base en chaque point de quadrature
         IAFCT=1
         IADFCT=1  
         CALL ELEMFB (NUTYLE,KELFCT(1:3),COOR,FCTB,DFCTB,PAUTIL,IMPFCT)
      ENDIF
      CALL PRFXMJ (-1,'*Ecobas*')  
                                                                    END !Ecobas
