*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE REPLOC (NDIM,NUTYFG,NBDLFG,CORDLF
     &                  ,NBNEFA,CORLOC,DFCTBA
     &                  ,IRPLOC,REPERE,UNREPR,NIVIMP,IMPFCH)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Avril 1988)
* Derniere modification : D.Martin (3 Juillet 1997)
* Version 1.0.0
*  
* Calcul des vecteurs du repere local (n,t) ou (n,t_1,t_2) sur les noeuds
* du bord d'un element (calcul des vecteurs normaux seuls si IRPLOC =1)
*  
*-- Arguments --  
* NDIM   dimension d'espace
* NUTYFG numero de type de face ou de l'arete de l'element geometrique
* NBDLFG nombre de points sur la face ou l'arete de l'element geometrique
* CORDLF tableaux de coordonnees des points de la face ou de l'arete geometrique
* NBNEFA nombre de noeuds de la face ou de l'arete de l'element
* CORLOC tableau des coordonnees locales sur la face de reference pour 
*        l'interpolation concernee
* DFCTBA tableau recepteur des derivees de fonctions de base geometriques
* IRPLOC indiaue le calcul des normales aux noeuds si IRPLOC=1
*        ou le calcul des reperes locaux (normales et tangentes) si IRPLOC=NDIM
* REPERE tableau des composantes des normales (si IRPLOC=1)
*        tableau des composantes des normales et tangentes (si IRPLOC=NDIM)
* UNREPR tableau des composantes des vecteurs du repere local en un noeud
* NIVIMP niveau d'impression
* IMPFCH numero logique du fichier d'impression
*-- Procedures utilisees --
* ELEMFB fournit les derivees de fonctions de base
* DELAFO fournit la (les) tangentes sur l'arete/la face de l'element courant
* PROEXT calcul d'un vecteur normal au bord (produit exterieur)
* NORMIZ normalisation d'un vecteur
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      INTEGER       NDIM,NUTYFG,NBDLFG,NBNEFA,IRPLOC,NIVIMP
     &             ,IMPFCH
      REAL          CORLOC(NDIM-1,*),DFCTBA(NDIM-1,*),CORDLF(NDIM,*)
     &             ,REPERE(NDIM,NBNEFA,*),UNREPR(NDIM,*)
*
      CHARACTER     WCH,FORMA*119,PRFXAF*8
      INTEGER       INEF,IDM,IDIOT,I,J,IMPZZZ
      REAL          BID,X
      COMMON/FORMAH/FORMA,WCH
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*Reploc*')
      IMPZZZ=IMPFCH
      IMPZZZ=0
*  
*     Boucle sur les noeuds de la face/arete
*
      DO 10 INEF=1,NBNEFA
*  
*        Calcul des derivees des fonctions de base geometriques au noeud  
*        de l'element de reference associe au INEF-eme noeud de la face
*
         CALL ELEMFB (NUTYFG,'NYN',CORLOC(1,INEF),BID,DFCTBA,BID,IMPZZZ)!ef3d
* 
*        Calcul de NDIM-1 vecteur(s) tangent(s) au bord de l'element
* 
         CALL DELAFO (NDIM,NDIM-1,NBDLFG,CORDLF,DFCTBA,IMPZZZ
     &               ,UNREPR(1,2))                                      !calelem
*
*        Calcul du vecteur normal par 'produit exterieur' et normalisation
*
         CALL PROEXT (NDIM,UNREPR(1,2),UNREPR)                          !utilite
         CALL NORMIZ (NDIM,UNREPR,X)                                    !utilite
         DO 2 IDM=1,NDIM  
            REPERE(IDM,INEF,1)=UNREPR(IDM,1)
2        CONTINUE  
*  
         IF (IRPLOC.NE.1) THEN  
*  
*           Normalisation de la premiere tangente calculee ci-dessus
* 
            CALL NORMIZ (NDIM,UNREPR(1,2),X)                            !utilite
            DO 3 IDM=1,NDIM
               REPERE(IDM,INEF,2)=UNREPR(IDM,2)
3           CONTINUE
*  
*           Calcul de la seconde tangente par produit exterieur (en 3d)
*
            IF (NDIM.EQ.3) CALL PROEXT (NDIM,UNREPR,REPERE(1,INEF,3))   !utilite
*
         ENDIF
*
10    CONTINUE 
*
      CALL PRFXMJ (-1,'*Reploc*')
      IF (NIVIMP.LE.0.AND.IMPFCH.GT.0) RETURN  
*-----------------------------------------------------------------------
*     Impressions (particulierement elegantes !)
      WCH=CHAR(NDIM+48)  
      WRITE (IMPFCH,*) PRFXAF( ),'>*Reploc*'
      FORMA
     &='(1(''     i '','//WCH//'('' Normal(i,'',I1,'') ''),'' '',:),'
     &//WCH//'('//WCH//'('' Tangt'',I1,''(i,'',I1,'') ''),'' '':))'
      IDIOT=0  
      IF (IRPLOC.EQ.1) THEN 
         IDIOT=NDIM+(-1)**NDIM 
         FORMA(2:2)=CHAR(IDIOT+49)
      ENDIF
      WRITE (IMPFCH,FORMA) ((IDM,IDM=1,NDIM),J=0,MIN(IDIOT,NBNEFA-1))
     &,((J,IDM,IDM=1,NDIM),J=1,IRPLOC-1)
      I=0
      FORMA='('//CHAR(IDIOT+49)//'(I6,'' '','//CHAR(IRPLOC+48)
     &//'('//WCH//'E13.5,'' '':)))'
11    I=I+1
      IF (I.GT.NBNEFA) RETURN
      IDIOT=MIN (IDIOT,NBNEFA-I)
      IF (IDIOT.NE.0) WRITE (IMPFCH,FORMA)
     &   (I+J,(REPERE(IDM,I+J,1),IDM=1,NDIM),J=0,IDIOT)
      IF (IDIOT.EQ.0) WRITE (IMPFCH,FORMA)
     &   I,((REPERE(IDM,I,J),IDM=1,NDIM),J=1,IRPLOC)
      I=I+IDIOT
      GOTO 11 
                                                                    END !Reploc
