*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE DSKMCO (NCHDOM,NUDOG ,NUDOK ,NBELDO 
     &                  ,NBNECO,NBNELI,NVSTKM,MATDIM,NIVIMP,IMPFCH)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin & O.deBayser (Avril 1988) 
* Derniere modification : D.Martin (16 mars 2008)
* Version 2
*  
* Creation des tableaux de stockage 'Morse-Colonne' des termes sur un domaine
*  
*-- Arguments d'entree -- 
* NCHDOM nombre de caracteres par nom de domaine
* NUDOG  numero du domaine geometrique
* NUDOK  numero du domaine de calcul
* NBELDO nombre d'elements (de couples element-face) du domaine
* NBNECO nombre de noeuds pour l'interpolation en colonne
* NBNELI nombre de noeuds pour l'interpolation en ligne
* NVSTKM niveau du tableau de stockage 'Morse-C' ('Morse-colonne')
* NIVIMP numero de niveau d'impression
* IMPFCH numero d'unite logique d'impression
*  
*-- Arguments en sortie --
* MATDIM: Nombre de coefficients non nuls des matrices 
*  
*-- Structure du tableau rempli --
* &EGALP
*   1        -> NBNECO (Nombre de colonnes de la matrice)
*   2        -> NBNELI (Nombre de ligneses de la matrice)
*   3        -> NBPLAJ (Nombre de plages)  
*   4        -> MATDIM Nombre de coefficients non nuls des termes  
*   5        -> NULAPL(I) , I=1,NBNECO (Numeros de derniere plage des colonnes)  
*   NBNELI+5 -> IPLAGE(J,I) , J=1,2, I=1,NBPLAJ
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      INTEGER  NCHDOM,NUDOG ,NUDOK ,NBELDO,NBNECO,NBNELI,NVSTKM,MATDIM
     &        ,NIVIMP,IMPFCH
*
      INCLUDE 'ALLOC'  
      INCLUDE 'CONTEX' 
*
      INTEGER LGSTMO,IRCODE
      INTEGER MCPLAG,NL,NC,NB,MCPLAL,MCPLIL,LGMORS,MCGOCO,LGREEL
     &       ,NBPLAG,MCGOLI,NBPLAX,MCMDOM,L,IDL,JDL,IGLO,JGLO
      CHARACTER     ERCODE*120 
      COMMON/FORMAH/ERCODE 
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*DskMco*')
*
      ERCODE(1:1)='!'  
      CALL TBRR1  (ERCODE,'&EGALP',NVSTKM,MCPLAG)
*
      IF (ERCODE(1:1).EQ.' ') THEN
*        Le tableau de stockage Morse-C existe : ses attributs
         CALL TBAR1  (ERCODE,'&EGALP',NVSTKM,MCPLAG)
         CALL EXMORS (MCPLAG,IST(MCPLAG),NC,NL,NB,MCPLAL,MCPLIL,MATDIM) !utilite
*
      ELSE
*
*        Surestimation de la taille du tableau de stockage
         LGMORS=LGSTMO ('C',NBNELI,NBNECO,0)                            !utilite
*
*        Creation du tableau de stockage des matrices Morse-C
         CALL TBRR1  (ERCODE,'#RGNDO',NUDOK ,MCGOCO)
         CALL TBCRSU ('&EGALP',NVSTKM,1,LGMORS,LGREEL,'c')
*        Nombre maximum de plages du tableau de stockage
         NBPLAX=(LGREEL-NBNECO-4)/2
*
         CALL TBAR2  (ERCODE,'#RGNDO',NUDOK,MCGOCO
     &                      ,'&EGALP',NVSTKM,MCPLAG)
*
*--      Remplissage du tableau de stockage Morse-C
*
*        Mise a zero du sous-tableau NULAPL
         MCPLIL=MCPLAG+4+NBNECO
         DO 1 L=MCPLAG+4,MCPLIL-1
            IST(L)=0
1        CONTINUE
         NBPLAG=0
         MCGOLI=MCGOCO+IST(MCGOCO)-1
         MCGOCO=MCGOCO+1
*
         DO 10 L=1,NBELDO
*           Boucle sur les noeuds de l'element en colonne
            DO 121 JDL=1,IST(MCGOCO)
               JGLO=IST(MCGOCO+JDL)
*              Boucle sur les noeuds de l'element en ligne
               DO 122 IDL=1,IST(MCGOLI)
                  IGLO=IST(MCGOLI+IDL)
                  CALL CMPLGN (JGLO,IGLO,NBNECO,NBPLAG,IST(MCPLAG+4)
     &                        ,IST(MCPLIL),NBPLAX,IRCODE)               !morse
                  IF (IRCODE.NE.0) GOTO 99992
122            CONTINUE
121         CONTINUE
            MCGOCO=MCGOCO+IST(MCGOCO)+1
            MCGOLI=MCGOLI+IST(MCGOLI)+1
10       CONTINUE
         LGMORS=4+NBNECO+2*NBPLAG
         MATDIM=IST(MCPLAG-1+LGMORS)
         IST(MCPLAG)  =NBNECO
         IST(MCPLAG+1)=NBNELI
         IST(MCPLAG+2)=NBPLAG
         IST(MCPLAG+3)=MATDIM
*
         IF (IMPFCH.GT.0.AND.NIVIMP.GT.0) THEN
            CALL TBAR1  (ERCODE,'#OMDOM',1,MCMDOM)
            MCMDOM=MCMDOM+NCHDOM*(NUDOG-1)-1
            DO 30 L=1,NCHDOM
               MCMDOM=MCMDOM+1
30             ERCODE(L:L)=AST(MCMDOM)
               CALL PRMORS (IST(MCPLAG),'C','Stockage Morse-C, domaine'
     &                     ,ERCODE(1:NCHDOM),NUDOK,NIVIMP,IMPFCH)       !prsd
         ENDIF
*
*        Modification des tableaux de stockage
*        par concatenation de plages voisines
*        CALL CMPCAT (2,NBNECO,IST(MCPLAG+4),IST(MCPLIL))               !morse
*
*        NBPLAG       =IST(MCPLAG+3+NBNECO)
*        LGMORS       =4+NBNECO+2*NBPLAG
*        MATDIM       =IST(MCPLAG-1+LGMORS) 
*        IST(MCPLAG+2)=NBPLAG
*        IST(MCPLAG+3)=MATDIM
         CALL TBAJST('&EGALP',NVSTKM,LGMORS)                     
      ENDIF
      CALL PRFXMJ (-1,'*DskMco*')  
      RETURN
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
99992 CALL TBAR1  (ERCODE,'#OMDOM',1,MCMDOM)                     
      MCMDOM=MCMDOM+NCHDOM*(NUDOG-1)-1
      DO 99982 NL=1,NCHDOM
         MCMDOM=MCMDOM+1
         ERCODE(NL:NL)=AST(MCMDOM)
99982 CONTINUE
      CALL ENCLER (JGLO,ERCODE(NCHDOM+1:NCHDOM+6))
      CALL ENCLER (L,ERCODE(NCHDOM+6:NCHDOM+11))
      CALL BAISE  ('Nombre de plages sous-estime dans la creation de la'
     &//' structure Morse-colonne sur le domaine '//ERCODE(1:NCHDOM)
     &//' . Maximum atteint sur la colonne '//ERCODE(NCHDOM+1:NCHDOM+6)
     &//' dans l''element '//ERCODE(NCHDOM+6:NCHDOM+11)//'.')
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 2 : D.Martin (16 mars 2008)
*  Correction d'un bug lorsque qu'un terme vide n'est pas un bloc diagonal
* Version 1 : D.Martin (1er Fevrier 2002)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !DskMco
