*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE DSKMLI (NCHDOM,NUDOG ,NUDOK ,NBELDO 
     &                  ,NBNECO,NBNELI,NVSTKM,MATDIM,NIVIMP,IMPFCH)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin & O.deBayser (Avril 1988) 
* Derniere modification : D.Martin (16 mars 2008)
* Version 2
*  
* Creation des tableaux de stockage 'Morse-Ligne' des termes sur un domaine
*  
*-- Arguments d'entree -- 
* NCHDOM nombre de caracteres par nom de domaine
* NUDOG  numero du domaine geometrique
* NUDOK  numero du domaine de calcul
* NBELDO nombre d'elements (de couples element-face) du domaine
* NBNECO nombre de noeuds pour l'interpolation en colonne
* NBNELI nombre de noeuds pour l'interpolation en ligne
* NVSTKM niveau du tableau de stockage 'Morse-L' ('Morse-Ligne')
* NIVIMP numero de niveau d'impression
* IMPFCH numero d'unite logique d'impression
*  
*-- Arguments en sortie --
* MATDIM nombre de coefficients non nuls des matrices
*  
*-- Structure du tableau rempli --
* &PLAGE
*   1        -> NBNELI (Nombre de lignes de la matrice)
*   2        -> NBNECO (Nombre de colonnes de la matrice)
*   3        -> NBPLAJ (Nombre de plages)  
*   4        -> MATDIM Nombre de coefficients non nuls des termes  
*   5        -> NULAPL(I) , I=1,NBNECO (Numeros de derniere plage des lignes)  
*   NBNELI+5 -> IPLAGE(J,I) , J=1,2, I=1,NBPLAJ
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      INTEGER  NCHDOM,NUDOG ,NUDOK ,NBELDO,NBNECO,NBNELI,NVSTKM,MATDIM
     &        ,NIVIMP,IMPFCH
*
      INCLUDE 'ALLOC'  
      INCLUDE 'CONTEX' 
*
      INTEGER LGSTMO,IRCODE
      INTEGER MCPLAG,NL,NC,NB,MCPLAL,MCPLIL,LGMORS,MCGOCO,LGREEL
     &       ,NBPLAG,MCGOLI,NBPLAX,MCMDOM,L,IDL,JDL,IGLO,JGLO
      CHARACTER     ERCODE*120 
      COMMON/FORMAH/ERCODE 
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*DskMli*')
*
      ERCODE(1:1)='!'  
      CALL TBRR1  (ERCODE,'&PLAGE',NVSTKM,MCPLAG)
*
      IF (ERCODE(1:1).EQ.' ') THEN
*        Le tableau de stockage existe : ses attributs
         CALL TBAR1  (ERCODE,'&PLAGE',NVSTKM,MCPLAG)
         CALL EXMORS (MCPLAG,IST(MCPLAG),NL,NC,NB,MCPLAL,MCPLIL,MATDIM) !utilite
*
      ELSE
*
*        Surestimation de la taille du tableau de stockage
         LGMORS=LGSTMO ('L',NBNELI,NBNECO,0)                            !utilite
*
*        Creation du tableau de stockage des matrices Morse-L
         CALL TBRR1  (ERCODE,'#RGNDO',NUDOK ,MCGOCO)
         CALL TBCRSU ('&PLAGE',NVSTKM,1,LGMORS,LGREEL,'c')
*        Nombre maximum de plages du tableau de stockage
         NBPLAX=(LGREEL-NBNELI-4)/2
*
         CALL TBAR2  (ERCODE,'#RGNDO',NUDOK,MCGOCO
     &                      ,'&PLAGE',NVSTKM,MCPLAG)
*
*--      Remplissage du tableau de stockage Morse-C
*
*        Mise a zero du sous-tableau NULAPL du tableau
         MCPLIL=MCPLAG+4+NBNELI
         DO 1 L=MCPLAG+4,MCPLIL-1
            IST(L)=0
1        CONTINUE
         NBPLAG=0
         MCGOLI=MCGOCO+IST(MCGOCO)-1
         MCGOCO=MCGOCO+1
*
         DO 10 L=1,NBELDO
*           Boucle sur les noeuds de l'element en ligne
            DO 111 IDL=1,IST(MCGOLI)
               IGLO=IST(MCGOLI+IDL)
*              Boucle sur les noeuds de l'element en colonne
               DO 112 JDL=1,IST(MCGOCO)
                  JGLO=IST(MCGOCO+JDL)
                  CALL CMPLGN (IGLO,JGLO,NBNELI,NBPLAG,IST(MCPLAG+4)
     &                        ,IST(MCPLIL),NBPLAX,IRCODE)               !morse
                  IF (IRCODE.NE.0) GOTO 99991
112            CONTINUE
111         CONTINUE
            MCGOCO=MCGOCO+IST(MCGOCO)+1
            MCGOLI=MCGOLI+IST(MCGOLI)+1
10       CONTINUE
         LGMORS=4+NBNELI+2*NBPLAG
         MATDIM=IST(MCPLAG-1+LGMORS)
         IST(MCPLAG)  =NBNELI
         IST(MCPLAG+1)=NBNECO
         IST(MCPLAG+2)=NBPLAG
         IST(MCPLAG+3)=MATDIM
*
         IF (IMPFCH.GT.0.AND.NIVIMP.GT.0) THEN
            CALL TBAR1  (ERCODE,'#OMDOM',1,MCMDOM)
            MCMDOM=MCMDOM+NCHDOM*(NUDOG-1)-1
            DO 30 L=1,NCHDOM
               MCMDOM=MCMDOM+1
30             ERCODE(L:L)=AST(MCMDOM)
               CALL PRMORS (IST(MCPLAG),'L','Stockage Morse-L, domaine'
     &                     ,ERCODE(1:NCHDOM),NUDOK,NIVIMP,IMPFCH)       !prsd
         ENDIF
*
*        Modification des tableaux de stockage
*        par concatenation de plages voisines
*        CALL CMPCAT (2,NBNELI,IST(MCPLAG+4),IST(MCPLIL))               !morse
*
*        NBPLAG       =IST(MCPLAG+3+NBNELI)
*        LGMORS       =4+NBNELI+2*NBPLAG
*        MATDIM       =IST(MCPLAG-1+LGMORS)
*        IST(MCPLAG+2)=NBPLAG
*        IST(MCPLAG+3)=MATDIM
         CALL TBAJST('&PLAGE',NVSTKM,LGMORS)                     
      ENDIF
      CALL PRFXMJ (-1,'*DskMli*')  
      RETURN
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
99991 CALL TBAR1  (ERCODE,'#OMDOM',1,MCMDOM)                     
      MCMDOM=MCMDOM+NCHDOM*(NUDOG-1)-1
      DO 99981 NL=1,NCHDOM
         MCMDOM=MCMDOM+1
         ERCODE(NL:NL)=AST(MCMDOM)
99981 CONTINUE
      CALL ENCLER (IGLO,ERCODE(NCHDOM+1:NCHDOM+6))
      CALL ENCLER (L,ERCODE(NCHDOM+6:NCHDOM+11))
      CALL BAISE  ('Nombre de plages sous-estime dans la creation de la'
     &//' structure Morse-ligne sur le domaine '//ERCODE(1:NCHDOM)
     &//' . Maximum atteint sur la ligne '//ERCODE(NCHDOM+1:NCHDOM+6)
     &//' dans l''element '//ERCODE(NCHDOM+6:NCHDOM+11)//'.')
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 2 : D.Martin (16 mars 2008)
*  Correction d'un bug lorsque qu'un terme vide n'est pas un bloc diagonal
* Version 1 : D.Martin (20 Decembre 2000)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !DskMli
