*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE EFTBAD (NDIM  ,NDMDOM,NBPTXQ,IAPOIQ,IACORQ,IAPOID
     &                  ,IACORD,IACORF,LGTABB
     &                  ,NVSTKB,IAPTRB,NVSTKL,IAPTRL,NVSTKC,IAPTRC
     &                  ,INDEXC,IDLHRC,INTRPC,NBNEFC,NBDLFC,NBNEXC
     &                  ,KELFCC,IANDIC,IANUFC,IADLFC,ADFCTC,ADDFCC
     &                  ,ADDWDC
     &                  ,INDEXL,IDLHRL,INTRPL,NBNEFL,NBDLFL,NBNEXL
     &                  ,KELFCL,IANDIL,IANUFL,IADLFL,ADFCTL,ADDFCL
     &                  ,ADDWDL
     &                  ,INTRPG,NBNEFG,NBDLFG,KELFCG,IANUFG,IADLFG
     &                  ,ADFCTG,ADDFCG,LGMXTB,LGWORK,NIVIMP,IMPFCH)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin & O.deBayser (Avril 1988) 
* Derniere modification : D.Martin (10 Juin 2005)
* Version 2
* Modifications anterieures : 
* D.Martin (5 Juillet 2000)
*  
* Calcul des adresses relatives de tableaux de preparation 'elements finis'
* dans le tableau $WORK$ de niveau NUDOK
*  
*-- Arguments d'entree --  
* NDIM   dimension d'espace
* NDMDOM dimension du domaine geometrique
* NBPTXQ nombre maximum de points de quadrature sur le domaine
* LGTABB tableau de travail
*     Contenu du tableau LGTABB
*     ( 1) longueur cumulee des matrices elementaires reelles
*     ( 2) longueur cumulee des matrices elementaires complexes 
*     ( 3) LGJACO, longueur de la matrice jacobienne
*     ( 4) LGJAJA, longueur de la mat. jacobienne inverse x par sa transposee
*     ( 5) LGFCTC, longueur du tableau des fonctions de base pour
*                  l'interpolation en colonne
*     ( 6) LGFCTL, idem pour l'interpolation en ligne
*     ( 7) LGFCTG, idem pour l'interpolation geometrique
*     ( 8) LGDFCC, longueur du tableau des derivees des fonctions de base pour
*                  l'interpolation en colonne
*     ( 9) LGDFCL, idem pour l'interpolation en ligne
*     (10) LGDFCG, idem pour l'interpolation geometrique  
* NVSTKB niveau du tableau de stockage des matrices 'BiMorse' sur le domaine
* NVSTKL niveau du tableau de stockage des matrices 'Morse-L' sur le domaine
* NVSTKC niveau du tableau de stockage des matrices 'Morse-C' sur le domaine
* INDEXC indice d'extension du domqine en colonne
* IDLHRC indicateur d'interpolation 'Hermite' en colonne
* INTRPC indice d'interpolation en colonne
* NBNEFC nb. max. de noeuds dans un 'element' pour l'interp. en colonne
* NBDLFC nb. max. de d.l. dans un 'element' pour l'interp. en colonne  
* NBNEXC nb. max de colonnes pour les termes du domaine
* KELFCC indice de calcul des fonctions de base pour l'interp. en colonne
*       (1:1)='Y' ==> calcul des fonctions de base 
*       (2:2)='Y' ==> calcul des derivees des fonctions de base
*       (3:3)='Y' ==> calcul des derivees secondes des fct. de base
*       (4:4)='C' ==> calculs selon l'interpolation en 'colonne'
*             'G' ==> calculs selon l'interpolation geometrique
*       (5:5)='Y' ==> on conserve les calculs en tous les points
*                     de quadrature pour l'element suivant
*       (6:6)='Y' ==> implique le calcul de la matrice jacobienne dans
*                     tout l'element (integrands contenant une derivee)
*                     inutilise pour les calculs en dimension d'espace
* INDEXL indice d'extension du domaine en ligne
* IDLHRL indicateur d'interpolation 'Hermite' en ligne
* INTRPL indice d'interpolation en ligne
* NBNEFL nb. max. de noeuds dans un 'element' pour l'interp. en ligne
* NBDLFL nb. max. de d.l. dans un 'element' pour l'interp. en ligne
* NBNEXL nb. max de lignes pour les termes du domaine
* KELFCL indice de calcul des fonctions de base pour l'interp. en ligne
*       (i:i) , i=1,2,3 ==> cf. KELFCC
*       (4:4)='C' ==> calculs selon l'interpolation en 'colonne'
*             'G' ==> calculs selon l'interpolation geometrique
*             'L' ==> calculs selon l'interpolation en 'ligne'
*       (j:j) , j=5,6 ==> cf. KELFCC
* INTRPC indice d'interpolation geometrique
* NBNEFG nombre max de noeuds geometriques sur les elements du domaine
* NBDLFG nombre max de d.l. geometriques sur les elements du domaine
* KELFCG indice de calcul des fonctions de base geometriques
*     (i:i) , i=1,2,3 ==> cf. KELFCC
*     (4:4) inutilise
*     (5:5) ==> cf. KELFCC ou KELFCL
*     (6:6) ==> impose le calcul effectif des fonctions de base de l'interpol.
*               geometrique (e.g. dans le cas d'un integrand a coeff. variable)
* LGMXTB longueur disponible en memoire centrale a l'appel de cette procedure
*-- Arguments de sortie --
* IAPOIQ adr. rel. du tab. des poids de quadrature
* IACORQ      "      "         coordonnees des points de quadrature
* IAPOID adr. rel. du tab. des poids de quadrature (domaine etendu)
* IACORD      "      "         coordonnees des points de quadrature (idem)
* IACORF      "      "         coordonnees des noeuds sur la 'face'
* IAPTRB adresse du tab. des pointeurs des coeff. dans les matrices BiMorse
* IAPTRL adresse du tab. des pointeurs des coeff. dans les matrices Morse-L
* IAPTRC adresse du tab. des pointeurs des coeff. dans les matrices Morse-C  
* IANDIC,IANDIL,IADLFG,IANUFL,IADLFL adr. de divers de tab. de numerotations
* ADFCTC,ADFCTL,ADFCTG adresses pour les tableaux des valeurs des fonctions
*        de base dans l'elt. de ref. pour les 3 interpolations
*       (en colonne, en ligne et geometrique)
* ADDFCC,ADDFCL,ADDFCG adresses pour les tableaux des derivees des fonctions
*        de base dans l'elt. de ref. pour les 3 interpolations
* LGWORK longueur du tableau de travail 'elements finis' $WORK$  
* NIVIMP,IMPFCH quincaillerie habituelle
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) KELFCC,KELFCL,KELFCG
      INTEGER       NDIM,NDMDOM,NBPTXQ,IAPOIQ,IACORQ,IAPOID,IACORD
     &             ,IACORF,NVSTKB,IAPTRB,NVSTKL,IAPTRL,NVSTKC,IAPTRC
     &             ,INDEXC,IDLHRC,INTRPC,NBNEFC,NBDLFC,NBNEXC,IANDIC
     &             ,IANUFC,IADLFC,ADFCTC,ADDFCC,ADDWDC
     &             ,INDEXL,IDLHRL,INTRPL,NBNEFL,NBDLFL,NBNEXL,IANDIL
     &             ,IANUFL,IADLFL,ADFCTL,ADDFCL,ADDWDL
     &             ,INTRPG,NBNEFG,NBDLFG,IANUFG,IADLFG,ADFCTG,ADDFCG
     &             ,LGMXTB,LGWORK,NIVIMP,IMPFCH,LGTABB(*)
*      Variables locales
      INTEGER NEXTAD,LGMAX,I,ADDWDG,LGDWDG,LGDWDC,LGDWDL
*----------------------------------------------------------------------- 
*     Adresses dans le tableau des poids et points de quadrature
*     Longueurs resp. NBPTXQ et NDMDOM*NBPTXQ  
      IAPOIQ = 1
      IAPOID = IAPOIQ
      IF (MAX (INDEXC,INDEXL).GT.0) IAPOIQ = IAPOIQ + NBPTXQ
      IACORQ = IAPOIQ+NBPTXQ 
      IACORD = IACORQ
      IF (MAX (INDEXC,INDEXL).GT.0) IACORQ = IACORQ + NDIM*NBPTXQ
      NEXTAD = IACORQ + NDMDOM*NBPTXQ
*  
*     Adresse des tableaux de stockage 'Bi-Morse', 'Morse-L' et 'Morse-C' 
      IAPTRB = NEXTAD
      IF (NVSTKB.GT.0) THEN
         IAPTRB = NEXTAD
         NEXTAD = NEXTAD + 2*(NBNEXL + NBNEXC) + NBNEXL*NBNEXC
      ENDIF
      IAPTRL = NEXTAD
      IF (NVSTKL.GT.0) THEN
         IAPTRL = NEXTAD
         NEXTAD = NEXTAD + NBNEXL*(2 + NBNEXC)
      ENDIF
      IAPTRC = NEXTAD
      IF (NVSTKC.GT.0) THEN
         IAPTRC = NEXTAD
         NEXTAD = NEXTAD + NBNEXC*(2 + NBNEXL)
      ENDIF      
*  
*     Adresses pour des tableaux des coordonnees et derivees des fonctions 
*     en restriction a une face de dimension NDMDOM < NDIM (le cas echeant)
      IACORF = 0
      IF ((KELFCG(6:6).EQ.'Y'.AND.NDIM.NE.NDMDOM)  
     &.OR.(KELFCC(6:6).EQ.'Y'.AND.INDEXC.NE.0) 
     &.OR.(KELFCL(6:6).EQ.'Y'.AND.INDEXL.NE.0))  THEN  
         IACORF = NEXTAD
         NEXTAD = IACORF + NDIM*NBDLFG 
      ENDIF
*
*     Adresses pour les tableaux de numerotations locales NDLINL,NDLINC
      IANDIC = NEXTAD
      IF (IDLHRC.GT.0) NEXTAD = NEXTAD + NBNEFC
      IANDIL = NEXTAD
      IF (IDLHRL.GT.0) NEXTAD = NEXTAD + NBNEFL
*
*     Adr. pour les tableaux de numerotations locales NUNEFG,NUDLFG
      IANUFG = NEXTAD
      IADLFG = NEXTAD
      IANUFC = NEXTAD
      IADLFC = NEXTAD
      IANUFL = NEXTAD
      IADLFL = NEXTAD
      IF (NDMDOM.LT.NDIM) THEN
         NEXTAD = NEXTAD + MAX (NBNEFG,NBNEFC,NBNEFL)
*<<<<    IADLFG = NEXTAD
*<<<<    NEXTAD = NEXTAD + MAX (NBDLFG,NBDLFC,NBDLFL)
*>>>>    Lignes precedentes (2) changees le 10/06/2005 en les 5 qui suivent
*        (provoquaient un bug d'adressage dans calelem/caleld)
         IADLFG = IANUFG
         IF (NBDLFG.NE.NBNEFG) THEN
            NEXTAD = NEXTAD + MAX (NBDLFG,NBDLFC,NBDLFL)
            IADLFG = NEXTAD
         ENDIF
*>>>>
*
*        Adr. pour les tableaux de numerotations locales NUNEFC,NUDLFC
*        (necessaires dans ExStel/Exstew meme si inutilises par la suite)
         IF (INTRPC.EQ.INTRPG) THEN
            IANUFC = IANUFG
            IADLFC = IADLFG
         ELSE
            IANUFC = NEXTAD
            NEXTAD = IANUFC + NBNEFC
            IADLFC = NEXTAD
            IF (IDLHRC.GT.0) NEXTAD = NEXTAD + NBDLFC
         ENDIF
*
*        Adr. pour les tableaux de numerotations locales NUNEFL,NUDLFL
         IF (INTRPL.EQ.INTRPG) THEN
            IANUFL = IANUFG
            IADLFL = IADLFG
         ELSEIF (INTRPL.EQ.INTRPC) THEN
            IANUFL = IANUFC
            IADLFL = IADLFC
         ELSE
            IANUFL = NEXTAD
            NEXTAD = IANUFL + NBNEFL
            IADLFL = NEXTAD
            IF (IDLHRL.GT.0) NEXTAD = NEXTAD + NBDLFL
         ENDIF
      ENDIF
*
*     Longueur disponible en M.C.
*
      LGMAX = LGMXTB-NEXTAD  
      DO 1 I = 5,10  
         LGMAX = LGMAX-LGTABB(I)
 1    CONTINUE
*
*     Longueur des tableaux contenant les derivees des fonctions de base
*     pour les interpolations en ligne et colonne
*
      LGDWDG=LGTABB(10)
      LGDWDC=LGTABB( 8)
      LGDWDL=LGTABB( 9)
*  
*     Peut-on conserver les calculs des fonctions de base et derivees  
*     en tous les points de quadrature (par ordre de priorite fonctions
*     de base geometriques, puis en colonne, puis en ligne)
*  
      CALL EFADFB (LGMAX,LGTABB(7),LGTABB(10),NBPTXQ,KELFCG(5:5))       !caltrm
      CALL EFADFB (LGMAX,LGTABB(5),LGTABB( 8),NBPTXQ,KELFCC(5:5))       !caltrm
      CALL EFADFB (LGMAX,LGTABB(6),LGTABB( 9),NBPTXQ,KELFCL(5:5))       !caltrm
*
*     Adresse(s) dans le tableau pour les fonctions de base
*     dans l'element de reference ...
*     ... geometriques
      ADFCTG = NEXTAD
      LGWORK = ADFCTG + LGTABB(7)
*     ... pour l'interpolation en colonne
      IF (KELFCC(4:4).EQ.'G') THEN
         ADFCTC = ADFCTG
      ELSE
         ADFCTC = LGWORK
         LGWORK = LGWORK + LGTABB(5)
      ENDIF
*     ... pour l'interpolation en ligne
      IF (KELFCL(4:4).EQ.'G') THEN
         ADFCTL = ADFCTG
      ELSEIF (KELFCL(4:4).EQ.'C') THEN
         ADFCTL = ADFCTC
      ELSE
         ADFCTL = LGWORK
         LGWORK = LGWORK + LGTABB(6)
      ENDIF
*     Adresse(s) dans le tableau pour les derivees des fonctions de base
*     dans l'element de reference et l'element courant ...
*     ... geometriques
      ADDFCG = LGWORK
      LGWORK = LGWORK + LGTABB(10)
      ADDWDG = LGWORK
      LGWORK = LGWORK + LGDWDG
*     ... pour l'interpolation en colonne
      IF (KELFCC(4:4).EQ.'G') THEN
         ADDFCC = ADDFCG
         ADDWDC = ADDWDG
      ELSE
         ADDFCC = LGWORK
         LGWORK = LGWORK + LGTABB(8)
         ADDWDC = LGWORK
         LGWORK = LGWORK + LGDWDC
      ENDIF
*     ... pour l'interpolation en ligne
      IF (KELFCL(4:4).EQ.'G') THEN
         ADDFCL = ADDFCG
         ADDWDL = ADDWDG
      ELSEIF (KELFCL(4:4).EQ.'C') THEN
         ADDFCL = ADDFCC
         ADDWDL = ADDWDC
      ELSE
         ADDFCL = LGWORK
         LGWORK = LGWORK + LGTABB(9)
         ADDWDL = LGWORK
         LGWORK = LGWORK + LGDWDL
      ENDIF
*  
      IF (NIVIMP.LE.0) RETURN
*
      IF (IAPOID.NE.IAPOIQ) THEN
         WRITE (IMPFCH,100) LGWORK,IAPOIQ,IACORQ,IAPOID,IACORD
      ELSE
         WRITE (IMPFCH,100) LGWORK,IAPOIQ,IACORQ 
      ENDIF
      WRITE (IMPFCH,105) IAPTRB,IAPTRL,IAPTRC
      IF (IACORF.NE.0) WRITE (IMPFCH,101) IACORF  
      IF (IDLHRC.NE.0.OR.IDLHRL.NE.0) WRITE(IMPFCH,103) IANDIC,IANDIL
      IF (NDMDOM.LT.NDIM)
     &WRITE (IMPFCH,104) IANUFG,IADLFG,IANUFC,IADLFC,IANUFL,IADLFL
      WRITE (IMPFCH,107) ADFCTG,ADDFCG,ADDWDG
     &                  ,ADFCTC,ADDFCC,ADDWDC,ADFCTL,ADDFCL,ADDWDL
      WRITE (IMPFCH,106) KELFCG,KELFCC,KELFCL
*----------------------------------------------------------------------
 100  FORMAT(/' *EFtbad* Adresses dans le tableau $WORK$ de longueur '  
     &,I10/T9,'Poids et points de quadrature',T48,2I10,:/
     &     T9,'Poids et points de quadr. sur le bord',T48,2I10)
 101  FORMAT(T9,'Coordonnees sur la face',T48,I10)
 103  FORMAT(T9,'Numerotations locales (Hermite)',T48,2I10)
 104  FORMAT(T9,'Numerotations locales de face',T48,6I10)
 105  FORMAT(T9,'Pointeurs (BiMorse,Morse-C,Morse-L)',T48,3I10)
 106  FORMAT(T9,'Indices des fonctions de base (geo/col/lig)'
     &,T54,3(A,'    '))
 107  FORMAT(T9,'Fonct. de base et derivees (geometrie)',T48,3I10
     &      /T9,'Fonct. de base et derivees (colonne)',T48,3I10
     &      /T9,'Fonct. de base et derivees (ligne)',T48,3I10)
                                                                    END !EFtbad

