*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE PAVLBM (NBNEUL,NNEITL,NUNEFL,RGNELI,NCPINL
     &                  ,NBNEUC,NNEITC,NUNEFC,RGNECO,NCPINC,NBNEMI
     &                  ,MCMATR,MCMATC,MCTELM,TYPTRM,MCDIAG,MCTRIL
     &                  ,PTRDBL,LGLIBM,PTRBMO,RST,CST)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin & O.DeBayser (Mars 1992) 
* Derniere modification : D.Martin (5 Octobre 2001)
* Version 1.0.2
*  
* Assemblage d'une matrice elementaire dans la diagonale et la partie 
* triangulaire inferieure de la matrice d'un terme stocke 'Bi-Morse'
* pour des inconnues vectorielles en ligne ou colonne
*  
*-- Arguments d'entree --
* NBNEUL nombre des noeuds en ligne dans l'element
* NNEITL nombre de lignes de la matrice elementaire comptees en noeuds
* NUNEFL correspondance noeud de face - noeud dans l'element en ligne
* RGNELI RGNELI(ne) est la ligne de la matrice assemblee associee au noeud ne
* NCPINL nombre de composantes de l'inconnue en ligne
* NBNEUC nombre de noeuds en colonne dans l'element
* NNEITC nombre de colonnes de la matrice elementaire comptees en noeuds  
* NUNEFC correspondance noeud de face - noeud dans l'element en colonne
* RGNECO RGNECO(ne) est la colonne de la matrice assemblee associee au noeud ne
* NCPINC nombre de composantes de l'inconnue en colonne
* NBNEMI nombre de noeuds minimum en ligne ou en colonne
* MCMATR adresse absolue du tableau des matrices elementaires reelles
* MCMATC adresse absolue du tableau des matrices elementaires complexes
* MCTELM adresse relative de la matrice elementaire 
* TYPTRM type REEL ou COMPLEXE du terme
* MCDIAG adresse abs. de la diagonale du terme Bi-Morse assemble
* MCTRIL adresse abs. de la partie triangulaire inf. du terme Bi-Morse assemble
* PTRDBL tableau des adresses de debut des lignes de la matrice compactee
* LGLIBM tableau des longueurs des lignes de la matrice compactee
* PTRBMO tableau des adresses a partir du debut de la ligne dans la matrice
*        Bi-Morse des coefficients d'une matrice elementaire
* RST,CST super-tableau version reelle et complexe
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPTRM
      INTEGER       NBNEUL,NNEITL,NUNEFL(*),RGNELI(*),NCPINL
     &             ,NBNEUC,NNEITC,NUNEFC(*),RGNECO(*),NCPINC
     &             ,MCMATR,MCMATC,MCTELM,MCDIAG,MCTRIL,NBNEMI
     &             ,PTRDBL(*),PTRBMO(NBNEUL,*),LGLIBM(*)
      REAL          RST(*)
      COMPLEX       CST(*)
*
      INTEGER       MCDOEF,MCCOEF,MCPRDL,MCDBLI,ADMATA,INCREL,INCREC
     &             ,INEL,NELI,NGNELI,ICPL
     &             ,INEC,NECO,NGNECO,ICPC,MCDBDI,LGTRUE,NCPMIN,NCFSSD
*-----------------------------------------------------------------------
      INCREL=NCPINL*NNEITL
      INCREC=1
* 
      NCPMIN=MIN(NCPINL,NCPINC)
      NCFSSD=MAX(1,NCPMIN)
      IF (NCFSSD.LT.3) NCFSSD=NCFSSD-1
*
      IF (TYPTRM(1:1).EQ.'R') THEN
*
*        Cas d'une matrice reelle
*
         MCDOEF=MCMATR-1+MCTELM
*        Boucle sur les noeuds en ligne de l'element
         DO 209 INEL=1,NNEITL
            NELI=INEL
            IF (NNEITL.NE.NBNEUL) NELI=NUNEFL(INEL)
            NGNELI=RGNELI(NELI)
            LGTRUE=NCPINC*LGLIBM(NELI)
            MCDBLI=MCTRIL+NCPINL*NCPINC*(PTRDBL(NELI)-1)
     &                   +NCFSSD*MIN (NGNELI-1,NBNEMI)
            MCDBDI=MCDIAG+NCPMIN*(NGNELI-1)
*           Boucle les composantes de l'inconnue en ligne au noeud
            DO 203 ICPL=1,NCPINL
               MCCOEF=MCDOEF
*              Boucle sur les noeuds en colonne de l'element
               DO 202 INEC=1,NNEITC
                  NECO=INEC
                  IF (NNEITC.NE.NBNEUC) NECO=NUNEFC(INEC)
                  NGNECO=RGNECO(NECO)
                  IF (NGNELI.LT.NGNECO) THEN
*                    Bloc de coefficients sur-diagonaux
                     MCCOEF=MCCOEF+NCPINC*INCREL
                  ELSEIF (NGNELI.EQ.NGNECO) THEN
*                    Coefficients sous-diagonaux correspondant a un bloc
*                    diagnal de la matrice elementaire contenant un
*                    coefficient diagonal
                     ADMATA=MCDBLI+LGTRUE
                     DO 200 ICPC=1,ICPL-1
                        RST(ADMATA)=RST(ADMATA)+RST(MCCOEF)
                        MCCOEF=MCCOEF+INCREL
                        ADMATA=ADMATA+1
200                  CONTINUE
*                    Coefficient diagonal
                     RST(MCDBDI)=RST(MCDBDI)+RST(MCCOEF)
                     MCDBDI=MCDBDI+1
                     MCCOEF=MCCOEF+(NCPINC-ICPL+1)*INCREL
                  ELSE
*                    Bloc de Coefficients sous-diagonaux
                     MCPRDL=MCDBLI+NCPINC*(PTRBMO(NELI,NECO)-1)
                     DO 201 ADMATA=MCPRDL,MCPRDL+NCPINC-1 
                        RST(ADMATA)=RST(ADMATA)+RST(MCCOEF)
201                     MCCOEF=MCCOEF+INCREL
                  ENDIF
202            CONTINUE
               MCDBLI=MCDBLI+LGTRUE
               IF (NGNELI.LE.NBNEMI) MCDBLI=MCDBLI+ICPL-1
               MCDOEF=MCDOEF+INCREC
203         CONTINUE
209      CONTINUE
*
      ELSE
*
*        Cas d'une matrice complexe
*
         MCDOEF=MCMATC-1+MCTELM
         DO 409 INEL=1,NNEITL
            NELI=INEL
            IF (NNEITL.NE.NBNEUL) NELI=NUNEFL(INEL)
            NGNELI=RGNELI(NELI)
            LGTRUE=NCPINC*LGLIBM(NELI)
            MCDBLI=MCTRIL+NCPINL*NCPINC*(PTRDBL(NELI)-1)
     &                          +NCFSSD*MIN (NGNELI-1,NBNEMI)
            MCDBDI=MCDIAG+NCPMIN*(NGNELI-1)
*
            DO 403 ICPL=1,NCPINL
               MCCOEF=MCDOEF
*
               DO 402 INEC=1,NNEITC
                  NECO=INEC
                  IF (NNEITC.NE.NBNEUC) NECO=NUNEFC(INEC)
                  NGNECO=RGNECO(NECO)
                  IF (NGNELI.LT.NGNECO) THEN
*
                     MCCOEF=MCCOEF+NCPINC*INCREL
                  ELSEIF (NGNELI.EQ.NGNECO) THEN
*
                     ADMATA=MCDBLI+LGTRUE
                     DO 400 ICPC=1,ICPL-1
                        CST(ADMATA)=CST(ADMATA)+CST(MCCOEF)
                        MCCOEF=MCCOEF+INCREL
                        ADMATA=ADMATA+1
400                  CONTINUE
*
                     CST(MCDBDI)=CST(MCDBDI)+CST(MCCOEF)
                     MCDBDI=MCDBDI+1
                     MCCOEF=MCCOEF+(NCPINC-ICPL+1)*INCREL
                  ELSE
*
                     MCPRDL=MCDBLI+NCPINC*(PTRBMO(NELI,NECO)-1)
                     DO 401 ADMATA=MCPRDL,MCPRDL+NCPINC-1
                        CST(ADMATA)=CST(ADMATA)+CST(MCCOEF)
401                     MCCOEF=MCCOEF+INCREL
                  ENDIF
402            CONTINUE
               MCDBLI=MCDBLI+LGTRUE
               IF (NGNELI.LE.NBNEMI) MCDBLI=MCDBLI+ICPL-1
               MCDOEF=MCDOEF+INCREC
403         CONTINUE
409      CONTINUE
      ENDIF
                                                                    END !PavLbm
