*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE PAVMCO (NBNEUL,NNEITL,NUNEFL,NCPINL
     &                  ,NBNEUC,NNEITC,NUNEFC,NCPINC
     &                  ,MCMATR,MCMATC,MCTELM,TYPTRM,MCTERM
     &                  ,PTRDMO,PTRMOR,LGCOMO,RST,CST)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Mars 1992)
* Derniere modification : D.Martin (18 Mars 2002)
* Version 1.0.2
*
* Assemblage d'une matrice elementaire dans la matrice
* d'un terme stocke Morse-Colonne
* pour des inconnues vectorielles en ligne ou en colonne
*
*-- Arguments d'entree --
* NBNEUL nombre des noeuds en ligne dans l'element
* NNEITL nombre de lignes de la matrice elementaire comptees en noeuds
* NUNEFL correspondance noeud de face - noeud dans l'element en ligne
*
* NCPINL nombre de composantes de l'inconnue en ligne
* NBNEUC nombre de noeuds en colonne dans l'element
* NNEITC nombre de colonnes de la matrice elementaire comptees en noeuds
* NUNEFC correspondance noeud de face - noeud dans l'element en colonne
*
* NCPINC nombre de composantes de l'inconnue en colonne
* MCMATR adresse absolue du tableau des matrices elementaires reelles
* MCMATC adresse absolue du tableau des matrices elementaires complexes
* MCTELM adresse relative de la matrice elementaire
* TYPTRM type REEL ou COMPLEXE de la matrice assembles
* MCTERM adresse absolue du terme assemble
* PTRDMO tableau des adresses de debut des colonnes de la matrice assemblee
* LGCOMO tableau des longueurs des colonnes de la matrice assemblee
* PTRMOR tableau des adresses a partir du debut de la ligne dans la matrice
*        assemblee des coefficients d'une matrice elementaire
* RST,CST super-tableau version reelle et complexe
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPTRM
      INTEGER       NBNEUL,NNEITL,NUNEFL(*),NCPINL
     &             ,NBNEUC,NNEITC,NUNEFC(*),NCPINC
     &             ,MCMATR,MCMATC,MCTELM,MCTERM
     &             ,PTRDMO(*),PTRMOR(NBNEUL,*),LGCOMO(*)
      REAL          RST(*)
      COMPLEX       CST(*)
*
      INTEGER       MCDOEF,MCCOEF,MCPRDL,MCDBCO,ADMATA,INCREL,INCREC
     &             ,INEL,NELI,INEC,NECO,ICPC
*-----------------------------------------------------------------------
      INCREL=NCPINL*NNEITL
      INCREC=1
*
      IF (TYPTRM(1:1).EQ.'R') THEN
*
*        Cas d'une matrice reelle
*
         MCDOEF=MCMATR-1+MCTELM
*        Boucle sur les noeuds en colonne de l'element
         DO 209 INEC=1,NNEITC
            NECO=INEC
            IF (NNEITC.NE.NBNEUC) NECO=NUNEFC(INEC)
            MCDBCO=MCTERM+NCPINC*NCPINL*(PTRDMO(NECO)-1)
*           Composantes de l'inconnue au noeud en colonne
            DO 203 ICPC=1,NCPINC
*              Noeuds en ligne de l'element
               MCCOEF=MCDOEF
               DO 202 INEL=1,NNEITL
                  NELI=INEL
                  IF (NNEITL.NE.NBNEUL) NELI=NUNEFL(INEL)
*
                  MCPRDL=MCDBCO+NCPINL*(PTRMOR(NELI,NECO)-1)
*                 Composantes de l'inconnue au noeud  en ligne
                  DO 201 ADMATA=MCPRDL,MCPRDL+NCPINL-1
                     RST(ADMATA)=RST(ADMATA)+RST(MCCOEF)
*                    Changement de ligne dans la matrice elementaire 
                     MCCOEF=MCCOEF+INCREC
201               CONTINUE
202            CONTINUE
*              Changement de colonne dans la matrice elementaire
               MCDOEF=MCDOEF+INCREL
*              Changement de colonne dans la matrice assemblee colonne
*              apres colonne (Morse-C)
               MCDBCO=MCDBCO+LGCOMO(NECO)*NCPINL
203         CONTINUE
209      CONTINUE
*
      ELSE
*
*        Cas d'une matrice complexe
*
         MCDOEF=MCMATC-1+MCTELM
         DO 409 INEC=1,NNEITC
            NECO=INEC
            IF (NNEITC.NE.NBNEUC) NECO=NUNEFC(INEC)
            MCDBCO=MCTERM+NCPINC*NCPINL*(PTRDMO(NECO)-1)
            DO 403 ICPC=1,NCPINC
               MCCOEF=MCDOEF
               DO 402 INEL=1,NNEITL
                  NELI=INEL
                  IF (NNEITL.NE.NBNEUL) NELI=NUNEFL(INEL)
                  MCPRDL=MCDBCO+NCPINL*(PTRMOR(NELI,NECO)-1)
                  DO 401 ADMATA=MCPRDL,MCPRDL+NCPINL-1
                     CST(ADMATA)=CST(ADMATA)+CST(MCCOEF)
                     MCCOEF=MCCOEF+INCREC
401               CONTINUE
402            CONTINUE
               MCDOEF=MCDOEF+INCREL
               MCDBCO=MCDBCO+LGCOMO(NECO)*NCPINL
403         CONTINUE
409      CONTINUE
      ENDIF
                                                                    END !Pavmco
