*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE PAVMLI (NBNEUL,NNEITL,NUNEFL,NCPINL
     &                  ,NBNEUC,NNEITC,NUNEFC,NCPINC
     &                  ,MCMATR,MCMATC,MCTELM,TYPTRM,MCTERM
     &                  ,PTRDMO,PTRMOR,LGLIMO,RST,CST)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Mars 1992) 
* Derniere modification : D.Martin (25 septembre 2005)
* Version 2
*
* Assemblage d'une matrice elementaire dans la matrice
* d'un terme stocke Morse-Ligne
* pour des inconnues vectorielles en ligne ou en colonne
*  
*-- Arguments d'entree --
* NBNEUL nombre des noeuds en ligne dans l'element
* NNEITL nombre de lignes de la matrice elementaire comptees en noeuds
* NUNEFL correspondance noeud de face - noeud dans l'element en ligne
*
* NCPINL nombre de composantes de l'inconnue en ligne
* NBNEUC nombre de noeuds en colonne dans l'element
* NNEITC nombre de colonnes de la matrice elementaire comptees en noeuds  
* NUNEFC correspondance noeud de face - noeud dans l'element en colonne
*
* NCPINC nombre de composantes de l'inconnue en colonne
* MCMATR adresse absolue du tableau des matrices elementaires reelles
* MCMATC adresse absolue du tableau des matrices elementaires complexes
* MCTELM adresse relative de la matrice elementaire 
* TYPTRM type REEL ou COMPLEXE du terme assemble
* MCTERM adresse absolue de la matrice assembles
* PTRDMO tableau des adresses de debut des lignes de la matrice assemblee
* LGLIMO tableau des longueurs des lignes de la matrice assemblee
* PTRMOR tableau des adresses a partir du debut de la ligne dans la matrice
*        assemblee des coefficients d'une matrice elementaire
* RST,CST super-tableau version reelle et complexe
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPTRM
      INTEGER       NBNEUL,NNEITL,NUNEFL(*),NCPINL
     &             ,NBNEUC,NNEITC,NUNEFC(*),NCPINC
     &             ,MCMATR,MCMATC,MCTELM,MCTERM
     &             ,PTRDMO(*),PTRMOR(NBNEUL,*),LGLIMO(*)
      REAL          RST(*)
      COMPLEX       CST(*)
*
      INTEGER       MCDOEF,MCCOEF,MCPRDL,MCDBLI,ADMATA,INCREL,INCREC
     &             ,INEL,NELI,INEC,NECO,ICPL
*-----------------------------------------------------------------------
      INCREL=NCPINL*NNEITL
      INCREC=1
*
      IF (TYPTRM(1:1).EQ.'R') THEN
*
*        Cas d'une matrice reelle
*
         MCDOEF=MCMATR-1+MCTELM
*        Boucle sur les noeuds en ligne de l'element
         DO 205 INEL=1,NBNEUL
            NELI=INEL
            IF (NNEITL.NE.NBNEUL) NELI=NUNEFL(INEL)
            MCDBLI=MCTERM+NCPINL*NCPINC*(PTRDMO(NELI)-1)
*           Composantes de l'inconnue au noeud  en ligne 
            DO 204 ICPL=1,NCPINL
               MCCOEF=MCDOEF
*              Noeuds en colonne de l'element
               DO 201 INEC=1,NBNEUC
                  NECO=INEC
                  IF (NNEITC.NE.NBNEUC) NECO=NUNEFC(INEC)
                  MCPRDL=MCDBLI+NCPINC*(PTRMOR(NELI,NECO)-1)
*                 Composantes de l'inconnue au noeud en colonne
                  DO 200 ADMATA=MCPRDL,MCPRDL+NCPINC-1
                     RST(ADMATA)=RST(ADMATA)+RST(MCCOEF)
*                    Changement de colonne dans la matrice elementaire  
                     MCCOEF=MCCOEF+INCREL
200               CONTINUE
201            CONTINUE
*              Changement de ligne dans la matrice elementaire
               MCDOEF=MCDOEF+INCREC
*              Changement de ligne dans la matrice assemblee ligne 
*              apres ligne (Morse-L)
               MCDBLI=MCDBLI+LGLIMO(NELI)*NCPINC
204         CONTINUE
205      CONTINUE
*
      ELSE
*
*        Cas d'une matrice complexe
*
         MCDOEF=MCMATC-1+MCTELM
         DO 405 INEL=1,NBNEUL
            NELI=INEL
            IF (NNEITL.NE.NBNEUL) NELI=NUNEFL(INEL)
            MCDBLI=MCTERM+NCPINL*NCPINC*(PTRDMO(NELI)-1)
*           Boucle les composantes de l'inconnue en ligne au noeud
            DO 404 ICPL=1,NCPINL
               MCCOEF=MCDOEF
*              Boucle sur les noeuds en colonne de l'element
               DO 401 INEC=1,NBNEUC
                  NECO=INEC
                  IF (NNEITC.NE.NBNEUC) NECO=NUNEFC(INEC)
                  MCPRDL=MCDBLI+NCPINC*(PTRMOR(NELI,NECO)-1)
                  DO 400 ADMATA=MCPRDL,MCPRDL+NCPINC-1
                     CST(ADMATA)=CST(ADMATA)+CST(MCCOEF)
                     MCCOEF=MCCOEF+INCREL
400               CONTINUE
401            CONTINUE
               MCDBLI=MCDBLI+LGLIMO(NELI)*NCPINC
               MCDOEF=MCDOEF+INCREC
404         CONTINUE
405      CONTINUE
      ENDIF
                                                                    END !PavMli
