*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE PAVUBM (NBNEUL,NNEITL,NUNEFL,RGNELI,NCPINL
     &                  ,NBNEUC,NNEITC,NUNEFC,RGNECO,NCPINC,NBNEMI
     &                  ,MCMATR,MCMATC,MCTELM,TYPTRM,MCTRIU
     &                  ,PTRDBU,LGCOBM,PTRBMO,RST,CST)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin & O.DeBayser (Mars 1992) 
* Derniere modification : D.Martin (5 Octobre 2001)
* Version 1.0.2
*  
* Assemblage d'une matrice elementaire dans la partie triangulaire superieure
* de la matrice d'un terme stocke 'Bi-Morse' 
* pour des inconnues vectorielles en ligne ou colonne
*  
*-- Arguments d'entree --
* NBNEUL nombre des noeuds en ligne dans l'element
* NNEITL nombre de lignes comptees en noeuds de la matrice elementaire
* NUNEFL correspondance noeud de face - noeud dans l'element en ligne
* RGNELI RGNELI(ne) est la ligne de la matrice assemblee associee au noeud ne
* NCPINL nombre de composantes de l'inconnue  en ligne
* NBNEUC nombre de noeuds en colonne dans l'element
* NNEITC nombre de colonnes comptees en noeuds de la matrice elementaire  
* NUNEFC correspondance noeud de face - noeud dans l'element en colonne
* RGNECO RGNECO(ne) est la colonne de la matrice assemblee associee au noeud ne
* NCPINC nombre de composantes de l'inconnue  en colonne
* NBNEMI nombre de noeuds minimum en ligne ou en colonne en numerot. globale
* MCMATR adresse absolue du tableau des matrices elementaires reelles
* MCMATC adresse absolue du tableau des matrices elementaires complexes
* MCTELM adresse relative de la matrice elementaire 
* TYPTRM type REEL ou COMPLEXE du terme
* MCTRIU adresse abs. de la partie triangulaire inf. du terme Bi-Morse assemble
* PTRDBU tableau des adresses de debut des lignes de la matrice compactee
* LGCOBM tableau des longueurs des lignes de la matrice compactee
* PTRBMO tableau des adresses a partir du debut de la ligne dans la matrice
*        Bi-Morse des coefficients d'une matrice elementaire
* RST,CST super-tableau version reelle et complexe
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPTRM
      INTEGER       NBNEUL,NNEITL,NUNEFL(*),RGNELI(*),NCPINL
     &             ,NBNEUC,NNEITC,NUNEFC(*),RGNECO(*),NCPINC
     &             ,NBNEMI,MCMATR,MCMATC,MCTELM,MCTRIU
     &             ,PTRDBU(*),PTRBMO(NBNEUL,*),LGCOBM(*)
      REAL          RST(*)
      COMPLEX       CST(*)
*
      INTEGER       MCDOEF,MCCOEF,MCPRDL,MCDBCO,ADMATA
     &             ,INEL,NELI,NGNELI,ICPL
     &             ,INEC,NECO,NGNECO,ICPC,LGTRUE,NCPMIN,NCFSSD
*-----------------------------------------------------------------------
      NCPMIN=MIN(NCPINL,NCPINC)
      NCFSSD=MAX(1,NCPMIN)
      IF (NCFSSD.LT.3) NCFSSD=NCFSSD-1
*
      IF (TYPTRM(1:1).EQ.'R') THEN
*
*        Cas d'une matrice reelle
*
         MCCOEF=MCMATR-1+MCTELM
*        Boucle sur les noeuds en colonne de l'element
         DO 209 INEC=1,NNEITC
            NECO=INEC
            IF (NNEITC.NE.NBNEUC) NECO=NUNEFC(INEC)
            NGNECO=RGNECO(NECO)
            LGTRUE=NCPINL*LGCOBM(NECO)
            MCDBCO=MCTRIU+NCPINC*NCPINL*(PTRDBU(NECO)-1)
     &                   +NCFSSD*MIN((NGNECO-1),NBNEMI)
*           Composantes de l'inconnue en colonne au noeud
            DO 203 ICPC=1,NCPINC 
*              Boucle sur les noeuds en ligne de l'element
               DO 202 INEL=1,NNEITL
                  NELI=INEL
                  IF (NNEITL.NE.NBNEUL) NELI=NUNEFL(INEL)
*
                  NGNELI=RGNELI(NELI)
                  IF (NGNECO.LT.NGNELI) THEN
*                    Bloc de coefficients sous-diagonaux
                     MCCOEF=MCCOEF+NCPINL
                  ELSEIF (NGNECO.EQ.NGNELI) THEN
*                    Coefficient sur-diagonaux correspondant a un bloc
*                    diagonal de la matrice elementaire
*
                     ADMATA=MCDBCO+LGTRUE
                     DO 200 ICPL=1,ICPC-1
                        RST(ADMATA)=RST(ADMATA)+RST(MCCOEF)
                        MCCOEF=MCCOEF+1
                        ADMATA=ADMATA+1
200                  CONTINUE
*
                     MCCOEF=MCCOEF+NCPINL-ICPC+1 
                  ELSE
                     MCPRDL=MCDBCO+NCPINL*(PTRBMO(NELI,NECO)-1)
                     DO 201 ADMATA=MCPRDL,MCPRDL+NCPINL-1 
                        RST(ADMATA)=RST(ADMATA)+RST(MCCOEF) 
                        MCCOEF=MCCOEF+1
201                  CONTINUE
                  ENDIF
202            CONTINUE
               MCDBCO=MCDBCO+LGTRUE
               IF (NGNECO.LE.NBNEMI) MCDBCO=MCDBCO+ICPC-1
203         CONTINUE
209      CONTINUE
*
      ELSE
*
*        Cas d'une matrice complexe
*
         MCCOEF=MCMATC-1+MCTELM
         DO 409 INEC=1,NNEITC
            NECO=INEC
            IF (NNEITC.NE.NBNEUC) NECO=NUNEFC(INEC)
            NGNECO=RGNECO(NECO)
            LGTRUE=NCPINL*LGCOBM(NECO)
            MCDBCO=MCTRIU+NCPINC*NCPINL*(PTRDBU(NECO)-1)
     &                          +NCFSSD*MIN((NGNECO-1),NBNEMI)
* 
            DO 403 ICPC=1,NCPINC
*
               DO 402 INEL=1,NNEITL
                  NELI=INEL
                  IF (NNEITL.NE.NBNEUL) NELI=NUNEFL(INEL)
                  NGNELI=RGNELI(NELI)
                  IF (NGNECO.LT.NGNELI) THEN
                     MCCOEF=MCCOEF+NCPINL
                  ELSEIF (NGNECO.EQ.NGNELI) THEN
*
                     ADMATA=MCDBCO+LGTRUE
                     DO 400 ICPL=1,ICPC-1
                        CST(ADMATA)=CST(ADMATA)+CST(MCCOEF)
                        MCCOEF=MCCOEF+1
                        ADMATA=ADMATA+1
400                  CONTINUE
*
                     MCCOEF=MCCOEF+NCPINL-ICPC+1
                  ELSE
                     MCPRDL=MCDBCO+NCPINL*(PTRBMO(NELI,NECO)-1)
                     DO 401 ADMATA=MCPRDL,MCPRDL+NCPINL-1
                        CST(ADMATA)=CST(ADMATA)+CST(MCCOEF)
                        MCCOEF=MCCOEF+1
401                  CONTINUE
                  ENDIF
402            CONTINUE
               MCDBCO=MCDBCO+LGTRUE
               IF (NGNECO.LE.NBNEMI) MCDBCO=MCDBCO+ICPC-1
403         CONTINUE
409      CONTINUE
      ENDIF
                                                                    END !PavUbm
