*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE CALNOY (SIGMA,GAMMA,NBMORG,KLNOYO,NBDONG,NMDONG,NMLAMB
     &                  ,NMGREN,NVGREN,NMDGRE,NVDGRE)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin & O.DeBayser & E.Masson (Decembre 1989)
* Derniere modification : D.Martin (30 Mai 2000)
* Version # 1.0.1
*  
* Calcul de la fonction de green et de ses derivees, puis des noyaux
* avec prise en compte des eventuelles symetries (version COMPLEXE)
*  
*-- Arguments d'entree -- 
* SIGMA  nom du domaine de couplage
* GAMMA  nom du domaine portant la representation integrale
* NBMORG nombre de morceaux de GAMMA  
* KLNOYO type de noyau ('DIRichlet', 'NEUmann' ou 'FOUrier')  
* NBDONG nombre de donnees associees au calcul de la fonction de Green  
* NMDONG tableau des noms des donnees associees a la fonction de Green  
* NMLAMB nom de la constante associee au calcul du noyau  
* NMGREN nom des tableaux des valeurs de g (noyau du terme de double couche)  
* NVGREN niveau du premier des tableaux des valeurs de g  
*        (les niveaux des autres tableaux sont consecutifs)
* NMDGRE nom des tab. des valeurs de la derivee normale de g (simple couche)  
* NVGREN niveau du premier des tableaux des valeurs de la derivee normale de g
*        (les niveaux des autres tableaux sont consecutifs)
*-- Tableaux remplis en sortie -- 
* Les tableaux de noms NMGREN,NMDGRE
*  
*-- Routines utilisees
* SYMGRE calcul de la fonction de Green et de ses derivees   
*        avec prise en compte des (anti/)symetries.
* GREEN  calcul de la fonction de Green et de ses derivees
* NOYDIR noyaux de simple et double couche pour le couplage de type Dirichlet
* NOYNEU noyaux de simple et double couche pour le couplage de type Neuman
* NOYFOU noyaux de simple et double couche pour le couplage de type Fourier.
*  
*-- Remarques --  
* - La routine ADIM (adimensionnalisation) peut avoir ete appelee ;
*   dans ce cas le tableau de coordonnees est #ORNOA et non plus #ORNOE  
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) SIGMA,GAMMA,NMGREN,NMDGRE,KLNOYO,NMLAMB,NMDONG(*)
      INTEGER       NBMORG,NBDONG,NVGREN,NVDGRE  
*  
      INCLUDE 'ALLOC'
      INCLUDE 'CONTEX'
*
      INTEGER       INTDON,INTTYP,KELCHN
      INTEGER       INTCON,ITENTI,ITREEL,ITCOMP,ITFGRE,ITNOYO,ITLAMB
     &             ,MCMDOM,MCTRDO,MCDTRM,LGETRM,NBTERM,INCTRM,NCHTRM
     &             ,NUDOSI,IATRSI,NUDGSI,IGEOME,INDMIX,NUSCHQ
     &             ,INDEXC,IDLHRC,INTRSC,NBNECX,NBNEEX,NBDLEX,NBNEFX
     &             ,NBDLFX,NVNUSC,INDEXL,IDLHRL,INTRSL,NBNECY,NBNEEY
     &             ,NBDLEY,NBNEFY,NBDLFY,NVNUSL,NVNOSC,INUTIL,NBTRDO
     &             ,NIVIMS,NUDOGA,IATRGA,NUDGGA,INTRGC,NVNUGC
     &             ,INTRGL,NVNUGL,NVNOGL,NIVIMG,NIVIMP
     &             ,MCNUSC,ADNUSC,MCCOSC,ADCOSC,MCNOSC,ADNOSC
     &             ,MCNUGL,ADNUGL,MCCOGL,ADCOGL,MCNOGL,ADNOGL
     &             ,LAMBDE,NUMORG,NBCOGR,NBLIGR,NBCOMO,NBCOLA
     &             ,IDLGAM,IDLSIG,NUCOGL,NVTBAS,MCTBAS,ITTBAS,LGWOR2
     &             ,NWGREN,NWDGRE,MCGREN,MCDGRE,MCFGR4,MCEGRE,MCCST2
     &             ,I,NIVIM1,NIVIM2,NCATR1,NCATR2,LGMORE,LGFGRE
     &             ,MCESTE,MCRSTE,MCCSTE,NUCSTE,ITYDON,NIVDON,ITYPDO
     &             ,MCMDON,MCDONN,LGEDON,NBDONN,INCDON,NCHDON,NUDONN
      REAL          LAMBDR
      COMPLEX       LAMBDC
      CHARACTER     TYPINT*4
      CHARACTER     ERCODE*120,NMCOOR*6,NMTBAS*6,TYFGRE*4
      COMMON/FORMAH/ERCODE 
      EQUIVALENCE  (ERCODE(89:89),NMTBAS),(ERCODE(105:105),NMCOOR)
      EQUIVALENCE  (LAMBDE,LAMBDR,LAMBDC)
*
*     Tableaux pour les symetries dans le calcul de la fonction de Green
*     de dimension 3 quelle que soit la dimension d'espace
      INTEGER       TBISYM(3)
      REAL          TBPSYM(3)
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*CalNoy*')
*  
      CALL ALFANU (NBCHNT,NMGREN)                                       !utilite
      CALL ALFANU (NBCHNT,NMDGRE)                                       !utilite
      INTCON = INTDON ('CONSTANTE')                                     !utilite
      ITENTI = INTTYP ('ENTIER')                                        !utilite
      ITREEL = INTTYP ('REEL')                                          !utilite
      ITCOMP = INTTYP ('COMPLEXE')                                      !utilite
      ITFGRE = ITCOMP
      ITNOYO = ITFGRE
      TYFGRE = TYPINT (ITFGRE)                                          !utilite
*  
      CALL TBAR3  (ERCODE,'#OMDOM',1,MCMDOM,'#TERDO',1,MCTRDO
     &                   ,'$SDTRM',1,MCDTRM)                     
      CALL SDEXDB (IST(MCDTRM),LGETRM,NBTERM,INCTRM,NCHTRM)             !utilite
*  
*     Caracteristiques du domaine de couplage (SIGMA)  
*  
      CALL KLDOMN (SIGMA,0,-1,IST(MCTRDO),AST(MCMDOM),NBDOMG
     &            ,NUDOSI,IATRSI)                                       !sdexplo
      IATRSI = MCTRDO+IATRSI 
      CALL GETDOM (IST(IATRSI),NUDGSI,NUDOSI,IGEOME,INDMIX,NUSCHQ
     &            ,INDEXC,IDLHRC,INTRSC,NBNECX,NBNEEX,NBDLEX,NBNEFX
     &            ,NBDLFX,NVNUSC
     &            ,INDEXL,IDLHRL,INTRSL,NBNECY,NBNEEY,NBDLEY,NBNEFY
     &            ,NBDLFY,NVNUSL,NVNOSC,INUTIL,NBTRDO,NIVIMS)           !sdexplo
*  
*     Caracteristiques du domaine de representation integrale (GAMMA)
*  
      CALL KLDOMN (GAMMA,-1,0,IST(MCTRDO),AST(MCMDOM),NBDOMG
     &            ,NUDOGA,IATRGA)                                       !sdexplo
      IATRGA = MCTRDO+IATRGA
      CALL GETDOM (IST(IATRGA),NUDGGA,NUDOGA,IGEOME,INDMIX,NUSCHQ
     &            ,INDEXC,IDLHRC,INTRGC,NBNECX,NBNEEX,NBDLEX,NBNEFX
     &            ,NBDLFX,NVNUGC
     &            ,INDEXL,IDLHRL,INTRGL,NBNECY,NBNEEY,NBDLEY,NBNEFY
     &            ,NBDLFY,NVNUGL,NVNOGL,INUTIL,NBTRDO,NIVIMG)           !sdexplo
*  
*     Tableaux de coordonnees (adimensionalises ou non)  
*
      NIVIMP = MAX (NIVIMS,NIVIMG)
      NMCOOR = '#ORNOE'  
      ERCODE(1:1) = '!'  
      CALL TBRR4  (ERCODE,'#ORNOA',INTRSC,MCCOSC,'#ORNOE',INTRSC,MCCOSC
     &                   ,'#GNEDO',NVNUSC,MCNUSC,'#GNEDO',NVNUGL,MCNUGL)
      IF (ERCODE(1:1).EQ.' ') THEN
         CALL TBSAVE ('#ORNOE',INTRSC)
         NMCOOR = '#ORNOA'
      ENDIF
      IF (INTRGL.NE.INTRSC) CALL TBRR1  (ERCODE,NMCOOR,INTRGL,MCCOGL)
*
*     Tableaux des normales
*
      CALL TBRR1  (ERCODE,'&NORMA',NVNOGL,ADNOGL)
      IF (KLNOYO(:3).NE.'DIR')  
     &   CALL TBRR1  (ERCODE,'&NORMA',NVNOSC,MCNOSC)
*  
*--   Valeurs des constantes associees a la fonction de Green
*     placees dans $WORK$/2 quels que soient leur type
*  
      NMTBAS = ' '
      NVTBAS = 0
      LGWOR2 = MAX (1,NBDONG)
      CALL TBCREE ('$WORK$',2,ITCOMP,LGWOR2,'c')
      CALL TBAR6  (ERCODE,'$WORK$',2,MCCST2,'#OMDON',1,MCMDON
     &                   ,'$DONNE',1,MCDONN,'$ECSTE',1,MCESTE
     &                   ,'$RCSTE',1,MCRSTE,'$CCSTE',1,MCCSTE)
      CALL SDEXDB (IST(MCDONN),LGEDON,NBDONN,INCDON,NCHDON)             !utilite
*
      DO 10 I = 1,NBDONG
         NUDONN = KELCHN (NMDONG(I),AST(MCMDON),NBDONN,NCHDON)          !utilite
         IF (NUDONN.LE.0) CALL ERDONN (1,NMDONG(I))                     !utilite
         CALL SDEXCO (NUDONN,IST(MCDONN),ITYDON,NIVDON,ITYPDO)          !utilite
         IF (ITYDON.EQ.INTCON) THEN
            IF (ITYPDO.EQ.ITCOMP) THEN
               CST(MCCST2-1+I) = CST(MCCSTE+NIVDON)
            ELSEIF (ITYPDO.EQ.ITREEL) THEN
               CST(MCCST2-1+I) = RST(MCRSTE+NIVDON)
            ELSEIF (ITYPDO.EQ.ITENTI) THEN
               CST(MCCST2-1+I) = IST(MCESTE+NIVDON)
            ENDIF 
         ELSEIF (ITYDON.EQ.INTDON ('TABLEAU')) THEN                     !utilite
            NMTBAS = NMDONG(I)
            NVTBAS = NIVDON
            ITTBAS = ITYPDO
            IF (ITTBAS.NE.ITCOMP) GOTO 99991
         ENDIF
10    CONTINUE
*
*--   Constante de couplage Lambda (cas KLNOYO='FOUrier')
*     Sa valeur est placee dans la variable LAMBDC complexe quelque soit
*     le type de la donnee constante LAMBDA 
*
      LAMBDC = 0.
      IF (KLNOYO(1:3).EQ.'FOU')  THEN
         NUDONN = KELCHN (NMLAMB,AST(MCMDON),NBDONN,NCHDON)             !utilite
         IF (NUDONN.LE.0) CALL ERDONN (1,NMLAMB)                        !utilite
         CALL TBAR1  (ERCODE,'$DONNE',1,MCDONN)
         CALL SDEXCO (NUDONN,IST(MCDONN),ITYDON,NUCSTE,ITLAMB)          !utilite
         CALL OUTCST (NUCSTE,ITLAMB,LAMBDE,LAMBDR,LAMBDC,ERCODE)        !sdexplo
         IF (ITLAMB.EQ.ITCOMP) ITNOYO = ITCOMP
      ENDIF
*
      IF (NMTBAS.NE.' ')
     &   CALL TBRR1  (ERCODE,NMTBAS,NVTBAS,MCTBAS)
*
*     Decodage de ISYMGR (Symetries pour la fonction de Green)   
      IF (ISYMGR.GT.0) CALL KLSYME (NDIM,ISYMGR,TBISYM)                 !sdexplo
*
*     Tableau $WORK$,4 pour les coefficients de la fonction de Green
*
      LGFGRE = 1+NDIM+NDIM+NDIM*NDIM 
*      IF (KLNOYO(:3).NE.'DIR') LGFGRE = LGFGRE+NDIM*NDIM 
      LGMORE = 0
      IF (ISYMGR.GT.0) LGMORE = LGMORE+LGFGRE
      CALL TBCREE ('$WORK$',4,ITFGRE,LGFGRE+LGMORE,'c')
*  
*     Evaluation des tailles des tableaux a creer  
*     Tailles du morceau courant et du dernier morceau d'une matrice noyau
* 
      CALL TBAR2  (ERCODE,'#GNEDO',NVNUSC,MCNUSC,'#GNEDO',NVNUGL,MCNUGL)
      NBCOGR = IST(MCNUGL+1)
      NBLIGR = IST(MCNUSC+1)
      NBCOMO = (NBCOGR-1)/NBMORG+1
      NBCOLA = NBCOGR-(NBMORG-1)*NBCOMO
*
*                   Boucle sur les morceaux de GAMMA
*                   --------------------------------
      IDLGAM = 1
      NWGREN = NVGREN  
      NWDGRE = NVDGRE  
      DO 50 NUMORG = 1,NBMORG 
         IF (NUMORG.EQ.NBMORG) NBCOMO = NBCOLA 
         CALL TBCREE (NMGREN,NWGREN,ITNOYO,NBLIGR*NBCOMO,'c')
         CALL TBCREE (NMDGRE,NWDGRE,ITNOYO,NBLIGR*NBCOMO,'c')
         IF (NIVIMS.GT.1) 
     &      WRITE (IMPSDR,10001) NUMORG,NBLIGR,NBCOMO
         ERCODE(1:1) = '!'
         CALL TBAR8  (ERCODE,NMCOOR,INTRSC,MCCOSC,'#GNEDO',NVNUSC,MCNUSC
     &                    ,'#GNEDO',NVNUGL,MCNUGL,'&NORMA',NVNOGL,MCNOGL
     &                    , NMGREN ,NWGREN,MCGREN, NMDGRE ,NWDGRE,MCDGRE
     &                    ,'$WORK$',4,MCFGR4,'$WORK$',2,MCCST2)
         MCCOGL = MCCOSC
         IF (INTRGL.NE.INTRSC) CALL TBAR1  (ERCODE,NMCOOR,INTRGL,MCCOGL)
         IF (KLNOYO(:3).NE.'DIR') 
     &                       CALL TBAR1  (ERCODE,'&NORMA',NVNOSC,MCNOSC)
         MCTBAS = 1
         IF (NMTBAS.NE.' ')   CALL TBAR1  (ERCODE,NMTBAS,NVTBAS,MCTBAS)
*
*        Boucle sur les points de GAMMA (Representation integrale)
*
         ADNUGL = MCNUGL+1+IDLGAM
         ADNOGL = MCNOGL+NDIM*(IDLGAM-1)
         NIVIM1 = NIVIMG
*
         IF (ISYMGR.GT.0) THEN
            MCEGRE = MCFGR4+LGFGRE
*
            DO 20 NUCOGL = 1,NBCOMO
               ADCOGL = MCCOGL+NDIM*(IST(ADNUGL)-1)
               IF (NIVIMG.LT.NUCOGL) NIVIM1 = 0 
               IF (NIVIM1.GT.2) WRITE (IMPSDR,10002) NUCOGL,IDLGAM 
     &                               ,(RST(I),I=ADNOGL,ADNOGL+NDIM-1)
               ADNUSC = MCNUSC+2
               NIVIM2 = NIVIM1
               IF (KLNOYO(1:3).EQ.'FOU') THEN
*  
*                 Calcul des noyaux pour le couplage 'FOURIER'
*
                  ADNOSC = MCNOSC
*  
*                 Boucle sur les points de SIGMA (Frontiere de couplage)
*
                  DO 21 IDLSIG = 1,NBLIGR 
                     IF (NIVIM2.LT.IDLSIG) NIVIM2 = 0
                     ADCOSC = MCCOSC+NDIM*(IST(ADNUSC)-1)
                     CALL TAZERO (LGFGRE,TYFGRE,MCFGR4,AST,IST,RST,CST) 
                     CALL SYMGRE (KLNOYO,NDIM,CST(MCCST2),RST(ADCOGL)
     &                           ,RST(ADCOSC),TBPSYM,TBISYM,LGFGRE
     &                           ,CST(MCFGR4),CST(MCEGRE),CST(MCTBAS))  !calgre
                     CALL NOYFOU (NDIM,RST(ADNOGL),RST(ADNOSC),LAMBDC
     &                           ,CST(MCFGR4),CST(MCGREN),CST(MCDGRE)
     &                           ,NIVIM2,IMPSDR)                        !calgre
                     MCGREN = MCGREN+1 
                     MCDGRE = MCDGRE+1
                     ADNUSC = ADNUSC+1
                     ADNOSC = ADNOSC+NDIM  
21                CONTINUE
*
               ELSEIF (KLNOYO(1:3).EQ.'NEU') THEN
*
*                 Calcul des noyaux pour le couplage 'NEUMANN'
*
                  ADNOSC = MCNOSC
                  DO 22 IDLSIG = 1,NBLIGR
                     IF (NIVIM2.LT.IDLSIG) NIVIM2 = 0
                     ADCOSC = MCCOSC+NDIM*(IST(ADNUSC)-1)
                     CALL TAZERO (LGFGRE,TYFGRE,MCFGR4,AST,IST,RST,CST) 
                     CALL SYMGRE (KLNOYO,NDIM,CST(MCCST2),RST(ADCOGL)
     &                           ,RST(ADCOSC),TBPSYM,TBISYM
     &                           ,LGFGRE,CST(MCFGR4),CST(MCEGRE)
     &                           ,CST(MCTBAS))                          !calgre
                     CALL NOYNEU (NDIM,RST(ADNOGL),RST(ADNOSC)
     &                           ,CST(MCFGR4),CST(MCGREN),CST(MCDGRE)
     &                           ,NIVIM2,IMPSDR)                        !calgre
                     MCGREN = MCGREN+1 
                     MCDGRE = MCDGRE+1
                     ADNUSC = ADNUSC+1
                     ADNOSC = ADNOSC+NDIM  
22                CONTINUE
*
               ELSEIF (KLNOYO(1:3).EQ.'DIR') THEN 
*
*                 Calcul des noyaux pour le couplage 'DIRICHLET'
* 
                  DO 23 IDLSIG = 1,NBLIGR
                     IF (NIVIM2.LT.IDLSIG) NIVIM2 = 0 
                     ADCOSC = MCCOSC+NDIM*(IST(ADNUSC)-1)
                     CALL TAZERO (LGFGRE,TYFGRE,MCFGR4,AST,IST,RST,CST) 
                     CALL SYMGRE (KLNOYO,NDIM,CST(MCCST2),RST(ADCOGL)
     &                           ,RST(ADCOSC),TBPSYM,TBISYM
     &                           ,LGFGRE,CST(MCFGR4),CST(MCEGRE)
     &                           ,CST(MCTBAS))                          !calgre
                     CALL NOYDIR (NDIM,RST(ADNOGL)
     &                           ,CST(MCFGR4),CST(MCGREN),CST(MCDGRE)
     &                           ,NIVIM2,IMPSDR)                        !calgre
                     MCGREN = MCGREN+1 
                     MCDGRE = MCDGRE+1
                     ADNUSC = ADNUSC+1
23                CONTINUE
               ENDIF
               ADNUGL = ADNUGL+1
               ADNOGL = ADNOGL+NDIM 
               IDLGAM = IDLGAM+1
20          CONTINUE
         ELSE
*
*           Cas sans symetrie
*
            DO 30 NUCOGL = 1,NBCOMO
               ADCOGL = MCCOGL+NDIM*(IST(ADNUGL)-1)
               IF (NIVIMG.LT.NUCOGL) NIVIM1 = 0 
               IF (NIVIM1.GT.2) WRITE (IMPSDR,10002) NUCOGL,IDLGAM  
     &                            ,(RST(I),I=ADNOGL,ADNOGL+NDIM-1)
               ADNUSC = MCNUSC+2
               NIVIM2 = NIVIM1
               IF (KLNOYO(1:3).EQ.'FOU') THEN
                  ADNOSC = MCNOSC
                  DO 31 IDLSIG = 1,NBLIGR
                     IF (NIVIM2.LT.IDLSIG) NIVIM2 = 0
                     ADCOSC = MCCOSC+NDIM*(IST(ADNUSC)-1)
                     CALL TAZERO (LGFGRE,TYFGRE,MCFGR4,AST,IST,RST,CST) 
                     CALL GREEN  (KLNOYO,CST(MCCST2),RST(ADCOGL)
     &                           ,RST(ADCOSC),CST(MCFGR4),CST(MCTBAS))  
                     CALL NOYFOU (NDIM,RST(ADNOGL),RST(ADNOSC),LAMBDC
     &                           ,CST(MCFGR4),CST(MCGREN),CST(MCDGRE)
     &                           ,NIVIM2,IMPSDR)                        !calgre
                     MCGREN = MCGREN+1 
                     MCDGRE = MCDGRE+1
                     ADNUSC = ADNUSC+1
                     ADNOSC = ADNOSC+NDIM  
31                CONTINUE
               ELSEIF (KLNOYO(1:3).EQ.'NEU') THEN
                  ADNOSC = MCNOSC
                  DO 32 IDLSIG = 1,NBLIGR
                     IF (NIVIM2.LT.IDLSIG) NIVIM2 = 0
                     ADCOSC = MCCOSC+NDIM*(IST(ADNUSC)-1)
                     CALL TAZERO (LGFGRE,TYFGRE,MCFGR4,AST,IST,RST,CST) 
                     CALL GREEN  (KLNOYO,CST(MCCST2),RST(ADCOGL)
     &                           ,RST(ADCOSC),CST(MCFGR4),CST(MCTBAS))  
                     CALL NOYNEU (NDIM,RST(ADNOGL),RST(ADNOSC)
     &                           ,CST(MCFGR4),CST(MCGREN),CST(MCDGRE)
     &                           ,NIVIM2,IMPSDR)                        !calgre
                      MCGREN = MCGREN+1 
                      MCDGRE = MCDGRE+1
                      ADNUSC = ADNUSC+1
                      ADNOSC = ADNOSC+NDIM  
32                CONTINUE
               ELSEIF (KLNOYO(1:3).EQ.'DIR') THEN 
                  DO 33 IDLSIG = 1,NBLIGR
                     IF (NIVIM2.LT.IDLSIG) NIVIM2 = 0
                     ADCOSC = MCCOSC+NDIM*(IST(ADNUSC)-1)
                     CALL TAZERO (LGFGRE,TYFGRE,MCFGR4,AST,IST,RST,CST) 
                     CALL GREEN  (KLNOYO,CST(MCCST2),RST(ADCOGL)
     &                           ,RST(ADCOSC),CST(MCFGR4),CST(MCTBAS))
                     CALL NOYDIR (NDIM,RST(ADNOGL)
     &                           ,CST(MCFGR4),CST(MCGREN),CST(MCDGRE)
     &                           ,NIVIM2,IMPSDR)                        !calgre
                     MCGREN = MCGREN+1 
                     MCDGRE = MCDGRE+1
                     ADNUSC = ADNUSC+1
33                CONTINUE
               ENDIF
               ADNUGL = ADNUGL+1
               ADNOGL = ADNOGL+NDIM 
               IDLGAM = IDLGAM+1
30          CONTINUE
         ENDIF
*
         IF (NIVIMP.GT.0) THEN
            CALL PRTERM (NMGREN,NWGREN,NIVIMP,IMPSDR)                   !prsd
            CALL PRTERM (NMDGRE,NWDGRE,NIVIMP,IMPSDR)                   !prsd
         ENDIF
         CALL TBSAVE (NMGREN,NWGREN)
         CALL TBSAVE (NMDGRE,NWDGRE)
*
         NWGREN = NWGREN+1
         NWDGRE = NWDGRE+1
50    CONTINUE
*
      CALL TBSAVE ('&NORMA',NVNOGL)
      CALL TBSAVE ( NMCOOR ,INTRSC)
      CALL TBSAVE ('#GNEDO',NVNUSC)
      CALL TBSAVE ('#GNEDO',NVNUGL)
      IF (NMTBAS.NE.' ')       CALL TBSAVE ( NMTBAS ,NVTBAS)
      IF (KLNOYO(:3).NE.'DIR') CALL TBSAVE ('&NORMA',NVNOSC)
      IF (INTRGL.NE.INTRSC)    CALL TBSAVE ( NMCOOR ,INTRGL)
      CALL TBTUER ('$WORK$',2)
      CALL TBTUER ('$WORK$',4)
*
      IF (NIVIMS.GE.0.AND.NIVIMG.GE.0.AND.IMPPAL.GT.0) THEN
         CALL PRNTRM (NMGREN,NVGREN,ERCODE,NCATR1)
         CALL PRNTRM (NMDGRE,NVDGRE,ERCODE(NCATR1+1:),NCATR2)
         WRITE (IMPPAL,10000) KLNOYO,ERCODE(1:NCATR1)
     &                       ,ERCODE(NCATR1+1:NCATR1+NCATR2)
      ENDIF
      CALL PRFXMJ (-1,'*CalNoy*')  
      RETURN
*-----------------------------------------------------------------------
10000 FORMAT(/T2,'*CalNoy* Calcul des noyaux de Green complexes pour le'
     &,' couplage ',A,/,T11,A,' et ',A)
10001 FORMAT(/T2,'*CalNoy* Morceau no',I2,' des matrices de Green - '
     &,I5,' lignes et',I5,' colonnes')
10002 FORMAT('   Colonne',I4,' (d.l. de Gamma no',I4,' Normale : ' 
     &,3E10.4,')')
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
99991 CALL PRNTRM (NMTBAS,NIVDON,ERCODE,NCATR1)
      CALL BAISE  ('Le tableau '//ERCODE(1:NCATR1)
     &//' est necessairement complexe.') 
                                                                    END !CalNoy
