*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE SYMGRE (TYNOYO,NDIM,TABCSG,POINTP,POINTM,PPRIME  
     &                  ,NS,LGFGRE,FGRE,EGRE,TABASS)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin & O.deBayser (Octobre 1989) 
* Derniere modification : D.Martin (30 Mai 1996)
* Version 1.0.0
*  
* Calcul de la fonction de Green pour un couple de points (POINTP,POINTM)  
* y compris la prise en compte des symetries eventuelles du probleme par
* rapport aux plans de coordonnees.
*  
*-- Arguments --  
* TYNOYO: Chaine de caracteres indiquant le type du noyau a calculer
* NDIM  : dimension d'espace
* TABCSG: tableau contenant les constantes necessaires pour le calcul  
* POINTP: point sur la frontiere de calcul de representation integrale 
* POINTM: point numero 2 (en general sur la frontiere de couplage) 
* PPRIME: tab. de dim. 3 (independament de NDIM) : points symetriques de POINTP
* NS    : tableau de dim. 3 (independament de NDIM) contenant les indices  
*         de symetrie du probleme par rapport aux plans de coordonnees :
*          NS(i) = 1 pas de symetrie en la i-eme variable d'espace 
*          NS(i) = 2  symetrie en la i-eme variable d'espace
*          NS(i) =-2  antisymetrie en la i-eme variable d'espace
*          Remarques : Si NDIM =1 ce tableau doit necessairement contenir  
*                      les valeurs 1 en 2 et 3-ieme positions  
*                      Si NDIM =2 ce tableau doit necessairement contenir  
*                      la valeur 1 en 3-ieme positions 
* FGRE   : tableau contenant les valeurs de la fonction de Green et de ses 
*          derivees, cumulant les valeurs pour les symetries
* EGRE  : idem mais pour un seul couple de points (tableau de travail) 
* TABASS : Tableau de travail pour Green
* Remarque importante : Rangement des tableaux FGRE et EGRE
* FGRE(1)          : valeur de la fonction de Green
* FGRE(2) a FGRE(1+NDIM)      :derivees par rapport a M :dG/dxm,dG/dym,(dG/dzm)
* FGRE(2+NDIM) a FGRE(1+2*NDIM):derivees par rapport a P:dG/dxp,dG/dyp,(dG/dzp)
* FGRE(2+2*NDIM) a la fin du tableau : derivees secondes croisees dans l'ordre 
*                                       d2G/dxmdxp,d2G/dxmdyp,(d2G/dxmdzp)
*                                       d2G/dymdxp,d2G/dymdyp,(d2G/dymdzp)
*                                      (d2G/dzmdxp,d2G/dzmdyp,(d2G/dzmdzp))
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYNOYO 
      INTEGER       NDIM,NS(3),LGFGRE  
      REAL          POINTP(*),POINTM(*),PPRIME(3)  
      COMPLEX       TABCSG(*),FGRE(*),EGRE(*),TABASS(*)
*
      INTEGER       ISZ,LASZ,IZ,ISY,LASY,IY,ISX,LASX,IX,IS,I
*-----------------------------------------------------------------------
      PPRIME(3)=POINTP(3)  
      ISZ=1
      LASZ=0
      DO 10 IZ=1,ABS(NS(3))
         PPRIME(2)=POINTP(2)
         ISY=1 
         LASY=0
         DO 9 IY=1,ABS(NS(2))  
            PPRIME(1)=POINTP(1)
            ISX=1  
            LASX=0 
            DO 8 IX=1,ABS(NS(1))
               IS=ISX*ISY*ISZ
               DO 6 I=1,LGFGRE 
6                 EGRE(I)=0.
               CALL GREEN  (TYNOYO,TABCSG,PPRIME,POINTM,EGRE,TABASS)    !Appl-  
               DO 1 I=NDIM+2,LASX,NDIM 
                  EGRE(I)=-EGRE(I)
1              CONTINUE 
               DO 2 I=NDIM+3,LASY,NDIM 
                  EGRE(I)=-EGRE(I) 
2              CONTINUE
               DO 3 I=NDIM+4,LASZ,NDIM 
                  EGRE(I)=-EGRE(I)
3              CONTINUE 
               IF (IS.GT.0) THEN
                  DO 4 I=1,LGFGRE  
                     FGRE(I)=FGRE(I)+EGRE(I)
4                 CONTINUE
               ELSE
                  DO 5 I=1,LGFGRE  
                     FGRE(I)=FGRE(I)-EGRE(I)
5                 CONTINUE
               ENDIF  
               PPRIME(1)=-POINTP(1)
               ISX=SIGN(1,NS(1))
               LASX=LGFGRE
8           CONTINUE
            PPRIME(2)=-POINTP(2)
            ISY=SIGN(1,NS(2))  
            LASY=LGFGRE
9        CONTINUE
         PPRIME(3)=-POINTP(3)  
         ISZ=SIGN(1,NS(3)) 
         LASZ=LGFGRE
10    CONTINUE
                                                                    END !Symgre
