*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE CERELR (NMESSE,NVESSE,NMMATR,NVMATR,NMVECT,NVVECT
     &                  ,ROTONS,ROTACE,NUKACE,NUINCE,NVDUCE,NBNECE
     &                  ,NVNUCE,NCPICE,NVCOCE,NUDOMG,NCHDOM,NIVICE
     &                  ,NMDOAL,NVDOAL,TYDOAL,NVANAL,HOMOGN
     &                  ,MULTIM,NVDSMA
     &                  ,NBCOLO,NUINCC,NVNUMC,NBNEMC,NCPINC
     &                  ,NBLIGN,NUINCL,NVNUML,NBNEML,NCPINL
     &                  ,NMSTOK,NVSTOK,INSTOK,ITYSYM,TYMATR
     &                  ,NBVECT,TYVECT,IRCODE)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin (Janvier 1999)
* Derniere modification : D.Martin (29 Septembre 2000)
* Version 1.0.2
*
* Elimination d'une condition essentielle de Dirichlet ou de transmission
* dans une matrice et un vecteur apres eventuel passage dans le repere local,
* puis retour dans le repere global.
*
* Utilise dans Cdesse et Cdtran
*
*-- Arguments d'entree --
* NMESSE nom de la condition essentielle
* NVESSE niveau de ce terme
* NMMATR nom de la matrice
* NVMATR niveau de ce terme
* NMVECT nom du vecteur
* NVVECT niveau de ce terme
* ROTONS indicateur de passage au repere local (n,t1,t2) pour le matrice
* ROTACE indicateur de passage au repere local (n,t1,t2) pour la donnee
*        de la condition essentielle
* NUKACE numero de type de la condition essentielle
* NUINCE numero de l'inconnue de la condition essentielle
* NVDUCE numerotation des d.l. de la condition essentielle
* NBNECE nombre de noeuds de la condition essentielle
* NVDUCE numerotation des noeuds de la condition essentielle
* NCPICE nombre de composantes par noeud de l'inconnue de la condition
* NVCOCE niveau du tableau &RPLOC contenant le repere local
* NUDOMG numero du domaine geometrique portant la condition (pour impression)
* NCHDOM nombre de caracteres par nom de domaine (pour impression)
* NIVICE niveau d'impression du terme condition essentielle
* NMDOAL nom du tableau contenant les valeurs de la condition essentielle
*        (peut etre egal a NMESSE)
* NVDOAL son niveau (peut etre egal a NVESSE)
* TYDOAL type de declaration de ce tableau
* NVANAL nombre de valeurs par noeuds de ce tableau
* HOMOGN indicateur de condition homogene
* MULTIM indicateur de matrice multi-inconnues
* NVDSMA niveau du tableau &DSMBL de structure multi-inconnue de la matrice
* NBCOLO nombre de colonnes de la matrice
* NUINCC numero de l'inconnue en colonne de la matrice
* NVNUMC numerotation de noeuds en colonne de la matrice
* NBNEMC nombre de noeuds en colonne de la matrice (cas uni-inconnue)
*        nombre de colonnes de la matrice          (cas multi-inconnues)
* NCPINC nombre de composantes de l'inconnue en colonne de la matrice
* NBLIGN nombre de lignes de la matrice
* NUINCL numero de l'inconnue en ligne de la matrice
* NVNUML numerotation de noeuds en ligne de la matrice
* NBNEML nombre de noeuds en ligne de la matrice  (cas uni-inconnue)
*        nombre de lignes de la matrice          (cas multi-inconnues)
* NCPINL nombre de composantes de l'inconnue en ligne de la matrice
* NMSTOK nom du tableau des pointeurs de stockage de la matrice
* NVSTOK son niveau
* INSTOK indicateur du type de stockage de la matrice
* ITYSYM indicateur de symetrie de la matrice
* TYMATR type de declaration de la matrice
* TYVECT type de declaration du vecteur
* NBVECT nombre de colonnes du vecteur
* IRCODE code de retour de la procedure
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) NMESSE,NMMATR,NMVECT,NMDOAL,TYDOAL,NMSTOK,TYMATR
     &             ,TYVECT
      CHARACTER     ROTONS,ROTACE,MULTIM
      LOGICAL       HOMOGN
      INTEGER       NVESSE,NVMATR,NVVECT
     &             ,NUKACE,NUINCE,NVDUCE,NBNECE,NVNUCE,NCPICE,NVCOCE
     &             ,NUDOMG,NCHDOM,NIVICE,NVDOAL,NVANAL,NVDSMA,NBVECT
     &             ,NBCOLO,NUINCC,NVNUMC,NBNEMC,NCPINC
     &             ,NBLIGN,NUINCL,NVNUML,NBNEML,NCPINL
     &             ,NVSTOK,INSTOK,ITYSYM,IRCODE
*
      INCLUDE 'ALLOC'
      INCLUDE 'CONTEX'
*
      CHARACTER     PRFXAS*8
      CHARACTER     ERCODE*120,NGNECE*6
      INTEGER       NDPLOC,NVNECE,MCNECE,MCVECT,MCESSE
     &             ,NEAVCE,MCNUMC,NEAVLE,MCNUML,MCSTOK,MCMATR,NCATR1
      COMMON/FORMAH/ERCODE
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*CeRelr*')
*
      IF (ROTONS.EQ.'Y') THEN
*
*        Passage au repere local (n , t1 [,t2]) pour l'application des
*        conditions essentielles vectorielles 'u'.N et 'u'^N
*
         NDPLOC=NCPICE
         CALL TBCREE ('$WORK$',1,1,NBLIGN+NBCOLO,'c')
*
*        Reperes locaux calcules a partir du tableau des normales
*
         CALL TBRR1  (ERCODE,'#GNEDO',NVNUCE,MCNECE)
         CALL TBCREE ('&RPLOC',NVCOCE,2,NDPLOC*NDPLOC*NBNECE,'c')
         CALL CEPLOC ('&RPLOC',NVCOCE,NDIM,NDPLOC,NBNECE,NVNUCE,NVCOCE
     &               ,NUDOMG,NCHDOM,NIVICE,IMPSDR)                      !cdesse
*
*        Changement de repere
*
         CALL CEROTL (1     ,NUINCE,NCPICE,NBNECE,NVNUCE,NVDUCE
     &               ,'$WORK$',1,NDPLOC,'&RPLOC',NVCOCE,'REEL'
     &               ,MULTIM,NVDSMA,NVNUMC,NVNUML,NBNEMC,NBNEML
     &               ,NMSTOK,NVSTOK,INSTOK,ITYSYM,NMMATR,NVMATR
     &               ,TYMATR,NMVECT,NVVECT,TYVECT,NBVECT
     &               ,ROTACE,NMDOAL,NVDOAL,TYDOAL,NIVICE,IMPSDR)         !cdesse
      ENDIF
      IF (NIVICE.GT.0.AND.IMPSDR.GT.0) THEN
         IF (ROTONS(1:1).EQ.'Y') THEN
            CALL PRNTRM (NMMATR,NVMATR,ERCODE(1:16),NCATR1)
            WRITE (IMPSDR,10000) PRFXAS(1),'la matrice'
     &                          ,ERCODE(1:NCATR1)
            IF (NMVECT(1:1).NE.' '.AND..NOT.HOMOGN) THEN
               CALL PRNTRM (NMVECT,NVVECT,ERCODE(1:16),NCATR1)
               WRITE (IMPSDR,10000) PRFXAS(1),'le vecteur'
     &                             ,ERCODE(1:NCATR1)
               CALL PRTERM (NMVECT,NVVECT,NIVICE,IMPSDR)
               CALL TBRR1  (ERCODE,NMVECT,NVVECT,MCVECT)
            ENDIF
         ENDIF
         IF (ROTACE(1:1).EQ.'Y') THEN
            CALL PRNTRM (NMDOAL,NVDOAL,ERCODE(1:16),NCATR1)
            WRITE (IMPSDR,10000) PRFXAS(1),'le vecteur'
     &                          ,ERCODE(1:NCATR1)
            CALL PRTERM (NMDOAL,NVDOAL,NIVICE,IMPSDR)
            CALL TBRR1  (ERCODE,NMDOAL,NVDOAL,MCESSE)
         ENDIF
      ENDIF
*
*     Elimination dans la matrice et, le cas echeant, le vecteur.
*
      NGNECE='#GNEDO'
      NVNECE=NVNUCE
      CALL CENUME (NUINCE,NCPICE,NUKACE,NVNUCE,NVDUCE,MULTIM,NVDSMA
     &            ,NGNECE,NVNECE,'$WORK$',2
     &            ,NUINCC,NVNUMC,NEAVCE,MCNUMC
     &            ,NUINCL,NVNUML,NEAVLE,MCNUML)                         !cdesse
*
      CALL TBAR3  (ERCODE,NMSTOK,NVSTOK,MCSTOK,NGNECE,NVNECE,MCNECE
     &                   ,NMMATR,NVMATR,MCMATR)
      MCVECT=0
      IF (NMVECT(1:1).NE.' ') CALL TBAR1  (ERCODE,NMVECT,NVVECT,MCVECT)
      MCESSE=0
      IF (.NOT.HOMOGN) CALL TBAR1  (ERCODE,NMDOAL,NVDOAL,MCESSE)
*
      CALL CEELIM (MULTIM,NCPICE,NUKACE,NUINCE,IST(MCNECE+1)
     &          ,IST(MCNECE+2),TYDOAL,NVANAL,MCESSE
     &          ,NUINCC,NCPINC,IST(MCNUMC+1),IST(MCNUMC+2),NEAVCE,NBNEMC
     &          ,NUINCL,NCPINL,IST(MCNUML+1),IST(MCNUML+2),NEAVLE,NBNEML
     &          ,NVNUML.EQ.NVNUMC,ITYSYM,TYMATR,MCMATR,NMSTOK,MCSTOK
     &          ,TYVECT,MCVECT,NBVECT,NBLIGN,IST,RST,CST,NIVICE,IMPSDR
     &          ,IRCODE)                                                !cdesse
*
      IF (IRCODE.EQ.1) RETURN
      IF (NGNECE.EQ.'$WORK$') CALL TBTUER ('$WORK$',2)
*
      IF (ROTONS.EQ.'Y') THEN
*
*        Retour dans le repere cartesien (e1, e2 [,e3])
*
         CALL CEROTL (NDPLOC,NUINCE,NCPICE,NBNECE,NVNUCE,NVDUCE
     &               ,'$WORK$',1,NDPLOC,'&RPLOC',NVCOCE,'REEL'
     &               ,MULTIM,NVDSMA,NVNUMC,NVNUML,NBNEMC,NBNEML
     &               ,NMSTOK,NVSTOK,INSTOK,ITYSYM,NMMATR,NVMATR,TYMATR
     &               ,NMVECT,NVVECT,TYVECT,NBVECT
     &               ,ROTACE,NMDOAL,NVDOAL,TYDOAL,0,IMPSDR)             !cdesse
         CALL TBTUER ('$WORK$',1)
         CALL TBTUER ('&RPLOC',NVCOCE)
      ENDIF
      CALL PRFXMJ (-1,'*CeRelr*')
*-----------------------------------------------------------------------
10000 FORMAT (T2,A,'>*CeRelr* Passage au repere local dans ',A,' ',A)
                                                                    END !CeRelr
