*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE CTCHMA (NCPICE,NBNECE,TYDOAS,NBVANE,NBSUNE,MCDOAS
     &                  ,TYCHMA,MCCHMA,SKIPNE,RST,CST)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Fevrier 1998)
* Derniere modification : D.Martin (12 Avril 2000)
* Version 1.0.2
*  
* Calcul des coefficients des matrices P orthonormale de changement d'inconnues
* U = P.u (par noeud) pour une condition de transmission de la forme
*                        k_1 u_1 = k_2 u_2 + [ g ].
* 
*           ( (k_1^2+k_2^2)^{-1/2)*k_1  -(k_1^2+k_2^2)^{-1/2}*k_2 )
* avec  P = (                                                     ) = P^t^{-1}
*           ( (k_1^2+k_2^2)^{-1/2)*k_2   (k_1^2+k_2^2)^{-1/2}*k_1 )
*
* ou k_1 et k_2 sont 
* - soit des constantes verifiant k_1^2 + k_2^2 non nul
* - soit des matrices diagonales (assimiles a des vecteurs) telles que
*   (k_1^2 + k_2^2) soit definie positive
*  
*-- Arguments d'entree --
* NCPICE nombre de composantes des inconnues de la condition de transmission
* NBNECE nombre de noeuds de la condition de transmission
* TYDOAS types, reel ou complexe, des donnees k_1, k_2 et g
* NBVANE nombres de valeurs par noeud pour les donnees k_1, k_2 et g
* NBSUNE increment de passage au noeud suivant dans le tableau contenant
*        les matrices de changement d'inconnues P
* MCDOAS adresses des donnees k_1, k_2 et g
* TYCHMA type , reel ou complexe, des matrices de changement de variable
* MCCHMA adresse du tableau de travail contenant coefficients des matrices 
*        de changement d'inconnues
* SKIPNE passage d'un noeud au suivant dans les tableaux ci-dessous k_1 et k_2
*        (0 si matrice constante, NCPICE si diagonale, NCPICE*NCPICE sinon)
* RST,CST super-tableaux version reelle ou complexe
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYCHMA,TYDOAS(*)
      INTEGER       NCPICE,NBNECE,MCCHMA
     &             ,NBVANE(*),NBSUNE(*),MCDOAS(*),SKIPNE(*)
      REAL          RST(*)
      COMPLEX       CST(*)
*
      INTEGER       MCK1,MCK1N,MCK2,MCK2N,MCK3,MCK3N,NBNECH(3),NUDOAS
     &             ,MMCK,INCK,INCIN,I,J,SKIPPP
      REAL          RP11,RP12,MODULE
*      COMPLEX       CP11,CP12
      CHARACTER     TYDO*4
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*Ctchma*')
*
*     Calcul des coefficients caracteristiques du changement de variables
*
      MCK1=MCCHMA
      DO 1999 NUDOAS=1,2
         NBNECH(NUDOAS)=NBNECE
         IF (SKIPNE(NUDOAS).EQ.0) NBNECH(NUDOAS)=1
         IF (NBVANE(NUDOAS).LE.NCPICE) THEN
            IF (MCDOAS(NUDOAS).LE.0) THEN
*
*              Si le donnee k_i n'est pas definie, on pose k_i = 1,
*
               IF (TYCHMA(1:1).EQ.'R') THEN
                  DO 1221 I=1,NBNECH(NUDOAS)*NCPICE
                     RST(MCK1)= 1.
                     MCK1=MCK1+1
1221              CONTINUE
               ELSE
                  DO 1421 I=1,NBNECH(NUDOAS)*NCPICE
                     CST(MCK1)= 1.
                     MCK1=MCK1+1
1421              CONTINUE
               ENDIF
            ELSE
*
*              Si k_i est une constante ou matrice diagonale par noeud
*
               TYDO = TYDOAS(NUDOAS)
               MCK1N= MCDOAS(NUDOAS)
               INCK = NBSUNE(NUDOAS)
               INCIN= 1
               IF (NBVANE(NUDOAS).EQ.1.AND.INCK.EQ.0) INCIN=0
               IF (TYCHMA(1:1).EQ.'R') THEN 
                  DO 1224 I=1,NBNECH(NUDOAS)
                     MMCK=MCK1N
                     DO 1223 J=1,NCPICE
                        RST(MCK1)= RST(MMCK)
                        MCK1=MCK1+1
                        MMCK=MMCK+INCIN
1223                 CONTINUE
                     MCK1N=MCK1N+INCK
1224              CONTINUE
               ELSEIF (TYDO(1:2).EQ.'CO') THEN
                  DO 1444 I=1,NBNECH(NUDOAS)
                     MMCK=MCK1N
                     DO 1443 J=1,NCPICE
                        CST(MCK1)= CST(MMCK)
                        MCK1=MCK1+1
                        MMCK=MMCK+INCIN
1443                 CONTINUE
                     MCK1N=MCK1N+INCK
1444              CONTINUE
               ELSEIF (TYDO(1:1).EQ.'R') THEN
                  DO 1424 I=1,NBNECH(NUDOAS)
                     MMCK=MCK1N
                     DO 1423 J=1,NCPICE
                        CST(MCK1)= RST(MMCK)
                        MCK1=MCK1+1
                        MMCK=MMCK+INCIN
1423                 CONTINUE
                     MCK1N=MCK1N+INCK
1424              CONTINUE
               ENDIF
            ENDIF
         ENDIF
1999  CONTINUE
*
*     Normalisation des coefficients : k_i devient k_i/sqrt(k_1^2+k_2^2)
*
      SKIPPP=MAX (SKIPNE(1),SKIPNE(2))
      MCK1=MCCHMA
      MCK2=MCCHMA+NCPICE*NBNECH(1)
      MCK3=MCK2  +NCPICE*NBNECH(2)
      IF (TYCHMA(1:1).EQ.'R') THEN
         DO 2202 I=1,MAX (NBNECH(1),NBNECH(2))
            MCK1N=MCK1
            MCK2N=MCK2
            MCK3N=MCK3
            DO 2201 J=1,NCPICE
               RP11=RST(MCK1N)
               RP12=RST(MCK2N)
               MODULE    =1./SQRT(RP11*RP11+RP12*RP12)
               RST(MCK1N)=RP11*MODULE
               RST(MCK2N)=RP12*MODULE
               RST(MCK3N)=MODULE
               MCK1N     =MCK1N+1
               MCK2N     =MCK2N+1
               MCK3N     =MCK3N+1
2201        CONTINUE
            MCK1=MCK1+SKIPNE(1)
            MCK2=MCK2+SKIPNE(2)
            MCK3=MCK3+SKIPPP
2202     CONTINUE
      ELSE
         CALL BAISE ('Cas d''un changement de variables a coefficients'
     &             //' complexe non prevu !')
      ENDIF
*
      CALL PRFXMJ (-1,'*Ctchma*')  
      END
