*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE CTCHVA (SENSCH,NMCHMA,NVCHMA,TYCHMA,SKIPNE,NBDOAS
     &                  ,TYDOAS,NVANAS,NBSUNE,MCDOAS,NMRGDL,NVRGDL
     &                  ,NCPICE,NBNECE,HOMOGN
     &                  ,NMMATR,NVMATR,ITYSYM,TYMATR,STRSYM
     &                  ,NMSCMB,NVSCMB,TYSCMB,NBVECT
     &                  ,NBNECO,NBNELI,NMSTOK,NVSTOK,NIVICE)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Mai 1998)
* Derniere modification : D.Martin (30 Mars 2004)
* Version 1.4
*
* Changement de variable U = P u pour une condition de transmission
* pour transformer le systeme  A u = b en P A P^{-1} U = P b
* et ramener la condition de transmission a une condition de Dirichlet.
*
*           ( (k_1^2+k_2^2)^{-1/2)*k_1   -(k_1^2+k_2^2)^{-1/2}*k_2 )
* avec  P = (                                                      ) = P^t^{-1}
*           ( (k_1^2+k_2^2)^{-1/2)*k_2    (k_1^2+k_2^2)^{-1/2}*k_1 )

*
*-- Arguments d'entree --
* SENSCH +1 si changement de variables        (calcul de P A x P^{-1})
*        -1 si retour aux variables initiales (calcul de P^{-1} A' x P)
* NMCHMA tableau des coefficients de changement de variables
* NVCHMA son niveau
* TYCHMA son type (reel ou complexe)
* SKIPNE passage d'un noeud au suivant dans les tableaux k_1, k_2 et g
*        (0 si matrice constante, NCPICE sinon)
* TYDOAS types, reel ou complexe, des donnees k_1, k_2 et g
* NVANAS tableaux des nombres de valeurs par noeud des donnees k_1, k_2 et g
* NBSUNE tableaux des increments de passage au noeud suivant dans les tableaux
*        k_1, k_2 et g
* MCDOAS tableaux des adresses des donnees associees k_1, k_2 et g
* NMRGDL tableau des rangs des d.l. dans les lignes/colonnes de la matrice
* NVRGDL son niveau
* NCPICE nombre de composantes des inconnues de la condition de transmission
* NBNECE nombre de noeuds portant la condition de transmission
* HOMOGN indicateur de condition de transmission homogene ou non
* NMMATR nom du terme matriciel sur lequel porte l'elimination
* NVMATR niveau de ce terme
* ITYSYM indicateur de matrice symetrique
* TYMATR type de declaration de la matrice
* STRSYM indicateur de matrice structurellement symetrique
* NMSCMB nom du terme vecteur sur lequel porte l'elimination
* NVSCMB niveau de ce terme
* TYSCMB type de declaration du vecteur
* NBNECO nombre de colonnes de la matrice
* NBNELI nombre de lignes de la matrice et du vecteur
* NMSTOK nom du tableau de stockage de la matrice
* NIVICE niveau d'impression du terme condition essentielle
* NVSTOK niveau du tableau de stockage de la matrice
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      INTEGER       SENSCH,SKIPNE(*),NBDOAS,NVANAS(*),NBSUNE(*)
     &             ,MCDOAS(*),NVCHMA,NVRGDL,NVMATR,ITYSYM,STRSYM
     &             ,NVSCMB,NBVECT,NBNECO,NBNELI,NVSTOK,NCPICE,NBNECE
     &             ,NIVICE
      CHARACTER*(*) NMCHMA,TYCHMA,TYDOAS(*),NMRGDL
     &             ,NMMATR,TYMATR,NMSCMB,TYSCMB
     &             ,NMSTOK
      LOGICAL       HOMOGN
*
      INCLUDE 'ALLOC'
      INCLUDE 'CONTEX'
*
      INTEGER       INDSYM
      INTEGER       MCSTOK,MCMATR,MCMATL,MCMATU,MCSCMB
     &             ,MCCHM1,MCCHM2,MCCHM3,MCRAN1,MCRAN2
     &             ,NBL,NBPLAL,MCPLAL,MCPLIL,LGL
     &             ,NBC,NBPLAU,MCPLAU,MCPLIU,LGU
      CHARACTER     PRFXAS*8
      CHARACTER     ERCODE*120
      COMMON/FORMAH/ERCODE
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*Ctchva*')
*
      CALL TBAR4  (ERCODE,NMSTOK,NVSTOK,MCSTOK,NMCHMA,NVCHMA,MCCHM1
     &                   ,NMMATR,NVMATR,MCMATR,NMRGDL,NVRGDL,MCRAN1)
      MCSCMB=0
      IF (NMSCMB(1:1).NE.' ') CALL TBAR1  (ERCODE,NMSCMB,NVSCMB,MCSCMB)
*
*     Caracteristiques de la structure Bimorse de la matrice
*
      CALL EXBMOR (MCSTOK,IST(MCSTOK),NBL,NBPLAL,MCPLAL,MCPLIL,LGL
     &                               ,NBC,NBPLAU,MCPLAU,MCPLIU,LGU)
*     Adresse de la partie triangulaire inferieure
      MCMATL=MCMATR+MIN (NBL,NBC)
*     Adresse de la partie triangulaire superieure
      MCMATU=MCMATL
      IF (ITYSYM.EQ.INDSYM(' ')) THEN
         MCMATU=MCMATL+LGL
      ELSE
         CALL BAISE  ('Cas des matrices symetriques a prevoir')
      ENDIF
*
      MCCHM2=MCCHM1+NCPICE+SKIPNE(1)*(NBNECE-1)
      MCRAN2=MCRAN1+NBNECE
*
*++   Calcul des coefficients caracteristiques k_1 et k_2 de P
*
      CALL CTCHMA (NCPICE,NBNECE,TYDOAS,NVANAS,NBSUNE,MCDOAS
     &            ,TYCHMA,MCCHM1,SKIPNE,RST,CST)
*
*++   Prise en compte du changement de variables pour une donnee g
*     de condition de transmission non homogene
*
*     La donnee g est transformee en g/sqrt(k_1^2+k_2^2) si SENSCH=1
*     et retour aux valeurs initiales si SENSCH=-1
*
      IF (.NOT.HOMOGN) THEN
          MCCHM3=MCCHM2+NCPICE+SKIPNE(2)*(NBNECE-1)
          CALL CTCHDO (SENSCH,NCPICE,NBNECE,NBDOAS,TYDOAS(NBDOAS)
     &                ,NVANAS(NBDOAS),MCDOAS(NBDOAS),TYCHMA,MCCHM3
     &                ,SKIPNE,RST,CST)
      ENDIF
*
*--   Effet du changt d'inconnues dans les colonnes des d.l. de transmission
*
*       dans la partie triangulaire superieure
*                  ( a_{i,k}   a_{i,l} ) x P^{-SENSCH}  i=1,.,k
      IF (NIVICE.GT.1.AND.IMPSDR.GT.0) THEN
         WRITE (IMPSDR,10001) PRFXAS (1),'*CtChBu*','colonnes','sup.'
      ENDIF
      CALL CTCHBU (SENSCH,NCPICE,NBNECE,IST(MCRAN1),IST(MCRAN2)
     &            ,IST(MCPLAU),IST(MCPLIU),TYMATR,MCMATR,MCMATU
     &            ,TYCHMA,MCCHM1,MCCHM2,SKIPNE(1),SKIPNE(2),RST,CST
     &            ,NIVICE,IMPSDR)
*       dans les parties triangul. inf. et sup. strictes
*                   ( a_{i,k}   a_{i,l} ) x P^{-SENSCH} i=k+1,.,l-1
      IF (NIVICE.GT.1.AND.IMPSDR.GT.0) THEN
         WRITE (IMPSDR,10001) PRFXAS (1),'*CtChBm*','colonnes'
     &                       ,'inf. & sup.'
      ENDIF
      CALL CTCHBM (SENSCH,NCPICE,NBNECE,IST(MCRAN1),IST(MCRAN2)
     &            ,IST(MCPLAL),IST(MCPLIL),IST(MCPLAU),IST(MCPLIU)
     &            ,TYMATR,MCMATL,MCMATU
     &            ,TYCHMA,MCCHM1,MCCHM2,SKIPNE(1),SKIPNE(2),RST,CST
     &            ,NIVICE,IMPSDR)
*       dans la partie triangulaire inferieure
*                  ( a_{i,k}   a_{i,l} ) x P^{-SENSCH}  i=l,.,n
      IF (NIVICE.GT.1.AND.IMPSDR.GT.0) THEN
         WRITE (IMPSDR,10001) PRFXAS (1),'*CtChBl*','colonnes','inf.'
      ENDIF
      CALL CTCHBL (SENSCH,NCPICE,NBNECE,IST(MCRAN1),IST(MCRAN2)
     &            ,NBNELI,IST(MCPLAL),IST(MCPLIL),TYMATR,MCMATR,MCMATL
     &            ,TYCHMA,MCCHM1,MCCHM2,SKIPNE(1),SKIPNE(2),RST,CST
     &            ,NIVICE,IMPSDR)
*
*--   Changement d'inconnues dans le vecteur (second membre)
*                  ( b_{k} )
*     P^{SENSCH} x ( ...   )
*                  ( b_{l} )
      IF (MCSCMB.GT.0) THEN
         CALL CTCHSC (SENSCH,NCPICE,NBNECE,IST(MCRAN1),IST(MCRAN2)
     &               ,TYSCMB,MCSCMB,NBVECT,NBL
     &               ,TYCHMA,MCCHM1,MCCHM2,SKIPNE(1),SKIPNE(2),RST,CST)
10       CONTINUE
      ENDIF
*
*--   Effet du changt d'inconnues dans les lignes des d.l. de transmission
*
      IF (STRSYM.EQ.0) THEN
         MCRAN1=MCRAN2+NBNECE
         MCRAN2=MCRAN1+NBNECE
      ENDIF
*
*       dans la partie triangulaire inferieure
*                  ( a_{k,j} )
*     P^{SENSCH} x (   ...   )  j=1,.,k
*                  ( a_{l,j} )
      IF (NIVICE.GT.1.AND.IMPSDR.GT.0) THEN
         WRITE (IMPSDR,10001) PRFXAS (1),'*CtChBu*','lignes','inf.'
      ENDIF
      CALL CTCHBU (SENSCH,NCPICE,NBNECE,IST(MCRAN1),IST(MCRAN2)
     &            ,IST(MCPLAL),IST(MCPLIL),TYMATR,MCMATR,MCMATL
     &            ,TYCHMA,MCCHM1,MCCHM2,SKIPNE(1),SKIPNE(2),RST,CST
     &            ,NIVICE,IMPSDR)
*       dans les parties triangulaires inf. et sup. strictes
*                  ( a_{k,j} )                           ({k+1,k} ..{k+1,l})
*     P^{SENSCH} x (   ...   )  j=k+1,.,l-1 dans le bloc |...
*                  ( a_{l,j} )                           ({l-1,k} ..{l-1,l})
      IF (NIVICE.GT.1.AND.IMPSDR.GT.0) THEN
         WRITE (IMPSDR,10001) PRFXAS (1),'*CtChBm*','lignes'
     &                       ,'inf. & sup.'
      ENDIF
      CALL CTCHBM (SENSCH,NCPICE,NBNECE,IST(MCRAN1),IST(MCRAN2)
     &            ,IST(MCPLAU),IST(MCPLIU),IST(MCPLAL),IST(MCPLIL)
     &            ,TYMATR,MCMATU,MCMATL
     &            ,TYCHMA,MCCHM1,MCCHM2,SKIPNE(1),SKIPNE(2),RST,CST
     &            ,NIVICE,IMPSDR)
*       dans la partie triangulaire superieure
*                  ( a_{k,j} )
*     P^{SENSCH} x (   ...   )  j=l,.,n
*                  ( a_{l,j} )
      IF (NIVICE.GT.1.AND.IMPSDR.GT.0) THEN
         WRITE (IMPSDR,10001) PRFXAS (1),'*CtChBl*','lignes','sup.'
      ENDIF
      CALL CTCHBL (SENSCH,NCPICE,NBNECE,IST(MCRAN1),IST(MCRAN2)
     &            ,NBNECO,IST(MCPLAU),IST(MCPLIU),TYMATR,MCMATR,MCMATU
     &            ,TYCHMA,MCCHM1,MCCHM2,SKIPNE(1),SKIPNE(2),RST,CST
     &            ,NIVICE,IMPSDR)
*
      CALL PRFXMJ (-1,'*Ctchva*')
*-----------------------------------------------------------------------
10001 FORMAT (T2,A,'>*CtChva*>',A,' Changement d''inconnues dans les '
     &,A,' de la partie triang. ',A)
                                                                    END !CeElim

