*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE CALRI1 (TABCOR,NIVCOR,GAMMA,NBDONG,NMDONG
     &                  ,NMVEC1,NVVEC1,NMVEC2,NVVEC2 
     &                  ,NMVEC3,NVVEC3,NIVIMP)   
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur: D.Martin (Juin 1991) 
* Derniere modification : D.Martin (12 juin 2004)
* Version 2
*   
* Calcul de representation integrale en des points donnes par leurs coordonnees
* avec prise en compte des eventuelles symetries, version complexe, 
* sous la forme
*
*      V3(M)= Int_Gamma { V1(p).dG/dn(M,p) - V2(P).G(M,p) } dp
*
* ou G(M,p) est le noyau de Green (de type Dirichlet)
*   
*-- Arguments d'entree --  
* TABCOR tableau contenant les points ou l'on calcule la representation integrale
*        (Attention: la taille de ce tableau definit le nombre de points)
* NIVCOR son niveau
* NBDONG nombre de donnees associees au calcul de la fonction de Green  
* NMDONG tableau des noms des donnees affectes a la fonction de Green   
* NMVEC1 nom du vecteur "densite de potentiel de double couche" V1
* NVVEC1 son niveau
* NMVEC2 nom du vecteur "densite de potentiel de simple couche" V2
* NVVEC2 son niveau
* GAMMA  nom du domaine portant la representation integrale
* NMVEC3 nom du (tableau contenant le) vecteur resultat V3
* NVVEC3 son niveau
* NIVIMP niveau d'impression
* 
*-- Routines utilisees --
* GREEN  calcul de la fonction de Green et de ses derivees
* SYMGRE calcul de Green et de ses derivees avec des eventuelles symetries.
* NOYDIR calcul des noyaux de Green de type Dirichlet
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TABCOR,GAMMA,NMDONG(*),NMVEC1,NMVEC2,NMVEC3
      INTEGER       NIVCOR,NBDONG,NVVEC1,NVVEC2,NVVEC3,NIVIMP
*   
      INCLUDE 'ALLOC'   
      INCLUDE 'CONTEX'  
*
      CHARACTER   TYPINT*4
      CHARACTER   ERCODE*120,NMTBAS*6,TYNOYO*3
      CHARACTER*4 TYGREE,TYVEC1,TYVEC2
      INTEGER     INTTYP,INTDON,KELCHN,KLTERM
      INTEGER     INTCON,ITENTI,ITREEL,ITCOMP,ITGREE,NIVEAU,LGMORE
     &           ,MCMDOM,MCTRDO,LGERDO,NBDOMK,INCRDO,NCHDOM
     &           ,MCMTRM,MCDTRM,LGETRM,NBTERM,INCTRM,NCHTRM,NUTERM
     &           ,MCINCO,ITYINC,NBTNEU
     &           ,MCMDON,MCDONN,LGEDON,NBDONN,INCDON,NCHDON,ITYDON
     &           ,NIVDON,ITDONN,MCESTE,MCRSTE,MCCSTE
     &           ,ITYCAL,NUKALE,NUDONN,NVDSMB,NVDUCV,NVNUCV,NVDULV
     &           ,NVNULV,INSTOK,NVSTOK,IATRGA,NIVIVC
     &           ,ITVEC1,LGVEC1,NVALN1,NUINC1,NNVEC1,NUINL1,NNVEL1
     &           ,INTER1,MCVEC1,NIVIM1
     &           ,ITVEC2,LGVEC2,NVALN2,NUINC2,NNVEC2,NUINL2,NNVEL2
     &           ,INTERP,MCVEC2,NIVIM2,ITVEC3,MCVEC3,MCCORN
     &           ,NUDGGA,NUDOGA,IGEOME,INDMIX,NUSCHQ,MCBCOR
     &           ,INDEXC,IDLHRC,INTRGC,NBNECX,NBNEEX,NBDLEX,NBNEFX
     &           ,NBDLFX,NVNUGC,INDEXL,IDLHRL,INTRGL,NBNECY,NBNEEY
     &           ,NBDLEY,NBNEFY,NBDLFY,NVNOGA,INUTIL,NBTRDO,NIVIMG
     &           ,ADCOGA,MCNOGA,ADNOGA,NBNOGA,MCNUGA,ADNUGA,IDLGAM
     &           ,ADCOSI,IDLSIG,NVNUGA,ITTBAS,NVTBAS,MCTBAS,MCFGR4
     &           ,NBPCOR,LGFGRE,MCGREN,IAGREN,MCDGRE,IADGRE,MCEGRE
     &           ,MCFGRN,MCCST2,LGWOR2,NCATR1,NCATR2,I
      REAL        RESUL1,RESUL2
      COMPLEX     CESUL1,CESUL2
      EQUIVALENCE (RESUL1,CESUL1),(RESUL2,CESUL2)
      COMMON/FORMAH/ERCODE  
      EQUIVALENCE (ERCODE(73:73),TYVEC1),(ERCODE(81:81),TYVEC2)
     &           ,(ERCODE(89:89),TYGREE),(ERCODE(105:105),NMTBAS)
*
*     Tableaux pour les symetries dans le calcul de la fonction de Green
*     de dimension 3 quelle que soit la dimension d'espace
      INTEGER       TBISYM(3)
      REAL          TBPSYM(3)
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*CalRI1*')

      INTCON=INTDON ('CONSTANTE')                                       !utilite
      ITENTI=INTTYP ('ENTIER')                                          !utilite
      ITREEL=INTTYP ('REEL')                                            !utilite
      ITCOMP=INTTYP ('COMPLEXE')                                        !utilite
      ITGREE=ITCOMP
      ITVEC3=ITGREE
*
*     Nombre de points ou l'on calcul la representation integrale
*
      CALL TBRR1  (ERCODE,TABCOR,NIVCOR,NBPCOR)
      NBPCOR=NBPCOR/NDIM
*
      CALL TBRR2  (ERCODE,NMVEC1,NVVEC1,LGVEC1,NMVEC2,NVVEC2,LGVEC2)
*
      CALL TBAR5  (ERCODE,'#OMTRM',1,MCMTRM,'$SDTRM',1,MCDTRM
     &                   ,'#OMDOM',1,MCMDOM,'#TERDO',1,MCTRDO
     &                   ,'#NCONU',1,MCINCO) 
      CALL SDEXDB (IST(MCTRDO),LGERDO,NBDOMK,INCRDO,NCHDOM)             !sdexplo
      CALL SDEXDB (IST(MCDTRM),LGETRM,NBTERM,INCTRM,NCHTRM)             !sdexplo
*
*--   Attributs du terme de densite de double couche V1  (NMVEC1)
*
      NUTERM=KLTERM (NMVEC1,NVVEC1,AST(MCMTRM),IST(MCDTRM))             !sdexplo
      IF (NUTERM.LE.0) CALL ERTERM (1,NMVEC1,NVVEC1)                    !utilite
      CALL GETTRM (IST(MCDTRM+LGETRM+INCTRM*(NUTERM-1)),NIVEAU
     &            ,ITVEC1,ITYCAL,NUKALE,NUDONN,NVALN1,NVDSMB
     &            ,NUINC1,NVDUCV,NNVEC1,NVNUCV
     &            ,NUINL1,NVDULV,NNVEL1,NVNULV
     &            ,INSTOK,NVSTOK,NVNOGA,IATRGA,NIVIVC)                  !sdexplo
      IF (NUINC1.NE.NDFINC) THEN
          NUINL1=NUINC1
          NNVEL1=NNVEC1
      ENDIF
      CALL SDEXCO (NUINL1,IST(MCINCO),ITYINC,INTER1,NBTNEU)             !sdexplo
      TYVEC1=TYPINT (ITVEC1)
      IF (ITVEC1.EQ.ITCOMP) ITVEC3=ITVEC1
*
*--   Attributs du terme de densite de simple couche V2 (NMVEC2)   
*
      NUTERM=KLTERM (NMVEC2,NVVEC2,AST(MCMTRM),IST(MCDTRM))             !sdexplo
      IF (NUTERM.LE.0) CALL ERTERM (1,NMVEC2,NVVEC2)                    !utilite
      CALL GETTRM (IST(MCDTRM+LGETRM+INCTRM*(NUTERM-1)),NIVEAU
     &            ,ITVEC2,ITYCAL,NUKALE,NUDONN,NVALN2,NVDSMB
     &            ,NUINC2,NVDUCV,NNVEC2,NVNUCV 
     &            ,NUINL2,NVDULV,NNVEL2,NVNULV
     &            ,INSTOK,NVSTOK,NVNOGA,IATRGA,NIVIVC)                  !sdexplo
      IF (NUINC2.NE.NDFINC) THEN
          NUINL2=NUINC2
          NNVEL2=NNVEC2
      ENDIF
      CALL SDEXCO (NUINL2,IST(MCINCO),ITYINC,INTERP,NBTNEU)             !sdexplo
      TYVEC2=TYPINT (ITVEC2)
      IF (ITVEC2.EQ.ITCOMP) ITVEC3=ITVEC2
*
      IF (INTER1.NE.INTERP) GOTO 99992
      IF (NNVEL1.NE.NNVEL2.OR.LGVEC1.NE.LGVEC2) GOTO 99993
*   
*--   Caracteristiques du domaine de representation integrale (GAMMA)
*   
      CALL KLDOMN (GAMMA,0,-1,IST(MCTRDO),AST(MCMDOM),NBDOMG
     &            ,NUDOGA,IATRGA)                                       !sdexplo
      IATRGA=MCTRDO+IATRGA
      CALL GETDOM (IST(IATRGA),NUDGGA,NUDOGA,IGEOME,INDMIX,NUSCHQ
     &            ,INDEXC,IDLHRC,INTRGC,NBNECX,NBNEEX,NBDLEX,NBNEFX
     &            ,NBDLFX,NVNUGC
     &            ,INDEXL,IDLHRL,INTRGL,NBNECY,NBNEEY,NBDLEY,NBNEFY
     &            ,NBDLFY,NVNUGA,NVNOGA,INUTIL,NBTRDO,NIVIMG)           !sdexplo
*
      IF (INTRGL.NE.INTERP) GOTO 99994
*
*--   Tableau des coordonnees, normales sur Gamma
*
      CALL TBRR3  (ERCODE,'#ORNOE',INTERP,MCCORN,'&NORMA',NVNOGA,NBNOGA 
     &                   ,'#GNEDO',NVNUGA,MCNUGA)
      NBNOGA=NBNOGA/NDIM
      IF (NNVEL2.NE.NBNOGA) GOTO 99995
*  
*--   Valeurs des constantes associees a la fonction de Green
*     placees dans $WORK$/2 quels que soient leur type
*  
      NMTBAS = ' '
      NVTBAS = 0
      LGWOR2 = MAX (1,NBDONG)
      CALL TBCREE ('$WORK$',2,ITCOMP,LGWOR2,'c')
      CALL TBAR6  (ERCODE,'$WORK$',2,MCCST2,'#OMDON',1,MCMDON
     &                   ,'$DONNE',1,MCDONN,'$ECSTE',1,MCESTE
     &                   ,'$RCSTE',1,MCRSTE,'$CCSTE',1,MCCSTE)
      CALL SDEXDB (IST(MCDONN),LGEDON,NBDONN,INCDON,NCHDON)             !sdexplo
*
      DO 10 I = 1,NBDONG
         NUDONN = KELCHN (NMDONG(I),AST(MCMDON),NBDONN,NCHDON)          !utilite
         IF (NUDONN.LE.0) CALL ERDONN (1,NMDONG(I))                     !utilite
         CALL SDEXCO (NUDONN,IST(MCDONN),ITYDON,NIVDON,ITDONN)          !sdexplo
         IF (ITYDON.EQ.INTCON) THEN
            IF (ITDONN.EQ.ITCOMP) THEN
               CST(MCCST2-1+I) = CST(MCCSTE+NIVDON)
            ELSEIF (ITDONN.EQ.ITREEL) THEN
               CST(MCCST2-1+I) = CMPLX (RST(MCRSTE+NIVDON),0.)
            ELSEIF (ITDONN.EQ.ITENTI) THEN
               CST(MCCST2-1+I) = CMPLX (REAL (IST(MCESTE+NIVDON)),0.)
            ENDIF 
         ELSEIF (ITYDON.EQ.INTDON ('TABLEAU')) THEN                     !utilite
            NMTBAS = NMDONG(I)
            NVTBAS = NIVDON
            ITTBAS = ITDONN
            IF (ITTBAS.NE.ITCOMP) GOTO 99996
         ENDIF
10    CONTINUE
      IF (NMTBAS.NE.' ') CALL TBRR1  (ERCODE,NMTBAS,NVTBAS,MCTBAS)
*
*--   Symetries pour la fonction de Green
*
      IF (ISYMGR.GT.0) CALL KLSYME (NDIM,ISYMGR,TBISYM)                 !sdexplo
*   
*--   Tableau $WORK$,4 pour les coefficients de la fonction de Green
*
      TYGREE=TYPINT (ITGREE)                                            !utilite
      LGFGRE=1+NDIM+NDIM+NDIM*NDIM
      LGMORE=0
      IF (ISYMGR.GT.0) LGMORE=LGMORE+LGFGRE
      CALL TBCREE ('$WORK$',4,ITGREE,2*NBNOGA+LGFGRE+LGMORE,'c')
*
*--   Creation du vecteur resultat
*
      CALL TBCREE (NMVEC3,NVVEC3,ITVEC3,NBPCOR,'c')
*
      CALL TBAR9  (ERCODE, TABCOR ,NIVCOR,MCBCOR, NMVEC3 ,NVVEC3,MCVEC3
     &                   ,'&NORMA',NVNOGA,MCNOGA,'#ORNOE',INTERP,MCCORN
     &                   , NMVEC1 ,NVVEC1,MCVEC1, NMVEC2 ,NVVEC2,MCVEC2
     &                   ,'#GNEDO',NVNUGA,MCNUGA,'$WORK$',   4  ,MCFGRN
     &                   ,'$WORK$',2 ,MCCST2)
      MCTBAS = 1
      IF (NMTBAS.NE.' ')   CALL TBAR1  (ERCODE,NMTBAS,NVTBAS,MCTBAS)
*     Adresses du noyau de double couche, simple couche et fonction GREEN
      MCGREN=MCFGRN
      MCDGRE=MCGREN+NBNOGA
      MCFGR4=MCDGRE+NBNOGA
*     Adresse du tableau de travail pour le fonction de Green
*     (n'est utile qu'en cas de symetries)   
      MCEGRE=MCFGR4+LGFGRE
*
*     Boucle sur les points ou l'on calcule la representation integrale
*
      ADCOSI=MCBCOR
      TYNOYO='DIR'
      DO 30 IDLSIG=1,NBPCOR
         ADNUGA=MCNUGA+2
         ADNOGA=MCNOGA
         IF (NIVIMP.GT.2) WRITE (IMPSDR,10010) IDLSIG
     &                      ,(RST(I),I=ADCOSI,ADCOGA+NDIM-1)
         IAGREN=MCGREN
         IADGRE=MCDGRE
*
*        Boucle sur les points de GAMMA (Representation integrale)  
         NIVIM1=NIVIMG  
         DO 20 IDLGAM=1,NBNOGA
            ADCOGA=MCCORN+NDIM*(IST(ADNUGA)-1)
            IF (NIVIMG.LT.IDLGAM) NIVIM1=0  
*            IF (NIVIM1.GT.2) WRITE (IMPSDR,10010) IDLGAM   
*     &                      ,(RST(I),I=ADCOGA,ADCOGA+NDIM-1)
*     &                      ,(RST(I),I=ADNOGA,ADNOGA+NDIM-1)
            NIVIM2=NIVIMP   
            IF (NIVIM2.LT.1) NIVIM2=0   
*           Remise a zero du tableau (Partie 'FGREEN')  
            CALL TAZERO (LGFGRE,TYGREE,MCFGR4,AST,IST,RST,CST)          !utilite
*   
*           Calcul des valeurs de la fct. de Green et de ses derivees
*
            IF (ISYMGR.LE.0) THEN
               CALL GREEN  (TYNOYO,CST(MCCST2),RST(ADCOGA),RST(ADCOSI)
     &                     ,CST(MCFGR4),CST(MCTBAS))                    !appl-
            ELSE
               CALL SYMGRE (TYNOYO,NDIM,CST(MCCST2),RST(ADCOGA) 
     &                     ,RST(ADCOSI),TBPSYM,TBISYM,LGFGRE
     &                     ,CST(MCFGR4),CST(MCEGRE),CST(MCTBAS))        !calgre
            ENDIF
*
*           Calcul des noyaux (de type 'DIRICHLET')
            CALL NOYDIR (NDIM,RST(ADNOGA) 
     &                  ,CST(MCFGR4),CST(IAGREN),CST(IADGRE)
     &                  ,NIVIM2,IMPSDR)                                 !calgre
*
            IAGREN=IAGREN+1
            IADGRE=IADGRE+1
            ADNUGA=ADNUGA+1 
            ADNOGA=ADNOGA+NDIM 
20       CONTINUE
*     
         CALL TSCALT (NBNOGA,TYGREE,MCDGRE,RST,CST,TYVEC1,MCVEC1,RST,CST
     &               ,CESUL1)                                           !utilite
         CALL TSCALT (NBNOGA,TYGREE,MCGREN,RST,CST,TYVEC2,MCVEC2,RST,CST
     &               ,CESUL2)                                           !utilite
         IF (ITVEC3.EQ.ITREEL) THEN
            RST(MCVEC3)=RESUL1-RESUL2
         ELSE
            CST(MCVEC3)=CESUL1-CESUL2
         ENDIF
         MCVEC3=MCVEC3+1
*
         ADCOSI=ADCOSI+NDIM
30    CONTINUE
*
      CALL TBTUER ('$WORK$',2)
      CALL TBTUER ('$WORK$',4) 
      IF (NMTBAS.NE.' ') CALL TBSAVE (NMTBAS,NVTBAS)
      CALL TBSAVE ( TABCOR ,NIVCOR) 
      CALL TBSAVE ('&NORMA',NVNOGA)
      CALL TBSAVE ('#GNEDO',NVNUGA)
      CALL TBSAVE ('#ORNOE',INTERP) 
      CALL TBSAVE ( NMVEC1 ,NVVEC1) 
      CALL TBSAVE ( NMVEC2 ,NVVEC2) 
      CALL TBSAVE ( NMVEC3 ,NVVEC3)
*
      CALL PRFXMJ (-1,'*CalRI1*')   
      RETURN
*-----------------------------------------------------------------------
10009 FORMAT (T2,'Calri1',' (Point no',I4,' Coords : ',3E10.4,')')
10010 FORMAT (T3,' (Gamma dl no',I4,' Coords : ',3E10.4
     &,' Normale : ',3E10.4,')')
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
99992 CALL ERTERS (4,NMVEC1,NVVEC1,NMVEC2,NVVEC2)
99993 CALL ERTERS (7,NMVEC1,NVVEC1,NMVEC2,NVVEC2)
99994 CALL PRNTRM (NMVEC1,NVVEC1,ERCODE,NCATR1)
      CALL PRNTRM (NMVEC2,NVVEC2,ERCODE(NCATR1+1:),NCATR2)
      ERCODE(NCATR1+NCATR2+1:)=GAMMA
      CALL BAISE  ('Interpolations incoherentes pour les termes '
     &//ERCODE(1:NCATR1)//', '//ERCODE(NCATR1+1:NCATR1+NCATR2)
     &//' et le domaine '//ERCODE(NCATR1+NCATR2+1:NCATR1+NCATR2+NCHDOM))
99995 CALL PRNTRM (NMVEC1,NVVEC1,ERCODE,NCATR1)
      CALL PRNTRM (NMVEC2,NVVEC2,ERCODE(NCATR1+1:),NCATR2)
      ERCODE(NCATR1+NCATR2+1:)=GAMMA
      CALL BAISE  ('Nombre de noeuds differents pour les termes '
     &//ERCODE(1:NCATR1)//', '//ERCODE(NCATR1+1:NCATR1+NCATR2)
     &//' et le domaine '//ERCODE(NCATR1+NCATR2+1:NCATR1+NCATR2+NCHDOM))
99996 CALL PRNTRM (NMTBAS,NVTBAS,ERCODE,NCATR1)
      CALL BAISE  ('Le tableau '//ERCODE(1:NCATR1)
     &//' est necessairement complexe')
                                                                    END !CalRI1
                                                                    