*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE CALRI2 (TABCOR,NIVCOR,GAMMA,TYNOYO,NBDONG,NMDONG,NMLAMB
     &                  ,NMCST1,NMVEC1,NVVEC1,NMCST2,NMVEC2,NVVEC2
     &                  ,NMVEC3,NVVEC3,NIVIMP)   
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur: D.Martin (Juin 1991) 
* Derniere modification : D.Martin (12 juin 2004)
* Version 2
*   
* Calcul de representation integrale en des points donnes par leurs coordonnees
* avec prise en compte des eventuelles symetries, version complexe,
* sous la forme
*
* V3(M) = C1 * Int_Gamma V1(p).dNG/dn(M,p) dp + C2 * Int_Gamma V2(P).NG(M,p) dp
*
* ou NG(M,p)=G(M,p)                     pour un type de noyau Dirichlet
*           =dG/dnM(M,p)                pour un type de noyau Neuman
*           =dG/dnM(M,p)+Lambda G(M,p)  pour un type de noyau Fourier
*   
*-- Arguments d'entree --  
* TABCOR tableau contenant les points ou l'on veut la R.I.
*        (Attention: la taille de ce tableau definit le nombre de points,
*        Si il y a des normales aux points, elles doivent etre rangees
*        dans le meme ordre a la suite des points dans le tableau TABCOR.
* NIVCOR niveau correspondant
* GAMMA  nom du domaine portant la representation integrale
* TYNOYO type de noyau ('DIRichlet', 'NEUmann' ou 'FOUrier')   
* NBDONG nombre de constantes pour le calcul de la fonction de Green   
* NMDONG tableau des noms des constantes affectes a la fonction de Green   
* NMLAMB nom de la constante associee au calcul du noyau   
* NMCST1 nom de la constante relative au vecteur "densite de double couche" V1
* NMVEC1 nom du (tableau contenant le) vecteur "densite de double couche" V1
* NVVEC1 son niveau
* NMCST2 nom de la constante relative a vecteur "densite de simple couche" V2
* NMVEC2 nom du (tableau contenant le) vecteur "densite de simple couche" V2
* NVVEC2 son niveau
* NMVEC3 nom du (tableau contenant le) vecteur resultat
* NVVEC3 son niveau
* NIVIMP niveau d'impression
* 
*-- Routines utilisees --
* GREEN  calcul de la fonction de Green et de ses derivees
* SYMGRE calcul de Green et de ses derivees avec des eventuelles symetries.
* NOYDIR calcul des noyaux de simple et double couche pour le cas Dirichlet  
* NOYNEU  " Neumann 
* NOYFOU  " Fourier.
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TABCOR,GAMMA,TYNOYO,NMLAMB,NMDONG(*)   
      CHARACTER*(*) NMCST1,NMVEC1,NMCST2,NMVEC2,NMVEC3
      INTEGER       NIVCOR,NBDONG,NVVEC1,NVVEC2,NVVEC3,NIVIMP
*   
      INCLUDE 'ALLOC'   
      INCLUDE 'CONTEX'  
*   
      CHARACTER   ERCODE*120,TYPINT*4
      CHARACTER*4 TYGREE,TYVEC1,TYVEC2
      INTEGER     INTDON,INTTYP,KELCHN,KLTERM
      INTEGER     INTCON,ITENTI,ITREEL,ITCOMP,ITGREE,NIVEAU,LGMORE
     &           ,MCMDOM,MCTRDO,LGERDO,NBDOMK,INCRDO,NCHDOM
     &           ,MCMTRM,MCDTRM,LGETRM,NBTERM,INCTRM,NCHTRM,NUTERM
     &           ,MCINCO,ITYINC,NBTNEU
     &           ,MCMDON,MCDONN,LGEDON,NBDONN,INCDON,NCHDON,ITYDON
     &           ,NIVDON,ITDONN,MCESTE,MCRSTE,MCCSTE
     &           ,ITYDO1,NIV1,ITDON1,ITYDO2,NIV2,ITDON2
     &           ,ITYCAL,NUKALE,NUDONN,NVDSMB,NVDUCV,NVNUCV,NVDULV
     &           ,NVNULV,INSTOK,NVSTOK,IATRGA,NIVIVC
     &           ,ITVEC1,LGVEC1,NVALN1,NUINC1,NNVEC1,NUINL1,NNVEL1
     &           ,INTER1,MCVEC1,NIVIM1,NUCST1
     &           ,ITVEC2,LGVEC2,NVALN2,NUINC2,NNVEC2,NUINL2,NNVEL2
     &           ,INTERP,MCVEC2,NIVIM2,NUCST2,ITVEC3,MCVEC3,MCCORN
     &           ,NUDGGA,NUDOGA,IGEOME,INDMIX,NUSCHQ,MCBCOR,LONCOR
     &           ,INDEXC,IDLHRC,INTRGC,NBNECX,NBNEEX,NBDLEX,NBNEFX
     &           ,NBDLFX,NVNUGC,INDEXL,IDLHRL,INTRGL,NBNECY,NBNEEY
     &           ,NBDLEY,NBNEFY,NBDLFY,NVNOGA,INUTIL,NBTRDO,NIVIMG
     &           ,ADCOGA,MCNOGA,ADNOGA,NBNOGA,MCNUGA,ADNUGA,IDLGAM
     &           ,ADCOSI,MCNOSI,ADNOSI,IDLSIG,NVNUGA,MCTBAS
     &           ,LGFGRE,MCGREN,IAGREN,MCDGRE,IADGRE,MCFGRE
     &           ,NBPCOR,MCFGRN,MCEGRE,MCCSTG,LAMBDE,ITLAMB,NUCSTE
     &           ,NCATR1,NCATR2,I
      REAL        LAMBDR,RCST1,RCST2,RESUL1,RESUL2
      COMPLEX     LAMBDC,CCST1,CCST2,CESUL1,CESUL2
      EQUIVALENCE (RESUL1,CESUL1),(RESUL2,CESUL2)
     &           ,(RCST1,CCST1),(RCST2,CCST2)
      COMMON/FORMAH/ERCODE  
      EQUIVALENCE (ERCODE(73:73),TYVEC1),(ERCODE(81:81),TYVEC2)
     &           ,(ERCODE(89:89),TYGREE)
      EQUIVALENCE (LAMBDE,LAMBDR,LAMBDC)
*
*     Tableaux pour les symetries dans le calcul de la fonction de Green
*     de dimension 3 quelle que soit la dimension d'espace
      INTEGER       TBISYM(3)
      REAL          TBPSYM(3)
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*CalRI2*')
*
      INTCON=INTDON ('CONSTANTE')                                       !utilite
      ITENTI=INTTYP ('ENTIER')                                          !utilite
      ITREEL=INTTYP ('REEL')                                            !utilite
      ITCOMP=INTTYP ('COMPLEXE')                                        !utilite
      ITGREE=ITCOMP
      ITVEC3=ITGREE
*
*     Nombre de points ou l'on calcul la representation integrale
*
      CALL TBRR1  (ERCODE,TABCOR,NIVCOR,LONCOR)
      NBPCOR=LONCOR/NDIM
*     Si il y a des normales aux points elles doivent
*     se trouver a la suite des points dans le tableau TABCOR
      IF (TYNOYO(1:3).NE.'DIR') THEN
         NBPCOR=NBPCOR/2
         LONCOR=LONCOR/2
      ENDIF
*
      CALL TBRR2  (ERCODE,NMVEC1,NVVEC1,LGVEC1,NMVEC2,NVVEC2,LGVEC2)
*
      CALL TBAR5  (ERCODE,'#OMTRM',1,MCMTRM,'$SDTRM',1,MCDTRM
     &                   ,'#OMDOM',1,MCMDOM,'#TERDO',1,MCTRDO
     &                   ,'#NCONU',1,MCINCO)
      CALL SDEXDB (IST(MCTRDO),LGERDO,NBDOMK,INCRDO,NCHDOM)             !sdexplo
      CALL SDEXDB (IST(MCDTRM),LGETRM,NBTERM,INCTRM,NCHTRM)             !sdexplo
*   
*--   Attributs du terme de densite de double couche V1 (NMVEC1)   
*   
      NUTERM=KLTERM (NMVEC1,NVVEC1,AST(MCMTRM),IST(MCDTRM))             !sdexplo
      IF (NUTERM.LE.0) CALL ERTERM (1,NMVEC1,NVVEC1)                    !utilite
      CALL GETTRM (IST(MCDTRM+LGETRM+INCTRM*(NUTERM-1)),NIVEAU
     &            ,ITVEC1,ITYCAL,NUKALE,NUDONN,NVALN1,NVDSMB
     &            ,NUINC1,NVDUCV,NNVEC1,NVNUCV
     &            ,NUINL1,NVDULV,NNVEL1,NVNULV
     &            ,INSTOK,NVSTOK,NVNOGA,IATRGA,NIVIVC)                  !sdexplo
      IF (NUINC1.NE.NDFINC) THEN
          NUINL1=NUINC1
          NNVEL1=NNVEC1
      ENDIF
      CALL SDEXCO (NUINL1,IST(MCINCO),ITYINC,INTER1,NBTNEU)             !sdexplo
      TYVEC1=TYPINT (ITVEC1)                                            !utilite
      IF (ITVEC1.EQ.ITCOMP) ITVEC3=ITVEC1
*
*--   Attributs du terme de densite de simple couche V2 (NMVEC2)   
*   
      NUTERM=KLTERM (NMVEC2,NVVEC2,AST(MCMTRM),IST(MCDTRM))             !sdexplo
      IF (NUTERM.LE.0) CALL ERTERM (1,NMVEC2,NVVEC2)                    !utilite
      CALL GETTRM (IST(MCDTRM+LGETRM+INCTRM*(NUTERM-1)),NIVEAU
     &            ,ITVEC2,ITYCAL,NUKALE,NUDONN,NVALN2,NVDSMB
     &            ,NUINC2,NVDUCV,NNVEC2,NVNUCV 
     &            ,NUINL2,NVDULV,NNVEL2,NVNULV
     &            ,INSTOK,NVSTOK,NVNOGA,IATRGA,NIVIVC)                  !sdexplo
      IF (NUINC2.NE.NDFINC) THEN
          NUINL2=NUINC2
          NNVEL2=NNVEC2
      ENDIF
      CALL SDEXCO (NUINL2,IST(MCINCO),ITYINC,INTERP,NBTNEU)             !sdexplo
      TYVEC2=TYPINT (ITVEC2)                                            !utilite
      IF (ITVEC2.EQ.ITCOMP) ITVEC3=ITVEC2
*
      IF (INTER1.NE.INTERP) GOTO 99992
      IF (NNVEL1.NE.NNVEL2.OR.LGVEC1.NE.LGVEC2) GOTO 99993
*   
*--   Caracteristiques du domaine de representation integrale (GAMMA)
*   
      CALL KLDOMN (GAMMA,0,-1,IST(MCTRDO),AST(MCMDOM),NBDOMG
     &            ,NUDOGA,IATRGA)                                       !sdexplo
      IATRGA=MCTRDO+IATRGA
      CALL GETDOM (IST(IATRGA),NUDGGA,NUDOGA,IGEOME,INDMIX,NUSCHQ
     &            ,INDEXC,IDLHRC,INTRGC,NBNECX,NBNEEX,NBDLEX,NBNEFX
     &            ,NBDLFX,NVNUGC
     &            ,INDEXL,IDLHRL,INTRGL,NBNECY,NBNEEY,NBDLEY,NBNEFY
     &            ,NBDLFY,NVNUGA,NVNOGA,INUTIL,NBTRDO,NIVIMG)           !sdexplo
*
      IF (INTRGL.NE.INTERP) GOTO 99994
*
*--   Tableau des coordonnees, normales sur Gamma
*
      CALL TBRR3  (ERCODE,'#ORNOE',INTERP,MCCORN,'&NORMA',NVNOGA,NBNOGA
     &                   ,'#GNEDO',NVNUGA,MCNUGA)
      NBNOGA=NBNOGA/NDIM
      IF (NNVEL2.NE.NBNOGA) GOTO 99995
*   
*--   Creation du tableau local des valeurs de green et ses derivees
*
      TYGREE=TYPINT (ITGREE)                                            !utilite
      LGFGRE=1+NDIM+NDIM+NDIM*NDIM    
      LGMORE=NBDONG  
      IF (ISYMGR.GT.0) LGMORE=LGMORE+LGFGRE
      CALL TBCREE ('&GREEN',1,ITGREE,2*NBNOGA+LGFGRE+LGMORE,'c') 
*
      CALL TBAR6  (ERCODE,'&GREEN',1,MCFGRN,'#OMDON',1,MCMDON 
     &                   ,'$DONNE',1,MCDONN,'$ECSTE',1,MCESTE
     &                   ,'$RCSTE',1,MCRSTE,'$CCSTE',1,MCCSTE) 
      CALL SDEXDB (IST(MCDONN),LGEDON,NBDONN,INCDON,NCHDON)             !sdexplo
*
*--   Recherche de la constante de couplage Lambda (cas KLNOYO='FOUrier')
*     Sa valeur est placee dans la variable LAMBDC complexe quelque soit
*     le type de la donnee constante LAMBDA
      LAMBDC = 0.   
      IF (TYNOYO(1:3).EQ.'FOU') THEN   
         NUDONN=KELCHN (NMLAMB,AST(MCMDON),NBDONN,NCHDON)               !utilite
         IF (NUDONN.LE.0) CALL ERDONN (1,NMLAMB)                        !utilite
         CALL SDEXCO (NUDONN,IST(MCDONN),ITYDON,NUCSTE,ITLAMB)          !sdexplo
         IF (ITYDON.NE.INTCON) CALL ERDONN (2,NMLAMB)                   !utilite
         CALL OUTCST (NUCSTE,ITLAMB,LAMBDE,LAMBDR,LAMBDC,ERCODE)        !sdexplo
      ENDIF
*   
*     Recherche de la valeur des constantes associees a la fonction de Green
*   
      MCCSTG=MCFGRN+2*NBNOGA+LGFGRE  
      DO 10 I=1,NBDONG 
         NUDONN=KELCHN (NMDONG(I),AST(MCMDON),NBDONN,NCHDON)            !utilite
         IF (NUDONN.LE.0) CALL ERDONN (1,NMDONG(I))                     !utilite
         CALL SDEXCO (NUDONN,IST(MCDONN),ITYDON,NIVDON,ITDONN)          !sdexplo
         IF (ITYDON.EQ.INTCON) THEN 
            IF (ITDONN.EQ.ITCOMP) THEN
               CST(MCCSTG-1+I)=CST(MCCSTE+NIVDON)
            ELSEIF (ITDONN.EQ.ITREEL) THEN
               CST(MCCSTG-1+I)=CMPLX (RST(MCRSTE+NIVDON),0.) 
            ELSEIF (ITDONN.EQ.ITENTI) THEN
               CST(MCCSTG-1+I)=CMPLX (REAL (IST(MCESTE+NIVDON)),0.)
            ENDIF  
         ENDIF 
10    CONTINUE   
*   
*--   Recherche des constantes multiplicatives
*
      CALL TBAR5  (ERCODE,'#OMDON',1,MCMDON
     &                   ,'$DONNE',1,MCDONN,'$ECSTE',1,MCESTE
     &                   ,'$RCSTE',1,MCRSTE,'$CCSTE',1,MCCSTE)
      CALL SDEXDB (IST(MCDONN),LGEDON,NBDONN,INCDON,NCHDON)             !sdexplo
      NUCST1=KELCHN (NMCST1,AST(MCMDON),NBDONN,NCHDON)                  !utilite
      NUCST2=KELCHN (NMCST2,AST(MCMDON),NBDONN,NCHDON)                  !utilite
      CALL SDEXCO (NUCST1,IST(MCDONN),ITYDO1,NIV1,ITDON1)               !sdexplo
      IF (ITDON1.EQ.ITCOMP) ITVEC3=ITDON1
      CALL SDEXCO (NUCST2,IST(MCDONN),ITYDO2,NIV2,ITDON2)               !sdexplo
      IF (ITDON2.EQ.ITCOMP) ITVEC3=ITDON2
      IF (ITYDO1.NE.INTCON) CALL ERDONN (2,NMCST1)
      IF (ITYDO2.NE.INTCON) CALL ERDONN (2,NMCST2)
*
      IF (ITDON1.EQ.ITCOMP) THEN
         CCST1=CST(MCCSTE+NIV1)
      ELSEIF (ITDON1.EQ.ITREEL) THEN
         CCST1=CMPLX (RST(MCRSTE+NIV1),0.)
      ELSEIF (ITDON1.EQ.ITENTI) THEN
         CCST1=CMPLX (REAL (IST(MCESTE+NIV1)),0.)
      ENDIF
*
      IF (ITDON2.EQ.ITCOMP) THEN
         CCST2=CST(MCCSTE+NIV2)
      ELSEIF (ITDON2.EQ.ITREEL) THEN
         CCST2=CMPLX (RST(MCRSTE+NIV2),0.)
      ELSEIF (ITDON2.EQ.ITENTI) THEN
         CCST2=CMPLX (REAL (IST(MCESTE+NIV2)),0.)
      ENDIF
*
*--   Symetries pour la fonction de Green
*  
      IF (ISYMGR.GT.0) CALL KLSYME (NDIM,ISYMGR,TBISYM)                 !sdexplo
*
*--   Creation du vecteur resultat
*
      CALL TBCREE (NMVEC3,NVVEC3,ITVEC3,NBPCOR,'c')
*
      ERCODE(1:1)='!'
      CALL TBAR8  (ERCODE, TABCOR ,NIVCOR,MCBCOR, NMVEC3 ,NVVEC3,MCVEC3
     &                   ,'&NORMA',NVNOGA,MCNOGA,'#ORNOE',INTERP,MCCORN
     &                   , NMVEC1 ,NVVEC1,MCVEC1, NMVEC2 ,NVVEC2,MCVEC2
     &                   ,'#GNEDO',NVNUGA,MCNUGA,'&GREEN',   1  ,MCFGRN)
*
      MCNOSI=MCBCOR+LONCOR   
*     Adresses du noyau de double couche, simple couche et fonction GREEN
      MCGREN=MCFGRN
      MCDGRE=MCGREN+NBNOGA
      MCFGRE=MCDGRE+NBNOGA  
*     Adresse de la premiere constante associee a la fonction de Green   
      MCCSTG=MCFGRE+LGFGRE   
*     Adresse du tableau de travail pour le fonction de Green
*     (n'est utile qu'en cas de symetries)   
      MCEGRE=MCCSTG+NBDONG   
*
*     Boucle sur les points ou l'on calcule la representation integrale
*
      ADCOSI=MCBCOR
      ADNOSI=MCNOSI
      DO 30 IDLSIG=1,NBPCOR
         ADNUGA=MCNUGA+2
         ADNOGA=MCNOGA
         IAGREN=MCGREN
         IADGRE=MCDGRE
*
*        Boucle sur les points de GAMMA (Representation integrale)  
         NIVIM1=NIVIMG  
         DO 20 IDLGAM=1,NBNOGA  
            ADCOGA=MCCORN+NDIM*(IST(ADNUGA)-1)
            IF (NIVIMG.LT.IDLGAM) NIVIM1=0  
            IF (NIVIM1.GT.2) WRITE (IMPSDR,10010) IDLGAM   
     &                             ,(RST(I),I=ADNOGA,ADNOGA+NDIM-1)
            NIVIM2=NIVIMP   
            IF (NIVIM2.LT.1) NIVIM2=0   
*           Remise a zero du tableau '&GREEN' (Partie 'FGREEN')  
            CALL TAZERO (LGFGRE,TYGREE,MCFGRE,AST,IST,RST,CST)          !utilite
*   
*           Calcul des valeurs de la fct. de Green et de ses derivees  
*   
            MCTBAS=MCCSTG
            IF (ISYMGR.LE.0) THEN 
               CALL GREEN  (TYNOYO,CST(MCCSTG),RST(ADCOGA),RST(ADCOSI)
     &                     ,CST(MCFGRE),CST(MCTBAS))                    !appl-
            ELSE 
               CALL SYMGRE (TYNOYO,NDIM,CST(MCCSTG),RST(ADCOGA) 
     &                     ,RST(ADCOSI),TBPSYM,TBISYM,LGFGRE 
     &                     ,CST(MCFGRE),CST(MCEGRE),CST(MCTBAS))        !calgre 
            ENDIF   
*   
*           Calcul proprement dit des noyaux   
*   
            IF (TYNOYO(1:3).EQ.'FOU') THEN 
*              noyaux pour le couplage 'FOURIER'  
               CALL NOYFOU (NDIM,RST(ADNOGA),RST(ADNOSI),LAMBDC
     &                     ,CST(MCFGRE),CST(IAGREN),CST(IADGRE)
     &                     ,NIVIM2,IMPSDR)                              !calgre 
            ELSEIF (TYNOYO(1:3).EQ.'NEU') THEN   
*              noyaux pour le couplage 'NEUMANN'  
               CALL NOYNEU (NDIM,RST(ADNOGA),RST(ADNOSI)
     &                     ,CST(MCFGRE),CST(IAGREN),CST(IADGRE)
     &                     ,NIVIM2,IMPSDR)                              !calgre 
            ELSE 
*              noyaux pour le couplage 'DIRICHLET'
               CALL NOYDIR (NDIM,RST(ADNOGA) 
     &                     ,CST(MCFGRE),CST(IAGREN),CST(IADGRE)
     &                     ,NIVIM2,IMPSDR)                              !calgre 
            ENDIF   
            IAGREN=IAGREN+1
            IADGRE=IADGRE+1
            ADNUGA=ADNUGA+1
            ADNOGA=ADNOGA+NDIM  
20       CONTINUE
*   
         CALL TSCALT (NBNOGA,TYGREE,MCDGRE,RST,CST,TYVEC1,MCVEC1,RST,CST
     &               ,CESUL1)                                           !utilite
         CALL TSCALT (NBNOGA,TYGREE,MCGREN,RST,CST,TYVEC2,MCVEC2,RST,CST
     &               ,CESUL2)                                           !utilite
         IF (ITVEC3.EQ.ITREEL) THEN
            RST(MCVEC3)=RCST1*RESUL1+RCST2*RESUL2
         ELSE
            CST(MCVEC3)=CCST1*CESUL1+CCST2*CESUL2
         ENDIF
         MCVEC3=MCVEC3+1
         ADNOSI=ADNOSI+NDIM
         ADCOSI=ADCOSI+NDIM
30    CONTINUE
*
      CALL TBTUER ('&GREEN',1) 
      CALL TBSAVE ( TABCOR ,NIVCOR)
      CALL TBSAVE ('&NORMA',NVNOGA) 
      CALL TBSAVE ('#GNEDO',NVNUGA)  
      CALL TBSAVE ('#ORNOE',INTERP)
      CALL TBSAVE ( NMVEC1 ,NVVEC1) 
      CALL TBSAVE ( NMVEC2 ,NVVEC2) 
      CALL TBSAVE ( NMVEC3 ,NVVEC3)
*
      CALL PRFXMJ (-1,'*CalRI2*')
      RETURN
*-----------------------------------------------------------------------
10010 FORMAT (T3,' (d.l. de Gamma no',I4,' Normale : ',3E10.4,')')
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
99992 CALL ERTERS (4,NMVEC1,NVVEC1,NMVEC2,NVVEC2)
99993 CALL ERTERS (7,NMVEC1,NVVEC1,NMVEC2,NVVEC2)
99994 CALL PRNTRM (NMVEC1,NVVEC1,ERCODE,NCATR1)
      CALL PRNTRM (NMVEC2,NVVEC2,ERCODE(NCATR1+1:),NCATR2)
      ERCODE(NCATR1+NCATR2+1:)=GAMMA
      CALL BAISE  ('Interpolations incoherentes pour les termes '
     &//ERCODE(1:NCATR1)//', '//ERCODE(NCATR1+1:NCATR1+NCATR2)
     &//' et le domaine '//ERCODE(NCATR1+NCATR2+1:NCATR1+NCATR2+NCHDOM))
99995 CALL PRNTRM (NMVEC1,NVVEC1,ERCODE,NCATR1)
      CALL PRNTRM (NMVEC2,NVVEC2,ERCODE(NCATR1+1:),NCATR2)
      ERCODE(NCATR1+NCATR2+1:)=GAMMA
      CALL BAISE  ('Nombre de noeuds differents pour les termes '
     &//ERCODE(1:NCATR1)//', '//ERCODE(NCATR1+1:NCATR1+NCATR2)
     &//' et le domaine '//ERCODE(NCATR1+NCATR2+1:NCATR1+NCATR2+NCHDOM))
                                                                    END !CalRI2
