*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE NOYVC (SIGMA,NMGREE,NVGREE,NMVEC1,NVVEC1,NMVEC3,NVVEC3
     &                 ,NIVIMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin (15 decembre 2006) 
* Derniere modification : D.Martin (15 decembre 2006)
* Version 0
*  
* Calcul d'un produit Noyau de Green par un vecteur:
*
*              V3(M) = NG(M,p).V1(p) (M sur SIGMA, p sur Gamma)
*
* ou NG(M,p) est le noyau de Green 
*  
*-- Arguments d'entree -- 
* SIGMA  nom du domaine geometrique ou l'on calcule la representation integrale
* NMGREE nom du tableau contenant la matrice noyau de Green
* NVGREE le niveau de ce tableau
* NMVEC1 nom du terme contenant le vecteur operande (V1)
* NVVEC1 son niveau
* NMVEC3 nom du terme contenant le vecteur resultat (V3)
* NVVEC3 son niveau
* NIVIMP niveau d'impression du vecteur resultat
* Remarque : Le domaine d'integration de la representation integrale (Gamma)
*            est implicitement contenu dans le calcul des noyaux de Green
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) SIGMA,NMGREE,NMVEC1,NMVEC3
      INTEGER       NVGREE,NVVEC1,NVVEC3,NIVIMP
*  
      INCLUDE 'CONTEX' 
      INCLUDE 'ALLOC'  
*  
      CHARACTER  ERCODE*120,TYPINT*4
      CHARACTER*4 TYGREE,TYVEC1,TYVEC3
      INTEGER    INDSTO,INTYKL,KLTERM
      INTEGER    ITGREE,ITVEC3,IPRLIG,IDRLIG,NWHTRM
     &          ,LGMTRM,MCMTRM,MCMDOM,MCTRDO,IATRSI
     &          ,NUDGSI,NUDOSI,IGEOME,INDMIX,INDEXC,IDLHRC,NVNUSI
     &          ,INDEXL,IDLHRL,NVNUSL,NVCOSI,INUTIL,NBTRDO,NIVISI
     &          ,LGDTRM,MCDTRM,LGETRM,NBTERM,INCTRM,NCHTRM,NUTERM
     &          ,ITYVEC,ITYCAL,NUKALE,NUDONN,NVALNE,IPTDIR 
     &          ,NUINCV,NUDUCV,NBNECV,NVNUCV,NUINLV,NVDULV,NBNOGA,NVNULV
     &          ,INSTOK,NVSTOK,NVCOOO,IATRGA,NIVIVC,NIVEAU
     &          ,NVVOLD,ITYOLD,NUKOLD,NVSOLD,NCOOLD,IATOLD,NIVOLD
     &          ,NUCOLD,NDCOLD,NBCOLD,NVCOLD,NULOLD,NDLOLD,NBLOLD,NVLOLD     
     &          ,MCNUSI,NBNOSI,NUTRVC,MCGREE,NUGREN,MCVEC3,MCVEC1,I
      COMMON/FORMAH/ERCODE 
      EQUIVALENCE (ERCODE(97:97),TYGREE),(ERCODE(105:105),TYVEC1)
     &           ,(ERCODE(113:113),TYVEC3)
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*Noyvc* ')
*  
*     Type (REEL ou COMPLEXE des noyaux de Green)  
*
      CALL TBTYPE (NMGREE,NVGREE,ITGREE)
      TYGREE=TYPINT (ITGREE)                                            !utilite
      ITVEC3=ITGREE
*  
      CALL TBRR2  (ERCODE,'#OMTRM',1,LGMTRM,'$SDTRM',1,LGDTRM)
      CALL TBAR4  (ERCODE,'#OMTRM',1,MCMTRM,'$SDTRM',1,MCDTRM
     &                   ,'#OMDOM',1,MCMDOM,'#TERDO',1,MCTRDO)
      CALL SDEXDB (IST(MCDTRM),LGETRM,NBTERM,INCTRM,NCHTRM)             !sdexplo
*  
*     Caracteristiques du domaine de representation integrale en colonne
*     pour decrire les caracteristiques en ligne de la matrice NOYAU
*  
      CALL KLDOMN (SIGMA,0,-1,IST(MCTRDO),AST(MCMDOM),NBDOMG
     &            ,NUDOSI,IATRSI)                                       !sdexplo
      IATRSI=MCTRDO+IATRSI 
      CALL GETDOR (IST(IATRSI),NUDGSI,NUDOSI,IGEOME,INDMIX 
     &            ,INDEXC,IDLHRC,NVNUSI,INDEXL,IDLHRL,NVNUSL
     &            ,NVCOSI,INUTIL,NBTRDO,NIVISI)                         !sdexplo
*  
*     Recherche des attributs du terme NMVEC1  
*  
      NUTRVC=KLTERM (NMVEC1,NVVEC1,AST(MCMTRM),IST(MCDTRM))             !sdexplo
      IF (NUTRVC.LE.0) CALL ERTERM (1,NMVEC1,NVVEC1)                    !utilite
      CALL GETTRM (IST(MCDTRM+LGETRM+INCTRM*(NUTRVC-1)),NIVEAU
     &            ,ITYVEC,INSTOK,NUKALE,NUDONN,NVALNE,IPTDIR 
     &            ,NUINCV,NUDUCV,NBNECV,NVNUCV
     &            ,NUINLV,NVDULV,NBNOGA,NVNULV
     &            ,INSTOK,NVSTOK,NVCOOO,IATRGA,NIVIVC)                  !sdexplo
      TYVEC1=TYPINT (ITYVEC)                                            !utilite
      IF (TYVEC1.EQ.'COMP') ITVEC3=ITYVEC
      TYVEC3=TYPINT (ITVEC3)                                            !utilite
* 
*--   Creation du Vecteur resultat 
      CALL TBRR1  (ERCODE,'#GNEDO',NVNUSI,MCNUSI)
      CALL TBAR1  (ERCODE,'#GNEDO',NVNUSI,MCNUSI)
      NBNOSI=IST(MCNUSI+1)
*
*     Introduction du resultat dans les structures #OMTRM et $SDTRM
      CALL TBAR2  (ERCODE,'$SDTRM',1,MCDTRM,'#OMTRM',1,MCMTRM)
      NUTERM=KLTERM (NMVEC3,NVVEC3,AST(MCMTRM),IST(MCDTRM))             !sdexplo
      IF (NUTERM.LE.0) THEN 
         CALL ALFANU (NBCHNT,NMVEC3)                                    !utilite
         NWHTRM=MAX (NCHTRM,LEN(NMVEC3))
         CALL SDAJST ('$SDTRM',1,LGDTRM,LGETRM,INCTRM,NBTERM,1)         !sdexplo
         CALL SDAJST ('#OMTRM',1,LGMTRM,0,NWHTRM,NBTERM,1)              !sdexplo
         CALL TBAR2  (ERCODE,'#OMTRM',1,MCMTRM,'$SDTRM',1,MCDTRM)
         CALL RIPCHN (AST(MCMTRM),NBTERM,NCHTRM,NMVEC3)                 !utilite
         CALL SDMKDB (IST(MCDTRM),LGETRM,NBTERM,INCTRM,NCHTRM)          !sdexplo
         NUTERM=NBTERM 
         IATOLD=0  
         NVSOLD=0  
         NUDONN=0
      ELSE 
         CALL GETTRM (IST(MCDTRM+LGETRM+INCTRM*(NUTERM-1)),NVVOLD
     &               ,ITYOLD,ITYCAL,NUKOLD,NUDONN,NVALNE,IPTDIR
     &               ,NUCOLD,NDCOLD,NBCOLD,NVCOLD
     &               ,NULOLD,NDLOLD,NBLOLD,NVLOLD
     &               ,INSTOK,NVSOLD,NCOOLD,IATOLD,NIVOLD)               !sdexplo
      ENDIF
*  
      ITYCAL=INTYKL ('ASSEMB')                                          !utilite
      INSTOK=INDSTO ('PLEIN-L')                                         !utilite
      IF (NVCOSI.LE.0) NVCOSI=NCOOLD
      CALL PUTTRM (IST(MCDTRM+LGETRM+INCTRM*(NUTERM-1)),NVVEC3
     &            ,ITVEC3,ITYCAL,1,NUDONN,NVALNE,NDFDSM
     &            ,NDFINC,NDFDUM,1     ,NDFNUM
     &            ,NUINLV,NDFDUM,NBNOSI,NVNUSI
     &            ,INSTOK,NDFSTO,NVCOSI,IATOLD,NIVIMP)                  !sdexplo
*  
      CALL TBCREE (NMVEC3,NVVEC3,ITVEC3,NBNOSI,'c')
      CALL TBAR1  (ERCODE,NMVEC3,NVVEC3,MCVEC3)
      CALL TAZERO (NBNOSI,TYVEC3,MCVEC3,AST,IST,RST,CST)                !utilite
*  
*--   Multiplication NOYAU*NMVEC1 dans NMVEC3
*
      CALL TBRR2  (ERCODE,NMGREE,NVGREE,MCGREE,NMVEC1,NVVEC1,MCVEC1)
      CALL TBAR3  (ERCODE,NMGREE,NVGREE,MCGREE,NMVEC1,NVVEC1,MCVEC1
     &                      ,NMVEC3,NVVEC3,MCVEC3)
*
      CALL PLXPL  (TYGREE,NBNOSI,NBNOGA,'PLEIN-C',MCGREE
     &            ,TYVEC1,1,'PLEIN-L',MCVEC1,'PLEIN-L',MCVEC3,RST,CST)  !multmat
*      CALL PLVC   (NBNOSI,NBNOGA,TYGREE,MCGREE
*     &            ,TYVEC1,MCVEC1,TYVEC3,MCVEC3,RST,CST)                 !multmat
      CALL TBSAVE (NMGREE,NVGREE)
*  
*     Sauvegarde du vecteur multiplicatif  
      CALL TBSAVE (NMVEC1,NVVEC1)
*     Impression et sauvegarde du vecteur resultat 
      IF (NIVIMP.GE.0) CALL PRTERM (NMVEC3,NVVEC3,NIVIMP,IMPSDR)        !prsd
      CALL TBSAVE (NMVEC3,NVVEC3)
      CALL PRFXMJ (-1,'*Noyvc* ')  
                                                                    END !Noyvc
