*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE HEARFA (NUFACE,NUARFA)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (janvier 2002)
* Derniere modification : D.Martin (21 juin 2002)
* Version 1.0.1
*
* Retourne les numeros (locaux) des aretes bordant une face d'un hexaedre
*
*-- Variable d'entree --
* NUFACE numero de face
*
*-- Variables de sortie --
* NUARFA numeros signes des aretes des faces i.e.
*        NUARFA(i) est le numero de l'arete i de la face NUFACE dans l'hexaedre
*        avec le signe + ou -.
*        Le signe - indique que les numeros des noeuds extremites 
*        et les non sommets de l'arete sont ranges selon un ordre de croissance 
*        des numeros locaux dans l'element de reference inverse, par exemple
*        * Sur l'arete 4 (x=y=0) les extremites sont 4 et 8 
*          et pour l'element de Lagrange Q4
*          la numerotation locale est  4  36  24  12  8, d'ou -4,
*        * sur l'arete 1 (x=y=1) on a  2   9  21  22  6, d'ou  1
*          pour un element Q4.
*
*  Numerotations des sommets, aretes et faces
*
*                         Arete s1 s2 equation |  Face  s1  s2  s3  s4  eq.
*       8------IX-----7     1    2  6 x=1 y=1  |    1    2   6   5   1  x=1
*      /|            /|     2    7  6 y=1 z=1  |    2    7   6   2   3  y=1
*    XI |          II |     3    5  6 z=1 x=1  |    3    5   6   7   8  z=1
*   /   IV        /   X     4    8  4 x=0 y=0  |    4    3   4   8   7  x=0
*  5------III----6    |     5    1  4 x=0 y=0  |    5    8   4   1   5  y=0
*  |    |        |    |     6    3  4 x=0 y=0  |    6    1   4   3   2  z=0
*  |    4-----VI-|----3     7    1  5 y=1 z=0  |---------------------------
* VII  /        1|   /      8    2  3 z=1 x=0  |  Face  a1  a2  a3  a4
*  |  V          | VIII     9    7  8 x=1 y=0  |    1    1  -3  -7  12
*  |/            |/        10    3  7 y=0 z=1  |    2    2  -1   8  10
*  1-----XII-----2         11    5  8 z=0 x=1  |    3    3  -2   9 -11
*                          12    1  2 x=0 y=1  |    4    6  -4  -9 -10
*                                              |    5    4  -5   7  11
*                                              |    6    5  -6  -8 -12
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      INTEGER  NUFACE,NUARFA(*)
*------------------------------------------------------------------------
*
*     Numerotation locale des aretes
      IF (NUFACE.EQ.1) THEN
*        Face 1 (x1 = 1)
         NUARFA(1) =  1
         NUARFA(2) = -3
         NUARFA(3) = -7
         NUARFA(4) = 12
      ELSEIF (NUFACE.EQ.2) THEN
*        Face 2 (x2 = 1)
         NUARFA(1) =  2
         NUARFA(2) = -1
         NUARFA(3) =  8
         NUARFA(4) = 10
      ELSEIF (NUFACE.EQ.3) THEN
*        Face 3 (x3 = 1)
         NUARFA(1) =  3
         NUARFA(2) = -2
         NUARFA(3) =  9
         NUARFA(4) =-11
      ELSEIF (NUFACE.EQ.4) THEN
*        Face 4 (x1 = 0)
         NUARFA(1) =  6
         NUARFA(2) = -4
         NUARFA(3) = -9
         NUARFA(4) =-10
      ELSEIF (NUFACE.EQ.5) THEN
*        Face 5 (x2 = 0)
         NUARFA(1) =  4
         NUARFA(2) = -5
         NUARFA(3) =  7
         NUARFA(4) = 11
      ELSEIF (NUFACE.EQ.6) THEN
*        Face 6 (x3 = 0)
         NUARFA(1) =  5
         NUARFA(2) = -6
         NUARFA(3) = -8
         NUARFA(4) =-12
      ENDIF
                                                                    END !Hearfa

