*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE HELKAR (TYPE,NUARET,NUTYAR,NBNEAR,NUNEAR)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Fevrier 1988)
* Derniere modification : D.Martin (21 juin 2002)
* Version 1.1.1
*  
* Numerotation locale des noeuds d'une arete de l'element de reference 
* Hexaedre de Lagrange (ordre k) a (k+1)^3 noeuds
*
*-- Variables d'entree --
* TYPE   =K pour hexaedre de Lagrange de type k
* NUARET numero de l'arete
*
*-- Variables de sortie --
* NUTYAR numero de type de l'arete (numero de type de l'element 1d associe)
* NBNEAR nombre de noeuds de l'arete
* NUNEAR tableau des numeros locaux des noeuds de l'arete
*
*
*                         Arete s1 s2 equation |  Face  s1  s2  s3  s4  eq.
*       8------IX-----7     1    2  6 x=1 y=1  |    1    2   6   5   1  x=1
*      /|            /|     2    7  6 y=1 z=1  |    2    7   6   2   3  y=1
*    XI |          II |     3    5  6 z=1 x=1  |    3    5   6   7   8  z=1
*   /   IV        /   X     4    8  4 x=0 y=0  |    4    3   4   8   7  x=0
*  5------III----6    |     5    1  4 x=0 y=0  |    5    8   4   1   5  y=0
*  |    |        |    |     6    3  4 x=0 y=0  |    6    1   4   3   2  z=0
*  |    4-----VI-|----3     7    1  5 y=1 z=0  |---------------------------
* VII  /        1|   /      8    2  3 z=1 x=0  |  Face  a1  a2  a3  a4
*  |  V          | VIII     9    7  8 x=1 y=0  |    1    1  -3  -7  12
*  |/            |/        10    3  7 y=0 z=1  |    2    2  -1   8  10
*  1-----XII-----2         11    5  8 z=0 x=1  |    3    3  -2   9 -11
*                          12    1  2 x=0 y=1  |    4    6  -4  -9 -10
*                                              |    5    4  -5   7  11
*                                              |    6    5  -6  -8 -12
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      INTEGER  TYPE,NUARET,NUTYAR,NBNEAR,NUNEAR(*)
*
      INTEGER  TYPNEW,MORE,I,LAGDMX,NTYPFO
      PARAMETER ( LAGDMX = 100, NTYPFO = 1000 )
*------------------------------------------------------------------------
      TYPNEW=TYPE
      IF (TYPNEW.GT.LAGDMX) TYPNEW=TYPNEW-LAGDMX
      IF (TYPNEW.EQ.202) TYPNEW=2
*
      IF (NUARET.EQ.1) THEN
*        Arete 1 (x = 1, y = 1)
         NUNEAR(1)=2 
         NUNEAR(2)=6
      ELSEIF (NUARET.EQ.2) THEN
*        Arete 1 (y = 1, z = 1)
         NUNEAR(1)=7 
         NUNEAR(2)=6
      ELSEIF (NUARET.EQ.3) THEN
*        Arete 1 (z = 1, x = 1)
         NUNEAR(1)=5
         NUNEAR(2)=6
      ELSEIF (NUARET.EQ.4) THEN
*        Arete 1 (x = 0, y = 0)
         NUNEAR(1)=8
         NUNEAR(2)=4
      ELSEIF (NUARET.EQ.5) THEN
*        Arete 1 (y = 0, z = 0)
         NUNEAR(1)=1
         NUNEAR(2)=4
      ELSEIF (NUARET.EQ.6) THEN
*        Arete 1 (z = 0, x = 0)
         NUNEAR(1)=3
         NUNEAR(2)=4
      ELSEIF (NUARET.EQ.7) THEN
*        Arete 1 (x = 1, y = 0)
         NUNEAR(1)=1
         NUNEAR(2)=5
      ELSEIF (NUARET.EQ.8) THEN
*        Arete 1 (y = 1, z = 0)
         NUNEAR(1)=2
         NUNEAR(2)=3
      ELSEIF (NUARET.EQ.9) THEN
*        Arete 1 (z = 1, x = 0)
         NUNEAR(1)=7
         NUNEAR(2)=8
      ELSEIF (NUARET.EQ.10) THEN
*        Arete 1 (x = 0, y = 1)
         NUNEAR(1)=3
         NUNEAR(2)=7
      ELSEIF (NUARET.EQ.11) THEN
*        Arete 1 (y = 0, z = 1)
         NUNEAR(1)=5
         NUNEAR(2)=8
      ELSEIF (NUARET.EQ.12) THEN
*        Arete 1 (z = 0, x = 1)
         NUNEAR(1)=1
         NUNEAR(2)=2
      ENDIF
*
*     Numerotation des noeuds non-sommets sur les aretes
*
      MORE=8
      DO 1 I=3,TYPNEW+1
         NUNEAR(I)=NUARET+MORE
         MORE=MORE+12
1     CONTINUE
*
      NUTYAR=10*NTYPFO+TYPE
      NBNEAR=TYPNEW+1
                                                                    END !Helkar
