*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE HEXAQA (NUARET,NBPTQ,POIDQ1,COORQ1,POIDQ3,COORQ3)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Fevrier 1994)
* Derniere modification : D.Martin (03 Decembre 2001)
* Version 1.1.0
*
* Formule de quadrature sur une arete de l'hexaedre de reference 
* construite a partir d'une formule de quadrature sur le segment.
*
*-- Arguments d'entree --
* NUARET numero de l'arete de l'element
* NBPTQ  nombre de points de la formule de quadrature du segment
* POIDQ1 poids de la formule de quadrature sur le segment
* COORQ1 coordonnees des points de la formule de quadrature sur le segment
* POIDQ3 poids de la formule de quadrature sur l'arete de l'hexaedre
* COORQ3 coordonnees des points de la formule de quadrature sur l'arete
*
*  Numerotations des sommets, aretes et faces
*                         Arete s1 s2         |  Face  s1  s2  s3  s4  eq.
*       8------IX-----7     1    2  6 x=1 y=1 |    1    2   6   5   1  x=1
*      /|            /|     2    7  6 y=1 z=1 |    2    7   6   2   3  y=1
*    XI |          II |     3    5  6 z=1 x=1 |    3    5   6   7   8  z=1
*   /   IV        /   X     4    8  4 x=0 y=0 |    4    3   4   8   7  x=0
*  5------III----6    |     5    1  4 x=0 y=0 |    5    8   4   1   5  y=0
*  |    |        |    |     6    3  4 x=0 y=0 |    6    1   4   3   2  z=0
*  |    4-----VI-|----3     7    1  5 y=1 z=0 |---------------------------
* VII  /        1|   /      8    2  3 z=1 x=0 |  Face  a1  a2  a3  a4
*  |  V          | VIII     9    7  8 x=1 y=0 |    1    1  -3  -7  12
*  |/            |/        10    3  7 y=0 z=1 |    2    2  -1   8  10
*  1-----XII-----2         11    5  8 z=0 x=1 |    3    3  -2   9 -11
*                          12    1  2 x=0 y=1 |    4    6  -4  -9 -10
*                                             |    5    4  -5   7  11
*                                             |    6    5  -6  -8 -12
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      INTEGER NUARET,NBPTQ
      REAL    POIDQ1(*),COORQ1(*),POIDQ3(*),COORQ3(3,*)
*
      INTEGER I
*-----------------------------------------------------------------------
      IF (NUARET.EQ.1) THEN
*        Arete 1 : droite x1=1.,x2=1.
         DO 1 I=1,NBPTQ
            COORQ3(1,I) = 1.
            COORQ3(2,I) = 1.
            COORQ3(3,I) = COORQ1(I)
            POIDQ3(I)   = POIDQ1(I)
1        CONTINUE
      ELSEIF (NUARET.EQ.2) THEN
*        Arete 2 : droite x2=1.,x3=1.
         DO 2 I=1,NBPTQ
            COORQ3(1,I) = COORQ1(I)
            COORQ3(2,I) = 1.
            COORQ3(3,I) = 1.
            POIDQ3(I)   = POIDQ1(I)
2        CONTINUE
      ELSEIF (NUARET.EQ.3) THEN
*        Arete 3 : droite x1=1.,x3=1.
         DO 3 I=1,NBPTQ
            COORQ3(1,I) = 1.
            COORQ3(2,I) = COORQ1(I)
            COORQ3(3,I) = 1.
            POIDQ3(I)   = POIDQ1(I)
3        CONTINUE
      ELSEIF (NUARET.EQ.4) THEN
*        Arete 4 : droite x1=0.,x2=0.
         DO 4 I=1,NBPTQ
            COORQ3(1,I) = 0.
            COORQ3(2,I) = 0.
            COORQ3(3,I) = COORQ1(I)
            POIDQ3(I)   = POIDQ1(I)
4        CONTINUE
      ELSEIF (NUARET.EQ.5) THEN
*        Arete 5 : droite x2=0.,x3=0.
         DO 5 I=1,NBPTQ
            COORQ3(1,I) = COORQ1(I)
            COORQ3(2,I) = 0.
            COORQ3(3,I) = 0.
            POIDQ3(I)   = POIDQ1(I)
5        CONTINUE
      ELSEIF (NUARET.EQ.6) THEN
*        Arete 6 : droite x1=0.,x3=0.
         DO 6 I=1,NBPTQ
            COORQ3(1,I) = 0.
            COORQ3(2,I) = COORQ1(I)
            COORQ3(3,I) = 0.
            POIDQ3(I)   = POIDQ1(I)
6        CONTINUE
      ELSEIF (NUARET.EQ.7) THEN
*        Arete 7 : droite x1=1.,x2=0.
         DO 7 I=1,NBPTQ
            COORQ3(1,I) = 1.
            COORQ3(2,I) = 0.
            COORQ3(3,I) = COORQ1(I)
            POIDQ3(I)   = POIDQ1(I)
7        CONTINUE
      ELSEIF (NUARET.EQ.8) THEN
*        Arete 8 : droite x2=1.,x3=0.
         DO 8 I=1,NBPTQ
            COORQ3(1,I) = COORQ1(I)
            COORQ3(2,I) = 1.
            COORQ3(3,I) = 0.
            POIDQ3(I)   = POIDQ1(I)
8       CONTINUE
      ELSEIF (NUARET.EQ.9) THEN
*        Arete 9 : droite x1=0.,x3=1.
         DO 9 I=1,NBPTQ
            COORQ3(1,I) = 0.
            COORQ3(2,I) = COORQ1(I)
            COORQ3(3,I) = 1.
            POIDQ3(I)   = POIDQ1(I)
9        CONTINUE
      ELSEIF (NUARET.EQ.10) THEN
*        Arete 10 : droite x1=0.,x2=1.
         DO 10 I=1,NBPTQ
            COORQ3(1,I) = 0.
            COORQ3(2,I) = 1.
            COORQ3(3,I) = COORQ1(I)
            POIDQ3(I)   = POIDQ1(I)
10     CONTINUE
      ELSEIF (NUARET.EQ.11) THEN
*        Arete 11 : droite x2=0.,x3=1.
         DO 11 I=1,NBPTQ
            COORQ3(1,I) = COORQ1(I)
            COORQ3(2,I) = 0.
            COORQ3(3,I) = 1.
            POIDQ3(I)   = POIDQ1(I)
11       CONTINUE
      ELSEIF (NUARET.EQ.12) THEN
*        Arete 12 : droite x1=1.,x3=0.
         DO 12 I=1,NBPTQ
            COORQ3(1,I) = 1.
            COORQ3(2,I) = COORQ1(I)
            COORQ3(3,I) = 0.
            POIDQ3(I)   = POIDQ1(I)
12       CONTINUE

      ELSE
         CALL ERELEM (5,'HexaQa',NUARET)                                !Utilite
      ENDIF
                                                                    END !HexaQa
