*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE HEXAQD (NDIM,NUMERO,NBPTQ,POIDQ,COORQ,DEXACT)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Fevrier 1988)
* Derniere modification : D.Martin (06 juillet 2005)
* Version 2
*  
* Formules de quadrature sur l'hexaedre [0.,1.] x [0.,1.] x [0.,1.]
* (programme pour que les valeurs des coeff. soit corrects en  double precision
*  pour les compilations implicites en double precision)
*  
*-- Variables d'entree -- 
* NDIM   dimension d'espace  
* NUMERO numero du schema de quadrature (convention)
*       (par convention = degre de precision pour les formules de Gauss)
*-- Variables en sortie --
* NBPTQ  nombre de points du schema de quadrature
* POIDQ  tableau des poids du schema de quadrature
* COORQ  tableau des coordonnees des points de quadrature
* DEXACT degre de precision de la formule (pour impression)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      INTEGER NDIM,NUMERO,NBPTQ,DEXACT
      REAL    POIDQ(*),COORQ(NDIM,*)
*
      INTEGER I,J,K,NSAUT,NSAUTI,NSAUTJ,NSAUTK,NUPTQ,NBPTQ1
      REAL    X,Y,P1,P2,P3,P4
      INTEGER DEGMAX
      PARAMETER ( DEGMAX = 99 )
*-----------------------------------------------------------------------
      IF (NUMERO.EQ.-1) THEN
*
*        Formule 'nodale' utilisant les noeuds de l'element Q1
*        exacte pour Q1 (Formule des trapezes tensorielle)
*
         DEXACT=1
         NBPTQ=8
         CALL HELKCO (ABS(NUMERO),NDIM,COORQ)
         P1=1./8.
         DO 1001 I=1,NBPTQ
            POIDQ(I)=P1
1001     CONTINUE
*
      ELSEIF (NUMERO.EQ.-2) THEN
*
*        Formule 'nodale' utilisant les noeuds de l'element Q2
*        exacte pour Q3 (Formule de Simpson tensorielle)
*
         DEXACT=3
         NBPTQ=27
         CALL HELKCO (ABS(NUMERO),NDIM,COORQ)
         P1=1./216.
         DO 1002 I=1,8
            POIDQ(I)=P1
1002     CONTINUE
         P1=1./54.
         DO 1003 I=9,20
            POIDQ(I)=P1
1003     CONTINUE
         P1=2./27.
         DO 1004 I=21,26
            POIDQ(I)=P1
1004     CONTINUE
         POIDQ(27)=8./27.
*
      ELSEIF (NUMERO.EQ.1) THEN
*
*        Formule de GAUSS a 1 point exacte pour Q1([0.,1.]**3)
*
         DEXACT=1
         NBPTQ=1
         DO 10 I=1,NDIM
10          COORQ(I,1)=.5
         POIDQ(1)=1.
*
      ELSEIF (NUMERO.EQ.3) THEN
*
*        Formule de GAUSS a 8 points exacte pour Q3([0.,1.]**3)
*        obtenue par 'Cube tensoriel' de la formule 1-D a 2 points
*
         NBPTQ=(1+NUMERO)/2
         NSAUT=NBPTQ*NBPTQ+NBPTQ+1
         CALL SEGMQD (NDIM,NUMERO,NBPTQ,POIDQ,COORQ,NSAUT,DEXACT)
         NSAUTI=1
         NUPTQ=0
         DO 32 I=1,NBPTQ
            X=COORQ(1,NSAUTI)
            NSAUTJ=1
            DO 31 J=1,NBPTQ
               Y=COORQ(1,NSAUTJ)
               NSAUTK=1
               DO 30 K=1,NBPTQ
                  NUPTQ=NUPTQ+1
                  COORQ(1,NUPTQ)=X
                  COORQ(2,NUPTQ)=Y
                  COORQ(3,NUPTQ)=COORQ(1,NSAUTK)
                  NSAUTK=NSAUTK+NSAUT
30             CONTINUE
               NSAUTJ=NSAUTJ+NSAUT
31          CONTINUE
            NSAUTI=NSAUTI+NSAUT
32       CONTINUE
         NBPTQ=NUPTQ
         P1=1./8
         DO 33 I=1,NBPTQ
            POIDQ(I)=P1
33       CONTINUE
* 
      ELSEIF (NUMERO.EQ.5) THEN
*
*        Formule de GAUSS a 27 points exacte pour Q5([0.,1.]**3)  
*        obtenue par 'Cube tensoriel' de la formule 1-D a 3 points  
*
         NBPTQ=(1+NUMERO)/2
         NSAUT=NBPTQ*NBPTQ+NBPTQ+1
         CALL SEGMQD (NDIM,NUMERO,NBPTQ,POIDQ,COORQ,NSAUT,DEXACT)
         NSAUTI=1
         NUPTQ=0  
         DO 52 I=1,NBPTQ
            X=COORQ(1,NSAUTI) 
            NSAUTJ=1
            DO 51 J=1,NBPTQ
               Y=COORQ(1,NSAUTJ)  
               NSAUTK=1
               DO 50 K=1,NBPTQ
                  NUPTQ=NUPTQ+1
                  COORQ(1,NUPTQ)=X
                  COORQ(2,NUPTQ)=Y
                  COORQ(3,NUPTQ)=COORQ(1,NSAUTK)
                  NSAUTK=NSAUTK+NSAUT
50             CONTINUE
               NSAUTJ=NSAUTJ+NSAUT
51          CONTINUE
            NSAUTI=NSAUTI+NSAUT
52       CONTINUE
         NBPTQ=NUPTQ  
*        Poids: 5*5*5/(8*9*9*9)=.0214334705
*               5*8*8/(8*9*9*9)=.05486968448
*               5*5*8/(8*9*9*9)=.0342935528
*               8*8*8/(8*9*9*9)=.087791495
         P1=125./5832.
         P2=40./729.
         P3=25./729.
         P4=64./729.
         DO 53 I=1,7,2
            POIDQ(I)   = P1
            POIDQ(I+20)= P1
            POIDQ(I+10)= P2
53       CONTINUE
         DO 54 I=2,26,2
            POIDQ(I)= P3
54       CONTINUE
         POIDQ(5)   = P2
         POIDQ(9)   = P1
         POIDQ(14)  = P4
         POIDQ(19)  = P1
         POIDQ(23)  = P2
*
      ELSEIF (MOD(NUMERO,2).EQ.1.AND.NUMERO.LE.DEGMAX) THEN
*
*        Formule de GAUSS a (k+1)^3 points exacte pour Q_{2k-1}([0.,1.]**3)
*        obtenue par 'Cube tensoriel' de la formule 1-D a k+1 points
*        avec k = (NUMERO+1)/2
*
         NBPTQ=(1+NUMERO)/2
         NSAUT=NBPTQ*NBPTQ+NBPTQ+1
         CALL SEGMQD (NDIM,NUMERO,NBPTQ1,POIDQ,COORQ,NSAUT,DEXACT)
         NSAUTI=1
         NUPTQ=0
         DO 112 I=1,NBPTQ1
            X=COORQ(1,NSAUTI)
            NSAUTJ=1
            DO 111 J=1,NBPTQ1
               Y=COORQ(1,NSAUTJ)
               NSAUTK=1
               DO 110 K=1,NBPTQ1
                  NUPTQ=NUPTQ+1
                  COORQ(1,NUPTQ)=X
                  COORQ(2,NUPTQ)=Y
                  COORQ(3,NUPTQ)=COORQ(1,NSAUTK)
                  NSAUTK=NSAUTK+NSAUT
110            CONTINUE
               NSAUTJ=NSAUTJ+NSAUT
111         CONTINUE
            NSAUTI=NSAUTI+NSAUT
112      CONTINUE
         NBPTQ=NUPTQ
         DO 115 I=NBPTQ1,1,-1
            X=POIDQ(I)
            DO 114 J=NBPTQ1,1,-1
               Y=POIDQ(J)
               DO 113 K=NBPTQ1,1,-1
                  POIDQ(NUPTQ)=X*Y*POIDQ(K)
                  NUPTQ=NUPTQ-1
113            CONTINUE
114         CONTINUE
115      CONTINUE
*
      ELSE
*
         CALL HEXXQD (NDIM,NUMERO,NBPTQ,POIDQ,COORQ,DEXACT)
      ENDIF
                                                                    END !HexaQD
