*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE PRISQD (NDIM,NUMERO,NBPTQ,POIDQ,COORQ,DEXACT)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Fevrier 1988)
* Derniere modification : D.Martin (14 janvier 2000)
* Version 1.0.1
*  
* Formules de quadrature sur le prisme rectangle isocele unite
* (programme pour que les valeurs des coeff. soit corrects en  double precision
*  pour les compilations implicites en double precision)
*  
*-- Variables d'entree -- 
* NDIM   dimension de COORQ 
* NUMERO numero du schema de quadrature (convention)
*  
*-- Variables en sortie --
* NBPTQ  nombre de points du schema de quadrature
* POIDQ  tableau des poids du schema de quadrature
* COORQ  tableau des coordonnees des points de quadrature
* DEXACT degre de precision de la formule (pour impression)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      INTEGER NDIM,NUMERO,NBPTQ,DEXACT
      REAL    POIDQ(*),COORQ(NDIM,*)
*
      INTEGER NBPTS,NB1,NB2,I,J
      REAL    AUX1,AUX2,POID1,POID2,POID3,POID4
*-----------------------------------------------------------------------
      IF (NUMERO.EQ.1) THEN
*
*        Formule a 1 point exacte pour P1 sur le triangle x P1 sur [0,1]
*
         DEXACT=1
         AUX1 =1./3.
         NBPTQ=1
         COORQ(1,1)= AUX1
         COORQ(2,1)= AUX1
         COORQ(3,1)=.5
         POIDQ(1)=.5
*
*      ELSEIF (NUMERO.EQ.3) THEN
*
*        Produit tensoriel d'une formule de GAUSS a 2 points sur [0,1]
*        et d'une formule de STROUD a ? points pour le triangle unite
*        --> Formule exacte pour P3 sur le triangle 'tensoriel' P3 [0,1]
*
*         CALL TRIAQD (NDIM,NUMERO,NBPTQ,POIDQ,COORQ,DEXACT)
*         CALL SEGMQD (NDIM,NUMERO,NBPTS,POIDQ,COORQ(3,1),NBPTQ,DEXACT)
*
      ELSEIF (NUMERO.EQ.5) THEN
*
*        Produit tensoriel d'une formule de GAUSS a 3 points sur [0,1]
*        et d'une formule de STROUD a 7 points pour le triangle unite
*        --> Formule exacte pour P5 sur le triangle 'tensoriel' P5 [0,1]
*
*     Coordonnees: (6-SQRT(15))/21  =.1012865073
*                  (6+SQRT(15))/21  =.4701420641
*                  (9+2*SQRT(15))/21=.7974269852
*                  (9-2*SQRT(15))/21=.0597158718
*     Poids:       (9/80)*(8/9)*(1/2)=1/20              =.05
*                  (9/80)*(5/9)*(1/2)=5/160             =.03125
*                  (1/2)*(5/9)*(1/2)*(155-SQRT(15))/1200=.0174915528
*                  (1/2)*(5/9)*(1/2)*(155+SQRT(15))/1200=.0183880767
*                  (1/2)*(8/9)*(1/2)*(155-SQRT(15))/1200=.0279864844
*                  (1/2)*(8/9)*(1/2)*(155+SQRT(15))/1200=.0294209228
*
         CALL TRIAQD (NDIM,NUMERO,NBPTQ,POIDQ,COORQ,DEXACT)
         CALL SEGMQD (NDIM,NUMERO,NBPTS,POIDQ,COORQ(3,1),NBPTQ,DEXACT)
*
         DO 22 I=1,NBPTQ  
            NB1=I+NBPTQ
            NB2=NB1+NBPTQ 
            DO 20 J=1,2
               COORQ(J,NB1)=COORQ(J,I)
               COORQ(J,NB2)=COORQ(J,I)
20          CONTINUE
            DO 21 J=1,NBPTS*NBPTQ,NBPTQ
               COORQ(3,I+J-1)=COORQ(3,J)
21          CONTINUE  
22       CONTINUE  
         NBPTQ=NBPTS*NBPTQ
         AUX2  = SQRT (15.)
         AUX1  = (155-AUX2)/43200.
         AUX2  = (155+AUX2)/43200.
         POID1 = 5.*AUX1
         POID2 = 5.*AUX2
         POID3 = 8.*AUX1
         POID4 = 8.*AUX2
         DO 23 J=1,3  
             POIDQ(J+1)  = POID1
             POIDQ(J+15) = POID1
             POIDQ(J+4)  = POID2
             POIDQ(J+18) = POID2
             POIDQ(J+8)  = POID3
             POIDQ(J+11) = POID4
23        CONTINUE
         POID1     = 5./160.
         POIDQ(1)  = POID1
         POIDQ(8)  = 1./20.
         POIDQ(15) = POID1
*
      ELSE
*
         CALL PRXXQD (NDIM,NUMERO,NBPTQ,POIDQ,COORQ,DEXACT)
      ENDIF
                                                                    END !PrisQD
