*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      FUNCTION PTXQUA (K,N)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Septembre 2000)
* Derniere modification : D.Martin (16 fevrier 2010)
* Version 3
*
* Nombre maximum de points de quadrature d'une formule de quadrature de degre K
* en dimension N
*
*-- Argument d'entree --
* K code entier definissant la formule de quadrature
*   0 < K < 100 formule de Gauss de degre K (segments, quadrangles, hexaedres)
* 100 < K < 200 formule de Gauss-Lobatto de degre K-30 (segments, quadrangles)
*       K = -1  formule utilisant les noeuds des elements de Lagrange d'ordre 1
*       K = -2  formule utilisant les noeuds des elements de Lagrange d'ordre 2
* N dimension du domaine d'integration
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      INTEGER K,N,PTXQUA
*
      CHARACTER ERR*4,ER2*2
      INTEGER   DEGMAX
      PARAMETER ( DEGMAX = 99)
*-----------------------------------------------------------------------
      PTXQUA = 0
*
*     Formule de Gauss ou issues d'icelles
*
      IF (K.GT.0) THEN
*
*        Formules de Gauss-Legendre (tensorielles ou produits coniques)
*
         IF (K.LE.DEGMAX) THEN
            IF (N.EQ.1) THEN
               PTXQUA=((K+1)/2)
            ELSEIF (N.EQ.2) THEN
*              PTXQUA=((K+1)/2)**N
*              Modifie pour le cas des triangles (produits coniques)
               PTXQUA=((K+3)/2)*((K+5)/2)
            ELSEIF (N.EQ.3) THEN
*              PTXQUA=((K+1)/2)**N
*              Modifie pour le cas des tetraedres (produits coniques)
               PTXQUA=((K+3)/2)*((K+5)/2)*((K+7)/2)
            ENDIF
*
         ELSEIF (K.GT.DEGMAX.AND.K.LE.2*DEGMAX+1) THEN
*
*           Formules tensorielles de Gauss-Lobatto
*
            PTXQUA=((K-DEGMAX+3)/2)**N
*
         ENDIF
*
*     Formules nodales variees
*
      ELSEIF (K.EQ.-1) THEN
*        OK pour un segment, quadrangle, hexaedre
*        = 3 pour un triangle, = 4 pour un tetraedre, = ? pour un prisme
*
         PTXQUA=2**N
*
      ELSEIF (K.EQ.-2) THEN
*        OK pour un segment, quadrangle, hexaedre
*        = 3 pour un triangle, = ? pour un tetraedre, = ? pour un prisme
*
         PTXQUA=3**N
*
      ELSEIF (K.EQ.-3) THEN
*        = 4 pour les segment
*        = 7 pour un triangle, = ? pour un tetraedre, = ? pour un prisme
*
         PTXQUA=4**N
         IF (N.EQ.2) THEN
            PTXQUA=7
         ELSEIF (N.EQ.3) THEN
            PTXQUA=0
         ENDIF
*
      ELSEIF (K.EQ.-4) THEN
*        = 5 pour un segment
*        = ? pour un triangle, = ? pour un tetraedre, = ? pour un prisme
*
         PTXQUA=5**N
         IF ( N.GT.1) THEN
            PTXQUA=0
         ENDIF
*
      ENDIF
*
      IF (PTXQUA.EQ.0) THEN
         CALL ENCLER (K,ERR)
         CALL ENCLER (N,ER2)
         CALL BAISE  ('<PtxQua> Formule de quadrature de numero non'
     &               //' prevu :'//ERR//' en dimension'//ER2//'.')
      ENDIF
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* File history
* Version 3 : D.Martin (10 fevrier 2010)
* - reecrit pour prendre en compte les formules coniques de haut degre
* - et les formules 'nodales' oubliees ( K < 0 )
* Version 2 : D.Martin (10 octobre 2005)
* Version 1 : D.Martin (05 juillet 2005)
* - mis a jour pour les formules tensorielles de haut degre
* Version 0 : D.Martin (04 septembre 2000)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !PtxQua



