*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE QUADQD (NDIM,NUMERO,NBPTQ,POIDQ,COORQ,DEXACT)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Fevrier 1988)
* Derniere modification : D.Martin (17 septembre 2010)
* Version 3
*
* Formules de quadrature sur le quadrangle [0.,1.] x [0.,1.]
* (programme pour que les valeurs des coeff. soit corrects en double precision
*  pour les compilations implicites en double precision)
*
*-- Variables d'entree -- 
* NDIM   dimension d'espace
* NUMERO numero du schema de quadrature (convention)
*   (par convention = degre de precision pour les formules de Gauss)
*                   = 100+degre de precision pour les formules de Gauss-Lobatto)
*-- Variables en sortie --
* NBPTQ  nombre de points du schema de quadrature
* POIDQ  tableau des poids du schema de quadrature
* COORQ  tableau des coordonnees des points de quadrature
* DEXACT degre de precision de la formule (pour impression)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      INTEGER NDIM,NUMERO,NBPTQ,DEXACT
      REAL    POIDQ(*),COORQ(NDIM,*)
*
      INTEGER I,J,NSAUT,NSAUTI,NSAUTJ,NUPTQ,NBPTQ1
      REAL    X,P1,P2,PM
      INTEGER     DEGMAX
      PARAMETER ( DEGMAX = 99 )
*-----------------------------------------------------------------------
      IF (NUMERO.EQ.-1) THEN
*****     IF (NUMERO.EQ.-1.OR.NUMERO.EQ.DEGMAX+2) THEN
*
*        Formule 'nodale' utilisant les noeuds de l'element l'element Q1
*        exacte pour Q1 (Formule du trapeze tensorielle)
*        
*        3-------2 = Formule de GAUSS-LOBATTO de degre 1 (numero=101)
*        |       |   aux noeuds de l'element Q1
*        |       |  
*        |       | 
*        4-------1
         DEXACT=1
         NBPTQ=4
         CALL QULKCO (ABS(NUMERO),NDIM,COORQ)
         P1=1./4
         DO 1001 I=1,NBPTQ
            POIDQ(I)=P1
1001     CONTINUE
*
         ELSEIF (NUMERO.EQ.-2) THEN
*****      ELSEIF (NUMERO.EQ.-2.OR.NUMERO.EQ.DEGMAX+4) THEN
*
*        Formule 'nodale' utilisant les noeuds de l'element l'element Q2
*        exacte pour Q3 (Formule de Simpson tensorielle)
*
*        3---6---2 = Formule de GAUSS-LOBATTO de degre 3 (numero=103)
*        |       |   aux noeuds de l'element Q2 
*        7   9   5 
*        |       |
*        4---8---1
         DEXACT=3
         NBPTQ=9
         CALL QULKCO (ABS(NUMERO),NDIM,COORQ)
         P1=1./36
         P2=1./9
         DO 1002 I=1,4
            POIDQ(I)  =P1
            POIDQ(I+4)=P2
1002     CONTINUE
         POIDQ(9)=4./9
*
      ELSEIF (NUMERO.EQ.1) THEN
*
*        Formule de GAUSS a 1 point exacte pour Q1([0.,1.]**2)  
*
         DEXACT=1
         DO 10 I=1,NDIM
            COORQ(1,I)=.5
10       CONTINUE
         POIDQ(1)=1.
         NBPTQ=1
*
      ELSEIF (NUMERO.EQ.3) THEN
*
*        *-----*-----*
*        |  2     4  |  Formule de GAUSS a 4 points exacte
*        *           *  pour Q3([0.,1.]**2) obtenue par
*        |  1     3  |  'Carre tensoriel' de la formule 1-D a 2 points
*        *-----*-----*
         NSAUT=1+(NUMERO+1)/2
         CALL SEGMQD (NDIM,NUMERO,NBPTQ,POIDQ,COORQ,NSAUT,DEXACT)
         NUPTQ=0
         NSAUTI=1
         DO 31 I=1,NBPTQ
            X=COORQ(1,NSAUTI)
            NSAUTJ=1
            DO 30 J=1,NBPTQ
               NUPTQ=NUPTQ+1
               COORQ(1,NUPTQ)=X
               COORQ(2,NUPTQ)=COORQ(1,NSAUTJ)
               NSAUTJ=NSAUTJ+NSAUT
30          CONTINUE
            NSAUTI=NSAUTI+NSAUT
31       CONTINUE
         NBPTQ=NUPTQ
         P1=POIDQ(1)*POIDQ(1)
         POIDQ(1)=P1
         POIDQ(2)=P1
         POIDQ(3)=P1
         POIDQ(4)=P1
         NBPTQ=NUPTQ
*  
      ELSEIF (NUMERO.EQ.5) THEN
*
*        *-----*-----*
*        | 3   6   9 |  Formule de GAUSS a 9 points exacte
*        |           |  pour Q5([0.,1.]**2) obtenue par
*        * 2   5   8 *  'Carre tensoriel' de la formule 1-D a 3 points
*        |           |
*        | 1   4   7 |
*        *-----*-----*
         NSAUT=1+(NUMERO+1)/2
         CALL SEGMQD (NDIM,NUMERO,NBPTQ,POIDQ,COORQ,NSAUT,DEXACT)
         NUPTQ=0
         NSAUTI=1
         DO 51 I=1,NBPTQ
            X=COORQ(1,NSAUTI) 
            NSAUTJ=1
            DO 50 J=1,NBPTQ
               NUPTQ=NUPTQ+1  
               COORQ(1,NUPTQ)=X
               COORQ(2,NUPTQ)=COORQ(1,NSAUTJ)
               NSAUTJ=NSAUTJ+NSAUT
50          CONTINUE
            NSAUTI=NSAUTI+NSAUT
51       CONTINUE
         NBPTQ=NUPTQ
         P1=POIDQ(1)*POIDQ(1)
         P2=POIDQ(1)*POIDQ(2)
         PM=POIDQ(2)*POIDQ(2)
         POIDQ(1)=P1
         POIDQ(2)=P2
         POIDQ(3)=P1
         POIDQ(4)=P2
         POIDQ(5)=PM
         POIDQ(6)=P2
         POIDQ(7)=P1
         POIDQ(8)=P2
         POIDQ(9)=P1
*
      ELSEIF (MOD(NUMERO,2).EQ.1.AND.NUMERO.LE.DEGMAX) THEN
*
*        Formule de GAUSS a n^2 points exacte pour Q_{2n-1}([0.,1.]**2)
*        obtenue par 'Carre tensoriel' de la formule 1-D a n points
*        avec n = (NUMERO+1)/2
*
         NSAUT=1+(NUMERO+1)/2
         CALL SEGMQD (NDIM,NUMERO,NBPTQ1,POIDQ,COORQ,NSAUT,DEXACT)
         NUPTQ=0
         NSAUTI=1
         DO 111 I=1,NBPTQ1
            X=COORQ(1,NSAUTI)
            NSAUTJ=1
            DO 110 J=1,NBPTQ1
               NUPTQ=NUPTQ+1
               COORQ(1,NUPTQ)=X
               COORQ(2,NUPTQ)=COORQ(1,NSAUTJ)
               NSAUTJ=NSAUTJ+NSAUT
110         CONTINUE
            NSAUTI=NSAUTI+NSAUT
111      CONTINUE
         NBPTQ=NUPTQ
         DO 115 I=NBPTQ1,1,-1
            X=POIDQ(I)
            DO 114 J=NBPTQ1,1,-1
               POIDQ(NUPTQ)=X*POIDQ(J)
               NUPTQ=NUPTQ-1
114         CONTINUE
115      CONTINUE
*
      ELSEIF (MOD(NUMERO,2).EQ.1.AND.NUMERO.GT.DEGMAX) THEN
*
*        Formule de GAUSS-LOBATTO a n^2 points exacte pour Q_{2n-3}([0.,1.]**2)
*        obtenue par 'Carre tensoriel' de la formule 1-D a n points
*        avec n = (NUMERO-(DEGMAX+1)+3)/2
*
*       Numerotation locale des noeuds des elements Qk, k= 3, 4, 5
*       3--10---6---2  3--14--10---6---2  3--18--14--10---6---2
*       |           |  |               |  |                   |
*       7  15  14   9  7  19  22  18  13  7  23  30  26  22  17
*       |           |  |               |  |                   |
*       11  16  13  5  11 23  25  21   9 11  27  35  34  29  13
*       |           |  |               |  |                   |
*       4---8--12---1 15  20  24  17   5 15  31  36  33  25   9
*            k=3       |               |  |                   |
*                      4---8--12--16---1 19  24  28  32  21   5
*                              k=4        |                   |
*                                         4---8--12--16--20---1
*                                                  k=5
*
*       La programmation ci-dessous utilise la numerotation ci-dessous
*       4---8---12--16
*       |           |
*       3   7   11  15
*       |           |
*       2   6   10  14
*       |           |
*       1---5----9--13
*            k=3
         NSAUT=1+(NUMERO-(DEGMAX+1)+3)/2
         CALL SEGMQD (NDIM,NUMERO,NBPTQ1,POIDQ,COORQ,NSAUT,DEXACT)
         NUPTQ=0
         NSAUTI=1
         DO 211 I=1,NBPTQ1
            X=COORQ(1,NSAUTI)
            NSAUTJ=1
            DO 210 J=1,NBPTQ1
               NUPTQ=NUPTQ+1
               COORQ(1,NUPTQ)=X
               COORQ(2,NUPTQ)=COORQ(1,NSAUTJ)
               NSAUTJ=NSAUTJ+NSAUT
210         CONTINUE
            NSAUTI=NSAUTI+NSAUT
211      CONTINUE
         NBPTQ=NUPTQ
         DO 215 I=NBPTQ1,1,-1
            X=POIDQ(I)
            DO 214 J=NBPTQ1,1,-1
               POIDQ(NUPTQ)=X*POIDQ(J)
               NUPTQ=NUPTQ-1
214         CONTINUE
215      CONTINUE
*
      ELSE
*
         CALL QUXXQD (NDIM,NUMERO,NBPTQ,POIDQ,COORQ,DEXACT)
      ENDIF
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* File history
* Version 3 : D. Martin (17 septembre 2010)
* - ajout des formules de Gauss-Lobatto pour le quadrangle [0,1]*[0,1]
* Version 3 : D. Martin (06 juillet 2005)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !QuadQd
