*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE QULKP1 (K,NBLMP1,NUPOP1)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (27 Juin 2001)
* Derniere modification : D.Martin (12 Fevrier 2004)
* Version 1.1
*
* Construction des tableaux de correspondance des numerotations locales
* entre elements quadrangulaires Qk et elements triangulaires P1.
* Le quadrangle Qk est decoupe en k^2 quadrangles Q1 eux-memes decoupes
* en 2 triangles, par exemple, le quadrangle Q2 est decoupe en 8 triangles P1 :
*
*          3-------6-------2                3-------6-------2 
*          |               |                | \  #6 | \  #4 |
*          |               |                |   \   |   \   |
*          |               |                | #5  \ | #3  \ |
*          7       9       5   --------->   7-------9-------5
*          |               |                | \  #8 | \  #2 | 
*          |               |                |   \   |   \   |
*          |               |                | #7  \ | #1  \ |
*          4-------8-------1                4-------8-------1
*
*-- Argument d'entree --
* K      pour un quadrangle de degre k
*-- Arguments de sortie --
* NBLMP1 nombre de triangles P1
* NUPOP1 tableau de correspondance entre triangle P1 et quadrangle Qk
*        NUPOP1 (s,t) = numero dans le quadrangle Qk du sommet s du triangle t
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      INTEGER K,NBLMP1,NUPOP1(3,*)
*
      INTEGER NK,P1,P2,P3,P4,KORNE1,KORNER,PREC,J
*----------------------------------------------------------------------
      NBLMP1=0 
      NK=K 
      PREC=0
      KORNE1=1
*
10    IF (NK.GT.2) THEN   
*
*       3--10---6---2   3--14--10---6---2   3--18--14--10---6---2
*       |           |   |               |   |                   |
*       7  15  14   9   7  19  22  18  13   7  23  30  26  22  17
*       |           |   |               |   |                   |
*       11 16  13   5  11  23  25  21   9  11  27  35  34  29  13
*       |           |   |               |   |                   |
*       4---8--12---1  15  20  24  17   5  15  31  36  33  25   9
*            k=3        |               |   |                   |
*                       4---8--12--16---1  19  24  28  32  21   5
*                               k=4         |                   |
*                                           4---8--12--16--20---1
*                                                     k=5
*
*       La numerotation est creee en commencant par la couche d'elements
*       la plus externe et l'element contenant le point No 1.
*       Elle est reiteree pour les couches plus internes jusqu'a
*       obtenir 1 ou 4 quadrangles.
*
*       Numero du premier 'coin' interne de la couche 
*            (13 pour k=3, 17 pour k=4, 21 pour k=5)
        KORNE1=KORNE1+4*NK
        KORNER=KORNE1-PREC
*
        P1=KORNER-1
        P2=1
        P4=KORNER
        DO 11 J=1,NK-1
           P3=P2+4
           NBLMP1=NBLMP1+1
           NUPOP1(1,NBLMP1) = P1 + PREC
           NUPOP1(2,NBLMP1) = P2 + PREC
           NUPOP1(3,NBLMP1) = P4 + PREC
           NBLMP1=NBLMP1+1
           NUPOP1(1,NBLMP1) = P3 + PREC
           NUPOP1(2,NBLMP1) = P4 + PREC
           NUPOP1(3,NBLMP1) = P2 + PREC
*
           P2=P2+4
           P1=P4
           IF (J.EQ.NK-2) THEN
              KORNER=KORNER+1
              P4=KORNER
           ELSE
              P4=P4+4
           ENDIF
11      CONTINUE
*
        P1=KORNER
        P3=2
        P2=P1-5
        DO 12 J=1,NK-1
           P4=P3+4
           NBLMP1=NBLMP1+1
           NUPOP1(1,NBLMP1) = P1 + PREC
           NUPOP1(2,NBLMP1) = P2 + PREC
           NUPOP1(3,NBLMP1) = P4 + PREC
           NBLMP1=NBLMP1+1
           NUPOP1(1,NBLMP1) = P3 + PREC
           NUPOP1(2,NBLMP1) = P4 + PREC
           NUPOP1(3,NBLMP1) = P2 + PREC
*
           P3=P4
           P2=P1
           IF (J.EQ.NK-2) THEN
              KORNER=KORNER+1
              P1=KORNER
           ELSE
              P1=P1+4
           ENDIF
12      CONTINUE
*
        P2=KORNER
        P3=P2-5
        P4=3
        DO 13 J=1,NK-1
           P1=P4+4
           NBLMP1=NBLMP1+1
           NUPOP1(1,NBLMP1) = P1 + PREC
           NUPOP1(2,NBLMP1) = P2 + PREC
           NUPOP1(3,NBLMP1) = P4 + PREC
           NBLMP1=NBLMP1+1
           NUPOP1(1,NBLMP1) = P3 + PREC
           NUPOP1(2,NBLMP1) = P4 + PREC
           NUPOP1(3,NBLMP1) = P2 + PREC
*
           P3=P2
           P4=P1
           IF (J.EQ.NK-2) THEN
              KORNER=KORNER+1
              P2=KORNER
           ELSE
              P2=P2+4
           ENDIF
13      CONTINUE
*
        P1=4
        P3=KORNER
        P4=P3-5
        DO 14 J=1,NK-1
           P2=P1+4
           NBLMP1=NBLMP1+1
           NUPOP1(1,NBLMP1) = P1 + PREC
           NUPOP1(2,NBLMP1) = P2 + PREC
           NUPOP1(3,NBLMP1) = P4 + PREC
           NBLMP1=NBLMP1+1
           NUPOP1(1,NBLMP1) = P3 + PREC
           NUPOP1(2,NBLMP1) = P4 + PREC
           NUPOP1(3,NBLMP1) = P2 + PREC
*
           P1=P2
           P4=P3
           IF (J.EQ.NK-2) THEN
              KORNER=KORNE1-PREC
              P3=KORNER
           ELSE
              P3=P3+4
           ENDIF
14       CONTINUE
*
         PREC=KORNE1-1
         NK=NK-2
         GOTO 10
      ENDIF
*
*     On numerote finalement le ou les quadrangles centraux
*
      IF (NK.EQ.1) THEN
*
*     3-------2      3-------2
*     |       |      | \  #2 |
*     |       |  ->  |   \   |
*     |       |      | #1  \ |
*     4-------1      4-------1  
*
         NBLMP1=NBLMP1+1
         NUPOP1(1,NBLMP1) = 4 + PREC
         NUPOP1(2,NBLMP1) = 1 + PREC
         NUPOP1(3,NBLMP1) = 3 + PREC
         NBLMP1=NBLMP1+1
         NUPOP1(1,NBLMP1) = 2 + PREC
         NUPOP1(2,NBLMP1) = 3 + PREC
         NUPOP1(3,NBLMP1) = 1 + PREC
*
      ELSEIF (NK.EQ.2) THEN
*      
*     3-------6-------2      3-------6-------2 
*     |               |      | \  #6 | \  #4 |
*     |               |      |   \   |   \   |
*     |               |      | #5  \ | #3  \ |
*     7       9       5  ->  7-------9-------5
*     |               |      | \  #8 | \  #2 | 
*     |               |      |   \   |   \   |
*     |               |      | #7  \ | #1  \ |
*     4-------8-------1      4-------8-------1
*
         NBLMP1=NBLMP1+1
         NUPOP1(1,NBLMP1) = 8 + PREC
         NUPOP1(2,NBLMP1) = 1 + PREC
         NUPOP1(3,NBLMP1) = 9 + PREC
         NBLMP1=NBLMP1+1
         NUPOP1(1,NBLMP1) = 5 + PREC
         NUPOP1(2,NBLMP1) = 9 + PREC
         NUPOP1(3,NBLMP1) = 1 + PREC
         NBLMP1=NBLMP1+1
         NUPOP1(1,NBLMP1) = 9 + PREC
         NUPOP1(2,NBLMP1) = 5 + PREC
         NUPOP1(3,NBLMP1) = 6 + PREC
         NBLMP1=NBLMP1+1
         NUPOP1(1,NBLMP1) = 2 + PREC
         NUPOP1(2,NBLMP1) = 6 + PREC
         NUPOP1(3,NBLMP1) = 5 + PREC
         NBLMP1=NBLMP1+1
         NUPOP1(1,NBLMP1) = 7 + PREC
         NUPOP1(2,NBLMP1) = 9 + PREC
         NUPOP1(3,NBLMP1) = 3 + PREC
         NBLMP1=NBLMP1+1
         NUPOP1(1,NBLMP1) = 6 + PREC
         NUPOP1(2,NBLMP1) = 3 + PREC
         NUPOP1(3,NBLMP1) = 9 + PREC
         NBLMP1=NBLMP1+1
         NUPOP1(1,NBLMP1) = 4 + PREC
         NUPOP1(2,NBLMP1) = 8 + PREC
         NUPOP1(3,NBLMP1) = 7 + PREC
         NBLMP1=NBLMP1+1
         NUPOP1(1,NBLMP1) = 9 + PREC
         NUPOP1(2,NBLMP1) = 7 + PREC
         NUPOP1(3,NBLMP1) = 8 + PREC
*
      ENDIF
      END
