*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE SEEQKB (KDEG,KELFCT,XSP,P,DP,DDP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : L.Goudenege (3 avril 2009)
* Derniere modification : Y.Lafranche (18 janvier 2011), D.Martin (06 fevrier 2011)
* Version 2
*
* Fonctions de base et derivees sur le segment de Lagrange d'ordre k
* pour les noeuds equidistants
*
*-- Arguments d'entree --
* KDEG  ordre de l'element de Lagrange (k)
* KELFCT variable CHARACTER indiquant
*        si KELFCT(1:1).NE.'N' -> calcul des fonctions de base
*        si KELFCT(2:2).NE.'N' -> calcul des derivees premieres des fct. de base
*        si KELFCT(3:3).NE.'N' -> calcul des derivees secondes des fct. de base
* XSP    coordonnees du point ou l'on demande la calcul
*-- Arguments de sortie --
* P     tableau contenant les valeurs des fonctions au point XSP
* DP        -       -            -     -  derivees premieres
* DDP       -       -            -     -  derivees secondes
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) KELFCT
      INTEGER KDEG
      REAL    XSP,P(*),DP(*),DDP(*)
*
      INTEGER I,J,L,KP1MI
      DOUBLE PRECISION KDEGX,TMP
*-----------------------------------------------------------------------
      KDEGX = KDEG * DBLE(XSP)
*
*  RAPPEL : Les noeuds sont ranges comme suit
*
*  2----1  2---3---1  2---4---3---1  2---5---4---3---1  2---6---5---4---3---1
*
*    k=1      k=2          k=3              k=4                  k=5
*
*  ou le point numero 1 est le point d'abcisse 1., 
*     le point numero 2 est le point d'abcisse 0,
*     le point numero 3 est le point d'abcisse 1-1/k,   
*     le point numero 4 est le point d'abcisse 1-2/k, etc. 
*
*     Calcul des fonctions de base au point XSP.
*
      IF (KELFCT(1:1).NE.'N') THEN
*        Produit des valeurs x-xj / xi-xj (i<>j)
*        On range les abscisses dans le bon sens. Donc la boucle sur I
*        commence en 1 puis 0 puis 1 - (I-2)/KDEG.
*        P(1) = eval du polynome de Lagrange au point 1.
*        P(2) = eval du polynome de Lagrange au point 0.
*        P(I) = eval du polynome de Lagrange au point 1-(I-2)/KDEG.

*        Calcul de P(1)
         P(1) = 1.
         DO L = 0, KDEG-1
            P(1) = P(1) * (KDEGX-L) / (KDEG-L)
         ENDDO
*        Calcul de P(2)
         P(2) = 1.
         DO L = 1, KDEG
            P(2) = P(2) * (1.-KDEGX/L)
         ENDDO
*        Calcul de P(I) pour 3 <= I <= KDEG+1
         KP1MI = KDEG
         DO I = 3, KDEG+1
            P(I) = 1.
            KP1MI = KP1MI-1 
            DO L = 0, KDEG
               IF (KP1MI.NE.L) THEN
                  P(I) = P(I) * (KDEGX-L)/(KP1MI-L)
                ENDIF
            ENDDO
         ENDDO
      ENDIF
*
*     Calcul des dérivées des fonctions de base au point XSP.
*
      IF (KELFCT(2:2).NE.'N') THEN
*        Produit des valeurs x-xk / xi-xk (k<>i,j)
*        Multiplication par 1/ xi-xj
*        Puis somme sur les j.
*        Calcul de DP(1) 
         DP(1) = 0.
         DO J = 0, KDEG-1
            TMP = 1.D0
            DO L = 0, KDEG-1
               IF (L.NE.J) THEN
                  TMP = (TMP * (KDEGX-L)) / (KDEG-L)
               ENDIF
            ENDDO
            TMP = (TMP * KDEG) / (KDEG-J)
            DP(1) = DP(1) + TMP
         ENDDO
*        Calcul de DP(2)
         DP(2) = 0.
         DO J = 1, KDEG
            TMP = 1.D0
            DO L = 1, KDEG
               IF (L.NE.J) THEN
                  TMP = TMP * (1.-KDEGX / L)
               ENDIF
            ENDDO
            TMP = -(TMP * KDEG) / J
            DP(2) = DP(2) + TMP
         ENDDO
*        Calcul de DP(I) pour 3 <= I <= KDEG+1
         KP1MI = KDEG
         DO I = 3, KDEG+1
            DP(I) = 0.
            KP1MI = KP1MI-1 
            DO J = 0, KDEG
               IF (J.NE.KP1MI) THEN
                  TMP = 1.D0
                  DO L = 0, KDEG
                     IF (L.NE.J.AND.L.NE.KP1MI) THEN
                        TMP = (TMP * (KDEGX-L)) / (KP1MI-L)
                     ENDIF
                  ENDDO
                  TMP = (TMP * KDEG) / (KP1MI-J)
                  DP(I) = DP(I) + TMP
               ENDIF
            ENDDO
         ENDDO
      ENDIF
*
*     Derivees secondes
*
      IF (KELFCT(3:3).NE.'N') THEN
          CALL BAISE ('Calcul des derivees secondes non implementes.')
      ENDIF
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*--File history
* Version 2 : Y.Lafranche (18 janvier 2011), D.Martin (06 fevrier 2011)
* - bug dans le calcul de P(I) : KP1MI = KDEG-1 devient KP1MI = KP1MI-1
* Version 1 : D.Martin & L.Goudenege (15 decembre 2009)
* - programmation separee du corps de la routine SELKFB
* Version 0 : L.Goudenege (3 avril 2009)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !Seeqkb
