*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE SEGLKB (KDEG,KELFCT,XSP,P,DP,DDP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin & G.Vial (Septembre 2000) & L.Goudenege (3 avril 2009)
* Derniere modification : D.Martin (15 decembre 2009)
* Version 3
*
* Fonctions de base et derivees sur le segment de Lagrange d'ordre KDEG
* pour l'interpolation de Lagrange aux points de Gauss-Lobatto
* pour 2 < KDEG < 100.
*
*-- Arguments d'entree --
* KDEG   ordre de l'element de Lagrange
* KELFCT variable CHARACTER indiquant
*        si KELFCT(1:1).NE.'N' Calcul des fonctions de base
*        si KELFCT(2:2).NE.'N' Calcul des derivees premieres des fct. de base
*        si KELFCT(3:3).NE.'N' Calcul des derivees secondes des fct. de base
* X      coordonnees du point ou l'on demande la calcul
*-- Arguments de sortie --
* P     tableau contenant les valeurs des fonctions au point X
* DP        -       -            -     -  derivees premieres
* DDP       -       -            -     -  derivees secondes
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) KELFCT
      INTEGER KDEG
      REAL    XSP,P(*),DP(*),DDP(*)
*
      INTEGER LAGDMX
      PARAMETER ( LAGDMX = 100)
      DOUBLE PRECISION CORLOC(1,(LAGDMX-1)/2)
*
      INTEGER I,J,K
      DOUBLE PRECISION X,TMP
*-----------------------------------------------------------------------
      X=DBLE(XSP)
*     On recupere les points de Gauss-Lobatto ranges comme suit
*
*  2---4---3---1  2---5---4---3---1  2---6---5---4---3---1
*
*    k=3              k=4                  k=5
*
*  ou le point numero 1 est le point d'abcisse 1., 
*     le point numero 2 est le point d'abcisse 0,
*     le point numero 3 est le point de Gauss-Lobatto de plus grande abscisse
*     differente de 1.
* 
      CALL SELKCO (100+KDEG,1,CORLOC)
*
      IF (KELFCT(1:1).NE.'N') THEN
*        Calcul des fonctions de base au point XSP.
*        Produit des valeurs x-xj / xi-xj (i<>j)
         DO I=1,KDEG+1
            TMP = 1.D0
            DO J=1,KDEG+1
               IF (I.NE.J)
     &         TMP = TMP*(X-CORLOC(1,J))/(CORLOC(1,I)-CORLOC(1,J))
            ENDDO
            P(I) = TMP 
         ENDDO
      ENDIF
*
      IF (KELFCT(2:2).NE.'N') THEN
*        Calcul des derivees des fonctions de base au point XSP.
*        Produit des valeurs x-xk / xi-xk (k<>i,j)
*        Multiplication par 1/ xi-xj , puis somme sur les j.
         DO I=1,KDEG+1
            DP(I) = 0.
            DO J=1,KDEG+1
               IF (J.NE.I) THEN
                  TMP = 1.D0
                  DO K=1,KDEG+1
                    IF (K.NE.J.AND.K.NE.I)
     &              TMP = TMP*(X-CORLOC(1,K))/(CORLOC(1,I)-CORLOC(1,K))
                  ENDDO
                  TMP = TMP / (CORLOC(1,I)-CORLOC(1,J))
                  DP(I) = DP(I) + TMP
               ENDIF
            ENDDO
         ENDDO
      ENDIF
*
      IF (KELFCT(3:3).NE.'N') THEN
        CALL BAISE ('Calcul des derivees secondes non implementes.')
      ENDIF
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 3 : L.Goudenege (3 avril 2009), D.Martin (15 decembre 2009)
* - version incorporee dans Melina
* - generalisation a tous degres (limite au degre 100 !)
* Version 2 : D.Martin (10 juillet 2005)
* Version 1 : D.Martin & G.Vial (Septembre 2000)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      END                                                               !SeGLkb
