*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE SELKFB (KDEG,KELFCT,XSP,P,DP,DDP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin & G.Vial (Septembre 2000)
* Derniere modification : D.Martin & L. Goudenège (14 decembre 2009)
* Version 2
*
* Fonctions de base et derivees sur le segment de Lagrange d'ordre k (KDEG)
*    0 < k < 100 --> pour les points equidistants a l'ordre k
*  100 < k < 103 --> identique au cas ou k = 1 ou 2
*  102 < k < 200 --> pour les points de Gauss-Lobatto a l'ordre k-100
*
*-- Arguments d'entree --
* KDEG   ordre de l'element de Lagrange pour le cas des points equidistants
*    100+ordre de l'element de Lagrange pour le cas des points de Gauss-Lobatto
* KELFCT variable CHARACTER indiquant
*        si KELFCT(1:1).NE.'N' Calcul des fonctions de base
*        si KELFCT(2:2).NE.'N' Calcul des derivees premieres des fct. de base
*        si KELFCT(3:3).NE.'N' Calcul des derivees secondes des fct. de base
* X      coordonnees du point ou l'on demande la calcul
*-- Arguments de sortie --
* P     tableau contenant les valeurs des fonctions au point XSP
* DP        -       -            -     -  derivees premieres
* DDP       -       -            -     -  derivees secondes
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) KELFCT
      INTEGER KDEG
      REAL    XSP,P(*),DP(*),DDP(*)
*
      CHARACTER*4 ERRSTR
      INTEGER LAGDMX, KNEW
      PARAMETER ( LAGDMX = 100)
      DOUBLE PRECISION X,TIERS,XM1,DXM1,XM1DXM,TXM1,TXM2,KXM1,KXM2,KXM3
     &                ,KXM1M2,YYY
*-----------------------------------------------------------------------
      X = DBLE(XSP)
      KNEW = KDEG
      IF (KNEW.GT.LAGDMX) KNEW = KNEW-LAGDMX
*
*  RAPPEL : Les noeuds sont ranges comme suit
*
*     2----1  2---3---1  2---4---3---1  2---5---4---3---1  2---6---5---4---3---1
*
*       k=1      k=2          k=3              k=4                  k=5
*
*  i.e. les points internes son ranges par abscisse decroissante
*  et le point numero 1 est le point d'abcisse 1.; 
*     le point numero 2 est le point d'abcisse 0,;
*     le point numero 3 est a l'abcisse 1-1/k pour les abscisses equidistantes;
*                       est le point de Gauss-Lobatto de plus grande abscisse
*                 differente de 1 pour les noeuds aux points de Gauss-Lobatto. 
*
*
      IF (KNEW.EQ.1) THEN
*-----------------------------------------------------------------------
*        Element de Lagrange P1 : 2----1
*-----------------------------------------------------------------------
         XM1 = X-1.
*        Fonctions de base
         IF (KELFCT(1:1).NE.'N') THEN
            P(1)=X
            P(2)=-XM1
         ENDIF
*        Derivees premieres
         IF (KELFCT(2:2).NE.'N') THEN
            DP(1)=1.
            DP(2)=-1.
         ENDIF
*        Derivees secondes
         IF (KELFCT(3:3).NE.'N') THEN
            DDP(1)=0.
            DDP(2)=0.
         ENDIF
*
      ELSEIF (KNEW.EQ.2) THEN
*-----------------------------------------------------------------------
*        Element de Lagrange P2 : 2---3---1
*-----------------------------------------------------------------------
         XM1 = X-1.
         YYY=4.*X
*        Fonctions de base
         IF (KELFCT(1:1).NE.'N') THEN
            P(1)=X*(X+XM1)
            P(2)=XM1*(XM1+X)
            P(3)=-YYY*XM1
         ENDIF
*        Derivees premieres
         IF (KELFCT(2:2).NE.'N') THEN
            DP(1)=YYY-1
            DP(2)=YYY-3
            DP(3)=4-2*YYY
         ENDIF
*        Derivees secondes
         IF (KELFCT(3:3).NE.'N') THEN
            DDP(1)=4.
            DDP(2)=4.
            DDP(3)=-8.
         ENDIF
*
      ELSEIF (KDEG.LE.3) THEN
*-----------------------------------------------------------------------
*        Element de Lagrange P3 : 2---4---3---1 aux abscisses equidistantes
*-----------------------------------------------------------------------
         XM1 = X-1.
         YYY=4.5D0*X*XM1
         KXM1=3*X-1.
         KXM2=KXM1-1.
         KXM1M2=KXM1*KXM2/2.
*        Fonctions de base
         IF (KELFCT(1:1).NE.'N') THEN
            P(1)= X *KXM1M2
            P(2)=-XM1*KXM1M2
            P(3)=-YYY*KXM1
            P(4)= YYY*KXM2
         ENDIF
*       Derivees premieres
         IF (KELFCT(2:2).NE.'N') THEN
            DP(1)= 1    +( -9+13.5D0*X)*X
            DP(2)=-5.5D0+( 18-13.5D0*X)*X
            DP(3)=-4.5D0+( 36-40.5D0*X)*X
            DP(4)= 9    +(-45+40.5D0*X)*X
         ENDIF
*
      ELSEIF (KDEG.LE.4) THEN 
*-----------------------------------------------------------------------
*        Element de Lagrange P4 : 2---5---4---3---1 aux abscisses equidistantes
*-----------------------------------------------------------------------
         XM1 = X-1.
         TIERS=1.D0/3
         KXM1=4*X-1.
         DXM1=2*X-1
*        Fonctions de base
         IF (KELFCT(1:1).NE.'N') THEN
            KXM3=KXM1-2.
            P(1)=    TIERS*X*DXM1*KXM1*KXM3
            P(2)=    TIERS  *DXM1*KXM1*KXM3*XM1
            P(3)=-16*TIERS*X*DXM1*KXM1     *XM1
            P(4)=  4      *X     *KXM1*KXM3*XM1
            P(5)=-16*TIERS*X*DXM1     *KXM3*XM1
         ENDIF
*        Derivees premieres
         IF (KELFCT(2:2).NE.'N') THEN
            DP(1)=TIERS*(8*X-3)*(1+(-12+(16*X))*X)
            DP(2)=TIERS*(8*X-5)*(5+(-20+(16*X))*X)
            DP(3)=16*TIERS+(-224*TIERS+(224-(512*TIERS*X))*X)*X
            DP(4)=4*DXM1*(3+(-32+(32*X))*X)
            DP(5)=16+(-416*TIERS+(288-(512*TIERS*X))*X)*X
         ENDIF
*
      ELSEIF (KDEG.LT.LAGDMX) THEN
*-----------------------------------------------------------------------
*           Elements de Lagrange de degre superieur a 5
*-----------------------------------------------------------------------
*
*        Version >= 3
         CALL SEEQKB(KDEG,KELFCT,XSP,P,DP,DDP)
*
      ELSEIF (KDEG.LT.2*LAGDMX) THEN
*-----------------------------------------------------------------------
*        Elements de Lagrange de degre > 3 aux abscisses de Gauss-Lobatto
*-----------------------------------------------------------------------
*
         CALL SEGLKB(KNEW,KELFCT,XSP,P,DP,DDP)
*
      ELSE
         CALL ENCLER (KDEG,ERRSTR)
         CALL BAISE ('Interpolation de Lagrange '
     &              //'non prevue pour K = '//ERRSTR//' .')
      ENDIF
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 3 : L.Goudenege (3 avril 2009), D.Martin (14 decembre 2009)
* - version incorporee dans Melina
* - creation de la routine SEEQFB pour le cas des noeuds equidistants
* - generalisation a tous degres (limite au degre 100 !)
* Version 2 : D.Martin & G. Vial (10 juillet 2005)
* - ajout des noeuds aux points de Gauss-Lobatto de degree 11 a 20, 24, 32, 40
* Version 1 : D.Martin & G. Vial (05 septembre 2001)
* - ajout des noeuds aux points de Gauss-Lobatto de degree 1 a 10
* Version 0 : D.Martin & G. Vial (30 Août 2001)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      END
