*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE SEX2SE (K,NDIM,NE,IQ1,IQ2,IQ3,IQ4,CO1,CO2,SENS,NUX2SE)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin (Novembre 2001)
* Derniere modification : D.Martin (21 Janvier 2002)
* Version 1.0.1
*
* Tableau de correspondance des noeuds dans l'element de reference pour
*    un quadrangle de Lagrange (ordre k) a (k+1)^2 noeuds 
*    une face d'un hexaedre de Lagrange (ordre k) a (k+1)^2 noeuds
* construites a partir des coordonnees de l'element 1D de Lagrange (ordre k)
*
*-- Variables d'entree --
* K       pour quadrangle ou hexaedre de type k
* NE      numero du 1er noeud concerne (1 pour un quadrangle)
*         en sortie du dernier
* NDIM    dimension d'espace
* CO1,CO2 numero des coordonnees utiles
*         (1,2 pour un quadrangle/ coordonnees variables de la face en 3D)
* SENS    sens de parcours de la face
*-- Variables en sortie --
* NUX2SE tableau de correspondance des noeuds du segment et du quadrangle
*
*-- Numerotations locales --
*
*  Segments Pk
*  2----1  2---3---1  2---4---3---1  2---5---4---3---1  2---6---5---4---3---1
*    k=1      k=2          k=3              k=4                  k=5
*
*  Quadrangles Qk
*  3----2  3---6---2  3--10---6---2  3--14--10---6---2  3--18--14--10---6---2
*  |    |  |       |  |           |  |               |  |                   |
*  4----1  7   9   5  7  15  14   9  7  19  22  18  13  7  23  30  26  22  17
*   k=1    |       |  |           |  |               |  |                   |
*          4---8---1 11  16  13   5 11  23  25  21   9 11  27  35  34  29  13
*             k=2     |           |  |               |  |                   |
*                     4---8--12---1 15  20  24  17   5 15  31  36  33  25   9
*                          k=3       |               |  |                   |
*                                    4---8--12--16---1 19  24  28  32  21   5
*                                            k=4        |                   |
*                                                       4---8--12--16--20---1
*                                                                k=5

*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      INTEGER  K,NE,NDIM,CO1,CO2,IQ1,IQ2,IQ3,IQ4,SENS
     &        ,NUX2SE(NDIM,*)
*
      INTEGER  NK,Q,NE1,IQQ1,IQQ2,IQQ3,IQQ4,I33,I44,I,J,JJ
*-----------------------------------------------------------------------
      NK=K
      Q=1+(K/2)
      NE1=NE
      IQQ1=IQ1
      IQQ2=IQ2
      IQQ3=IQ3
      IQQ4=IQ4
*
      DO 20 I=1,Q
         IF (NK.GT.0) THEN
            IF (SENS.EQ.1) THEN
*
*              Les 4 sommets du 'perimetre' en cours
*
               NE=NE+1
               NUX2SE(CO1,NE)=IQQ1
               NUX2SE(CO2,NE)=IQQ2
               NE=NE+1
               NUX2SE(CO1,NE)=IQQ1
               NUX2SE(CO2,NE)=IQQ1
               NE=NE+1
               NUX2SE(CO1,NE)=IQQ2
               NUX2SE(CO2,NE)=IQQ1
               NE=NE+1
               NUX2SE(CO1,NE)=IQQ2
               NUX2SE(CO2,NE)=IQQ2
*
*              Les noeuds 'internes' aux aretes du 'perimetre' en cours
*
               I33=IQQ3
               I44=IQQ4
               DO 15 J=2,NK
                  NE=NE+1
                  NUX2SE(CO1,NE)=IQQ1
                  NUX2SE(CO2,NE)=I44
                  NE=NE+1
                  NUX2SE(CO1,NE)=I33
                  NUX2SE(CO2,NE)=IQQ1
                  NE=NE+1
                  NUX2SE(CO1,NE)=IQQ2
                  NUX2SE(CO2,NE)=I33
                  NE=NE+1
                  NUX2SE(CO1,NE)=I44
                  NUX2SE(CO2,NE)=IQQ2
                  I44=I44-1
                  I33=I33+1
15             CONTINUE
*
            ELSE
               NE=NE+1
               NUX2SE(CO1,NE)=IQQ1
               NUX2SE(CO2,NE)=IQQ2
               NE=NE+1
               NUX2SE(CO1,NE)=IQQ2
               NUX2SE(CO2,NE)=IQQ2
               NE=NE+1
               NUX2SE(CO1,NE)=IQQ2
               NUX2SE(CO2,NE)=IQQ1
               NE=NE+1
               NUX2SE(CO1,NE)=IQQ1
               NUX2SE(CO2,NE)=IQQ1
*
               I33=IQQ3
               I44=IQQ4
               DO 16 J=2,NK
                  NE=NE+1
                  NUX2SE(CO1,NE)=I33
                  NUX2SE(CO2,NE)=IQQ2
                  NE=NE+1
                  NUX2SE(CO1,NE)=IQQ2
                  NUX2SE(CO2,NE)=I44
                  NE=NE+1
                  NUX2SE(CO1,NE)=I44
                  NUX2SE(CO2,NE)=IQQ1
                  NE=NE+1
                  NUX2SE(CO1,NE)=IQQ1
                  NUX2SE(CO2,NE)=I33
                  I44=I44-1
                  I33=I33+1
16             CONTINUE
            ENDIF
            IQQ1=IQQ1+1
            IQQ2=IQQ2-1
            IF (IQQ1-1.EQ.1) THEN
               IQQ1=IQ3
               IQQ2=IQ4
            ENDIF
            IQQ3=IQQ3+1
            IQQ4=IQQ4-1
            NK=NK-2
         ELSE
*
*           Le point central le cas echeant
*
            NE=NE+1
            NUX2SE(CO1,NE)=IQQ1
            NUX2SE(CO2,NE)=IQQ1
         ENDIF
20    CONTINUE
                                                                    END !Sex2Se

