*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE SEX3SE (K,NDIM,NE,NUX3SE)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin (Novembre 2001)
* Derniere modification : D.Martin (03 Decembre 2001)
* Version 1.0.0
*
* Tableau de correspondance noeuds dans l'element de reference pour
*    un quadrangle de Lagrange (ordre k) a (k+1)^2 noeuds 
*    une face d'un hexaedre de Lagrange (ordre k) a (k+1)^2 noeuds
* construites a partir des coordonnees de l'element 1D de Lagrange (ordre k)
*
*-- Variables d'entree --
* K       pour hexaedre de type k
* NDIM    dimension d'espace
*
*-- Variables en sortie --
* NE     nombre de noeuds
* NUX3SE tableau de correspondance des noeuds du segment et de l'hexaedre
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      INTEGER  K,NDIM,NUX3SE(NDIM,*)
*
      INTEGER NK,NE,NEP,Q,I,J,Z
     &       ,IQ1,IQ1P,IQ2,IQ2P,IQ3,IQ3C,IQ4,IQ4C
     &       ,IZ1,IZ1P,IZ2,IZ2P,IZ3,IZ3C,IZ4,IZ4C,I3
*-----------------------------------------------------------------------
*
      NK=K
      Q=1+(K/2)
      IQ1=1
      IQ2=2
      IQ3=3
      IQ4=NK+1
      IZ1=2
      IZ2=1
      IZ3=NK+1
      IZ4=3
*
      DO 50 I=1,Q
         IF (NK.GT.0) THEN
            IQ1P=IQ1
            IQ2P=IQ2
            IZ1P=IZ1
            IZ2P=IZ2
*
*           Les 8 sommets du 'perimetre' en cours
*
            I3=IZ1
            DO 10 Z=1,2
               NE=NE+1
               NUX3SE(1,NE)=IQ1
               NUX3SE(2,NE)=IQ2
               NUX3SE(3,NE)=I3
               NE=NE+1
               NUX3SE(1,NE)=IQ1
               NUX3SE(2,NE)=IQ1
               NUX3SE(3,NE)=I3
               NE=NE+1
               NUX3SE(1,NE)=IQ2
               NUX3SE(2,NE)=IQ1
               NUX3SE(3,NE)=I3
               NE=NE+1
               NUX3SE(1,NE)=IQ2
               NUX3SE(2,NE)=IQ2
               NUX3SE(3,NE)=I3
               I3=IZ2
10          CONTINUE
*
*           Les noeuds 'internes' des 12 aretes du 'perimetre' en cours
*
            IQ3C=IQ3
            IQ4C=IQ4
            IZ3C=IZ3
            IZ4C=IZ4
            DO 15 J=2,NK
*              Arete 2 --> 6, Noeuds 9, 21, ...
               NE=NE+1
               NUX3SE(1,NE)=IQ1
               NUX3SE(2,NE)=IQ1
               NUX3SE(3,NE)=IZ3C
*              Arete 7 --> 6, Noeuds 10, 22, ...
               NE=NE+1
               NUX3SE(1,NE)=IQ4C
               NUX3SE(2,NE)=IQ1
               NUX3SE(3,NE)=IZ2
*              Arete 5 --> 6, Noeuds 11, 23, ...
               NE=NE+1
               NUX3SE(1,NE)=IQ1
               NUX3SE(2,NE)=IQ4C
               NUX3SE(3,NE)=IZ2
*              Arete 8 --> 4, Noeuds 12, 24, ...
               NE=NE+1
               NUX3SE(1,NE)=IQ2
               NUX3SE(2,NE)=IQ2
               NUX3SE(3,NE)=IZ4C
*              Arete 1 --> 4, Noeuds 13, 25, ...
               NE=NE+1
               NUX3SE(1,NE)=IQ3C
               NUX3SE(2,NE)=IQ2
               NUX3SE(3,NE)=IZ1
*              Arete 3 --> 4, Noeuds 14, 26, ...
               NE=NE+1
               NUX3SE(1,NE)=IQ2
               NUX3SE(2,NE)=IQ3C
               NUX3SE(3,NE)=IZ1
*              Arete 1 --> 5, Noeuds 15, 27, ...
               NE=NE+1
               NUX3SE(1,NE)=IQ1
               NUX3SE(2,NE)=IQ2
               NUX3SE(3,NE)=IZ3C
*              Arete 2 --> 3, Noeuds 16, 28, ...
               NE=NE+1
               NUX3SE(1,NE)=IQ3C
               NUX3SE(2,NE)=IQ1
               NUX3SE(3,NE)=IZ1
*              Arete 7 --> 8, Noeuds 17, 29, ...
               NE=NE+1
               NUX3SE(1,NE)=IQ2
               NUX3SE(2,NE)=IQ3C
               NUX3SE(3,NE)=IZ2
*              Arete 3 --> 7, Noeuds 18, 30, ...
               NE=NE+1
               NUX3SE(1,NE)=IQ2
               NUX3SE(2,NE)=IQ1
               NUX3SE(3,NE)=IZ3C
*              Arete 5 --> 8, Noeuds 19, 31, ...
               NE=NE+1
               NUX3SE(1,NE)=IQ3C
               NUX3SE(2,NE)=IQ2
               NUX3SE(3,NE)=IZ2
*              Arete 1 --> 2, Noeuds 20, 32, ...
               NE=NE+1
               NUX3SE(1,NE)=IQ1
               NUX3SE(2,NE)=IQ4C
               NUX3SE(3,NE)=IZ1
*
               IQ3C=IQ3C+1
               IQ4C=IQ4C-1
               IZ3C=IZ3C-1
               IZ4C=IZ4C+1
15          CONTINUE             
*
*           Les noeuds 'internes' des 6 faces du 'perimetre' en cours
*
*
            IQ1=IQ1+1
            IQ2=IQ2-1
            IQ3=IQ3+1
            IQ4=IQ4-1
            IZ1=IZ1-1
            IZ2=IZ2+1
            IZ3=IZ3-1
            IZ4=IZ4+1
            IF (I.EQ.1) THEN
               IQ1=3
               IQ2=NK+1
               IZ1=NK+1
               IZ2=3
            ENDIF
            IF (NK.GE.2) THEN
*
*              Face 1 : 2   6   5   1 , x=1
*
               NEP=NE+1
               CALL SEX2SE (NK-2,NDIM,NE,IQ1,IQ2,IQ3,IQ4,2,3,1,NUX3SE)
               DO 21 J=NEP,NE
                  NUX3SE(1,J)=IQ1P
21             CONTINUE
*
*              Face 2 : 7   6   2   3 , y=1
*
               NEP=NE+1
               CALL SEX2SE (NK-2,NDIM,NE,IQ1,IQ2,IQ3,IQ4,3,1,1,NUX3SE)
               DO 22 J=NEP,NE
                  NUX3SE(2,J)=IQ1P
22             CONTINUE
*
*              Face 3 :  5   6   7   8 , z=1
*
               NEP=NE+1
               CALL SEX2SE (NK-2,NDIM,NE,IQ1,IQ2,IQ3,IQ4,1,2,1,NUX3SE)
               DO 23 J=NEP,NE
                  NUX3SE(3,J)=IZ2P
23             CONTINUE
*
*              Face 4 :  3   4   8   7 , x=0
*
               NEP=NE+1
               CALL SEX2SE (NK-2,NDIM,NE,IQ1,IQ2,IQ3,IQ4,2,3,-1,NUX3SE)
               DO 24 J=NEP,NE
                  NUX3SE(1,J)=IQ2P
24             CONTINUE
*
*              Face 5 :  8   4   1   5 , y=0
*
               NEP=NE+1
               CALL SEX2SE (NK-2,NDIM,NE,IQ1,IQ2,IQ3,IQ4,3,1,-1,NUX3SE)
               DO 25 J=NEP,NE
                  NUX3SE(2,J)=IQ2P
25             CONTINUE
*
*              Face 6 :  1   4   3   2 , z=0
*
               NEP=NE+1
               CALL SEX2SE (NK-2,NDIM,NE,IQ1,IQ2,IQ3,IQ4,1,2,-1,NUX3SE)
               DO 26 J=NEP,NE
                  NUX3SE(3,J)=IZ1P
26             CONTINUE
*
            ENDIF
*
            NK=NK-2
         ELSEIF (NK.EQ.0) THEN
*
*           Le point central le cas echeant
*
            NE=NE+1
            DO 49 J=1,NDIM
               NUX3SE(J,NE)=IQ1
49          CONTINUE
         ENDIF
50    CONTINUE
                                                                    END !Sex3Se
