*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE TETRQD (NDIM,NUMERO,NBPTQ,POIDQ,COORQ,DEXACT)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Fevrier 1988)
* Derniere modification : D.Martin (25 mai 2009)
* Version 2
*  
* Formules de quadrature sur le tetraedre unite
* (programme pour que les valeurs des coeff. soit corrects en  double precision
*  pour les compilations implicites en double precision)
*  
*-- Variables d'entree -- 
* NDIM   dimension d'espace  
* NUMERO numero du schema de quadrature (convention)
*  
*-- Variables en sortie --
* NBPTQ nombre de points du schema de quadrature
* POIDQ tableau des poids du schema de quadrature
* COORQ tableau des coordonnees des points de quadrature
* DEXACT degre de precision de la formule (pour impression)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      INTEGER NDIM,NUMERO,NBPTQ,DEXACT
      REAL    POIDQ(*),COORQ(NDIM,*)
*
      INTEGER IP1,IP5,IP9,IP12,I,L
      REAL    POID1,POID2,POID3,CORQ1,CORQ2,CORQ3,CORQ4,CORQ5,CORQ6,AUX
*-----------------------------------------------------------------------
      IF (NUMERO.EQ.-1) THEN
*
*        Formule 'nodale' utilisant les noeuds de l'element Lagrange P1
*        exacte pour P1
*
         DEXACT=1
         AUX=1./24.
         NBPTQ=4
         CALL TELKCO (1,NDIM,COORQ)
         DO 1001 I=1,NBPTQ
            POIDQ(I)=AUX
1001     CONTINUE
*
      ELSEIF (NUMERO.EQ.1) THEN
*
*        Formule  a 1 point exacte pour P1
*
         DEXACT=1
         NBPTQ=1
         POID1 = 1./6.
         CORQ1 = 1./4.
         DO 10 I=1,NDIM
10          COORQ(I,1) = CORQ1
         POIDQ(1) = POID1
*
      ELSEIF (NUMERO.EQ.3) THEN
*
*        Formule de STROUD a 8 points exacte pour P3 
*
         DEXACT = 3
         NBPTQ = 8
         COORQ(1,1) = 1.
         COORQ(2,2) = 1.
         COORQ(3,4) = 1.
         AUX = 1./3.
         DO 32 I = 5,8
            DO 31 J=1,NDIM
31             COORQ(J,I) = AUX
32       CONTINUE
         DO 33 I = 5,7
            COORQ(I-4,I) = 0.
33       CONTINUE
         AUX = 1./240.
         DO 34 I = 1,4
            POIDQ(I) = AUX
34       CONTINUE 
         AUX = 9*AUX
         DO 35 I = 5,8
            POIDQ(I) = AUX
35       CONTINUE 
*
      ELSEIF (NUMERO.EQ.5) THEN
*
*        Formule de STROUD a 15 points exacte pour P5 
*
*       Poids:         (2665+14*SQRT(15)/(37800*6) =.0119895139 
*                      (2665-14*SQRT(15)/(37800*6) =.0115113678 
*                       20/(378*6)                 =.008818342  
*                      (16/135)*(1/6)              =.01975308
*       Coordonnees:   (7-SQRT(15))/34             =.091971078  
*                      (7+SQRT(15))/34             =.3197936279 
*                      (10-2*SQRT(15))/40          =.0563508327 
*                      (10+2*SQRT(15))/40          =.4436491673 
*                      (13+3*SQRT(15))/34          =.7240867659 
*                      (13-3*SQRT(15))/34          =.0406191164
         DEXACT=5
         NBPTQ = 15
         AUX   = SQRT (15.)
         POID1 = (2665+14*AUX)/226800.
         POID2 = (2665-14*AUX)/226800.
         POID3 = 20./2268.
         CORQ1 = (7.-AUX)/34.
         CORQ2 = (7.+AUX)/34.
         CORQ3 = (10.-2*AUX)/40.
         CORQ4 = (10.+2*AUX)/40.
         CORQ5 = (13.+3*AUX)/34.
         CORQ6 = (13.-3*AUX)/34.
         AUX=1./4.
         DO 51 I=1,4  
            IP1=I+1  
            IP5=I+5  
            IP9=I+9  
            IP12=I+12
            POIDQ(IP1) = POID1
            POIDQ(IP5) = POID2
            DO 50 L  =1,3
               COORQ(L,IP1)  = CORQ1
               COORQ(L,IP5)  = CORQ2
               IF (I.EQ.4) GOTO 51  
               COORQ(L,IP9)  = CORQ3
               COORQ(L,IP12) = CORQ4
50          CONTINUE 
            POIDQ(IP9)  = POID3
            POIDQ(IP12) = POID3
51       CONTINUE 
         COORQ(2,5)    = CORQ1
         COORQ(2,9)    = CORQ2
         DO 52 L=1,3  
            COORQ(L,1)    = AUX
            COORQ(L,L+2)  = CORQ5
            COORQ(L,L+6)  = CORQ6
            COORQ(L,L+9)  = CORQ4
            COORQ(L,L+12) = CORQ3
52       CONTINUE
         POIDQ(1) = 16./810.
*
      ELSE
*
         CALL TEXXQD (NDIM,NUMERO,NBPTQ,POIDQ,COORQ,DEXACT)
      ENDIF
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 2 : D.Martin (25 mai 2009)
* - ajout d'une formule exacte pour P3
* Version 1.0.4 : D.Martin (12 janvier 2001)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !TetrQD
