*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE TRIAQD (NDIM,NUMERO,NBPTQ,POIDQ,COORQ,DEXACT)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Fevrier 1988) 
*          P.Carcaud (Juillet 2005) pour les formules 8, 12, 16, 20, 24
* Derniere modification : Y.Lafranche (08 janvier 2011), D.Martin (06 fevrier 2011)
* Version 5
*  
* Formules de quadrature sur le triangle rectangle isocele unite
* (programme pour que les valeurs des coeff. soit corrects en  double precision
*  pour les compilations implicites en double precision)
*  
*-- Variables d'entree --
* NDIM   dimension de COORQ  
* NUMERO numero du schema de quadrature (convention)
*
*   Formules "nodales"
*  -1   exacte P1 a 3 points ("trapezes)
*  -2   exacte P2 a 3 points (ajoutee le 20 juin 2004)
*  -3   exacte P3 a 7 points
*  
*   Formules diverses
*   1   exacte P1  a 1 point
*   2   exacte P2  a 3 points (ajoutee le 30 septembre 2005)
*   3   exacte P3  a 7 points (ajoutee le 20 juin 2004)
*  4-5  exacte P5  a 7 points
*
*   Formules de 'Gauss' (non optimales !!!)
*   6   exacte P6  a 20 points (ajoutee le 03 octobre 2005)
*  7-8  exacte P8  a 30 points (ajoutee le 30 septembre 2005)
*  9-10 exacte P10 a 42 points (ajoutee le 03 octobre 2005)
* 11-12 exacte P12 a 56 points (ajoutee le 30 septembre 2005)
* 2K-1,2K exacte P_{2K} a (2K+3)/2*(2K+5)/2 points (ajoutee le 03 octobre 2005)
*
*-- Variables en sortie --
* NBPTQ  nombre de points du schema de quadrature
* POIDQ  tableau des poids du schema de quadrature
* COORQ  tableau des coordonnees des points de quadrature
* DEXACT degre de precision de la formule (pour impression)
*
*-- Bibliographie --
* STROUD A.H. Approximate Calculation of Multiple Integrals
*             Prentice Hall, 1971
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      INTEGER NDIM,NUMERO,NBPTQ,DEXACT
      REAL    POIDQ(*),COORQ(NDIM,*)
*
      INTEGER I,J,NBPTQ1,DEGMAX,NBPMAX
      PARAMETER ( DEGMAX = 24, NBPMAX = (DEGMAX+2)/2 )
      DOUBLE PRECISION POIDQ1(NBPMAX),COORQ1(NBPMAX)
      DOUBLE PRECISION POIDQ2(NBPMAX),COORQ2(NBPMAX)
      REAL    AUX,PP1,PP2,PP3,PP4
*
      IF (NUMERO.GE.0) THEN
*-----------------------------------------------------------------------
*        Miscellaneous Formulas
*-----------------------------------------------------------------------
*
         IF (NUMERO.LE.1) THEN
*
*        Formule a 1 point exacte pour P1 sur le triangle
*
         DEXACT=1
         NBPTQ=1
         AUX=1./3. 
         COORQ(1,1)=AUX
         COORQ(2,1)=AUX
         POIDQ(1)=.5
*
         ELSEIF (NUMERO.LE.2) THEN
*
*        Formule de Hammer-Stroud a 3 points
*        exacte pour P2 sur le triangle (STROUD p. 307)
*
         DEXACT=2
         NBPTQ =3
         AUX=1./6.
         COORQ(1,1)=2./3.
         COORQ(2,1)=AUX
         POIDQ(1)=AUX
         COORQ(1,2)=AUX
         COORQ(2,2)=2./3.
         POIDQ(2)=AUX
         COORQ(1,3)=AUX
         COORQ(2,3)=AUX
         POIDQ(3)=AUX
*
         ELSEIF (NUMERO.LE.3) THEN
*
*        Formule 'nodale' a 7 points utilisant les noeuds de l'element P2 
*        et le barycentre
*        exacte pour P3 sur le triangle (STROUD p. 308)
*
         DEXACT=3
         AUX=2./30.
         NBPTQ=7
         CALL TRLKCO (2,NDIM,COORQ)
         DO 30 I=1,3
            POIDQ(I)=0.025
            POIDQ(I+3)=AUX
 30      CONTINUE
	      AUX=1./3.
         COORQ(1,7)=AUX
         COORQ(2,7)=AUX
         POIDQ(7)=0.225
*
         ELSEIF (NUMERO.LE.5) THEN
*
*        Formule de Radon, Hammer, Marlowe & Stroud a 7 points
*        exacte pour P5 sur le triangle (STROUD p.314)
*
*        Poids:         9/80=270/1200*2       =.1125
*                      (155-SQRT(15)/1200*2   =.5*.1259391806
*                      (155+SQRT(15)/1200*2   =.5*.1323941528
*        Coordonnees:  (6-SQRT(15))/21        =.10128650732
*                      (6+SQRT(15))/21        =.47014206411
*                      (9+2*SQRT(15))/21      =.79742698535
*                      (9-2*SQRT(15))/21      =.05971587181
         DEXACT=5
         NBPTQ=7
         AUX=SQRT(15.)
         PP1=(6.-AUX)/21.
         PP2=(6.+AUX)/21.
         PP3=(9.+2.*AUX)/21.
         PP4=(9.-2.*AUX)/21.
         COORQ(1,1)=1./3.
         COORQ(2,1)=1./3.
         COORQ(1,2)=PP1
         COORQ(2,2)=PP1
         COORQ(1,3)=PP1
         COORQ(2,3)=PP3
         COORQ(1,4)=PP3
         COORQ(2,4)=PP1
         COORQ(1,5)=PP2
         COORQ(2,5)=PP2
         COORQ(1,6)=PP2
         COORQ(2,6)=PP4
         COORQ(1,7)=PP4
         COORQ(2,7)=PP2
         PP1=(155.-AUX)/2400.
         PP2=(155.+AUX)/2400.
         POIDQ(1)=9./80.
         DO 52 I=1,3  
            POIDQ(I+1)=PP1
            POIDQ(I+4)=PP2
 52      CONTINUE
*
         ELSEIF (NUMERO.LE.DEGMAX) THEN
*
*        Formules non optimales construites par produit conique a (n+1)(n+2) points
*        pour n pair, n(n+1) pour n impair exactes pour P_{2n} sur le triangle 
*        (STROUD p. 28-31 & p. 314) utilisant 2 formules de Gauss-Legendre 1D
*
         DEXACT=NUMERO
         IF (MOD(NUMERO,2).GT.0) DEXACT=NUMERO+1
         NBPTQ1=(DEXACT+2)/2
*
*        Formule de Gauss-Legendre 1D
*        (seuls les (NBPTQ1+1)/2 points entre 0 et 0.5 (et les poids associes)
*        sont calcules et retournes dans COORQ2 et POIDQ2)
         CALL GAUSLE (NBPTQ1+1,COORQ1,POIDQ1)
         CALL GAUSLE (NBPTQ1,COORQ2,POIDQ2)
*
*        Reconstitution par symetrie de l'ensemble des points et poids 1D 
         J = NBPTQ1+1
         DO 61 I=1,(NBPTQ1+1)/2
            COORQ1(J) = 1-COORQ1(I)
            POIDQ1(J) = POIDQ1(I)
            J=J-1
 61      CONTINUE   
         J = NBPTQ1
         DO 62 I=1,NBPTQ1/2
            COORQ2(J) = 1-COORQ2(I)
            POIDQ2(J) = POIDQ2(I)
            J=J-1
 62      CONTINUE
*
*        On calcule les points et poids 2D par produit conique
         NBPTQ = 0
         DO 63 I=1,NBPTQ1+1
            DO 64 J=1,NBPTQ1
               NBPTQ = NBPTQ+1
               POIDQ(NBPTQ)   = POIDQ1(I)*POIDQ2(J)*(1-COORQ1(I))
               COORQ(1,NBPTQ) = COORQ1(I)
               COORQ(2,NBPTQ) = (1-COORQ1(I))*COORQ2(J)
 64         CONTINUE
 63      CONTINUE
*
         ELSE
            CALL TRXXQD (NDIM,NUMERO,NBPTQ,POIDQ,COORQ,DEXACT)
         ENDIF
*
      ELSE
*-----------------------------------------------------------------------
*        Formules Nodales
*-----------------------------------------------------------------------
         IF (NUMERO.EQ.-1) THEN
*
*        Formule 'nodale' utilisant les noeuds de l'element Lagrange P1
*        exacte pour P1
*
         DEXACT=1
         AUX=1./6.
         NBPTQ=3
         CALL TRLKCO (1,NDIM,COORQ)
         DO 1001 I=1,NBPTQ
            POIDQ(I)=AUX
 1001    CONTINUE
*
         ELSEIF (NUMERO.EQ.-2) THEN
*
*        Formule 'nodale' utilisant les noeuds de l'element P1 non conforme
*        exacte pour P2
*
         DEXACT=2
         AUX=1./6.
         NBPTQ=3
         CALL TR201C (NDIM,COORQ)
         DO 1002 I=1,NBPTQ
            POIDQ(I)=AUX
 1002    CONTINUE
*
         ELSEIF (NUMERO.EQ.-3) THEN
*
*        Formule 'nodale' utilisant les noeuds de l'element P2 et le barycentre
*        exacte pour P3 sur le triangle (STROUD p. 308)
*
         DEXACT=3
         AUX=2./30.
         NBPTQ=7
         CALL TRLKCO (2,NDIM,COORQ)
         DO 1003 I=1,3
            POIDQ(I)=0.025
            POIDQ(I+3)=AUX
 1003    CONTINUE
	      AUX=1./3.
         COORQ(1,7)=AUX
         COORQ(2,7)=AUX
         POIDQ(7)=0.225
*
         ELSE
*
            CALL TRXXQD (NDIM,NUMERO,NBPTQ,POIDQ,COORQ,DEXACT)
*
         ENDIF
*
      ENDIF
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 5 : Y.Lafranche (08 janvier 2011), D.Martin (06 fevrier 2011)
*  - Bug pour NUMERO > 5 (produit conique) provoquant un debordement de tableau:
*  permutation des arguments numero 1 dans les deux appels de GAUSLE
* Version 4 : D.Martin (07 octobre 2008)
*  - Bug dans la formule du produit conique ! 
*  - Bug dans la formule Hammer-Stroud a 3 points ! (On avait 3/4 au lieu de 2/3)
* Version 3 : D.Martin (08 mai 2008)
*  - Correction du bug introduit en 2005 interdisant l'utilisation des formules 
*  nodales ( NUSCHQ < 0 )
* Version 2 : D.Martin (03 octobre 2005)
*  - Modif pour les formules 8, 12, 16, 20, 24
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !TriaQd
