*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE TRLKP1 (K,NBLMP1,NBPOP1)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (12 Juillet 2001)
* Derniere modification : D.Martin (12 Fevrier 2004)
* Version 1.1
*
* Construction des tableaux de correspondance des numerotations locales
* entre elements triangulaires Pk et elements triangulaires P1.
* Le quadrangle Pk est decoupe en k^2 triangles P1 par exemple, 
* le triangle P2 est decoupe en 4 triangles P1 :
*
*          2                            3 
*          | \                          | \
*          |   \                        |   \
*          |     \                      | #2  \
*          5       4       ------->     5-------4
*          |         \                  | \  #4 | \
*          |           \                |   \   |   \
*          |             \              | #3  \ | #1  \
*          3-------6-------1            3-------6-------1
*
*-- Argument d'entree --
* K      pour un quadrangle de degre k
*-- Arguments de sortie --
* NBLMP1 nombre de triangles P1
* NBPOP1 tableau de correspondance entre triangles P1 et triangle Pk
*        NBPOP1 (s,t) = numero dans le triangle Pk du sommet s du triangle t
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      INTEGER K,NBLMP1,NBPOP1(3,*)
*
      INTEGER NK,P1,P2,P3,P4,KORNE1,KORNER,PREC,J
*----------------------------------------------------------------------
      NBLMP1=0
      NK=K   
      PREC=0
      KORNE1=1
*
10    IF (NK.GT.3) THEN   
*
*       2                  2                      2
*       | \                | \                    | \
*       5  10              5  13                  5  16
*       |     \            |     \                |     \
*       8  14   7          8  17  10              8  20  13
*       |         \        |         \            |         \
*      11  15  13   4     11  20  19   7         11  23  25  10
*       |             \    |             \        |             \
*       3---6---9--12---1 14  18  21  16   4     14  26  28  22   7
*              k=4         |                 \    |                 \
*                          3---6---9--12--15---1 17  21  24  27  19   4
*                                   k=5           |                    \
*                                                 3---6---9--12--15--18--1
*                                                            k=6
*       La numerotation est creee en commencant par la couche d'elements
*       la plus externe et l'element contenant le point No 1.
*       Elle est reiteree pour les couches plus internes jusqu'a
*       obtenir 1,4 ou 9 triangles.
*
*        Numero du premier 'coin' interne de la couche 
*             (13 pour k=4, 16 pour k=5, 19 pour k=6)
         KORNER=KORNE1-PREC
         KORNE1=KORNE1+3*NK
*
         P1=KORNER
         P3=KORNE1-1-PREC
         P4=KORNE1-PREC
         DO 11 J=1,NK-2
            P2=P1+3
            NBLMP1=NBLMP1+1
            NBPOP1(1,NBLMP1) = P1 + PREC
            NBPOP1(2,NBLMP1) = P2 + PREC
            NBPOP1(3,NBLMP1) = P3 + PREC
            NBLMP1=NBLMP1+1
            NBPOP1(1,NBLMP1) = P4 + PREC
            NBPOP1(2,NBLMP1) = P3 + PREC
            NBPOP1(3,NBLMP1) = P2 + PREC
            P1=P2
            P3=P4
            P4=P4+3
            P2=P2+3
            IF (J.EQ.NK-3) THEN
               KORNE1=KORNE1+1
               P4=KORNE1-PREC
            ENDIF
11       CONTINUE
         NBLMP1=NBLMP1+1
         NBPOP1(1,NBLMP1) = P1 + PREC
         NBPOP1(2,NBLMP1) = P2 + PREC
         NBPOP1(3,NBLMP1) = KORNE1
*
         KORNER=KORNER+1
         P1=P2
         P2=KORNER
         P3=P2+3
         P4=KORNE1-PREC
         DO 12 J=1,NK-2
            P3=P2+3
            NBLMP1=NBLMP1+1
            NBPOP1(1,NBLMP1) = P1 + PREC
            NBPOP1(2,NBLMP1) = P2 + PREC
            NBPOP1(3,NBLMP1) = P3 + PREC
            NBLMP1=NBLMP1+1
            NBPOP1(1,NBLMP1) = P3 + PREC
            NBPOP1(2,NBLMP1) = P4 + PREC
            NBPOP1(3,NBLMP1) = P1 + PREC
            P1=P4
            P2=P3
            P4=P4+3
            IF (J.EQ.NK-3) THEN
               KORNE1=KORNE1+1
               P4=KORNE1-PREC
            ENDIF
12       CONTINUE
         NBLMP1=NBLMP1+1
         P3=P2+3
         NBPOP1(1,NBLMP1) = P1 + PREC
         NBPOP1(2,NBLMP1) = P2 + PREC
         NBPOP1(3,NBLMP1) = P3 + PREC
*
         KORNER=KORNER+1
         P4=P1
         P2=P3
         P3=KORNER
         DO 13 J=1,NK-2
            P1=P3+3
            NBLMP1=NBLMP1+1
            NBPOP1(1,NBLMP1) = P1 + PREC
            NBPOP1(2,NBLMP1) = P2 + PREC
            NBPOP1(3,NBLMP1) = P3 + PREC
            NBLMP1=NBLMP1+1
            NBPOP1(1,NBLMP1) = P2 + PREC
            NBPOP1(2,NBLMP1) = P1 + PREC
            NBPOP1(3,NBLMP1) = P4 + PREC
            P3=P1
            P2=P4
            P1=P1+3
            P4=P4+3
            IF (J.EQ.NK-3) THEN
               KORNE1=KORNE1-2
               P4=KORNE1-PREC
            ENDIF
13       CONTINUE
         NBLMP1=NBLMP1+1
         NBPOP1(1,NBLMP1) = P1 + PREC
         NBPOP1(2,NBLMP1) = P2 + PREC
         NBPOP1(3,NBLMP1) = P3 + PREC
*
         PREC=KORNE1-1
         NK=NK-3
         GOTO 10
      ENDIF
*
*     On numerote finalement le ou les triangles centraux
*
      IF (NK.EQ.1) THEN 
*
         NBLMP1=NBLMP1+1
         NBPOP1(1,NBLMP1) = 1 + PREC
         NBPOP1(2,NBLMP1) = 2 + PREC
         NBPOP1(3,NBLMP1) = 3 + PREC
*
      ELSEIF (NK.EQ.2) THEN
*      
*       2                      3 
*       | \                    | \
*       |   \                  |   \
*       |     \                | #2  \
*       5       4     ------>  5-------4
*       |         \            | \  #4 | \
*       |           \          |   \   |   \
*       |             \        | #3  \ | #1  \
*       3-------6-------1      3-------6-------1
*
         NBLMP1=NBLMP1+1
         NBPOP1(1,NBLMP1) = 1 + PREC
         NBPOP1(2,NBLMP1) = 4 + PREC
         NBPOP1(3,NBLMP1) = 6 + PREC
         NBLMP1=NBLMP1+1
         NBPOP1(1,NBLMP1) = 4 + PREC
         NBPOP1(2,NBLMP1) = 2 + PREC
         NBPOP1(3,NBLMP1) = 5 + PREC
         NBLMP1=NBLMP1+1
         NBPOP1(1,NBLMP1) = 6 + PREC
         NBPOP1(2,NBLMP1) = 5 + PREC
         NBPOP1(3,NBLMP1) = 3 + PREC
         NBLMP1=NBLMP1+1
         NBPOP1(1,NBLMP1) = 5 + PREC
         NBPOP1(2,NBLMP1) = 6 + PREC
         NBPOP1(3,NBLMP1) = 4 + PREC
*
      ELSEIF (NK.EQ.3) THEN
*      
*       2                       2
*       | \                     | \
*       |   \                   |#4 \
*       5     7                 5-----7 
*       |       \    ------->   | \#5 | \ 
*       |         \             |#6 \ |#3 \
*       8    10     4           8----10-----4
*       |             \         | \ #8| \#2 | \
*       |               \       |#7 \ |#9 \ |#1 \
*       3-----6-----9-----1     3-----6-----9-----1
*
         NBLMP1=NBLMP1+1
         NBPOP1(1,NBLMP1) = 1 + PREC
         NBPOP1(2,NBLMP1) = 4 + PREC
         NBPOP1(3,NBLMP1) = 9 + PREC
         NBLMP1=NBLMP1+1
         NBPOP1(1,NBLMP1) =10 + PREC
         NBPOP1(2,NBLMP1) = 9 + PREC
         NBPOP1(3,NBLMP1) = 4 + PREC
         NBLMP1=NBLMP1+1
         NBPOP1(1,NBLMP1) = 4 + PREC
         NBPOP1(2,NBLMP1) = 7 + PREC
         NBPOP1(3,NBLMP1) =10 + PREC
         NBLMP1=NBLMP1+1
         NBPOP1(1,NBLMP1) = 7 + PREC
         NBPOP1(2,NBLMP1) = 2 + PREC
         NBPOP1(3,NBLMP1) = 5 + PREC
         NBLMP1=NBLMP1+1
         NBPOP1(1,NBLMP1) = 5 + PREC
         NBPOP1(2,NBLMP1) =10 + PREC
         NBPOP1(3,NBLMP1) = 7 + PREC
         NBLMP1=NBLMP1+1
         NBPOP1(1,NBLMP1) =10 + PREC
         NBPOP1(2,NBLMP1) = 5 + PREC
         NBPOP1(3,NBLMP1) = 8 + PREC
         NBLMP1=NBLMP1+1
         NBPOP1(1,NBLMP1) = 6 + PREC
         NBPOP1(2,NBLMP1) = 8 + PREC
         NBPOP1(3,NBLMP1) = 3 + PREC
         NBLMP1=NBLMP1+1
         NBPOP1(1,NBLMP1) = 6 + PREC
         NBPOP1(2,NBLMP1) = 8 + PREC
         NBPOP1(3,NBLMP1) = 10 + PREC
         NBLMP1=NBLMP1+1
         NBPOP1(1,NBLMP1) = 9 + PREC
         NBPOP1(2,NBLMP1) =10 + PREC
         NBPOP1(3,NBLMP1) = 6 + PREC
*
      ENDIF
      END

