*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE EFLAVA (NMTRMA,NVTRMA,NMTRSP,NVTRSP
     &                  ,NMTERM,NVTERM,NIVIMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (15 Juin 2003)
* Derniere modification : D.Martin & C. Chambeyron (13 novembre 2009)
* Version 3
*
* Calcul d'un terme 'element finis localises' : R = A S A* 
*                      R_{ij} = A_{ip} * S_{pk} * A*_{kj}
*
* A designe la matrice NMTRMA,NVTRMA
* S designe le terme spectral NMTRSP,NVTRSP
*
* La matrice A est de type Elements Finis Localises (ELFLQU) et 
* est stockee PLEIN-L ou PLEIN-C
*
* La matrice S est de type spectrale (ELFLQU ou SPECTR) et est soit
* diagonale (cas orthogonal) et stockee
* - PLEIN-C si ses inconnues en colonne et ligne sont identiques
* - PLEIN-L si son inconnue en ligne a un numero superieur a celui de son 
*           inconnue en colonne
* - PLEIN-C si son inconnue en ligne a un numero inferieur a celui de son 
*           inconnue en colonne
*
* La matrice R est stockee BIMORSE (ses inconnues sont identiques)
*
*-- Arguments --
* NMTRMA nom de la matrice (A), variationnel en ligne, spectral en colonne
* NVTRMA niveau de la matrice (A)
* NMTRSP nom du vecteur des coefficients spectraux (S)
* NVTRSP niveau du vecteur des coefficients spectraux (S)
* NMTERM nom de la matrice Elements Finis Localises assemblee (R)
* NVTERM niveau du terme (R)
* NIVIMP niveau d'impression
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*)  NMTRMA,NMTRSP,NMTERM 
      INTEGER        NVTRMA,NVTRSP,NVTERM,NIVIMP
*
      INCLUDE 'ALLOC'
      INCLUDE 'CONTEX'
*
      LOGICAL    CRETRM
      CHARACTER  STOIND*7,TYPINT*9,YKLINT*6,SYMIND*14
      CHARACTER  TYPEMA*9,STOIMA*7,TYPESP*9,STOISP*7,TYPEMR*9,STOIMR*7
     &          ,ERCODE*120,PRFXAS*8,STOISW*7
      INTEGER    INDSTO,INDSYM,INTTYP,INTYKL,KLNIVE,KLTERM
      INTEGER    NUTERM,MCDTRM,LGETRM,INCTRM,NCHTRM,NBTERM,MCMTRM
     &          ,LGMTRM,LGDTRM,MCDONN,MCMDON,MCINCO,NIVIMT
     &          ,ITYPTR,ITYCAL,NUKALE,NUDONN,KTYSYM,KVDSMB,INDSTR,NVSTRM
     &          ,ITYPMA,KUINXC,KVDUXC,NBCOMA,KVNUXC,ITYSYM
     &          ,NUINML,NVDUAL,NBLIMA,NVNUAL,INDSMA,IATRDA,MCTRMA
     &          ,ITYPSP,NBCOSP,KUINSL,KVDUSL,NBLISP,KVNUSL,INDSSP,MCTRSP
     &          ,KVNUMS,KVCORC,IATRDO,NIVIMA 
     &          ,KUCTRM,KVDCRM,KBCOLO,KVNCRM
     &          ,KULTRM,KVDLRM,KBLIGN,KVNLRM,KNDSTR,KVSTRM
     &          ,KVVTRM,IATTRM,LGTERM,MCTERM,INDSMW,MCMATW
     &          ,LGBMOR,MCBMOR,LGINCO,NCATR1,NCATR2,NCATR3
     &          ,NCPINL,INTERL,NBTNEL
      COMMON/FORMAH/ERCODE
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*EflAVA*')
*
      IF (NIVIMP.GT.0.AND.IMPSDR.GT.0) THEN
         CALL PRNTRM (NMTERM,NVTERM,ERCODE,NCATR1)                      !utilite
         CALL PRNTRM (NMTRMA,NVTRMA,ERCODE(NCATR1+1:),NCATR2)
         CALL PRNTRM (NMTRSP,NVTRSP,ERCODE(NCATR1+NCATR2+1:),NCATR3)
         WRITE (IMPSDR,10001) PRFXAS(0),ERCODE(1:NCATR1)
     &      ,ERCODE(NCATR1+1:NCATR1+NCATR2)
     &      ,ERCODE(NCATR1+NCATR2+1:NCATR1+NCATR2+NCATR3)
     &      ,ERCODE(NCATR1+1:NCATR1+NCATR2)
      ENDIF
*
*     On initialise l'indice de stockage comme Bi-Morse
      STOIMR = 'BIMORSE'
*
      CALL TBRR3  (ERCODE,'#OMTRM',1,LGMTRM
     &                   ,'$SDTRM',1,LGDTRM,'#NCONU',1,LGINCO)
      CALL TBAR3  (ERCODE,'#OMTRM',1,MCMTRM
     &                   ,'$SDTRM',1,MCDTRM,'#NCONU',1,MCINCO)
      CALL SDEXDB (IST(MCDTRM),LGETRM,NBTERM,INCTRM,NCHTRM)             !sdexplo
*
*     Caracteristiques de la matrice A
*
      NUTERM=KLTERM (NMTRMA,NVTRMA,AST(MCMTRM),IST(MCDTRM))             !sdexplo
      IF (NUTERM.LE.0) CALL ERTERM (1,NMTRMA,NVTRMA)                    !utilite
      CALL GETTRM (IST(MCDTRM+LGETRM+INCTRM*(NUTERM-1)),KVVTRM
     &            ,ITYPMA,ITYCAL,NUKALE,NUDONN,KTYSYM,KVDSMB
     &            ,KUINXC,KVDUXC,NBCOMA,KVNUXC
     &            ,NUINML,NVDUAL,NBLIMA,NVNUAL
     &            ,INDSMA,KVNUMS,KVCORC,IATRDA,NIVIMA)                  !sdexplo
      TYPEMA=TYPINT (ITYPMA)                                            !utilite
      STOIMA=STOIND (INDSMA)                                            !utilite
      IF (ITYCAL.NE.INTYKL ('ELFLQU').AND.
     &    ITYCAL.NE.INTYKL ('ELFLOC')) CALL ERTERM (44,NMTRMA,NVTRMA)   !utilite
      IF (STOIMA.NE.'PLEIN-L'.AND.STOIMA.NE.'PLEIN-C') 
     &   CALL ERTERM (18,NMTRMA,NVTRMA)                                 !utilite
*     Nb de composantes de l'inconnue en ligne de la matrice A
      CALL SDEXCO (NUINML,IST(MCINCO),NCPINL,INTERL,NBTNEL)             !utilite
*
*     Caracteristiques du terme des coefficients spectraux S
*
      NUTERM=KLTERM (NMTRSP,NVTRSP,AST(MCMTRM),IST(MCDTRM))             !sdexplo
      IF (NUTERM.LE.0) CALL ERTERM (1,NMTRSP,NVTRSP)                    !utilite
      CALL GETTRM (IST(MCDTRM+LGETRM+INCTRM*(NUTERM-1)),KVVTRM
     &            ,ITYPSP,ITYCAL,NUKALE,NUDONN,ITYSYM,KVDSMB
     &            ,KUINXC,KVDUXC,NBCOSP,KVNUXC
     &            ,KUINSL,KVDUSL,NBLISP,KVNUSL
     &            ,INDSSP,KVNUMS,KVCORC,IATRDO,NIVIMA)                  !sdexplo
      TYPESP=TYPINT (ITYPSP)                                            !utilite
      STOISP=STOIND (INDSSP)                                            !utilite
      IF (ITYCAL.NE.INTYKL ('SPECTR').AND.ITYCAL.NE.INTYKL ('ELFLQU'))
     & CALL ERTERM (41,NMTRSP,NVTRSP)                                   !utilite
*
      ITYSYM=INDSYM(' ')                                                !utilite
      IF (NBLISP.GT.1.AND.NBCOSP.GT.1) THEN
*
*        Cas d'un terme spectral non diagonal
*
         IF (NBCOMA.NE.NBLISP.OR.NBCOMA.NE.NBCOSP) GOTO 90001
      ELSE
         STOISP = 'DIAG'
         ITYSYM=INDSYM ('SYMETRIQUE')                                   !utilite
         IF (TYPESP(1:1).EQ.'C') ITYSYM=INDSYM('AUTOADJOINT')           !utilite
         IF (NBLISP.GT.1) THEN
            IF (NBCOMA.NE.NBLISP) GOTO 90001
         ELSE
            IF (NBCOMA.NE.NBCOSP) GOTO 90001
         ENDIF
      ENDIF
*
*     Creation ou mise a jour des caracteristiques du terme resultat (R)
*
      NUDONN=NDFDON
      NVSTRM=NDFSTO
      KVSTRM=NDFSTO      
      NIVIMT=NIVIMP
      NUTERM=KLTERM (NMTERM,NVTERM,AST(MCMTRM),IST(MCDTRM))             !sdexplo
      IF (NUTERM.GT.0) THEN
         CALL GETTRM (IST(MCDTRM+LGETRM+INCTRM*(NUTERM-1)),KVVTRM
     &               ,ITYPTR,ITYCAL,NUKALE,NUDONN,KTYSYM,KVDSMB
     &               ,KUCTRM,KVDCRM,KBCOLO,KVNCRM
     &               ,KULTRM,KVDLRM,KBLIGN,KVNLRM
     &               ,KNDSTR,KVSTRM,KVCORC,IATTRM,NIVIMT)               !sdexplo
      ENDIF
*
      TYPEMR=TYPEMA
      IF (TYPESP(1:8).EQ.'COMPLEXE') TYPEMR=TYPESP
      ITYPTR=INTTYP (TYPEMR)                                            !utilite
      INDSTR=INDSTO (STOIMR)                                            !utilite
      ITYCAL=INTYKL ('ELFLOC')                                          !utilite
      NUKALE=1
      IF (KVSTRM.NE.NDFSTO) NVSTRM=KVSTRM
      IF (STOIMR.EQ.'BIMORSE'.AND.NVSTRM.EQ.NDFSTO) NVSTRM=KLNIVE()     !sdexplo
*
*     Longueur du tableau recepteur du terme resultat
*
****  Pas de symetrie pour le moment
      ITYSYM=INDSYM(' ')                                                !utilite
      IF (ITYSYM.EQ.INDSYM(' ')) THEN
          LGTERM=NBLIMA*NBLIMA*NCPINL*NCPINL
      ELSE
          LGTERM=NBLIMA*NCPINL+(NBLIMA*NCPINL*(NBLIMA*NCPINL-1))/2
      ENDIF
*
*     Creation du terme resultat
*
      CALL CRTERM (NMTERM,NVTERM,LGTERM,ITYPTR,ITYCAL,NUKALE
     &            ,NUDONN,ITYSYM,NDFDSM
     &            ,NUINML,NVDUAL,NBLIMA,NVNUAL
     &            ,NUINML,NVDUAL,NBLIMA,NVNUAL
     &            ,INDSTR,NVSTRM,NDFCOR,IATRDA,NIVIMT,.FALSE.,CRETRM)   !sdexplo
*
*     Creation de la structure Bi-Morse symetrique du terme resultat (R)
*
***      IF (STOIMR.EQ.'BIMORSE')
      CALL CRBMOP (NVSTRM,NBLIMA,NBLIMA,IST)                            !sdexplo
*
*     Adresse des termes
*
      IF (STOISP(1:4).NE.'DIAG')
     &   CALL TBCREE ('EFL_WK',0,ITYPTR,NBLIMA*NBCOMA,' ')
      CALL TBRR3  (ERCODE,NMTRMA,NVTRMA,MCTRMA,NMTRSP,NVTRSP,MCTRSP
     &                   ,NMTERM,NVTERM,MCTERM)
      CALL TBAR3  (ERCODE,NMTRMA,NVTRMA,MCTRMA,NMTRSP,NVTRSP,MCTRSP
     &                   ,NMTERM,NVTERM,MCTERM)
*
*     Construction du terme resultat (R)
*
      IF (STOISP(1:4).EQ.'DIAG') THEN
*
*        Cas d'une matrice spectrale diagonale
*
*        Stockage de la matrice resultat sous forme BIMORSE
         CALL EFLBRO (TYPEMA,NBLIMA*NCPINL,STOIMA,MCTRMA
     &                      ,NBLIMA*NCPINL,STOIMA,MCTRMA
     &               ,TYPESP,NBCOMA,MCTRSP,ITYSYM,MCTERM,RST,CST)       !efloca
*
      ELSE
*
*        Produit des matrices A et S : C_{ik} = A_{ip} * S{pk}
*
         CALL TBAR1  (ERCODE,'EFL_WK',0,MCMATW)
         STOISW='PLEIN-L'
         IF (STOISP.EQ.'BIMORSE') THEN
            CALL PLXPLB (TYPEMA,NBLIMA*NCPINL,NBCOMA,STOIMA,MCTRMA
     &                  ,TYPESP       ,NBCOMA,STOISP,MCTRSP
     &                                       ,STOISW,MCMATW,RST,CST)    !multmat
         ELSEIF (STOISP.EQ.'PLEIN-L'.OR.STOISP.EQ.'PLEIN-C') THEN
            CALL PLXPL  (TYPEMA,NBLIMA*NCPINL,NBCOMA,STOIMA,MCTRMA
     &                  ,TYPESP       ,NBCOMA,STOISP,MCTRSP
     &                                       ,STOISW,MCMATW,RST,CST)    !multmat
         ELSE
            CALL ERTERM (18,NMTRSP,NVTRSP)                              !utilite
         ENDIF
*
*        Produit des matrices AS et A* : R_{ij} = C_{ik} * A*_{kj}
*
*        Stockage de la matrice resultat sous forme BIMORSE
         CALL EFLBRM (NBLIMA*NCPINL,STOISW,MCMATW
     &               ,TYPEMA,NBLIMA*NCPINL,NBCOMA,STOIMA,MCTRMA
     &               ,TYPEMR,ITYSYM,STOIMR,MCTERM,RST,CST)              !efloca
         CALL TBTUER ('EFL_WK',0)
*
      ENDIF
*
*--   Sauvegarde & Impressions
      IF (NIVIMP.GT.0) CALL PRTERM (NMTERM,NVTERM,NIVIMP,IMPSDR)        !sdexplo
      CALL TBSAVE (NMTRMA,NVTRMA)
      CALL TBSAVE (NMTRSP,NVTRSP)
      CALL TBSAVE (NMTERM,NVTERM)
 
      IF (NIVIMP.GE.0.AND.IMPPAL.GT.0) THEN
         CALL PRNTRM (NMTERM,NVTERM,ERCODE,NCATR1)                      !utilite
         IF (ITYSYM.EQ.INDSYM (' ')) THEN
            WRITE (IMPPAL,10000) PRFXAS (0),ERCODE(1:NCATR1)
     &                     ,YKLINT(ITYCAL),TYPEMR,STOIMR                !utilite
         ELSE
            WRITE (IMPPAL,10000) PRFXAS (0),ERCODE(1:NCATR1)
     &                     ,YKLINT(ITYCAL),TYPEMR,STOIMR,SYMIND(ITYSYM) !utilite
         ENDIF
      ENDIF
*
      CALL PRFXMJ (-1,'*EflAVA*')
      RETURN
*
90001 WRITE(*,*) '* *EflAVA *******************************************'
     &,'***************************'
      WRITE(*,*) 'A : ',NBLIMA,' x ',NBCOMA,' V : ',NBLISP,' x ',NBCOSP
     &,' A* : ',NBCOMA,' x ',NBLIMA
      CALL ERTERM (43,NMTRSP,NVTRSP)                                    !utilite
*-----------------------------------------------------------------------
10001 FORMAT(/T2,A8,' Calcul du terme element finis localise assemble '
     &,A/T11,'Produit des termes ',A,' * ',A,' * (',A,')*')
10000 FORMAT(T5,A8,' Terme ',A,' de type ',A,', ',A,:,', ',A,:,', ',A)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 3 : D.Martin & C. Chambeyron (13 novembre 2009)
* - cas d'une matrice A a inconnue variationnelle vectorielle
* Version 2 : D.Martin & C. Chambeyron (20 octobre 2006)
*  - ajout du cas non orthogonal : le terme spectral est dans ce cas
*    une matrice non diagonale
*  - eclatement du produit des 3 matrices
*  - utilisation des procedures utilitaires de multiplication de matrices
*    dans le cas de matrices non stockees BIMORSE (PLXPL, PLXPLB)
* Version 1.1 : D.Martin (16 juin 2005)
*  - bug d'initialisation du niveau de stockage initial du resultat
* Version 1 : D.Martin (15 juin 2003)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !EflAVA
