*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE EFLBRM (NBLIMC,STOIMC,MCTRMC
     &                  ,TYPEMB,NBLIMB,NBCOMB,STOIMB,MCTRMB
     &                  ,TYPETR,ITYSYM,STOIMR,MCTERM,RST,CST)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : C.Chambeyron (6 Juillet 2006)
* Derniere modification : D.Martin (20 octobre 2006)
* Version 2
*
* Calcul d'un terme element finis localise stocke sous forme 'BiMORSE'
*          R_{ij} = C_{ik} * B*_{kj} = A_{ip} * S_{pk} * B*_{kj}
*
* La matrice (intermediaire) C est stockee PLEIN-L
* La matrice B est stockee PLEIN-L ou PLEIN-C
* La matrice resultat bien que pleine est stockee sous forme BIMORSE
*
*-- Arguments d'entree --
* NBLIMC nombre de lignes de la matrice C et du resultat R (et de A)
* MCTRMC adresse de la matrice C stockee sous forme PLEIN-L
* TYPEMB type (REEL ou COMPLEXE) de la B
* NBLIMB nombre de lignes de la matrice B et de colonnes du resultat R 
* NBCOMB nombre de colonnes de la matrice B (et de colonnes de C)
* STOIMB type de stockage de la matrice B (necessairement PLEIN-C ou PLEIN-L)
* MCTRMB adresse du terme en colonne, matrice B
* TYPETR type de la matrice resultat R et de la matrice C
* ITYSYM indicateur de symetrie du terme resultat
* STOIMB type de stockage de la matrice R (necessairement BIMORSE)
* MCTERM adresse du terme resultat stocke BIMORSE
* RST,CST super tableau, version REEL et COMPLEXE
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) STOIMC,TYPEMB,STOIMB,TYPETR,STOIMR
      INTEGER NBLIMC,MCTRMC,NBLIMB,NBCOMB,MCTRMB,MCTERM,ITYSYM
      REAL    RST(*)
      COMPLEX CST(*)
*
      INTEGER INDSTO,INDSYM
      INTEGER INCMBL,INCMBC,INCMCL,INCMCC,I,J,CIK,CI1,BJ1,BJK,RIJ
      REAL    RSOM
      COMPLEX CSOM
*-----------------------------------------------------------------------
*
*     Increments de changement de ligne ou de colonne selon le stockage
*
*     INCMBL : increment d'adresse pour le changement de ligne dans B
*     INCMBC : increment d'adresse pour le changement de colonne dans B
      INCMBL=1
      INCMBC=1
      IF (STOIMB(1:7).EQ.'PLEIN-C') THEN
         INCMBL=NBLIMB
      ELSEIF (STOIMB(1:7).EQ.'PLEIN-L') THEN
         INCMBC=NBCOMB
      ELSE
         CALL BAISE  ('EflBrm: Matrice B stockee ni PLEIN-L ni PLEIN-C')
      ENDIF
*
      IF (STOIMC(1:7).NE.'PLEIN-L')
     &   CALL BAISE  ('EflBrm: Matrice C non stockee PLEIN-L.')
      IF (STOIMR(1:7).NE.'BIMORSE')
     &   CALL BAISE  ('EflBrm: Matrice R non stockee BIMORSE.')
      INCMCL=1
      INCMCC=NBCOMB
      RIJ=MCTERM
*
      IF (TYPETR(1:1).EQ.'C') THEN
*        Matrice Resultat (NBLIMC*NBLIMB) complexe
*
         IF (TYPEMB(1:1).EQ.'C') THEN
*           Matrice B complexe et matrice spectrale complexe
*
*           Partie diagonale de R
*
            BJ1=MCTRMB
            CIK=MCTRMC
            DO 442 I=1,MIN (NBLIMC,NBLIMB)
               CSOM=0.
*              Calcul du coefficient R_{ii} 
*              Produit de la ligne i de C par la ligne i de B
               DO 441 BJK=BJ1,BJ1-1+NBCOMB*INCMBL,INCMBL
                  CSOM=CSOM+CST(CIK)*CONJG(CST(BJK))
                  CIK=CIK+INCMCL
441            CONTINUE
               BJ1=BJ1+INCMBC
               CST(RIJ)=CSOM
               RIJ=RIJ+1
442         CONTINUE
*
*           Partie triangulaire inferieure de R
*
            CI1=MCTRMC
            DO 445 I=1,NBLIMC
               BJ1=MCTRMB
*              Calcul des coefficients R_{ij}  j = 1, i-1
*              Produit de la ligne i de C par les lignes j de B
               DO 444 J=1,MIN(I-1,NBLIMB)
                  CSOM=0.
*                 Premier coeff. de la ligne i de C
                  CIK=CI1
                  DO 443 BJK=BJ1,BJ1-1+NBCOMB*INCMBL,INCMBL
                     CSOM=CSOM+CST(CIK)*CONJG(CST(BJK))
                     CIK=CIK+INCMCL
443               CONTINUE
                  BJ1=BJ1+INCMBC
                  CST(RIJ)=CSOM
                  RIJ=RIJ+1
444            CONTINUE
               CI1=CI1+INCMCC
445         CONTINUE
*
*           Partie triangulaire superieure de R
*
            IF (ITYSYM.NE.INDSYM('SYMETRIQUE')) THEN
               BJ1=MCTRMB
               DO 449 J=1,NBLIMB
*                 Calcul des coefficients R_{ij}  i = 1, j-1
*                 Produit des lignes i de C par la ligne j de B
                  CIK=MCTRMC
                  DO 448 I=1,MIN(J-1,NBLIMC)
                     CSOM=0.
                     DO 447 BJK=BJ1,BJ1-1+NBCOMB*INCMBL,INCMBL
                        CSOM=CSOM+CST(CIK)*CONJG(CST(BJK))
                        CIK=CIK+INCMCL
447                  CONTINUE
                     CST(RIJ)=CSOM
                     RIJ=RIJ+1 
448               CONTINUE
                  BJ1=BJ1+INCMBC
449            CONTINUE
            ENDIF
*
         ELSE
*           Matrice B reelle et matrice spectrale complexe
*
*           Partie diagonale de R
*
            BJ1=MCTRMB
            CIK=MCTRMC
            DO 422 I=1,MIN (NBLIMC,NBLIMB)
               CSOM=0.
*              Calcul du coefficient R_{ii} 
*              Produit de la ligne i de C par la ligne i de B
               DO 421 BJK=BJ1,BJ1-1+NBCOMB*INCMBL,INCMBL
                  CSOM=CSOM+CST(CIK)*RST(BJK)
                  CIK=CIK+INCMCL
421            CONTINUE
               BJ1=BJ1+INCMBC
               CST(RIJ)=CSOM
               RIJ=RIJ+1
422         CONTINUE
*
*           Partie triangulaire inferieure de R
*
            CI1=MCTRMC
            DO 425 I=1,NBLIMC
               BJ1=MCTRMB
*              Calcul des coefficients R_{ij}  j = 1, i-1
*              Produit de la ligne i de C par les lignes j de B
               DO 424 J=1,MIN(I-1,NBLIMB)
                  CSOM=0.
*                 Premier coeff. de la ligne i de C
                  CIK=CI1
                  DO 423 BJK=BJ1,BJ1-1+NBCOMB*INCMBL,INCMBL
                     CSOM=CSOM+CST(CIK)*RST(BJK)
                     CIK=CIK+INCMCL
423               CONTINUE
                  BJ1=BJ1+INCMBC
                  CST(RIJ)=CSOM
                  RIJ=RIJ+1
424            CONTINUE
               CI1=CI1+INCMCC
425         CONTINUE
*
*           Partie triangulaire superieure de R
*
            IF (ITYSYM.NE.INDSYM('SYMETRIQUE')) THEN
               BJ1=MCTRMB
               DO 429 J=1,NBLIMB
*                 Calcul des coefficients R_{ij}  i = 1, j-1
*                 Produit des lignes i de C par la ligne j de B
                  CIK=MCTRMC
                  DO 428 I=1,MIN(J-1,NBLIMC)
                     CSOM=0.
                     DO 427 BJK=BJ1,BJ1-1+NBCOMB*INCMBL,INCMBL
                        CSOM=CSOM+CST(CIK)*RST(BJK)
                        CIK=CIK+INCMCL
427                  CONTINUE
                     CST(RIJ)=CSOM
                     RIJ=RIJ+1 
428               CONTINUE
                  BJ1=BJ1+INCMBC
429            CONTINUE
            ENDIF
         ENDIF
*
      ELSE
*        Matrice Resultat reelle ==>
*        Matrice B reelle et matrice spectrale reelle
*
*           Partie diagonale de R
*
            BJ1=MCTRMB
            CIK=MCTRMC
            DO 222 I=1,MIN (NBLIMC,NBLIMB)
               RSOM=0.
*              Calcul du coefficient R_{ii} 
*              Produit de la ligne i de C par la ligne i de B
               DO 221 BJK=BJ1,BJ1-1+NBCOMB*INCMBL,INCMBL
                  RSOM=RSOM+RST(CIK)*RST(BJK)
                  CIK=CIK+INCMCL
221            CONTINUE
               BJ1=BJ1+INCMBC
               RST(RIJ)=RSOM
               RIJ=RIJ+1
222         CONTINUE
*
*           Partie triangulaire inferieure de R
*
            CI1=MCTRMC
            DO 225 I=1,NBLIMC
               BJ1=MCTRMB
*              Calcul des coefficients R_{ij}  j = 1, i-1
*              Produit de la ligne i de C par les lignes j de B
               DO 224 J=1,MIN(I-1,NBLIMB)
                  RSOM=0.
*                 Premier coeff. de la ligne i de C
                  CIK=CI1
                  DO 223 BJK=BJ1,BJ1-1+NBCOMB*INCMBL,INCMBL
                     RSOM=RSOM+RST(CIK)*RST(BJK)
                     CIK=CIK+INCMCL
223               CONTINUE
                  BJ1=BJ1+INCMBC
                  RST(RIJ)=RSOM
                  RIJ=RIJ+1
224            CONTINUE
               CI1=CI1+NBCOMB
225         CONTINUE
*
*           Partie triangulaire superieure de R
*
            IF (ITYSYM.NE.INDSYM('SYMETRIQUE')) THEN
               BJ1=MCTRMB
               DO 229 J=1,NBLIMB
*                 Calcul des coefficients R_{ij}  i = 1, j-1
*                 Produit des lignes i de C par la ligne j de B
                  CIK=MCTRMC
                  DO 228 I=1,MIN(J-1,NBLIMC)
                     RSOM=0.
                     DO 227 BJK=BJ1,BJ1-1+NBCOMB*INCMBL,INCMBL
                        RSOM=RSOM+RST(CIK)*RST(BJK)
                        CIK=CIK+INCMCL
227                  CONTINUE
                     RST(RIJ)=RSOM
                     RIJ=RIJ+1 
228               CONTINUE
                  BJ1=BJ1+INCMBC
229            CONTINUE
            ENDIF
*
      ENDIF
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 2 : D.Martin (20 octobre 2006)
*  - eclatement du produit des 3 matrices en Efl1rm et Eflbrm 
* Version 1 : C. Chambeyron (6 Juillet 2006)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !ElfBrm
