*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE EFLBRO (TYPEMA,NBLIMA,STOIMA,MCTRMA
     &                         ,NBLIMB,STOIMB,MCTRMB
     &                  ,TYPESP,NBCOMA,MCTRSP,ITYSYM,MCTERM,RST,CST)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (novembre 1999)
* Derniere modification : D.Martin (12 octobre 2006)
* Version 2
*
* Calcul par 'produit tensoriel' d'un terme element finis localise BiMORSE
* pour un terme spectral 'diagonal'
*                       R_{ij} = A_{ik} * V_k * B*_{kj}
*
*-- Arguments d'entree --
* TYPEMA type (REEL ou COMPLEXE) des matrice A et B
* NBLIMA nombre de ligne du resultat et de la matrice A
* STOIMA type de stockage de la premiere matrice A (PLEIN-L ou PLEIN-C)
* MCTRMA adresse du terme en ligne, matrice A
* NBLIMB nombre de colonne du resultat et de ligne de la matrice B
* STOIMB type de stockage de la seconde matrice B (PLEIN-C ou PLEIN-L)
* MCTRMB adresse du terme en colonne, matrice B
* TYPESP type (REEL ou COMPLEXE) du vecteur V
* NBCOMA nombre de coefficients de V
* MCTRSP adresse du vecteur (matrice diagonale) spectral S
* ITYSYM indicateur de symetrie du terme resultat
* MCTERM adresse du terme resultat stocke BIMORSE
* RST,CST super tableau, version REELLE et COMPLEXE
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPEMA,STOIMA,STOIMB,TYPESP
      INTEGER NBLIMA,MCTRMA,NBLIMB,MCTRMB,NBCOMA,MCTRSP,MCTERM,ITYSYM
      REAL    RST(*)
      COMPLEX CST(*)
*
      INTEGER INDSYM
      INTEGER INCMAL,INCMAC,INCMBL,INCMBC,I,J,AI1,AIK,SK,B1K,BJK,RIJ
      REAL    RSOM
      COMPLEX CSOM
*------------------------------------------------------------------------
*
*     Increments de changement de ligne ou de colonne selon le stockage
*
*     INCMAL : increment d'adresse pour le changement de ligne dans A
*     INCMAC : increment d'adresse pour le changement de colonne dans A
      IF(STOIMA(1:7).EQ.'PLEIN-C') THEN
         INCMAC=NBLIMA  
         INCMAL=1        
      ELSEIF (STOIMA(1:7).EQ.'PLEIN-L') THEN
         INCMAC=1
         INCMAL=NBCOMA   
      ENDIF
*     INCMBL : increment d'adresse pour le changement de ligne dans B
*     INCMBC : increment d'adresse pour le changement de colonne dans B
      IF(STOIMB(1:7).EQ.'PLEIN-C') THEN
         INCMBL=NBLIMB
         INCMBC=1          
      ELSEIF (STOIMB(1:7).EQ.'PLEIN-L') THEN
         INCMBL=1
         INCMBC=NBCOMA    
      ENDIF
*
      RIJ=MCTERM
      AI1=MCTRMA
      B1K=MCTRMB
*
      IF (TYPEMA(1:1).EQ.'C') THEN
*
* (I)    Les matrices EFL sont complexes
*
*        Calcul des coefficients diagonaux
*
         DO 441 I=1,MIN (NBLIMA,NBLIMB)
            AIK=AI1
            BJK=B1K
            CSOM=0.
*
*           R_ii = A_ik * S_k * B_jk
            IF (TYPESP(1:1).EQ.'C') THEN
*
* (I-1)        Terme spectral complexe
*
               DO 440 SK=MCTRSP,MCTRSP-1+NBCOMA
                  CSOM=CSOM+CST(AIK)*CST(SK)*CONJG(CST(BJK))
                  AIK=AIK+INCMAC
                  BJK=BJK+INCMBL
440            CONTINUE
            ELSE
*
* (I-2)        Terme spectral reel
*
               DO 420 SK=MCTRSP,MCTRSP-1+NBCOMA
                  CSOM=CSOM+CST(AIK)*RST(SK)*CONJG(CST(BJK))
                  AIK=AIK+INCMAC
                  BJK=BJK+INCMBL
420            CONTINUE
            ENDIF
            CST(RIJ)=CSOM
            RIJ=RIJ+1
            AI1=AI1+INCMAL
            B1K=B1K+INCMBC
441      CONTINUE
*
*        Calcul des coefficients de la partie triangulaire inferieure
*
         AI1=MCTRMA
         DO 444 I=1,NBLIMA
            B1K=MCTRMB
            DO 443 J=1,MIN(I-1,NBLIMB)
               AIK=AI1
               BJK=B1K
               CSOM=0.
               IF (TYPESP(1:1).EQ.'C') THEN
                  DO 442 SK=MCTRSP,MCTRSP-1+NBCOMA
                     CSOM=CSOM+CST(AIK)*CST(SK)*CONJG(CST(BJK))
                     AIK=AIK+INCMAC
                     BJK=BJK+INCMBL
442               CONTINUE
               ELSE
                  DO 422 SK=MCTRSP,MCTRSP-1+NBCOMA
                     CSOM=CSOM+CST(AIK)*RST(SK)*CONJG(CST(BJK))
                     AIK=AIK+INCMAC
                     BJK=BJK+INCMBL
422               CONTINUE
               ENDIF
               CST(RIJ)=CSOM
               RIJ=RIJ+1
               B1K=B1K+INCMBC
443         CONTINUE
            AI1=AI1+INCMAL
444      CONTINUE
*
*        Calcul des coefficients de la partie triangulaire superieure
*
         IF (ITYSYM.NE.INDSYM('SYMETRIQUE')) THEN
            B1K=MCTRMB
            DO 447 J=1,NBLIMB
               AI1=MCTRMA
               DO 446 I=1,MIN(J-1,NBLIMA)
                  AIK=AI1
                  BJK=B1K
                  CSOM=0.
                  IF (TYPESP(1:1).EQ.'C') THEN
                     DO 445 SK=MCTRSP,MCTRSP-1+NBCOMA
                        CSOM=CSOM+CST(AIK)*CST(SK)*CONJG(CST(BJK))
                        AIK=AIK+INCMAC
                        BJK=BJK+INCMBL
445                  CONTINUE
                  ELSE
                     DO 425 SK=MCTRSP,MCTRSP-1+NBCOMA
                        CSOM=CSOM+CST(AIK)*RST(SK)*CONJG(CST(BJK))
                        AIK=AIK+INCMAC
                        BJK=BJK+INCMBL
425                  CONTINUE
                  ENDIF
                  CST(RIJ)=CSOM
                  RIJ=RIJ+1
                  AI1=AI1+INCMAL
446            CONTINUE
               B1K=B1K+INCMBC
447         CONTINUE
         ENDIF
*
      ELSE
*
* (II)   Les matrices EFL sont reelles
*
         IF (TYPESP(1:1).EQ.'C') THEN
* (II-1)    Terme spectral complexe
            DO 241 I=1,MIN (NBLIMA,NBLIMB)
               AIK=AI1
               BJK=B1K
               CSOM=0.
               DO 240 SK=MCTRSP,MCTRSP-1+NBCOMA
                  CSOM=CSOM+RST(AIK)*CST(SK)*RST(BJK)
                  AIK=AIK+INCMAC
                  BJK=BJK+INCMBL
240             CONTINUE
               CST(RIJ)=CSOM
               RIJ=RIJ+1
               AI1=AI1+INCMAL
               B1K=B1K+INCMBC
241         CONTINUE
*
            AI1=MCTRMA
            DO 244 I=1,NBLIMA
               B1K=MCTRMB
               DO 243 J=1,MIN(I-1,NBLIMB)
                  AIK=AI1
                  BJK=B1K
                  CSOM=0.
                  DO 242 SK=MCTRSP,MCTRSP-1+NBCOMA
                     CSOM=CSOM+RST(AIK)*CST(SK)*RST(BJK)
                     AIK=AIK+INCMAC
                     BJK=BJK+INCMBL
242               CONTINUE
                  CST(RIJ)=CSOM
                  RIJ=RIJ+1
                  B1K=B1K+INCMBC
243            CONTINUE
              AI1=AI1+INCMAL
244         CONTINUE
*
            IF (ITYSYM.NE.INDSYM('SYMETRIQUE')) THEN
            B1K=MCTRMB
            DO 247 J=1,NBLIMB
               AI1=MCTRMA
               DO 246 I=1,MIN(J-1,NBLIMA)
                  AIK=AI1
                  BJK=B1K
                  CSOM=0.
                  DO 245 SK=MCTRSP,MCTRSP-1+NBCOMA
                     CSOM=CSOM+RST(AIK)*CST(SK)*RST(BJK)
                     AIK=AIK+INCMAC
                     BJK=BJK+INCMBL
245                CONTINUE
                  CST(RIJ)=CSOM
                  RIJ=RIJ+1
                  AI1=AI1+INCMAL
246            CONTINUE
               B1K=B1K+INCMBC
247         CONTINUE
            ENDIF
*
        ELSE
* (II-2)    Terme spectral reel
            DO 221 I=1,MIN (NBLIMA,NBLIMB)
               AIK=AI1
               BJK=B1K
               RSOM=0.
               DO 220 SK=MCTRSP,MCTRSP-1+NBCOMA
                  RSOM=RSOM+RST(AIK)*RST(SK)*RST(BJK)
                  AIK=AIK+INCMAC
                  BJK=BJK+INCMBL
220             CONTINUE
               RST(RIJ)=RSOM
               RIJ=RIJ+1
               AI1=AI1+INCMAL
               B1K=B1K+INCMBC
221        CONTINUE
*
            AI1=MCTRMA
            DO 224 I=1,NBLIMA
               B1K=MCTRMB
               DO 223 J=1,MIN(I-1,NBLIMB)
                  AIK=AI1
                  BJK=B1K
                  RSOM=0.
                  DO 222 SK=MCTRSP,MCTRSP-1+NBCOMA
                     RSOM=RSOM+RST(AIK)*RST(SK)*RST(BJK)
                     AIK=AIK+INCMAC
                     BJK=BJK+INCMBL
222               CONTINUE
                  RST(RIJ)=RSOM
                  RIJ=RIJ+1
                  B1K=B1K+INCMBC
223            CONTINUE
              AI1=AI1+INCMAL
224         CONTINUE
*
            IF (ITYSYM.NE.INDSYM('SYMETRIQUE')) THEN
               B1K=MCTRMB
               DO 227 J=1,NBLIMB
                  AI1=MCTRMA
                  DO 226 I=1,MIN(J-1,NBLIMA)
                     AIK=AI1
                     BJK=B1K
                     RSOM=0.
                     DO 225 SK=MCTRSP,MCTRSP-1+NBCOMA
                        RSOM=RSOM+RST(AIK)*RST(SK)*RST(BJK)
                        AIK=AIK+INCMAC
                        BJK=BJK+INCMBL
225                  CONTINUE
                     RST(RIJ)=RSOM
                     RIJ=RIJ+1
                     AI1=AI1+INCMAL
226               CONTINUE
                  B1K=B1K+INCMBC
227            CONTINUE
            ENDIF
*
        ENDIF
*
      ENDIF
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 2 : D.Martin (12 octobre 2006)
*  - ajout de l'argument ITYSYM pour prise en compte de la symetrie
* Version 1.3 : D.Martin (6 fevrier 2004)
* Version 1 : D.Martin (novembre 1999)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !ElfBro
