*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE TRPLOC (NDMDOM,NDMDOE,NBELDO,ITYPTR,MCTERM,MCDGAM
     &                  ,MCDNU,MCDALF,MCDINT,MCPLOC,NBFDEV,NBTFON
     &                  ,IGEOME,MCASSC,NBLIGN,INTRPL,IDLHRL,NCPINL
     &                  ,BLOKEL,LISTEL,CORPTL,NBPTEL,IRGNDO,NGNEDO
     &                  ,TABCOF,NUNEFA,NUDLFA,NEDUDL,IRGDLN,NDLINL
     &                  ,RST,CST,NIVIMP,IMPFCH)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs: M.Lenoir, B.Vigan, C.Politis, M.Rouquette (Octobre 1990)
* Derniere modification D.Martin (8 Avril 2003)
* Version 1.0.1
*
* Construction d'un tableau elements finis localises preassembles
*
*-- Arguments d'entree --
* NDMDOM dimension d'espace du domaine
* NBELDO nombre d'elements
* ITYPTR type de declaration du terme (reel ou complexe)
* MCTERM adresse du terme preassemble dans le tableau du bon type
* MCDGAM adresse complexe des coef. des termes des fonctions propres e.f.l.
* MCDNU  adresse complexe des exp. des termes des fonctions propres e.f.l.
* MCDALF adresse complexe des exp. sec. des termes des fonctions propres e.f.l.
* MCDINT adresse des integrales exactes
* MCPLOC adresse du terme elementaire dans le tableau du bon type
* NBFDEV nombre de fonctions propres e.f.l.
* NBTFON nombre de termes composants chaque fonction propre
* IGEOME indice de geometrie de la frontiere
* MCASSC adresse complexe du tableau associe pour les constantes
* NBLIGN nombre de d.l. sur le domaine
* INTRPL numero de type de l'interpolation de l'inconnue element finis
* IDLHRL indice de presence de degre de liberte de hermite
* NCPINL nombre de composantes de l'inconnue element finis
* BLOKEL tableau des types d'elements
* LISTEL liste elements-faces du domaine
* CORPTL tableau des points par elements du domaine
* NBPTEL nombre de noeuds dans l'element
* IRGNDO tableau contenant pour chaque element le numero global des noeuds
* NGNEDO tableau contenant la numerotation globale du domaine
* TABCOF tableau des coefficients de chaque monomes des fonctions de base
* NUNEFA numerotation locale des noeuds de la face
* NUDLFA numerotation locale des d.l. de la face
* NEDUDL pointeur vers le noeud du d.l.
* IRGDLN tableau contenant pour chaque element le numero global des d.l.
* NDLINL tableau contenant pour chaque element le nombre de d.l. par noeud
* RST,CST Super tableau, version reelle ou complexe
* NIVIMP niveau d'impression
* IMPFCH unite logique d'impression
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      INTEGER NDMDOM,NDMDOE,NBELDO,ITYPTR,MCTERM,MCDGAM,MCDNU
     &       ,MCDALF,MCDINT,MCPLOC,NBFDEV,NBTFON,IGEOME
     &       ,MCASSC,NBLIGN,INTRPL,IDLHRL,NCPINL,NIVIMP,IMPFCH
      INTEGER BLOKEL(*),LISTEL(*),NUDLFA(*),NUNEFA(*),NDLINL(*)
      INTEGER IRGNDO(*),NGNEDO(*),NBPTEL(*),NEDUDL(*),IRGDLN(*)
      REAL    CORPTL(*),TABCOF(*),RST(*)
      COMPLEX CST(*)
*
      INCLUDE 'CONTEX'
*
      CHARACTER AST,TYPINT*4
      INTEGER IST,IARGND,NUTYAV,NUTYFV,NUFAAV,LISRAN,INCCOR,IEL
     &       ,NUTGEO,NUEL,NUTYPE,LAGHER,KCL,NBNEEL,NBDLEL,NUF,NUTYFA
     &       ,NBNEFA,NBDLFA,NDEGRE
      REAL    THEMIN,THEMAX
*---------------------------------------------------------------------
      CALL PRFXMJ (1,'*Trploc*')
*
      IARGND=1
      NUTYAV=0
      NUTYFV=0
      NUFAAV=0
      LISRAN=0
      INCCOR=1
*
*     Boucle sur tous les elements du domaine
*
      DO 1 IEL=1,NBELDO
*       Iinformations sur les elements finis
         CALL EXSTEL (NDMDOM,NDMDOE,LISRAN,LISTEL,BLOKEL,NUTGEO,INTRPL
     &               ,NUEL,NUTYAV,NUTYPE,LAGHER,KCL,NBNEEL,NBDLEL
     &               ,NUF,NUFAAV,NUTYFA,NBNEFA,NBDLFA,NUNEFA,NUDLFA
     &               ,0,IMPFCH)                                         !sdexplo
*
*        Tableau de numerotation des d.l. (interpolation Hermite)
         IF (IDLHRL.GT.0) CALL ELEMDL (NUTYFA,NBNEFA,NBDLFA,NEDUDL
     &                                ,IRGDLN,NDLINL)                   !ef3d
*
*        Tableau des coefficients des fonctions de bases
*
         IF (NUTYFA.NE.NUTYFV) CALL ELEMCF (NUTYFA,NBDLFA,TABCOF)       !ef3d
         NUTYAV=NUTGEO
         NUTYFV=NUTYFA
         NUFAAV=NUF
*
*        Recherche des extremites de l'element
         CALL EFLGBO (NDIM,CORPTL(INCCOR),IGEOME,MCASSC,CST
     &               ,THEMIN,THEMAX)                                    !efloca
         INCCOR=INCCOR+NBPTEL(INCCOR)*NDIM+2
*
*        Calculs elementaires
*
         NDEGRE=INTRPL
         CALL TAZERO (NBDLFA*NBFDEV,TYPINT(ITYPTR),MCPLOC
     &               ,AST,IST,RST,CST)                                  !utilite
*
         CALL ELPLOC (ITYPTR,MCDGAM,MCDNU,MCDALF,MCDINT,MCPLOC,NBFDEV
     &               ,NBTFON,THEMIN,THEMAX,TABCOF,NDEGRE,NBDLFA,RST,CST)!efloca
*
*        Preassemblage
*
         CALL ASPLOC (IDLHRL,ITYPTR,MCTERM,MCPLOC,NBNEFA,NBDLFA
     &               ,NBLIGN,NBFDEV,NUNEFA,NDLINL,IRGNDO(IARGND+1)
     &               ,NGNEDO,NCPINL,RST,CST)                            !efloca
         IARGND=IARGND+IRGNDO(IARGND)+1
*
1     CONTINUE

      CALL PRFXMJ (-1,'*Trploc*')
                                                                    END !Trploc
