*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE GLNUNE (NDIM,INTERP,NBTEL,NBBBLK,NU1BLK,BLOKEL
     &                  ,NBSOMX,NUGSOL,NBAREX,NUGARL,NBFACX,NUGFAL
     &                  ,NBNEEX,NEAREX,NEFACX,NUNEFA,NUDLFA
     &                  ,NGNESO,NGNEAR,NGNEFA,NBTNEU,NGNEEL
     &                  ,NBPTEX,NUPTFA,CORPTL,CORLOC,CORPTF,CORNEF
     &                  ,FCTBAS,EPSMAC,NIVIMP,IMPFCH)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin & O.deBayser (Decembre 1988) 
* Refonte complete (D.Martin) Octobre 1995 
* Derniere modification D.Martin (29 juin 2002)
* Version 1.2.4
*
* Creation d'une numerotation des noeuds du maillage pour une interpolation
*
*-- Arguments d'entree -- 
* NDIM   dimension d'espace
* INTERP numero de type de l'interpolation en cours
* NBTEL  nombre total d'elements du maillage  
* NBBBLK nombre de blocs d'elements consecutifs de meme type  
* NU1BLK numero de type des elements du premier bloc  
* BLOKEL description des elements du maillage par blocs
* NBSOMX nombre max. de sommets dans un element du maillage
* NUGSOL tableau de numerotation globale des sommets des elements du maillage
* NBAREX nombre max. d'aretes dans un element du maillage  (2D ou 3D)
* NUGARL tableau de numerotation globale des aretes des elements du maillage
* NBFACX nombre max. de faces dans un element du maillage (3D)
* NUGFAL tableau de la numerotation des faces des elements du maillage  
* NBNEEX nombre max. de noeuds dans un element du maillage pour l'interpolation
* NEAREX nombre maximum de noeuds sur une arete du maillage (2D ou 3D)
* NEFACX nombre maximum de noeuds sur une face du maillage (3D)
* NIVIMP niveau d'impression  
* IMPFCH numero d'unite logique d'impression  
*
*-- Arguments de sortie --
* NBTNEU nombre de noeuds pour l'interpolation en cours
* NGNEEL numerotation globale (par element) des noeuds pour l'interpolation
*
*-- Tableaux de travail --
* NUNEFA numerotation locale des noeuds d'une face ou arete
* NUDLFA idem pour les d.l. (inutile ici mais il est necessaire de declarer
*         ce tableau, c'est un argument du sous-programme ELEMFA)
* NGNESO numerotation globale des noeuds sommets pour l'interpolation en cours
* NGNEAR numerotation globale des noeuds des aretes ... (2D ou 3D)
* NGNEFA numerotation globale des noeuds des faces  ... (3D)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      INTEGER  NDIM,INTERP,NBTEL,NBBBLK,NU1BLK,BLOKEL(*)
     &        ,NBNEEX,NEAREX,NEFACX
     &        ,NBSOMX,NUGSOL(NBSOMX,*),NGNESO(*)
     &        ,NBAREX,NUGARL(NBAREX,*),NGNEAR(NEAREX,*)
     &        ,NBFACX,NUGFAL(NBFACX,*),NGNEFA(NEFACX+2,*)
     &        ,NUNEFA(*),NUDLFA(*)
     &        ,NBTNEU,NGNEEL(NBNEEX,*),NIVIMP,IMPFCH
      INTEGER  NBPTEX,NUPTFA(*)
      REAL     CORPTL(*),CORLOC(*),CORPTF(*),CORNEF(*)
     &        ,FCTBAS(*),EPSMAC
*
      CHARACTER PRFXAS*8
      INTEGER  L,NBELBK,NUTGEO,LGEBLK,PTRBLK,NUBBLK,NUELBK,NUTYLE
     &        ,LAGHER,KCLASS,NBSOMT,NBARET,NBFACE,NBNEEL,NBDLEL,NBNEFW
     &        ,NBDLFW,NBNEAW,NBDLAW,NELOCA,NESOME,NEARET,NEFACE,NEINTE,I
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*GlnuNe*')
      IF (IMPFCH.GT.0.AND.NIVIMP.GT.3)
     &   WRITE (IMPFCH,10000) PRFXAS(1)//'>*GlnuNe*',INTERP
*
      NBTNEU=0 
      L=0  
*
      NBELBK=NBTEL 
      NUTGEO=NU1BLK
      IF (NBBBLK.GT.1) THEN 
         LGEBLK = BLOKEL(1)
         NBELBK = BLOKEL(LGEBLK+1)  
         PTRBLK = LGEBLK+1+BLOKEL(3) 
      ENDIF
      DO 10 NUBBLK=1,NBBBLK
*        Calcul du numero de type des elements du bloc 
         NUTYLE=INTERP+100*(NUTGEO/100)
*
         CALL ELEMNB (NUTYLE,LAGHER,KCLASS,NBSOMT,NBARET,NBFACE
     &               ,NBNEEL,NBDLEL,NBNEAW,NBDLAW,NBNEFW,NBDLFW)        !ef3d
         CALL ELEMNE (NUTYLE,NESOME,NEARET,NEFACE,NEINTE)               !ef3d
         DO 9 NUELBK=1,NBELBK
* 
            L=L+1
*
*           Numerotation globale des noeuds sommets de l'element
*
            IF (NESOME.GT.0)
     &         CALL GLNUNS (L,NBSOMT,NBSOMX,NUGSOL,NGNESO
     &                     ,NBNEEX,NGNEEL,NBTNEU)                       !glonum
*
*           Numerotation globale des noeuds non-sommets des aretes
*
            IF (NEARET.GT.0) 
     &         CALL GLNUNA (L,NUTYLE,NBARET,NBAREX,NUGARL,NEAREX
     &                     ,NGNEAR,NUDLFA,NUDLFA,NBNEEX,NGNEEL,NBTNEU)  !glonum
*
*           Numerotation globale des noeuds des faces non sur des aretes 
*
            IF (NEFACE.GT.0) 
     &         CALL GLNUNF (L,NUTYLE,NBFACE,NBFACX,NUGFAL,NEFACX
     &                     ,NGNEFA,NUDLFA,NUDLFA,NBNEEX,NGNEEL,NBTNEU
     &                     ,NUTGEO,NDIM,NBPTEX,NUPTFA,CORPTL,CORLOC
     &                     ,CORPTF,CORNEF,FCTBAS,EPSMAC)
*  
*           Numerotation des noeuds interieurs des elements
*
            IF (NEINTE.GT.0) THEN
               DO 1 NELOCA=1,NBNEEL
                  IF (NGNEEL(NELOCA,L).LE.0) THEN
                     NBTNEU           = NBTNEU+1  
                     NGNEEL(NELOCA,L) = NBTNEU
                   ENDIF
1              CONTINUE
            ENDIF
*  
            IF (IMPFCH.GT.0.AND.NIVIMP.GT.3)
     &         WRITE (IMPFCH,10001) L,NESOME,NEARET,NEFACE,NEINTE
            IF (IMPFCH.GT.0.AND.NIVIMP.GT.10)
     &         WRITE (IMPFCH,10002) (NGNEEL(NELOCA,L),NELOCA=1,NBNEEL)
9        CONTINUE  
         IF (NBBBLK.GT.1) THEN  
            NBELBK = BLOKEL(PTRBLK)
            NUTGEO = BLOKEL(PTRBLK+1)  
            PTRBLK = PTRBLK+BLOKEL(3)
         ENDIF 
10    CONTINUE
      CALL PRFXMJ (-1,'*GlnuNe*') 
*----------------------------------------------------------------------- 
10000 FORMAT(/T2,A,' Numerotation globale des noeuds pour '
     &,'l''interpolation #',I3)
10001 FORMAT('   Element  ',I7,', NeSomt=',I2,', NeAret=',I2
     &,', NeFace=',I2,', NeInte=',I2)
10002 FORMAT((T13,10(I7,'  ')))
                                                                    END !GlnuNe

