*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE NUMNEU
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs: D.Martin & O.deBayser (Decembre 1988)
* Refonte complete : D.Martin (Octobre 1995)
* Derniere modification : D.Martin (05 juillet 2005)
* Version 2
*
* 1) Creation si necessaire des numerotations globales 
*    des sommets (tableau #GNEEL de niveau 1), 
*    des aretes (#GAREL de niveau 1), 
*    des faces (#GFAEL de niveau 1) 
* 2) Creation des numerotations globales des noeuds  
*   (tableaux #GNEEL de niveaux INTERP) pour toutes les interpolations INTERP 
*   non isoparametriques des inconnues 'Valeurs Nodales' de l'application
* 3) Creation des tableaux globaux des coordonnees des noeuds
*   (tableaux #ORNOE de niveaux INTERP) pour toutes les interpolations 
*   des inconnues de type 'Valeurs Nodales'.
*
* On commence par la creation de la numerotation globale des sommets,
* des aretes et des faces de tous les elements du maillage
* a partir de la numerotation globale des points du maillage
*
* Pour les interpolations isoparametriques on utilise evidement la
* numerotation globale des points du maillage
*
*-- Procedures utilisees --
* NEELEM determination de la position des noeuds (sommets, sur aretes, ...)
* MAXNBL utilitaire de calcul des dimensions de tableaux temporaires
* GLNSAF numerotation globale des sommets, aretes, faces du maillage
*   GLNUSO numerotation globale des sommets des elements du maillage
*   GLNUFA numerotation globale des aretes ou faces des elements du maillage
* GLNUNE numerotation globale des noeuds pour une interpolation non isoparam.
*   GLNUNS numerotation globale des noeuds sommets d'elements
*   GLNUNA numerotation globale des noeuds des noeuds non sommets des aretes
*   GLNUNF numerotation globale des noeuds des faces non situes sur les aretes
* GLCNIS tableau global des coordonnees des noeuds du maillage
*        pour les interpolations (des inconnues) non isoparametriques
* GLCISO tableau global des coordonnees des points du maillage
*
*-- Structures de donnees utilisees ou modifiees --
* #GNEEL de niveau NIVENG : numerotation globale des points du maillage
* #LOKEL : description des elements du maillage
* #NCONU : caracteristiques (remplissage du nombre de noeuds) des inconnues
* #ORPTL : coordonnees des points des elements du maillage
*-- Structures de donnees creees --
* #GNEEL,1 : numerotation globale des sommets du maillage (si NIVENG.NE.1)
* #GAREL,1 : numerotation globale des aretes du maillage
* #GFAEL,1 : numerotation globale des faces du maillage 
* #GNEEL de niveau INTERP: numerotation globale des noeuds pour l'interpolation
*            de numero INTERP, si INTERP.NE.NIVENG
* #ORNOE de niveau INTERP: tableaux de coordonnees de noeuds en numerotation
*            globale pour toutes les interpolations
*
*-- Tableaux temporaires --
* $WORK$,0 : tableaux de numerotations locales sur les faces ou aretes
* $WORK$,4 : travail de travail pour reperer les aretes / faces
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      INCLUDE 'ALLOC'  
      INCLUDE 'CONTEX' 
*
      INTEGER MCINCO,LGEINC,NBINCO,INCINC,NCHINC,INCONU,NCPINC,ADMINC
     &       ,INTERP,NIVIMP,NIVIMR,LGWOR0,LGSOEL,LGAREL,LGFAEL,LGWOR4
     &       ,ADCLOC,ADGNSO,ADGNAR,ADGNFA,ADNUFA,ADDLFA,ADCOEL,ADFCTB
     &       ,NESOMX,NEAREX,NEFACX,NEINTX,NESOME,NEARET,NEFACE,NEINTE
     &       ,MCNUFA,MCBLOK,MCMINC,MCCORG,LGNEEG,MCNEEL,MCCORI,MCNENG
     &       ,LAGHER,NBSOMX,NBAREX,NBFACX,NBNEEX,NBDLEX,NBNEAX,NBSOAX
     &       ,NBNEFX,NBSOFX,NBDLAX,NBDLFX,NBINCV,LGNEEL,NBTNEU,NBTNE1
     &       ,MC,L0WORK,MCWOR0,MCSOEL,MCAREL,MCFAEL,MCWOR4,NCPIAV,INTPAV
     &       ,INCV,I,ICRENG,LGDEGX,MXDEG
     &       ,MCWOR5,MCNEEW,MCVOIS,LBANDI,LBANDO,LPROFI,LPROFO
     &       ,NCAINC,ADPTFA,ADCOPF
      CHARACTER ERCODE*120,PRFXAS*8,RENUMM*1,THEINC*20
      COMMON/FORMAH/ERCODE
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*NumNeu*')
      NIVIMP = 0
      NIVIMR = 0
      RENUMM ='Y'
      IF (IMPSDR.GT.0.AND.NIVIMP.GT.0) WRITE (IMPSDR,10000) PRFXAS (0)
*     Tableau de travail (tableau NUNEFA, NUDLFA)
      LGWOR0 = 2000*NDIM
      CALL TBCREE ('$WORK$',0,1,LGWOR0,'c')
      CALL TBAR1  (ERCODE,'$WORK$',0,MCNUFA)                     
      MCBLOK = 1
      IF (NBBBLK.GT.1) CALL TBAR1  (ERCODE,'#LOKEL',1,MCBLOK)
      CALL TBAR1  (ERCODE,'#NCONU',1,MCINCO)                     
      CALL SDEXDB (IST(MCINCO),LGEINC,NBINCO,INCINC,NCHINC)             !utilite
*
*     Nombre max de sommets par element, d'aretes, de faces,
*            de sommets par arete, de sommets par face
*            pour tous les elements du maillage
      CALL MAXNBL (1,NBBBLK,NU1BLK,IST(MCBLOK),LAGHER,NBSOMX,NBAREX
     &            ,NBFACX,NBNEEX,NBDLEX,NBNEAX,NBSOAX,NBNEFX,NBSOFX)    !sdexplo
*
*     Determination de la position des noeuds pour toutes les interpolations
*     NESOMX nombre de noeuds sommets pour une des interpolations
*            > 0 ===> construction de la numerotation globale des sommets
*                si necessaire (i.e. si NIVENG.NE.1)
*     NEAREX nombre de noeuds non sommets sur les aretes ...
*            > 0 ===> construction de la numerotation globale des aretes
*     NEFACX nombre de noeuds sur les faces non sur les aretes ...
*            > 0 ===> construction de la numerotation globale des faces
*     NEINTX nombre de noeuds internes ...
      NESOMX = 0
      NEAREX = 0
      NEFACX = 0
      NEINTX = 0
      NBINCV = 0
      DO 10 INCONU=1,NBINCO
*
         CALL PRNINC (INCONU,' ',THEINC,NCAINC)                         !sdexplo
*
         CALL SDEXCO (INCONU,IST(MCINCO),NCPINC,INTERP,NBTNEU)          !utilite
         IF (INTERP.GE.0) THEN
            IF (IMPPAL.GT.0.AND.NBINCV.LE.0) WRITE (IMPPAL,*)
            IF (INTERP.NE.NIVENG) THEN
               IF (IMPPAL.GT.0)
     &            WRITE (IMPPAL,10010) PRFXAS(0),INTERP,THEINC(1:NCAINC)
               CALL NEELEM (INTERP,NBBBLK,NU1BLK,IST(MCBLOK),IST(MCNUFA)
     &                     ,IST(MCNUFA+100),NESOME,NEARET,NEFACE,NEINTE)!sdexplo
               NESOMX = MAX (NESOMX,NESOME)
               NEAREX = MAX (NEAREX,NEARET)
               NEFACX = MAX (NEFACX,NEFACE)
               NEINTX = MAX (NEINTX,NEINTE)
            ELSE
               IF (IMPPAL.GT.0) THEN
                  WRITE (IMPPAL,10011) PRFXAS(0),NIVENG,THEINC(1:NCAINC)
               ENDIF
            ENDIF
            NBINCV = NBINCV+1
         ENDIF
10    CONTINUE
      IF (IMPSDR.GT.0.AND.NIVIMP.GT.9)
     &   WRITE (IMPSDR,10005) PRFXAS(0),NESOMX,NEAREX,NEFACX,NEINTX
*
*     Longueur des tableaux des numerotations globales des
*     aretes (2D et 3D) et des faces (3D)
      NBTSOG = NBTPTG
      NBTARG = 0
      NBTFAG = 0
      LGSOEL = 0
      LGAREL = 0
      LGFAEL = 0
      LGWOR4 = 0
      IF (NIVENG.NE.1) LGSOEL = NBSOMX*NBTEL
      IF (NEAREX.GT.0) LGAREL = NBAREX*NBTEL
      IF (NEFACX.GT.0) LGFAEL = NBFACX*NBTEL
      CALL TBRR2  (ERCODE,'#ORPTL',0,MCCORG,'#GNEEL',NIVENG,LGNEEG)
*
*     Boucle sur les inconnues 'Valeurs Nodales' (INTERP > 0) pour retrouver
*     toutes les interpolations des inconnues declarees.
*
      ICRENG = 0
      ADMINC = 0
      DO 100 INCONU = 1,NBINCO
*
         CALL PRNINC (INCONU,' ',THEINC,NCAINC)                         !sdexplo
*        Recherche du numero de type d'interpolation
         CALL TBAR1  (ERCODE,'#NCONU',1,MCINCO)
         CALL SDEXCO (INCONU,IST(MCINCO),NCPINC,INTERP,NBTNEU)          !utilite
*
         IF (INTERP.GE.0) THEN
         NBTNE1 = 0
*
         IF (INTERP.NE.NIVENG) THEN
*
*           Cas d'une interpolation non isoparametrique :
*           Creation si necessaire des tableaux de numerotation globale
*           des sommets, aretes, faces des elements du maillage.
*
            IF (LGWOR4.LE.0) THEN
*
               CALL GLNSAF (NBSOMX,LGSOEL,NBSOAX,NBAREX,LGAREL
     &                     ,NBSOFX,NBFACX,LGFAEL,LGWOR4,NIVIMP,IMPSDR)  !glonum
               IF (LGSOEL.GT.0) THEN
                  NBTNE1=NBTSOG
                  NBTNEU=NBTSOG
               ENDIF
            ENDIF
*
*           Verification de l'existence de la numerotation globale
*           correspondant a l'interpolation en cours
*
            ERCODE(1:1) = '!'
            CALL TBRR1  (ERCODE,'#GNEEL',INTERP,MC)
*
            IF (ERCODE(1:1).EQ.'C') THEN
*
*              Creation proprement dite de la numerotation globale des
*              noeuds pour l'interpolation de l'inconnue en cours, a
*              l'aide du tableau des faces ou des aretes.
*
               CALL MAXNBL (INTERP,NBBBLK,NU1BLK,IST(MCBLOK),LAGHER
     &                     ,NBSOMX,NBAREX,NBFACX,NBNEEX,NBDLEX
     &                     ,NBNEAX,NBDLAX,NBNEFX,NBDLFX)                !sdexplo
               LGNEEL=NBNEEX*NBTEL
               CALL TBCREE ('#GNEEL',INTERP,1,LGNEEL,'c')
*
*              Tableau de travail $WORK$,0 pour les calculs locaux
*              Adresses relatives des tableaux NGNEFA,NUNEFA,NUDLFA pour
*              le calcul de la numerotation dans $WORK$,0 (cf. GLNUNE)
*
               ADGNSO = 0
               ADGNAR = ADGNSO+NBTSOG
               ADGNFA = ADGNAR+NEAREX*NBTARG
               ADNUFA = ADGNFA+(NEFACX+2)*NBTFAG
               ADDLFA = ADNUFA+NBNEFX
               L0WORK = ADDLFA+NBDLFX
               ADCLOC = L0WORK
               ADPTFA = L0WORK
               ADCOPF = L0WORK
               ADCOEL = L0WORK
               ADFCTB = L0WORK
               IF (NDIM.GE.3) THEN
*                 Adresses relatives des tableaux NUPTFA,CORLOC,CORNEF,CORPTF
*                ,FCTBAS pour le calcul des coordonnees de noeuds des faces
*                 servant a numeroter les faces communes a 2 elements en 3D
                  ADPTFA = L0WORK
                  ADCLOC = ADPTFA+NBNEFX
                  ADCOEL = ADCLOC+(NDIM-1)*NBNEFX
                  ADCOPF = ADCOEL+NDIM*NBNEFX
                  ADFCTB = ADCOPF+NDIM*NBPTFX
                  L0WORK = ADFCTB+NBPTFX
               ENDIF
               IF (LGWOR0.LT.L0WORK) THEN
                  CALL TBAJST ('$WORK$',0,L0WORK)
                  LGWOR0 = L0WORK
               ENDIF
               MCSOEL = 1
               MCAREL = 1
               MCFAEL = 1
               IF (NBBBLK.GT.1) CALL TBAR1  (ERCODE,'#LOKEL',1,MCBLOK)
               CALL TBAR2 (ERCODE,'$WORK$',0,MCWOR0
     &                           ,'#GNEEL',INTERP,MCNEEL)
               CALL TAZERO (L0WORK,'E',MCWOR0,AST,IST,RST,CST)          !utilite
               CALL TBAR1  (ERCODE,'#GNEEL',1,MCSOEL)
               IF (LGAREL.GT.0) CALL TBAR1  (ERCODE,'#GAREL',1,MCAREL)
               IF (LGFAEL.GT.0) CALL TBAR1  (ERCODE,'#GFAEL',1,MCFAEL)
               IF (NDIM.GE.3)   CALL TBAR1  (ERCODE,'#ORPTL',0,MCCORG)
               CALL GLNUNE (NDIM,INTERP,NBTEL,NBBBLK,NU1BLK,IST(MCBLOK)
     &                     ,NBSOMX,IST(MCSOEL),NBAREX,IST(MCAREL)
     &                     ,NBFACX,IST(MCFAEL)
     &                     ,NBNEEX,NEAREX,NEFACX,IST(MCWOR0+ADNUFA)
     &                     ,IST(MCWOR0+ADDLFA),IST(MCWOR0+ADGNSO)
     &                     ,IST(MCWOR0+ADGNAR),IST(MCWOR0+ADGNFA)
     &                     ,NBTNEU,IST(MCNEEL)
     &                     ,NBPTEX,IST(MCWOR0+ADPTFA),RST(MCCORG)
     &                     ,RST(MCWOR0+ADCLOC),RST(MCWOR0+ADCOPF)
     &                     ,RST(MCWOR0+ADCOEL),RST(MCWOR0+ADFCTB),EPSMAC
     &                     ,NIVIMP,IMPSDR)                              !glonum
               IF (IMPSDR.GT.0.AND.NIVIMP.GT.5) THEN
                  WRITE (IMPSDR,10003) PRFXAS(0)
     &                                ,INTERP,THEINC(1:NCAINC)
                  CALL PRNUGL (IST(MCBLOK),NBBBLK,NU1BLK,NBTEL,INTERP
     &                        ,NBNEEX,IST(MCNEEL),IMPSDR)               !prsd
               ENDIF
*
               IF (RENUMM.EQ.'Y'.AND.INTERP.GT.0.AND.NBTEL.GT.2) THEN
*
*                 Renumerotation pour 'optimisation' de la largeur de bande
*
                  CALL TBCREE ('$WORK$',5,1,5*NBTNEU+1+LGNEEL,'c')
                  CALL TBAR2  (ERCODE,'#GNEEL',INTERP,MCNEEL
     &                               ,'$WORK$',5,MCWOR5)
                  CALL RINFOR (NBTNEU,NBTEL,NBNEEX,IST(MCNEEL)
     &                        ,IST(MCWOR5),MXDEG,IST(MCWOR5+3*NBTNEU)
     &                        ,NIVIMR,IMPSDR)                           !glonum
                  LGDEGX=IST(MCWOR5+4*NBTNEU)
*
                  IF (MXDEG.LE.0)
     &               CALL BAISE  (' Degre maximum negatif ou nul')      !utilite
                  IF (IMPSDR.GT.0.AND.NIVIMR.GE.10)
     &               WRITE (IMPSDR,10021) PRFXAS(0),NBTNEU,NBTEL,NBNEEX
     &                                   ,LGDEGX/NBTNEU+1,MXDEG
*
                  CALL TBCREE ('$WORK$',6,1,LGDEGX,'c')
*
                  CALL TBAR3  (ERCODE,'#GNEEL',INTERP,MCNEEL
     &                        ,'$WORK$',5,MCWOR5,'$WORK$',6,MCVOIS)
*
                  MCNEEW=MCWOR5+5*NBTNEU+1
                  CALL RALGOR (NBTNEU,NBTEL,NBNEEX,IST(MCNEEL)
     &                       ,IST(MCWOR5),IST(MCWOR5+NBTNEU)
     &                       ,IST(MCWOR5+2*NBTNEU)
     &                       ,MXDEG,IST(MCWOR5+3*NBTNEU),IST(MCVOIS)
     &                       ,IST(MCWOR5+4*NBTNEU+1),IST(MCNEEW)
     &                       ,LBANDI,LBANDO,LPROFI,LPROFO,NIVIMR,IMPSDR)!glonum
*
                  CALL TBTUER ('$WORK$',6)
*
                  IF (LPROFI.GT.LPROFO) THEN
                     IF (IMPSDR.GT.0.AND.NIVIMP.GE.0)
     &               WRITE (IMPSDR,10022) PRFXAS(0),INTERP,NBTNEU
     &                                   ,LPROFI,LPROFO,LBANDI,LBANDO
                     CALL TBAR2  (ERCODE,'$WORK$',5,MCWOR5
     &                           ,'#GNEEL',INTERP,MCNEEL)
                     CALL TDANST (LGNEEL,'ENT',MCWOR5+5*NBTNEU+1
     &                           ,IST,RST,CST,'ENT',MCNEEL,IST,RST,CST) !utilite
                     IF (IMPSDR.GT.0.AND.NIVIMP.GT.5) THEN
                        WRITE (IMPSDR,10003) PRFXAS(0)//'>*PrNugl*'
     &                                      ,INTERP,THEINC(1:NCAINC)
                        CALL PRNUGL (IST(MCBLOK),NBBBLK,NU1BLK,NBTEL
     &                              ,INTERP,NBNEEX,IST(MCNEEL),IMPSDR)  !prsd
                     ENDIF
                     IF (IMPPAL.GT.0) WRITE (IMPPAL,10012) PRFXAS(0)
     &                                     ,INTERP,NBTNEU,LPROFO,LBANDO
                  ELSE
                     IF (IMPSDR.GT.0.AND.NIVIMP.GE.0)
     &                  WRITE (IMPSDR,10023) PRFXAS(0),INTERP,NBTNEU
     &                                     ,LPROFI,LPROFO,LBANDI,LBANDO
                     IF (IMPPAL.GT.0) WRITE (IMPPAL,10012) PRFXAS(0)
     &                                     ,INTERP,NBTNEU,LPROFI,LBANDI
                  ENDIF
                  CALL TBTUER ('$WORK$',5)
*
               ENDIF
*
            ELSEIF (NBTNE1.LE.0) THEN
*
*              Si l'interpolation a deja ete vue il faut chercher le nombre
*              de noeuds et le stocker pour l'inconnue en cours
*
               CALL TBAR1  (ERCODE,'#NCONU',1,MCINCO)
               DO 80 INCV = INCONU-1,1,-1
                  CALL SDEXCO (INCV,IST(MCINCO),NCPIAV,INTPAV,NBTNEU)   !utilite
                  IF (INTPAV.EQ.INTERP) THEN
                     CALL TBAR1  (ERCODE,'#OMINC',1,MCMINC)
                     WRITE (IMPSDR,10006) PRFXAS(0),INTERP
     &                  ,(AST(I),I=MCMINC+ADMINC,MCMINC+ADMINC+NCHINC-1)
                     GOTO 81
                  ENDIF
80             CONTINUE
               GOTO 99991
            ENDIF
*
81          CALL SDMKCO (INCONU,IST(MCINCO),NCPINC,INTERP,NBTNEU)       !utilite
*
            ERCODE(1:1) = '!'
            CALL TBRR1  (ERCODE,'#ORNOE',INTERP,MC)
            IF (ERCODE(1:1).EQ.'C') THEN
*
*              Creation du tableau de coordonnees des noeuds pour
*              l'interpolation de l'inconnue en cours
               CALL TBCREE ('#ORNOE',INTERP,2,NDIM*NBTNEU,'c')
*
*              Initialisation de la 1ere coordonnee de chaqque noeud     
               CALL TBAR1  (ERCODE,'#ORNOE',INTERP,MCCORI)
               DO 90 INCV=1,NBTNEU
                  RST(MCCORI)=REAMAX
                  MCCORI=MCCORI+NDIM
90             CONTINUE
*
*              Adresse relatives des tableaux CORLOC,COOREL,FCTBAS pour
*              le calcul des coordonnees (cf. GLCNIS) dans $WORK$,0
               ADCLOC = 0
               ADCOEL = ADCLOC+NDIM*NBNEEX
               ADFCTB = ADCOEL+NDIM*NBNEEX
               L0WORK = ADFCTB+NBPTEX
               IF (LGWOR0.LT.L0WORK) THEN
                  CALL TBAJST ('$WORK$',0,L0WORK)
                  LGWOR0 = L0WORK
               ENDIF
               IF (NBBBLK.GT.1) CALL TBAR1  (ERCODE,'#LOKEL',1,MCBLOK)
               CALL TBAR4  (ERCODE,'$WORK$',0,MCWOR0
     &                            ,'#GNEEL',INTERP,MCNEEL
     &                            ,'#ORPTL',0     ,MCCORG
     &                            ,'#ORNOE',INTERP,MCCORI)
               CALL GLCNIS (NDIM,INTERP,NBTEL,NBBBLK,NU1BLK
     &                     ,IST(MCBLOK),NBPTEX,NBNEEX,IST(MCNEEL)
     &                     ,RST(MCCORG),RST(MCWOR0+ADCLOC)
     &                     ,RST(MCWOR0+ADFCTB),RST(MCWOR0+ADCOEL)
     &                     ,RST(MCCORI),REAMAX,EPSMAC,NIVIMP,IMPSDR
     &                     ,IMPMES)                                     !glonum
               IF (IMPSDR.GT.0.AND.NIVIMP.GT.0) THEN
                  CALL TBAR1  (ERCODE,'#OMINC',1,MCMINC)
                  WRITE (IMPSDR,10004) PRFXAS(0)//'>*Prcoor*',INTERP
     &                  ,(AST(I),I=MCMINC+ADMINC,MCMINC+ADMINC+NCHINC-1)
                  CALL PRCOOR (NDIM,NBTNEU,RST(MCCORI),' Cornoe'
     &                        ,NIVIMP,IMPSDR)                           !prsd
               ENDIF
            ENDIF
            CALL TBSAVE ('#ORNOE',INTERP)
*
         ELSE
*
*           Si l'interpolation de l'inconnue est isoparametrique et
*           l'interpolation geometrique est non Hermite,
*           la numerotation des noeuds est celle du maillage:
*           il s'agit du tableau  '#GNEEL' de niveau NIVENG
*
            IF (ICRENG.LE.0) THEN
               ICRENG = ICRENG+1
*
*              Coordonnees des points (de l'interpolation isoparametrique)
*              creation du tableau #ORNOE de niveau INTERP = NIVENG
               CALL TBCREE ('#ORNOE',INTERP,2,NDIM*NBTPTG,'c')
               IF (NBBBLK.GT.1) CALL TBAR1  (ERCODE,'#LOKEL',1,MCBLOK)
               CALL TBAR3  (ERCODE,'#GNEEL',NIVENG,MCNENG
     &                            ,'#ORPTL',0     ,MCCORG
     &                            ,'#ORNOE',INTERP,MCCORI)
*
               CALL GLCISO (NDIM,NBTEL,NBBBLK,NU1BLK,IST(MCBLOK)
     &                     ,NBPTEX,IST(MCNENG),RST(MCCORG),RST(MCCORI)) !glonum
*
               IF (IMPSDR.GT.0.AND.NIVIMP.GT.0)
     &            CALL PRCOOR (NDIM,NBTPTG,RST(MCCORI),' Corpoi'
     &                        ,NIVIMP,IMPSDR)                           !prsd
               CALL TBSAVE ('#ORNOE',INTERP)
            ENDIF
         ENDIF
         ENDIF
99       ADMINC=ADMINC+NCHINC
100   CONTINUE
*
*     Destruction des tableaux temporaires
      CALL TBTUER ('$WORK$',0)
      IF (LGWOR4.GT.0) THEN
         CALL TBTUER ('$WORK$',4)
*
*       Sauvegarde de la numerotation globale des points, des aretes
*       des faces des elements du maillage et des coordonnees des points
         IF (LGSOEL.GT.0) CALL TBSAVE ('#GNEEL',1)
         IF (LGAREL.GT.0) CALL TBSAVE ('#GAREL',1)
         IF (LGFAEL.GT.0) CALL TBSAVE ('#GFAEL',1)
      ENDIF
      CALL TBSAVE ('#GNEEL',NIVENG)
*
      IF (IMPSDR.GT.0.AND.NIVIMP.GT.0) WRITE(IMPSDR,10002) PRFXAS(0)
      CALL PRFXMJ (-1,'*NumNeu*')
      RETURN
*-----------------------------------------------------------------------
10000 FORMAT(/,'* ',A8,' ',98('*'))
10002 FORMAT(' ',96('*'),'  ',A8,'  *')
10003 FORMAT(/,A,' Numerotation globale   pour l''interpolation ',I3
     &,', inconnue ',A)
10004 FORMAT(/,A,' Coordonnees des noeuds pour l''interpolation ',I3
     &,', inconnue ',50(A1))
10006 FORMAT(/,A8,' Interpolation ',I3,' deja rencontree'
     &,', inconnue ',50(A1))
10005 FORMAT(A8,' Nombre maximum de noeuds',T35,' sommets :',T60,I2
     &/T10,' d''un element',T35,' non sommets sur aretes :',T60,I2
     &/T35,' sur face non sur arete :',T60,I2
     &/T35,' internes :',T60,I2)
10010 FORMAT(A8,' Interpolation non-isoparametrique de type ',I3
     &,' pour l''inconnue ',A)
10011 FORMAT(A8,' Interpolation   isoparametrique   de type ',I3
     &,' pour l''inconnue ',A)
10012 FORMAT(A8,' Optimisation de la numerotation pour'
     &,' l''interpolation de type ',I3,/T11,'Nombre de noeuds',I6
     &,', profil inferieur',I9,',  largeur de bande',I6)
10021 FORMAT(A8,' Nombre de noeuds du maillage NBTNEU=',I8
     &       ,', Nombre d''elements NBTEL=',I5
     &       ,', Nombre max. de noeuds par elt. NBNEEX=',I8
     &/,T9,' Degre moyen LGDEGX/NBTNEU =',I8,', maximum MXDEG =',I8)
10022 FORMAT(/,A8,' Interpolation de numero ',I3
     &/T10,' Le nombre de coeff. du profil a ete reduit (',I8
     &,' d.l.) de ',I9,' a ',I9
     &/T10,' La largeur de bande (en noeuds) passe de ',I5,' a ',I5)
10023 FORMAT(/,A8,' Interpolation de numero ',I3
     &/T10,' La numerotation est deja optimale (',I8,' d.l.)'
     &/T10,' Nombre de coeff. du profil inferieur (en noeuds) :'
     &,I9,' > ',I9,/T10,' Largeur de bande (en noeuds) :',I5,' > ',I5)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
99991 CALL PRINCO (IMPPAL)
      CALL ENCLER (INTERP,ERCODE(1:3))
      CALL BAISE  (' Incoherence dans le tableau #NCONU.'
     &//' On ne retrouve pas l''interpolation de numero '//ERCODE(1:3))
                                                                    END !NumNeu       
