*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE GBEGIN (NOMINT,CHDIM,OUTPUT)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : M.Lenoir & F.Mahe (1992), D.Martin (mars.2002)
* Derniere modification : D.Martin (23.mars.2010)
* Version 2
*
* Ouverture des fichiers de dessin pour post-traitement a l'aide de
*  - medit ( OUTPUT = 'mesh&bb' )
*  - unighn, grame, matme ( OUTPUT = 'unighn' )
*  - paraview ( OUTPUT = 'paraview' )
*
*-- Arguments d'entree --
* NOMINT nom de la donnee chaine de caractere caracterisant le nom des 
*        fichiers resultats (nom interne du fichier).
*        Le nom du fichier (nom externe) est la valeur de cette donnee.
* CHDIM  chaine definissant la dimension du dessin 
*        '1D','1D5','2D','2D5','3D' -> format 'unighn'
*        '1D','2D','3D'             -> format 'medit'
* OUTPUT chaine de caracteres definisssant le format des fichiers graphiques
*        'unighn'  = 'grame'        -> format 'unighn'
*        'mesh&bb' = 'medit'        -> format mesh&bb de Medit
*        'paraview'                 -> format vtk datafile pour Paraview
*
*- Structure $NomDS
*==================
* Entete    : Vide
* Corps     : Nom, Dimension pour chaque dessin, Format de fichier
*
*- Structure $InfDS
* Entete    : longueur entete     , nb de dessins       , lg max dans $NomDS
*             (LGENDS=6           , NBDESS              , LGCHAR
*             nb de chaines/dessin, nb de caract./dessin, nb de caract./domaine
*              NBCARD=3           , INCNDS=2            , INCNDO=8)
*       puis, pour chaque dessin
* Corps     : nombre de domaines, nombre de resultats
*             (NBDOM            ,NBRES)
*       puis, pour chaque domaine
* Corps bis : NUDOMG,NUDOMK,NXLIST,NVNUME,NBELDO,NBNETR,INTRPC,NDMDOM
*             pour chaque domaine
*
*- FORMAT 'mesh&bb'
*==================
*----- Fichier de geometrie (.mesh)
* Entete :  MeshVersionFormatted 1 
*           Dimension $Dim
* Corps  :  Vertices $Nbpts
*           {liste des coordonnees des Nbpts noeuds du domaine + $Ref}
*           < Edges | Triangles | Quadrilaterals | Tetrahedra | Hexaedra > $Nbel
*           {liste des numeros des noeuds des Nbel elements du domaine + $Ref}
*           End
*----- Fichiers de resultats (.bb)
* Entete :  $Dim $NbMet $NbVal $Type
* Corps  : {liste des $NbVal valeurs par noeuds}
*
*- FORMAT 'unighn'
*=================
*----- Fichier de geometrie (GE...)
* 1) Cas ou tous les elements du mailage sont de meme type
* Entete :  GEO 
*           NOM `GE//Nomext` 
*           Chdim Type_des_elements
*           NBPTS $Nbpts NBEL $Nbel
* Corps  :  NUMEL
*           {liste des numeros des noeuds des Nbel elements du domaine}
*           POINTS
*           {liste des coordonnees des Nbpts noeuds du domaine}
*           FIN
* 2) Cas d'elements de types distincts
* Entete :  GEO 
*           NOM `GE//Nomext` 
*           $Chdim 
*           NBPTS $Nbpts NBEL $Nbel
* Corps  :  NUMEL
*           {Type_de_l'element
*           liste des numeros des noeuds des Nbel elements}
*           POINTS
*           {liste des coordonnees des noeuds du domaine}
*           FIN
*
* ou Nomext est le nom du dessin (valeur de la constante NOMINT)
*    Chdim      la valeur de CHDIM
*    Type_des_elements est le type d'un element (Pk,Qk,Rk)
*    Type_de_l'element est le type d'un element (Pk,Qk,Rk)
*    Nbpts      le nombre de points du domaine
*    Nbel       le nombre d'elements du domaine
*
*----- Fichiers de resultats
* Entete :  RES
*           NOM 'Commentaire'
*           REFGEO `GE//Nomext`
* Corps  :  VAL
*           {liste des valeurs (en colonne par composantes)}
*           FIN
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) NOMINT,CHDIM,OUTPUT
*
      INCLUDE 'ALLOC'
      INCLUDE 'CONTEX'
*
      INTEGER   KLUNIT
      INTEGER   LNOUTP,LNPRFX,LNSUFX,LNMEXT,NUNITR,AD1DOM,NBCHN,IAD
     &         ,LGENDS,NBDESS,LGCHAR,LGCHAW,NBCARD,INCNDS,INCNDO
     &         ,LGNMDS,MCNMDS,LGINDS,MCINDS
      LOGICAL   EXISTE,OUVERT
      CHARACTER LOWERC*8
      CHARACTER OUTPWT*8,PRFX*8,SUFX*8,NOMEXT*72,ERCODE*120
      COMMON/FORMAH/ERCODE
      DATA      LGENDS,NBCARD,INCNDS,INCNDO/6,3,2,8/
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'<GBegin>')
*
      OUTPWT = LOWERC (OUTPUT)
*
*--   Test sur les valeurs de CHDIM
*
      IF (CHDIM(1:1).NE.'1'.AND.CHDIM(1:1).NE.'2'.AND.CHDIM(1:1).NE.'3')
     &   GOTO 90002
      IF (CHDIM(2:2).NE.'D'.AND.CHDIM(2:2).NE.'d') GOTO 90002
      IF (OUTPWT(1:6).EQ.'unighn') THEN
         IF (LEN (CHDIM).GE.3.AND.CHDIM(3:3).NE.'5') GOTO 90002
      ELSEIF (LEN(CHDIM).GE.3) THEN
         GOTO 90002
      ENDIF
*
*--   Ouverture du fichier de resultats
*
      LNOUTP = LEN (OUTPUT)
      CALL GCHARS (OUTPWT,'r',OUTPWT,LNOUTP,PRFX,LNPRFX,SUFX,LNSUFX)
      CALL GFILEN (NOMINT,PRFX,LNPRFX,SUFX,LNSUFX,NOMEXT,LNMEXT)
*
      INQUIRE (FILE=NOMEXT(1:LNMEXT),EXIST=EXISTE,OPENED=OUVERT)
      IF (EXISTE.AND.OUVERT) GOTO 90001
      NUNITR=KLUNIT ( )                                                 !sdexplo
      OPEN (UNIT=NUNITR,FILE=NOMEXT(1:LNMEXT),STATUS='UNKNOWN'
     &     ,ACCESS='SEQUENTIAL',FORM='FORMATTED',ERR=90000)
*
*--   Structure de donnees
*
      ERCODE(1:1)='!'
      CALL TBRR2  (ERCODE,'$NomDS',1,LGNMDS,'$InfDS',1,LGINDS)
      IF (ERCODE(1:1).EQ.'C') THEN
*
*        Creation
*
         LGNMDS=1
         LGINDS=LGENDS
         CALL TBCREE ('$NomDS',1,11,LGNMDS,'c')
         CALL TBCREE ('$InfDS',1,1,LGINDS,'c')
*
*        Mise en place de l'en-tete
*
         CALL TBAR1(ERCODE,'$InfDS',1,MCINDS)
         IST(MCINDS)  =LGENDS
         IST(MCINDS+1)=0
         IST(MCINDS+2)=0
         IST(MCINDS+3)=NBCARD
         IST(MCINDS+4)=INCNDS
         IST(MCINDS+5)=INCNDO
      ENDIF
*
*     Ajustement du tableau des noms de domaines
*
      CALL TBAR1  (ERCODE,'$InfDS',1,MCINDS)                     
      LGCHAR=IST(MCINDS+2)
      LGCHAW=MAX (LGCHAR,LEN (NOMINT),LEN (CHDIM),LEN (OUTPWT))
      NBDESS=IST(MCINDS+1)
      NBCHN =NBCARD*NBDESS
      CALL SDAJST ('$NomDS',1,LGNMDS,0,LGCHAW,NBCHN,3)                  !sdexplo
      CALL TBAR1  (ERCODE,'$NomDS',1,MCNMDS)                     
      CALL RIPCHN (AST(MCNMDS),NBCHN,LGCHAR,NOMINT)                     !utilite
      CALL RIPCHN (AST(MCNMDS),NBCHN,LGCHAR,CHDIM)                      !utilite
      CALL RIPCHN (AST(MCNMDS),NBCHN,LGCHAR,OUTPWT)                     !utilite
      IST(MCINDS+2)=LGCHAR
*
*     Ajustement du tableau des caracteristiques des domaines
*
      LGINDS=LGINDS+INCNDS
      CALL TBAJST ('$InfDS',1,LGINDS)
      CALL TBAR1  (ERCODE,'$InfDS',1,MCINDS)                     
*
*     Decalage pour introduction du nouveau dessin
*
      NBDESS=NBDESS+1
      IST(MCINDS+1)=NBDESS
      AD1DOM=MCINDS+LGENDS+INCNDS*NBDESS
      DO 1 IAD=MCINDS+LGINDS-1,AD1DOM,-1
         IST(IAD)=IST(IAD-INCNDS)
1     CONTINUE
      IST(AD1DOM-2)=0
      IST(AD1DOM-1)=0
*
      CALL PRFXMJ (-1,'<GBegin>')
      RETURN
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
90000 CALL BAISE  ('Erreur a l''ouverture du fichier de resultats'
     &            //NOMEXT(1:LNMEXT))
90001 ERCODE=NOMINT
      NBCHN=INDEX (ERCODE,' ')-1
      CALL BAISE  ('Le dessin '//ERCODE(1:NBCHN)// '('//NOMEXT(1:LNMEXT)
     & //') est deja ouvert!')
90002 ERCODE=CHDIM
      CALL BAISE  ('Dimension du dessin non valide : CHDIM = '
     &//ERCODE(1:3))
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 2 : D.Martin (23.mars.2010)
* ajout d'un commentaire pour le format Paraview
* Version 1 : D.Martin (17.juin.2002)
* Version 0 : M.Lenoir & F.Mahe (1992), D.Martin (mars.2002)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !GBegin                         
