*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE DIVBT1 (MIXTE,NBL,DWL,NCL,NBC,WC,NCC,NDIM
     &                  ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : O.DeBayser (Avril 1993)
* Derniere modification : D.Martin (28 juin 2005)
* Version 2
*
* Calcul de l'integrale elementaire pour inconnues vectorielles a 3 composantes
* en dimension 2, en facteur de Beta dans l'integrale
*                       div_Beta(u) div_Beta(v)
*
* ou div_Beta(u) = du_1/dx_1 + du_2/dx_2 - Beta u_3 :
*
*                     |     0          0      dWi/dx1.Wj |
*     a_{i1,i,j1,j} = |     0          0      dWi/dx2.Wj |
*                     | Wi.dWj/dx1 Wi.dWj/dx2     0      |
*
*-- Arguments --
* MIXTE  indicateur de mixite des interpolations
* NBL    nombre de lignes (comptees en noeuds) de la matrice elementaire
* DWL    derivees des fonctions a l'image du point de quadrature dans
*        l'element courant pour l'interpolation en ligne
* NCL    nombre de composantes pour l'inconnue en ligne
* NBC    nombre de colonnes (comptees en noeuds) de la matrice elementaire
* WC     valeurs des fonctions de base au point de quadrature
* NCC    nombre de composantes pour l'inconnue en colonne
* NDIM   dimension d'espace
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand ou du coeff. variable
* COFVAR coeff. variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coeff. variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    numero du fichier d'impression si non nul
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPE
      INTEGER       MIXTE,NBC,NCC,NBL,NCL,NDIM,IMP
      REAL          WC(*),DWL(NDIM,*),ELDIPQ
      REAL          COFVAR,COEFFR(NCL,NBL,NCC,*)
      COMPLEX       COFVAC,COEFFC(NCL,NBL,NCC,*)
*
      INTEGER       J,I,I1
      REAL          RX,WCJ
      COMPLEX       CX
*-----------------------------------------------------------------------
*
      IF (TYPE(1:1).EQ.'R') THEN
*
         DO 23 J=1,NBC
            WCJ=ELDIPQ*WC(J)
            DO 22 I=1,NBL
               DO 21 I1=1,NDIM
                  RX=COEFFR(I1,I,NCC,J)
                  RX=RX+ COFVAR * WCJ * DWL(I1,I)
                  COEFFR(I1,I,NCC,J)=RX
                  COEFFR(NCL,J,I1,I)=RX
21             CONTINUE
22          CONTINUE
23       CONTINUE
*
      ELSE
*
         DO 43 J=1,NBC
            WCJ=ELDIPQ*WC(J)
            DO 42 I=1,NBL
               DO 41 I1=1,NDIM
                  CX=COEFFC(I1,I,NCC,J)
                  CX=CX + COFVAC * WCJ * DWL(I1,I)
                  COEFFC(I1,I,NCC,J)=CX
                  COEFFC(NCL,J,I1,I)=CX
41             CONTINUE
42          CONTINUE
43       CONTINUE
*
      ENDIF
*
      IF (IMP.GT.0) THEN
         J=NBC
         IF (MIXTE.EQ.0) J=0
         CALL PRINTV ('Div_beta_1 U Div_beta_1 V (DIVBT1)'
     &               ,NCL,NBL,NCC,NBC,J,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
                                                                    END !DivBt1
