*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE DIVBT2 (MIXTE,NBL,WL,NCL,NBC,WC,NCC,NDIM
     &                  ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : O.DeBayser (Avril 1992)
* Derniere modification : D.Martin (28 juin 2005)
* Version 2
*
* Calcul de l'integrale elementaire pour inconnues vectorielles a 3 composantes
* en dimension 2, en facteur de Beta^2 dans l'integrale
*                       div_Beta(u) div_Beta(v)
*
* ou div_Beta(u) = du_1/dx1 + du_2/dx2 - Beta u_3 :
*
*                      |    0       0       0   |
*      a_{i1,i,j1,j} = |    0       0       0   |
*                      |    0       0     WiWj  |
*
*-- Arguments --
* MIXTE  indicateur de mixite des interpolations
* NBL    nombre de lignes (comptees en noeuds) de la matrice elementaire
* WL     fonctions de base au point de quadrature pour l'interpolation
*        en ligne
* NCL    nombre de composantes pour l'inconnue en ligne
* NBC    nombre de colonnes (comptees en noeuds) de la matrice elementaire
* WC     fonctions de base au point de quadrature pour l'interpolation
*        en colonne
* NCC    nombre de composantes pour l'inconnue en ligne
* NDIM   dimension d'espace
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand
* COFVAR coefficient variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coefficient variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    unite logique d'impression (si > 0)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPE
      INTEGER       MIXTE,NBL,NCL,NBC,NCC,NDIM,IMP
      REAL          WL(*),WC(*),ELDIPQ
     &             ,COFVAR,COEFFR(NCL,NBL,NCC,*)
      COMPLEX       COFVAC,COEFFC(NCL,NBL,NCC,*)
*
      INTEGER       I,J
      REAL          RX,WCJ
      COMPLEX       CX
*-----------------------------------------------------------------------
*
      IF (TYPE(1:1).EQ.'R') THEN
*
         DO 22 J=1,NBC
            WCJ=ELDIPQ * WC(J)
            DO 21 I=1,J
               RX=COEFFR(NCL,I,NCC,J)
               RX=RX + COFVAR * WCJ * WL(I)
               COEFFR(NCL,I,NCC,J)=RX
               COEFFR(NCC,J,NCL,I)=RX
21          CONTINUE
22       CONTINUE
*
      ELSE
*
         DO 42 J=1,NBC
            WCJ=ELDIPQ * WC(J)
            DO 41 I=1,J
               CX=COEFFC(NCL,I,NCC,J)
               CX=CX + COFVAC * WCJ * WL(I)
               COEFFC(NCL,I,NCC,J)=CX
               COEFFC(NCC,J,NCL,I)=CX
41          CONTINUE
42        CONTINUE
*
      ENDIF
*
      IF (IMP.GT.0) THEN
         J=NBC
         IF (MIXTE.EQ.0) J=0
         CALL PRINTV ('Div_beta2 U Div_beta2 V (U3V3) (DIVBT2)'
     &               ,NCL,NBL,NCC,NBC,J,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
                                                                    END !DivBt2
