*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE DIVPML (MIXTE,NBL,WL,DWL,NBC,WC,DWC,IDX,NDIM
     &                  ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : G. Legendre (Fevrier 2002)
* Derniere modification : D.Martin (18 octobre 2006)
* Version 1
*
* Calcul de l'integrale elementaire divU divV  (cf. DIVDIV)
* modifie pour utilisation dans une PML, dans lequel une ou des derivees
* partielles dw/dx_i sont remplacees par une direction pseudo-convective
*                  Alpha dw/dx_i + Lambda(i) w
*
* Remarque : Les valeurs de Alpha et des Lambda sont transmises par COMMON
*
*-- Arguments --
* MIXTE  indicateur de mixite des interpolations
* NBL    nombre de lignes (comptees en noeuds) de la matrice elementaire
* WL     valeurs des fonctions de base a l'image du point de quadrature
*        dans l'element courant pour l'interpolation en ligne
* DWL    derivees des fonctions a l'image du point de quadrature dans
*        l'element courant pour l'interpolation en ligne
* NBC    nombre de colonnes (comptees en noeuds) de la matrice elementaire
* WC     valeurs des fonctions de base a l'image du point de quadrature
*        dans l'element courant pour l'interpolation en colonne
* DWC    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en colonne
* IDX    tableau indiquant la ou les directions i de la 'derivee convective'
*                   Alpha dw/dx_i + Lambda(i) w
* NDIM   dimension d'espace
* ELDIPQ element differentiel elementaire au point de quadrature
*        (Valeur absolue du determinant jacobien ou element de surface/longueur)
*        multiplie par le poids de quadrature et
*        divise par le carre du determinant jacobien
* TYPE   type (REEL ou COMPLEXE) de la matrice elementaire
* COFVAR coeff. variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coeff. variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    numero du fichier d'impression si non nul
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPE
      INTEGER       MIXTE,NBL,NBC,NDIM,IDX(*),IMP
      REAL          WL(*),WC(*)
      REAL          DWL(NDIM,*),DWC(NDIM,*),ELDIPQ
     &             ,COFVAR,COEFFR(NDIM,NBL,NDIM,*)
      COMPLEX       COFVAC,COEFFC(NDIM,NBL,NDIM,*)
*
*     Pas tres jojo, le COMMON, mais faute de mieux
      COMPLEX       ALP,LAM(3)
      COMMON        /PML/ALP,LAM
*
      INTEGER       J,J1,I,I1
      COMPLEX       CX,DWCJ
*-----------------------------------------------------------------------
*
      IF (TYPE(1:1).EQ.'R') THEN
*
         CALL BAISE  ('Integrand DIVDIV dans PML de type COMPLEXE !')
         STOP
*
      ELSE
*
         DO 44 J=1,NBC
            DO 43 J1=1,NDIM
               IF (IDX(J1).EQ.0) THEN
                  DWCJ=ELDIPQ*DWC(J1,J)
               ELSEIF (IDX(J1).EQ.1) THEN
                  DWCJ=ELDIPQ*(ALP*DWC(J1,J)+LAM(J1)*WC(J))
               ENDIF
               DO 42 I1=1,NDIM
                  DO 41 I=1,NBL
                     CX=COEFFC(I1,I,J1,J)
                     IF (IDX(I1).EQ.0) THEN
                        CX=CX+COFVAC*DWCJ*DWL(I1,I)
                     ELSEIF (IDX(I1).EQ.1) THEN
                        CX=CX+COFVAC*DWCJ*(ALP*DWL(I1,I)-LAM(I1)*WL(I))
                     ENDIF
                     COEFFC(I1,I,J1,J)=CX
 41               CONTINUE
 42            CONTINUE
 43         CONTINUE
 44      CONTINUE
*
      ENDIF
*
      IF (IMP.GT.0) THEN
         J=NBC
         IF (MIXTE.EQ.0) J=0
         CALL PRINTV ('DivU DivV (DIVPML)'
     &               ,IDX,NDIM,NBL,NDIM,NBC,J,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
                                                                    END !DivPml
