*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE DIVUQ (NBL,WL,NBC,DWC,NDIM
     &                 ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin & F.Mahe (Mars 1997)
* Derniere modification : D.Martin (28 juin 2005)
* Version 2
*
* Calcul des integrales elementaires divU Q (U vectorielle, Q scalaire)
*
*                 | dW_j/dx1 |
*    a_{i,j1,j} = | dW_j/dx2 | T_i
*                 | dW_j/dx3 |
*
* ou U_n = U_n,j W_j , j=1,...,NBC, n=1,NDIM (inconnue en colonne vectorielle)
*        = n-eme composante de U
*    Q   = p_i T_i   , i=1,...,NBL (inconnue en ligne scalaire)
*
*-- Arguments --
* NBL    nombre de lignes de la matrice elementaire
* WL     valeurs des fonctions de base pour l'interpolation en ligne
*        (correspondant a la fonction-test scalaire Q)
* NBC    nombre de colonnes de la matrice elementaire
* DWC    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en colonne
*        (correspondant a l'inconnue vectorielle U)
* NDIM   dimension d'espace = nombre de composantes de U
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand ou du coeff. variable
* COFVAR coeff. variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coeff. variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    numero du fichier d'impression si non nul
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPE
      INTEGER       NBL,NBC,NDIM,IMP
      REAL          WL(*),DWC(NDIM,*),ELDIPQ
      REAL          COFVAR,COEFFR(NBL,NDIM,*)
      COMPLEX       COFVAC,COEFFC(NBL,NDIM,*)
*
      INTEGER       J1,J,I
      REAL          DWCJ
*-----------------------------------------------------------------------
      IF (TYPE(1:1).EQ.'R') THEN
*
         DO 24 J=1,NBC
            DO 23 J1=1,NDIM
               DWCJ=ELDIPQ * DWC(J1,J)
               DO 22 I=1,NBL
                  COEFFR(I,J1,J)=COEFFR(I,J1,J) + COFVAR * DWCJ * WL(I)
22             CONTINUE
23          CONTINUE
24       CONTINUE
*
      ELSE
*
         DO 44 J=1,NBC
            DO 43 J1=1,NDIM
               DWCJ=ELDIPQ * DWC(J1,J)
               DO 42 I=1,NBL
                  COEFFC(I,J1,J)=COEFFC(I,J1,J) + COFVAC * DWCJ * WL(I)
42             CONTINUE
43          CONTINUE
44       CONTINUE
*
      ENDIF
*
      IF (IMP.GT.0)
     &   CALL PRINTV ('divU Q (DIVUQ)'
     &               ,1,NBL,NDIM,NBC,NBC,TYPE,COEFFR,COEFFC,IMP)
                                                                    END !DivUQ
