*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE DTWDTZ (NBL,DWL,NBC,DWC,NDIM,NORMAL
     &                  ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Juin 2003)
* Derniere modification : D.Martin (26 mars 2009)
* Version 3
*
* Calcul de l'integrale elementaire dU/dt dV/dt pour inconnues scalaires
*    (seulement valable en dimension 2, calcule avec domaine etendu)
*
*-- Arguments --
* NBL    nombre de lignes de integrale elementaire
* DWL    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en ligne
* NBC    nombre de colonnes de l'integrale elementaire
* DWC    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en colonne
* NDIM   dimension d'espace
* NORMAL vecteur normal a l'image du point de quadrature dans l'elt. courant
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrale eementaire ou du coeff. variable
* COFVAR coeff. variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrales elementaires aux points
*        de quadrature (cas REEL)
* COFVAC toeff. variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrales elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP numero du fichier d'impression (pas d'impression si nul) .
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPE
      INTEGER       NBL,NBC,NDIM,IMP
      REAL          DWL(NDIM,*),DWC(NDIM,*),ELDIPQ,NORMAL(*)
      REAL          COFVAR,COEFFR(NBL,*)
      COMPLEX       COFVAC,COEFFC(NBL,*)
*
      INTEGER       J,I
      REAL          DWJDT,DWIDT
*-----------------------------------------------------------------------
      IF (TYPE(1:1).EQ.'R') THEN
*
         DO 24 J=1,NBC
*           Calcul de dWj/dt
            DWJDT = -DWC(1,J) * NORMAL(2) + DWC(2,J) * NORMAL(1)
            DO 23 I=1,J
*              Calcul de dWi/dt
               DWIDT = -DWL(1,I) * NORMAL(2) + DWL(2,I) * NORMAL(1)
               COEFFR(I,J)=COEFFR(I,J) + COFVAR*ELDIPQ*DWJDT*DWIDT
               COEFFR(J,I)=COEFFR(I,J)
23          CONTINUE
24       CONTINUE
*
      ELSE
*
         DO 44 J=1,NBC
            DWJDT =-DWC(1,J) * NORMAL(2) + DWC(2,J) * NORMAL(1)
            DO 43 I=1,NBL
               DWIDT=-DWL(1,I) * NORMAL(2) + DWL(2,I) * NORMAL(1)
               COEFFC(I,J)=COEFFC(I,J) + COFVAC*ELDIPQ*DWJDT*DWIDT
               COEFFC(J,I)=COEFFC(I,J)
43          CONTINUE
44       CONTINUE
      ENDIF
*
      IF (IMP.GT.0)
     &   CALL PRINTG ('dU/dt dV/dt (DTWDTZ, via CALIND)'
     &               ,NBL,NBC,0,TYPE,COEFFR,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 3 : D.Martin & G.Vial (26 mars 2009)
* Faute de frappe DWC au lieu de DWL lignes commencant par DWIDT = -
* Version 2 : D.Martin (28 juin 2005)
                                                                    END !dtWdtW
