*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE DWDW (MIXTE,L,M,NBL,DWL,NBC,DWC,NDIM
     &                ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Fevrier 1988)
* Derniere modification : D.Martin (28 juin 2005)
* Version 2
*
* Calcul de l'integrale elementaire dU/dxl dV/dxm, inconnues scalaires
*
*-- Arguments --
* MIXTE  Indice de mixite des interpolations en colonne/ligne
* L, M   l et m dans dWi/dxl.dWj/dxm
* NBL    nombre de lignes de la matrice elementaire
* DWL    derivees des fonctions a l'image du point de quadrature dans
*        l'element courant pour l'interpolation en ligne
* NBC    nombre de colonnes de la matrice elementaire
* DWC    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en colonne
* NDIM   dimension d'espace
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand ou du coeff. variable
* COFVAR coeff. variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coeff. variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    numero du fichier d'impression (pas d'impression si nul)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER TYPE*(*)
      INTEGER   MIXTE,L,M,NBL,NBC,NDIM,IMP
      REAL      DWC(NDIM,*),DWL(NDIM,*),ELDIPQ
     &         ,COFVAR,COEFFR(NBL,*)
      COMPLEX   COFVAC,COEFFC(NBL,*)
*
      INTEGER   J,I
      REAL      RCF,DWCJ
      COMPLEX   CCF
*-----------------------------------------------------------------------
*
      IF (TYPE(1:1).EQ.'R') THEN
*
         IF (L.EQ.M.AND.MIXTE.EQ.0) THEN
*           Integrand symetrique
            DO 22 J=1,NBC
               DWCJ=ELDIPQ*DWC(L,J)
               DO 21 I=1,J
                  RCF=COEFFR(I,J) + COFVAR * DWCJ * DWL(M,I)
                  COEFFR(I,J)=RCF
                  COEFFR(J,I)=RCF
21             CONTINUE
22          CONTINUE
         ELSE
            DO 32 J=1,NBC
               DWCJ=ELDIPQ*DWC(L,J)
               DO 31 I=1,NBL
                  COEFFR(I,J)=COEFFR(I,J) + COFVAR * DWCJ * DWL(M,I)
31             CONTINUE
32          CONTINUE
         ENDIF
*
      ELSE
*
         IF (L.EQ.M.AND.MIXTE.EQ.0) THEN
*           Integrand symetrique
            DO 42 J=1,NBC
               DWCJ=ELDIPQ*DWC(L,J)
               DO 41 I=1,J
                  CCF=COEFFC(I,J) + COFVAC * DWCJ * DWL(M,I)
                  COEFFC(I,J)=CCF
                  COEFFC(J,I)=CCF
41             CONTINUE
42          CONTINUE
         ELSE
            DO 52 J=1,NBC
               DWCJ=ELDIPQ*DWC(L,J)
               DO 51 I=1,NBL
                  COEFFC(I,J)=COEFFC(I,J) + COFVAC * DWCJ * DWL(M,I)
51             CONTINUE
52          CONTINUE
         ENDIF
*
      ENDIF
*
      IF (IMP.GT.0) THEN
         J=NBC
         IF (L.EQ.M.AND.MIXTE.EQ.0) J=0
         CALL PRINTG ('dU/dx'//CHAR(L+48)//' dV/dx'//CHAR(M+48)
     &               //' (DWDW)'
     &               ,NBL,NBC,J,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
                                                                    END !dWdW
