*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE EIIEJJ (MIXTE,NBL,DWL,NBC,DWC,NDIM
     &                  ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Decembre 1999)
* Derniere modification : D.Martin (28 juin 2005)
* Version 2
*
* Calcul de l'integrale elementaire pour inconnues vectorielles a 2 composantes
*
*                        |        0          dWi/dx1.dWj/dx2 |
*        a_{i1,i,j1,j} = |                                   |
*                        | dWi/dx2.dWj/dx1          0        |
*
* issu, par exemple, de la decomposition de l'integrand de l'elasticite,
* pour le calcul de l'integrale
*                  Int_K [ du1/dx1 dv2/dx2 + du2/dx2 dv1/dx1 ] dx
*
*-- Arguments --
* MIXTE  indicateur de mixite des interpolations
* NBL    nombre de lignes (comptees en noeuds) de la matrice elementaire
* DWL    derivees des fonctions a l'image du point de quadrature dans
*        l'element courant pour l'interpolation en ligne
* NBC    nombre de colonnes (comptees en noeuds) de la matrice elementaire
* DWC    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en colonne
* NDIM   dimension d'espace
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand
* COFVAR coefficient variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coefficient variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    unite logique d'impression (si > 0)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPE
      INTEGER       MIXTE,NBL,NBC,NDIM,IMP
      REAL          DWL(NDIM,*),DWC(NDIM,*),ELDIPQ
     &             ,COFVAR,COEFFR(NDIM,NBL,NDIM,*)
      COMPLEX       COFVAC,COEFFC(NDIM,NBL,NDIM,*)
*
      INTEGER       J,I,INTGKL
      REAL          RCF,DWC1J,DWC2J
*      COMPLEX       CCF
*-----------------------------------------------------------------------
*
      IF (TYPE(1:1).EQ.'R') THEN
*
         DO 22 J=1,NBC
            DWC1J = ELDIPQ * DWC(1,J)
            DWC2J = ELDIPQ * DWC(2,J)
            DO 21 I=1,J
               RCF = COEFFR(2,I,1,J)
               RCF = RCF + COFVAR * DWC1J * DWL(2,I)
               COEFFR(2,I,1,J)=RCF
               COEFFR(1,J,2,I)=RCF
*
               RCF=COEFFR(1,I,2,J)
               RCF = RCF + COFVAR * DWC2J * DWL(1,I)
               COEFFR(1,I,2,J)=RCF
               COEFFR(2,J,1,I)=RCF
21          CONTINUE
22       CONTINUE
*
      ELSE
*
         CALL ERINTG (7,INTGKL('Ui,iVj,j'))
*
      ENDIF
*
      IF (IMP.GT.0) THEN
         J=NBC
         IF (MIXTE.EQ.0) J=0
         CALL PRINTV ('Ui,iVj,j (EIIEJJ)'
     &  ,NDIM,NBL,NDIM,NBC,J,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
                                                                    END !EiiEjj
