*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE EIJBT0 (MIXTE,NBL,DWL,NCL,NBC,DWC,NCC,NDIM
     &                  ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : L.Dahi & A.S.Bonnet (1997)
* Derniere modification : D.Martin (28 juin 2005)
* Version 2
*
* Calcul de l'integrale elementaire pour inconnues vectorielles a 3 composantes
*
* obtenue pour Beta=0 dans l'integrale [E_ij_Beta(u).E_ij_Beta(v)]
*                   | |
* ou E_ij_Beta(u) = | |
*                   | |
*-- Arguments --
* MIXTE  indicateur de mixite des interpolations
* NCL    nombre de composantes pour l'inconnue en ligne
* DWL    derivees des fonctions a l'image du point de quadrature dans
*        l'element courant pour l'interpolation en ligne
* NBL    nombre de lignes (comptees en noeuds) de la matrice elementaire
* NBC    nombre de colonnes (comptees en noeuds) de la matrice elementaire
* DWC    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en colonne
* NCC    nombre de composantes pour l'inconnue en colonne
* NDIM   dimension d'espace
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de la matrice elementaire
* COFVAR coeff. variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coeff. variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPE
      INTEGER       MIXTE,NBL,NCL,NBC,NCC,NDIM,IMP
      REAL          DWL(NDIM,*),DWC(NDIM,*),ELDIPQ
      REAL          COFVAR,COEFFR(NCL,NBL,NCC,*)
      COMPLEX       COFVAC,COEFFC(NCL,NBL,NCC,*)
*
      INTEGER       J,I,J1,I1,JDER,IDER
      REAL          RX,RY,DWJDX
      COMPLEX       CX,CY
*-----------------------------------------------------------------------
*
      IF (TYPE(1:1).EQ.'R') THEN
*
         DO 24 J1=1,NDIM
            DO 23 I1=1,NDIM
               IF (J1.EQ.I1) THEN
                  DO 2 J=1,NBC
                     DWJDX=2.*ELDIPQ*DWC(J1,J)
                     DO 1 I=1,J-1
                        RX=COEFFR(J1,I,J1,J)
                        RX=RX+COFVAR*DWJDX*DWL(I1,I)
                        COEFFR(I1,I,J1,J)=RX
                        COEFFR(J1,J,I1,I)=RX
1                    CONTINUE
2                 CONTINUE
                  DO 3 J=1,NBC
                     COEFFR(I1,J,J1,J)=COEFFR(I1,J,J1,J)+2.
     &                                *COFVAR*ELDIPQ*DWC(J1,J)*DWL(I1,J)
3                 CONTINUE
               ENDIF
               JDER=J1-(J1/NDIM)*NDIM+1
               IDER=I1-(I1/NDIM)*NDIM+1
               DO 22 J=1,NBC
                  DWJDX=ELDIPQ*DWC(JDER,J)
                  DO 21 I=1,J-1
                     RY=COFVAR*DWJDX*DWL(IDER,I)
                     IF (JDER.EQ.IDER) THEN
                        RX=COEFFR(NCL,I,NCC,J)
                        RX=RX+RY
                        COEFFR(NCL,I,NCC,J)=RX
                        COEFFR(NCL,J,NCC,I)=RX
                     ENDIF
                     RX=COEFFR(I1,I,J1,J)
                     RX=RX+RY
                     COEFFR(I1,I,J1,J)=RX
                     COEFFR(J1,J,I1,I)=RX
21                CONTINUE
22             CONTINUE
23          CONTINUE
24       CONTINUE
         DO 27 J1=1,NDIM
            DO 26 I1=1,J1
               JDER=J1-(J1/NDIM)*NDIM+1
               IDER=I1-(I1/NDIM)*NDIM+1
               DO 25 J=1,NBC
                  DWJDX=ELDIPQ*DWC(JDER,J)
                  RY=COFVAR*DWJDX*DWL(IDER,J)
                  IF (JDER.EQ.IDER)
     &               COEFFR(NCL,J,NCC,J)=COEFFR(NCL,J,NCC,J)+RY
                  RX=COEFFR(I1,J,J1,J)
                  RX=RX+RY
                  COEFFR(I1,J,J1,J)=RX
                  COEFFR(J1,J,I1,J)=RX
25             CONTINUE
26          CONTINUE
27       CONTINUE
*
      ELSE
*
         DO 44 J1=1,NDIM
            DO 43 I1=1,NDIM
               IF (J1.EQ.I1) THEN
                  DO 4 J=1,NBC
                     DWJDX=2.*ELDIPQ*DWC(J1,J)
                     DO 5 I=1,J-1
                        CX=COEFFC(J1,I,J1,J)
                        CX=CX+COFVAC*DWJDX*DWL(I1,I)
                        COEFFC(I1,I,J1,J)=CX
                        COEFFC(J1,J,I1,I)=CX
5                    CONTINUE
4                 CONTINUE
                  DO 6 J=1,NBC
                     DWJDX=2.*ELDIPQ*DWC(J1,J)
                     COEFFC(I1,J,J1,J)=COEFFC(I1,J,J1,J)+2.
     &                                *COFVAC*ELDIPQ*DWC(J1,J)*DWL(I1,J)
6                 CONTINUE
               ENDIF
               JDER=J1-(J1/NDIM)*NDIM+1
               IDER=I1-(I1/NDIM)*NDIM+1
               DO 42 J=1,NBC
                  DWJDX=ELDIPQ*DWC(JDER,J)
                  DO 41 I=1,J-1
                     CY=COFVAC*DWJDX*DWL(IDER,I)
                     IF (JDER.EQ.IDER) THEN
                        CX=COEFFC(NCL,I,NCC,J)
                        CX=CX+CY
                        COEFFC(NCL,I,NCC,J)=CX
                        COEFFC(NCL,J,NCC,I)=CX
                     ENDIF
                     CX=COEFFC(I1,I,J1,J)
                     CX=CX+CY
                     COEFFC(I1,I,J1,J)=CX
                     COEFFC(J1,J,I1,I)=CX
41                CONTINUE
42             CONTINUE
43          CONTINUE
44       CONTINUE
         DO 47 J1=1,NDIM
            DO 46 I1=1,J1
               JDER=J1-(J1/NDIM)*NDIM+1
               IDER=I1-(I1/NDIM)*NDIM+1
               DO 45 J=1,NBC
                  DWJDX=ELDIPQ*DWC(JDER,J)
                  CY=COFVAC*DWJDX*DWL(IDER,J)
                  IF (JDER.EQ.IDER)
     &               COEFFC(NCL,J,NCC,J)=COEFFC(NCL,J,NCC,J)+CY
                  CX=COEFFC(I1,I,J1,J)
                  CX=CX+CY
                  COEFFC(I1,J,J1,J)=CX
                  COEFFC(J1,J,I1,J)=CX
45             CONTINUE
46          CONTINUE
47       CONTINUE
*
      ENDIF
*
      IF (IMP.GT.0) THEN
         J=NBC
         IF (MIXTE.EQ.0) J=0
         CALL PRINTV ('Eij_ij_beta0(U) E_ij_beta0(V) (EOJBT0)'
     &               ,NCL,NBL,NCC,NBC,J,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
                                                                    END !Eijbt0
