*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE EIJEIJ (MIXTE,NBL,DWL,NBC,DWC,NDIM
     &                  ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : L.Dahi, A.S.Bonnet-BenDhia (Mai 1998)
* Derniere modification : V.Baronian & E.Luneville (15 septembre 2008)
* Version 3
*
* Calcul de l'integrand elementaire
*             Som_{i,j} 1/2 ( Epsilon_{ij}(u) Epsilon_{ij}(v) )
* avec Epsilon_{ij}(u) = 1/2 ( du_i/dx_j + du_j/dx_i )
*
*-- Arguments --
* NBL    nombre de lignes (comptees en noeuds) de l'integrand
* DWL    derivees des fonctions a l'image du point de quadrature dans
*        l'element courant pour l'interpolation en ligne
* NBC    nombre de colonnes (comptees en noeuds) de l'integrand
* DWC    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en colonne
* NDIM   dimension d'espace
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand
* COFVAR coeff. variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coeff. variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    numero du fichier d'impression si non nul
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPE
      INTEGER       MIXTE,NBL,NBC,NDIM,IMP
      REAL          DWL(NDIM,*),DWC(NDIM,*),ELDIPQ
      REAL          COFVAR,COEFFR(NDIM,NBL,NDIM,*)
      COMPLEX       COFVAC,COEFFC(NDIM,NBL,NDIM,*)
*
      INTEGER       J,J1,I,I1
      REAL          RX,DWJDX,RS
      COMPLEX       CX,CS
*-----------------------------------------------------------------------
*
      IF (TYPE(1:1).EQ.'R') THEN
*
         DO 29 J=1,NBC
            DO 28 I=1,NBL
               DO 21 J1=1,NDIM
                  DWJDX=0.5*COFVAR*ELDIPQ*DWL(J1,I)
                  DO 20 I1=1,NDIM
                     RX=COEFFR(I1,I,J1,J)
                     RX=RX+DWJDX*DWC(I1,J)
                     COEFFR(I1,I,J1,J)=RX
20                CONTINUE                     
21             CONTINUE 
* termes supplementaires sur les elements diagonaux
               RS=0.
               DO 22 I1=1,NDIM
                  RS=RS+DWC(I1,J)*DWL(I1,I)
22             CONTINUE           
               RS=0.5*RS*ELDIPQ*COFVAR                    
               DO 23 I1=1,NDIM
                  RX=COEFFR(I1,I,I1,J)+RS
                  COEFFR(I1,I,I1,J)=RX
23             CONTINUE  
28          CONTINUE               
29       CONTINUE   
*
      ELSE
*
         DO 49 J=1,NBC
            DO 48 I=1,NBL
               DO 41 J1=1,NDIM
                  DWJDX=0.5*COFVAC*ELDIPQ*DWL(J1,I)
                  DO 40 I1=1,NDIM
                     CX=COEFFC(I1,I,J1,J)
                     CX=CX+DWJDX*DWC(I1,J)
                     COEFFC(I1,I,J1,J)=CX
40                CONTINUE                     
41             CONTINUE 
               RS=0.
               DO 42 I1=1,NDIM
                  RS=RS+DWC(I1,J)*DWL(I1,I)
42             CONTINUE                             
               CS=0.5*RS*ELDIPQ*COFVAC                     
               DO 43 I1=1,NDIM
                  CX=COEFFC(I1,I,I1,J)+CS
                  COEFFC(I1,I,I1,J)=CX                 
43             CONTINUE   
48          CONTINUE               
49       CONTINUE   
*
      ENDIF
*
      IF (IMP.GT.0) THEN
         J=NBC
         IF (MIXTE.EQ.0) J=0
         CALL PRINTV ('Epsilon_ij(U) Epsilon_ij(V) (EIJEIJ)'
     &               ,NDIM,NBL,NDIM,NBC,0,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
                                                                    END !EijEij
