*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE EIJEJI (MIXTE,NBL,DWL,NBC,DWC,NDIM
     &                  ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Decembre 1999)
* Derniere modification : D.Martin (28 juin 2005)
* Version 2
*
* Calcul de l'integrale elementaire
*             Int_K [ ( du1/dx2 + du2/dx1 ) ( dv1/dx2 + dv2/dx1 ) ] dx
*
* en dimension 2 :
*                | dWi/dx2.dWj/dx2  dWi/dx2.dWj/dx1 |
* a_{i1,i,j1,j}= |                                  |
*                | dWi/dx1.dWj/dx2  dWi/dx1.dWj/dx1 |
*
* en dimension 3 :
*                |  dWi/dx2.dWj/dx2  dWi/dx2.dWj/dx1  dWi/dx3.dWj/dx1  |
*                | +dWi/dx3.dWj/dx3                                    |
*                |                                                     |
* a_{i1,i,j1,j}= |  dWi/dx1.dWj/dx2  dWi/dx1.dWj/dx1  dWi/dx3.dWj/dx2  |
*                |                   +dWi/dx3.dWj/dx3                  |
*                |                                                     |
*                |  dWi/dx1.dWj/dx3  dWi/dx2.dWj/dx3  dWi/dx1.dWj/dx1  |
*                |                                    +dWi/dx2.dWj/dx2 |
*
*-- Arguments --
* NBL    nombre de lignes (comptees en noeuds) de l'integrand
* DWL    derivees des fonctions a l'image du point de quadrature dans
*        l'element courant pour l'interpolation en ligne
* NBC    nombre de colonnes (comptees en noeuds) de l'integrand
* DWC    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en colonne
* NDIM   dimension d'espace
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand
* COFVAR coeff. variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coeff. variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    numero du fichier d'impression si non nul
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPE
      INTEGER       MIXTE,NBL,NBC,NDIM,IMP
      REAL          DWL(NDIM,*),DWC(NDIM,*),ELDIPQ
      REAL          COFVAR,COEFFR(NDIM,NBL,NDIM,*)
      COMPLEX       COFVAC,COEFFC(NDIM,NBL,NDIM,*)
*
      INTEGER       J,J1,JDER,I,I1,IDER,NUT,NBTERM
      REAL          RCF,DWJDX
      COMPLEX       CCF
*-----------------------------------------------------------------------
*
      IF (TYPE(1:1).EQ.'R') THEN
*
         DO 25 J1=1,NDIM
            DO 24 I1=1,NDIM
               NBTERM=1
               JDER=I1
               IDER=J1
               IF (J1.EQ.I1) THEN
                  NBTERM=NDIM-1
                  JDER=J1-(J1/NDIM)*NDIM+1
                  IDER=JDER
               ENDIF
               DO 23 NUT=1,NBTERM
                  DO 22 J=1,NBC
                     DWJDX=ELDIPQ*DWC(JDER,J)
                     DO 21 I=1,J-1
                        RCF=COEFFR(I1,I,J1,J)
                        RCF=RCF + COFVAR * DWJDX * DWL(IDER,I)
                        COEFFR(I1,I,J1,J)=RCF
                        COEFFR(J1,J,I1,I)=RCF
21                   CONTINUE
                     IF (I1.LE.J1) THEN
                        RCF=COEFFR(I1,J,J1,J)
                        RCF=RCF + COFVAR * DWJDX * DWL(IDER,J)
                        COEFFR(I1,J,J1,J)=RCF
                        COEFFR(J1,J,I1,J)=RCF
                     ENDIF
22                CONTINUE
                  JDER=J1-((J1+1)/NDIM)*NDIM+2
                  IDER=JDER
23             CONTINUE
24          CONTINUE
25       CONTINUE
*
      ELSE
*
         DO 45 J1=1,NDIM
            DO 44 I1=1,NDIM
               NBTERM=1
               JDER=I1
               IDER=J1
               IF (J1.EQ.I1) THEN
                  NBTERM=NDIM-1
                  JDER=J1-(J1/NDIM)*NDIM+1
                  IDER=JDER
               ENDIF
               DO 43 NUT=1,NBTERM
                  DO 42 J=1,NBC
                     DWJDX=ELDIPQ*DWC(JDER,J)
                     DO 41 I=1,J-1
                        CCF=COEFFC(I1,I,J1,J)
                        CCF=CCF + COFVAC * DWJDX * DWL(IDER,I)
                        COEFFC(I1,I,J1,J)=CCF
                        COEFFC(J1,J,I1,I)=CCF
41                   CONTINUE
                     IF (I1.LE.J1) THEN
                        CCF=COEFFC(I1,J,J1,J)
                        CCF=CCF + COFVAC * DWJDX * DWL(IDER,J)
                        COEFFC(I1,J,J1,J)=CCF
                        COEFFC(J1,J,I1,J)=CCF
                     ENDIF
42                CONTINUE
                  JDER=J1-((J1+1)/NDIM)*NDIM+2
                  IDER=JDER
43             CONTINUE
44          CONTINUE
45       CONTINUE
*
      ENDIF
*
      IF (IMP.GT.0) THEN
         J=NBC
         IF (MIXTE.EQ.0) J=0
         CALL PRINTV ('Ui,jVj,i (EIJEJI)'
     &               ,NDIM,NBL,NDIM,NBC,J,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
                                                                    END !EijEji
