*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE EIJPML (MIXTE,NBL,DWL,NBC,DWC,IDX,NDIM
     &                  ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : L.Dahi, A.S.Bonnet-BenDhia, G.Legendre (Mai 1998)
* Derniere modification : V.Baronian & E.Luneville (15 septembre 2008)
* Pour la version PML : G.Legendre (27 octobre 2008)
* Version 3
*
* Calcul de l'integrand elementaire
*             Som_{i,j} 1/2 ( Epsilon_{ij}(u) Epsilon_{ij}(v) )
* avec Epsilon_{ij}(u) = 1/2 ( du_i/dx_j + du_j/dx_i )
* (cf. EIJEIJ) modifie pour utilisation dans une PML, dans lequel une ou des
* derivees partielles dw/dx_i sont remplacees par Alpha dw/dx_i
*
* Remarques : Par rapport aux autres integrands "PML", on a Lambda(i)=0.
*             La valeur de Alpha est transmise par COMMON
*-- Arguments --
* NBL    nombre de lignes (comptees en noeuds) de l'integrand
* DWL    derivees des fonctions a l'image du point de quadrature dans
*        l'element courant pour l'interpolation en ligne
* NBC    nombre de colonnes (comptees en noeuds) de l'integrand
* DWC    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en colonne
* IDX    tableau indiquant la ou les directions i de modifications des
*        derivees
* NDIM   dimension d'espace
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand
* COFVAR coeff. variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coeff. variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    numero du fichier d'impression si non nul
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPE
      INTEGER       MIXTE,NBL,NBC,NDIM,IDX(*),IMP
      REAL          DWL(NDIM,*),DWC(NDIM,*),ELDIPQ
      REAL          COFVAR,COEFFR(NDIM,NBL,NDIM,*)
      COMPLEX       COFVAC,COEFFC(NDIM,NBL,NDIM,*)
*
      INTEGER       J,J1,I,I1
      COMPLEX       CX,CS,DWJDX
!     Pas tres jojo, le COMMON, mais faute de mieux
      COMPLEX       ALP,LAM(3)
      COMMON        /PML/ALP,LAM
*-----------------------------------------------------------------------
*
      IF (TYPE(1:1).EQ.'R') THEN
*
         CALL BAISE  ('EIJEIJ PML est un integrand de type COMPLEXE !')
         STOP 
*
      ELSE
*
         DO 49 J=1,NBC
            DO 48 I=1,NBL
               DO 41 J1=1,NDIM
                  IF (IDX(J1).EQ.0) THEN
                     DWJDX=0.5*COFVAC*ELDIPQ*DWL(J1,I)
                  ELSEIF (IDX(J1).EQ.1) THEN
                     DWJDX=0.5*COFVAC*ELDIPQ*ALP*DWL(J1,I)
                  ENDIF
                  DO 40 I1=1,NDIM
                     CX=COEFFC(I1,I,J1,J)
                     IF (IDX(I1).EQ.0) THEN
                        CX=CX+DWJDX*DWC(I1,J)
                     ELSEIF (IDX(I1).EQ.1) THEN
                        CX=CX+DWJDX*ALP*DWC(I1,J)
                     ENDIF
                     COEFFC(I1,I,J1,J)=CX
40                CONTINUE                     
41             CONTINUE 
               CS=(0.,0.)
               DO 42 I1=1,NDIM
                  IF (IDX(I1).EQ.0) THEN
                     CS=CS+DWC(I1,J)*DWL(I1,I)
                  ELSEIF (IDX(I1).EQ.1) THEN
                     CS=CS+ALP*DWC(I1,J)*ALP*DWL(I1,I)
                  ENDIF
42             CONTINUE                             
               CS=0.5*CS*ELDIPQ*COFVAC                     
               DO 43 I1=1,NDIM
                  CX=COEFFC(I1,I,I1,J)+CS
                  COEFFC(I1,I,I1,J)=CX                 
43             CONTINUE   
48          CONTINUE               
49       CONTINUE   
*
      ENDIF
*
      IF (IMP.GT.0) THEN
         J=NBC
         IF (MIXTE.EQ.0) J=0
         CALL PRINTV ('Epsilon_ij(U) Epsilon_ij(V) (EIJPML)'
     &               ,IDX,NDIM,NBL,NDIM,NBC,0,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
                                                                    END !EijPml
