*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE GRAWW (NBL,WL,NBC,DWC,NDIM
     &                 ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin & O.DeBayser (Avril 1993)
* Derniere modification : D.Martin (28 juin 2005)
* Version 2
*
* Calcul des integrales elementaires gradU.V (U scalaire, V vectorielle)
*
*    a_{i1,i,j} = ( dW_j/dx1 dW_j/dx2 dW_j/dx3 ) T_i
*
* ou U   = u_j W_j     ; j=1,...,NBC (inconnue en colonne scalaire)
*    V_n = v_{n,i} T_i ; i=1,...,NBL, i1=1,NDIM (inconnue en ligne vectorielle)
*        = n-eme composante de V
*-- Arguments --
* NBL    nombre de lignes de la matrice elementaire
* WL     valeurs des fonctions de base pour l'interpolation en ligne
*        (correspondant a la fonction-test vectorielle V)
* NBC    nombre de colonnes de la matrice elementaire
* DWC    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en colonne
*        (correspondant a l'inconnue scalaire U)
* NDIM   dimension d'espace
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand ou du coeff. variable
* COFVAR coeff. variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coeff. variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    numero du fichier d'impression si non nul
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPE
      INTEGER       NBL,NBC,NDIM,IMP
      REAL          WL(*),DWC(NDIM,*),ELDIPQ
      REAL          COFVAR,COEFFR(NDIM,NBL,*)
      COMPLEX       COFVAC,COEFFC(NDIM,NBL,*)
*
      INTEGER       I1,I,J
*-----------------------------------------------------------------------
      IF (TYPE(1:1).EQ.'R') THEN
*
         DO 24 J=1,NBC
            DO 22 I=1,NBL
               DO 21 I1=1,NDIM
                  COEFFR(I1,I,J)=COEFFR(I1,I,J) 
     &                          + COFVAR * ELDIPQ * DWC(I1,J) * WL(I)
21             CONTINUE
22          CONTINUE
24       CONTINUE
*
      ELSE
*
         DO 44 J=1,NBC
            DO 42 I=1,NBL
               DO 41 I1=1,NDIM
                  COEFFC(I1,I,J)=COEFFC(I1,I,J)
     &                          + COFVAC * ELDIPQ * DWC(I1,J) * WL(I)
41             CONTINUE
42          CONTINUE
44       CONTINUE
*
      ENDIF
*
      IF (IMP.GT.0)
     &   CALL PRINTV ('gradU.V (GRAWW)'
     &               ,NDIM,NBL,1,NBC,NBC,TYPE,COEFFR,COEFFC,IMP)
                                                                    END !GraWW
