*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE GRVGRV (MIXTE,NBL,DWL,NBC,DWC,NDIM
     &                  ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : O.DeBayser (Fevrier 1993)
* Derniere modification : D.Martin (28 juin 2005)
* Version 2
*
* Calcul de l'integrale elementaire GradU.GradV pour inconnues vectorielles
*
*-- Arguments --
* MIXTE  indicateur de mixite de l'integrand
* NBL    nombre de lignes (comptees en noeuds) de la matrice elementaire
* DWL    derivees des fonctions a l'image du point de quadrature dans
*        l'element courant pour l'interpolation en ligne
* NBC    nombre de colonnes (comptees en noeuds) de la matrice elementaire
* DWC    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en colonne
* NDIM   dimension d'espace
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand
* COFVAR coeff. variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coeff. variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    numero du fichier d'impression si non nul
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPE
      INTEGER       MIXTE,NBL,NBC,NDIM,IMP
      REAL          DWL(NDIM,*),DWC(NDIM,*),ELDIPQ
      REAL          COFVAR,COEFFR(NDIM,NBL,NDIM,*)
      COMPLEX       COFVAC,COEFFC(NDIM,NBL,NDIM,*)
*
      INTEGER       J,J1,I,ID
      REAL          RX,DWIDWJ
      COMPLEX       CX
*-----------------------------------------------------------------------
*
      IF (TYPE(1:1).EQ.'R') THEN
*
            DO 23 J=1,NBC
               DO 22 I=1,J
                  RX=COEFFR(1,I,1,J)
                  DWIDWJ=0.
                  DO 21 ID=1,NDIM
                     DWIDWJ=DWIDWJ+DWC(ID,J)*DWL(ID,I)
21                CONTINUE
                  RX=RX+COFVAR*ELDIPQ*DWIDWJ
                  COEFFR(1,I,1,J)=RX
22             CONTINUE
23          CONTINUE
            DO 26 J=1,NBC
               DO 25 I=1,J
                  RX=COEFFR(1,I,1,J)
                  COEFFR(1,J,1,I)=RX
                  DO 24 J1=2,NDIM
                     COEFFR(J1,I,J1,J)=RX
                     COEFFR(J1,J,J1,I)=RX
24                CONTINUE
25             CONTINUE
26          CONTINUE
*
      ELSE
*
            DO 43 J=1,NBC
               DO 42 I=1,J
                  CX=COEFFC(1,I,1,J)
                  DWIDWJ=0.
                  DO 41 ID=1,NDIM
                     DWIDWJ=DWIDWJ+DWC(ID,J)*DWL(ID,I)
41                CONTINUE
                  CX=CX+COFVAC*ELDIPQ*DWIDWJ
                  COEFFC(1,I,1,J)=CX
42             CONTINUE
43             CONTINUE
            DO 46 J=1,NBC
               DO 45 I=1,J
                  CX=COEFFC(1,I,1,J)
                  COEFFC(1,J,1,I)=CX
                  DO 44 J1=2,NDIM
                     COEFFC(J1,I,J1,J)=CX
                     COEFFC(J1,J,J1,I)=CX
44                CONTINUE
45             CONTINUE
46          CONTINUE
*
      ENDIF
*
      IF (IMP.GT.0) THEN
         J=NBC
         IF (MIXTE.EQ.0) J=0
         CALL PRINTV ('GradU.GradV inconnues vectorielles (GRVGRV)'
     &               ,NDIM,NBL,NDIM,NBC,J,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
                                                                    END !GrvGrv
