*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE INTNOS (NUINTG,ALPHA,BETA,GAMMA,DELTA)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Avril 2004)
* Derniere modification : D.Martin (3/6 decembre 2010)
* Version 4
*
* Retourne des valeurs de alpha, beta, gamma, delta pour les integrands
* vectoriels de la forme
*     Ualpha,gammaVbeta,delta = D_{gamma} U_{alpha} D_{delta} V_{beta}
*     ou de la forme
*     Oper1 U Oper2 V avec Oper_i = Identite, Grad, Rot, Div, E_ij ...
*
* NUINTG numero de l'integrand
* ALPHA, BETA, GAMMA, DELTA 
*     = i derivee ieme pour i = 1,2,3
*     = 4 Grad
*     = 5 Rot
*     = 6 Div
*     = 7 Epsilon_ij
*     = 8 Rot_{beta_0}
*     = 9 Rot_{beta_1}
*     =10 Rot_{beta_2}
*     =11 Div_{beta_0}
*     =12 Div_{beta_1}
*     =13 Div_{beta_2}
*     =14 Epsilon_ij_Beta_0
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      INTEGER NUINTG,ALPHA,BETA,GAMMA,DELTA
*      
      INTEGER NUIREL,GRAD,ROT,DIV,EIJ
     &       ,ROTBT0,ROTBT1,ROTBT2,DIVBT0,DIVBT1,DIVBT2,EIJBT0
      DATA GRAD,ROT,DIV,EIJ
     &    /   4,  5,  6,  7/
      DATA ROTBT0,ROTBT1,ROTBT2,DIVBT0,DIVBT1,DIVBT2,EIJBT0
     &    /     8,     9,    10,    11,    12,    13,    14/
*-----------------------------------------------------------------------
      ALPHA=0
      BETA =0
      GAMMA=0
      DELTA=0
      IF (NUINTG.LE.12) THEN
         BETA =1
         IF (NUINTG.EQ.1) THEN
            RETURN
         ELSEIF (NUINTG.LE.4) THEN
            DELTA=NUINTG-1
         ELSEIF (NUINTG.LE.6) THEN
            ALPHA=1
         ELSEIF (NUINTG.LE.9) THEN
            ALPHA=1
            DELTA=NUINTG-6
         ELSE
            ALPHA=1
            GAMMA=NUINTG-9
         ENDIF
      ELSEIF (NUINTG.LE.21) THEN
         NUIREL=NUINTG-13
         ALPHA=1
         BETA =1
         GAMMA=1+(NUIREL)/3
         DELTA=1+MOD(NUIREL,3)
      ELSEIF (NUINTG.LE.26) THEN
         ALPHA=1
         BETA =1
         IF (NUINTG.EQ.22) THEN
            GAMMA=GRAD
            DELTA=GRAD
         ELSEIF (NUINTG.EQ.23) THEN
            GAMMA=ROT
            DELTA=ROT
         ELSEIF (NUINTG.EQ.24) THEN
            GAMMA=DIV
            DELTA=DIV
         ELSEIF (NUINTG.EQ.25) THEN
            GAMMA=EIJ
            DELTA=EIJ
         ELSE
            GAMMA=GRAD
            DELTA=GRAD
         ENDIF
      ELSEIF (NUINTG.LE.39) THEN
         ALPHA=1
         BETA =1
         IF (NUINTG.EQ.27) THEN
            GAMMA=GRAD
         ELSEIF (NUINTG.EQ.28) THEN
            GAMMA=DIV
         ELSEIF (NUINTG.EQ.29) THEN
            GAMMA=ROT
         ELSEIF (NUINTG.EQ.30) THEN
            DELTA=DIV
         ELSEIF (NUINTG.EQ.31) THEN
            DELTA=GRAD
         ELSEIF (NUINTG.EQ.32) THEN
            DELTA=ROT
         ELSEIF (NUINTG.EQ.33) THEN
            GAMMA=ROTBT0
            DELTA=ROTBT0
         ELSEIF (NUINTG.EQ.34) THEN
            GAMMA=ROTBT1
            DELTA=ROTBT1
         ELSEIF (NUINTG.EQ.35) THEN
            GAMMA=ROTBT2
            DELTA=ROTBT2
         ELSEIF (NUINTG.EQ.36) THEN
            GAMMA=DIVBT0
            DELTA=DIVBT0
         ELSEIF (NUINTG.EQ.37) THEN
            GAMMA=DIVBT1
            DELTA=DIVBT1
         ELSEIF (NUINTG.EQ.38) THEN
            GAMMA=DIVBT2
            DELTA=DIVBT2
         ELSEIF (NUINTG.EQ.39) THEN
            GAMMA=EIJBT0
            DELTA=EIJBT0
         ENDIF
      ELSEIF (NUINTG.LE.40) THEN
         ALPHA=1
         BETA =1
      ELSEIF (NUINTG.LE.41) THEN
         BETA =1
         DELTA=DIV
      ELSEIF (NUINTG.LE.50) THEN
         NUIREL=NUINTG-42
         ALPHA=1+NUIREL/3
         BETA =1+MOD(NUIREL,3)
      ELSEIF (NUINTG.LE.131) THEN
*        Integrands U\alpha,\gamma V\beta,\delta
         NUIREL=NUINTG-51
         ALPHA=1+NUIREL/27
         GAMMA=1+MOD(NUIREL/9,3)
         BETA =1+MOD(NUIREL/3,3)
         DELTA=1+MOD(NUIREL,3)
      ELSEIF (NUINTG.LE.133) THEN
         ALPHA=1
         BETA =1
      ELSEIF (NUINTG.LE.160) THEN
*        Integrands U\alpha,\gamma V\beta
         NUIREL=NUINTG-134
         ALPHA=1+NUIREL/9
         GAMMA=1+MOD(NUIREL/3,3)
         BETA =1+MOD(NUIREL,3)
      ELSEIF (NUINTG.LE.187) THEN
*        Integrands U\alpha V\beta, \delta
         NUIREL=NUINTG-161
         ALPHA=1+MOD(NUIREL,3)
         BETA =1+NUIREL/9 
         DELTA=1+MOD(NUIREL/3,3)
      ELSEIF (NUINTG.LE.194) THEN
*        Integrands Rot Rot PML
         GAMMA=ROT
         DELTA=ROT
      ELSEIF (NUINTG.LE.201) THEN
*        Integrands Div Div PML
         GAMMA=DIV
         DELTA=DIV
      ELSEIF (NUINTG.LE.208) THEN
*        Integrands Epsilon_ij Epsilon_ij PML
         GAMMA=EIJ
         DELTA=EIJ
      ELSEIF (NUINTG.LE.215) THEN
*        Integrands U.RotV PML
         DELTA=ROT
      ELSEIF (NUINTG.LE.222) THEN
*        Integrands RotU.V PML
         GAMMA=ROT
      ELSEIF (NUINTG.LE.225) THEN
*         Integrands U_\alpha V
         ALPHA=NUINTG-222
      ELSEIF (NUINTG.LE.228) THEN
*        Integrands U V_\beta
         BETA =NUINTG-225
      ENDIF
*-----------------------------------------------------------------------
***      Test Program for Intnos & IntgNm
****
***      INTEGER NUINTG,ALPHA,BETA,GAMMA,DELTA
***      CHARACTER*8 INTGNM
***      DO 1 NUINTG=41,228
***      WRITE(10,9) NUINTG,INTGNM(NUINTG)
***    9 FORMAT (I4,' : ',A)
***      CALL INTNOS (NUINTG,ALPHA,BETA,GAMMA,DELTA)
***      IF (ALPHA.GE.1.AND.GAMMA.GE.1.AND.BETA.GE.1.AND.DELTA.GE.1) THEN
***         WRITE(*,11) NUINTG,ALPHA,GAMMA,BETA,DELTA
***      ELSEIF (ALPHA.GE.1.AND.GAMMA.GE.1.AND.BETA.GE.1) THEN
***         WRITE(*,12) NUINTG,ALPHA,GAMMA,BETA
***      ELSEIF (ALPHA.GE.1.AND.BETA.GE.1.AND.DELTA.GE.1) THEN
***         WRITE(*,13) NUINTG,ALPHA,BETA,DELTA
***      ELSEIF (ALPHA.GE.1.AND.BETA.GE.1) THEN
***         WRITE(*,14) NUINTG,ALPHA,BETA
***      ELSEIF (BETA.GE.1.AND.DELTA.GE.1) THEN
***         WRITE(*,15) NUINTG,BETA,DELTA
***      ENDIF
***    1 CONTINUE
***   11 FORMAT (I4,' : U',I1,',',I1,'V',I1,',',I1)
***   12 FORMAT (I4,' : U',I1,',',I1,'V',I1,'  ')
***   13 FORMAT (I4,' : U',I1,'V',I1,',',I1,'  ')
***   14 FORMAT (I4,' : U',I1,'V',I1,'    ')
***   15 FORMAT (I4,' : V',I1,:,',',I1)
***      END
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File History
* Version 4 : D.Martin (03 decembre 2010)
* - test pour integrands PML 187 < NUINTG < 228
* Version 3 : D.Martin (06 novembre 2009)
* - ajout d'un groupe (29e) d'integrands supplementaires 203 -> 208 :
*   integrands U_{alpha}V et UV_{alpha} V.Baronian et C.Chambeyron (02/09/2008)
* Version 2 : D.Martin (03 octobre 2005)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !IntNos
