*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE INTTAY (NUINTG,NDIM,NDMDOM,NBDLXC,NBDFXC,NBDLXL,NBDFXL
     &                  ,NBDLXG,NBDFXG,NDOM,LGTABB,KELFCC,KELFCL,TAYINT)
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Mars 1992)
* Derniere modification : D.Martin & C.Chambeyron (7 novembre 2002)
* Versio 1.0.1
*  
* Calcul de la taille d'une integrale elementaire et des indicateurs de
* calcul des fonctions de base et de leurs derivees pour cette integrale 
*  
*-- Argument d'entree --  
* NUINTG numero d'integrand
* NDIM   dimension d'espace
* NDMDOM dimension du domaine de calcul
* NBDLXC nombre max. de d.l. en colonne par element 'volumique'
* NBDFXC nombre max. de d.l. en colonne par face d'element 
* NBDLXL nombre max. de d.l. en ligne par element 'volumique'  
* NBDFXL nombre max. de d.l. en ligne par face d'element
* NBDLXG nombre max. de points par element 'volumique' 
* NBDFXG nombre max. de points par face d'element
*-- Argument de sortie -- 
* NDOM   dimension des tableaux de derivees des fonctions de base sur le domaine
*        de calcul en cours.
* LGTABB( 3) LGJACO, longueur de la matrice jacobienne
*       ( 4) LGJAJA, longueur de la matrice jacobienne inverse 
*                    multipliee par sa transposee
*       ( 5) LGFCTC, longueur du tableau des fonctions de base pour
*                    l'interpolation en colonne
*       ( 6) LGFCTL, idem pour l'interpolation en ligne
*       ( 8) LGDFCC, longueur du tableau des derivees des fonctions
*                    de base pour l'interpolation en colonne
*       ( 9) LGDFCL, idem pour l'interpolation en ligne
*       (10) LGDFCG, idem pour l'interpolation geometrique
* KECFCC indice de calcul des fonctions de base pour l'interp. en colonne
*       (1:1)='Y' ==> calcul des fonctions de base 
*       (2:2)='Y' ==> calcul des derivees des fonctions de base
*       (3:3)='Y' ==> calcul des derivees secondes des fct. de base
*       (4:4)='C' ==> calculs selon l'interpolation en 'colonne'
*             'G' ==> calculs selon l'interpolation geometrique
*       (5:5)='Y' ==> on conserve les calculs en tous les points
*                     de quadrature pour l'element suivant
*       (6:6)='Y' ==> implique le calcul de la matrice jacobienne dans
*                     tout l'element(integrands contenant une derivee)
*                     inutilise pour les calculs en dimension d'espace
* KELFCL indice de calcul des fonctions de base pour l'interp. en ligne
*       (i:i) , i=1,2,3 ==> cf. KELFCC 
*       (4:4)='L' ==> calculs selon l'interpolation en 'ligne' 
*             'G' ==> calculs selon l'interpolation geometrique
*             'C' ==> calculs selon l'interpolation 'colonne'  
*       (j:j) , j=5,6 ==> cf. KELFCC
* TAYINT taille de l'integrand comptee pour des inconnues scalaires
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) KELFCC,KELFCL
      INTEGER       NUINTG,NDIM,NDMDOM,NBDLXC,NBDFXC,NBDLXL,NBDFXL
     &             ,NBDLXG,NBDFXG,NDOM,LGTABB(*),TAYINT
*     Variables locales
      INTEGER       KC,INDEXC,KL,INDEXL,INSYME
*-----------------------------------------------------------------------
      CALL INTATR (NUINTG,KC,INDEXC,KL,INDEXL,INSYME)                   !integra
*
      IF (KC.EQ.-1) THEN
*
*        Cas d'une integrale uni-colonne (terme de second membre)
*
         IF (KL.EQ.0) THEN
            TAYINT     = NBDFXL
            LGTABB( 6) = NBDFXL
            KELFCL(1:1)= 'Y' 
         ELSEIF (KL.EQ.1) THEN
            KELFCL(2:2)='Y' 
            LGTABB( 3) = NDIM*NDIM
            IF (INDEXL.EQ.0) THEN
*              Integrand sans extension
               TAYINT     = NBDFXL 
               LGTABB( 9) = NBDFXL
               LGTABB(10) = NBDFXG  
            ELSEIF (INDEXL.EQ.1) THEN
*              Derivees des fonctions de base dans tout l'element
               NDOM       = NDIM
               KELFCL(6:6)='Y' 
               TAYINT     = NBDLXL
               LGTABB( 9) = NBDLXL
               LGTABB(10) = NBDLXG
            ENDIF
         ENDIF 
      ELSEIF (KL.EQ.-1) THEN
*
*        Cas d'un interpolant uni-ligne
*
         IF (KC.EQ.0) THEN
            TAYINT     = NBDFXC
            LGTABB( 5) = NBDFXC
            KELFCC(1:1)='Y' 
         ELSEIF (KC.EQ.1) THEN
            KELFCC(2:2)='Y'
            LGTABB( 3) = NDIM*NDIM  
            IF (INDEXC.EQ.0) THEN
               TAYINT     = NBDFXC 
               LGTABB( 8) = NBDFXC
               LGTABB(10) = NBDFXG
            ELSEIF (INDEXC.EQ.1) THEN
*              Derivees des fonctions de base dans tout l'element 
               NDOM       = NDIM
               KELFCC(6:6)='Y' 
               TAYINT     = NBDLXC
               LGTABB( 8) = NBDLXC
               LGTABB(10) = NBDLXG
            ENDIF 
         ENDIF 
      ELSE
*
*        Cas d'une integrale ligne et colonne (terme matriciel)
*
         LGTABB(3)=NDIM*NDIM
         IF (KC.EQ.0) THEN
            LGTABB( 5) = NBDFXC
            KELFCC(1:1)='Y' 
         ELSEIF (KC.EQ.1) THEN
*           Integrands contenant une derivee en colonne 
            KELFCC(2:2)='Y' 
            IF (INDEXC.EQ.0) THEN
*              Integrands ne necessitant pas d'extension
               LGTABB( 8) = NBDFXC
               LGTABB(10) = NBDFXG  
            ELSEIF (INDEXC.EQ.1) THEN
*              Derivees des fonctions de base dans tout l'element
               NDOM       = NDIM 
               KELFCC(6:6)='Y' 
               LGTABB( 8) = NBDLXC
               LGTABB(10) = NBDLXG
            ENDIF 
         ENDIF 
         IF (KL.EQ.0) THEN
            LGTABB( 6) = NBDFXL
            KELFCL(1:1)='Y' 
         ELSEIF (KL.EQ.1) THEN
*           Integrands contenant une derivee en ligne
            KELFCL(2:2)='Y' 
            IF (INDEXL.EQ.0) THEN
*              Integrands ne necessitant pas d'extension
               LGTABB( 9) = NBDFXL
               LGTABB(10) = NBDFXG  
            ELSEIF (INDEXL.EQ.1) THEN
*              Derivees des fonctions de base dans tout l'element
               NDOM       = NDIM  
               KELFCL(6:6)='Y' 
               LGTABB( 9) = NBDLXL
               LGTABB(10) = NBDLXG
           ENDIF
         ENDIF 
*        Longueur du tableau contenant la matrice elementaire associee  
         KC=MAX(0,KC)
         KL=MAX(0,KL)
         TAYINT=(1-KC)*(1-KL)*LGTABB( 5)*LGTABB( 6)
     &         +(1-KC)*KL    *LGTABB( 5)*LGTABB( 9)
     &         +KC    *(1-KL)*LGTABB( 8)*LGTABB( 6)
     &         +KC    *KL    *LGTABB( 8)*LGTABB( 9)
      ENDIF
                                                                    END !IntTay
