*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE NGRAWW (NBL,WL,NBC,DWC,NDIM,NORMAL
     &                  ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Juillet 1992)
* Derniere modification : D.Martin (28 juin 2005)
* Version 2
*
* Calcul de l'integrale elementaire dU/dn.V = \sum dUi/dn.Vi avec 
* inconnues vectorielles
*
*-- Arguments --
* NBL    nombre de lignes de l'integrand
* WL     fonctions de base pour l'interpolation en ligne
* NBC    nombre de colonnes de l'integrand
* DWC    derivees des fonctions de base pour l'interpolation en colonne
* NDIM   dimension d'espace
* NORMAL vecteur normal a l'image du point de quadrature dans l'elt. courant
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand ou du coeff. variable
* COFVAR coeff. variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coeff. variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP numero du fichier d'impression (pas d'impression si nul) .
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPE
      INTEGER       NBL,NBC,NDIM,IMP
      REAL          WL(*),DWC(NDIM,*),ELDIPQ,NORMAL(*)
      REAL          COFVAR,COEFFR(NDIM,NBL,NDIM,*)
      COMPLEX       COFVAC,COEFFC(NDIM,NBL,NDIM,*)
*
      INTEGER       J,J1,I
      REAL          RCF,DWJDN
      COMPLEX       CCF
*-----------------------------------------------------------------------
      IF (TYPE(1:1).EQ.'R') THEN
*
         DO 23 J=1,NBC
*           Calcul de dWj/dn
            DWJDN=0.
            DO 21 J1=1,NDIM
               DWJDN=DWJDN+ELDIPQ*DWC(J1,J)*NORMAL(J1)
21          CONTINUE
            DO 22 I=1,NBL
               COEFFR(1,I,1,J)=COEFFR(1,I,1,J)+COFVAR*DWJDN*WL(I)
22          CONTINUE
23       CONTINUE
         DO 26 J=1,NBC
            DO 25 I=1,NBL
               RCF=COEFFR(1,I,1,J)
               DO 24 J1=2,NDIM
                  COEFFR(J1,I,J1,J)=RCF
24             CONTINUE
25          CONTINUE
26       CONTINUE
*
      ELSE
*
         DO 43 J=1,NBC
            DWJDN=0.
            DO 41 J1=1,NDIM
               DWJDN=DWJDN+ELDIPQ*DWC(J1,J)*NORMAL(J1)
41          CONTINUE
            DO 42 I=1,NBL
               COEFFC(1,I,1,J)=COEFFC(1,I,1,J)+COFVAC*DWJDN*WL(I)
42          CONTINUE
43       CONTINUE
         DO 46 J=1,NBC
            DO 45 I=1,NBL
               CCF=COEFFC(1,I,1,J)
               DO 44 J1=2,NDIM
                  COEFFC(J1,I,J1,J)=CCF
44             CONTINUE
45          CONTINUE
46       CONTINUE
      ENDIF
*
      IF (IMP.GT.0)
     &   CALL PRINTV ('dU/dn.V (NGRAWW)'
     &               ,NDIM,NBL,NDIM,NBC,NBC,TYPE,COEFFR,COEFFC,IMP)
                                                                    END !NGraWW
