*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE PROTV (NBL,DWL,NBC,WC,NDIM
     &                 ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : G.Legendre (Mai 2002)
* Derniere modification : D.Martin (28 juin 2005)
* Version 2
*
* Calcul de l'integrale elementaire P rotV en dimension 2
*
*         a_{i1,i,j} = W_j ( -dT_i/dx2, dT_i/dx1)
*
* ou P   = p_j W_j, j=1,...,NBC (inconnue en colonne scalaire)
* et V_n = V_{n,i} T_i, i=1,...,NBL, n=1,NDIM (inconnue en ligne vectorielle)
*        = n-eme composante de V
*-- Arguments --
* NBL    nombre de lignes de la matrice elementaire
* DWL    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en ligne
*        (correspondant a la fonction-test vectorielle V)
* NBC    nombre de colonnes de la matrice elementaire
* WC     valeurs des fonctions de base pour l'interpolation encolonne
*        (correspondant a l'inconnue scalaire P)
* NDIM   dimension d'espace = nombre de composantes de V
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand ou du coeff. variable
* COFVAR coeff. variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coeff. variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    numero du fichier d'impression si non nul
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPE
      INTEGER       NBC,NBL,NDIM,IMP
      REAL          WC(*),DWL(NDIM,*),ELDIPQ
      REAL          COFVAR,COEFFR(NDIM,NBL,*)
      COMPLEX       COFVAC,COEFFC(NDIM,NBL,*)
*
      INTEGER       J,I,I1,IDER
      REAL          DWLI,ELTDIS
*-----------------------------------------------------------------------
      IF (TYPE(1:1).EQ.'R') THEN
*
         DO 23 J=1,NBC
            DO 22 I=1,NBL
               ELTDIS=-ELDIPQ
               DO 21 I1=1,NDIM
                  IDER=I1-(I1/NDIM)*NDIM+1
                  DWLI=ELTDIS*DWL(IDER,I)
                  ELTDIS=-ELTDIS
                  COEFFR(I1,I,J)=COEFFR(I1,I,J)+COFVAR*WC(J)*DWLI
21             CONTINUE
22          CONTINUE
23       CONTINUE
*
      ELSE
*
         DO 43 J=1,NBC
            DO 42 I=1,NBL
               ELTDIS=-ELDIPQ
               DO 41 I1=1,NDIM
                  IDER=I1-(I1/NDIM)*NDIM+1
                  DWLI=ELTDIS*DWL(IDER,I)
                  ELTDIS=-ELTDIS
                  COEFFC(I1,I,J)=COEFFC(I1,I,J)+COFVAC*WC(J)*DWLI
41             CONTINUE
42          CONTINUE
43       CONTINUE
*
      ENDIF
*
      IF (IMP.GT.0)
     &   CALL PRINTV ('P rotV (PROTV)'
     &               ,NDIM,NBL,1,NBC,NBC,TYPE,COEFFR,COEFFC,IMP)
                                                                    END !ProtV
