*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE PROTVP(NBL,WL,DWL,NBC,WC,IDX,NDIM
     &                 ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : G. Legendre (Mai 2002)
* Derniere modification : D.Martin (18 octobre 2006)
* Version 2
*
* Calcul de l'integrale elementaire P rotV en dimension 2 (cf. PROTV)
* modifie pour utilisation dans PML, dans lequel une ou des derivees
* partielles dw/dx_i sont remplacees par une direction pseudo-convective
*                  Alpha dw/dx_i + Lambda(i) w
*
*-- Arguments --
* NBL    nombre de lignes de la matrice elementaire
* WL     valeurs des fonctions de base pour l'interpolation en ligne
*        (correspondant a la fonction-test vectorielle V)
* DWL    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en ligne
*        (correspondant a la fonction-test vectorielle V)
* NBC    nombre de colonnes de la matrice elementaire
* WC     valeurs des fonctions de base pour l'interpolation encolonne
*        (correspondant a l'inconnue scalaire P)
* IDX    tableau indiquant la ou les directions i de la 'derivee convective'
*                   Alpha dw/dx_i + Lambda(i) w
* NDIM   dimension d'espace = nombre de composantes de V
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand ou du coeff. variable
* COFVAR coeff. variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coeff. variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    numero du fichier d'impression si non nul
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPE
      INTEGER       NBC,NBL,NDIM,IMP,IDX(*)
      REAL          WC(*),WL(*),DWL(NDIM,*),ELDIPQ
      REAL          COFVAR,COEFFR(NDIM,NBL,*)
      COMPLEX       COFVAC,COEFFC(NDIM,NBL,*)
*
*     Pas tres jojo, le COMMON, mais faute de mieux
      COMPLEX       ALP,LAM(3)
      COMMON        /PML/ALP,LAM
*
      INTEGER       J,I,I1,IDER
      REAL          ELTDIS
      COMPLEX       DWLI
*-----------------------------------------------------------------------
      IF (TYPE(1:1).EQ.'R') THEN
*
         CALL BAISE  ('Integrand UROTVP dans PML de type COMPLEXE !')
*
      ELSE
*
         DO 43 J=1,NBC
            DO 42 I=1,NBL
               ELTDIS=-ELDIPQ
               DO 41 I1=1,NDIM
                  IDER=I1-(I1/NDIM)*NDIM+1
                  IF (IDX(IDER).EQ.0) THEN
                     DWLI=ELTDIS*DWL(IDER,I)
                  ELSE
                     DWLI=ELTDIS*(ALP*DWL(IDER,I)-LAM(IDER)*WL(I))
                  ENDIF
                  ELTDIS=-ELTDIS
                  COEFFC(I1,I,J)=COEFFC(I1,I,J)+COFVAC*WC(J)*DWLI
41             CONTINUE
42          CONTINUE
43       CONTINUE
*
      ENDIF
*
      IF (IMP.GT.0)
     &   CALL PRINTV ('P rotV-PML (PROTVP)'
     &               ,NDIM,NBL,1,NBC,NBC,TYPE,COEFFR,COEFFC,IMP)
                                                                    END !ProtVp
