*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE ROTBT0 (MIXTE,NBL,DWL,NCL,NBC,DWC,NCC,NDIM
     &                  ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : O.DeBayser (Avril 1993)
* Derniere modification : D.Martin (28 juin 2005)
* Version 2
*
* Calcul de l'integrale elementaire pour inconnues vectorielles a 3 composantes
* en dimension 2,  obtenue pour Beta=0 dans l'integrale
*                       div_Beta(u) div_Beta(v)
*                  (du_3/dx_2  - Beta u_2 )
* ou rot_Beta(u) = (Beta u_1   - du_3/dx_1)
*                  (du_2/dx_1  - du_1/dx_2)
*
*                 |  dWi/dx2.dWj/dx2  -dWi/dx2.dWj/dx1          0         |
*                 |                                                       |
* a_{i1,i,j1,j} = | -dWi/dx1.dWj/dx2   dWi/dx1.dWj/dx1          0         |
*                 |                                      dWi/dx1.dWj/dx1  |
*                 |         0                 0         +dWi/dx2.dWj/dx2  |
*-- Arguments --
* MIXTE  indicateur de mixite des interpolations
* NBL nombre de lignes (comptees en noeuds) de la matrice elementaire
* DWL    derivees des fonctions a l'image du point de quadrature dans
*        l'element courant pour l'interpolation en ligne
* NCL nombre de composantes pour l'inconnue en ligne
* NBC nombre de colonnes (comptees en noeuds) de la matrice elementaire
* DWC    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en colonne
* NCC nombre de composantes pour l'inconnue en colonne
* NDIM   dimension d'espace
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de la matrice elementaire
* COFVAR coeff. variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coeff. variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPE
      INTEGER       MIXTE,NBL,NCL,NBC,NCC,NDIM,IMP
      REAL          DWL(NDIM,*),DWC(NDIM,*),ELDIPQ
      REAL          COFVAR,COEFFR(NCL,NBL,NCC,*)
      COMPLEX       COFVAC,COEFFC(NCL,NBL,NCC,*)
*
      INTEGER       J,J1,JDER,I,I1,IDER
      REAL          RX,RY,DWCJ,ELTDIS
      COMPLEX       CX,CY
*-----------------------------------------------------------------------
*
      IF (TYPE(1:1).EQ.'R') THEN
*
         DO 25 J1=1,NDIM
            DO 24 I1=1,NDIM
               ELTDIS=-ELDIPQ
               IF (J1.EQ.I1) ELTDIS=-ELTDIS
               JDER=J1-(J1/NDIM)*NDIM+1
               IDER=I1-(I1/NDIM)*NDIM+1
               DO 22 J=1,NBC
                  DWCJ=ELTDIS*DWC(JDER,J)
                  DO 21 I=1,J-1
                     RY=COFVAR*DWCJ*DWL(IDER,I)
                     IF (JDER.EQ.IDER) THEN
                        RX=COEFFR(NCL,I,NCC,J)
                        RX=RX+RY
                        COEFFR(NCL,I,NCC,J)=RX
                        COEFFR(NCC,J,NCL,I)=RX
                     ENDIF
                     RX=COEFFR(I1,I,J1,J)
                     RX=RX+RY
                     COEFFR(I1,I,J1,J)=RX
                     COEFFR(J1,J,I1,I)=RX
21                CONTINUE
22             CONTINUE
24          CONTINUE
25       CONTINUE
         DO 28 J1=1,NDIM
            DO 27 I1=1,J1
               ELTDIS=-ELDIPQ
               IF (J1.EQ.I1) ELTDIS=-ELTDIS
               JDER=J1-(J1/NDIM)*NDIM+1
               IDER=I1-(I1/NDIM)*NDIM+1
               DO 26 J=1,NBC
                  RY=COFVAR*ELTDIS*DWC(JDER,J)*DWL(IDER,J)
                  IF (JDER.EQ.IDER) THEN
                     RX=COEFFR(NCC,J,NCC,J)
                     RX=RX+RY
                     COEFFR(NCC,J,NCC,J)=RX
                     COEFFR(NCC,J,NCC,J)=RX
                  ENDIF
                  RX=COEFFR(I1,J,J1,J)
                  RX=RX+RY
                  COEFFR(I1,J,J1,J)=RX
                  COEFFR(J1,J,I1,J)=RX
26             CONTINUE
27          CONTINUE
28       CONTINUE
*
      ELSE
*
         DO 45 J1=1,NDIM
            DO 44 I1=1,NDIM
               ELTDIS=-ELDIPQ
               IF (J1.EQ.I1) ELTDIS=-ELTDIS
               JDER=J1-(J1/NDIM)*NDIM+1
               IDER=I1-(I1/NDIM)*NDIM+1
               DO 42 J=1,NBC
                  DWCJ = ELTDIS*DWC(JDER,J)
                  DO 41 I=1,J-1
                     CY=COFVAC*DWCJ*DWL(IDER,I)
                     IF (JDER.EQ.IDER) THEN
                        CX=COEFFC(NCL,I,NCC,J)
                        CX=CX+CY
                        COEFFC(NCL,I,NCC,J)=CX
                        COEFFC(NCC,J,NCL,I)=CX
                     ENDIF
                     CX=COEFFC(I1,I,J1,J)
                     CX=CX+CY
                     COEFFC(I1,I,J1,J)=CX
                     COEFFC(J1,J,I1,I)=CX
41                CONTINUE
42             CONTINUE
44          CONTINUE
45       CONTINUE
         DO 48 J1=1,NDIM
            DO 47 I1=1,J1
               ELTDIS=-ELDIPQ
               IF (J1.EQ.I1) ELTDIS=-ELTDIS
               JDER=J1-(J1/NDIM)*NDIM+1
               IDER=I1-(I1/NDIM)*NDIM+1
               DO 46 J=1,NBC
                  CY=COFVAC*ELTDIS*DWC(JDER,J)*DWL(IDER,J)
                  IF (JDER.EQ.IDER) THEN
                     CX=COEFFC(NCC,J,NCC,J)
                     CX=CX+CY
                     COEFFC(NCC,J,NCC,J)=CX
                     COEFFC(NCC,J,NCC,J)=CX
                  ENDIF
                  CX=COEFFC(I1,J,J1,J)
                  CX=CX+CY
                  COEFFC(I1,J,J1,J)=CX
                  COEFFC(J1,J,I1,J)=CX
46             CONTINUE
47          CONTINUE
48       CONTINUE
      ENDIF
*
      IF (IMP.GT.0) THEN
         J=NBC
         IF (MIXTE.EQ.0) J=0
         CALL PRINTV ('Rot_beta0(U) Rot_beta0(V) (ROTBT0)'
     &               ,NCL,NBL,NCC,NBC,J,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
                                                                   END  !Rotbt0
