*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE ROTBT1 (MIXTE,NBL,WL,NCL,NBC,DWC,NCC,NDIM
     &                  ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : O.DeBayser (Avril 1993)
* Derniere modification : D.Martin (28 juin 2005)
* Version 2
*
* Calcul de l'integrale elementaire pour inconnues vectorielles a 3 composantes
* en dimension 2, en facteur de Beta dans l'integrale
*                        rot_Beta(u).rot_Beta(v)
*                  (du_3/dx_2  - Beta u_2 )
* ou rot_Beta(u) = (Beta u_1   - du_3/dx_1)
*                  (du_2/dx_1  - du_1/dx_2) :
*
*                      |     0          0      Wi.dWj/dx1 |
*      a_{i1,i,j1,j} = |     0          0      Wi.dWj/dx2 |
*                      | dWi/dx1.Wj dWi/dx2.Wj     0      |
*-- Arguments --
* MIXTE  indicateur de mixite des interpolations
* NBL    nombre de lignes (comptees en noeuds) de la matrice elementaire
* WL     valeurs des fonctions de base au point de quadrature
* NCL    nombre de composantes pour l'inconnue en ligne
* NBC    nombre de colonnes (comptees en noeuds) de la matrice elementaire
* DWC    derivees des fonctions a l'image du point de quadrature dans
*        l'element courant pour l'interpolation en colonne
* NCC    nombre de composantes pour l'inconnue en colonne
* NDIM   dimension d'espace
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand ou du coeff. variable
* COFVAR coeff. variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coeff. variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    numero du fichier d'impression si non nul
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPE
      INTEGER       MIXTE,NBL,NCL,NBC,NCC,NDIM,IMP
      REAL          WL(*),DWC(NDIM,*),ELDIPQ
      REAL          COFVAR,COEFFR(NCL,NBL,NCC,*)
      COMPLEX       COFVAC,COEFFC(NCL,NBL,NCC,*)
*
      INTEGER       J,J1,I
      REAL          RX,DWCJDX
      COMPLEX       CX
*-----------------------------------------------------------------------
*
      IF (TYPE(1:1).EQ.'R') THEN
*
         DO 23 J1=1,NDIM
            DO 22 J=1,NBC
               DWCJDX=ELDIPQ*DWC(J1,J)
               DO 21 I=1,NBL
                  RX=COEFFR(J1,I,NCC,J)
                  RX=RX + COFVAR * DWCJDX * WL(I)
                  COEFFR(J1,I,NCC,J)=RX
                  COEFFR(NCL,J,J1,I)=RX
21             CONTINUE
22          CONTINUE
23       CONTINUE
*
      ELSE
*
         DO 43 J1=1,NDIM
            DO 42 J=1,NBC
               DWCJDX=ELDIPQ*DWC(J1,J)
               DO 41 I=1,NBL
                  CX=COEFFC(J1,I,NCC,J)
                  CX=CX + COFVAC * DWCJDX * WL(I)
                  COEFFC(J1,I,NCC,J)=CX
                  COEFFC(NCL,J,J1,I)=CX
41             CONTINUE
42          CONTINUE
43       CONTINUE
*
      ENDIF
*
      IF (IMP.GT.0) THEN
         J=NBC
         IF (MIXTE.EQ.0) J=0
         CALL PRINTV ('Rot_beta1(U) Rot_beta1(V) (ROTBT1)'
     &               ,NCL,NBL,NCC,NBC,0,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
                                                                    END !RotBt1
