*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE ROTPML (MIXTE,NBL,WL,DWL,NBC,WC,DWC,IDX,NDIM
     &                  ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : G. Legendre (Fevrier 2002)
* Derniere modification : D.Martin (18 octobre 2006)
* Version 1
*
* Calcul de l'integrale elementaire RotU.RotV (cf. ROTROT)
* modifie pour utilisation dans une PML, dans lequel une ou des derivees
* partielles dw/dx_i sont remplacees par une direction pseudo-convective
*                  Alpha dw/dx_i + Lambda(i) w
*
* Remarque : Les valeurs de Alpha et des Lambda sont transmises par COMMON
*
*-- Arguments --
* MIXTE  indicateur de mixite des interpolations
* NBL    nombre de lignes (comptees en noeuds) de la matrice elementaire
* WL     valeurs des fonctions de base a l'image du point de quadrature
*        dans l'element courant pour l'interpolation en ligne
* DWL    derivees des fonctions a l'image du point de quadrature dans
*        l'element courant pour l'interpolation en ligne
* NBC    nombre de colonnes (comptees en noeuds) de la matrice elementaire
* WC     valeurs des fonctions de base a l'image du point de quadrature
*        dans l'element courant pour l'interpolation en colonne
* DWC    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en colonne
* IDX    tableau indiquant la ou les directions i de la 'derivee convective'
*                   Alpha dw/dx_i + Lambda(i) w
* NDIM   dimension d'espace
* ELDIPQ element differentiel elementaire au point de quadrature
*        multiplie par le poids de quadrature
* TYPE   type (REEL ou COMPLEXE) de la matrice elementaire
* COFVAR coeff. variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coeff. variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    numero du fichier d'impression si non nul
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPE
      INTEGER       MIXTE,NBL,NBC,NDIM,IDX(*),IMP
      REAL          WL(*),WC(*)
      REAL          DWL(NDIM,*),DWC(NDIM,*),ELDIPQ
      REAL          COFVAR,COEFFR(NDIM,NBL,NDIM,*)
      COMPLEX       COFVAC,COEFFC(NDIM,NBL,NDIM,*)
*
*     Pas tres jojo, le COMMON, mais faute de mieux
      COMPLEX       ALP,LAM(3)
      COMMON        /PML/ALP,LAM
*
      INTEGER       I,J,I1,J1,NUT,NBTERM,IDER,JDER
      REAL          ELTDIS
      COMPLEX       CX,DWCJ
*-----------------------------------------------------------------------
*
      IF (TYPE(1:1).EQ.'R') THEN
*
         CALL BAISE  ('Integrand ROTROT dans PML de type COMPLEXE !')
         STOP
*
      ELSE
*
         DO 45 I1=1,NDIM
            DO 44 J1=1,NDIM
               NBTERM=1
               IDER=J1
               JDER=I1
               ELTDIS=-ELDIPQ
               IF (I1.EQ.J1) THEN
                  NBTERM=NDIM-1
                  ELTDIS=-ELTDIS
                  IDER=I1-(I1/NDIM)*NDIM+1
                  JDER=IDER
               ENDIF  
               DO 43 NUT=1,NBTERM  
                  DO 42 I=1,NBC
                     IF (IDX(IDER).EQ.0) THEN
                        DWCJ=ELTDIS*DWC(IDER,I)
                     ELSEIF (IDX(IDER).EQ.1) THEN
                        DWCJ=ELTDIS*(ALP*DWC(IDER,I)+LAM(IDER)*WC(I))
                     ENDIF
                     DO 41 J=1,NBL
                        CX=COEFFC(J1,J,I1,I)
                        IF (IDX(JDER).EQ.0) THEN
                           CX=CX+COFVAC*DWCJ*DWL(JDER,J)
                        ELSEIF (IDX(JDER).EQ.1) THEN
                           CX=CX+COFVAC
     &                        *DWCJ*(ALP*DWL(JDER,J)-LAM(JDER)*WL(J))
                        ELSE
                        ENDIF
                        COEFFC(J1,J,I1,I)=CX
 41                  CONTINUE
 42               CONTINUE
                  IDER=I1-((I1+1)/NDIM)*NDIM+2
                  JDER=IDER
 43            CONTINUE
 44         CONTINUE
 45      CONTINUE
*
      ENDIF
*
      IF (IMP.GT.0) THEN
         I=NBC
         IF (MIXTE.EQ.0) I=0
         CALL PRINTV ('RotU.RotV (ROTPML)'
     &               ,IDX,NDIM,NBL,NDIM,NBC,I,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
                                                                    END !RotPml
