*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE ROTROT (MIXTE,NBL,DWL,NBC,DWC,NDIM
     &                  ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Fevrier 1988)
* Derniere modification : D.Martin (28 juin 2005)
* Version 2
*
* Calcul de l'integrale elementaire Rot Wi . Rot Wj :
* en dimension 3 :
*                |  dWi/dx2.dWj/dx2  -dWi/dx2.dWj/dx1  -dWi/dx3.dWj/dx1 |
*                | +dWi/dx3.dWj/dx3                                     |
*                |                                                      |
* a_{i1,i,j1,j}= | -dWi/dx1.dWj/dx2   dWi/dx1.dWj/dx1  -dWi/dx3.dWj/dx2 |
*                |                   +dWi/dx3.dWj/dx3                   |
*                |                                                      |
*                | -dWi/dx1.dWj/dx3  -dWi/dx2.dWj/dx3   dWi/dx1.dWj/dx1 |
*                |                                     +dWi/dx2.dWj/dx2 |
* en dimension 2 :
*                |  dWi/dx2.dWj/dx2  -dWi/dx2.dWj/dx1 |
* a_{i1,i,j1,j}= |                                    |
*                | -dWi/dx1.dWj/dx2   dWi/dx1.dWj/dx1 |
*-- Arguments --
* MIXTE  indicateur de mixite des interpolations
* NBL    nombre de lignes (comptees en noeuds) de la matrice elementaire
* DWL    derivees des fonctions a l'image du point de quadrature dans
*        l'element courant pour l'interpolation en ligne
* NBC    nombre de colonnes (comptees en noeuds) de la matrice elementaire
* DWC    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en colonne
* NDIM   dimension d'espace
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de la matrice elementaire
* COFVAR coeff. variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coeff. variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    numero du fichier d'impression si non nul
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPE
      INTEGER       MIXTE,NBL,NBC,NDIM,IMP
      REAL          DWL(NDIM,*),DWC(NDIM,*),ELDIPQ
      REAL          COFVAR,COEFFR(NDIM,NBL,NDIM,*)
      COMPLEX       COFVAC,COEFFC(NDIM,NBL,NDIM,*)
*
      INTEGER       I,J,I1,J1,NUT,NBTERM,IDER,JDER
      REAL          RX,DWCJ,ELTDIS
      COMPLEX       CX
*-----------------------------------------------------------------------
*
      IF (TYPE(1:1).EQ.'R') THEN
*
         DO 25 I1=1,NDIM
            DO 24 J1=1,NDIM
               NBTERM=1
               IDER=J1
               JDER=I1
               ELTDIS=-ELDIPQ
               IF (I1.EQ.J1) THEN
                  ELTDIS=-ELTDIS
                  NBTERM=NDIM-1
                  IDER=I1-(I1/NDIM)*NDIM+1
                  JDER=IDER
               ENDIF
               DO 23 NUT=1,NBTERM
                  DO 22 I=1,NBC
                     DWCJ=ELTDIS*DWC(IDER,I)
                     DO 21 J=1,I-1
                        RX=COEFFR(J1,J,I1,I)
                        RX=RX+COFVAR*DWCJ*DWL(JDER,J)
                        COEFFR(J1,J,I1,I)=RX
                        COEFFR(I1,I,J1,J)=RX
21                   CONTINUE
                     IF (J1.LE.I1) THEN
                        RX=COEFFR(J1,I,I1,I)
                        RX=RX+COFVAR*DWCJ*DWL(JDER,I)
                        COEFFR(J1,I,I1,I)=RX
                        COEFFR(I1,I,J1,I)=RX
                     ENDIF
22                CONTINUE
                  IDER=I1-((I1+1)/NDIM)*NDIM+2
                  JDER=IDER
23             CONTINUE
24          CONTINUE
25       CONTINUE
*
      ELSE
*
         DO 45 I1=1,NDIM
            DO 44 J1=1,NDIM
               NBTERM=1
               IDER=J1
               JDER=I1
               ELTDIS=-ELDIPQ
               IF (I1.EQ.J1) THEN
                  NBTERM=NDIM-1
                  ELTDIS=-ELTDIS
                  IDER=I1-(I1/NDIM)*NDIM+1
                  JDER=IDER
               ENDIF  
               DO 43 NUT=1,NBTERM  
                  DO 42 I=1,NBC
                     DWCJ=ELTDIS*DWC(IDER,I)
                     DO 41 J=1,I-1
                        CX=COEFFC(J1,J,I1,I)
                        CX=CX+COFVAC*DWCJ*DWL(JDER,J)
                        COEFFC(J1,J,I1,I)=CX
                        COEFFC(I1,I,J1,J)=CX
41                   CONTINUE
                     IF (J1.LE.I1) THEN
                        CX=COEFFC(J1,I,I1,I)
                        CX=CX+COFVAC*DWCJ*DWL(JDER,I)
                        COEFFC(J1,I,I1,I)=CX
                        COEFFC(I1,I,J1,I)=CX
                     ENDIF
42                CONTINUE
                  IDER=I1-((I1+1)/NDIM)*NDIM+2
                  JDER=IDER
43             CONTINUE
44          CONTINUE
45       CONTINUE
*
      ENDIF
*
      IF (IMP.GT.0) THEN
         I=NBC
         IF (MIXTE.EQ.0) I=0
         CALL PRINTV ('RotU.RotV (ROTROT)'
     &               ,NDIM,NBL,NDIM,NBC,I,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
                                                                    END !RotRot
