*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE ROTUQP (NBL,WL,NBC,WC,DWC,IDX,NDIM
     &                  ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : G.Legendre (9 Juillet 2002)
* Derniere modification : D.Martin (18 octobre 2006)
* Version 2
*
* Calcul des integrales elementaires rotU Q en dimension 2 (cf. ROTUQ)
* modifie pour utilisation dans PML, dans lequel une ou des derivees
* partielles dw/dx_i sont remplacees par une direction pseudo-convective
*                  Alpha dw/dx_i + Lambda(i) w
*
*-- Arguments --
* NBL    nombre de lignes de la matrice elementaire
* WL     valeurs des fonctions de base pour l'interpolation en ligne
*        (correspondant a la fonction-test scalaire Q)
* NBC    nombre de colonnes de la matrice elementaire
* WC     valeurs des fonctions de base pour l'interpolation en colonne
*        (correspondant a l'inconnue vectorielle U)
* DWC    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en colonne
*        (correspondant a l'inconnue vectorielle U)
* IDX    tableau indiquant la ou les directions i de la 'derivee convective'
*                   Alpha dw/dx_i + Lambda(i) w
* NDIM   dimension d'espace = nombre de composantes de U
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand ou du coeff. variable
* COFVAR coeff. variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coeff. variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    numero du fichier d'impression si non nul
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPE
      INTEGER       NBL,NBC,NDIM,IMP,IDX(*)
      REAL          WL(*),WC(*),DWC(NDIM,*),ELDIPQ
      REAL          COFVAR,COEFFR(NBL,NDIM,*)
      COMPLEX       COFVAC,COEFFC(NBL,NDIM,*)
*
*     Pas tres jojo, le COMMON, mais faute de mieux
      COMPLEX       ALP,LAM(3)
      COMMON        /PML/ALP,LAM
*
      INTEGER       J1,J,I,JDER
      REAL          ELTDIS
      COMPLEX       DWCJ
*-----------------------------------------------------------------------
      IF (TYPE(1:1).EQ.'R') THEN
*
         CALL BAISE  ('Integrand ROTUQP dans PML de type COMPLEXE !')
         STOP
*
      ELSE
*
         DO 44 J=1,NBC
            ELTDIS=-ELDIPQ
            DO 43 J1=1,NDIM
               JDER=J1-(J1/NDIM)*NDIM+1
               IF (IDX(JDER).EQ.0) THEN
                  DWCJ=ELTDIS*DWC(JDER,J)
               ELSE
                  DWCJ=ELTDIS*(ALP*DWC(JDER,J)+LAM(JDER)*WC(J))
               ENDIF
               ELTDIS=-ELTDIS
               DO 42 I=1,NBL
                  COEFFC(I,J1,J)=COEFFC(I,J1,J)+COFVAC*DWCJ*WL(I)
42             CONTINUE
43          CONTINUE
44       CONTINUE
*
      ENDIF
*
      IF (IMP.GT.0)
     &   CALL PRINTV ('rotU Q-PML (ROTUQP)'
     &               ,1,NBL,NDIM,NBC,NBC,TYPE,COEFFR,COEFFC,IMP)
                                                                    END !RotUQp
