*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE ROTUV (MIXTE,NBL,WL,NBC,DWC,NDIM
     &                 ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*  Auteur : C.Chambeyron (19 decembre 2010)
*  Version 0
* 
*  Calcul des integrales elementaires rotU.V en dimension 3 :
*                       |      0   -dW_j/dx3  dW_j/dx2 |
*   a_{i1,i,j1,j} =     | dW_j/dx3    0      -dW_j/dx1 | W_i
*                       |-dW_j/dx2  dW_j/dx1      0    |
*
*  ou U_n = U_{n,j} W_j, j=1,...,NBC, n=1,NDIM (inconnue en colonne vectorielle)
*         = n-eme composante de U
*  et V_n = V_{n,i} W_i, i=1,...,NBL, n=1,NDIM (inconnue en ligne vectorielle)
*         = n-eme composante de V
* -- Arguments --
*  MIXTE  indicateur de mixite des interpolations
*  NBL    nombre de lignes de la matrice elementaire
*  WL     valeurs des fonctions de base pour l'interpolation en LIGNE
*  NBC    nombre de colonnes de la matrice elementaire
*  DWL    derivees des fonctions a l'image du point quadrature dans
*         l'element courant pour l'interpolation en colonne
*         (correspondant a la fonction-test vectorielle V)
*  NDIM   dimension d'espace = nombre de composantes de V
*  ELDIPQ poids de quadrature multiplie par l'element differentiel
*  TYPE   type (REEL ou COMPLEXE) de l'integrand ou du coeff. variable
*  COFVAR coeff. variable (cas REEL)
*  COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*         de quadrature (cas REEL)
*  COFVAC coeff. variable (cas COMPLEXE)
*  COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*         de quadrature (cas COMPLEXE)
*  IMP    numero du fichier d'impression si non nul
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPE
      INTEGER       MIXTE,NBC,NBL,NDIM,IMP
      REAL          WL(*),DWC(NDIM,*),ELDIPQ
      REAL          COFVAR,COEFFR(NDIM,NBL,NDIM,*)
      COMPLEX       COFVAC,COEFFC(NDIM,NBL,NDIM,*)
 
      INTEGER       I,J,I1,J1
      REAL          RX
      COMPLEX       CX
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IF (TYPE(1:1).EQ.'R') THEN
 
         DO 23 J=1,NBC
            DO 22 I=1,NBL
               RX=COFVAR*WL(I)*ELDIPQ
               COEFFR(1,I,2,J)=COEFFR(1,I,2,J)-RX*DWC(3,J)
               COEFFR(1,I,3,J)=COEFFR(1,I,3,J)+RX*DWC(2,J)
               COEFFR(2,I,1,J)=COEFFR(2,I,1,J)+RX*DWC(3,J)
               COEFFR(2,I,3,J)=COEFFR(2,I,3,J)-RX*DWC(1,J)
               COEFFR(3,I,1,J)=COEFFR(3,I,1,J)-RX*DWC(2,J)
               COEFFR(3,I,2,J)=COEFFR(3,I,2,J)+RX*DWC(1,J)
22          CONTINUE                                    
23       CONTINUE
      ELSE
         DO 43 J=1,NBC
            DO 42 I=1,NBL
               CX=COFVAC*WL(I)*ELDIPQ
               COEFFC(1,I,2,J)=COEFFC(1,I,2,J)-CX*DWC(3,J)
               COEFFC(1,I,3,J)=COEFFC(1,I,3,J)+CX*DWC(2,J)
               COEFFC(2,I,1,J)=COEFFC(2,I,1,J)+CX*DWC(3,J)
               COEFFC(2,I,3,J)=COEFFC(2,I,3,J)-CX*DWC(1,J)
               COEFFC(3,I,1,J)=COEFFC(3,I,1,J)-CX*DWC(2,J)
               COEFFC(3,I,2,J)=COEFFC(3,I,2,J)+CX*DWC(1,J)
42          CONTINUE                                    
43       CONTINUE
      ENDIF
*
      IF (IMP.GT.0) THEN
         I=NBC
         IF (MIXTE.EQ.0) I=0
         CALL PRINTV ('RotU.V (ROTUV)'
     &               ,NDIM,NBL,NDIM,NBC,I,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File History
* Version 0 : C. Chambeyron & G. Legendre (19 decembre 2010)
* - fabrique a partir de l'integrand transpose URotV
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !RotUV
