*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE ROTUVP (MIXTE,NBL,WL,NBC,WC,DWC,IDX,NDIM
     &                 ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*  Auteur : C.Chambeyron (9 decembre 2010)
*  Version 1
* 
*  Calcul des integrales elementaires U.rotV en dimension 3 :
*                       |      0      dW_j/dx3  -dW_j/dx2 |
*   a_{i1,i,j1,j} =     | -dW_j/dx3       0      dW_j/dx1 | W_i
*                       |  dW_j/dx2  -dW_j/dx1       0    |
*
*  ou U_n = U_{n,j} W_j, j=1,...,NBC, n=1,NDIM (inconnue en colonne vectorielle)
*         = n-eme composante de U
*  et V_n = V_{n,i} W_i, i=1,...,NBL, n=1,NDIM (inconnue en ligne vectorielle)
*         = n-eme composante de V
*
*  pour utilisation dans une PML,  une ou des derivees partielles dw/dx_i
*  sont remplacees par une direction pseudo-convective
*                  Alpha dw/dx_i + Lambda(i) w
* -- Arguments --
*  MIXTE  indicateur de mixite des interpolations
*  NBL    nombre de lignes de la matrice elementaire
*  DWL    derivees des fonctions a l'image du point quadrature dans
*         l'element courant pour l'interpolation en ligne
*         (correspondant a la fonction-test vectorielle V)
*  NBC    nombre de colonnes de la matrice elementaire
*  WC     valeurs des fonctions de base pour l'interpolation encolonne
*         (correspondant a l'inconnue scalaire P)
*  NDIM   dimension d'espace = nombre de composantes de V
*  ELDIPQ poids de quadrature multiplie par l'element differentiel
*  TYPE   type (REEL ou COMPLEXE) de l'integrand ou du coeff. variable
*  COFVAR coeff. variable (cas REEL)
*  COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*         de quadrature (cas REEL)
*  COFVAC coeff. variable (cas COMPLEXE)
*  COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*         de quadrature (cas COMPLEXE)
*  IMP    numero du fichier d'impression si non nul
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPE
      INTEGER       MIXTE,NBC,NBL,NDIM,IMP,IDX(*)
      REAL          WC(*),DWC(NDIM,*),ELDIPQ,WL(*)
      REAL          COFVAR,COEFFR(NDIM,NBL,NDIM,*)
      COMPLEX       COFVAC,COEFFC(NDIM,NBL,NDIM,*)
 
      INTEGER       I,J,I1,J1,JDER
      COMPLEX       CX,DWCJ(NDIM)
*     Pas tres jojo, le COMMON, mais faute de mieux
      COMPLEX       ALP,LAM(3)
      COMMON        /PML/ALP,LAM
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IF (TYPE(1:1).EQ.'R') THEN
*
         CALL BAISE  ('Integrand ROTUVP dans PML de type COMPLEXE !')
         STOP
* 
      ELSE
*      
         DO 43 J=1,NBC
            DO 41 JDER=1,NDIM
               IF (IDX(JDER).EQ.0) THEN
                  DWCJ(JDER)=DWC(JDER,J)
               ELSEIF (IDX(JDER).EQ.1) THEN
                  DWCJ(JDER)=(ALP*DWC(JDER,J)+LAM(JDER)*WC(J))
               ENDIF
41          CONTINUE
            DO 42 I=1,NBL
               CX=COFVAC*WL(I)*ELDIPQ 
               COEFFC(1,I,2,J)=COEFFC(1,I,2,J)-CX*DWCJ(3)
               COEFFC(1,I,3,J)=COEFFC(1,I,3,J)+CX*DWCJ(2)
               COEFFC(2,I,1,J)=COEFFC(2,I,1,J)+CX*DWCJ(3)
               COEFFC(2,I,3,J)=COEFFC(2,I,3,J)-CX*DWCJ(1)
               COEFFC(3,I,1,J)=COEFFC(3,I,1,J)-CX*DWCJ(2)
               COEFFC(3,I,2,J)=COEFFC(3,I,2,J)+CX*DWCJ(1)  
42          CONTINUE                                
43       CONTINUE
      ENDIF
*
      IF (IMP.GT.0) THEN
         I=NBC
         IF (MIXTE.EQ.0) I=0
         CALL PRINTV ('rotU.V (ROTUV)'
     &               ,NDIM,NBL,NDIM,NBC,I,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File History
* Version 0 : C. Chambeyron & G. Legendre (decembre 2010)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !rotuvp
