*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE UROTV (MIXTE,NBL,DWL,NBC,WC,NDIM
     &                 ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : G. Legendre (16 juillet 2009)
* Version 0
*
* Calcul des integrales elementaires U.rotV en dimension 3 :
*                      |      0      dW_i/dx3  -dW_i/dx2 |
*  a_{i1,i,j1,j} = W_j | -dW_i/dx3       0      dW_i/dx1 |
*                      |  dW_i/dx2  -dW_i/dx1       0    |
*
* ou U_n = U_{n,j} W_j, j=1,...,NBC, n=1,NDIM (inconnue en colonne vectorielle)
*        = n-eme composante de U
* et V_n = V_{n,i} W_i, i=1,...,NBL, n=1,NDIM (inconnue en ligne vectorielle)
*        = n-eme composante de V
*-- Arguments --
* MIXTE  indicateur de mixite des interpolations
* NBL    nombre de lignes de la matrice elementaire
* DWL    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en ligne
*        (correspondant a la fonction-test vectorielle V)
* NBC    nombre de colonnes de la matrice elementaire
* WC     valeurs des fonctions de base pour l'interpolation encolonne
*        (correspondant a l'inconnue scalaire P)
* NDIM   dimension d'espace = nombre de composantes de V
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand ou du coeff. variable
* COFVAR coeff. variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coeff. variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    numero du fichier d'impression si non nul
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPE
      INTEGER       MIXTE,NBC,NBL,NDIM,IMP
      REAL          WC(*),DWL(NDIM,*),ELDIPQ
      REAL          COFVAR,COEFFR(NDIM,NBL,NDIM,*)
      COMPLEX       COFVAC,COEFFC(NDIM,NBL,NDIM,*)
*
      INTEGER       I,J,I1,J1
      REAL          RX
      COMPLEX       CX
*-----------------------------------------------------------------------
      IF (TYPE(1:1).EQ.'R') THEN
*
         DO 23 J=1,NBC
            DO 22 I=1,NBL
               RX=COFVAR*WC(J)*ELDIPQ
               COEFFR(1,I,2,J)=COEFFR(1,I,2,J)+RX*DWL(3,I)
               COEFFR(1,I,3,J)=COEFFR(1,I,3,J)-RX*DWL(2,I)
               COEFFR(2,I,1,J)=COEFFR(2,I,1,J)-RX*DWL(3,I)
               COEFFR(2,I,3,J)=COEFFR(2,I,3,J)+RX*DWL(1,I)
               COEFFR(3,I,1,J)=COEFFR(3,I,1,J)+RX*DWL(2,I)
               COEFFR(3,I,2,J)=COEFFR(3,I,2,J)-RX*DWL(1,I)
22          CONTINUE
23       CONTINUE
*
      ELSE
*
         DO 43 J=1,NBC
            DO 42 I=1,NBL
               CX=COFVAC*WC(J)*ELDIPQ
               COEFFC(1,I,2,J)=COEFFC(1,I,2,J)+CX*DWL(3,I)
               COEFFC(1,I,3,J)=COEFFC(1,I,3,J)-CX*DWL(2,I)
               COEFFC(2,I,1,J)=COEFFC(2,I,1,J)-CX*DWL(3,I)
               COEFFC(2,I,3,J)=COEFFC(2,I,3,J)+CX*DWL(1,I)
               COEFFC(3,I,1,J)=COEFFC(3,I,1,J)+CX*DWL(2,I)
               COEFFC(3,I,2,J)=COEFFC(3,I,2,J)-CX*DWL(1,I)
42          CONTINUE
43       CONTINUE
*
      ENDIF
*
      IF (IMP.GT.0) THEN
         I=NBC
         IF (MIXTE.EQ.0) I=0
         CALL PRINTV ('U.RotV (UROTV)'
     &               ,NDIM,NBL,NDIM,NBC,I,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File History
* Version 0 : G. Legendre (16 juillet 2009)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !UrotV
