*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE UROTVP (MIXTE,NBL,WL,DWL,NBC,WC,IDX,NDIM
     &                 ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*  Auteur : C. Chambeyron (decembre 2010)
*  Version 1
* 
*  Calcul des integrales elementaires U.rotV en dimension 3 :
*                       |      0      dW_i/dx3  -dW_i/dx2 |
*   a_{i1,i,j1,j} = W_j | -dW_i/dx3       0      dW_i/dx1 |
*                       |  dW_i/dx2  -dW_i/dx1       0    |
*
*  ou U_n = U_{n,j} W_j, j=1,...,NBC, n=1,NDIM (inconnue en colonne vectorielle)
*         = n-eme composante de U
*  et V_n = V_{n,i} W_i, i=1,...,NBL, n=1,NDIM (inconnue en ligne vectorielle)
*         = n-eme composante de V
*
*  pour utilisation dans une PML,  une ou des derivees partielles dw/dx_i
*  sont remplacees par une direction pseudo-convective
*                  Alpha dw/dx_i + Lambda(i) w
*
* Remarque : Les valeurs de Alpha et des Lambda sont transmises par COMMON
*
* -- Arguments --
*  MIXTE  indicateur de mixite des interpolations
*  NBL    nombre de lignes de la matrice elementaire
*  DWL    derivees des fonctions a l'image du point quadrature dans
*         l'element courant pour l'interpolation en ligne
*         (correspondant a la fonction-test vectorielle V)
*  NBC    nombre de colonnes de la matrice elementaire
*  WC     valeurs des fonctions de base pour l'interpolation encolonne
*         (correspondant a l'inconnue scalaire P)
*  NDIM   dimension d'espace = nombre de composantes de V
*  ELDIPQ poids de quadrature multiplie par l'element differentiel
*  TYPE   type (REEL ou COMPLEXE) de l'integrand ou du coeff. variable
*  COFVAR coeff. variable (cas REEL)
*  COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*         de quadrature (cas REEL)
*  COFVAC coeff. variable (cas COMPLEXE)
*  COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*         de quadrature (cas COMPLEXE)
*  IMP    numero du fichier d'impression si non nul
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPE
      INTEGER       MIXTE,NBC,NBL,NDIM,IMP,IDX(*)
      REAL          WC(*),DWL(NDIM,*),ELDIPQ, WL(*)
      REAL          COFVAR,COEFFR(NDIM,NBL,NDIM,*)
      COMPLEX       COFVAC,COEFFC(NDIM,NBL,NDIM,*)
 
      INTEGER       I,J,I1,J1,IDER
      COMPLEX       CX,DWLI(NDIM)
      COMPLEX       ALP,LAM(3)
      COMMON        /PML/ALP,LAM
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IF (TYPE(1:1).EQ.'R') THEN
*
         CALL BAISE  ('Integrand UROTVP dans PML de type COMPLEXE !')
         STOP
*
      ELSE
*
         DO 43 I=1,NBL
            DO 41 IDER=1,NDIM
               IF (IDX(IDER).EQ.0) THEN
                  DWLI(IDER)=DWL(IDER,I)
               ELSEIF (IDX(IDER).EQ.1) THEN
                  DWLI(IDER)=(ALP*DWL(IDER,I)+LAM(IDER)*WL(I))
               ENDIF
41          CONTINUE
            DO 42 J=1,NBC
               CX=COFVAC*WC(J)*ELDIPQ   
               COEFFC(1,I,2,J)=COEFFC(1,I,2,J)+CX*DWLI(3)
               COEFFC(1,I,3,J)=COEFFC(1,I,3,J)-CX*DWLI(2)
               COEFFC(2,I,1,J)=COEFFC(2,I,1,J)-CX*DWLI(3)
               COEFFC(2,I,3,J)=COEFFC(2,I,3,J)+CX*DWLI(1)
               COEFFC(3,I,1,J)=COEFFC(3,I,1,J)+CX*DWLI(2)
               COEFFC(3,I,2,J)=COEFFC(3,I,2,J)-CX*DWLI(1)
42          CONTINUE                                
43       CONTINUE
 
      ENDIF
*
      IF (IMP.GT.0) THEN
         I=NBC
         IF (MIXTE.EQ.0) I=0
         CALL PRINTV ('U.rotV (UROTV)'
     &               ,NDIM,NBL,NDIM,NBC,I,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File History
* Version 0 : C. Chambeyron & G. Legendre (decembre 2010)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !UrotVP
