*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE WAGWAG (GAMMA,ALPHA,NBL,DWL,NCL,NBC,DWC,NCC,NDIM
     &                  ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (juillet 2002)
* Derniere modification : D.Martin (28 juin 2005)
* Version 2
*
* Calcul de l'integrale elementaire U_{alpha,gamma} V_{alpha,gamma}
* pour inconnues vectorielles
*
* a_{i1,i,j1,j} = delta_{i1,alpha}delta_{j1,alpha} dWi/dx_{gamma}.dWj/dx_{gamma}
*
* pour le calcul de l'integrale
*              Int_K [ du_{alpha}/dx{gamma} dv_{alpha}/dx_{gamma} ] dx
*
*-- Arguments --
* GAMMA  GAMMA dans les formules ci-dessus
* ALPHA  ALPHA dans les formules ci-dessus
* NBL    nombre de lignes de la matrice elementaire
* DWL    derivees des fonctions a l'image du point de quadrature dans
*        l'element courant pour l'interpolation en ligne
* NCL    nombre de composantes de la fonction-test
* NBC    nombre de colonnes de la matrice elementaire
* DWC    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en colonne
* NCC    nombre de composantes de l'inconnue
* NDIM   dimension d'espace
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand
* COFVAR coefficient variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coefficient variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    unite logique d'impression (si > 0)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPE
      INTEGER       ALPHA,GAMMA,NBL,NCL,NBC,NCC,NDIM,IMP
      REAL          DWL(NDIM,*),DWC(NDIM,*),ELDIPQ
     &             ,COFVAR,COEFFR(NCL,NBL,NCC,*)
      COMPLEX       COFVAC,COEFFC(NCL,NBL,NCC,*)
*
      INTEGER       I,J
      REAL          RX
      COMPLEX       CX
      CHARACTER     ALPHAC,GAMMAC
*-----------------------------------------------------------------------
*
      IF (TYPE(1:1).EQ.'R') THEN
*
         DO 22 J=1,NBC
            DO 21 I=1,J
               RX = COEFFR(ALPHA,I,ALPHA,J)
               RX = RX + COFVAR * ELDIPQ * DWC(GAMMA,J) * DWL(GAMMA,I)
               COEFFR(ALPHA,I,ALPHA,J) = RX
               COEFFR(ALPHA,J,ALPHA,I) = RX
21          CONTINUE
22       CONTINUE
*
      ELSE
*
         DO 42 J=1,NBC
            DO 41 I=1,J
               CX = COEFFC(ALPHA,I,ALPHA,J)
               CX = CX + COFVAC * ELDIPQ * DWC(GAMMA,J) * DWL(GAMMA,I)
               COEFFC(ALPHA,I,ALPHA,J) = CX
               COEFFC(ALPHA,J,ALPHA,I) = CX
41          CONTINUE
42        CONTINUE
*
      ENDIF
*
      IF (IMP.GT.0) THEN
         CALL ENCLER (ALPHA,ALPHAC)
         CALL ENCLER (GAMMA,GAMMAC)
         CALL PRINTV 
     &  ('U'//ALPHAC//','//GAMMAC//'V'//ALPHAC//','//GAMMAC//' (WagWag)'
     &  ,NCL,NBL,NCC,NBC,0,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
                                                                    END !WagWag
