*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE WAWB (MIXTE,BETA,NBL,WL,NCL,ALPHA,NBC,WC,NCC
     &                ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (4 mai 2004)
* Derniere modification : D.Martin (2 septembre 2008)
* Version 3
*
* Calcul de l'integrale elementaire pour inconnues vectorielles
*
* a_{i1,i,j1,j} = delta_[i1,beta} delta_{j1,alpha} WiWj
*
* pour le calcul de l'integrale
*              Int_K [ u_{alpha} v_{beta} ] dx
*
*-- Arguments --
* MIXTE  indicateur de mixite des interpolations
* BETA   beta dans les formules ci-dessus
* NBL    nombre de lignes de la matrice elementaire
* WL     fonctions de base au point de quadrature pour l'interpolation
*        en ligne
* ALPHA  alpha dans les formules ci-dessus
* NBC    nombre de colonnes de la matrice elementaire
* WC     fonctions de base au point de quadrature pour l'interpolation
*        en colonne
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand
* COFVAR coefficient variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coefficient variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    unite logique d'impression (si > 0)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPE
      INTEGER       MIXTE,BETA,NBL,NCL,ALPHA,NBC,NCC,IMP
      REAL          WL(*),WC(*),ELDIPQ
     &             ,COFVAR,COEFFR(NCL,NBL,NCC,*)
      COMPLEX       COFVAC,COEFFC(NCL,NBL,NCC,*)
*
      INTEGER       I,J
      CHARACTER     ALPHAC,BETAC
*-----------------------------------------------------------------------
*
      IF (TYPE(1:1).EQ.'R') THEN
*
         DO 22 J=1,NBC
            DO 21 I=1,NBL
               COEFFR(BETA,I,ALPHA,J) = COEFFR(BETA,I,ALPHA,J)
     &                                + COFVAR * ELDIPQ * WC(J) * WL(I)
 21         CONTINUE
 22      CONTINUE
*
      ELSE
*
         DO 42 J=1,NBC
            DO 41 I=1,NBL
               COEFFC(BETA,I,ALPHA,J) = COEFFC(BETA,I,ALPHA,J)
     &                                + COFVAC * ELDIPQ * WC(J) * WL(I)
 41         CONTINUE
 42       CONTINUE
      ENDIF
*
      IF (IMP.GT.0) THEN
         CALL ENCLER (ALPHA,ALPHAC)
         CALL ENCLER (BETA,BETAC)
         CALL PRINTV ('U'//ALPHAC//'V'//BETAC//' (WaWb)'
     &               ,NCL,NBL,NCC,NBC,NBC,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 3 : D.Martin (2 sep. 2008)
* - bug boucle 41 I = 1,J remplacé par I = 1,NBL (non symétrique)
* Version 2 : D.Martin (3 oct. 2005)
* Version 0 : D.Martin (4 mai 2004)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !WaWb
