*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE WAWBD  (DELTA,BETA,NBL,DWL,NCL,ALPHA,NBC,WC,NCC,NDIM
     &                  ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (juin 2003)
* Derniere modification : D.Martin (28 juin 2005)
* Version 2
*
* Calcul de l'integrale elementaire U_{alpha} V_{beta,delta}
* pour inconnues vectorielles
*
* a_{i1,i,j1,j} = delta_{i1,alpha}delta_{j1,beta} dWi/dx_{delta}.Wj
*
* pour le calcul de l'integrale
*                 Int_K [ u_{alpha} dv_{beta}/dx{delta} ] dx
*
*-- Arguments --
* DELTA  DELTA (derivee partielle) dans les formules ci-dessus
* BETA   BETA dans les formules ci-dessus (num. de composante de la fonc-test) 
* NBL    nombre de lignes de la matrice elementaire
* DWL    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en ligne
* NCL    nombre de composantes de la fonction-test
* ALPHA  ALPHA dans les formules ci-dessus (num. de composante de l'inconnue) 
* NBC    nombre de colonnes de la matrice elementaire
* WC     fonctions a l'image du point de quadrature dans
*        l'element courant pour l'interpolation en colonne
* NCC    nombre de composantes de l'inconnue
* NDIM   dimension d'espace
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand
* COFVAR coefficient variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coefficient variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    unite logique d'impression (si > 0)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPE
      INTEGER       DELTA,ALPHA,BETA,NBL,NCL,NBC,NCC,NDIM,IMP
      REAL          WC(*),DWL(NDIM,*),ELDIPQ
     &             ,COFVAR,COEFFR(NCL,NBL,NCC,*)
      COMPLEX       COFVAC,COEFFC(NCL,NBL,NCC,*)
*
      INTEGER       I,J
      CHARACTER     DELTAC,ALPHAC,BETAC
*-----------------------------------------------------------------------
*
      IF (TYPE(1:1).EQ.'R') THEN
*
         DO 22 J=1,NBC
            DO 21 I=1,NBL
               COEFFR(BETA,I,ALPHA,J) = COEFFR(BETA,I,ALPHA,J) 
     &                          + COFVAR * ELDIPQ * WC(J) * DWL(DELTA,I)
21          CONTINUE
22       CONTINUE
*
      ELSE
*
         DO 42 J=1,NBC
            DO 41 I=1,NBL
               COEFFC(BETA,I,ALPHA,J) = COEFFC(BETA,I,ALPHA,J) 
     &                          + COFVAC * ELDIPQ * WC(J) * DWL(DELTA,I)
41          CONTINUE
42        CONTINUE
*
      ENDIF
*
      IF (IMP.GT.0) THEN
         CALL ENCLER (DELTA,DELTAC)
         CALL ENCLER (ALPHA,ALPHAC)
         CALL ENCLER (BETA,BETAC)
         CALL PRINTV 
     &  ('U'//ALPHAC//' V'//BETAC//','//DELTAC//' (WaWbd)'
     &  ,NCL,NBL,NCC,NBC,NBC,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
                                                                    END !WaWbd
