*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE WAWVEC (MIXTE,NBL,WL,NCL,NBC,WC,NCC
     &                  ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (25 Avril 2002)
* Derniere modification : D.Martin (28 juin 2005)
* Version 2
*
* Calcul de l'integrale elementaire: U.AV avec inconnues vectorielles 2D et 3D
* ou A est une matrice 2x2 en 2d ou 3x3 en 3d (cf. WWIVEC)
*
*    a_{i1,i,j1,j} = A_{i1,j1} Wj Ti
*
* ou U_j1 = u_j1,j W_j ; j=1,...,NBC , j1=1,...,NCC (inconnue vect. en colonne)
*    V_i1 = v_i1,i T_i ; j=1,...,NBL , i1=1,...,NCL (inconnue vect. en ligne)
*
*-- Arguments --
* NBL    nombre de lignes (comptee en noeuds) de l'integrand
* WL     valeurs des fonctions de base au point de quadrature
*        pour l'interpolation en ligne (T_i)
* NCL    nombre de composantes de l'inconnue en ligne
* NBC    nombre de colonnes (comptee en noeuds) de l'integrand
* WC     valeurs des fonctions de base au point de quadrature
*        pour l'interpolation en colonne (W_j)
* NCC    nombre de composantes de l'inconnue en colonne
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand
* COFVAR tableau contenant la matrice des coefficients variables (cas REEL)
*        range par colonne dans l'ordre fortran : a11, a21, [a31, ]a12, ...
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC tableau contenant la matrice des coefficients variables (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    numero d'unite logique du fichier d'impression (si > 0)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPE
      INTEGER       MIXTE,NBL,NCL,NBC,NCC,IMP
      REAL          WL(*),WC(*),ELDIPQ
      REAL          COFVAR(NCL,*),COEFFR(NCL,NBL,NCC,*)
      COMPLEX       COFVAC(NCL,*),COEFFC(NCL,NBL,NCC,*)
*
      INTEGER       J,J1,I,I1
      REAL          RCF
*-----------------------------------------------------------------------
      IF (TYPE(1:1).EQ.'R') THEN
*
         DO 24 J=1,NBC
            DO 23 J1=1,NCC
               DO 22 I=1,NBL
                  RCF = ELDIPQ * WC(J) * WL(I)
                  DO 21 I1=1,NCL
                     COEFFR(I1,I,J1,J) = COEFFR(I1,I,J1,J)
     &                                 + COFVAR(I1,J1) * RCF
21                CONTINUE
22             CONTINUE
23          CONTINUE
24       CONTINUE
*
      ELSE
*
         DO 44 J=1,NBC
            DO 43 J1=1,NCL
               DO 42 I=1,NBL
                  RCF = ELDIPQ * WC(J) * WL(I)
                  DO 41 I1=1,NCC
                     COEFFC(I1,I,J1,J) = COEFFC(I1,I,J1,J) 
     &                                 + COFVAC(I1,J1) * RCF
41                CONTINUE
42             CONTINUE
43          CONTINUE
44       CONTINUE
      ENDIF
*
      IF (IMP.GT.0) THEN
         J=0
         IF (MIXTE.EQ.1) J=0
         CALL PRINTV ('U.AV (WWAVEC)'
     &               ,NCL,NBL,NCC,NBC,J,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
                                                                    END !WAWvec
