*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE WGRAW (NBL,DWL,NBC,WC,NDIM
     &                 ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin, O.DeBayser (Avril 1993)
* Derniere modification : E.Luneville (3 octobre 2008)
* Version 3
*
* Calcul des integrales elementaires  U.gradV
*                    | dT_i/dx1 |
*    a_{i,j1,j} = Wj | dT_i/dx2 |
*                    | dT_i/dx3 |
*
* ou U_n  = u_{n,j} W_j ; j=1,...,NBC, j1=1,NDIM (inconnue en col. vectorielle)
*        = n-eme composante de U
*    V   = v_i T_i      ; i=1,...,NBL (inconnue en ligne scalaire)
*-- Arguments --
* NBL    nombre de lignes de la matrice elementaire
* DWL    derivees des fonctions a l'image du point quadrature dans
*        l'element courant pour l'interpolation en ligne
*        (correspondant a la fonction-test scalaire V)
* NBC    nombre de colonnes de la matrice elementaire
* WC     valeurs des fonctions de base pour l'interpolation encolonne
*        (correspondant a l'inconnue vectorielle U)
* NDIM   dimension d'espace = nombre de composantes de U
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand ou du coeff. variable
* COFVAR coeff. variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coeff. variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    numero du fichier d'impression si non nul
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE      
      CHARACTER*(*) TYPE
      INTEGER       NBC,NBL,NDIM,IMP
      REAL          WC(*),DWL(NDIM,*),ELDIPQ
      REAL          COFVAR,COEFFR(NBL,NDIM,*)
      COMPLEX       COFVAC,COEFFC(NBL,NDIM,*)
*
      INTEGER       I,J1,J
      REAL          WCJ
*-----------------------------------------------------------------------
      IF (TYPE(1:1).EQ.'R') THEN
*
         DO 24 J=1,NBC
            WCJ=ELDIPQ * WC(J)
            DO 23 J1=1,NDIM
              DO 22 I=1,NBL
                  COEFFR(I,J1,J)=COEFFR(I,J1,J)+COFVAR * WCJ * DWL(J1,I)
22             CONTINUE
23          CONTINUE
24       CONTINUE
*
      ELSE
*
         DO 44 J=1,NBC
            WCJ=ELDIPQ * WC(J)
            DO 43 J1=1,NDIM
               DO 42 I=1,NBL
                  COEFFC(I,J1,J)=COEFFC(I,J1,J)+COFVAC * WCJ * DWL(J1,I)
42             CONTINUE
43          CONTINUE
44       CONTINUE
*
      ENDIF
*
      IF (IMP.GT.0)
     &   CALL PRINTV ('U gradV (WGRAW)'
     &               ,1,NBL,NDIM,NBC,NBC,TYPE,COEFFR,COEFFC,IMP)
                                                                    END !WgradW
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 3 : E.Luneville (3 octobre 2008)
* - bug boucle 22 : I1 non definie, remplace par J1
* Version 2 : D.Martin (3 octobre 2005)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
