*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE WWIVEC (MIXTE,NBL,WL,NCL,NBC,WC,NCC
     &                  ,ELDIPQ,TYPE,COFVAR,COEFFR,COFVAC,COEFFC,IMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Avril 1992)
* Derniere modification : D.Martin (28 juin 2005)
* Version 2
*
* Calcul de l'integrale elementaire: U.V avec inconnues vectorielles 2D et 3D
* (voir la generalisation dans WAWVEC)
*
*   a_{i1,i,j1,j} = delta_{i1,j1} Wj Ti
*
* ou U_j1 = u_j1,j W_j ; j=1,...,NBC , j1=1,...,NCC (inconnue vect. en colonne)
*    V_i1 = v_i1,i T_i ; j=1,...,NBL , i1=1,...,NCL (inconnue vect. en ligne)
*
*-- Arguments --
* NBL    nombre de lignes (comptee en noeuds) de l'integrand
* WL     valeurs des fonctions de base au point de quadrature
*        pour l'interpolation en ligne (T_i)
* NCL    nombre de composantes de l'inconnue en ligne
* NBC    nombre de colonnes (comptee en noeuds) de l'integrand
* WC     valeurs des fonctions de base au point de quadrature
*        pour l'interpolation en colonne (W_j)
* NCC    nombre de composantes de l'inconnue en colonne
* ELDIPQ poids de quadrature multiplie par l'element differentiel
* TYPE   type (REEL ou COMPLEXE) de l'integrand
* COFVAR coefficient variable (cas REEL)
* COEFFR tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas REEL)
* COFVAC coefficient variable (cas COMPLEXE)
* COEFFC tableau des valeurs cumulees des integrands elementaires aux points
*        de quadrature (cas COMPLEXE)
* IMP    numero d'unite logique du fichier d'impression (si > 0)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPE
      INTEGER       MIXTE,NBL,NCL,NBC,NCC,IMP
      REAL          WL(*),WC(*),ELDIPQ
      REAL          COFVAR,COEFFR(NCL,NBL,NCC,*)
      COMPLEX       COFVAC,COEFFC(NCL,NBL,NCC,*)
*
      INTEGER       J,I,J1
      REAL          RCF
      COMPLEX       CCF
*-----------------------------------------------------------------------
      IF (TYPE(1:1).EQ.'R') THEN
*
         IF (MIXTE.EQ.0) THEN
            DO 22 J=1,NBC
               RCF = COFVAR * ELDIPQ * WC(J)
               DO 21 I=1,J
                  COEFFR(1,I,1,J) = COEFFR(1,I,1,J) + RCF * WL(I)
21             CONTINUE
22          CONTINUE
            DO 25 J=1,NBC
               DO 24 I=1,J
                  RCF=COEFFR(1,I,1,J)
                  COEFFR(1,J,1,I)=RCF
                  DO 23 J1=2,NCC
                     COEFFR(J1,I,J1,J)=RCF
                     COEFFR(J1,J,J1,I)=RCF
23                CONTINUE
24             CONTINUE
25          CONTINUE
         ELSE
            DO 27 J=1,NBC
               RCF = COFVAR * ELDIPQ * WC(J)
               DO 26 I=1,NBL
                  COEFFR(1,I,1,J) = COEFFR(1,I,1,J) + RCF * WL(I)
26             CONTINUE
27          CONTINUE
            DO 30 J=1,NBC
               DO 29 I=1,NBL
                  RCF=COEFFR(1,I,1,J)
                  COEFFR(1,J,1,I)=RCF
                  DO 28 J1=2,NCC
                     COEFFR(J1,I,J1,J)=RCF
28                CONTINUE
29             CONTINUE
30          CONTINUE
         ENDIF
*
      ELSE
*
         IF (MIXTE.EQ.0) THEN
            DO 42 J=1,NBC
               CCF = COFVAC * ELDIPQ * WC(J)
               DO 41 I=1,J
                  COEFFC(1,I,1,J) = COEFFC(1,I,1,J) + CCF * WL(I)
41             CONTINUE
42           CONTINUE
            DO 45 J=1,NBC
               DO 44 I=1,J
                  CCF=COEFFC(1,I,1,J)
                  COEFFC(1,J,1,I)=CCF
                  DO 43 J1=2,NCC
                     COEFFC(J1,I,J1,J)=CCF
                     COEFFC(J1,J,J1,I)=CCF
43                CONTINUE
44             CONTINUE
45          CONTINUE
         ELSE
            DO 47 J=1,NBC
               CCF = COFVAC * ELDIPQ * WC(J)
               DO 46 I=1,NBL
                  COEFFC(1,I,1,J) = COEFFC(1,I,1,J) + CCF * WL(I)
46             CONTINUE
47           CONTINUE
            DO 50 J=1,NBC
               DO 49 I=1,NBL
                  CCF=COEFFC(1,I,1,J)
                  COEFFC(1,J,1,I)=CCF
                  DO 48 J1=2,NCC
                     COEFFC(J1,I,J1,J)=CCF
48                CONTINUE
49             CONTINUE
50          CONTINUE
         ENDIF
*
      ENDIF
*
      IF (IMP.GT.0) THEN
         J=NBC
         IF (MIXTE.EQ.0) J=0
         CALL PRINTV ('U.V (WWIVEC)'
     &               ,NCL,NBL,NCC,NBC,J,TYPE,COEFFR,COEFFC,IMP)
      ENDIF
                                                                    END !WWivec
