*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE LCINCO (NOMINC,LGMINC,INCONU,LGINCO,INTRPG,NDIM,NBTPTG
     &                  ,NIVIMP,IMPSDR,IMPMES,*)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin & O.DeBayser (Avril 1988)
* Derniere modification : D.Martin (6 novembre 2002)
* Version 1.0.2
*  
* LeCture du nom et des caracteristiques d'une nouvelle INCOnnue  
*  
*-- Arguments d'entree -- 
* NOMINC tableau des noms d'inconnues 
* LGMINC longueur du tableau des noms d'inconnues (eventuellement reactualisee)
* INCONU tableau des caracteristiques des inconnues
* LGINCO longueur actuelle du tableau #NCONU  
* INTRPG numero de type de l'interpolation geometrique
* NDIM   dimension d'espace
* NBTPTG nombre total de point du maillage
* NIVIMP undice d'impression secondaire
* IMPSDR unite d'impression secondaire
* IMPMES unite d'impression des messages  
*
*-- Mots-cles de la directive INCONNUE --
*  
* INCONNUE + < ||C 
* < Valeurs Nodales
*  [ DE TYPE < SCALAIRE / VECTORIELLE [ COMPOSANTES ||I ] > ]
*  [ INTERPOLATION
*    < ISOPARAMETRIQUE
*    / < DE TYPE  / D''ORDRE > ||I
*    / DE LAGRANGE < <P0 / Q0>
*                  / <P1 [NON CONFORME] / Q1>
*                  / <P2 / Q2 [SERENDIPITY]>
*                  / <P3 / Q3> / Q4 / Q5 / Q6 / Q7 / Q8 / Q9 / Q10
*                  >
*    / ELEMENTS D'ARETE [ ET CONDENSATION DE MASSE ]
*      < P0/Q0 / P1/Q1 / P2/Q2 >
*    / DE HERMITE [ P3 ]
*    >
*  ]
* / SPECTRALE
*     NOMBRE DE FONCTIONS PROPRES ||I  
*     [ NOMBRE DE TERMES PAR FONCTION ||I ]
* / SUPPLEMENTAIRE
*     [ DE TYPE < SCALAIRE / VECTORIELLE COMPOSANTES ||I > ]
* >
* Par defaut : Inconnue Valeurs Nodales Scalaire Interpolation Isoparametrique
*
* Pour les inconnues [Valeurs Nodales] type SCALAIRE par defaut
* Pour les inconnues VECTORIELLE dimension = NDIM par defaut
* Pour les inconnues Supplementaire type SCALAIRE par defaut
* Pour les inconues Spectrale pas de valeur par defaut
*  
* Cette directive complete les tableaux :  
* --------------------------------------
* NOMINC Tableau des noms d'inconnues sur NCHINC caracteres
* ++++++
* INCONU Tableau de description des inconnues 
* ++++++
*    entete de longueur 4
* INCONU(1)= LGDINC longueur de l'entete de INCONU (4)
* INCONU(2)= NBINCO nombre d'inconnues
* INCONU(3)= INCINC increment de parcour du tableau INCONU
* INCONU(4)= NCHINC nombre de caracteres par nom d'inconnue
*    boucle IINC = 1 a NBINCO
* ADINCO=LGDINC+INCINC*(IINC-1)+1
* INCONU(ADINCO  )= NCPINC nombre de composantes de l'inconnue
* INCONU(ADINCO+1)= INTINC type d'interpolation pour une inc. Valeurs Nodales
*                          -1 pour une inc. Spectrale ou Supplemantaire
* INCONU(ADINCO+2)= NBTNEU nb de noeuds pour une inc. Valeurs Nodales
*                 = NBFDEV+1000000*NBTFON pour une inc. Spectrale
*                   pour un terme Elements finis localises ('ELFLOC') calcule
*                   par integration exacte
*                 = NBFDEV nombre de fonctions propres pour une inc. Spectrale
*                   pour un terme Elements finis localises ('ELFLQU') calcule
*                   par integration numerique
*                 = nombre de composantes pour une inc. Supplementaire
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*)    NOMINC(*) 
      INTEGER          LGMINC,INCONU(*),LGINCO,INTRPG,NDIM,NBTPTG
     &                ,NIVIMP,IMPSDR,IMPMES
*
      INTEGER          KELCHN
      CHARACTER        TEXTE*80,ERCODE*120,FORM*90,COMBAK
      DOUBLE PRECISION FLOT
      INTEGER          NFIX,IRE,NER,NCAR,ILU,IOP
     &                ,LGEINC,NBINCO,INCINC,NCHINC,NUINCO,LGMINW
     &                ,NCPINC,LUTYPE,INTINC,NBTNEU,NBFDEV,NBTFON
      COMMON/REDCO1/   FLOT,NFIX,IRE,NER,NCAR,ILU,IOP(5)
      COMMON/REDCOM/   TEXTE
      COMMON/FORMAH/   ERCODE  
      EQUIVALENCE (ERCODE,FORM),(ERCODE(117:117),COMBAK)
*------------------------------------------------------------------------
      IF (COMBAK.NE.' ') GOTO 5
      CALL PRFXMJ (1,'*LcInco*')
*
*     Caracteristiques de la structure #NCONU
*
      CALL SDEXDB (INCONU,LGEINC,NBINCO,INCINC,NCHINC)                  !utilite
*
*     "Boucle de lecture des noms des inconnues"
*  
      CALL REDLE
2     IF (IRE.NE.4) GOTO 99
      IF (IMPMES.GT.0.AND.NIVIMP.GT.1) 
     &   WRITE(IMPMES,10004) '*LcInco*',NCAR,TEXTE(1:NCAR)
*     Test 1 : L'inconnue existe-t-elle deja ? 
      NUINCO=0 
      IF (NBINCO.GT.0.AND.NCAR.LE.NCHINC)
     &    NUINCO=KELCHN (TEXTE(1:NCAR),NOMINC,NBINCO,NCHINC)            !utilite
      IF (NUINCO.GT.0) GOTO 6  
*     Test2 : Le tableau NOMINC est-il assez grand ?
      LGMINW=(NBINCO+1)*MAX (NCAR,NCHINC)
      IF (LGMINC.GE.LGMINW) GOTO 5 
*        Sinon on sort du s-p pour l'agrandir,puis on revient (COMBAK='I') 
         COMBAK='I'
         LGMINC=LGMINW+MAX (NCAR,NCHINC)
         RETURN                                                         >>>>>>>>
*  
*     Valeurs par defaut du type de l'inconnue et de son type d'interpolation  
*  
5     COMBAK=' '
      NCPINC=1
      LUTYPE=0
      INTINC=INTRPG
      NBTNEU=NBTPTG
*
*     Caracteristiques de la structure #NCONU (qui a pu evoluer)
*
      CALL SDEXDB (INCONU,LGEINC,NBINCO,INCINC,NCHINC)                  !utilite
*
*     Le tableau de description des inconnues #NCONU est trop petit :  
      COMBAK='$'
      IF (LGINCO.LT.LGEINC+INCINC*(NBINCO+1)) RETURN 1  
      COMBAK=' '
*  
*--   On rajoute la nouvelle inconnue dans le tableau des inconnues
*  
      CALL RIPCHN (NOMINC,NBINCO,NCHINC,TEXTE(1:NCAR))                  !utilite
      CALL SDMKDB (INCONU,LGEINC,NBINCO,INCINC,NCHINC)                  !utilite
      NUINCO=NBINCO
      CALL SDMKCO (NUINCO,INCONU,NCPINC,INTINC,NBTNEU)                  !utilite
*
*     Lecture des attributs de l'inconnue  
*  
6     CALL REDLE
8     IF (IRE.EQ.4) GOTO 2 
      IF (IRE.NE.3) CALL REDERR (13,' ')
      IF (IMPMES.GT.0.AND.NIVIMP.GT.1) 
     &   WRITE (IMPMES,10003) '*LcInco*',TEXTE(1:NCAR)
*
      IF (TEXTE(1:6).EQ.'VALEUR'.OR.TEXTE(1:7).EQ.'ELEMENT') GOTO 6
      IF (TEXTE(1:6).EQ.'NODALE'.OR.TEXTE(1:4).EQ.'FINI') THEN
*
*        Type 'Valeurs Nodales'
*
         IF (LUTYPE.EQ.1) GOTO 99991
         LUTYPE=1
         GOTO 6
*
      ELSEIF (TEXTE(1:14).EQ.'SUPPLEMENTAIRE') THEN
*
*        Type 'Supplementaire'
*
         IF (LUTYPE.EQ.1) GOTO 99991
         LUTYPE=1
         INTINC=-1
         NBTNEU=0
         GOTO 6
*
      ELSEIF (TEXTE(1:8).EQ.'SPECTRAL') THEN
*
*        Type 'Spectrale' pour les Elements Finis Localises
*
         IF (LUTYPE.EQ.1) GOTO 99991
         LUTYPE=1
         INTINC=-1
         NBFDEV=0
         NBTFON=0
7        CALL REDLE
         IF (TEXTE(1:6).EQ.'NOMBRE') CALL REDLE
         IF (TEXTE(1:8).EQ.'FONCTION') THEN
            IF (IMPMES.GT.0.AND.NIVIMP.GT.2) 
     &         WRITE(IMPMES,10003) '*LcInco*',TEXTE(1:NCAR)
            CALL REDLE
            IF (TEXTE(1:6).EQ.'PROPRE') CALL REDLE
            IF (IRE.NE.1) CALL REDERR(11,' ')
            IF (IMPMES.GT.0.AND.NIVIMP.GT.2) 
     &         WRITE(IMPMES,10001) '*LcInco*',NFIX
            NBFDEV=NFIX
            GOTO 7
         ELSEIF (TEXTE(1:5).EQ.'TERME') THEN
            IF (IMPMES.GT.0.AND.NIVIMP.GT.2) 
     &         WRITE(IMPMES,10003) '*LcInco*',TEXTE(1:NCAR)
            CALL REDLE
            IF (TEXTE(1:8).EQ.'FONCTION') CALL REDLE
            IF (TEXTE(1:6).EQ.'PROPRE') CALL REDLE
            IF (IRE.NE.1) CALL REDERR(11,' ')
            IF (IMPMES.GT.0.AND.NIVIMP.GT.2) 
     &         WRITE(IMPMES,10001) '*LcInco*',NFIX
            NBTFON=NFIX
            GOTO 7
         ENDIF
         NBTNEU=NBFDEV+1000000*NBTFON
         IF (NBFDEV.LE.0) 
     &      CALL ERREUR (2,1,REAL(NBTNEU),' ','SPECTRALE')
*
      ELSEIF (TEXTE(1:13).EQ.'INTERPOLATION') THEN
*
         IF (INTINC.EQ.-1) CALL REDERR (2,'INTERPOLATION')
*
         CALL LCINTE (NUINCO,NOMINC,NCHINC,INTINC,NIVIMP,IMPSDR,IMPMES) !lecdire
*
         IF (INTINC.NE.INTRPG) NBTNEU=0
*
      ELSE
*
*        Lecture du type (scalaire ou vectorielle) de l'inconnue
*
         IF (TEXTE(1:4).EQ.'TYPE') CALL REDLE
         IF (IRE.NE.3) CALL REDERR(13,' ')
         IF (TEXTE(1:9).EQ.'VECTORIEL') THEN
            IF (IMPMES.GT.0.AND.NIVIMP.GT.2) 
     &         WRITE(IMPMES,10003) '*LcInco*',TEXTE(1:NCAR) 
            NCPINC=NDIM
            CALL REDLE
            IF (IMPMES.GT.0.AND.NIVIMP.GT.2) 
     &         WRITE (IMPMES,10003) '*LcInco*',TEXTE(1:NCAR)
            IF (IRE.NE.3) CALL REDERR(13,' ')
            IF (TEXTE(1:9).EQ.'DIMENSION'
     &      .OR.TEXTE(1:10).EQ.'COMPOSANTE') THEN
               CALL REDLE
               IF (IRE.NE.1) CALL REDERR(11,' ')
               IF (IMPMES.GT.0.AND.NIVIMP.GT.2) 
     &            WRITE (IMPMES,10001) '*LcInco*',NFIX
               NCPINC=NFIX
               GOTO 6
            ELSE
               GOTO 8
            ENDIF
         ENDIF 
         IF (TEXTE(1:8).NE.'SCALAIRE'.AND.TEXTE(1:9).NE.'VECTORIEL') 
     &      GOTO 89
         IF (IMPMES.GT.0.AND.NIVIMP.GT.2) 
     &      WRITE (IMPMES,10003) '*LcInco*',TEXTE(1:NCAR) 
         GOTO 6
      ENDIF
*
*     Renseignement de la structure #NCONU pour cette inconnue
*
89    CALL SDMKCO (NUINCO,INCONU,NCPINC,INTINC,NBTNEU)                  !utilite
*
*     Le cas echeant inconnue suivante
*
      GOTO 2  
*
*     Fin des lecture des Inconnues et de leur caracteristiques
*
99    IF (IMPSDR.GT.0.AND.NIVIMP.GT.1) THEN 
         IF (IMPSDR.NE.0) WRITE (IMPSDR,10010) '*LcInco*'
         CALL PRINCO (IMPSDR)                                           !prsd
      ENDIF
      CALL PRFXMJ (-1,'*LcInco*')
      RETURN
*----------------------------------------------------------------------- 
10000 FORMAT(T6,A8,' Lecture de la sous-directive : ',A)
10001 FORMAT(T6,A8,T56,' , suivi de nombre : ',I12)
10003 FORMAT(T6,A8,T56,' , suivi du mot : ',A) 
10004 FORMAT(T6,A8,T56,' , suivi de la chaine (*',I2,') : ',A) 
10010 FORMAT(/T2,A8,'  ','Nom et type d''interpolation des inconnues')
*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*     Type d'inconnue redefini
99991 CALL REDERR (3,'definissant l''inconnue')
                                                                    END !Lcinco
