*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE LCTERM (NOMINC,INCONU,NOMDON,DONNES,NDIM
     &                  ,TYPCAL,NMTERM,NCTERM,NVTERM,TYPTRM
     &                  ,NMDONN,NCDONN,ITYDON,NIVDON,ITYPDO,NUINCC
     &                  ,NUINCL,NUKALE,NVALNE,IRPLOC,ICORNE
     &                  ,IMPTRM,NIVIMP,IMPMES)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin & O.deBayser (Fevrier 1989) 
* Derniere modification : D.Martin (12 decembre 2002)
* Version 1.0.7
*  
* LeCture des caracteristiques d'un TERMe (ou d'une condition essentielle)
*  
*-- Arguments d'entree --
* NOMINC tableau des noms des inconnues
* INCONU structure #NCONU des caracteristiques des inconnues
* NOMDON tableau des noms des donnees
* DONNES structure $DONNE des caracteristiques des donnees
* NDIM   dimension d'espace
* TYPCAL type de calcul du terme courant ('ELFINI', 'ELFLOC', 'ELFLQU', ...)
* NIVIMP niveau d'impression des directives
* IMPMES unite d'impression des messages  
*-- Arguments de sortie --
* NMTERM nom du terme 
* NCTERM nombre de caracteres du nom du terme lu  
* NVTERM niveau du (tableau contenant le) terme
* NMDONN(1) nom de la donnee associee au terme
*       (2) le cas echeant de la donnee associee a la donnee dans le cas
*           ou NMDONN(1) une donnee 'FONCTION'
* NCDONN(1) nombre de caracteres du nom de la donnee lue
*       (2) le cas echeant de la donnee associee a la donnee ...
* ITYDON(1) type de la donnee associee au terme (CONSTANTE,FONCTION,...) 
*       (2) le cas echeant de la donnee associee a la donnee ...
* NIVDON(1) niveau de la donnee associee au terme
*       (2) le cas echeant de la donnee associee a la donnee ...
* ITYPDO(1) type de declaration de la donnee associee au terme
*       (2) le cas echeant de la donnee associee a la donnee ...
* NUINCC numero de l'inconnue en colonne  
* NUINCL numero de l'inconnue en ligne
* NUKALE numero du type d'integrand                 si TYPCAL = 'ELFINI'
*      / numero du type d'integrand                 si TYPCAL = 'MATELM'  
*      / numero du type de terme Valeurs Nodales    si TYPCAL = 'VALNOD' 
*      / numero de type de la condition essentielle si TYPCAL = 'C.ESSE'
*                                                            ou 'C.TRAN'
*      / numero de type de nature du terme          si TYPCAL = 'ELFLOC'
*      / numero d'integrand (1)                     si TYPCAL = 'ELFLQU'
*      / numero (1)                                 si TYPCAL = 'SPECTR'
*      / numero de type d'interpolant               si TYPCAL = 'INTERP'
*      / numero de forme lineaire ponctuelle        si TYPCAL = 'FORMLI'
* NVALNE type de symetrie declaree explicitement pour un terme 'ELFINI'
*      / nb valeurs par noeud pour un terme 'VALNOD', 'C.ESSE' ou 'C.TRAN'
* IRPLOC indice de calcul des normales aux noeuds du domaine de calcul
* ICORNE indice de calcul des coordonnees des noeuds du domaine de calcul
* IMPTRM niveau d'impression du terme
*
*-- Sous-directives de la directive TERME
*
*  TERME 
*  < [ ELEMENTS FINIS ] [STANDARD] [de Nom] &C*6 [de Niveau &I]
*      < [ Symetrique ^ AntiSymetrique ^ AutoAdjoint ^ AntiAdjoint ] >
*  ^ [ MATRICE ] ELEMENTAIRE       [de Nom] &C*6 [de Niveau &I]
*  ^   INTERPOLE                   [de Nom] &C*6 [de Niveau &I]
*  ^   FORME LINEAIRE PONCTUELLE   [de Nom] &C*6 [de Niveau &I]
*  ^   VALEURS NODALES             [de Nom] &C*6 [de Niveau &I]
*                                  [de Type]  [ < &A ^ &C > ]
*  ^   ELEMENTS FINIS LOCALISES    [de Nom] &C*6 [de Niveau &I]
*        INTEGRATION 
*          < ANALYTIQUE/EXACTE 
*          ^ NUMERIQUE 
*          >
*  ^   SPECTRAL                    [de Nom] &C*6 [de Niveau &I]
*  >
*  < [ Reel ^ Complexe ] >
*
* voir liste de valeurs de &A ou &C possibles dans INVANO (Lib-Utiliter) 
*
*-- Sous-directives de la directive CONDITION
*
*  CONDITION 
*  < ESSENTIELLE / DE TRANSMISSION > [de] [Nom] &C*6 [de Niveau &I] 
*                                    [de Type]  < &Ce / &Ct >
*  >
*
* ou &Ce est la concatenation d'un nom d'inconnue et d'une des constantes
*    caracteres : '=g', '.N=g', '^N=g', etc. (voir dans INCESS (Lib-Utiliter)
*
*    &Ct est la concatenation d'un nom d'inconnue et d'une des constantes
*    caracteres : '=', '.N=', '^N=', et d'un nom d'inconnue 
*    et d'une des constantes caracteres : '+g', '.N+g', '^N+g'.
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER        NOMINC(*),NOMDON(*),TYPCAL*6
      CHARACTER*(*)    NMTERM,TYPTRM,NMDONN(*)
      INTEGER          NCTERM,NVTERM,INCONU(*),DONNES(*)
     &                ,NCDONN(2),ITYDON(2),NIVDON(2),ITYPDO(2) 
     &                ,NUINCC,NUINCL,NUKALE,NDIM,NVALNE,IRPLOC,ICORNE
     &                ,IMPTRM,NIVIMP,IMPMES
*  
      DOUBLE PRECISION FLOT
      CHARACTER        TEXTE*80
      INTEGER          INDSYM,INVANO
      INTEGER          NFIX,IRE,NER,NCAR,ILU,IOP,LUDONX,LUDONN,LUINCO
     &                ,LUKALE,LUIMPR,LUVALN,NCPINC,INTINC,NBTNEU
      COMMON/REDCO1/   FLOT,NFIX,IRE,NER,NCAR,ILU,IOP(5)
      COMMON/REDCOM/   TEXTE 
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*LcTerm*')
*
      LUDONX=2
      LUDONN=0
      LUINCO=0
      LUKALE=0
      LUVALN=0
      LUIMPR=0
      TYPTRM='REEL'
*  
*     Lecture du nom du terme ou de la condition essentielle
*
      IF (TEXTE(1:NCAR).EQ.'NOM'.OR.TEXTE(1:4).EQ.'NAME') CALL REDLE
      IF (IRE.NE.4) CALL REDERR (14,' ')
      IF (IMPMES.GT.0.AND.NIVIMP.GT.2) 
     &   WRITE (IMPMES,10004) '*LcTerm*',NCAR,TEXTE(1:NCAR)
      NMTERM=TEXTE(1:NCAR)
      NCTERM=NCAR
      NVTERM=1
      CALL REDLE
*  
2     IF (LUKALE.LE.0.AND.IRE.EQ.4) THEN
*        On a lu une chaine de caracteres (IRE=4) definissant
*        un type de 'Valeurs Nodales', une 'Condition Essentielle'
*        ou une 'Condition de transmission'
         IF (TYPCAL.EQ.'VALNOD') GOTO 100
         IF (TYPCAL.EQ.'C.ESSE') GOTO 200
         IF (TYPCAL.EQ.'C.TRAN') GOTO 300
      ENDIF
      IF (IRE.NE.3) CALL REDERR (13,' ')
*
      IF (TEXTE(1:NCAR).EQ.'NIVEAU'.OR.TEXTE(1:NCAR).EQ.'LEVEL') THEN
*
*        On a lu 'NIVEAU' ou 'LEVEL'
         IF (IMPMES.GT.0.AND.NIVIMP.GT.2)
     &      WRITE (IMPMES,10000) '*LcTerm*',TEXTE(1:NCAR)
         CALL REDLE
         IF (IRE.NE.1) CALL REDERR (11,TEXTE(1:NCAR))
         IF (IMPMES.GT.0.AND.NIVIMP.GT.2)
     &      WRITE (IMPMES,10001) '*LcTerm*',NFIX
         NVTERM=NFIX
         CALL REDLE
         GOTO 2
*
      ELSEIF (TEXTE(1:10).EQ.'IMPRESSION'
     &    .OR.TEXTE(1:5).EQ.'PRINT') THEN
*
*        On a lu IMPRESSION / PRINT
         IF (IMPMES.GT.0.AND.NIVIMP.GT.2)
     &      WRITE (IMPMES,10000) '*LcTerm*',TEXTE(1:NCAR)
         IF (LUIMPR.EQ.1) GOTO 9999
         CALL REDLE
         IF (IRE.NE.3.OR.(TEXTE(1:NCAR).NE.'NIVEAU'
     &               .AND.TEXTE(1:5).NE.'LEVEL'))
     &      CALL REDERR (1,'NIVEAU d''impression')
         CALL REDLE
         IF (IRE.NE.1) CALL REDERR (11,' ')
         IF (IMPMES.GT.0.AND.NIVIMP.GT.2)
     &      WRITE (IMPMES,10001) '*LcTerm*',NFIX
         IMPTRM=NFIX
         CALL REDLE
         LUIMPR=1
         GOTO 2
*
      ELSEIF (TEXTE(1:8).EQ.'INCONNUE'
     &    .OR.TEXTE(1:7).EQ.'UNKNOWN') THEN
*
*        On a lu INCONNUE
         IF (IMPMES.GT.0.AND.NIVIMP.GT.2)
     &      WRITE (IMPMES,10000) '*LcTerm*',TEXTE(1:NCAR)
         IF (LUINCO.GT.1) CALL REDERR (2,TEXTE(1:NCAR))
         CALL LCINCT (TYPCAL,NOMINC,INCONU,NUINCC,NUINCL,NIVIMP,IMPMES) !lecdire
         LUINCO=LUINCO+1
         GOTO 2
*
      ELSEIF (TEXTE(1:8).EQ.'INTEGRAN') THEN
*
*        On a lu ' INTEGRAND '
         IF (IMPMES.GT.0.AND.NIVIMP.GT.2)
     &      WRITE (IMPMES,10000) '*LcTerm*',TEXTE(1:NCAR)
         IF (TYPCAL.NE.'ELFINI'.AND.TYPCAL.NE.'MATELM')
     &       CALL REDERR (2,'INTEGRAND')
         IF (LUKALE.NE.0) CALL REDERR (3,'INTEGRAND')
         CALL LCINTG (NUKALE,NIVIMP,IMPMES)                             !lecdire
         LUKALE=1
         GOTO 2
*
      ELSEIF (TEXTE(1:3).EQ.'SYM'
     &    .OR.TEXTE(1:7).EQ.'ANTISYM'
     &    .OR.TEXTE(1:11).EQ.'AUTOADJOINT'
     &    .OR.TEXTE(1:11).EQ.'ANTIADJOINT') THEN
*
*        On a lu '[ANTI]SYMETRIQUE' ou '[AUTO/ANTI]ADJOINTE'
         IF (IMPMES.GT.0.AND.NIVIMP.GT.2)
     &      WRITE (IMPMES,10003) '*LcTerm*',TEXTE(1:NCAR)
*
*        Type de symetrie explicitement declare pour un terme 'Elfini'
*
         IF (TYPCAL(1:3).NE.'ELF') CALL REDERR (2,TEXTE(1:NCAR))
         IF (LUVALN.NE.0) CALL REDERR (3,TEXTE(1:NCAR))
         NVALNE=INDSYM (TEXTE(1:NCAR))                                  !lecdire
         CALL REDLE
         LUVALN=1
         GOTO 2
*
      ELSEIF (TEXTE(1:NCAR).EQ.'REEL'.OR.TEXTE(1:NCAR).EQ.'REAL') THEN
         IF (IMPMES.GT.0.AND.NIVIMP.GT.2)
     &      WRITE (IMPMES,10003) '*LcTerm*',TEXTE(1:NCAR)
*
*        Type de declaration REEL explicitement donne
*
         TYPTRM='REEL'
         CALL REDLE
         GOTO 2
      ELSEIF (TEXTE(1:7).EQ.'COMPLEX') THEN
         IF (IMPMES.GT.0.AND.NIVIMP.GT.2)
     &      WRITE (IMPMES,10003) '*LcTerm*',TEXTE(1:NCAR)
*
*        Type de declaration COMPLEX explicitement donne
*
         TYPTRM='COMPLEXE'
         CALL REDLE
         GOTO 2
*
      ELSEIF (TEXTE(1:NCAR).EQ.'DERIVEE') THEN
*
*        On a lu ' DERIVEE ' (terme de type Valeurs Nodales)
         IF (IMPMES.GT.0.AND.NIVIMP.GT.2)
     &      WRITE (IMPMES,10000) '*LcTerm*',TEXTE(1:NCAR)
         IF (TYPCAL.NE.'VALNOD') CALL REDERR (2,TEXTE(1:NCAR))
         NUKALE=INVANO ('DF/DN')                                        !utilite
         IRPLOC=1
         ICORNE=1
         CALL REDLE
         IF (TEXTE(1:7).EQ.'NORMALE') CALL REDLE
         LUKALE=1
         LUVALN=1
         GOTO 2
*
      ELSEIF (TEXTE(1:NCAR).EQ.'FOURNI') THEN
*
*        On a lu ' FOURNI ' (terme de type Valeurs Nodales)
         IF (IMPMES.GT.0.AND.NIVIMP.GT.2)
     &      WRITE (IMPMES,10000) '*LcTerm*',TEXTE(1:NCAR)
         IF (TYPCAL.NE.'VALNOD') CALL REDERR (2,TEXTE(1:NCAR))
         NUKALE=INVANO ('TABF')                                         !utilite
         CALL LCVANO (NUKALE,NVALNE,IRPLOC,ICORNE,NIVIMP,IMPMES)        !lecdire
         LUKALE=1
         LUVALN=1
         GOTO 2
*
      ELSEIF (TEXTE(1:NCAR).EQ.'INTEGRATION') THEN
*
*        On a lu ' INTEGRATION ' (terme de type Elements Finis Localises)
         IF (IMPMES.GT.0.AND.NIVIMP.GT.2)
     &      WRITE (IMPMES,10000) '*LcTerm*',TEXTE(1:NCAR)
         IF (TYPCAL.NE.'ELFLOC'.AND.TYPCAL.NE.'ELFLQU') 
     &      CALL REDERR (2,TEXTE(1:NCAR))
         CALL REDLE
         IF (IRE.NE.3) CALL REDERR (3,' ')
         IF (IMPMES.GT.0.AND.NIVIMP.GT.2)
     &      WRITE (IMPMES,10003) '*LcTerm*',TEXTE(1:NCAR)
         IF (TEXTE(1:NCAR).EQ.'ANALYTIQUE'.OR.TEXTE(1:NCAR).EQ.'EXACTE') 
     &   THEN
            TYPCAL='ELFLOC'
         ELSEIF (TEXTE(1:NCAR).EQ.'NUMERIQUE') THEN
            TYPCAL='ELFLQU'
            IF (LUKALE.NE.0) CALL REDERR (2,'INTEGRATION NUMERIQUE')
            LUKALE=1
            NUKALE=1
         ELSE
*           Mot non attendu ici
            CALL REDERR (2,TEXTE(1:NCAR))
         ENDIF
         CALL REDLE
         GOTO 2
*
      ELSEIF (TEXTE(1:NCAR).EQ.'NATURE') THEN
*
*        On a lu ' NATURE '
         IF (IMPMES.GT.0.AND.NIVIMP.GT.2)
     &      WRITE (IMPMES,10000) '*LcTerm*',TEXTE(1:NCAR)
         IF (TYPCAL.NE.'ELFLOC') CALL REDERR (2,'NATURE')
         IF (LUKALE.NE.0) CALL REDERR (3,'NATURE')
         CALL LCFLOC (NUKALE,NIVIMP,IMPMES)                             !lecdire
         LUKALE=1
         GOTO 2
*
      ELSEIF (TEXTE(1:NCAR).EQ.'INTERPOLANT') THEN
*
*        On a lu ' INTERPOLANT '
         IF (IMPMES.GT.0.AND.NIVIMP.GT.2)
     &      WRITE (IMPMES,10000) '*LcTerm*',TEXTE(1:NCAR)
         IF (TYPCAL.NE.'INTERP') CALL REDERR (2,'INTERPOLANT')
         IF (LUKALE.NE.0) CALL REDERR (3,'INTEGRAND')
         CALL LCITRG (NUKALE,NIVIMP,IMPMES)                             !lecdire
         LUKALE=1
         GOTO 2
*
      ELSEIF (TEXTE(1:NCAR).EQ.'FORME') THEN
*
*        On a lu ' FORME '
         IF (IMPMES.GT.0.AND.NIVIMP.GT.2)
     &      WRITE (IMPMES,10000) '*LcTerm*',TEXTE(1:NCAR)
         IF (TYPCAL.NE.'FORMLI') CALL REDERR (2,'FORME')
         IF (LUKALE.NE.0) CALL REDERR (3,'FORME')
         CALL REDLE
         IF (TEXTE(1:4).EQ.'TYPE') CALL REDLE
         IF (IRE.NE.4) CALL REDERR (14,' ')
         IF (IMPMES.GT.0.AND.NIVIMP.GT.2)
     &      WRITE (IMPMES,10004) '*LcTerm*',NCAR,TEXTE(1:NCAR)
         CALL REDLE
*        Par defaut NUKALE=1 pour tous les types de terme
         NUKALE=1
         LUKALE=1
         GOTO 2
*
      ELSEIF (TEXTE(1:NCAR).EQ.'DONNEE'.OR.TEXTE(1:3).EQ.'DAT') THEN
*
*        On a lu Donnee
         IF (IMPMES.GT.0.AND.NIVIMP.GT.2)
     &      WRITE (IMPMES,10000) '*LcTerm*',TEXTE(1:NCAR)
         CALL LCDONT (NOMDON,DONNES,LUDONX,NMDONN,NCDONN,ITYDON,NIVDON
     &               ,ITYPDO,LUDONN,NIVIMP,IMPMES)                      !lecdire
         GOTO 2
*
      ENDIF
*
      IF (TEXTE(1:NCAR).EQ.'MATRICIEL'.OR.TEXTE(1:NCAR).EQ.'TENSORIEL'
     &.OR.TEXTE(1:NCAR).EQ.'SCALAIRE'.OR.TEXTE(1:NCAR).EQ.'VECTORIEL')
     &    GOTO 100
      IF (TEXTE(1:NCAR).NE.'TYPE') GOTO 9999
*
*        On a lu ' TYPE '
*       (terme de type Valeurs Nodales, Cond. Essentille ou transmision)
*
         IF (IMPMES.GT.0.AND.NIVIMP.GT.2)
     &      WRITE (IMPMES,10000) '*LcTerm*',TEXTE(1:NCAR)
         IF (LUKALE.NE.0) CALL REDERR (3,'TYPE')
         CALL REDLE
         IF (IRE.EQ.3) GOTO 2
         IF (IRE.NE.4) CALL REDERR (14,' ')
         IF (TYPCAL.EQ.'VALNOD') GOTO 100
         IF (TYPCAL.EQ.'C.ESSE') GOTO 200
         IF (TYPCAL.EQ.'C.TRAN') GOTO 300
         CALL REDERR (2,TEXTE(1:NCAR))
*
*        On a lu un nom de type de 'VALeurs NODales'
*         'F', 'DF/DN', 'TABF', 'NF', 'N.F', 'N^F', 'N^(N^F)', 'N(N.F)'
*         'N1F', 'N2F', 'N3F'
*
100      IF (LUKALE.EQ.0) NUKALE=INVANO ('F')                           !utilite
         CALL LCVANO (NUKALE,NVALNE,IRPLOC,ICORNE,NIVIMP,IMPMES)        !lecdire
         LUKALE=1
         LUVALN=1
         LUINCO=1
         GOTO 2
*  
*        On a lu ' CONDITION ESSENTIELLE '
*
200      CALL LCESSE (NOMINC,INCONU,NDIM,NUINCC
     &               ,NUKALE,NVALNE,IRPLOC,ICORNE,NIVIMP,IMPMES)        !lecdire
         LUKALE=1
         LUVALN=1
         LUINCO=1
         GOTO 2
*  
*        On a lu ' CONDITION DE TRANSMISSION '
*
300      CALL LCTRAN (NOMINC,INCONU,NDIM,NUINCC,NUINCL
     &               ,NUKALE,NVALNE,IRPLOC,ICORNE,NIVIMP,IMPMES)        !lecdire
         LUKALE=1
         LUVALN=1
         LUINCO=1
         GOTO 2
*---- On a fini
9999  IF (LUINCO.EQ.0) CALL REDERR (1,'INCONNUE')
*
*     Test de l'integrand 
*
      IF (TYPCAL(1:3).EQ.'ELF'.AND.LUKALE.EQ.0)
     &   CALL REDERR (1,'INTEGRAND')
*
*     Pour un terme de type Element Finis Localises, on verifie
*     la compatibilite entre le type de calcul et les informations
*     concernant l'inconnue spectrale
*
      IF (TYPCAL.EQ.'ELFLOC') THEN
*
*        Cas d'un terme Elements Finis Localises calcule par
*        integration exacte
*        On a ici NBTNEU = NBFDEV+1000000*NBTFON
         IF (NUINCC.GT.0) THEN
            CALL SDEXCO (NUINCC,INCONU,NCPINC,INTINC,NBTNEU)            !utilite
            IF (INTINC.EQ.-1) THEN
                IF (NBTNEU.LT.1000000) THEN
                   CALL GETCHN (NOMINC,NCHINC,NUINCC,TEXTE)             !utilite
                   CALL ERINCO (7,TEXTE(1:NCHINC))                      !utilite
                ENDIF
            ENDIF
         ENDIF
         IF (NUINCL.GT.0) THEN
            CALL SDEXCO (NUINCC,INCONU,NCPINC,INTINC,NBTNEU)            !utilite
            IF (INTINC.EQ.-1) THEN
                IF (NBTNEU.LT.1000000) THEN
                   CALL GETCHN (NOMINC,NCHINC,NUINCL,TEXTE)             !utilite
                   CALL ERINCO (7,TEXTE(1:NCHINC))                      !utilite
                ENDIF
            ENDIF
         ENDIF
*
      ELSEIF (TYPCAL.EQ.'ELFLQU') THEN
*
*        Cas d'un terme Elements Finis Localises calcule par
*        integration numerique
*        On a ici NBTNEU = NBFDEV
         IF (NUINCC.GT.0) THEN
            CALL SDEXCO (NUINCC,INCONU,NCPINC,INTINC,NBTNEU)            !utilite
            IF (INTINC.EQ.-1) THEN
                IF (NBTNEU.GE.1000000) THEN
                   CALL GETCHN (NOMINC,NCHINC,NUINCC,TEXTE)             !utilite
                   CALL ERINCO (8,TEXTE(1:NCHINC))                      !utilite
                ENDIF
            ENDIF
         ENDIF
         IF (NUINCL.GT.0) THEN
            CALL SDEXCO (NUINCC,INCONU,NCPINC,INTINC,NBTNEU)            !utilite
            IF (INTINC.EQ.-1) THEN
                IF (NBTNEU.GE.1000000) THEN
                   CALL GETCHN (NOMINC,NCHINC,NUINCL,TEXTE)             !utilite
                   CALL ERINCO (8,TEXTE(1:NCHINC))                      !utilite
                ENDIF
            ENDIF
         ENDIF
      ENDIF
*  
      CALL PRFXMJ (-1,'*LcTerm*')
*----------------------------------------------------------------------- 
10000 FORMAT(T9,A8,' Lecture de la sous-directive : ',A)
10001 FORMAT(T9,A8,T56,' , suivi du nombre : ',I12)
10003 FORMAT(T9,A8,T56,' , suivi du mot : ',A) 
10004 FORMAT(T9,A8,T56,' , suivi de la chaine (*',I2,') : ',A)
                                                                    END !LcTerm
