*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE LCGMAI (NIVIMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin & O.DeBayser (Avril 1988) 
* Derniere modification : D.Martin (25 Janvier 2002)
* Version 1.0.5
*  
* LeCture de la Geometrie sur le fichier de transfert du MAIllage
* - -           -                                        ---
* Cette procedure est appelee par
*  la procedure principale de lecture du maillage LcGeom (LecGeom)
*  le processeur de conversion de format de maillage (M2Nopo)
*
*-- Argument d'entree --
* NIVIMP niveau d'impression lors de la lecture du maillage
*
* Mots-clefs de la directive GEOMETRIE (lus sur le fichier du maillage)
* ------------------------------------
*  
*    [ IMPRESSION [DE NIVEAU] &I ] 
*
*--  Description des formats de lecture des coordonnees et de la numerotation
*
*    [ FORMAT [DE] [LECTURE]
*      + < [DES] COORDONNEES &Cc*50
*        / [DE] [LA] NUMEROTATION [GLOBALE] &Cn*50
*        / [SANS COMMENTAIRE] >
*    ]
*  
*    < DESCRIPTION [GLOBALE] [DU] [MAILLAGE]
*       <  [[NOM] [DES] VARIABLES [D''] ESPACE  + &C ]
*          [NOMBRE] [D''] ELEMENTS &I   >>
*  
*--  Lecture des types d'elements par blocs d'elements consecutifs de meme type
*  
*    + BLOC [DE] 
*        < < HEXAEDRES ^ PRISMES ^ TETRADEDRES 
*          ^ QUADRANGLES ^ TRIANGLES ^ SEGMENTS 
*          >
*          DE LAGRANGE 
*          [ AUX ABSCISSES < EQUIDISTANTES ^ DE GAUSS-LOBATTO > ]
*            < [DE TYPE] < P1 ^ Q1 ^ P2 ^ Q2 ^ P3 ^ Q3 ^ Q4 ... Q9 ^ Q10 >
*            ^ < D'ORDRE ^ DE DEGRE > ||I
*            >
*        ^ [TYPE] [GEOMETRIQUE] &A 
*        > : I& ELEMENTS
*  
*--  Lecture des coordonnees et de la numerotation du mailage par element  
*    dans le meme ordre que ci-dessus.
*  
*    READ(MAILUL,*) (AVEC commentaire, ligne ignoree)
*    ' Coordonnees des points de l'element ' selon le format Cc*50
*    READ(MAILUL,*) (AVEC commentaire, ligne ignoree)
*    ' Numeros globaux des points de l'element ' selon le format Cn*50
*  
*--  Description des sous domaines par les elements, ou les faces d'elements,
*                ou les aretes d'elements, ou les points qu'ils contiennent.  
*  
*    + < DOMAINE &C
*        + E[LEMENT] &I_1 [ /I_n ]
*        < [ + &I ] / F[ACE] &I_2 / A[RETE] &I_2 / P[OINT] &I_2 >>
*  
*--  Description des interfaces entre domaine, communication inter-processeurs 
*    pour un calcul parallele par decomposition de domaine (MS3D)  
*  
*    + < COMMUNICATION [INTER PROCESSEURS]  
*         + [PROCESSEUR]  VOISIN I1
*             < DOMAINE &C / MESSAGE + &I >
*
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      INTEGER NIVIMP
*
      INCLUDE 'CONTEX'
      INCLUDE 'ALLOC'
*
      CHARACTER TEXTE*80
      INTEGER   NFIX,IRE,NER,NCAR,ILU,IOP
     &         ,NUM,NRAN,LEC,IMP,IECR,KNUM,IRWD,NOCO,IRET,NAVANT
     &         ,LGVARE,MCVARE,LGEBLK,MCBLOK,LGBLOK,MCWOR0,LGFORM
     &         ,NADBLK,LGNUMG,LGCORG,MCNUMG,MCCORG,LGMDOM,MCTRDO
     &         ,LGERDO,NBDOMK,INCRDO,NCHDOM,MCNOMD,LGLIST,MCLIST
     &         ,MCDLFA,NBQUOI,LUFORM,IRCODE
     &         ,LGCOMM,MCCOMM,ADNOMD,NUDOMG,NBCARG,K,IP,JP,NBLIST
      DOUBLE PRECISION FLOT
      INTEGER   NBLTYP(7),NULTYP(7)
      REAL      JACOBF(9),JACOBM(9)
      COMMON/REDCO1/FLOT,NFIX,IRE,NER,NCAR,ILU,IOP(5)
      COMMON/REDCOM/TEXTE
      COMMON/REDCA1/NUM,NRAN,LEC,IMP,IECR,KNUM,IRWD,NOCO,IRET,NAVANT
*
      LOGICAL   COMENT
      CHARACTER ERCODE*120,FORMAC*52,FORMAN*52,COMBAK*1
      COMMON/FORMAH/ERCODE
      EQUIVALENCE (ERCODE(120:120),COMBAK)
*
      DATA FORMAC/'(6E12.5)'/,FORMAN/'(18I4)'/
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*LcGmai*')
      COMENT=.TRUE.
      LUFORM=0

1     IF (TEXTE(1:NCAR).EQ.'IMPRESSION'.OR.TEXTE(1:5).EQ.'PRINT') THEN
*
*--      Lecture du niveau d'impression du maillage
*
         IF (IMPMES.GT.0.AND.NIVIMP.GT.0)
     &      WRITE (IMPMES,10000) '*LcGmai*','IMPRESSION'
         NIVIMP=1
         CALL REDLE
         IF (IRE.EQ.3.AND.(TEXTE(1:NCAR).EQ.'NIVEAU'
     &                 .OR.TEXTE(1:NCAR).EQ.'LEVEL')) CALL REDLE
         IF (IRE.EQ.1) THEN
            NIVIMP=NFIX
            IF (IMPMES.GT.0.AND.NIVIMP.GT.0)
     &          WRITE (IMPMES,10006) '*LcGmai*',NFIX
            CALL REDLE
         ENDIF
         GOTO 1
*
      ELSEIF (TEXTE(1:NCAR).EQ.'FORMAT') THEN
*
*--     Format de lecture des coordonnees et des numerotation du maillage
*
        IF (LUFORM.GT.0) CALL REDERR (3,'FORMAT')
        CALL LCFORM (FORMAC,FORMAN,COMENT,NIVIMP,IMPMES)                !lecgeom
        LUFORM=LUFORM+1
        GOTO 1
*
      ENDIF
*
*--   Creation du tableau (#ARIAB,1) des noms des variables d'espace
*
      IF (IMPSDR.GT.0.AND.NIVIMP.GT.0) WRITE (IMPSDR,10002)'*LcGmai*'
      LGVARE=3+77
      CALL TBCREE ('#ARIAB',1,11,LGVARE,'c')                     
*
*--   Lecture des donnees globales du maillage et definition de NDIM
*
      COMBAK=' '
      CALL TBAR1  (ERCODE,'#ARIAB',1,MCVARE)                     
*
11    CALL LCDESG (NBTEL,NBCNVE,AST(MCVARE),LGVARE,NIVIMP
     &            ,COMBAK,NDIM,IMPMES,IMPSDR)                           !lecgeom
      IF (COMBAK.NE.' ') THEN
*        Le tableau #ARIAB doit etre agrandi avant introduction de la
*        nouvelle variable d'espace :
         CALL TBAJST ('#ARIAB',1,LGVARE)
         CALL TBAR1  (ERCODE,'#ARIAB',1,MCVARE)
         GOTO 11
      ENDIF
*
*--   Lecture des blocs d'elements consecutifs de meme type
*
      LGBLOK=0
      LGEBLK=3
      INCBLK=2
      COMBAK=' '
      MCBLOK=1
      CALL TAZERO (7,'E',1,AST,NBLTYP,RST,CST)                          !utilite
      CALL TAZERO (7,'E',1,AST,NULTYP,RST,CST)                          !utilite
      ERCODE(1:1)='!'
12    CALL TBAR2  (ERCODE,'#LOKEL',1,MCBLOK,'#ARIAB',1,MCVARE)
      CALL LCDESL (NDIM,NBTEL,NBPTEX,NBPTFX,INTRPG,NIVENG,NBBBLK,NU1BLK
     &            ,LGEBLK,INCBLK,LGBLOK,IST(MCBLOK),AST(MCVARE+LGVARE)
     &            ,NBLTYP,NULTYP,NIVIMP,LGFORM,COMBAK
     &            ,NADBLK,IMPMES,IMPSDR)                                !lecgeom
      IF (COMBAK.EQ.'g') THEN
*        Le tableau #LOKEL doit etre agrandi avant introduction d'un
*        nouveau bloc d'elements :
         CALL TBAJST ('#LOKEL',1,LGBLOK)
         GOTO 12
      ELSEIF (COMBAK.EQ.'c') THEN
*        Le tableau #LOKEL doit etre cree
         CALL TBCREE ('#LOKEL',1,1,LGBLOK,'c')
         GOTO 12
      ENDIF
      IF (NBBBLK.GE.2) CALL TBAJST ('#LOKEL',1,LGBLOK)
*
*--   Boucle sur les elements par blocs d'elements consecutifs de meme type
*       (Lecture des coordonnees et de la numerotation globale de points)
*
      LGNUMG=NBTEL*NBPTEX
      LGCORG=NBTEL*(1+NDIM*NBPTEX)
      CALL TBCREE ('#GNEEL',NIVENG,1,LGNUMG,'c')
      CALL TBCREE ('#ORPTL',0,2,LGCORG,'c')
      CALL TBCREE ('$WORK$',0,2,NDIM*NBPTEX,'c')
      CALL TBAR4  (ERCODE,'#GNEEL',NIVENG,MCNUMG,'#ORPTL',  0   ,MCCORG
     &                   ,'$WORK$',  0   ,MCWOR0,'#ARIAB',  1   ,MCVARE)
      MCBLOK=MCNUMG
      IF (NBBBLK.GE.2) CALL TBAR1  (ERCODE,'#LOKEL',1,MCBLOK)
      CALL TAZERO (LGNUMG,'E',MCNUMG,AST,IST,RST,CST)                   !utilite
*
      CALL RDELEM (LEC,IST(MCBLOK),NDIM,NBBBLK,NU1BLK,NBTEL,IST(MCCORG)
     &            ,RST(MCCORG),FORMAC,IST(MCNUMG),FORMAN,NBPTEX,NBTPTG
     &            ,COMENT,RST(MCWOR0),JACOBF,JACOBM)                   !lecgeom
      IF (IMPSDR.GT.0.AND.NIVIMP.GT.0)
     &   CALL PRELEM (IST(MCBLOK),NDIM,NBBBLK,NU1BLK,NBTEL,RST(MCCORG)
     &               ,NBPTEX,FORMAC,IST(MCNUMG),FORMAN,LGFORM
     &               ,AST(MCVARE+LGVARE),NBCNVE,AST(MCVARE)
     &               ,NIVIMP,IMPSDR)                                    !lecgeom
*
      CALL TBTUER ('$WORK$',0)
      CALL TBAJST ('#ARIAB',1,NDIM*NBCNVE)
*
*--   Description des domaines par element ou par couple (element,face)
*
      CALL TBRR1  (ERCODE,'#OMDOM',1,LGMDOM)                     
      CALL TBAR1  (ERCODE,'#TERDO',1,MCTRDO)                     
      CALL SDEXDB (IST(MCTRDO),LGERDO,NBDOMK,INCRDO,NCHDOM)             !utilite
      CALL REDLE
*
20    IF (IRE.NE.3) CALL REDERR (13,' ')
      IF (IMPMES.GT.0.AND.NIVIMP.GT.0)
     &   WRITE (IMPMES,10000)'*LcGmai*',TEXTE(1:NCAR)
*
      IF (TEXTE(1:6).EQ.'DOMAIN') THEN
*
*        Lecture du nom du domaine (donne par l'utilisateur)
*
         COMBAK=' '
21       CALL TBAR1  (ERCODE,'#OMDOM',1,MCNOMD)
         CALL LCNOMD (NCHDOM,AST(MCNOMD),LGMDOM,NBDOMG,COMBAK
     &               ,NUDOMG,NBCARG,NIVIMP,IMPMES)                      !lecgeom
         IF (COMBAK.NE.' ') THEN
            CALL TBAJST ('#OMDOM',1,LGMDOM)
            GOTO 21
         ENDIF
*        Estimation de la taille de 'LISTEL' pour le nouveau domaine
*        Pour les maillages avec peu d'elements, le nombre d'aretes
*        d'un domaine peut-etre plus grand que le nombre d'elements !
*        D'ou un minimum syndical et louchesque de 100.
         LGLIST=MAX(100,8+2*NBTEL)
*
         IF (NUDOMG.EQ.0) THEN
            NUDOMG=NBDOMG
*
*--         Estimation de la taille de 'LISTEL' pour le nouveau domaine
*
            CALL TBCREE ('#ISTEL',NUDOMG,1,LGLIST+NBPTFX,'c')
            CALL TBAR1  (ERCODE,'#ISTEL',NUDOMG,MCLIST)
            IST(MCLIST)=6
            IST(MCLIST+1)=99999
            DO 22 K=MCLIST+2,MCLIST+6
               IST(K)=0
22          CONTINUE
         ELSE
            CALL TBRR1  (ERCODE,'#ISTEL',NUDOMG,MCLIST)
            CALL TBAJST ('#ISTEL',NUDOMG,LGLIST+NBPTFX)
            CALL TBAR1  (ERCODE,'#ISTEL',NUDOMG,MCLIST)
         ENDIF
*
         MCBLOK=MCLIST
         IF (NBBBLK.GE.2) CALL TBAR1 (ERCODE,'#LOKEL',1,MCBLOK)
         MCDLFA=MCLIST+LGLIST
*
         CALL LCDOMA (IST(MCLIST),NBTEL,IST(MCBLOK),IST(MCDLFA),IRCODE) !lecgeom
         IF (IRCODE.NE.0) THEN
            CALL TBAR1  (ERCODE,'#OMDOM',1,MCNOMD)
            DO 90000 K=1,NBCARG
               TEXTE(K:K)=AST(MCNOMD-1+NCHDOM*(NUDOMG-1)+K)
90000       CONTINUE
         ENDIF
         IF (IRCODE.GT.0) THEN
            IF (IST(MCLIST+1).EQ.IST(MCLIST+2)) THEN
               CALL ERDOMA (8,TEXTE(1:NBCARG))
            ELSE
               CALL ERDOMA (4+IST(MCLIST+1),TEXTE(1:NBCARG))
            ENDIF
         ELSEIF (IRCODE.LT.0) THEN
            IF (IST(MCLIST+1).NE.IST(MCLIST+2)) THEN
               CALL ERDOMA (9,TEXTE(1:NBCARG))
            ELSE
               CALL ERDOMA (7,TEXTE(1:NBCARG))
            ENDIF
         ENDIF
*
*        Test du depaseement de capacite de LISTEL
         IF (IST(MCLIST+1).GT.LGLIST) THEN
            CALL ENCLER (LGLIST,ERCODE(1:6))
            CALL BAISE  ('Longueur #ISTEL '//ERCODE(1:6)//' depassee.')
         ENDIF
*
         IF (IMPSDR.GT.0.AND.NIVIMP.GT.0)
     &      CALL PRLIST (NUDOMG,NCHDOM,IMPSDR)                          !prsd
         CALL TBAJST ('#ISTEL',NUDOMG,IST(MCLIST))
         CALL TBSAVE ('#ISTEL',NUDOMG)
         GOTO 20
*
*
      ELSEIF (TEXTE(1:4).EQ.'COMM') THEN
*
         LGCOMM=NBTPTG
         CALL TBCREE ('#COMMU',1,1,LGCOMM,'c')
         CALL TBAR2  (ERCODE,'#COMMU',1,MCCOMM,'#OMDOM',1,MCNOMD)
         CALL LCCOMM (LGCOMM,IST(MCCOMM),NBDOMG,NCHDOM,AST(MCNOMD)
     &               ,NIVIMP,IMPMES)                                    !lecgeom
         IF (IMPSDR.GT.0.AND.NIVIMP.GT.0) THEN
            WRITE (IMPSDR,11001)'*LcGmai*'
            JP=MCCOMM
            DO 11000 IP=1,IST(MCCOMM)
               NUDOMG=IST(JP+2)
               NBLIST=IST(JP+3)
               ADNOMD=MCNOMD+NCHDOM*(NUDOMG-1)
               WRITE (IMPSDR,11002) IP,IST(JP+1),NUDOMG
     &                             ,(AST(K),K=ADNOMD,ADNOMD+NCHDOM-1)
               WRITE (IMPSDR,11003) (IST(JP+3+K),K=1,NBLIST)
               JP=JP+NBLIST+3
11000       CONTINUE
         ENDIF
         IF (LGCOMM.LT.NBTPTG) CALL TBAJST ('#COMMU',1,LGCOMM)
         CALL TBSAVE ('#COMMU',1)
         GOTO 20
*
      ELSEIF (TEXTE(1:3).NE.'FIN'.AND.TEXTE(1:3).NE.'END') THEN
*
         CALL REDERR (4,'DOMAINE')
      ENDIF
*
*     Mise ajour des structures
*
      CALL TBAJST ('#OMDOM',1,NCHDOM*NBDOMG)                     
      CALL TBAR1  (ERCODE,'#TERDO',1,MCTRDO)                     
      CALL SDEXDB (IST(MCTRDO),LGERDO,NBDOMK,INCRDO,NBQUOI)             !utilite
      CALL SDMKDB (IST(MCTRDO),LGERDO,NBDOMK,INCRDO,NCHDOM)             !utilite
*
      CALL PRFXMJ (-1,'*LcGmai*')
      RETURN
*-----------------------------------------------------------------------
10000 FORMAT(T3,A8,' Lecture de la sous-directive :',A)
10002 FORMAT(/T2,A8,' ',6('*'),' Debut de lecture du maillage ',68('*'))
10006 FORMAT(T3,A8,T56,' , suivi du nombre : ',I12)
*
11001 FORMAT(T5,A8,' Tableau #COMMU de communication inter-processeurs')
11002 FORMAT(T11,I3,') Processeur voisin :',I4,'  Domaine : ',50A1)
11003 FORMAT((T11,20I5))
                                                                     END !LcGmai
