*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE PTRBMS (NBNEUL,RGNELI,NBNEUC,RGNECO,NULAPL,IPLAGE
     &                  ,SYMSTR,PTRDBL,LGLIGN,LGCOLO,PTRMOR)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Mars 2002) 
* Derniere modification : D.Martin (26 juin 2002)
* Version 1.0.2
*  
* Calcul des pointeurs des coefficients d'une matrice elementaire dans 
* les parties triangulaires inferieure et superieure d'une matrice stockee 
* Bi-Morse a structure symetrique
*  
*-- Arguments d'entree --
* NBNEUL nombre de noeuds dans l'element pour l'interpolation en ligne
* RGNELI rang des noeuds en ligne de l'element dans la 
*        numerotation des noeuds pour l'interpolation en ligne du domaine
* NBNEUC nombre de noeuds dans l'element pour l'interpolation en colonne 
* RGNECO rang des noeuds en colonne de l'element dans la 
*        numerotation des noeuds pour l'interpolation en colonne du domaine  
* NULAPL tableau des numeros de derniere plage des lignes
* IPLAGE tableau des numeros de colonne de debut de plage
*        et des adresses des derniers coefficients de chaque plage
* SYMSTR indicateur de structure symetrique
*-- Arguments de sortie --
* PTRDBL pointeurs des coefficients de debut des lignes dans la partie triang.
*        inferieure de la matrice compactee
* LGLIGN tableau des longueurs des lignes de la partie triang. inferieure
* LGCOLO tableau des longueurs des lignes de la partie triang. superieure
* PTRMOR tableau des adresses relatives a partir du debut de ligne des
*        coefficients de la matrice elementaire dans la partie triangulaire
*        inferieure de la matrice
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      LOGICAL SYMSTR
      INTEGER NBNEUL,RGNELI(*),NBNEUC,RGNECO(*),NULAPL(*),IPLAGE(2,*)
     &       ,PTRDBL(*),LGLIGN(*),LGCOLO(*),PTRMOR(NBNEUL,*)
*
      INTEGER NELOCL,LIGNE,NUPRPL,NUDRPL,LGCUDB,LAPLAG,LAPLAV
     &       ,NELOCC,KOLON,KOLOAV,NCOMPT
*----------------------------------------------------------------------- 
*
      DO 20 NELOCL=1,NBNEUL
*
         LGLIGN(NELOCL)=0
         DO 10 NELOCC=1,NBNEUC
            PTRMOR(NELOCL,NELOCC)=0
10       CONTINUE
*
         LIGNE=RGNELI(NELOCL)
         NUPRPL=1  
         LGCUDB=0
         IF (LIGNE.GT.1) THEN 
            NUDRPL=NULAPL(LIGNE-1)  
            IF (LIGNE.GT.2) THEN
               NUPRPL=NULAPL(LIGNE-2)+1
               IF (NUPRPL.GT.1) LGCUDB=IPLAGE(2,NUPRPL-1)  
            ENDIF
            IF (NUDRPL.GE.NUPRPL) THEN
               LGLIGN(NELOCL)=IPLAGE(2,NUDRPL)-LGCUDB
               LAPLAV=0
               KOLOAV=-99999
               DO 11 NELOCC=1,NBNEUC
                  KOLON=RGNECO(NELOCC)
                  IF (LIGNE.GT.KOLON) THEN
*                    Recherche dichotomique du numero de la plage de 
*                    la ligne du stockage Bi-Morse contenant le noeud KOLON
                     CALL DIKLPL (KOLON,NUPRPL,NUDRPL,LGCUDB,IPLAGE
     &                           ,LAPLAG,*1000)
                     IF (LAPLAG.EQ.LAPLAV) THEN
                        NCOMPT=NCOMPT-KOLOAV+KOLON
                     ELSE
                        NCOMPT=KOLON+1-IPLAGE(1,LAPLAG)
                        IF (LAPLAG.GT.1) 
     &                     NCOMPT=NCOMPT+IPLAGE(2,LAPLAG-1)
                        LAPLAV=LAPLAG
                     ENDIF
                     KOLOAV=KOLON
                     PTRMOR(NELOCL,NELOCC)=NCOMPT-LGCUDB
                  ENDIF
11             CONTINUE
            ENDIF
         ENDIF
         PTRDBL(NELOCL)=LGCUDB+1
20    CONTINUE 
*
*     Symetrisation des tableaux de pointeurs, le cas echeant
*
      IF (SYMSTR) THEN
         DO 41 NELOCL=1,NBNEUL
            LGCOLO(NELOCL)=LGLIGN(NELOCL)
            DO 40 NELOCC=1,NBNEUC
               IF (PTRMOR(NELOCL,NELOCC).GT.0) 
     &            PTRMOR(NELOCC,NELOCL)=PTRMOR(NELOCL,NELOCC)
40          CONTINUE
41       CONTINUE
      ENDIF
      RETURN
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
1000  WRITE(*,*) ' *PtrBms* --> Echec de la recherche dichotomique'
      WRITE(*,*) '    NUPRPL,NUDRPL,LGLIGN',NUPRPL,NUDRPL,LGLIGN(NELOCL)
      DO 1001 LAPLAV=NUPRPL,NUDRPL
1001  WRITE(*,*) '    LAPLAV,IPLAGE(1 et 2,IPLA)'
     &,LAPLAV,IPLAGE(1,LAPLAV),IPLAGE(2,LAPLAV)
      WRITE(*,*) '    LIGNE,KOLON,LAPLAG,LGCUDB'
     &,LIGNE,KOLON,LAPLAG,LGCUDB
      WRITE(*,*) '    NELOCC,NCOMPT',NELOCC,NCOMPT
      CALL BAISE(' *PtrBms* ne trouve pas ses moutons ...')
                                                                    END !PtrBms
