*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE PLXPLA (TYPEMA,NBLIMA,NBCOMA,STOIMA,MCMATA
     &                  ,TYPEMB,NBCOMR       ,STOIMB,MCMATB
     &                                       ,STOIMR,MCMATR,RST,CST)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (14 octobre 2006)
* Derniere modification : D.Martin (14 decembre 2006)
* Version 0
*
* Produit A x B* de 2 matrices rectangulaires Pleines A et B 
*                        R_{ij} = A_{ik} * B*_{kj}
*
* Les matrices A et B reelles ou complexes peuvent etre stockees sous forme
* PLEIN-L, PLEIN-C et de meme pour la matrice Resultat R.
*
* On suppose que la matrice R est adressee dans le super-tableau du bon type.
*
*-- Arguments d'entree --
* TYPEMA type (REEL ou COMPLEXE) de la matrice A
* NBLIMA nombre de lignes de la matrice A et de lignes du resultat R
* NBCOMA nombre de colonnes de la matrice A et de colonnes de la matrice B
* STOIMA type de stockage de la matrice A (PLEIN-L ou PLEIN-C)
* MCMATA adresse de la matrice A
* NBCOMR nombre de lignes de la matrice B et de colonnes du resultat R
* TYPEMB type (REEL ou COMPLEXE) de la matrice B
* STOIMB type de stockage de la matrice B (PLEIN-L ou PLEIN-C)
* MCMATB adresse de la matrice B
* STOIMB type de stockage de la matrice R (PLEIN-L ou PLEIN-C)
* MCMATR adresse de la matrice resultat R
* RST,CST super tableau, version REEL et COMPLEXE
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPEMA,TYPEMB,STOIMA,STOIMB,STOIMR
      INTEGER NBLIMA,NBCOMA,MCMATA,NBCOMR,MCMATB,MCMATR
      REAL    RST(*)
      COMPLEX CST(*)
*
      INTEGER INCMAC,INCMAL,INCMBL,INCMBC,INCMRC,INCMRL,I,J,K
     &       ,AIK1,AIK,BKJ1,BKJ,RIJ,RIJ1
      REAL    RRIJ
      COMPLEX CRIJ
*-----------------------------------------------------------------------
*     Increments de changement de ligne ou de colonne selon le stockage
*
*     INCMAC : increment d'adresse pour le changement de colonne dans A
*     INCMAL : increment d'adresse pour le changement de ligne dans A
      INCMAL=1
      INCMAC=1
      IF (STOIMA.EQ.'PLEIN-C') THEN
         INCMAC=NBLIMA
      ELSEIF (STOIMA.EQ.'PLEIN-L') THEN
         INCMAL=NBCOMA
      ELSE
         CALL BAISE  ('PLxPLa: Matrice A non  PLEIN-L ou PLEIN-C.')
      ENDIF
*     INCMBL : increment d'adresse pour le changement de ligne dans B*
*     INCMBC : increment d'adresse pour le changement de colonne dans B*
      INCMBL=1
      INCMBC=1
      IF(STOIMB.EQ.'PLEIN-C') THEN
         INCMBL=NBCOMR
      ELSEIF (STOIMB.EQ.'PLEIN-L') THEN
         INCMBC=NBCOMA
      ELSE
        CALL BAISE  ('PLxPLa: Matrice B non  PLEIN-L ou PLEIN-C.')
      ENDIF
*     INCMRL : increment d'adresse pour le changement de ligne dans R
*     INCMRC : increment d'adresse pour le changement de colonne dans R
      INCMRC=1
      INCMRL=1
      IF(STOIMR.EQ.'PLEIN-C') THEN
         INCMRC=NBLIMA
      ELSEIF (STOIMR.EQ.'PLEIN-L') THEN
         INCMRL=NBCOMR
      ELSE
         CALL BAISE  ('PLxPLa: Matrice R non  PLEIN-L ou PLEIN-C.')
      ENDIF
*
      AIK1=MCMATA
      RIJ1=MCMATR
*
      IF (TYPEMA(1:1).EQ.'C') THEN
*  
         IF (TYPEMB(1:1).EQ.'C') THEN
*
*           Matrice A complexe et matrice B complexe ==> R complexe
*
            DO 443 I=1,NBLIMA
*              Adresse du 1er coeff. de la ligne I de R
               RIJ=RIJ1
*              Adresse du 1er coeff. de la colonne 1 de B*
               BKJ1=MCMATB
               DO 441 K=1,NBCOMR
                  CRIJ=0.
*                 Adresse du 1er coeff. de la ligne I de A
                  AIK=AIK1
                  BKJ=BKJ1
*                 Produit scalaire de la ligne I de A par la colonne K de B*
                  DO 440 J=1,NBCOMA
                     CRIJ=CRIJ+CST(AIK)*CONJG(CST(BKJ))
*                    Adresse du coeff. suivant sur la ligne I de A
                     AIK=AIK+INCMAC
                     BKJ=BKJ+INCMBL
440               CONTINUE
*                 Coefficient R_{ik}
                  CST(RIJ)=CRIJ
*                 Adresse du coeff. suivant sur la ligne I de R
                  RIJ=RIJ+INCMRC
*                 Adresse du 1er coeff. sur la colonne suivante de B*
                  BKJ1=BKJ1+INCMBC
441            CONTINUE
*              Adresse du 1er coeff. de la prochaine ligne de A
               AIK1=AIK1+INCMAL
*              Adresse du 1er coeff. de la prochaine ligne de R
               RIJ1=RIJ1+INCMRL
443         CONTINUE
*
         ELSE
*
*           Matrice A complexe et matrice B reelle ==> R complexe
*
            DO 423 I=1,NBLIMA
               RIJ=RIJ1
               BKJ1=MCMATB
               DO 421 K=1,NBCOMR
                  CRIJ=0.
                  AIK=AIK1
                  BKJ=BKJ1
                  DO 420 J=1,NBCOMA
                     CRIJ=CRIJ+CST(AIK)*RST(BKJ)
                     AIK=AIK+INCMAC
                     BKJ=BKJ+INCMBL
420               CONTINUE
                  CST(RIJ)=CRIJ
                  RIJ=RIJ+INCMRC
                  BKJ1=BKJ1+INCMBC
421            CONTINUE
               AIK1=AIK1+INCMAL
               RIJ1=RIJ1+INCMRL
423         CONTINUE
*
         ENDIF
*
      ELSE
*
         IF (TYPEMB(1:1).EQ.'C') THEN  
*
*           Matrice A reelle et matrice B complexe ==> R complexe
*
            DO 243 I=1,NBLIMA
               RIJ=RIJ1
               BKJ1=MCMATB
               DO 241 K=1,NBCOMR
                  CRIJ=0.
                  AIK=AIK1
                  BKJ=BKJ1
                  DO 240 J=1,NBCOMA
                     CRIJ=CRIJ+RST(AIK)*CONJG(CST(BKJ))
                     AIK=AIK+INCMAC
                     BKJ=BKJ+INCMBL
240               CONTINUE
                  CST(RIJ)=CRIJ
                  RIJ=RIJ+INCMRC
                  BKJ1=BKJ1+INCMBC
241            CONTINUE
               AIK1=AIK1+INCMAL
               RIJ1=RIJ1+INCMRL
243         CONTINUE
*
         ELSE
*         
*           Matrices A et B sont reelles ==> R reelle
*
            DO 223 I=1,NBLIMA
               RIJ=RIJ1
               BKJ1=MCMATB
               DO 221 K=1,NBCOMR
                  RRIJ=0.
                  AIK=AIK1
                  BKJ=BKJ1
                  DO 220 J=1,NBCOMA
                     RRIJ=RRIJ+RST(AIK)*RST(BKJ)
                     AIK=AIK+INCMAC
                     BKJ=BKJ+INCMBL
220               CONTINUE
                  RST(RIJ)=RRIJ
                  RIJ=RIJ+INCMRC
                  BKJ1=BKJ1+INCMBC
221            CONTINUE
               AIK1=AIK1+INCMAL
               RIJ1=RIJ1+INCMRL
223         CONTINUE
*
         ENDIF
*
      ENDIF
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 0 : D.Martin (14 octobre 2006)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !PlxPla
