*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE ELDEFI (NDMDOM,NDMDOE,NXLIST,LISTEL,BLOKEL,NUEL,NUFACE
     &                  ,INTRPG,NUTGEO,NBNEEG,NBDLEG,NUTYFG,NBNEFG
     &                  ,NBDLFG,NUDLFG
     &                  ,INTRPC,KC    ,NUTYLC,NBNEEC,NBDLEC,NUTYFC
     &                  ,NBNEFC,NBDLFC,NUNEFC,NUDLFC
     &                  ,INTRPL,KL    ,NUTYLL,NBNEEL,NBDLEL,NUTYFL
     &                  ,NBNEFL,NBDLFL,NUNEFL,NUDLFL
     &                  ,NUTGAV,NUFAAV,NIVIMP,IMPFCH)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Mars 1989)  
* Derniere modification : D.Martin (8 Decembre 1999)
*  
* Calcul des parametres d'interpolation pour l'element en cours
*  
*-- Arguments d'entree -- 
* NDMDOM dimension du domaine geometrique
* NDMDOE dimension du elements supports des constituants du domaine geometrique
* LISTEL tableau des numeros d'elements du domaine si NDMDOM=NDMDOE,
*        tableau des couples (no d'element,no de face/arete) du domaine sinon
* BLOKEL tableau des numeros de type des elements du maillage par blocs
* INTERG indice d'interpolation geometrique des elements
* INTRPC                        en colonne
* INTRPL                        en ligne 
* NUTGAV numero de type de l'element precedent
* NUFAAV numero de face ou de l'arete de l'element precedent (le cas echeant)
* NIVIMP parametre d'impression
* IMPFCH numero d'unite logique du fichier d'impression
*-- Arguments d'entree modifie en sortie --
* NXLIST rang dans LISTEL de l'element ou du couple (element,face) precedent
*-- Arguments de sortie --  
* NUEL   numero de l'element dans le maillage
* NUFACE numero de la face de dimension NDMDOM en cours (le cas echeant)  
* NUTGEO numero de type geometrique de l'element en cours
* NUTYLC                en colonne
* NUTYLL                en ligne
* KCLG   classe de l'element pour l'interpolation geometrique
* KC                                              en colonne
* KL                                              en ligne
* NBNEEG nombre de noeuds de l'element pour l'interpolation geometrique
* NBNEEC                                                    en colonne
* NBNEEL                                                    en ligne
* NBDLEG nombre de d.l. de l'element pour l'interpolation geometrique
* NBDLEC                                                  en colonne
* NBDLEL                                                  en ligne
* NUTYFG numero de type geometrique de la face
* NUTYFC                en colonne
* NUTYFL                en ligne
* NBNEFG nombre de noeuds sur la face (le cas echeant, par defaut = NBNEEG)
* NBNEFC                              (                           = NBNEEC)
* NBNEFL                              (                           = NBNEEL)
* NBDLFG nombre de D.L. sur la face   ( - - - - - - - - - - - - - = NBDLEG)
* NBDLFC                              (                           = NBDLEC)
* NBDLFL                              (                           = NBDLEL)
* NUNEFC tableau de correspondance (cas en colonne) entre:
*            la numerotation locale sur la face ou l'arete et
*            la numerotation locale dans l'element (le cas echeant)
* NUNEFL idem                      (cas en ligne  )
* NUDLFG tableau de correspondance en d.l. (cas geometrique) entre:
*            la numerotation locale sur la face ou l'arete et
*            la numerotation locale dans l'element (le cas echeant)
* NUDLFC idem                              (cas en colonne )
* NUDLFL idem                              (cas en ligne   )
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      INTEGER NDMDOM,NDMDOE,NXLIST,NUEL  ,NUFACE,INTRPG   ,NUTGEO
     &       ,NBNEEG,NBDLEG,NUTYFG,NBNEFG,NBDLFG
     &       ,INTRPC,KC,NUTYLC,NBNEEC,NBDLEC,NUTYFC,NBNEFC,NBDLFC
     &       ,INTRPL,KL,NUTYLL,NBNEEL,NBDLEL,NUTYFL,NBNEFL,NBDLFL
     &       ,NUTGAV,NUFAAV,NIVIMP,IMPFCH
      INTEGER LISTEL(*),BLOKEL(*),NUDLFG(*),NUNEFC(*),NUDLFC(*)
     &       ,NUNEFL(*),NUDLFL(*)
*
      INTEGER NIVAMP,LA,KCLG,NUFFAV
*-----------------------------------------------------------------------
      NIVAMP=NIVIMP
      IF (INTRPC.NE.INTRPG.OR.INTRPL.NE.INTRPG) NIVAMP=0
*  
*     Parametres geometriques de l'element 
*  
      CALL EXSTEL (NDMDOM,NDMDOE,NXLIST,LISTEL,BLOKEL,NUTGEO,INTRPG,NUEL
     &            ,NUTGAV,NUTGEO,LA,KCLG,NBNEEG,NBDLEG,NUFACE,NUFFAV
     &            ,NUTYFG,NBNEFG,NBDLFG,NUDLFG,NUDLFG,NIVAMP,IMPFCH)    !sdexplo
*
*     Parametres de l'element pour l'interpolation en colonne
*  
      IF (INTRPC.EQ.INTRPG) THEN 
         KC=KCLG
         NUTYLC=NUTGEO  
         NBNEEC=NBNEEG  
         NBDLEC=NBDLEG  
         NUTYFC=NUTYFG  
         NBNEFC=NBNEFG  
         NBDLFC=NBDLFG  
      ELSE  
         CALL EXSTEW (NDMDOM,INTRPC,NUTGEO,NUEL,NUTGAV,NUTYLC,LA,KC
     &               ,NBNEEC,NBDLEC,NUFACE,NUFAAV,NUTYFC,NBNEFC,NBDLFC
     &               ,NUNEFC,NUDLFC,NIVIMP,IMPFCH)                      !sdexplo
      ENDIF
*  
*     Parametres de l'element pour l'interpolation en ligne
*  
      IF (INTRPL.EQ.INTRPG) THEN 
         KL=KCLG
         NUTYLL=NUTGEO  
         NBNEEL=NBNEEG  
         NBDLEL=NBDLEG  
         NUTYFL=NUTYFG  
         NBNEFL=NBNEFG  
         NBDLFL=NBDLFG  
      ELSEIF (INTRPL.EQ.INTRPC) THEN
         KL=KC
         NUTYLL=NUTYLC  
         NBNEEL=NBNEEC  
         NBDLEL=NBDLEC  
         NUTYFL=NUTYFC  
         NBNEFL=NBNEFC  
         NBDLFL=NBDLFC  
      ELSE  
         CALL EXSTEW (NDMDOM,INTRPL,NUTGEO,NUEL,NUTGAV,NUTYLL,LA,KL
     &               ,NBNEEL,NBDLEL,NUFACE,NUFAAV,NUTYFL,NBNEFL,NBDLFL
     &               ,NUNEFL,NUDLFL,NIVIMP,IMPFCH)                      !sdexplo
      ENDIF
                                                                    END !Eldefi
