*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE MKGRIL (NMDOMN,ORIGIN,PASX,PASY,NBPTX,NBPTY
     &                  ,RINIT,TYELEM,GEOGRL,NIVIMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : J-L Bellier ( Janvier 1994 ) 
* Derniere modification : D.Martin (4 Juillet 1997)
*
* Creation d'un domaine geometrique de type grille rectangulaire
* ou couronne circulaire. 
* La maille est reguliere.
* Les elements du maillage de la grille sont des quadrangles ou
* des triangles d'ordre 1 ou 2. 
* 
*-- Arguments d'entree -- 
* NMDOMN: nom du domaine a creer
* ORIGIN: point origine de la grille
* PASX  : ecart entre deux abscisses de sommets consecutifs
* PASY  : idem  entre deux ordonnees 
* NBPTX : nombre de points de discretisation en x ( cas rectangulaire)
*         ou selon un rayon (cas des couronnes)
* NBPTY : idem que NBPTX mais sur l'axe des y (cas rectangulaire)
*         ou angulaire (pour les couronnes)
* RINIT : rayon interieur (ne sert que pour les couronnes circulaires)
* TYELEM: type et ordre des elements de la grille
* GEOGRL: geometrie de la grille :
*         --> 'R' : grille rectangulaire
*         --> 'C' : couronne circulaire
* NIVIMP: niveau d'impression
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) NMDOMN,TYELEM,GEOGRL
      REAL          ORIGIN(*)
      LOGICAL       UNTOUR
*
      INCLUDE 'CONTEX'
      INCLUDE 'ALLOC'
*
      CHARACTER     ERCODE*120,FORMAC*52,FORMAN*52
      COMMON/FORMAH/ERCODE
      DATA FORMAC/'(6E12.5)'/,FORMAN/'(18I4)'/
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*MkGril*')
      DEUXPI=8.*ATAN(1.)
*
*     Nombre d'elements de la grille. Tous les elements sont dans 1 seul bloc
*     d'elements de meme type
*     Remarque: on suppose que le nombre maximum de points par element
*     de la grille reste inferieur a NBPTEX (cf /COMMON/GLOBAL dans CONTEX )
      IF (TYELEM(1:2).EQ.'QU')  THEN
         NBELBK=  (NBPTX-1)*(NBPTY-1)
         IF (  NBPTEX.NE.0.AND.
     &       ((NBPTEX.LT.9.AND.TYELEM(3:4).EQ.'02').OR.
     &        (NBPTEX.LT.4.AND.TYELEM(3:4).EQ.'01')    )) GOTO 99991
      ELSEIF (TYELEM(1:2).EQ.'TR') THEN
         NBELBK=2*(NBPTX-1)*(NBPTY-1)
         IF (  NBPTEX.NE.0.AND.
     &       ((NBPTEX.LT.6.AND.TYELEM(3:4).EQ.'02').OR.
     &        (NBPTEX.LT.3.AND.TYELEM(3:4).EQ.'01')    )) GOTO 99991
      ELSE
         CALL BAISE(' Type d''element '//TYELEM(1:2)//' incompatible!') !Utilite
      ENDIF
*
*     Type entier des elements de la grille
      NUTGEO=NUTYEL(TYELEM)
      IF (NDIM.LE.0) NDIM=NUTGEO/10000
*
*     Remplissage du tableau decrivant les blocs d'elements finis 
      ERCODE(1:1)='!'
      CALL TBAR1  (ERCODE,'#LOKEL',1,MCBLOK)
      NBBBLK=NBBBLK+1
*     LGEBLK est pose a 3
      LGEBLK=3
      IF (ERCODE(1:1).EQ.'C') THEN

*        Le tableau n'existe pas : tous les elements deja rencontres,
*        sont de meme type NU1BLK en un seul bloc (NBBBLK=1).
         CALL TBCREE  ('#LOKEL',1,1,LGEBLK+INCBLK*NBBBLK,'c')
         CALL TBAR1  (ERCODE,'#LOKEL',1,MCBLOK)
         IST(MCBLOK  )=LGEBLK 
         IST(MCBLOK+1)=NBBBLK
         IST(MCBLOK+2)=INCBLK
         IST(MCBLOK+LGEBLK)=NBTEL
         IST(MCBLOK+LGEBLK+1)=NU1BLK
      ELSE 
         CALL TBRR1  (ERCODE,'#LOKEL',1,LGBLOK)
         IF (LGBLOK.LT. LGEBLK+INCBLK*NBBBLK) THEN
            LGBLOK=LGBLOK+INCBLK
            CALL TBAJST  ('#LOKEL',1,LGBLOK)
         ENDIF
         CALL TBAR1  (ERCODE,'#LOKEL',1,MCBLOK)
      ENDIF
*
*     Mise a jour du tableau pour le bloc nouveau
      IST(MCBLOK+1)=NBBBLK
      IST(MCBLOK+LGEBLK+(NBBBLK-1)*INCBLK  )=NBELBK
      IST(MCBLOK+LGEBLK+(NBBBLK-1)*INCBLK+1)=NUTGEO
*
      ERCODE(1:1)='!'
      CALL TBRR2  (ERCODE,'#GNEEL',NIVENG,LGNUMG,'#ORPTL',0,LGCORG)
      IF (ERCODE(1:1).EQ.'C') THEN
*        Cas ou il n'a pas ete lu de maillage
         INTRPG=NUTGEO-1000*(NUTGEO/1000)
         NIVENG=INTRPG
         NU1BLK=NUTGEO
         NBTEL =NBELBK
         CALL MAXNBL (INTRPG,NBBBLK,NU1BLK,IST(MCBLOK),NIVENG
     &               ,NBSOMX,NBAREX,NBFACX,NBNEEX,NBPTEX
     &               ,NBNEAX,NBDLAX,NBNEFX,NBPTFX)                      !Sdexplo
         LGNUMG=NBELBK*NBPTEX
*        On cree le tableau de numerotation globale
         CALL TBCREE  ('#GNEEL',NIVENG,1,LGNUMG,'c')
         LGCORG=NBELBK*(1+NDIM*NBPTEX)
*        On cree le tableau des coordonnees globales
         CALL TBCREE  ('#ORPTL',0,2,LGCORG,'c')
*        On cree le tableau des noms de variable d'espace
         LGVARE=43
         NBCNVE=1
         CALL TBCREE ('#ARIAB',1,11,LGVARE,'c')
         CALL TBAR1  (ERCODE,'#ARIAB',1,MCVARE)
         AST(MCVARE)  ='X'
         AST(MCVARE+1)='Y'
         IF (NDIM.EQ.3) AST(MCVARE+2)='Z'
      ELSE
*        Si un maillage a ete lu, on majore les tableaux existants
         CALL TBAJST ('#GNEEL',NIVENG,LGNUMG+NBELBK*NBPTEX)
         CALL TBAJST ('#ORPTL',   0  ,LGCORG+NBELBK*(1+NDIM*NBPTEX))
      ENDIF
*
*     Calcul des numeros globaux des points de la grille 
*     et de leurs coordonnees
*
      CALL TBAR1  (ERCODE,'#GNEEL',NIVENG,MCNUMG)
      MCNUGD=MCNUMG+NBTEL*NBPTEX
      CALL TAZERO (NBELBK*NBPTEX,'ENTIER',MCNUGD,AST,IST,RST,CST)       !Utilite
*
*     Numerotation globale proprement dite.
*     La variable UNTOUR sert a savoir si on a une couronne circulaire 
*     complete, ce qui impose un raccord des numeros des noeuds entre
*     la premiere rangee d'elements et la derniere
*
      UNTOUR=GEOGRL(1:1).EQ.'C'.AND.(NBPTY-1)*PASY.EQ.DEUXPI
      CALL MKGRNE (TYELEM,IST(MCNUGD),NBPTX,NBPTY,NBPTEX,NBTPTG,UNTOUR) !Sdexplo
*
      CALL TBCREE ('FORMES',1,11,50,'c')
      CALL TBAR5  (ERCODE,'#ORPTL',  0   ,MCCORG
     &                   ,'#GNEEL',NIVENG,MCNUMG,'#LOKEL',1,MCBLOK 
     &                   ,'FORMES',  1   ,MCFORM,'#ARIAB',1,MCVARE)
      CALL TAZERO (50,'CA',MCFORM,AST,IST,RST,CST)
      MCCORL=MCCORG+(1+NDIM*NBPTEX)*NBTEL
      MCNUGD=MCNUMG+NBTEL*NBPTEX
      CALL TAZERO (NBELBK*(1+NDIM*NBPTEX),'R',MCCORL,AST,IST,RST,CST)   !Utilite
      NBFORM=0
      LGFORM=0
      CALL RIPCHN (AST(MCFORM),NBFORM,LGFORM,'Quadrangles')             !Utilite
      CALL RIPCHN (AST(MCFORM),NBFORM,LGFORM,'  Triangles')             !Utilite
*
*     Coordonnees et orientation.
      CALL MKGRCO (NUTGEO,NDIM,IST(MCCORL),RST(MCCORL),IST(MCNUGD)       
     &            ,PASX,PASY,NBPTX,NBPTY,ORIGIN,RINIT,NBELBK,NBNEEL
     &            ,NBDLEL,NBPTEX,NBTPTG,GEOGRL)                         !Sdexplo
      IF (NIVIMP.GT.0) THEN
         NBBBL1=1
         CALL PRELEM (IST(MCBLOK),NDIM,NBBBL1,NUTGEO,NBELBK,RST(MCCORL)
     &               ,NBPTEX,FORMAC,IST(MCNUGD),FORMAN,LGFORM 
     &               ,AST(MCFORM),NBCNVE,AST(MCVARE),NIVIMP,IMPSDR)     !Lecgeom
      ENDIF
*
*     Mise a jour du tableau des noms de domaines
      CALL TBRR1  (ERCODE,'#OMDOM',1,LGMDOM)
      CALL TBAR1  (ERCODE,'#TERDO',1,MCTRDO)
      CALL SDEXDB (IST(MCTRDO),LGERDO,NBDOMK,LGCANO,NCHDOM)             !SDexplo
*     Verification de la non-existence du nom lu
      NUDOMG=0
      IF (NBDOMG.NE.0) NUDOMG=KELCHN (NMDOMN,AST(MCMDOM),NBDOMG,NCHDOM) !Utilite
      IF (NUDOMG.NE.0) CALL BAISE  ('ce nom de domaine existe deja')    !Utilite
*     Verification de la longueur de NOMDOM
      NEWCHD=MAX (LEN (NMDOMN),NCHDOM)
      IF (LGMDOM.LT.NEWCHD*(NBDOMG+1)) 
     &   CALL TBAJST('#OMDOM',1,NEWCHD*(NBDOMG+1))
      CALL TBAR1  (ERCODE,'#OMDOM',1,MCMDOM)
      CALL RIPCHN (AST(MCMDOM),NBDOMG,NCHDOM,NMDOMN)                    !Utilite
      CALL SDMKDB (IST(MCTRDO),LGERDO,NBDOMK,LGCANO,NCHDOM)             !SDexplo
*
*     Creation de la structure #ISTEL pour le nouveau domaine.
      LGSTEL=9+2*NBELBK
      CALL TBCREE ('#ISTEL',NBDOMG,1,LGSTEL,'c')
      CALL TBAR1  (ERCODE,'#ISTEL',NBDOMG,MCLIST)
      CALL TAZERO (LGSTEL,'ENTIER',MCLIST,AST,IST,RST,CST)              !Utilite
      IST(MCLIST  )=LGSTEL
      IST(MCLIST+1)=NDIM-1
      IST(MCLIST+2)=NDIM
      IST(MCLIST+3)=NBELBK
      IST(MCLIST+4)=NBNEEL
      IST(MCLIST+5)=NBDLEL
      IST(MCLIST+6)=NBNEEL
      IST(MCLIST+7)=NBDLEL
      NXLIST=MCLIST+8
      DO 50 L=1,NBELBK
         IST(NXLIST)=L+NBTEL
         IST(NXLIST+1)=0
         NXLIST=NXLIST+2
50    CONTINUE
      IF (NIVIMP.GT.0) CALL PRLIST (NBDOMG,NCHDOM,IMPSDR)               !Sdexplo
*
      NORDRE=NUTGEO-(NUTGEO/10)*10
      IF (NORDRE.NE.INTRPG) GOTO 99992
      NBTPTG=NBTPTG+NBPTX*NBPTY
      NBTEL =NBTEL +NBELBK
      CALL TBTUER ('FORMES',1)
      CALL TBSAVE ('#ISTEL',NBDOMG)
*
      CALL PRFXMJ (-1,'*MkGril*')
      RETURN
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
99991 CALL BAISE  ('le nombre maximum de points dans un element du'
     & //' maillage est insuffisant (?)')
99992 CALL BAISE  ('Interpolations differentes entre la nouvelle grille'
     & //' et le maillage pre-existant')
                                                                    END !Mkgril    
