*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE VCPREP (NMVECT,NVVECT,KPVECT,TYVEMO,LIOUCO
     &                  ,NMMATR,NVMATR,MULTIM,NVDSMA
     &                  ,NUINMA,NCPIMA,NVDUMA,NBNEMA,NVNUMA
     &                  ,NMVAUX,NVVAUX,NUVECT,NVDSMV,NBVECT
     &                  ,NMVOUT,NVVOUT,TYVOUT,NIVIMP)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Novembre 1998)
* Modifications : D.Martin  (31 Mars 2005), (8 Mars 2002)
* Version 2
*  
* Adequation des attributs d'un terme vecteur aux attributs d'une matrice,
* en ligne ou en colonne et, le cas echeant, mise a jour de sa structure
* La matrice et le vecteur sont uni-inconnue ou multi-inconnues
* et l'inconnue ou les inconnues sont scalaires ou vectorielles.
*
*-- Exemples d'utilisation --
*1) MatVec : Vecteur operande (vecteur colonne) d'un produit matrice x vecteur
*2) MatVec : Vecteur resultat (vecteur ligne)   d'un produit matrice x vecteur
*3) CdEsse :
*   CdTran : Second membre d'un systeme lineaire (vecteur colonne) 
*            pour elimination d'une condition essentielle
*  
*-- Arguments d'entree -- 
* NMVECT nom du vecteur original, du vecteur modifie si KPVECT=.FALSE.
* NVVECT niveau de ce terme
* KPVECT indicateur de (non-)modification du vecteur original (NMVECT)
*        (KPVECT=.TRUE. dans l'exemple 1, .FALSE pour les exemples 2 et 3)
* TYVEMO type de declaration pour modification du type du second membre
*        (par exemple type de la matrice, d'un vecteur operande, ...)
* LIOUCO 'LIGNE' ou 'COLONNE' pour les attributs en ligne ou colonne
* NMMATR nom de la matrice (pour impression en cas de bug)
* NVMATR son niveau        (pour impression en cas de bug)
* MULTIM 'Y' ou 'N' selon le caractere 'multi-inconnue' ou non de la matrice
* NVDSMA niveau de la structure d'assemblage multi-inconnue de la matrice
* NUINMA numero d'inconnue en ligne/colonne de la matrice (cas uni-inconnue)
* NCPIMA nombre de composantes de l'inconnue en ligne/colonne de la matrice
* NVDUMA niveau de numerotation en d.l. en ligne/colonne de la matrice (idem)
* NBNEMA nombre de noeuds en ligne/colonne de la matrice (cas uni-inconnue) 
*        nombre de lignes/colonnes de la matrice (cas multi-inconnues)
* NVNUMA niveau de numerotation en noeuds en ligne/colonne (cas uni-inconnue)
* NMVAUX nom d'un vecteur auxiliaire lorsque la structure du vecteur 
*        NMVECT,NVVECT doit etre modifiee
* NVVAUX son niveau
* NIVIMP niveau d'impression
*-- Arguments de sortie --
* NUVECT numero de terme du vecteur original
* NVDSMV niveau de la structure d'assemblage $DSMBL du vecteur en sortie
* NBVECT nombre de colonnes pour le cas d'un vecteur multi-colonnes
* NMVOUT nom du tableau contenant le vecteur resultat de la mise a jour (sortie)
*        = NMVAUX si NMVECT existe, doit etre modifie et KPVECT=.TRUE.
*        = NMVECT sinon
* NVVOUT son niveau (NVVAUX ou NVVECT comme ci-dessus)
* TYVOUT type de declaration du vecteur modifie, en sortie
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) NMVECT,NMMATR,TYVEMO,LIOUCO,MULTIM,NMVAUX
     &             ,NMVOUT,TYVOUT
      LOGICAL       KPVECT
      INTEGER       NVVECT,NVMATR,NVDSMA,NUINMA,NCPIMA,NVDUMA
     &             ,NBNEMA,NVNUMA,NVVAUX,NUVECT,NVDSMV,NBVECT,NVVOUT
     &             ,NIVIMP
*  
      INCLUDE 'ALLOC'
      INCLUDE 'CONTEX'
*
      INTEGER       INDASS,INDSTO,INTYKL,INTTYP,KLNIVE,KLTERM           !utilite
      INTEGER       MCDTRM,LGETRM,NBTERM,INCTRM,NCHTRM,INTEMA
     &             ,MCMTRM,MCINCO,LGEINC,NBINCO,INCINC,NCHINC
     &             ,NVWECT,ITVECT,ITYCAL,NUKALE,NUDONN,NVALNE
     &             ,KUINVE,KVDUVE,KBNEVE,KVNUVE,NUINVE,NVDUVE,NBNEVE
     &             ,NVNUVE,KNSTOK,KVSTOK,KVCOVE,IATRDO,ADTERM
     &             ,ITVOUT,LGVECT,NVXXVE,NBXXVE,NVXXMA,NBXXMA,NVINUT
     &             ,MCDSMV,NCPIMM,NCPIVE,INTEVE,NBTNEU,NIVIME
     &             ,MCVECT,MCVAUX,MCNEVE,MCNELI,II
      CHARACTER     TYPINT*8
      LOGICAL       CRETRM
      CHARACTER     ERCODE*120,TYVECT*8,MULTIV,YADLVS,YADLVV
      COMMON/FORMAH/ERCODE
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*VcPrep*')
*
      CALL TBAR3  (ERCODE,'$SDTRM',1,MCDTRM,'#OMTRM',1,MCMTRM
     &                   ,'#NCONU',1,MCINCO)                     
      CALL SDEXDB (IST(MCDTRM),LGETRM,NBTERM,INCTRM,NCHTRM)             !utilite
      CALL SDEXDB (IST(MCINCO),LGEINC,NBINCO,INCINC,NCHINC)             !utilite
*
      NVALNE=0
      NVDSMV=NDFDSM
      NMVOUT=NMVECT
      NVVOUT=NVVECT
      TYVOUT='REEL'
      NUVECT=KLTERM (NMVECT,NVVECT,AST(MCMTRM),IST(MCDTRM))             !sdexplo
*
      IF (NUVECT.LE.0) THEN
*
*        Le vecteur original n'existe pas : creation du vecteur
*        coherent avec les attributs en ligne/colonne de la matrice.
*
         IF (TYVEMO(1:2).EQ.'CO') TYVOUT='COMPLEXE'
         ITVOUT=INTTYP (TYVOUT)                                         !utilite
         ITYCAL=INTYKL ('ASSEMB')                                       !utilite
         IF (MULTIM(1:1).EQ.'Y')  THEN
            NUKALE=-INDASS (' ')                                        !utilite
*           Initialisation d'une structure "multi-inconnue" associee 
*           a la structure multi-inconnue en ligne/colonne de la matrice
            NVDSMV=KLNIVE ()                                            !sdexplo
            CALL MKSDDS (NVDSMA,LIOUCO,0,0,0,0,0,NDFCOR,NVDSMV)         !sdexplo
         ELSE
            CALL SDEXCO (NUINMA,IST(MCINCO),NCPIMM,INTEMA,NBTNEU)       !utilite
            NUKALE=-INDASS (' ')                                        !utilite
         ENDIF
         NVALNE=NCPIMA
         LGVECT=NBNEMA*NVALNE*NBVECT
         CALL CRTERM (NMVECT,NVVECT,LGVECT,ITVOUT,ITYCAL,NUKALE
     &               ,NDFDON,NVALNE,NVDSMV
     &               ,NDFINC,NDFDUM,NBVECT,NDFNUM
     &               ,NUINMA,NVDUMA,NBNEMA,NVNUMA
     &               ,0     ,NDFSTO,NDFCOR,0,NIVIMP,.FALSE.,CRETRM)     !sdexplo
         NUVECT=NBTERM+1
         CALL TBAR1  (ERCODE,NMVECT,NVVECT,MCVECT)
         CALL TAZERO (LGVECT,TYVOUT,MCVECT,AST,IST,RST,CST)             !utilite
*
      ELSE 
*
*        Le terme vecteur existe : ses caracteristiques
*
         ADTERM=LGETRM+INCTRM*(NUVECT-1)
         CALL GETTRM (IST(MCDTRM+ADTERM)  ,NVWECT,ITVECT,ITYCAL,NUKALE
     &               ,NUDONN,NVALNE,NVDSMV,KUINVE,KVDUVE,KBNEVE,KVNUVE
     &               ,NUINVE,NVDUVE,NBNEVE,NVNUVE,KNSTOK,KVSTOK,KVCOVE
     &               ,IATRDO,NIVIME)                                    !sdexplo
         IF (ITYCAL.EQ.NDFCAL) THEN
*
*           Terme non encore calcule : ses caracteristiques ne sont pas definies
*           il herite de celles de la matrice
*
            NVALNE=NCPIMA
            NUINVE=NUINMA
            NVDUVE=NVDUMA
            NBNEVE=NBNEMA
            NVNUVE=NVNUMA
         ELSE
            NBVECT=KBNEVE
            IF (KUINVE.NE.NDFINC) THEN
*
*              Le terme a une inconnue affectee, on conserve ses attributs
*              Est-ce bien raisonnable en particulier si NUINVE <> NUINMA ?
*
               NBVECT=NBNEVE
               NUINVE=KUINVE
               NVDUVE=KVDUVE
               NBNEVE=KBNEVE
               NVNUVE=KVNUVE
            ENDIF
         ENDIF
*        -> ENDIF (ITYCAL.EQ.NDFCAL) THEN
*
         TYVECT=TYPINT (ITVECT)
         TYVOUT=TYVECT  
         IF (TYVEMO(1:2).EQ.'CO') TYVOUT='COMPLEXE'
         ITVOUT=INTTYP (TYVOUT)                                         !utilite
*
*        Initialisation des niveau/nombre de d.l. pour changement
*        de la numerotation du second membre (en noeuds)
*
         NVXXVE=NVNUVE
         NBXXVE=NBNEVE
         NVXXMA=NVNUMA
         NBXXMA=NBNEMA
*
*        Type multi-inconnues ou uni-inconnue du vecteur
*
         MULTIV='N'
         IF (NUINVE.EQ.NDFINC.AND.NVDSMV.NE.NDFDSM) MULTIV='Y'
*
         IF (MULTIV.EQ.'Y') THEN
            IF (MULTIM(1:1).NE.'Y')
     &         CALL ERTERS (3,NMMATR,NVMATR,NMVECT,NVVECT)              !utilite
*
*           Restauration en m.c. de la structure multi-inconnues du vecteur
*
            CALL TBRR1  (ERCODE,'$DSMBL',NVDSMV,MCDSMV)
         ELSE
*           DM : 31/03/2005 2 lignes rajoutees (test) 
            IF (MULTIM.EQ.'Y')
     &         CALL ERTERS (3,NMVECT,NVVECT,NMMATR,NVMATR)              !utilite            
            CALL SDEXCO (NUINVE,IST(MCINCO),NCPIVE,INTEVE,NBTNEU)       !utilite
            CALL SDEXCO (NUINMA,IST(MCINCO),NCPIMM,INTEMA,NBTNEU)       !utilite
            NVINUT=1
            CALL FONUDL (NCPIVE,1,NVNUVE,NVNUVE,NVNUMA,NVNUMA,NVDUMA
     &                  ,NVDUMA,NVDUVE,NVINUT,YADLVS,YADLVV)            !sdexplo
            IF (YADLVS.EQ.'C')
     &         CALL NUM2DL (NCPIVE,NVNUVE,NVDUVE,NCPIVE*NBNEVE,NIVIME)  !sdexplo
*
*              Systeme lineaire uni-inconnue
*
            IF (INTEVE.NE.INTEMA) 
     &         CALL ERTERS (4,NMMATR,NVMATR,NMVECT,NVVECT)              !utilite
*           IF (NCPIVE.GT.1) THEN  ! ---> Modif du 08/03/2002
            IF (NCPIVE.GT.1.AND.NVALNE.GT.1) THEN
*              inconnue vectorielle et vecteur a valeurs vectorielles
               NVXXVE=NVDUVE
               NBXXVE=NBNEVE*NCPIVE
               NVXXMA=NVDUMA
               NBXXMA=NBNEMA*NCPIMA
            ENDIF
         ENDIF
*        -> ENDIF (MULTIV.EQ.'Y')
*
         ERCODE(1:1)='!'
         CALL TBRR1  (ERCODE,NMVECT,NVVECT,LGVECT)
         IF (ERCODE(1:1).EQ.'C') THEN
*           Le terme vecteur original existe mais pas le tableau ...
*           On le cree avec les nouvelles caracteristiques
            LGVECT=NBVECT*NBXXMA
            CALL TBCREE (NMVECT,NVVECT,ITVOUT,LGVECT,'c')
*
*           Mise a jour du vecteur dans la structure $SDTRM
*
            CALL TBAR1  (ERCODE,'$SDTRM',1,MCDTRM)
            CALL PUTTRM (IST(MCDTRM+ADTERM),NVVECT,ITVOUT
     &                  ,ITYCAL,NUKALE,NUDONN,NDFSYM,NVDSMV
     &                  ,NDFINC,NDFDUM,NBVECT,NDFNUM
     &                  ,NUINMA,NVDUMA,NBNEMA,NVNUMA
     &                  ,INDSTO(' '),NDFSTO,NDFCOR,IATRDO,NIVIME)       !sdexplo
            CALL TBAR1  (ERCODE,NMVECT,NVVECT,MCVECT)
            CALL TAZERO (LGVECT,TYVOUT,MCVECT,AST,IST,RST,CST)          !utilite
         ENDIF
*
         IF (ITVOUT.NE.ITVECT.OR.NBXXVE.NE.NBXXMA
     &  .OR.(MULTIM.EQ.'N'.AND.NVNUVE.NE.NVNUMA)) THEN
*
*           Le vecteur n'est pas du bon type (ITVOUT.NE.ITVECT),
*           le vecteur et la matrice n'ont pas autant de lignes/colonnes,
*           ou ont des numerotations de noeuds differentes;
*           On etend le vecteur selon la numerotation en ligne/colonne
*           de la matrice ou on change son type.
*
*           Recopie du vecteur dans le tableau intermediaire.
*
            CALL TBCREE (NMVAUX,NVVAUX,ITVOUT,NBVECT*NBXXMA,'c')
            CALL TBAR2  (ERCODE,NMVAUX,NVVAUX,MCVAUX 
     &                         ,NMVECT,NVVECT,MCVECT)
            CALL TDANST (LGVECT,TYVOUT,MCVECT,IST,RST,CST
     &                         ,TYVOUT,MCVAUX,IST,RST,CST)              !utilite
            IF (MULTIV.EQ.'Y') THEN
*
*              Cas multi-inconnues :
*              Adaptation a la structure en ligne/colonne de la matrice
*
               IF (NBVECT.GT.1) CALL BAISE
     &         ('Vecteur multi-colonnes et multi-inconnues non prevu.')
*
*              Mise a jour de la structure et des coeff. du vecteur
*              en accord avec la structure des lignes/colonnes de la matrice
*
               CALL EXDUVC (NVDSMV,NMVECT,NVVECT,TYVECT
     &                     ,NVDSMA,NMVAUX,NVVAUX,TYVOUT)                !sdexplo
            ELSE
*
*              Cas uni-inconnue :
*              Adaptation a la numerotation en ligne/colonne de la matrice
*
               CALL TBRR2  (ERCODE,'#GNEDO',NVXXVE,MCNEVE 
     &                            ,'#GNEDO',NVXXMA,MCNELI)
               CALL TBAR2  (ERCODE,NMVAUX,NVVAUX,MCVAUX
     &                            ,NMVECT,NVVECT,MCVECT)
               IF (NVNUVE.NE.NVNUMA) THEN
                  CALL TAZERO (NBVECT*NBXXMA,TYVOUT,MCVAUX
     &                        ,AST,IST,RST,CST)                         !utilite
                  CALL TBAR2 (ERCODE,'#GNEDO',NVXXVE,MCNEVE
     &                              ,'#GNEDO',NVXXMA,MCNELI)
                  CALL EXNUVC (NBXXVE,IST(MCNEVE+2),TYVECT,MCVECT,NBVECT
     &                        ,NBXXMA,IST(MCNELI+2),TYVOUT,MCVAUX
     &                        ,RST,CST)                                 !sdexplo
               ENDIF
            ENDIF
*
            IF (KPVECT) THEN
*
*              Le terme vecteur original est restitue intact :
*              le terme de sortie est le vecteur auxiliaire
*
               NMVOUT=NMVAUX
               NVVOUT=NVVAUX
*
            ELSE
*
*              Le terme vecteur original peut etre modifie, et l'a ete :
*              le terme de sortie est le vecteur original mis a jour
*              le vecteur auxiliaire est detruit
*
               IF (ITVOUT.NE.ITVECT.OR.NBXXVE.NE.NBXXMA)
     &            CALL TBCREE (NMVECT,NVVECT,ITVOUT,NBVECT*NBXXMA,'c')
               CALL TBAR2  (ERCODE,NMVAUX,NVVAUX,MCVAUX
     &                            ,NMVECT,NVVECT,MCVECT)
               CALL TDANST (NBVECT*NBXXMA,TYVOUT,MCVAUX,IST,RST,CST
     &                                   ,TYVOUT,MCVECT,IST,RST,CST)    !utilite
               CALL TBTUER (NMVAUX,NVVAUX)
*
*              Mise a jour du vecteur dans la structure $SDTRM
*              on lui affecte les attributs de la matrice.
*
               CALL TBAR1  (ERCODE,'$SDTRM',1,MCDTRM)
               CALL PUTTRM (IST(MCDTRM+ADTERM),NVVECT,ITVOUT
     &                     ,ITYCAL,NUKALE,NUDONN,NVALNE,NVDSMV
     &                     ,NDFINC,NDFDUM,NBVECT,NDFNUM
     &                     ,NUINVE,NVDUMA,NBNEMA,NVNUMA
     &                     ,INDSTO(' '),NDFSTO,NDFCOR,IATRDO,NIVIMP)    !sdexplo
*
            ENDIF
*           -> ENDIF (KPVECT) THEN
         ENDIF
*        -> IF (ITVOUT.NE.ITVECT.OR.NBXXVE.NE.NBXXMA ....) THEN
      ENDIF
*     -> ENDIF (NUVECT.LE.0) THEN
*
      CALL PRFXMJ (-1,'*VcPrep*')
                                                                    END !VcPrep
