*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE BCJ01 (X,J0,J1)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Juin 1993)  
* Derniere modification : D.Martin (29 Octobre 1999)
*  
* Calcul des fonctions de Bessel de 1ere espece d'ordre 0 et 1 
* (J0,J1) pour un argument X complexe
*  
* Bibliographie : HANDBOOK OF MATHEMATICAL FUNCTIONS  
*                 Ed. M.ABRAMOWITZ et I.A. STEGUN  
*                 Chap. 9 : Bessel Functions of Integer Order (p.359-433)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      COMPLEX    X,J0,J1 
*
      INTEGER    N,NZ,IERR
      REAL       UNSURN
      COMPLEX    XSUR2,X2SUR4,NXTERM,X2
      COMPLEX    JN(0:1)
      COMPLEX    UNSURX,HUISUX,S4SURX,XMPIS4,RSQRT,COSO,SINO,P0,P1,Q0,Q1
*-----------------------------------------------------------------------
      IF (ABS(X).LE.2.)  THEN
*
*        Developpements en serie (A.-S. page 360)
*
         XSUR2 =0.5*X
         X2SUR4=XSUR2*XSUR2
         NXTERM =-X2SUR4
         J0=1.+ NXTERM
         N=2
         UNSURN=0.5
         NXTERM=UNSURN*NXTERM
         J1=1.+ NXTERM
*
*        Sommation des series
1        NXTERM=-UNSURN*X2SUR4*NXTERM
         J0=J0 + NXTERM
         N =N+1
         UNSURN=1./N
         NXTERM=UNSURN*NXTERM
         J1=J1 + NXTERM
         IF (ABS(NXTERM).GT.1.E-10) GOTO 1
         J1=XSUR2 * J1
*
      ELSEIF (ABS(X).LT.4.) THEN
*
*        Approximations par fractions rationnelles
*
         X2=X*X
*  
         J0=(((((((( .33848331E-10*X2-.28955253E-7)*X2+.11382537E-4)*X2
     &              -.25800807E-2)*X2+.35152785E+0)*X2-.27784981E+2)*X2
     &              +.11474985E+4)*X2-.19834267E+5)*X2+.81324121E+5)
     &         /((X2+.49676332E+3)*X2+.81324121E+5)
         J1= (((((((-.47427073E-9 *X2+.35236457E-6)*X2
     &              -.11764050E-3)*X2+.21979684E-1)*X2
     &              -.23652384E+1)*X2+.13812534E+3)*X2
     &              -.37365026E+4)*X2+.31625994E+5)
     &         /((X2+.43349302E+3)*X2+.63251987E+5)*X
*
      ELSEIF (ABS(X).GT.8.) THEN
*
*        Developpements asymptotiques de Hankel (A.-S. page 364)
*
         UNSURX=1./X
         HUISUX=8.*UNSURX
         S4SURX=HUISUX*HUISUX
*  
*        Calcul des polynomes P0 et Q0
         P0=(((( .20938872E-6 *S4SURX-.20733706E-5)*S4SURX
     &          +.27345104E-4)*S4SURX-.10986286E-2)*S4SURX)+1. 
         Q0=((((-.93494515E-7 *S4SURX+.76210953E-6)*S4SURX
     &          -.69111476E-5)*S4SURX+.14304887E-3)*S4SURX
     &          -.156241E-1)*HUISUX
*  
*        Calcul des polynomes P1 et Q1
         P1=((((-.24033702E-6 *S4SURX+.24575202E-5)*S4SURX
     &          -.35163965E-4)*S4SURX+.183105E-2)  *S4SURX)+1.
         Q1=(((( .10578741E-6 *S4SURX-.88228987E-6)*S4SURX
     &          +.84491991E-5)*S4SURX-.20026909E-3)*S4SURX
     &          +.46875E-1)*HUISUX
*
*        Calcul de J0 et J1
         XMPIS4=X-.78539816
         RSQRT=.79788456*SQRT (UNSURX)
         COSO=RSQRT*COS (XMPIS4) 
         SINO=RSQRT*SIN (XMPIS4) 
         J0=P0*COSO-Q0*SINO
         J1=P1*SINO+Q1*COSO 
*
      ELSE
*
*        Calcul a l'aide du sous-programme de la librairie Amos
         CALL CBESJ  (X,0,1,2,JN,NZ,IERR)
         IF (NZ.NE.0.OR.IERR.NE.0) GOTO 10001
         J0=JN(0)
         J1=JN(1)
*
      ENDIF
      RETURN
*-----------------------------------------------------------------------
10001 STOP ' *BcJ01* Probleme au retour de CBESJ (LibAmos)'
                                                                    END !BcJ01
