*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE BCJY01 (X,J0,J1,Y0,Y1)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Juin 1993)  
* Derniere modification (D.Martin) le 10 Septembre 1996
*  
* Calcul des fonctions de Bessel de 1ere et 2nde espece d'ordre 0 et 1 
* (J0,J1,Y0,Y1) pour un argument X complexe
*  
* Bibliographie : HANDBOOK OF MATHEMATICAL FUNCTIONS  
*                 Ed. M.ABRAMOWITZ & I.A. STEGUN  
*                 Chap. 9 : Bessel Functions of Integer Order (p.359-433)
*
* Erreurs Maximales absolues/relatives ( pour Re(X) >= 0.)
*         Comparaison avec les routines CBESJ et CBESY d'Amos
*         (calcul pour 10000 valeurs equireparties dans chaque couronne)
*...............................................................................
*|             |      J0(X)    |      J1(X)    |      Y0(X)    |      Y1(X)    |
*...............................................................................
*  0. <|X|<  2. 2.6E-07/2.1E-07 2.4E-07/2.1E-07 5.2E-07/1.9E-06 4.1E-07/1.6E-06
*  2. <|X|<  4. 1.2E-05/6.3E-06 2.6E-05/8.4E-06 1.7E-05/8.9E-06 3.3E-05/7.6E-06
*
*  8. <|X|< 64.        /3.2E-06        /4.8E-06        /2.8E-06        /3.2E-06
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      COMPLEX    X,J0,J1,Y0,Y1 
*
      INTEGER    N,NZ,IERR
      REAL       UNSURN,SHARMO
      COMPLEX    XSUR2,X2SUR4,NXTERM,LNXPGA,X2
     &          ,JN(0:1),YN(0:1),HN(0:1)
     &          ,UNSURX,HUISUX,S4SURX,XMPIS4,RSQRT,COSO,SINO,P0,P1,Q0,Q1
*-----------------------------------------------------------------------
      IF (ABS(X).LE.2.)  THEN
*
*        Developpements en serie (A.-S. page 360)
*
         XSUR2  = 0.5*X
         X2SUR4 = XSUR2*XSUR2
         NXTERM=-X2SUR4
         J0 = 1.+NXTERM
         Y0 =   -NXTERM
         N=2
         UNSURN=0.5
*        SHARMO : Somme partielle de la serie harmonique (1 + 1/2 + 1/3 +.....)
         SHARMO=1+UNSURN
         NXTERM=UNSURN*NXTERM
         J1 = 1. + NXTERM
         Y1 = 1. + SHARMO * NXTERM
*
*        Sommation des series
1        NXTERM=-UNSURN*X2SUR4*NXTERM
         J0 = J0 + NXTERM
         Y0 = Y0 - SHARMO * NXTERM
         N=N+1
         UNSURN=1./N
         SHARMO=UNSURN+SHARMO
         NXTERM=UNSURN*NXTERM
         J1 = J1 + NXTERM
         Y1 = Y1 + SHARMO * NXTERM
         IF (ABS (SHARMO*NXTERM).GT.1.E-10) GOTO 1
*
         J1 = XSUR2  * J1
         LNXPGA = LOG(XSUR2)+.57721566
         Y0 = .63661977 * (Y0+LNXPGA*J0)
         Y1 =-.63661977 * (XSUR2*Y1-LNXPGA*J1+J0/X)
*
      ELSEIF (ABS(X).LT.4.) THEN
*
*        Approximations par fractions rationnelles
*
         X2=X*X
*  
*        Calcul de J0
         J0=(((((((( .33848331E-10*X2-.28955253E-7)*X2+.11382537E-4)*X2
     &              -.25800807E-2)*X2+.35152785E+0)*X2-.27784981E+2)*X2
     &              +.11474985E+4)*X2-.19834267E+5)*X2+.81324121E+5)
     &         /((X2+.49676332E+3)*X2+.81324121E+5)
*        Calcul de J1  
         J1=((((((( -.47427073E-9 *X2+.35236457E-6)*X2-.11764050E-3)*X2
     &              +.21979684E-1)*X2-.23652384E+1)*X2+.13812534E+3)*X2
     &              -.37365026E+4)*X2+.31625994E+5)
     &         /((X2+.43349302E+3)*X2+.63251988E+5)*X
*
         LNXPGA=LOG(X)
*        Calcul de Y0  
         Y0=((((((((-.10822471E-7 *X2+.10431688E-4)*X2-.44319104E-2)*X2
     &              +.10395610E+1)*X2-.13986012E+3)*X2+.10309017E+5)*X2
     &              -.36371586E+6)*X2+.43903566E+7)*X2-.18304580E+7)
     &        /(((X2+.66332162E+3)*X2+.19701913E+6)*X2+.24801510E+8)
         Y0=Y0+.63661977*J0*LNXPGA
*        Calcul de Y1  
         Y1=((((((((-.40215546E-11*X2+.36878633E-8)*X2-.15738841E-5)*X2 
     &              +.39245491E-3)*X2-.59689931E-1)*X2+.53561135E+1)*X2  
     &              -.25571167E+3)*X2+.51367887E+4)*X2-.18915814E+5)
     &         /((X2+.54491912E+3)*X2+.96481151E+5)*X
         Y1=Y1+.63661977*(J1*LNXPGA-1./X)
*
      ELSEIF (ABS(X).LE.8.) THEN
*
*        Calcul a l'aide des sous-programmes d'Amos
*
         CALL CBESY (X,0,1,2,YN,NZ,HN,IERR)
         IF (NZ.NE.0.OR.IERR.NE.0) GOTO 10002
         CALL CBESJ (X,0,1,2,JN,NZ,IERR)
         IF (NZ.NE.0.OR.IERR.NE.0) GOTO 10001
         J0=JN(0)
         J1=JN(1)
         Y0=YN(0)
         Y1=YN(1)
*
      ELSE
*
*        Developpements asymptotiques de Hankel (A.-S. page 364)
*
         UNSURX=1./X
         HUISUX=8.*UNSURX
         S4SURX=HUISUX*HUISUX
*  
*        Calcul des polynomes P0 et Q0
*  
         P0=1.+((((.20938872E-6 *S4SURX-.20733706E-5)*S4SURX
     &            +.27345104E-4)*S4SURX-.10986286E-2)*S4SURX) 
         Q0=  ((((-.93494515E-7 *S4SURX+.76210952E-6)*S4SURX
     &            -.69111476E-5)*S4SURX+.14304888E-3)*S4SURX
     &            -.15625E-1)*HUISUX
*  
*        Calcul des polynomes P1 et Q1 
*
         P1=1.-((((.24033702E-6 *S4SURX-.24575202E-5)*S4SURX
     &            +.35163965E-4)*S4SURX-.18310500E-2)*S4SURX)
         Q1=   ((((.10578741E-6 *S4SURX-.88228987E-6)*S4SURX
     &            +.84491991E-5)*S4SURX-.20026909E-3)*S4SURX
     &            +.46875E-1)*HUISUX

         XMPIS4=X-.78539816
         RSQRT=.79788456*SQRT(UNSURX)
         COSO=RSQRT*COS(XMPIS4) 
         SINO=RSQRT*SIN(XMPIS4)
*
*        Calcul de J0 et J1 
         J0=P0*COSO-Q0*SINO
         J1=P1*SINO+Q1*COSO
*
*        Calcul de Y0 et Y1
         Y0=P0*SINO+Q0*COSO 
         Y1=Q1*SINO-P1*COSO 
      ENDIF
      RETURN
*-----------------------------------------------------------------------
10001 STOP ' *BcJY01* Probleme au retour de CBESJ'
10002 STOP ' *BcJY01* Probleme au retour de CBESY'
                                                                    END !BcJY01
