*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE BEJ01 (X,J0,J1)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Janvier 1990)  
* Derniere modification (D.Martin) le 10 Septembre 1996
*  
* Calcul des fonctions de Bessel de 1ere espece d'ordre 0 et 1 
*                           (J0,J1)
*  
* Bibliographie : HANDBOOK Of MATHEMATICAL FUNCTIONS  
*                 Ed. M.ABRAMOWITZ & I.A. STEGUN  
*                 Chap. 9 : Bessel Functions of Integer Order (p.359-433)  
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      REAL             X,J0,J1
*
      DOUBLE PRECISION DX,DX2,HUISUX,S4SUX2,P0,P1,Q0,Q1,COSO,SINO  
      REAL             ABSX,UNSURX,XMPIS4,RSQRT
*-----------------------------------------------------------------------
      ABSX=ABS (X)
      IF (ABSX.LE.8.)  THEN
*
*        Approximation par fraction rationnelle
*
         J0=1.
         J1=0.
         IF (X.GT.1.E-06)  THEN
            DX=DBLE(X)
            DX2=DX*DX
            J0  =REAL((((((((( .338483314D-10*DX2-.289552532D-7)*DX2
     &                        +.113825372D-4)*DX2-.258008068D-2)*DX2
     &                        +.351527847D0 )*DX2-.277849812D+2)*DX2
     &                        +.114749848D+4)*DX2-.198342667D+5)*DX2
     &                        +.813241206D+5)
     &                  /((DX2+.49676332D+3)*DX2+.813241206D+5))
            J1  =REAL((((((((-.47427072618D-9 *DX2+.35236457401D-6)*DX2
     &                       -.11764050281D-3)*DX2+.21979683808D-1)*DX2
     &                       -.23652383961D+1)*DX2+.13812534164D+3)*DX2
     &                       -.373650260707D+4)*DX2+.31625993793D+5)
     &                 /((DX2+.433493017D+3)*DX2+.6325198765D+5)*DX)
         ENDIF  
      ELSE
*
*        Developpements asymptotiques de Hankel (A.-S. page 364)
*
         UNSURX=1./X
         HUISUX=8.D0*DBLE(UNSURX)
         S4SUX2=HUISUX*HUISUX 
         XMPIS4=X-.78539816
         RSQRT=.79788456*SQRT(ABS(UNSURX))
         COSO=DBLE(RSQRT*COS(XMPIS4))
         SINO=DBLE(RSQRT*SIN(XMPIS4))
*
*        Calcul des polynomes P0 et Q0
         P0=(((( .209388721D-6 *S4SUX2-.207337064D-5)*S4SUX2  
     &          +.273451041D-4)*S4SUX2-.109862863D-2)*S4SUX2)+1.D0
         Q0=((((-.934945152D-7 *S4SUX2+.762109516D-6)*S4SUX2
     &          -.691114765D-5)*S4SUX2+.143048876D-3)*S4SUX2
     &          -.1562499998D-1)*HUISUX
         J0=REAL(P0*COSO-Q0*SINO)
*
*        Calcul des polynomes P1 et Q1
         P1=((((-.2403370190D-6 *S4SUX2+.2457520174D-5)*S4SUX2
     &          -.3516396496D-4)*S4SUX2+.183105D-2)    *S4SUX2)+1.D0
         Q1=(((( .105787412D-6  *S4SUX2-.88228987D-6)  *S4SUX2 
     &          +.8449199096D-5)*S4SUX2-.2002690873D-3)*S4SUX2
     &          +.4687499995D-1)*HUISUX
         J1=REAL(P1*SINO+Q1*COSO)
      ENDIF
                                                                    END !BeJ01
