*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE BESFJN (X,N,J) 
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Janvier 1990)  
* Derniere modification (D.Martin) le 6 Septembre 1996
*  
* Calcul des fonctions de BESSEL spheriques de 1ere espece d'ordre 0 a N  
*  
* Bibliographie : HANDBOOK Of MATHEMATICAL FUNCTIONS  
*                 Ed. M.ABRAMOWITZ & I.A. STEGUN  
*                 Chap. 10 : Bessel Functions of Fractional Order (p.437-494)
*
**     Programme principal de test - 
*      valeur de Abramowitz & Stegun pp. 457, 458
*pp      REAL JN(0:2)  
*pp      IPAGE=457
*pp      WRITE(*,101) IPAGE,'j0(x)','j1(x)','j2(x)'  
*pp      X=0.
*pp      CALL BESFJN(X,2,JN)  
*pp      WRITE(*,100) X,(JN(K),K=0,2)
*pp      DO 1 I=1,50
*pp      X=X+0.1 
*pp      CALL BESFJN(X,2,JN)
*pp1     WRITE(*,100) X,(JN(K),K=0,2)
*pp100   FORMAT(T2,F5.2,3F15.8)
*pp101   FORMAT('1'//' Page',I4//TR3,'x  ',3(A13,TR2)/)
*pp      IPAGE=458
*pp      WRITE(*,101) IPAGE,'j0(x)','j1(x)','j2(x)'
*pp      X=5.0
*pp      CALL BESFJN(X,2,JN)  
*pp      WRITE(*,200) X,(JN(K),K=0,2)
*pp      DO 2 I=1,50
*pp      X=X+0.1 
*pp      CALL BESFJN(X,2,JN)
*pp2     WRITE(*,200) X,(JN(K),K=0,2)
*pp200   FORMAT(T2,F5.2,3(1PF15.8))
*pp201   FORMAT('1'//' Page',I4//TR3,'x   ',7(A11,TR2)/)
*pp      END 
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      INTEGER  N
      REAL     X,J(0:N)
*
      INTEGER  I,K,NINIT
      REAL     J0,JN,JNP1,JNP2,TERMEK,XI,X2IP1,X2IPKM,X2SUR,Z2,COEFF
*-----------------------------------------------------------------------
      IF (X.LT.1.E-06) THEN 
          J(0)=1.  
          DO 5 I=1,N
             J(I)=0.
5         CONTINUE  
*  
      ELSEIF (X.LT.1.) THEN
*  
*        Calcul par developpement en serie :
*  
*         Initialisation du facteur X**i/(1x3x...(2i+1))
          XI=1.
*         Initialisation de 2i+1
          X2IP1=1. 
          X2SUR=-X*X/2.
          DO 10 I=0,N  
             J(I)=1.  
             K=0  
             TERMEK=1.
             X2IPKM=X2IP1 
*            Calcul du terme courant de la serie  
12           K=K+1
*            Calcul de 2i+2k+1 
             X2IPKM=X2IPKM+2.  
*            Calcul de (-x*x/2)**k/(k!(2i+3)x(2i+5)..x(2i+2k+1)
             TERMEK=X2SUR*TERMEK/(K*X2IPKM)
             J(I)=J(I)+TERMEK  
             IF (ABS(XI*TERMEK).GT.1.E-9) GOTO 12  
             J(I)=XI*J(I) 
             X2IP1=X2IP1+2.
             XI=XI*X/X2IP1
10        CONTINUE
      ELSE 
*  
*      Calcul par recurrence regressive (Cf.A-S page 452 :MILLER's method) 
*  
         IF (X.LE.5.) THEN
            NINIT=INT (4.*X+21.)
         ELSEIF (X.LE.10) THEN  
            NINIT=INT (2.4*X+28.)
         ELSEIF (X.LE.20.) THEN 
            NINIT=INT (2.*X+32.)
         ELSEIF (X.LE.50.) THEN 
            NINIT=INT (1.5*X+42.)
         ELSE
            NINIT=INT (1.3*X+52.)
         ENDIF 
         DO 19 I=NINIT,N
            J(I)=0.
19       CONTINUE
         JNP2=0.  
         JNP1=1.E-30  
         IF (NINIT.LE.N) J(NINIT)=JNP1 
         X2IP1=NINIT+NINIT+1.
         DO 20 I=NINIT-1,0,-1
            JN=X2IP1*JNP1/X-JNP2
            X2IP1=X2IP1-2. 
            JNP2=JNP1 
            JNP1=JN
            IF (I.LE.N) J(I)=JN
            IF (ABS(JN).GT.1.E+35) THEN
               Z2=1.E+35/JN
               JNP2=Z2*JNP2
               JNP1=Z2*JNP1
               DO 21 K=I,N
                  J(K)=J(K)*Z2
21             CONTINUE
            ENDIF
20       CONTINUE
         J0=SIN(X)/X  
         COEFF=J0/J(0)
         J(0)=J0  
         DO 13 I=1,N  
            J(I)=J(I)*COEFF
13       CONTINUE  
      ENDIF
                                                                    END !Besfjn
