*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE BEY01 (X,Y0,Y1)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Septembre 1996)  
* Derniere modification (D.Martin) le 16 Septembre 1996
*  
* Calcul des fonctions de Bessel de 2nde espece d'ordre 0 et 1 (Y0,Y1)
*  
* Bibliographie : HANDBOOK Of MATHEMATICAL FUNCTIONS  
*                 Ed. M.ABRAMOWITZ & I.A. STEGUN  
*                 Chap. 9 : Bessel Functions of Integer Order (p.359-433) 
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      REAL             X,Y0,Y1 
*
      REAL             J0,J1,AXLOG,UNSURX,XMPIS4,RSQRT
      DOUBLE PRECISION DX2,HUISUX,S4SUX2,P0,P1,Q0,Q1,DX,COSO,SINO
*----------------------------------------------------------------------
      IF (X.LE.8.)  THEN
*
*        Approximation par fraction rationnelle
*
         DX=DBLE(X)
         DX2=DX*DX
*        Calcul de J0 et J1
         CALL BEJ01  (X,J0,J1)
         AXLOG=.63661977*LOG(X)
*        Calcul de Y0 et Y1 
         Y0=REAL(((((((((-.108224715D-7 *DX2+.104316882D-4)*DX2
     &                   -.4431910446D-2)*DX2+.1039561053D+1)*DX2
     &                   -.1398601246D+3)*DX2+.10309017074D+5)*DX2
     &                   -.36371585984D+6)*DX2+.43903566116D+7)*DX2
     &                   -.18304579902D+7)
     &            /(((DX2+.663321623D+3)*DX2+.19701912582D+6)*DX2
     &                   +.2480151036D+8))
     &     +J0*AXLOG
         Y1=REAL(((((((((-.40215545652D-11*DX2+.36878633134D-8)*DX2  
     &                   -.15738840726D-5)*DX2+.39245490948D-3)*DX2  
     &                   -.59689930762D-1)*DX2+.53561134713D+1)*DX2  
     &                   -.25571167226D+3)*DX2+.51367887399D+4)*DX2
     &                   -.1891581383D+5) 
     &             /((DX2+.5449191171D+3)*DX2+.96481151335D+5)*DX)
     &     +J1*AXLOG-.63661977/X 
      ELSE 
*
*        Developpements asymptotiques de Hankel (A.-S. page 364)
*
         UNSURX=1./X
         HUISUX=8.D0*DBLE(UNSURX)
         S4SUX2=HUISUX*HUISUX 
         XMPIS4=X-.78539816
         RSQRT=.79788456*SQRT(UNSURX)
         COSO=DBLE(RSQRT*COS(XMPIS4)) 
         SINO=DBLE(RSQRT*SIN(XMPIS4))
*
*        Calcul des polynomes P0, Q0, P1 et Q1
         P0=(((( .209388721D-6* S4SUX2-.207337064D-5)*S4SUX2  
     &          +.273451041D-4)*S4SUX2-.109862863D-2)*S4SUX2)+1.D0
         Q0=((((-.934945152D-7 *S4SUX2+.762109516D-6)*S4SUX2
     &          -.691114765D-5)*S4SUX2+.143048876D-3)*S4SUX2
     &          -.1562499998D-1)*HUISUX
         P1=((((-.240337019D-6* S4SUX2+.2457520174D-5)*S4SUX2
     &          -.351639649D-4)*S4SUX2+.183105D-2)    *S4SUX2)+1.D0 
         Q1=(((( .105787412D-6 *S4SUX2-.88228987D-6)  *S4SUX2 
     &          +.8449199096D-5)*S4SUX2-.2002690873D-3)*S4SUX2
     &          +.4687499995D-1)*HUISUX
         Y0=REAL(P0*SINO+Q0*COSO) 
         Y1=REAL(Q1*SINO-P1*COSO)
      ENDIF
                                                                    END !BeY01
