*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE STCH01 (Z,Y0,Y1,H0,H1)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs: C.Hazard & D.Martin (Decembre 1992)  
* Derniere modification (D.Martin) le 16 Septembre 1996
*  
* Calcul des fonctions de Struve d'ordre 0 et 1 (H0 & H1) en complexe 
*  
* Le sous-programme calcule tout d'abord
*     H0(z) et H1(z) pour Abs(z) < 2.3 (developpement en serie),
*     H0(z)-Y0(z) et H1(z)-Y1(z) sinon (approximation rationnelle),
* ou Y0 et Y1 sont les fonctions de Bessel de 2nde espece d'ordre 0 et 1
*
* et retourne H0(z) et H1(z)-2/Pi si Y0 et Y1 sont donnes pour |z| >= 2.3
*
* Precision verifiee : Developp. en serie  : 10-5 pour Z reel  < 2.3
*                      Approx. rationnelle : 2.10-5 pour Z reel >= 2.3  
*              
* Bibliographie :-HANDBOOK Of MATHEMATICAL FUNCTIONS  
*                 Ed. Milton ABRAMOWITZ & Irene A. STEGUN 
*                 Dover Publications 1970 (9th printing). 
*                 Chap. 12 : Struve Functions and Related Functions (p.495-502)
*                -MATHEMATICAL FUNCTIONS AND THEIR APPROXIMATIONS
*                 Yudell L.LUKE, Academic Press 1975.
*                 Chap. 10 : Lommel and Struve functions (pp. 423...)
*
*-- Arguments d'entree --
* Z  nombre complexe argument des fonctions a calculer
* Y0 =Y0(z) a fournir necessairement pour |z| > 2.3
* Y1 =Y1(z) a fournir necessairement pour |z| > 2.3
*-- Arguments de sortie --
* H0 =H0(z)      ( calcule le cas echeant comme (H0-Y0)+Y0 )
* H1 =H1(z)-2/Pi ( calcule le cas echeant comme (H1-Y1)+Y1 )
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      COMPLEX Z,Y0,Y1,H0,H1
*
      INTEGER N,NBTERM,K
      REAL DESUPI,ZABS,DKP1
      COMPLEX PHI0,F0,PHI1,F1,Z2,Z2K,TERME
      PARAMETER ( N=6 , DESUPI=.63661977)
      INTEGER A0(0:N),B0(0:N),A1(0:N),B1(0:N)
*  
      DATA A0/79053975,368918550,147567420,12046320,278850,2028,4/
     &    ,B0/23310855,285813360,137618550,11784822,276858,2024,4/
     &    ,A1/395269875,1106755650,316215900,20077200,380250,2340,4/
     &    ,B1/45884475,655581600,269806950,19035450,373410,2328,4/
*----------------------------------------------------------------------
      Z2=Z*Z
*
      ZABS=ABS (Z)
      IF (ZABS.LT.2.3) THEN
*
*        Developpement en serie entiere de H0 et H1 (M.Abramowicz-I.A.Stegun)
*
         NBTERM=6
         IF (ZABS.LE.2.0) NBTERM=4
         IF (ZABS.LE.0.8) NBTERM=2
*
         H0=Z
         H1=.33333333*Z2
         TERME=H1
         DKP1=3.
         DO 2 K=1,NBTERM
            TERME=-TERME*Z/DKP1
            H0=H0+TERME
            DKP1=DKP1+2.
            TERME=TERME*Z/DKP1
            H1=H1+TERME
2        CONTINUE
*        Calcul de H0 et de H1-2/Pi
         H0=DESUPI*H0
         H1=DESUPI*(H1-1.)
      ELSE
*
*        Approximation rationnelle de H0-Y0 et H1-Y1 (Y.L.Luke)
*        puis calcul de H0 et de H1-2/Pi
*
         F0=A0(0)
         PHI0=B0(0)
         F1=A1(0)
         PHI1=B1(0)
         Z2K=Z2
         DO 1 K=1,N
            F0=F0+A0(K)*Z2K
            PHI0=PHI0+B0(K)*Z2K
            F1=F1+A1(K)*Z2K
            PHI1=PHI1+B1(K)*Z2K
            Z2K=Z2*Z2K
1        CONTINUE
         H0=Y0+DESUPI*PHI0/(Z*F0)
         H1=Y1+DESUPI*PHI1/(Z2*F1)
      ENDIF
                                                                    END !StcH01
