*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      FUNCTION XBEJ0(X)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : CODY (Mathematics & Computer science division  
*                Argonne Natinal Laboratory)  
* Derniere modification (D.Martin) le 7 Fev. 90
*  
* this routine calculates the bessel function of the first kind
*   and order zero,  j-sub-0(x),  for real arguments x, where  
*   abs(x) .le. xmax.  the computation uses unpublished minimax
*   rational approximations for the first two intervals, and an
*   approximation from the book  computer approximations  by hart, 
*   et. al., wiley and sons, new york, 1968, for arguments larger  
*   than 8.0   this transportable program is patterned after the
*   machine-dependent funpack program BEJ0(x), but cannot match
*   that version for efficiency or accuracy.  this version uses
*   rational functions that theoretically approximate j-sub-0(x)
*   to at least 18 significant decimal digits for x .le. 8, and
*   at least 18 decimal places for x .gt. 8.  the accuracy 
*   achieved depends on the arithmetic system, the compiler, the
*   intrinsic functions, and proper selection of the machine-  
*   dependent constants.
*  
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*  
* explanation of machine-dependent constants
*  
*   pi2 = 2 / pi
*   twopi = 2 * pi 
*   twopi1 + twopi2 = 2 * pi to extra precision
*   xmax = largest acceptable argument.  the functions aint, sin
*          and cos must perform properly for  abs(x) .le. xmax.
*   xsmall = positive argument such that  1.0-(x/2)**2 = 1.0
*            to machine precision for all  abs(x) .le. xsmall. 
*   x0 = first zero of j-sub-0 
*   x01/26 + x02 = first zero of j-sub-0 to extra precision
*   x1 = second zero of j-sub-0
*   x11/256 + x12 = second zero of j-sub-0 to extra precision  
*  
*     approximate values for some important machines are:  
*  
*           ibm       cdc      univac     vax      ibm pc   ibm pc 
*           195       7600      1108     11/780        (8087)  
*          (d.p.)    (s.p.)    (d.p.)    (d.p.)    (s.p.)   (d.p.) 
*  
* xmax    6.87d+09  1.34e+08  4.30d+09  1.07d+09  8.19e+03  2.68d+08
* xsmall  9.09d-13  2.98e-08  2.33d-10  9.31d-10  1.22e-04  3.72d-09
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*  
*  the program returns the value 0 for  x .gt. xmax.
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      DIMENSION PRP(7),PRQ(8),P0(6),P1(6),QRP(5),QRQ(7),Q0(5),Q1(5)
*  mathematical constants  
      DATA PI2/6.3661977236758134308E-01/, 
     1     X0/2.4048255576957727686E+0/,X1/5.5200781102863106496E+0/,  
     2     X01/616.0E+0/,X02/-1.42444230422723137837E-03/, 
     3     X11/1413.0E+0/,X12/5.46860286310649596604E-04/, 
     4     TWOPI/6.2831853071795864769E0/, 
     5     TWOPI1/6.28125E0/,TWOPI2/1.9353071795864769253E-03/ 
*  machine-dependent constants 
      DATA XMAX/8.19E+03/,XSMALL/1.22E-09/ 
*  coefficients for rational approximation of  
*  j-0(x) / (x**2 - x0**2), xsmall .lt. abs(x) .le. 4.0
      DATA PRP/6.6302997904833794242E+06,-6.2140700423540120665E+08,
     1         2.7282507878605942706E+10,-4.1298668500990866786E+11,
     2        -1.2117036164593528341E-01, 1.0344222815443188943E+02,
     3        -3.6629814655107086448E+04/  
      DATA QRP/4.5612696224219938200E+05, 1.3985097372263433271E+08,
     1         2.6328198300859648632E+10, 2.3883787996332290397E+12,
     2         9.3614022392337710626E+02/  
*  coefficients for rational approximation of  
*  j-0(x) / (x**2 - x1**2), 4.0 .lt. abs(x) .le. 8.0
      DATA PRQ/4.4176707025325087628E+03, 1.1725046279757103576E+04,
     1         1.0341910641583726701E+04,-7.2879702464464618998E+03,
     2        -1.2254078161378989535E+04,-1.8319397969392084011E+03,
     3         4.8591703355916499363E+01, 7.4321196680624245801E+02/
      DATA QRQ/3.3307310774649071172E+02,-2.9458766545509337327E+03,
     1         1.8680990008359188352E+04,-8.4055062591169562211E+04,
     2         2.4599102262586308984E+05,-3.5783478026152301072E+05,
     3        -2.5258076240801555057E+01/  
*  coefficients for hart,s approximation, abs(x) .gt. 8.0  
      DATA P0/3.4806486443249270347E+03, 2.1170523380864944322E+04,
     1        4.1345386639580765797E+04, 2.2779090197304684302E+04,
     2        8.8961548424210455236E-01, 1.5376201909008354296E+02/
      DATA Q0/3.5028735138235608207E+03, 2.1215350561880115730E+04,
     1        4.1370412495510416640E+04, 2.2779090197304684318E+04,
     2        1.5711159858080893649E+02/
      DATA P1/-2.2300261666214198472E+01,-1.1183429920482737611E+02,
     1        -1.8591953644342993800E+02,-8.9226600200800094098E+01,
     2        -8.8033303048680751817E-03,-1.2441026745835638459E+00/
      DATA Q1/1.4887231232283756582E+03, 7.2642780169211018836E+03,
     1        1.1951131543434613647E+04, 5.7105024128512061905E+03,
     2        9.0593769594993125859E+01/
*  
      AX = ABS(X)  
      IF (AX .GT. XMAX) GO TO 300  
      IF (AX .GT. XSMALL) GO TO 40 
      XBEJ0 = 1.
      RETURN
   40 IF (AX .GT. 8.) GO TO 200
      IF (AX .GT. 4.) GO TO 100
*  xsmall .lt. abs(x) .le. 4.0 
      ZSQ = AX * AX
      XNUM = (PRP(5) * ZSQ + PRP(6)) * ZSQ + PRP(7)
      XDEN = ZSQ + QRP(5)  
      DO 70 I = 1, 4
         XNUM = XNUM * ZSQ + PRP(I)
         XDEN = XDEN * ZSQ + QRP(I)
   70 CONTINUE 
*  calculation to preserve accuracy
*  near the first zero of j-0  
      PROD = ((AX - X01/256.) - X02) * (AX + X0)
      XBEJ0 = PROD * XNUM / XDEN
      RETURN
*  4.0 .lt. abs(x) .le. 8.0
  100 ZSQ = 1. - (AX * AX) / 64.
      XNUM = PRQ(7) * ZSQ + PRQ(8) 
      XDEN = ZSQ + QRQ(7)  
      DO 120 I = 1, 6  
         XNUM = XNUM * ZSQ + PRQ(I)
         XDEN = XDEN * ZSQ + QRQ(I)
  120 CONTINUE 
*  calculation to preserve accuracy
*  near the second zero of j-0 
      PROD = (AX + X1) * ((AX - X11/256.) - X12)
      XBEJ0 = PROD * XNUM / XDEN
      RETURN
*  abs(x) .gt. 8.0 
  200 Z = 8. / AX  
      WZ = AX / TWOPI
      WZ = AINT(WZ) + .125
      UZ = (AX - WZ * TWOPI1) - WZ * TWOPI2
      ZSQ = Z * Z  
      XNUM = P0(5) * ZSQ + P0(6)
      XDEN = ZSQ + Q0(5)
      UP = P1(5) * ZSQ + P1(6) 
      DOWN = ZSQ + Q1(5)
      DO 250 I = 1, 4  
         XNUM = XNUM * ZSQ + P0(I) 
         XDEN = XDEN * ZSQ + Q0(I) 
         UP = UP * ZSQ + P1(I) 
         DOWN = DOWN * ZSQ + Q1(I) 
  250 CONTINUE 
      R0 = XNUM / XDEN 
      R1 = UP / DOWN
      XBEJ0 = SQRT(PI2/AX) * (R0*COS(UZ) - Z*R1*SIN(UZ)) 
      RETURN
*  error return for abs(x) .gt. xmax
  300 XBEJ0 = 0.
                                                                    END !XBeJ0
