*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE GAUSSC (NBLIGN,MAT,NBSEC,SEC,EPS,CHPIV
     &                  ,PIVMIN,PERMUT,CDETER,SINGUL)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (1980)
* Derniere modification : D.Martin (19 Juin 1996)
* Version 1.0.0
*  
* Resolution d'un systeme lineaire par la methode d'elimination de Gauss
* avec ou sans recherche de pivot maximum partiel ou total 
*                       (Cas complexe)
*  
*-- Arguments d'entree --
* NBLIGN nombre de lignes reel de la matrice MAT
* MAT    la matrice
* NBSEC  nombre de seconds membres
* SEC    le(s) second(s) membre(s) du systeme lineaire en entree,
*        la(les) solution(s) en sortie. Les seconds membres sont ranges
*        consecutivement en memoire centrale
* EPS    seuil  de non-inversibilite portant sur le pivot mimimum
* CHPIV  recherche de pivot maximum , partiel si CHPIV(1:1) = 'P' ;
*                                   , total   si CHPIV(1:1) = 'T' ;
*        sans recherche de pivot maximum sinon
*-- Arguments de sortie --
* PIVMIN module minimal des pivots de l'elimination de GAUSS
* CDETER determinant de la matrice
* SINGUL .TRUE. pour une matrice singuliere
*-- Tableau de travail (le cas echeant) --
* PERMUT tableau de permutation des inconnues en cas de recherche de pivot
*         maximum total(dimension au moins NBLIGN si recherche de pivot total) 
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) CHPIV
      INTEGER       NBLIGN,NBSEC,PERMUT(*)
      REAL          EPS,PIVMIN
      COMPLEX       MAT(NBLIGN,*),SEC(NBLIGN,*),CDETER
      LOGICAL       SINGUL
*
      INTEGER       LIGPIV,KOLPIV,K,I,J,JJ
      REAL          ABSPIV
      COMPLEX       TRUC,PIV
*-----------------------------------------------------------------------
      CDETER=1.
      PIVMIN=1.E+06
      SINGUL=.FALSE.
      IF (CHPIV(1:1).EQ.'T') THEN
         DO 1 I=1,NBLIGN
            PERMUT(I)=I
1        CONTINUE
      ENDIF
*  
*     Triangulation
* 
      DO 30 K=1,NBLIGN-1
      LIGPIV=K 
      KOLPIV=K 
      PIV=MAT(K,K) 
      ABSPIV=ABS (PIV)  
      IF (CHPIV(1:1).NE.'P'.AND.CHPIV(1:1).NE.'T') THEN
*        1 ) sans recherche de pivot maximum 
         IF (ABSPIV.LT.EPS) THEN
            DO 10 I=K+1,NBLIGN 
               LIGPIV=I
               PIV=MAT(I,K)
               ABSPIV=ABS (PIV)
               IF (ABSPIV.GE.EPS) GOTO 20
10          CONTINUE
            GOTO 99990
         ENDIF   
*          2 ) avec recherche de pivot maximum ... 
      ELSE 
*                                              ... partiel
         DO 11 I=K+1,NBLIGN
            IF (ABSPIV.LT.ABS (MAT(I,K))) THEN
               LIGPIV=I
               PIV=MAT(I,K)
               ABSPIV=ABS (PIV)
            ENDIF
11       CONTINUE
         IF (ABSPIV.LT.EPS) GOTO 99990
         IF (CHPIV(1:1).EQ.'T') THEN
*                                              ... total 
            DO 13 I=K+1,NBLIGN
               DO 12 J=K+1,NBLIGN
                  IF (ABSPIV.LT.ABS (MAT(I,J))) THEN
                     LIGPIV=I
                     KOLPIV=J
                     PIV=MAT(I,J)
                     ABSPIV=ABS (PIV)
                  ENDIF
12             CONTINUE 
13          CONTINUE
            IF (KOLPIV.GT.K) THEN
*              Permutation des colonnes K et KOLPIV (colonne du pivot) 
*              et des inconnues correspondantes
               J=PERMUT(KOLPIV)  
               PERMUT(KOLPIV)=PERMUT(K)  
               PERMUT(K)=J
               DO 14 I=1,NBLIGN  
                  TRUC=MAT(I,K) 
                  MAT(I,K)=MAT(I,KOLPIV)
                  MAT(I,KOLPIV)=TRUC
14             CONTINUE
            ENDIF 
         ENDIF  
      ENDIF
*
*     Permutation des lignes K et LIGPIV (ligne du pivot maximum)
*
20    IF (LIGPIV.GT.K) THEN 
         DO 21 J=1,NBSEC
            TRUC=SEC(K,J) 
            SEC(K,J)=SEC(LIGPIV,J)  
            SEC(LIGPIV,J)=TRUC
21       CONTINUE
         DO 22 J=K,NBLIGN 
            TRUC=MAT(K,J)
            MAT(K,J)=MAT(LIGPIV,J)  
            MAT(LIGPIV,J)=TRUC
22       CONTINUE  
      ENDIF
*
*     Elimination de l'inconnue K dans les lignes K+1, .. ,NBLIGN
*
      IF (PIVMIN.GT.ABSPIV) PIVMIN=ABSPIV
      CDETER=CDETER*PIV
      PIV=1./PIV
      DO 29 I=K+1,NBLIGN
         TRUC=-MAT(I,K)*PIV
         DO 27 J=K+1,NBLIGN
            MAT(I,J)=MAT(I,J)+TRUC*MAT(K,J)
27       CONTINUE
         DO 28 J=1,NBSEC
            SEC(I,J)=SEC(I,J)+TRUC*SEC(K,J) 
28       CONTINUE
29    CONTINUE
30    CONTINUE
*
      PIV=MAT(NBLIGN,NBLIGN)
      CDETER=CDETER*PIV
      ABSPIV=ABS (PIV)  
      IF (PIVMIN.GT.ABSPIV) PIVMIN=ABSPIV
      IF (ABSPIV.LT.EPS) GOTO 99990
*  
*     Resolution du systeme triangulaire superieur (remontee)
*
      DO 42 I=NBLIGN,1,-1
         PIV=1./MAT(I,I)  
         DO 41 JJ=1,NBSEC
            DO 40 J=NBLIGN,I+1,-1
               SEC(I,JJ)=SEC(I,JJ)-MAT(I,J)*SEC(J,JJ)
40          CONTINUE
            SEC(I,JJ)=SEC(I,JJ)*PIV
41       CONTINUE
42    CONTINUE
*  
*     Retour a la numerotation initiale des inconnues
*     dans le cas d'une recherche de pivot maximum total
*
      IF (CHPIV(1:1).EQ.'T') THEN
         DO 52 I=1,NBLIGN  
            IF (PERMUT(I).NE.I) THEN
               DO 51 J=I+1,NBLIGN  
                 IF (PERMUT(J).LE.I) THEN
                    PERMUT(J)=PERMUT(I)
                    PERMUT(I)=I
                    DO 50 JJ=1,NBSEC
                       TRUC=SEC(I,JJ)
                       SEC(I,JJ)=SEC(J,JJ) 
                       SEC(J,JJ)=TRUC
50                  CONTINUE
                 ENDIF
51             CONTINUE
            ENDIF 
52       CONTINUE  
      ENDIF
      RETURN
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
99990 SINGUL=.TRUE.
                                                                    END !GaussC
