*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE GBTB (MXITER,EPSIL,ITYSYA,STOCKA,TYPA,MCA,TYPB,MCB
     &                ,TYPX,MCX,MCR1,MCR2,MCP1,MCP2
     &                ,MCAX,RST,CST,NBITER,RESIDU,NIVIMP,IMPSDR)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Decembre 91)
* Derniere modification : D.Martin (8 Juillet 1997)
*
* Resolution de A x X = B par la methode du gradient biconjugue,
* la matrice etant stockee selon le compactage 'BIMORSE'.
* ATTENTION dans ce s-p on travaille avec des produits scalaires et  
*           des matrices transposees, au lieu de produits hermitiens 
*           et des matrices adjointes comme dans le cas complexe standard
*  
*-- Arguments d'entree -- 
* MXITER: nombre maximum d'iterations de gradient  
* EPSIL : seuil d'arret de l'algorithme iteratif
* ITYSYA: indice de symetrie de la matrice A
* STOCKA: tableau des pointeurs de stockage 'bimorse' de la matrice A   
* TYPA  : type (REEL ou COMPLEXE) de la matrice du systeme 
* MCA   : adres. dans le super-tableau du type adequat de la matrice du systeme
* TYPB  : type (REEL ou COMPLEXE) du 2nd membre du systeme 
* MCB   : adres. dans le super-tableau du type adequat du 2nd membre du systeme
* TYPX  : type (REEL ou COMPLEXE) du resultat  
* MCX   : adresse du vecteur resultat initial (=X0)
* MCR1,MCR2,MCP1,MCP2,MCAX: adresses de 5 vecteurs de travail de longueur 
*                           NBLIGN et de type TYPX.
* RST,CST,NIVIMP,IMPSDR : quincaillerie habituelle 
*-- Argument de sortie -- 
* NBITER: nombre d'iterations jusqu'a convergence  
* RESIDU: residu (R1k) a convergence
*-- Sous-programmes utilises --
* BMLVC : matrice triang. inf. bimorse x vecteur (Librairie Multmat)
* BMUVC : matrice triang. sup. bimorse x vecteur (Librairie Multmat)
* DVC   : matrice diagonale x vecteur (Librairie Multmat)  
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPA,TYPB,TYPX
      INTEGER       MXITER,ITYSYA,STOCKA(*),MCA,MCB,MCX
     &             ,MCR1,MCR2,MCP1,MCP2,MCAX,NBITER,NIVIMP,IMPSDR
      REAL          EPSIL,RESIDU,RST(*) 
      COMPLEX       CST(*)
*
      INTEGER       INDSYM
      INTEGER       IALFAK,IBETAK,IST,K
     &             ,NBLIGN,NBPLAL,MCASTL,MCASIL,LGAL,MCAL
     &             ,NBCOLO,NBPLAU,MCASTU,MCASIU,LGAU,MCAU
      REAL          RALFAK,RBETAK
      COMPLEX       CALFAK,CBETAK,SEUIL,TETAK
      EQUIVALENCE  (CALFAK,RALFAK,IALFAK),(CBETAK,RBETAK,IBETAK)
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*GBtb  *')
      IF (NIVIMP.GE.5.AND.IMPSDR.GT.0) WRITE (IMPSDR,10000) '*GBtb  *'
*
      CALL EXBMOR (1,STOCKA,NBLIGN,NBPLAL,MCASTL,MCASIL,LGAL
     &                     ,NBCOLO,NBPLAU,MCASTU,MCASIU,LGAU)           !Sdexplo
      MCAL=MCA+NBLIGN
      MCAU=MCAL
      IF (ITYSYA.EQ.INDSYM (' '))  MCAU=MCAL+LGAL
*
*     Phase d'initialisation (calcul des residus initiaux) 
*     Calcul de R1o = B - AxX , R2o = CONJG(R1o) et Pio=Rio
      CALL TAZERO (NBLIGN,TYPX,MCAX,TYPX,IST,RST,CST)
      CALL BMLVC  (NBLIGN,STOCKA(MCASTL),STOCKA(MCASIL),TYPA,MCAL 
     &            ,TYPX,MCX,TYPX,MCAX,RST,CST)                          !Multmat
      CALL DVC    (NBLIGN,TYPA,MCA,TYPX,MCX,TYPX,MCAX,RST,CST)          !Multmat
      CALL BMUVC  (NBLIGN,STOCKA(MCASTU),STOCKA(MCASIU),TYPA,MCAU
     &            ,TYPX,MCX,TYPX,MCAX,RST,CST)                          !Multmat
      CALL TMOINT (NBLIGN,TYPB,MCB,RST,CST,TYPX,MCAX,RST,CST
     &            ,TYPX,MCR1,RST,CST)
      CALL TCDANT (NBLIGN,TYPX,MCR1,RST,CST,TYPX,MCR2,RST,CST) 
*
*-> Directions de descente initiales : P1o= R1o, P2o= R2o
      CALL TDANST (NBLIGN,TYPX,MCR1,IST,RST,CST,TYPX,MCP1,IST,RST,CST) 
      CALL TDANST (NBLIGN,TYPX,MCR2,IST,RST,CST,TYPX,MCP2,IST,RST,CST)
* 
*     Calcul de TETAo = produit scalaire (P1o , R2o) 
      CALL TSCALT (NBLIGN,TYPX,MCP1,RST,CST,TYPX,MCR2,RST,CST,TETAK)
*
*     Iterations de gradient
      DO 1 K=1,MXITER
      NBITER=K
*->   Taux d'accroissement dans la direction de descente : 
*                            AKFAk = TETAk / (A.P1k , P2k)
      CALL TAZERO (NBLIGN,TYPX,MCAX ,TYPX,IST,RST,CST)
      CALL BMLVC  (NBLIGN,STOCKA(MCASTL),STOCKA(MCASIL),TYPA,MCAL 
     &             ,TYPX,MCP1,TYPX,MCAX,RST,CST)                        !Multmat
      CALL DVC    (NBLIGN,TYPA,MCA,TYPX,MCP1,TYPX,MCAX,RST,CST)         !Multmat
      CALL BMUVC  (NBLIGN,STOCKA(MCASTU),STOCKA(MCASIU),TYPA,MCAU
     &            ,TYPX,MCP1,TYPX,MCAX,RST,CST)                         !Multmat
      CALL TSCALT(NBLIGN,TYPX,MCAX ,RST,CST,TYPX,MCP2,RST,CST,CALFAK)
      CALFAK=TETAK/CALFAK
*->   Mise a jour du vecteur solution : Xk+1 = Xk + ALFAk x P1k
      CALL TPLUAT (NBLIGN,TYPX,MCX,RST,CST,TYPX,1,IALFAK,RALFAK,CALFAK
     &            ,TYPX,MCP1,RST,CST,TYPX,MCX,RST,CST)
*->   Residu sur la solution de A.X = B :  Rk+1 = Rk - ALFAk A.P1k
      CALFAK=-CALFAK 
      CALL TPLUAT (NBLIGN,TYPX,MCR1,RST,CST,TYPX,1,IALFAK,RALFAK,CALFAK
     &            ,TYPX,MCAX,RST,CST,TYPX,MCR1,RST,CST)
*->   Test d'arret des iterations  
      CALL THERMT (NBLIGN,TYPX,MCR1,RST,CST,TYPX,MCR1,RST,CST,SEUIL)
      RESIDU=SQRT (ABS (SEUIL)/NBLIGN)  
      IF (NIVIMP.GE.5) THEN
         IF (TYPX(1:1).EQ.'R') WRITE(IMPSDR,10002) NBITER,-RALFAK,RESIDU
         IF (TYPX(1:1).EQ.'C') WRITE(IMPSDR,10004) NBITER,-CALFAK,RESIDU
      END IF 
      IF (RESIDU.LE.EPSIL) GO TO 99999
*
*->   Calculs preparatoires pour l'iteration suivante     
*     Bi-Residu :            R2k+1 = R2k - ALFAK x tA.P2k
      CALL TAZERO (NBLIGN,TYPX,MCAX,TYPX,IST,RST,CST)
      CALL BMLVC  (NBLIGN,STOCKA(MCASTU),STOCKA(MCASIU),TYPA,MCAU
     &            ,TYPX,MCP2,TYPX,MCAX,RST,CST)                         !Multmat
      CALL DVC    (NBLIGN,TYPA,MCA,TYPX,MCP2,TYPX,MCAX,RST,CST)         !Multmat
      CALL BMUVC  (NBLIGN,STOCKA(MCASTL),STOCKA(MCASIL),TYPA,MCAL
     &            ,TYPX,MCP2,TYPX,MCAX,RST,CST)                         !Multmat
      CALL TPLUAT (NBLIGN,TYPX,MCR2,RST,CST,TYPX,1,IALFAK,RALFAK,CALFAK
     &            ,TYPX,MCAX ,RST,CST,TYPX,MCR2,RST,CST) 
*     Calcul de TETAk+1 : produit scalaire  (R1k+1 , R2k+1)  
      CBETAK=1./TETAK
      CALL TSCALT (NBLIGN,TYPX,MCAX,RST,CST,TYPX,MCR2,RST,CST,TETAK)
      CBETAK=CBETAK*TETAK
*     Calcul des directions de descente : 
*           P1k+1 = R1k+1 + BETAk*P1k et P2k+1 = R2k+1 + BETAk*P2k
      CALL TPLUAT (NBLIGN,TYPX,MCR1,RST,CST,TYPX,1,IBETAK,RBETAK,CBETAK
     &            ,TYPX,MCP1,RST,CST,TYPX,MCP1,RST,CST)
      CALL TPLUAT (NBLIGN,TYPX,MCR2,RST,CST,TYPX,1,IBETAK,RBETAK,CBETAK
     &            ,TYPX,MCP2,RST,CST,TYPX,MCP2,RST,CST)
1     CONTINUE   
*  
99999 CALL PRFXMJ (-1,'*GBTb  *')
*-----------------------------------------------------------------------  
10000 FORMAT(T6,A8
     *,' Iteration *  Taux  d''accroissement  * Norme du residu *')
10002 FORMAT(T15,I9,TR9,E11.4,TR9,E12.5)
10004 FORMAT(T15,I9,TR3,SS,E11.4,SP,E10.4,'*i',TR3,SS,E12.5)
                                                                    END !GBTb
