*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE GCJBP3 (MXITER,EPSIL,STOCKA,TYPA,MCA,TYPB,MCB
     &                  ,STOCKM,TYPM,MCM,TYPX,MCX,MCR,MCP 
     &                  ,MCAX,RST,CST,NBITER,RESIDU,NIVIMP,IMPSDR)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Juillet 91)
* Derniere modification : D.Martin (8 Juillet 1997)
*  
* Resolution de A x X = B par la methode du gradient conjugue preconditionne
* par une matrice M factorisee L x L* 
* La matrice A du systeme etant stockee selon le compactage 'BiMorse' 
* La matrice de precond. M factorisee LL* selon le stockage 'STOCKM'.
*  
*-- Arguments d'entree -- 
* MXITER: nombre maximum d'iterations de gradient  
* EPSIL : seuil d'arret de l'algorithme iteratif
* STOCKA: tableau des pointeurs de stockage 'BiMorse' de la matrice A   
* TYPA  : type (REEL ou COMPLEXE) de la matrice du systeme 
* MCA   : adresse dans le super-tableau du type adequat de la matrice du systeme
* TYPB  : type (REEL ou COMPLEXE) du 2nd membre du systeme 
* MCB   : adresse dans le super-tableau du type adequat du 2nd membre du systeme
* STOCKM: tableau des pointeurs de stockage 'STOCKM' de la matrice M
* TYPM  : type (REEL ou COMPLEXE) de la matrice de preconditionnement  
* MCM   : adresse dans le s-t du type adequat de la mat. de prec. factorisee
* TYPX  : type (REEL ou COMPLEXE) du resultat  
* MCX   : adresse du vecteur resultat initial (=X0)
* MCR,MCP,MCAX: adresses de 3 vecteurs de travail de longueur NBLIGN et de type 
*               TYPX (residu, dir. de descente, prod A.x).
* RST,CST,NIVIMP,IMPSDR : quincaillerie habituelle 
*-- Argument de sortie -- 
* NBITER: nombre d'iterations jusqu'a convergence  
* RESIDU: residu (R1k) a convergence
*-- Sous-programmes utilises --
* BMLVC : matrice triang. inf. bimorse x vecteur (Librairie Multmat)
* BMUVC : matrice triang. sup. bimorse x vecteur (Librairie Multmat)
* DVC   : matrice diagonale x vecteur (Librairie Multmat)  
* RSPRL : resolution d'un systeme triangulaire inferieur (matrice STOCKM) 
* RSPRAL: resolution d'un systeme triangulaire superieur (matrice STOCKM) 
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPA,TYPM,TYPB,TYPX 
      INTEGER       MXITER,STOCKA(*),MCA,MCB,STOCKM(*),MCM,MCX
     &             ,MCR,MCP,MCAX,NBITER,NIVIMP,IMPSDR
      REAL          EPSIL,RESIDU,RST(*) 
      COMPLEX       CST(*)
*
      INTEGER       IALFAK,IBETAK,IST,K
     &             ,NBLIGN,NBPLAL,MCASTL,MCASIL,LGAL,MCAL
     &             ,NBCOLO,NBPLAU,MCASTU,MCASIU,LGAU
     &             ,MCMSTL,LGML,MCML
     &             ,MCMSTU,LGMU
      REAL          RALFAK,RBETAK
      COMPLEX       CALFAK,CBETAK,SEUIL,TETAK
      EQUIVALENCE  (CALFAK,RALFAK,IALFAK),(CBETAK,RBETAK,IBETAK)
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*GCjbp3*')
      IF (NIVIMP.GE.5.AND.IMPSDR.GT.0) WRITE (IMPSDR,10000) '*GCjbp3*'
*
      CALL EXPROF (1,STOCKM,NBLIGN,MCMSTL,LGML,NBCOLO,MCMSTU,LGMU)      !Sdexplo
      MCML=MCM+NBLIGN
      CALL EXBMOR (1,STOCKA,NBLIGN,NBPLAL,MCASTL,MCASIL,LGAL 
     &                     ,NBCOLO,NBPLAU,MCASTU,MCASIU,LGAU)           !Sdexplo
      MCAL=MCA+NBLIGN
*
*     Phase d'initialisation (calcul des residus initiaux) 
*     Calcul de Ro = B - AxX et de Po = M-1xRo
      CALL TAZERO (NBLIGN,TYPX,MCAX,TYPX,IST,RST,CST)
      CALL BMLVC  (NBLIGN,STOCKA(MCASTL),STOCKA(MCASIL),TYPA,MCAL
     &            ,TYPX,MCX,TYPX,MCAX,RST,CST)                          !Multmat
      CALL DVC    (NBLIGN,TYPA,MCA,TYPX,MCX,TYPX,MCAX,RST,CST)          !Multmat
      CALL BMUVC  (NBLIGN,STOCKA(MCASTL),STOCKA(MCASIL),TYPA,MCAL
     &            ,TYPX,MCX,TYPX,MCAX,RST,CST)                          !Multmat
      CALL TMOINT (NBLIGN,TYPB,MCB,RST,CST,TYPX,MCAX,RST,CST
     &            ,TYPX,MCR ,RST,CST)                                   !Utilite
*->   Calcul de la direction de descente initiale :
*     Resolution de MxPo = Ro (descente puis remontee)
      CALL RSPRL  (NBLIGN,STOCKM(MCMSTL),TYPM,MCM,MCML,TYPX,MCR
     &            ,TYPX,MCP,RST,CST)                                    !Syslin
      CALL RSPRAL (NBLIGN,STOCKM(MCMSTL),TYPM,MCM,MCML,TYPX,MCP
     &            ,TYPX,MCP,RST,CST)                                    !Syslin
*     Calcul de TETAo = produit scalaire (Po , Ro) 
      CALL TSCALT(NBLIGN,TYPX,MCP,RST,CST,TYPX,MCR,RST,CST,TETAK)
*
*     Iterations de gradient
      DO 1 K=1,MXITER
      NBITER=K
*-> Taux d'accroissement dans la direction de descente : 
*                            AKFAk = TETAk / (A.Pk,Pk )
      CALL TAZERO (NBLIGN,TYPX,MCAX,TYPX,IST,RST,CST)
      CALL BMLVC  (NBLIGN,STOCKA(MCASTL),STOCKA(MCASIL),TYPA,MCAL
     &            ,TYPX,MCP,TYPX,MCAX,RST,CST)                          !Multmat
      CALL DVC    (NBLIGN,TYPA,MCA,TYPX,MCP,TYPX,MCAX,RST,CST)          !Multmat
      CALL BMUVC (NBLIGN,STOCKA(MCASTL),STOCKA(MCASIL),TYPA,MCAL
     &           ,TYPX,MCP,TYPX,MCAX,RST,CST)                           !Multmat
      CALL TSCALT (NBLIGN,TYPX,MCAX,RST,CST,TYPX,MCP,RST,CST,CALFAK)
      CALFAK=TETAK/CALFAK
*-> Mise a jour du vecteur solution : Xk+1 = Xk + ALFAk x Pk
      CALL TPLUAT (NBLIGN,TYPX,MCX,RST,CST,TYPX,1,IALFAK,RALFAK,CALFAK
     &            ,TYPX,MCP,RST,CST,TYPX,MCX,RST,CST)
*-> Residu sur la solution de A.X = B : Rk+1 = Rk - ALFAk A.Pk
      CALFAK=-CALFAK 
      CALL TPLUAT (NBLIGN,TYPX,MCR,RST,CST,TYPX,1,IALFAK,RALFAK,CALFAK
     &            ,TYPX,MCAX,RST,CST,TYPX,MCR,RST,CST)
*->   Test d'arret des iterations  
      CALL THERMT(NBLIGN,TYPX,MCR,RST,CST,TYPX,MCR,RST,CST,SEUIL)
      RESIDU=SQRT (ABS (SEUIL)/NBLIGN)  
      IF (NIVIMP.GE.5) THEN
         IF (TYPX(1:1).EQ.'R') WRITE(IMPSDR,10002) NBITER,-RALFAK,RESIDU
         IF (TYPX(1:1).EQ.'C') WRITE(IMPSDR,10004) NBITER,-CALFAK,RESIDU
      ENDIF 
      IF (RESIDU.LE.EPSIL) GOTO 99999
*     Calcul de TETAk+1 : produit scalaire  ((M-1.Rk+1) , Rk+1)  
      CALL RSPRL  (NBLIGN,STOCKM(MCMSTL),TYPM,MCM,MCML,TYPX,MCR
     &            ,TYPX,MCAX,RST,CST)                                    !Syslin
      CALL RSPRAL (NBLIGN,STOCKM(MCMSTL),TYPM,MCM,MCML,TYPX,MCAX
     &            ,TYPX,MCAX,RST,CST)                                    !Syslin
      CBETAK=1./TETAK
      CALL TSCALT (NBLIGN,TYPX,MCAX,RST,CST,TYPX,MCR,RST,CST,TETAK)
      CBETAK=CBETAK*TETAK
*     Calcul de la direction de descente : 
*                           Pk+1 = (M-1.Rk+1)+BETAk*Pk
      CALL TPLUAT (NBLIGN,TYPX,MCAX,RST,CST,TYPX,1,IBETAK,RBETAK,CBETAK
     &            ,TYPX,MCP,RST,CST,TYPX,MCP,RST,CST)
1     CONTINUE   
*  
99999 CALL PRFXMJ (-1,'*GCjbp3*')
*-----------------------------------------------------------------------
10000 FORMAT(T6,A8
     &,' Iteration *  Taux  d''accroissement  * Norme du residu *')
10002 FORMAT(T15,I9,TR9,E11.4,TR9,E12.5)
10004 FORMAT(T15,I9,TR3,SS,E11.4,SP,E10.4,'*i',TR3,SS,E12.5)
                                                                    END !GCjbp3
