*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE GCJMM3 (MXITER,EPSIL,STOCKA,TYPA,MCA,TYPB,MCB
     &                  ,STOCKM,TYPM,MCM,TYPX,MCX,MCR,MCP
     &                  ,MCAX,RST,CST,NBITER,RESIDU,NIVIMP,IMPSDR)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Juillet 1991)
* Derniere modification : D.Martin (8 Juillet 1997)
*  
* Resolution de A x X = B par la methode du gradient conjugue preconditionne
* par une matrice M factorisee L x L* (incompletement) , les matrices etant 
* stockees selon le compactage 'MORSE' par ligne.
*  
*-- Arguments d'entree -- 
* MXITER: nombre maximum d'iterations de gradient  
* EPSIL : seuil d'arret de l'algorithme iteratif
* STOCKA: tableau des pointeurs de stockage 'Morse' de la matrice A 
* TYPA  : type (REEL ou COMPLEXE) de la matrice du systeme 
* MCA   : adresse dans le super-tableau du type adequat de la matrice du systeme
* TYPB  : type (REEL ou COMPLEXE) du 2nd membre du systeme 
* MCB   : adresse dans le super-tableau du type adequat du 2nd membre du systeme
* STOCKM: tableau des pointeurs de stockage 'Morse' de la matrice M
* TYPM  : type (REEL ou COMPLEXE) de la matrice de preconditionnement  
* MCM   : adresse de la matrice de preconditionnement factorisee LU incomplete 
* TYPX  : type (REEL ou COMPLEXE) du resultat  
* MCX   : adresse du vecteur resultat initial (=X0)
* MCR,MCP,MCAX: adresses de 3 vecteurs de travail de longueur NBLIGN et de type 
*               TYPX (residu, dir. de descente, prod A.x).
* RST,CST,NIVIMP,IMPSDR : quincaillerie habituelle 
*-- Argument de sortie -- 
* NBITER: nombre d'iterations jusqu'a convergence  
* RESIDU: residu (R1k) a convergence
*-- Sous-programmes utilises --
* MOSAVC: matrice symetrique ou auto-adjointe morse x vecteur lib Multmat  
* RSMOL : resolution d'un systeme triangulaire inferieur (matrice MORSE ligne) 
* RSMOU : resolution d'un systeme triangulaire superieur (matrice MORSE ligne) 
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPA,TYPM,TYPB,TYPX
      INTEGER       MXITER,STOCKA(*),MCA,MCB,STOCKM(*),MCM,MCX
     &             ,MCR,MCP,MCAX,NBITER,NIVIMP,IMPSDR
      REAL          EPSIL,RESIDU,RST(*) 
      COMPLEX       CST(*)
*
      INTEGER       IALFAK,IBETAK,IST,K
     &             ,NBLIGN,MCASTL,MCASIL
      REAL          RALFAK,RBETAK
      COMPLEX       CALFAK,CBETAK,SEUIL,TETAK
      EQUIVALENCE  (CALFAK,RALFAK,IALFAK),(CBETAK,RBETAK,IBETAK)
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*GCjmm3*')
      IF (NIVIMP.GE.5.AND.IMPSDR.GT.0) WRITE(IMPSDR,10000) '*GCjmm3*'
*
      NBLIGN=STOCKA(1)
      MCASTL=5
      MCASIL=NBLIGN+5 
*
*     Phase d'initialisation (calcul des residus initiaux) 
*     Calcul de Ro = B - AxX et de Po = M-1xRo
      CALL TAZERO (NBLIGN,TYPX,MCAX ,TYPX,IST,RST,CST)
      CALL MOSAVC (NBLIGN,STOCKA(MCASTL),STOCKA(MCASIL),TYPA,MCA
     &            ,TYPX,MCX,TYPX,MCAX,RST,CST)                          !Multmat
      CALL TMOINT (NBLIGN,TYPB,MCB,RST,CST,TYPX,MCAX,RST,CST
     &            ,TYPX,MCR,RST,CST)
*->   Direction de descente initiale : Po= M-1.Ro
      CALL RSMOL  (NBLIGN,STOCKM(MCASTL),STOCKM(MCASIL),TYPM,MCM
     &            ,TYPX,MCR,TYPX,MCP,RST,CST)                           !Syslin
      CALL RSMOAL (NBLIGN,STOCKM(MCASTL),STOCKM(MCASIL),TYPM,MCM
     &            ,TYPX,MCP,TYPX,MCP,RST,CST)                           !Syslin
*     Calcul de TETAo = produit scalaire (Po , Ro) 
      CALL TSCALT (NBLIGN,TYPX,MCP,RST,CST,TYPX,MCR,RST,CST,TETAK)
*
*     Iterations de gradient
      DO 1 K=1,MXITER
      NBITER=K
*     Taux d'accroissement dans la direction de descente : 
*                            AKFAk = TETAk / (A.Pk,Pk )
      CALL TAZERO (NBLIGN,TYPX,MCAX,TYPX,IST,RST,CST)
      CALL MOSAVC (NBLIGN,STOCKA(MCASTL),STOCKA(MCASIL),TYPA,MCA
     &            ,TYPX,MCP,TYPX,MCAX,RST,CST)                          !Multmat
      CALL TSCALT (NBLIGN,TYPX,MCAX,RST,CST,TYPX,MCP,RST,CST,CALFAK)
      CALFAK=TETAK/CALFAK
*->   Mise a jour du vecteur solution : Xk+1 = Xk + ALFAk x Pk
      CALL TPLUAT (NBLIGN,TYPX,MCX,RST,CST,TYPX,1,IALFAK,RALFAK,CALFAK
     &            ,TYPX,MCP,RST,CST,TYPX,MCX,RST,CST)
*->   Residu sur la solution de A.X = B : Rk+1 = Rk - ALFAk A.Pk
      CALFAK=-CALFAK 
      CALL TPLUAT (NBLIGN,TYPX,MCR,RST,CST,TYPX,1,IALFAK,RALFAK,CALFAK
     &            ,TYPX,MCAX,RST,CST,TYPX,MCR,RST,CST)
*->   Test d'arret des iterations  
      CALL THERMT (NBLIGN,TYPX,MCR,RST,CST,TYPX,MCR,RST,CST,SEUIL)
      RESIDU=SQRT (ABS (SEUIL)/NBLIGN)  
      IF (NIVIMP.GE.5) THEN
         IF (TYPX(1:1).EQ.'R') WRITE(IMPSDR,10002) NBITER,-RALFAK,RESIDU
         IF (TYPX(1:1).EQ.'C') WRITE(IMPSDR,10004) NBITER,-CALFAK,RESIDU
      END IF 
      IF(RESIDU.LE.EPSIL) GO TO 99999
*
*->   Calculs preparatoires pour l'iteration suivante
*     Calcul de TETAk+1 : produit scalaire  ((M-1.Rk+1) , Rk+1)  
      CALL RSMOL  (NBLIGN,STOCKM(MCASTL),STOCKM(MCASIL),TYPM,MCM
     &            ,TYPX,MCR,TYPX,MCAX,RST,CST)                           !Syslin
      CALL RSMOAL (NBLIGN,STOCKM(MCASTL),STOCKM(MCASIL),TYPM,MCM
     &            ,TYPX,MCAX,TYPX,MCAX,RST,CST)                          !Syslin
      CBETAK=1./TETAK
      CALL TSCALT (NBLIGN,TYPX,MCAX,RST,CST,TYPX,MCR,RST,CST,TETAK)
      CBETAK=CBETAK*TETAK
*     Calcul de la direction de descente : Pk+1 = (M-1.Rk+1)+BETAk*Pk
      CALL TPLUAT (NBLIGN,TYPX,MCAX,RST,CST,TYPX,1,IBETAK,RBETAK,CBETAK
     &            ,TYPX,MCP,RST,CST,TYPX,MCP,RST,CST)
1     CONTINUE   
*  
99999 CALL PRFXMJ (-1,'*GCjmm3*')
*-----------------------------------------------------------------------  
10000 FORMAT(T6,A8
     &,' Iteration *  Taux  d''accroissement  * Norme du residu *')
10002 FORMAT(T15,I9,TR9,E11.4,TR9,E12.5)
10004 FORMAT(T15,I9,TR3,SS,E11.4,SP,E10.4,'*i',TR3,SS,E12.5)
                                                                    END !GCjmm3
