*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE GCSM (MXITER,EPSIL,STOCKA,TYPA,MCA,TYPB,MCB
     &                ,TYPX,MCX,MCRK,MCR0,MCP,MCH,MCG
     &                ,MCAX,RST,CST,NBITER,RESIDU,NIVIMP,IMPSDR)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin (Octobre 1991) 
* Derniere modification : D.Martin (8 Juillet 1997)  
*  
* Resolution de A x X = B par la methode du gradient conjugue 'squared'. 
* La matrice A du systeme est stockee selon le compactage 'Morse' par ligne.
*  
*-- Arguments d'entree -- 
* MXITER: nombre maximum d'iterations de gradient  
* EPSIL : seuil d'arret de l'algorithme iteratif
* STOCKA: tableau des pointeurs de stockage 'Morse' de la matrice A 
* TYPA  : type (REEL ou COMPLEXE) de la matrice du systeme 
* MCA   : adresse dans le s-t du type adequat de la matrice du systeme
* TYPB  : type (REEL ou COMPLEXE) du 2nd membre du systeme 
* MCB   : adresse dans le s-t du type adequat du 2nd membre du systeme
* TYPX  : type (REEL ou COMPLEXE) du resultat  
* MCX   : adresse du vecteur initial (=Xo) et du resultat
* MCRK,MCR0,MCP,MCH,MCG,MCAX: adresses de 6 vecteurs de travail de longueur
*                            NBLIGN et de type TYPX.
* RST,CST,NIVIMP,IMPSDR : quincaillerie habituelle 
*-- Argument de sortie -- 
* NBITER: nombre d'iterations jusqu'a convergence  
* RESIDU: residu (R) a convergence 
*-- Sous-programmes utilises --
* MOVC  : matrice Morse x vecteur (Librairie Multmat)  
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPA,TYPB,TYPX  
      INTEGER       MXITER,STOCKA(*),MCA,MCB,MCX
     &             ,MCRK,MCR0,MCP,MCH,MCG,MCAX,NBITER,NIVIMP,IMPSDR
      REAL          EPSIL,RESIDU,RST(*) 
      COMPLEX       CST(*)
*
      INTEGER       IALFAK,IBETAK,IST,K
     &             ,NBLIGN,MCASTL,MCASIL
      REAL          RALFAK,RBETAK
      COMPLEX       CALFAK,CBETAK,TETAK,SEUIL 
      EQUIVALENCE  (CALFAK,RALFAK,IALFAK),(CBETAK,IBETAK,RBETAK) 
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*GCSm  *')
      IF (NIVIMP.GE.5.AND.IMPSDR.GT.0) WRITE(IMPSDR,10000) '*GCSm  *'
*
      NBLIGN=STOCKA(1)
      MCASTL=5
      MCASIL=NBLIGN+5
*
*--   Phase d'initialisation (calcul des residus initiaux)
*     Calcul de Ro = B - A.Xo et Go = Po = Ro 
      CALL TAZERO (NBLIGN,TYPX,MCAX,TYPX,IST,RST,CST) 
      CALL MOVC   (NBLIGN,NBLIGN,STOCKA(5),STOCKA(MCASIL),TYPA,MCA 
     &            ,TYPX,MCX,TYPX,MCAX,RST,CST)                          !Multmat
      CALL TMOINT (NBLIGN,TYPB,MCB,RST,CST,TYPX,MCAX,RST,CST
     &            ,TYPX,MCR0,RST,CST)
      CALL TDANST (NBLIGN,TYPX,MCR0,IST,RST,CST,TYPX,MCG,IST,RST,CST)
      CALL TDANST (NBLIGN,TYPX,MCR0,IST,RST,CST,TYPX,MCP,IST,RST,CST)
      CALL TDANST (NBLIGN,TYPX,MCR0,IST,RST,CST,TYPX,MCRK,IST,RST,CST)
*
*     Calcul de TETAo = produit scalaire (Ro , Ro)
      CALL TSCALT (NBLIGN,TYPX,MCR0,RST,CST,TYPX,MCR0,RST,CST,TETAK)
*
*--   Iterations de gradient
      DO 1 K=1,MXITER
      NBITER=K
*     Accroissement dans la direction de descente : 
*                            AKFAk = TETAk / (A.Gk , Ro)
      CALL TAZERO (NBLIGN,TYPX,MCAX,TYPX,IST,RST,CST)
      CALL MOVC   (NBLIGN,NBLIGN,STOCKA(5),STOCKA(MCASIL),TYPA,MCA       
     &            ,TYPX,MCG,TYPX,MCAX,RST,CST)                          !Multmat
      CALL TSCALT (NBLIGN,TYPX,MCAX,RST,CST,TYPX,MCR0,RST,CST,CALFAK)
      CALFAK=TETAK/CALFAK
*     Calcul de Hk+1 = (Rk + BETAk Hk) - ALFAk A.Gk
      CALFAK=-CALFAK
      CALL TPLUAT (NBLIGN,TYPX,MCP,RST,CST,TYPX,1,IALFAK,RALFAK,CALFAK
     &            ,TYPX,MCAX,RST,CST,TYPX,MCH,RST,CST)
*->   Calcul de la direction de descente :  Pk = (Rk + BETAk Hk) + Hk+1
      CALL TPLUST (NBLIGN,TYPX,MCP,RST,CST,TYPX,MCH,RST,CST 
     &            ,TYPX,MCP,RST,CST)
*->   Mise a jour du vecteur solution : Xk+1 = Xk + ALFAk x Pk
      CALFAK=-CALFAK
      CALL TPLUAT (NBLIGN,TYPX,MCX,RST,CST,TYPX,1,IALFAK,RALFAK,CALFAK    
     &            ,TYPX,MCP,RST,CST,TYPX,MCX,RST,CST)
*->   Residu sur la solution de A.X = B : 
*                            Rk+1 = Rk - ALFAk A.Pk
      CALL TAZERO (NBLIGN,TYPX,MCAX,TYPX,IST,RST,CST)
      CALL MOVC   (NBLIGN,NBLIGN,STOCKA(5),STOCKA(MCASIL),TYPA,MCA 
     &           ,TYPX,MCP,TYPX,MCAX,RST,CST)                           !Multmat
      CALFAK=-CALFAK 
      CALL TPLUAT (NBLIGN,TYPX,MCRK,RST,CST,TYPX,1,IALFAK,RALFAK,CALFAK   
     &            ,TYPX,MCAX,RST,CST,TYPX,MCRK,RST,CST)
*->   Test d'arret des iterations 
      CALL THERMT (NBLIGN,TYPX,MCRK,RST,CST,TYPX,MCRK,RST,CST,SEUIL) 
      RESIDU=SQRT (ABS(SEUIL)/NBLIGN)
      IF (NIVIMP.GE.5) THEN
         IF (TYPX(1:1).EQ.'R') WRITE(IMPSDR,10002) NBITER,-RALFAK,RESIDU
         IF (TYPX(1:1).EQ.'C') WRITE(IMPSDR,10004) NBITER,-CALFAK,RESIDU
      ENDIF 
      IF (RESIDU.LE.EPSIL) GOTO 99999  
*
*     Calcul de TETAk+1 = (Rk+1 , Ro) et de BETAk+1 = TETAk+1 / TETAk
      CBETAK=1./TETAK
      CALL TSCALT (NBLIGN,TYPX,MCRK,RST,CST,TYPX,MCR0,RST,CST,TETAK)
      CBETAK=CBETAK*TETAK
*     Calcul de Pk+1=Rk+1 + BETAk+1 Hk+1
      CALL TPLUAT (NBLIGN,TYPX,MCRK,RST,CST,TYPX,1,IBETAK,RBETAK,CBETAK   
     &            ,TYPX,MCH,RST,CST,TYPX,MCP,RST,CST)
*     Calcul de Gk+1 = Rk+1 + (BETAk+1)**2 Gk + (2.BETAk+1) Hk+1
*                    = Pk+1 + BETAk+1 (Hk+1 + BETAk+1 Gk)
      CALL TPLUAT (NBLIGN,TYPX,MCH,RST,CST,TYPX,1,IBETAK,RBETAK,CBETAK
     &            ,TYPX,MCG,RST,CST,TYPX,MCG,RST,CST)
      CALL TPLUAT (NBLIGN,TYPX,MCP,RST,CST,TYPX,1,IBETAK,RBETAK,CBETAK
     &            ,TYPX,MCG,RST,CST,TYPX,MCG,RST,CST)
1     CONTINUE   
*  
99999 CALL PRFXMJ (-1,'*GCSm  *')
*----------------------------------------------------------------------- 
10000 FORMAT(T6,A8
     &,' Iteration *  Taux  d''accroissement  * Norme du residu *')
10002 FORMAT(T15,I9,TR9,E11.4,TR9,E12.5)
10004 FORMAT(T15,I9,TR3,SS,E11.4,SP,E10.4,'*i',TR3,SS,E12.5)
                                                                    END !GCSm
