*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE GCSMB1 (MXITER,EPSIL,STOCKA,TYPA,MCA,TYPB,MCB
     &                  ,STOCKM,TYPM,MCM,TYPX,MCX,MCRK,MCR0,MCP,MCH,MCG
     &                  ,MCAX,RST,CST,NBITER,RESIDU,NIVIMP,IMPSDR)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Novembre 1991) 
* Derniere modification : D.Martin (8 Juillet 1997)  
*  
* Resolution de A x X = B par la methode du gradient conjugue 'squared'  
* preconditionne par une matrice M factorisee L x U. 
* La matrice A du systeme est stockee selon le compactage 'Morse' par ligne.
* La matrice M factorisee est stockee selon le stockage 'BiMorse'. 
*
*  
*-- Arguments d'entree -- 
* MXITER: nombre maximum d'iterations de gradient  
* EPSIL : seuil d'arret de l'algorithme iteratif
* STOCKA: tableau des pointeurs de stockage 'Morse' de la matrice A 
* TYPA  : type (REEL ou COMPLEXE) de la matrice du systeme 
* MCA   : adresse dans le super-tableau du type adequat de la matrice du systeme
* TYPB  : type (REEL ou COMPLEXE) du 2nd membre du systeme 
* MCB   : adresse dans le super-tableau du type adequat du 2nd membre du systeme
* STOCKM: tableau des pointeurs de stockage 'Morse' de la matrice M
* TYPM  : type (REEL ou COMPLEXE) de la matrice de preconditionnement  
* MCM   : adresse de la matrice de preconditionnement factorisee LU incomplete 
* TYPX  : type (REEL ou COMPLEXE) du resultat  
* MCX   : adresse du vecteur initial (=Xo) et du resultat
* MCRK,MCR0,MCP,MCH,MCG,MCAX : adresses de 6 vecteurs de travail de longueur
*                             NBLIGN et de type TYPX.
* RST,CST,NIVIMP,IMPSDR : quincaillerie habituelle 
*-- Argument de sortie -- 
* NBITER: nombre d'iterations jusqu'a convergence  
* RESIDU: residu (Rk) a convergence 
*-- Sous-programmes utilises --
* MOVC  : matrice Morse x vecteur (Librairie Multmat)  
* RSBML1: resolution d'un systeme triang. inf. a diag. unite, matrice BiMorse 
* RSBMU : resolution d'un systeme triangulaire superieur, matrice BiMorse 
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPA,TYPM,TYPB,TYPX 
      INTEGER       MXITER,STOCKA(*),MCA,MCB,STOCKM(*),MCM,MCX,MCRK,MCR0
     &             ,MCP,MCH,MCG,MCAX,NBITER,NIVIMP,IMPSDR
      REAL          EPSIL,RESIDU,RST(*) 
      COMPLEX       CST(*)
*
      INTEGER       IALFAK,IBETAK,IST,K
     &             ,NBLIGN,NBPLAL,MCASTL,MCASIL
     &             ,NBCOLO,NBPLAU
     &             ,MCMSTL,MCMSIL,LGML,MCML
     &             ,MCMSTU,MCMSIU,LGMU,MCMU
     &             
      REAL          RALFAK,RBETAK
      COMPLEX       CALFAK,CBETAK,SEUIL,TETAK
      EQUIVALENCE  (CALFAK,RALFAK,IALFAK),(CBETAK,RBETAK,IBETAK)
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*GCSmb1*')
      IF (NIVIMP.GE.5.AND.IMPSDR.GT.0) WRITE (IMPSDR,10000) '*GCSmb1*'
*
      CALL EXBMOR (1,STOCKM,NBLIGN,NBPLAL,MCMSTL,MCMSIL,LGML
     &                     ,NBCOLO,NBPLAU,MCMSTU,MCMSIU,LGMU)           !Sdexplo
      MCML=MCM+NBLIGN
      MCMU=MCML+LGML
      MCASTL=5
      MCASIL=NBLIGN+5
*  
*     Phase d'initialisation (calcul des residus initiaux)
*     Calcul de Ro = M-1(B - A.Xo) et Go = Po = Ro 
      CALL TAZERO (NBLIGN,TYPX,MCAX,TYPX,IST,RST,CST)
      CALL MOVC   (NBLIGN,NBLIGN,STOCKA(MCASTL),STOCKA(MCASIL),TYPA,MCA
     &            ,TYPX,MCX,TYPX,MCAX,RST,CST)                          !Multmat
      CALL TMOINT (NBLIGN,TYPB,MCB,RST,CST,TYPX,MCAX,RST,CST
     &            ,TYPX,MCR0,RST,CST)
      CALL RSBML1 (NBLIGN,STOCKM(MCMSTL),STOCKM(MCMSIL),TYPM,MCML
     &            ,TYPX,MCR0,TYPX,MCR0,RST,CST)                         !Syslin
      CALL RSBMU  (NBLIGN,STOCKM(MCMSTU),STOCKM(MCMSIU),TYPM,MCM,MCMU
     &            ,TYPX,MCR0,TYPX,MCR0,RST,CST)                         !Syslin
      CALL TDANST (NBLIGN,TYPX,MCR0,IST,RST,CST,TYPX,MCG,IST,RST,CST)
      CALL TDANST (NBLIGN,TYPX,MCR0,IST,RST,CST,TYPX,MCP,IST,RST,CST)
      CALL TDANST (NBLIGN,TYPX,MCR0,IST,RST,CST,TYPX,MCRK,IST,RST,CST)
*     Calcul de TETAo = produit scalaire (Ro , Ro)
      CALL TSCALT (NBLIGN,TYPX,MCR0,RST,CST,TYPX,MCR0,RST,CST,TETAK)
*
*     Iterations de gradient
      DO 1 K=1,MXITER
      NBITER=K
*     Accroissement dans la direction de descente : 
*                            AKFAk = TETAk / (M-1.A.Gk , Ro)
*     Calcul de  M-1.A.Gk
      CALL TAZERO (NBLIGN,TYPX,MCAX,TYPX,IST,RST,CST)
      CALL MOVC   (NBLIGN,NBLIGN,STOCKA(MCASTL),STOCKA(MCASIL),TYPA,MCA
     &            ,TYPX,MCG,TYPX,MCAX,RST,CST)                          !Multmat
      CALL RSBML1 (NBLIGN,STOCKM(MCMSTL),STOCKM(MCMSIL),TYPM,MCML
     &            ,TYPX,MCAX,TYPX,MCAX,RST,CST)                         !Syslin
      CALL RSBMU  (NBLIGN,STOCKM(MCMSTU),STOCKM(MCMSIU),TYPM,MCM,MCMU
     &            ,TYPX,MCAX,TYPX,MCAX,RST,CST)                         !Syslin
      CALL TSCALT (NBLIGN,TYPX,MCAX,RST,CST,TYPX,MCR0,RST,CST,CALFAK)
      CALFAK=TETAK/CALFAK
*     Calcul de Hk+1 = (Rk + BETAk Hk) - ALFAk M-1.A.Gk
      CALFAK=-CALFAK
      CALL TPLUAT (NBLIGN,TYPX,MCP,RST,CST,TYPX,1,IALFAK,RALFAK,CALFAK
     &            ,TYPX,MCAX,RST,CST,TYPX,MCH,RST,CST)
*->   Calcul de la direction de descente : Pk = (Rk + BETAk Hk) + Hk+1
      CALL TPLUST (NBLIGN,TYPX,MCP,RST,CST,TYPX,MCH,RST,CST 
     &            ,TYPX,MCP,RST,CST)
*->   Mise a jour du vecteur solution : Xk+1 = Xk + ALFAk x Pk
      CALFAK=-CALFAK
      CALL TPLUAT (NBLIGN,TYPX,MCX,RST,CST,TYPX,1,IALFAK,RALFAK,CALFAK
     &            ,TYPX,MCP,RST,CST,TYPX,MCX,RST,CST)
*->   Residu sur la solution de A.X = B : Rk+1 = Rk - ALFAk M-1.A.Pk
      CALL TAZERO (NBLIGN,TYPX,MCAX,TYPX,IST,RST,CST)  
      CALL MOVC   (NBLIGN,NBLIGN,STOCKA(MCASTL),STOCKA(MCASIL),TYPA,MCA
     &            ,TYPX,MCP,TYPX,MCAX,RST,CST)                          !Multmat
      CALL RSBML1 (NBLIGN,STOCKM(MCMSTL),STOCKM(MCMSIL),TYPM,MCML
     &            ,TYPX,MCAX,TYPX,MCAX,RST,CST)                         !Syslin
      CALL RSBMU  (NBLIGN,STOCKM(MCMSTU),STOCKM(MCMSIL),TYPM,MCM,MCMU
     &            ,TYPX,MCAX,TYPX,MCAX,RST,CST)                         !Syslin
      CALFAK=-CALFAK 
      CALL TPLUAT (NBLIGN,TYPX,MCRK,RST,CST,TYPX,1,IALFAK,RALFAK,CALFAK
     &            ,TYPX,MCAX ,RST,CST,TYPX,MCRK ,RST,CST)
*->   Test d'arret des iterations 
      CALL THERMT (NBLIGN,TYPX,MCRK,RST,CST,TYPX,MCRK,RST,CST,SEUIL)
      RESIDU=SQRT (ABS (SEUIL)/NBLIGN)
      IF (NIVIMP.GE.5) THEN
         IF (TYPX(1:1).EQ.'R') WRITE(IMPSDR,10002) NBITER,-RALFAK,RESIDU
         IF (TYPX(1:1).EQ.'C') WRITE(IMPSDR,10004) NBITER,-CALFAK,RESIDU
      ENDIF 
      IF (RESIDU.LE.EPSIL) GOTO 99999  
*
*     Calcul de TETAk+1 = (Rk+1, Ro) et de BETAk+1 = TETAk+1 / TETAk
      CBETAK=1./TETAK
      CALL TSCALT (NBLIGN,TYPX,MCRK,RST,CST,TYPX,MCR0,RST,CST,TETAK)
      CBETAK=CBETAK*TETAK
*     Calcul de Pk+1=Rk+1 + BETAk+1 Hk+1
      CALL TPLUAT (NBLIGN,TYPX,MCRK,RST,CST,TYPX,1,IBETAK,RBETAK,CBETAK  
     &            ,TYPX,MCH,RST,CST,TYPX,MCP,RST,CST)
*     Calcul de Gk+1 = Rk+1 + (BETAk+1)**2 Gk + (2.BETAk+1) Hk+1
*                    = Pk+1 + BETAk+1 (Hk+1 + BETAk+1 Gk)
      CALL TPLUAT (NBLIGN,TYPX,MCH,RST,CST,TYPX,1,IBETAK,RBETAK,CBETAK
     &            ,TYPX,MCG,RST,CST,TYPX,MCG,RST,CST)
      CALL TPLUAT (NBLIGN,TYPX,MCP,RST,CST,TYPX,1,IBETAK,RBETAK,CBETAK
     &            ,TYPX,MCG,RST,CST,TYPX,MCG,RST,CST)
1     CONTINUE   
*  
99999 CALL PRFXMJ (-1,'*GCSmb1*')
*-----------------------------------------------------------------------  
10000 FORMAT(T6,A8
     &,' Iteration *  Taux  d''accroissement  * Norme du residu *')
10002 FORMAT(T15,I9,TR9,E11.4,TR9,E12.5)
10004 FORMAT(T15,I9,TR3,SS,E11.4,SP,E10.4,'*i',TR3,SS,E12.5)
                                                                    END !GCSmb1
