*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE INVMPL (TYMATR,STMATR,NBLIGN,MCMATR,MCMINV,RST,CST
     &                  ,EPS,IRCODE)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : C.Chambeyron (Octobre 2006)
* Derniere modification : D. Martin (23 octobre 2006)
* Version 0
*
* Calcul de l'INVerse d'une Matrice Pleine (stockage PLEIN-L ou PLEIN-C)
*
* La matrice inverse est retournee selon le meme stockage
*
*-- Arguments d'entree --
* TYMATR type (REEL ou COMPLEXE) de la matrice a inverser A
* STMATR type de stockage (PLEIN-C ou PLEIN-L)
* NBLIGN nombre de lignes des matrices
* MCMATR adresse de la matrice a inverser AA
* MCMINV adresse de la matrice inverse B
* RST, CST super-tableau version reelle et complexe
* EPS    seuil d'inversibilite de la matrice
* IRCODE indicateur d'erreur (matrice non inversible)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) STMATR*7,TYMATR*9
      INTEGER  MCMINV,MCMATR,IRCODE
      REAL EPS,RST(*)
      COMPLEX CST(*)
*
      INTEGER AK1,AKK,AKJ,AI1,AIJ,BK1,BKJ,BI1,BIJ,I,J,K
      REAL    RAKK,RAKI
      COMPLEX CAKK,CAKI
*-----------------------------------------------------------------------
*
*     Increments de changement de ligne ou de colonne selon le stockage
*
*     INCMAC increment d'adresse pour le changement de colonne
*     INCMAL increment d'adresse pour le changement de ligne
*
      IF ( STMATR(1:7).EQ.'PLEIN-C') THEN
         INCMAL = 1
         INCMAC = NBLIGN
      ELSEIF ( STMATR(1:7).EQ.'PLEIN-L') THEN
         INCMAL = NBLIGN
         INCMAC = 1
      ENDIF
*
      AK1 = MCMATR
      AKK = MCMATR
      BK1 = MCMINV
*
      IF (TYMATR(1:1).EQ.'R') THEN
         DO 29 K=1,NBLIGN
*
*           Pivot de la ligne k
            RAKK=RST(AKK)
            IF (ABS(RAKK).LT.EPS) THEN
*
*              Pivot nul : combinaison lineaire des lignes suivantes
*
               DO 22 I=K+1,NBLIGN
                  IF (ABS(RAKK).GE.EPS) GOTO 24          
                  AKJ = AK1
                  BKJ = BK1
                  DO 20 J=1,NBLIGN
                     RST(AKJ)= RST(AKJ)+RST(AKJ+I)
                     RST(BKJ)= RST(BKJ)+RST(BKJ+I)
                     AKJ = AKJ + INCMAC
                     BKJ = BKJ + INCMAC
 20               CONTINUE
                  RAKK=RST(AKK)
 22            CONTINUE
*              Matrice non inversible
               IRCODE=1
               RETURN
            ENDIF
*
*           Normalisation des lignes K des matrices A et B
*
 24         AKJ = AK1
            BKJ = BK1
            DO 25 J=1,NBLIGN
               RST(AKJ)=RST(AKJ)/RAKK
               AKJ=AKJ+INCMAC
               RST(BKJ)=RST(BKJ)/RAKK
               BKJ=BKJ+INCMAC
 25        CONTINUE
*
*           Reduction de Gauss-Jordan
*
            AI1 = MCMATR
            BI1 = MCMINV
            DO 27 I=1,NBLIGN
               IF (I.NE.K) THEN
                  RAKI = RST(AI1+(K-1)*INCMAC)
                  AIJ = AI1
                  AKJ = AK1
                  BIJ = BI1
                  BKJ = BK1
                  DO 26 J=1,NBLIGN
                     RST(AIJ)= RST(AIJ)-RAKI*RST(AKJ) 
                     AIJ = AIJ + INCMAC
                     AKJ = AKJ + INCMAC
                     RST(BIJ)= RST(BIJ)-RAKI*RST(BKJ)
                     BIJ = BIJ + INCMAC
                     BKJ = BKJ + INCMAC
 26              CONTINUE
               ENDIF
               AI1 = AI1 + 1
               BI1 = BI1 + 1
 27        CONTINUE
            AK1 = AK1 + INCMAL
            BK1 = BK1 + INCMAL
            AKK = AKK + NBLIGN + 1
 29     CONTINUE
*
      ELSE
*
         DO 49 K=1,NBLIGN
*
*           Pivot de la ligne k
            CAKK=CST(AKK)
            IF (ABS(CAKK).LT.EPS) THEN
               DO 42 I=K+1,NBLIGN
                  IF (ABS(CAKK).GE.EPS) GOTO 44          
                  AKJ = AK1
                  BKJ = BK1
                  DO 40 J=1,NBLIGN
                     CST(AKJ)= CST(AKJ)+CST(AKJ+I)
                     CST(BKJ)= CST(BKJ)+CST(BKJ+I)
                     AKJ = AKJ + INCMAC
                     BKJ = BKJ + INCMAC
 40               CONTINUE
                  CAKK=CST(AKK)
 42            CONTINUE
*              Matrice non inversible
               IRCODE=1
               RETURN
            ENDIF
*
 44         AKJ = AK1
            BKJ = BK1
            DO 45 J=1,NBLIGN
               CST(AKJ)=CST(AKJ)/CAKK
               AKJ=AKJ+INCMAC
               CST(BKJ)=CST(BKJ)/CAKK
               BKJ=BKJ+INCMAC
 45        CONTINUE
*
            AI1 = MCMATR
            BI1 = MCMINV
            DO 47 I=1,NBLIGN
               IF (I.NE.K) THEN
                  CAKI = CST(AI1+(K-1)*INCMAC)
                  AIJ = AI1
                  AKJ = AK1
                  BIJ = BI1
                  BKJ = BK1
                  DO 46 J=1,NBLIGN
                     CST(AIJ)= CST(AIJ)-CAKI*CST(AKJ) 
                     AIJ = AIJ + INCMAC
                     AKJ = AKJ + INCMAC
                     CST(BIJ)= CST(BIJ)-CAKI*CST(BKJ)
                     BIJ = BIJ + INCMAC
                     BKJ = BKJ + INCMAC
 46              CONTINUE
               ENDIF
               AI1 = AI1 + 1
               BI1 = BI1 + 1
 47        CONTINUE
            AK1 = AK1 + INCMAL
            BK1 = BK1 + INCMAL
            AKK = AKK + NBLIGN + 1
 49     CONTINUE
*
      ENDIF
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 0 : D.Martin (23 octobre 2006)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !InvMPL
