*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE LTLPR (TYPE,NBLINE,LENV,MCAD,MCAL,EPSIL
     &                 ,MCD,MCL,RST,CST,IRCODE)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : A.S.Bonnet (Septembre 1989) puis D.Martin (Juin 1991)
* Derniere modification : D.Martin (15 Juin 2002)
* Version 1.0.2
*  
* Factorisation L.tL d'une matrice stockee sous forme 'Profil'
*  
*-- Arguments de la Liste d'Entree --
* TYPE  type REEL ou COMPLEXE de la matrice
* NBLINE nombre de lignes de la matrice
* LENV   tableau contenant l'enveloppe inferieure stricte avec
*         LENV(I-1) adresse dans la partie triangulaire inferieure compactee
*         du 1er coeff. non nul de la ligne I, et LENV(I)=LENV(I-1) si le 1er 
*         coeff. de la ligne I+1 est le coeff. diagonal, et LENV(NBLINE)-1 
*         est la taille de la partie triangulaire inferieure stricte compactee.
* MCAD   adresse de la diagonale de la matrice dans le super-tab. du bon type
* MCAL   adresse de la partie triangulaire inf. dans le super-tab. du bon type
* EPSIL  seuil en dessous duquel la matrice n'est pas factorisable
*        i.e. un coeff. Di de la diagonale est tel que Di < EPSIL
* MCD    adresse de la diagonale de la matrice factorisee dans RST ou CST
* MCL    adresse de la partie triangulaire inf. de la matrice factorisee
* RST,CST super-tableau version REELLE et COMPLEXE
* IRCODE code de retour en cas d'erreur
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) TYPE
      INTEGER       NBLINE,LENV(*),MCAD,MCAL,MCD,MCL,IRCODE
      REAL          EPSIL,RST(*)
      COMPLEX       CST(*)
*
      INTEGER       LIGNE,MCDD,MCADD,IADLNX,IAD,NBCOF,KOL1,MCLL,J
      REAL          RCOEFF
      COMPLEX       CCOEFF
      EQUIVALENCE  (CCOEFF,RCOEFF)
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*LtLPr*')
*
      IRCODE=0
      LIGNE=1
      MCDD=MCD+1
      MCADD=MCAD+1
      IADLNX=1
      IF (TYPE(1:1).EQ.'R') THEN 
*
*        Cas d'une matrice a coefficients reels
*
         IF (RST(MCAD).LT.EPSIL) GOTO 99992
         RST(MCD)=SQRT (RST(MCAD))
         DO 21 LIGNE=2,NBLINE
            RCOEFF=RST(MCADD)
            MCADD=MCADD+1
*
*           Calcul des coeff. de la i-eme ligne de L (i=LIGNE)
*           (Lik)_(k.le.i-1) est solution du systeme triangulaire inferieur:
*                 Somme(k.le.j-1) Lik Ljk = Aij pour j=1,..,i-1
*
            IAD=IADLNX
            IADLNX=LENV(LIGNE)
            NBCOF=IADLNX-IAD
            IF (NBCOF.GT.0) THEN
               KOL1=LIGNE-NBCOF
*
               CALL RSPRL  (NBCOF,LENV(KOL1),'R',MCD-1+KOL1,MCL
     &                     ,'R',MCAL-1+IAD,'R',MCL-1+IAD,RST,CST)        !syslin
*
*              Calcul du coeff. diagonal de la LIGNE-eme ligne/colonne
*                 Somme(k.le.i-1) Lik Lik + Lii^2 = Aii
*
               MCLL=MCL-1+IADLNX
               DO 20 J=1,NBCOF
                  MCLL=MCLL-1
20                RCOEFF=RCOEFF-RST(MCLL)*RST(MCLL)
            ENDIF
            IF (RCOEFF.LT.-EPSIL) GOTO 99992
            IF (ABS(RCOEFF).LT.EPSIL) GOTO 99991
            RST(MCDD)=SQRT (ABS(RCOEFF))
21          MCDD=MCDD+1 

      ELSE   
*        Cas d'une matrice a coefficients complexes
         IF (ABS(CST(MCAD)).LT.EPSIL) GOTO 99993
         CST(MCD)=SQRT(CST(MCAD))
         DO 41 LIGNE=2,NBLINE
            CCOEFF=CST(MCADD)
            MCADD=MCADD+1
            IAD=IADLNX
            IADLNX=LENV(LIGNE)
            NBCOF=IADLNX-IAD
            IF (NBCOF.GT.0) THEN
               KOL1=LIGNE-NBCOF
               CALL RSPRL  (NBCOF,LENV(KOL1),'C',MCD-1+KOL1,MCL
     &                     ,'C',MCAL-1+IAD,'C',MCL-1+IAD,RST,CST)       !syslin
               MCLL=MCL-1+IADLNX
               DO 40 J=1,NBCOF
                  MCLL=MCLL-1
40                CCOEFF=CCOEFF-CST(MCLL)*CONJG (CST(MCLL))
            ENDIF
            IF (ABS(CCOEFF).LT.EPSIL) GOTO 99993
            CST(MCDD)=SQRT (CCOEFF)
41          MCDD=MCDD+1
      ENDIF
      CALL PRFXMJ (-1,'*LtLPr*')
      RETURN
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
99991 WRITE(*,*) '        Coeff. diagonal de la ligne ',LIGNE,RCOEFF
      IRCODE=LIGNE
      RETURN
99992 WRITE(*,*) '        Coeff. diagonal de la ligne ',LIGNE,RCOEFF
      IRCODE=-LIGNE
      RETURN
99993 WRITE(*,*) '        Coeff. diagonal de la ligne ',LIGNE,CCOEFF
      IRCODE=LIGNE
      RETURN
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 2 : D.Martin (2.avr.2010)
* - bug fixed in comment "Lii^2 = Aii-Somme(k.le.i-1) Lik Lki"
* Version 1.0.2 : D.Martin (15 Juin 2002)
* Version 0 : A.S.Bonnet (sep.1989) puis D.Martin (juin.1991)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !LtLPr
