*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE LUIBM (TYPE,NBLIGN,NULAPL,IPLAGL,NULAPU,IPLAGU
     &                 ,MCAD,MCAL,MCAU,EPSIL,MCD,MCL,MCU,RST,CST)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Octobre 1991)
* Derniere modification : D.Martin (8 Juillet 1997)
*  
* Factorisation L1 x U incomplete 'BiMorse' de la matrice A
* (i.e. on ne calcule que les coeff. Lij et Uij t.q. Aij # 0)  
*  
* ou A  est une matrice stockee 'Bi-Morse' 
*    L1 est une matrice triangulaire inferieure a diagonale unite 
*    U  est une matrice triangulaire superieure
*    L et U sont stockees selon la meme structure 'Bi-Morse' que A
*  
*-- Arguments d'entree -- 
* TYPE   type (reel ou complexe) de la matrice A a factoriser  
* NBLIGN nombre de lignes de la matrice a factoriser
* NULAPL tableau des numeros de derniere plage de chaque ligne de la partie
*        triangulaire inferieure de A
* IPLAGL tableau des numeros de colonne de debut de plage et adresses
*        des derniers coefficients de chaque plage pour la part. triang. inf.
* NULAPU tableau des numeros de derniere plage de chaque ligne de la partie
*        triangulaire superieure de A
* IPLAGU tableau des numeros de colonne de debut de plage et adresses
*        des derniers coefficients de chaque plage pour la part. triang. inf.
* MCAD   adrs. de la diag. de la mat. A dans RST (reel) ou CST (complexe)
* MCAL   adrs. de la part. triang. inf. de la mat. A dans RST ou CST
* MCAU   adrs. de la part. triang. sup. de la mat. A dans RST ou CST
* EPSIL  seuil minimal des valeurs des coeff. diagonaux
* MCD    adrs. de la diag. de la mat. U dans RST (reel) ou CST (complexe)
* MCL    adrs. de la matrice L dans RST ou CST
* MCU    adrs. de la part. triang. sup. de la mat. U dans RST ou CST   
* RST,CST super-tableau version reelle  et complexe
*  
*-- Remarques --  
*-> Si on ne desire pas garder la matrice A, les tableaux A et L1 et U peuvent  
*   occuper la meme zone memoire, il suffit pour ce faire de rentrer les memes 
*   adresses (MCA et MCLU) dans la liste d'entree.
*  
*-> Lorsque la matrice A est symetrique en structure les tableaux de stockage  
*   'Bi-Morse' (NULAPL & IPLAGL) et colonne(NULAPU & IPLAGU) coincident.
*  
*-- Sous-programme utilise --
* MOLMOC multiplication d'une ligne de matrice 'Morse ligne' par une colonne 
*        d'une matrice 'Morse colonne' (Librairie MULTMAT)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPE
      INTEGER       NBLIGN,NULAPL(*),IPLAGL(2,*),NULAPU(*),IPLAGU(2,*)
     &             ,MCAD,MCAL,MCAU,MCD,MCL,MCU
      REAL          EPSIL,RST(*) 
      COMPLEX       CST(*) 
*
      INTEGER       LGCUAL,PRPLAL,DRPLAL,LGCUAU,PRPLAU,DRPLAU,FIRST
      REAL          RCOEFF
      COMPLEX       CCOEFF 
      EQUIVALENCE   (CCOEFF,RCOEFF)
      DATA PIVMIN/1.E-04/
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*LUiBm*')
* 
      DRPLAL=0  
      LGCUAL=0
      DRPLAU=0  
      LGCUAU=0
      LICOM1=1
*   
      IF (TYPE(1:1).EQ.'C') THEN
*
         CCOEFF=CST(MCAD)  
         IF (ABS(CCOEFF).LT.EPSIL) 
     &      CALL BAISE  (' Premier coefficient de la matrice nul!') 
         CST(MCD)=CCOEFF  
*    
         DO 49 LIGKOL=2,NBLIGN
            PRPLAL=DRPLAL+1
            DRPLAL=NULAPL(LICOM1)
            IF (DRPLAL.LT.PRPLAL) GOTO 45
*
            LGCUMU=LGCUAL
            IADAIJ=LGCUMU
            IF (IPLAGL(1,PRPLAL).EQ.1) THEN
*
*              Coeff. de la 1ere colonne : Li1=Ai1/A11  
               CST(MCL+IADAIJ)=CST(MCAL+IADAIJ)/CST(MCAD)
               IADAIJ=IADAIJ+1
            ENDIF
*
*           Calcul des coeff. de la i-eme ligne de L :
*           (Lik ,k=1,..,i-1) est solution du systeme (triang. inferieur):
*           Somme(k.le.j-1) Lik Ukj = Aij  ,  j=1,..,i-1
*
            DO 44 NUPLA=PRPLAL,DRPLAL
               FIRST=IPLAGL(1,NUPLA)
               LGCUAV=LGCUMU+1
               LGCUMU=IPLAGL(2,NUPLA)
               LAST=FIRST+LGCUMU-LGCUAV
               LGCUU =0
               IF (FIRST.LE.2) THEN
                  FIRST=2
                  NUMPL2=0
               ELSE
                  NUMPL2=NULAPU(FIRST-2)
                  IF (NUMPL2.GT.0) LGCUU =IPLAGU(2,NUMPL2)
               ENDIF
               KOLLI1=FIRST-1
               DO 43 KOLLIG=FIRST,LAST
                  LGCUL =LGCUAL
                  NUMPL1=NUMPL2+1
                  NUMPL2=NULAPU(KOLLI1)
                  CCOEFF=0.
                  IF (NUMPL2.GE.NUMPL1) THEN
                    CALL MOLMOC (KOLLI1,PRPLAL,NUPLA ,LGCUL,IPLAGL
     &                          ,TYPE,MCL,RST,CST,NUMPL1,NUMPL2,LGCUU
     &                          ,IPLAGU,TYPE,MCU,RST,CST,CCOEFF)        !Multmat
                    LGCUU =IPLAGU(2,NUMPL2)
                  ENDIF
                  CCOEFF         =CST(MCAL+IADAIJ)-CCOEFF
                  CST(MCL+IADAIJ)=CCOEFF/CST(MCD+KOLLI1)
                  IADAIJ=IADAIJ+1 
                  KOLLI1=KOLLIG
43             CONTINUE
44          CONTINUE
*
45          CCOEFF=0.
            PRPLAU=DRPLAU+1
            DRPLAU=NULAPU(LICOM1)
            IF (DRPLAU.LT.PRPLAU) GOTO 48
            LGCUMU=LGCUAU
            IADAIJ=LGCUMU
            IF (IPLAGU(1,PRPLAU).EQ.1) THEN
*
*              Coeff. de la 1ere ligne : U1i=A1i  
               CST(MCU+IADAIJ)=CST(MCAU+IADAIJ)
               IADAIJ=IADAIJ+1
            ENDIF
*
*           Calcul des coeff. de la i-eme colonne de U :
*           (Uki k=1,..,i-1) est solution du systeme (triang. inferieur):
*           Somme(k.le.j-1) Ljk Uki + Uji = Aji  ,  j=1,..,i-1
*
            DO 47 NUPLA=PRPLAU,DRPLAU
               FIRST=IPLAGU(1,NUPLA) 
               LGCUAV=LGCUMU+1
               LGCUMU=IPLAGU(2,NUPLA)
               LAST=FIRST+LGCUMU-LGCUAV
               LGCUL =0
               IF (FIRST.LE.2) THEN
                  FIRST=2
                  NUMPL2=0
               ELSE
                  NUMPL2=NULAPL(FIRST-2)
                  IF (NUMPL2.GT.0) LGCUL =IPLAGL(2,NUMPL2)
               ENDIF
               KOLLI1=FIRST-1
               DO 46 KOLLIG=FIRST,LAST
                  LGCUU =LGCUAU
                  NUMPL1=NUMPL2+1
                  NUMPL2=NULAPL(KOLLI1)
                  CCOEFF=0.
                  IF (NUMPL2.GE.NUMPL1) THEN
                    CALL MOLMOC (KOLLI1,PRPLAU,NUPLA ,LGCUU,IPLAGU
     &                          ,TYPE,MCU,RST,CST,NUMPL1,NUMPL2,LGCUL
     &                          ,IPLAGL,TYPE,MCL,RST,CST,CCOEFF)        !Multmat
                    LGCUL =IPLAGL(2,NUMPL2)
                  ENDIF
                  CST(MCU+IADAIJ)=CST(MCAU+IADAIJ)-CCOEFF

                  IADAIJ=IADAIJ+1 
                  KOLLI1=KOLLIG
46             CONTINUE
47          CONTINUE
*  
*           Calcul du coefficient diagonal Uii :
*           Uii=Aii-Somme(k.le.i-1) Lik Uki
            CALL MOLMOC (LICOM1,PRPLAL,DRPLAL,LGCUAL,IPLAGL,TYPE
     &                  ,MCL,RST,CST,PRPLAU,DRPLAU,LGCUAU,IPLAGU
     &                  ,TYPE,MCU,RST,CST,CCOEFF)                       !Multmat
*
48          CCOEFF=CST(MCAD+LICOM1)-CCOEFF 
            IF (ABS(CCOEFF).LE.EPSIL) THEN
               CCOEFF=PIVMIN  
            ENDIF
            CST(MCD+LICOM1)=CCOEFF
            LICOM1=LIGKOL
            LGCUAL=IPLAGL(2,DRPLAL)
            LGCUAU=IPLAGU(2,DRPLAU)
49       CONTINUE
*
      ELSE
*
         RCOEFF=RST(MCAD)  
         IF (ABS(RCOEFF).LT.EPSIL) 
     &      CALL BAISE  (' Premier coefficient de la matrice nul!') 
         RST(MCD)=RCOEFF  
* 
         DO 29 LIGKOL=2,NBLIGN
            PRPLAL=DRPLAL+1
            DRPLAL=NULAPL(LICOM1)
            IF (DRPLAL.LT.PRPLAL) GOTO 25
*
            LGCUMU=LGCUAL
            IADAIJ=LGCUMU
            IF (IPLAGL(1,PRPLAL).EQ.1) THEN
*
*              Coeff. de la 1ere colonne : Li1=Ai1/A11  
               RST(MCL+IADAIJ)=RST(MCAL+IADAIJ)/RST(MCAD)
               IADAIJ=IADAIJ+1
            ENDIF
*
*           Calcul des coeff. de la i-eme ligne de L :
*           (Lik ,k=1,..,i-1) est solution du systeme (triang. inferieur):
*           Somme(k.le.j-1) Lik Ukj = Aij  ,  j=1,..,i-1
*
            DO 24 NUPLA=PRPLAL,DRPLAL
               FIRST=IPLAGL(1,NUPLA)
               LGCUAV=LGCUMU+1
               LGCUMU=IPLAGL(2,NUPLA)
               LAST=FIRST+LGCUMU-LGCUAV
               LGCUU =0
               IF (FIRST.LE.2) THEN
                  FIRST=2
                  NUMPL2=0
               ELSE
                  NUMPL2=NULAPU(FIRST-2)
                  IF (NUMPL2.GT.0) LGCUU =IPLAGU(2,NUMPL2)
               ENDIF
               KOLLI1=FIRST-1
               DO 23 KOLLIG=FIRST,LAST
                  LGCUL =LGCUAL
                  NUMPL1=NUMPL2+1
                  NUMPL2=NULAPU(KOLLI1)
                  CCOEFF=0.
                  IF (NUMPL2.GE.NUMPL1) THEN
                    CALL MOLMOC (KOLLI1,PRPLAL,NUPLA ,LGCUL ,IPLAGL
     &                          ,TYPE,MCL,RST,RST,NUMPL1,NUMPL2,LGCUU 
     &                          ,IPLAGU,TYPE,MCU,RST,RST,CCOEFF)        !Multmat
                    LGCUU =IPLAGU(2,NUMPL2)
                  ENDIF
                  RCOEFF         =RST(MCAL+IADAIJ)-RCOEFF
                  RST(MCL+IADAIJ)=RCOEFF/RST(MCD+KOLLI1)
                  IADAIJ=IADAIJ+1 
                  KOLLI1=KOLLIG
23             CONTINUE
24          CONTINUE
*
25          RCOEFF=0.
            PRPLAU=DRPLAU+1
            DRPLAU=NULAPU(LICOM1)
            IF (DRPLAU.LT.PRPLAU) GOTO 28
            LGCUMU=LGCUAU
            IADAIJ=LGCUMU
            IF (IPLAGU(1,PRPLAU).EQ.1) THEN
*
*              Coeff. de la 1ere ligne : U1i=A1i  
               RST(MCU+IADAIJ)=RST(MCAU+IADAIJ)
               IADAIJ=IADAIJ+1
            ENDIF
*
*           Calcul des coeff. de la i-eme colonne de U :
*           (Uki k=1,..,i-1) est solution du systeme (triang. inferieur):
*           Somme(k.le.j-1) Ljk Uki + Uji = Aji  ,  j=1,..,i-1
*
            DO 27 NUPLA=PRPLAU,DRPLAU
               FIRST=IPLAGU(1,NUPLA) 
               LGCUAV=LGCUMU+1
               LGCUMU=IPLAGU(2,NUPLA)
               LAST=FIRST+LGCUMU-LGCUAV
               LGCUL =0
               IF (FIRST.LE.2) THEN
                  FIRST=2
                  NUMPL2=0
               ELSE
                  NUMPL2=NULAPL(FIRST-2)
                  IF (NUMPL2.GT.0) LGCUL =IPLAGL(2,NUMPL2)
               ENDIF
               KOLLI1=FIRST-1
               DO 26 KOLLIG=FIRST,LAST
                  LGCUU =LGCUAU
                  NUMPL1=NUMPL2+1
                  NUMPL2=NULAPL(KOLLI1)
                  CCOEFF=0.
                  IF (NUMPL2.GE.NUMPL1) THEN
                    CALL MOLMOC (KOLLI1,PRPLAU,NUPLA,LGCUU,IPLAGU
     &                          ,TYPE,MCU,RST,RST,NUMPL1,NUMPL2,LGCUL
     &                          ,IPLAGL,TYPE,MCL,RST,RST,CCOEFF)        !Multmat
                    LGCUL =IPLAGL(2,NUMPL2)
                  ENDIF
                  RST(MCU+IADAIJ)=RST(MCAU+IADAIJ)-RCOEFF
                  IADAIJ=IADAIJ+1 
                  KOLLI1=KOLLIG
26             CONTINUE
27          CONTINUE
*  
*           Calcul du coefficient diagonal Uii :
*           Uii=Aii-Somme(k.le.i-1) Lik Uki
            CALL MOLMOC (LICOM1,PRPLAL,DRPLAL,LGCUAL,IPLAGL,TYPE,MCL
     &                  ,RST,RST,PRPLAU,DRPLAU,LGCUAU,IPLAGU,TYPE,MCU
     &                  ,RST,RST,CCOEFF)                                !Multmat
*
28          RCOEFF=RST(MCAD+LICOM1)-RCOEFF 
            IF (ABS(RCOEFF).LE.EPSIL) THEN
               RCOEFF=PIVMIN  
            ENDIF
            RST(MCD+LICOM1)=RCOEFF
            LICOM1=LIGKOL
            LGCUAL=IPLAGL(2,DRPLAL)
            LGCUAU=IPLAGU(2,DRPLAU)
29       CONTINUE
      ENDIF
      CALL PRFXMJ (-1,'*LUiBm*')  
*
*10000 FORMAT(/T2,A8,'  Coefficient diagonal de U nul sur la ligne' 
*     &                         ,I5,' valeur: ',E12.5,:,SP,E12.5,'*i')  
                                                                    END !LUiBm
