*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE LUIMO (TYPE,NBNELI,NULAPL,IPLAGE,MCA,EPSIL,MCLU
     &                 ,LPALUN,IEGALP,MCUW,NUPLDG,NUPLLI,RST,CST)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Fevrier 90)
* Derniere modification : D.Martin (3 Juillet 1997)
*  
* Factorisation L x U incomplete 'Morse' par ligne de la matrice A
*   (i.e. on ne calcule que les coeff. Lij et Uij t.q. Aij # 0)  
*  
* Ou A est une matrice stockee 'Morse' en ligne (tableaux NULAPL & IPLAGE)
*      dont on connait aussi les tableaux de stockage 'Morse' colonne
*      (Tableaux LPALUN & IEGALP)
*    L est une matrice triangulaire inferieure a diagonale unite 
*    U est une matrice triangulaire superieure
*  
*-- Arguments d'entree -- 
* TYPE   type (reel ou complexe) de la matrice A a factoriser  
* NBNELI nombre de lignes de la matrice a factoriser
* NULAPL tableau des numeros de derniere plage de chaque ligne de A
* IPLAGE tableau des numeros de colonne de debut de plage et longueurs 
*        cumulees de chaque plage (compactage 'Morse' par ligne)
* MCA    adresse de la matrice A dans le tableau RST (reel) ou CST (complexe)
* EPSIL  seuil minimal des coeff. diagonaux  
* MCLU   adresse des mat. L et U dans le tableau RST (reel) ou CST (complexe)  
* LPALUN tableau des numeros de derniere plage de chaque colonne de la matrice A
*        ou de sa partie triangulaire seule (compactage 'Morse' par colonne)
* IEGALP tableau des numeros de ligne de debut de plage et longueurs cumulees  
*        de chaque plage (compactage 'Morse' par colonne) de la matrice A  
*        ou de sa partie triangulaire seule
* MCUW   adresse du vecteur de travail pour le stockage de la matrice U
*        compactee selon le compactage 'Morse' par colonne de A
* NUPLDG tableau de travail pour stocker les numeros des plages contenant  
*        le coefficient diagonal dans le compactage en colonne de Uw
* NUPLLI tableau de travail pour le stockage des numeros de plage du coeff.
*        i de la colonne j de Uw pour le passage d'une ligne a l'autre 
* RST,CST super-table et complexeau version reelle 
*  
*-- Remarques --  
*-> en sortie les matrices L et U sont stockees 'Morse' ligne dans le  
*   tableau RST ou CST selon les memes tableaux de compactage que A
*  
*-> Le tableau Uw est un vecteur de travail de la longueur de A ou la partie
*   triangulaire superieure de la factorisee est stockee selon le compactage
*   'Morse' colonne de A (tableaux LPALUN & IEGALP)
*  
*-> Si on ne desire pas garder la matrice A, les tableaux A et LU peuvent  
*   occuper la meme zone memoire, il suffit pour ce faire de rentrer les memes 
*   adresses (MCA et MCLU) dans la liste d'entree  
*  
*-> Lorsque la matrice A est symetrique en structure les tableaux de stockage  
*   'Morse' ligne(NULAPL & IPLAGE) et colonne(LPALUN & IEGALP) peuvent coincider
*  
*-- Sous-programmes utilises --
* DIKLPL recherche dichotomique de la plage d'une ligne contenant une colonne
*          (Librairie MORSE)
* MOLMOC multiplication d'une ligne de matrice 'Morse ligne' par une colonne 
*          d'une matrice 'Morse colonne' (Librairie MULTMAT)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPE 
      INTEGER       NULAPL(*),IPLAGE(2,*),LPALUN(*),IEGALP(2,*)
     &             ,NUPLDG(*),NUPLLI(*)
      REAL          RST(*) 
      COMPLEX       CST(*) 
*  
      CHARACTER     ERCODE*120,PRFXAF*8 
      COMPLEX       CCOEFF 
      INTEGER       PRPLA,DRPLA,DGPLA,PRPLU,DRPLU,DGPLU
      COMMON/FORMAH/ERCODE 
      EQUIVALENCE   (CCOEFF,RCOEFF)
      DATA PIVMIN/1.E-04/
*  
* --- Qelques definitions de variables locales ------------------------
*  
* PRPLA  numero de la 1ere plage de la ligne i lors du calcul de Lij ou Uij
* DRPLA  numero de la derniere plage de la ligne i  "   "     "
* DGPLA  numero de la plage de la ligne i contenant le coeff. diagonal 
* LGCUAV nb. de coeff. des plages precedant la 1ere plage de la ligne i
* LGCUA  nb. de coeff. des plages precedant la plage courante de la ligne i
* KOLDEB numero de debut de colonne de la plage courante dans A
* KOLFIN numero de colonne de fin de plage de la plage courante dans A 
* IADAIJ  adresse dans A ou LU du coeff. courant Lij ou Uij
* LGCUL  nb. de coeff. des plages precedant une plage de la ligne courante de L
* PRPLU  numero de derniere plage de la colonne courante de Uw 
* LGCUU  nb de coeff. des plages precedant la plage courante de la colonne de Uw
*-----------------------------------------------------------------------
*  
      CALL PRFXMJ (1,'*LUiMo*')
*  
***   Report de la 1ere ligne de A dans la 1ere ligne de U selon les
*     deux types de compactages: 'Morse' ligne et 'Morse' colonne de A .
*  
      IF (TYPE(1:1).EQ.'C') THEN
*  
      CCOEFF=CST(MCA)  
      IF (ABS(CCOEFF).LT.EPSIL) 
     &   CALL BAISE(' Premier coefficient de la matrice nul!') 
      CST(MCUW)=CCOEFF 
      CST(MCLU)=CCOEFF 
      NUPLLI(1)=1  
      NUPLDG(1)=1  
      IADAIJ=1 
      DRPLU=LPALUN(1)  
      DO 40 KOLONJ=2,NBNELI
         LGCUU=IEGALP(2,DRPLU) 
         PRPLU=DRPLU+1 
         DRPLU=LPALUN(KOLONJ)  
*        Stockage de la plage diagonale de la colonne en cours de Uw
         CALL DIKLPL (KOLONJ,PRPLU,DRPLU,LGCUU,IEGALP,NUPLU,*90000)     !Morse 
         NUPLLI(KOLONJ)=PRPLU  
         NUPLDG(KOLONJ)=NUPLU  
         IF (IEGALP(1,PRPLU).EQ.1) THEN 
            CCOEFF=CST(MCA+IADAIJ) 
            CST(MCUW+LGCUU)=CCOEFF 
            CST(MCLU+IADAIJ)=CCOEFF
*           -----------------------
            IADAIJ=IADAIJ+1
         ENDIF
40    CONTINUE 
*  
***   Boucle sur les lignes (i > 1) des coefficents Lij et Uij a calculer  
*  
      DRPLA=NULAPL(1)  
      LGCUAV=IPLAGE(2,DRPLA)
      LIGIM1=1 
      DO 49 LIGNEI=2,NBNELI
         PRPLA =DRPLA+1
         DRPLA =NULAPL(LIGNEI) 
         LGCUA =LGCUAV 
*  
**       Boucle sur les plages 'sous-diagonales' de la ligne i :
**       Calcul de Lij pour i > j et Aij <> 0  
*  
         DO 42 NUPLA=PRPLA,DRPLA
*           adresse precedant le 1er coeff. Aij de la ligne i  
            LGP=LGCUA  
            LGCUA=IPLAGE(2,NUPLA)  
            KOLDEB=IPLAGE(1,NUPLA) 
            KOLFIN=KOLDEB+LGCUA-LGP-1  
            IF (KOLDEB.EQ.1) THEN
*              Calcul des coeff. de la 1ere colonne (Li1=Ai1/A11)  
               CST(MCLU+IADAIJ)=CST(MCA+IADAIJ)/CST(MCA)
*              -----------------------------------------
               IADAIJ=IADAIJ+1 
               KOLDEB=KOLDEB+1 
            ENDIF 
*           Boucle sur les colonnes (2<j<i) des coeff. Lij a calculer  
*           adresse dans Uw precedant la colonne KOLDEB
            DRPLU=LPALUN(KOLDEB-1) 
            LGCUU=IEGALP(2,DRPLU)  
            DO 41 KOLONJ=KOLDEB,MIN (KOLFIN,LIGIM1)
*              Calcul de Somme (Lik * Ukj) / Ujj pour 0<k<j tq Aik et Akj <> 0 
               PRPLU=DRPLU+1
               DRPLU=LPALUN(KOLONJ)
               DGPLU=NUPLDG(KOLONJ)
               LGCUL=LGCUAV
               CALL MOLMOC (KOLONJ-1,PRPLA,NUPLA,LGCUL,IPLAGE,TYPE,MCLU
     &                     ,RST,CST,PRPLU,DGPLU,LGCUU,IEGALP,TYPE,MCUW
     &                     ,RST,CST,CCOEFF)                             !Multmat
               CCOEFF=CST(MCA+IADAIJ)-CCOEFF
*              Division par le coefficient diagonal (Ujj)  
               MCUWIJ=MCUW+IEGALP(2,DGPLU-1)+KOLONJ-IEGALP(1,DGPLU)
               CST(MCLU+IADAIJ)=CCOEFF/CST(MCUWIJ) 
*              ----------------------------------- 
               LGCUU=IEGALP(2,DRPLU)
41             IADAIJ=IADAIJ+1 
            IF (KOLFIN.GE.LIGNEI) GO TO 43  
42          CONTINUE
         GOTO 90001
*  
43       DGPLA=NUPLA
         LGCUA=LGP 
*  
**       Calcul du coefficient diagonal Uii
*  
         PRPLU=DRPLU+1 
         DGPLU=NUPLDG(LIGNEI)  
         LGCUL=LGCUAV  
         CALL MOLMOC (LIGIM1,PRPLA,DGPLA,LGCUL,IPLAGE,TYPE,MCLU,RST,CST
     &               ,PRPLU,DGPLU,LGCUU,IEGALP,TYPE,MCUW,RST,CST,CCOEFF)!Multmat
         CCOEFF=CST(MCA+IADAIJ)-CCOEFF 
         IF (ABS(CCOEFF).LE.EPSIL) THEN 
            WRITE (*,10000) PRFXAF (),LIGNEI,CCOEFF
            CCOEFF=PIVMIN  
         ENDIF
         MCUWIJ=MCUW+IEGALP(2,DGPLU-1)+LIGNEI-IEGALP(1,DGPLU)  
         CST(MCUWIJ)=CCOEFF
         CST(MCLU+IADAIJ)=CCOEFF
*        -----------------------
         IADAIJ=IADAIJ+1
*  
**       Boucle sur les plages 'sur-diagonales' de la ligne i :
**       Calcul de Uij pour i < j et Aij <> 0  
*  
         DO 48 NUPLA=DGPLA,DRPLA
            LPG=LGCUA  
            LGCUA=IPLAGE(2,NUPLA)  
            KOLDEB=IPLAGE(1,NUPLA) 
            KOLFIN=KOLDEB+LGCUA-LPG-1  
            KOLDEB=MAX(KOLDEB,LIGNEI+1)
            DRPLU=LPALUN(KOLDEB-1) 
            LGCUU=IEGALP(2,DRPLU)  
*           Boucle sur les colonnes (j>i) des coeff. Uij a calculer
            DO 48 KOLONJ=KOLDEB,KOLFIN 
*              Calcul de Somme (Lik * Ukj) pour 0<k<i tel que Aik et Akj <> 0  
               PRPLU=DRPLU+1
               DRPLU=LPALUN(KOLONJ)
               NUPLI=NUPLLI(KOLONJ)
*              Stockage de la plage contenant le coeff. Uij dans Uw
               CALL DIKLPL (LIGNEI,NUPLI,NUPLDG(KOLONJ) 
     &                     ,IEGALP(2,NUPLI-1),IEGALP,LIPLU,*90002)      !Morse 
               NUPLLI(KOLONJ)=LIPLU
               LGCUL=LGCUAV
               CALL MOLMOC (LIGIM1,PRPLA,DGPLA,LGCUL,IPLAGE,TYPE,MCLU
     &                     ,RST,CST,PRPLU,LIPLU,LGCUU,IEGALP,TYPE,MCUW
     &                     ,RST,CST,CCOEFF)                             !Multmat
               CCOEFF=CST(MCA+IADAIJ)-CCOEFF
               MCUWIJ=MCUW+IEGALP(2,LIPLU-1)+LIGNEI-IEGALP(1,LIPLU)
               CST(MCUWIJ)=CCOEFF  
               CST(MCLU+IADAIJ)=CCOEFF 
*              ----------------------- 
               LGCUU=IEGALP(2,DRPLU)
               IADAIJ=IADAIJ+1 
48          CONTINUE
         LIGIM1=LIGNEI 
49       LGCUAV=LGCUA  
      ELSE 
*  
      RCOEFF=RST(MCA)  
      IF (ABS(RCOEFF).LT.EPSIL) 
     &   CALL BAISE(' Premier coefficient de la matrice nul!') 
      RST(MCUW)=RCOEFF 
      RST(MCLU)=RCOEFF 
      NUPLLI(1)=1  
      NUPLDG(1)=1  
      IADAIJ=1 
      DRPLU=LPALUN(1)  
      DO 10 KOLONJ=2,NBNELI
         LGCUU=IEGALP(2,DRPLU) 
         PRPLU=DRPLU+1 
         DRPLU=LPALUN(KOLONJ)  
         CALL DIKLPL (KOLONJ,PRPLU,DRPLU,LGCUU,IEGALP,NUPLU,*90000)     !Morse 
         NUPLLI(KOLONJ)=PRPLU  
         NUPLDG(KOLONJ)=NUPLU  
         IF (IEGALP(1,PRPLU).EQ.1) THEN 
            RCOEFF=RST(MCA+IADAIJ) 
            RST(MCUW+LGCUU)=RCOEFF 
            RST(MCLU+IADAIJ)=RCOEFF
*           -----------------------
            IADAIJ=IADAIJ+1
         ENDIF
10    CONTINUE 
      DRPLA=NULAPL(1)  
      LGCUAV=IPLAGE(2,DRPLA)
      LIGIM1=1 
      DO 19 LIGNEI=2,NBNELI
         PRPLA =DRPLA+1
         DRPLA =NULAPL(LIGNEI) 
         LGCUA =LGCUAV 
         DO 12 NUPLA=PRPLA,DRPLA
            LGP=LGCUA  
            LGCUA=IPLAGE(2,NUPLA)  
            KOLDEB=IPLAGE(1,NUPLA) 
            KOLFIN=KOLDEB+LGCUA-LGP-1  
            IF (KOLDEB.EQ.1) THEN
               RST(MCLU+IADAIJ)=RST(MCA+IADAIJ)/RST(MCA)
*              -----------------------------------------
               IADAIJ=IADAIJ+1 
               KOLDEB=KOLDEB+1 
            ENDIF 
            DRPLU=LPALUN(KOLDEB-1) 
            LGCUU=IEGALP(2,DRPLU)  
            DO 11 KOLONJ=KOLDEB,MIN (KOLFIN,LIGIM1)
               PRPLU=DRPLU+1
               DRPLU=LPALUN(KOLONJ)
               DGPLU=NUPLDG(KOLONJ)
               LGCUL=LGCUAV
               CALL MOLMOC (KOLONJ-1,PRPLA,NUPLA,LGCUL,IPLAGE,TYPE,MCLU 
     &                     ,RST,CST,PRPLU,DGPLU,LGCUU,IEGALP,TYPE,MCUW 
     &                     ,RST,CST,CCOEFF)                             !Multmat
               RCOEFF=RST(MCA+IADAIJ)-RCOEFF
               MCUWIJ=MCUW+IEGALP(2,DGPLU-1)+KOLONJ-IEGALP(1,DGPLU)
               RST(MCLU+IADAIJ)=RCOEFF/RST(MCUWIJ) 
*              ----------------------------------- 
               LGCUU=IEGALP(2,DRPLU)
11             IADAIJ=IADAIJ+1 
            IF (KOLFIN.GE.LIGNEI) GOTO 13  
12          CONTINUE
         GOTO 90001
13       DGPLA=NUPLA
         LGCUA=LGP 
         PRPLU=DRPLU+1 
         DGPLU=NUPLDG(LIGNEI)  
         LGCUL=LGCUAV  
         CALL MOLMOC (LIGIM1,PRPLA,DGPLA,LGCUL,IPLAGE,TYPE,MCLU,RST,CST
     &               ,PRPLU,DGPLU,LGCUU,IEGALP,TYPE,MCUW,RST,CST,CCOEFF)!Multmat
         RCOEFF=RST(MCA+IADAIJ)-RCOEFF 
         IF (ABS(RCOEFF).LE.EPSIL) THEN 
            WRITE (*,10000) PRFXAF ( ),LIGNEI,RCOEFF
            RCOEFF=PIVMIN  
         ENDIF
         MCUWIJ=MCUW+IEGALP(2,DGPLU-1)+LIGNEI-IEGALP(1,DGPLU)  
         RST(MCUWIJ)=RCOEFF
         RST(MCLU+IADAIJ)=RCOEFF
*        -----------------------
         IADAIJ=IADAIJ+1
         DO 18 NUPLA=DGPLA,DRPLA
            LPG=LGCUA  
            LGCUA=IPLAGE(2,NUPLA)  
            KOLDEB=IPLAGE(1,NUPLA) 
            KOLFIN=KOLDEB+LGCUA-LPG-1  
            KOLDEB=MAX(KOLDEB,LIGNEI+1)
            DRPLU=LPALUN(KOLDEB-1) 
            LGCUU=IEGALP(2,DRPLU)  
            DO 18 KOLONJ=KOLDEB,KOLFIN 
               PRPLU=DRPLU+1
               DRPLU=LPALUN(KOLONJ)
               NUPLI=NUPLLI(KOLONJ)
               CALL DIKLPL (LIGNEI,NUPLI,NUPLDG(KOLONJ) 
     &                     ,IEGALP(2,NUPLI-1),IEGALP,LIPLU,*90002)      !Morse 
               NUPLLI(KOLONJ)=LIPLU
               LGCUL=LGCUAV
               CALL MOLMOC (LIGIM1,PRPLA,DGPLA,LGCUL,IPLAGE,TYPE,MCLU
     &                     ,RST,CST,PRPLU,LIPLU,LGCUU,IEGALP,TYPE,MCUW
     &                     ,RST,CST,CCOEFF)                             !Multmat
               RCOEFF=RST(MCA+IADAIJ)-RCOEFF
               MCUWIJ=MCUW+IEGALP(2,LIPLU-1)+LIGNEI-IEGALP(1,LIPLU)
               RST(MCUWIJ)=RCOEFF  
               RST(MCLU+IADAIJ)=RCOEFF 
*              ----------------------- 
               LGCUU=IEGALP(2,DRPLU)
               IADAIJ=IADAIJ+1 
18          CONTINUE
         LIGIM1=LIGNEI 
19       LGCUAV=LGCUA  
      ENDIF
      CALL PRFXMJ (-1,'*LUiMo*') 
      RETURN
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
10000 FORMAT(/T2,A8,'  Coefficient diagonal de U nul sur la ligne' 
     &,I5,' valeur: ',E12.5,:,SP,E12.5,'*i')  
90000 CALL ENCLER (KOLONJ,ERCODE(:6))
      CALL BAISE  ('La colonne '//ERCODE(1:6)//' de Uw ne contient pas'
     &//' de coefficient diagonal ?')                                   !Utilite
90001 CALL ENCLER (LIGNEI,ERCODE(:6))                                   !Utilite
      CALL BAISE  ('Pas de plage diagonale sur la ligne'//ERCODE(:6)
     &//' de la matrice A ?')                                           !Utilite
90002 CALL ENCLER (KOLONJ,ERCODE(:18))                                  !Utilite
      CALL ENCLER (LIGNEI,ERCODE(:12))                                  !Utilite
      CALL ENCLER (IADAIJ+1,ERCODE(:6))                                 !Utilite
      CALL BAISE  (' La colonne '//ERCODE(13:18)//' ne contient pas de'
     &//' coefficient non nul en ligne '//ERCODE(7:12)//', alors que la'
     &//' matrice A y contient un coefficient non nul a l''adresse'    
     &//' relative '//ERCODE(:6)//' !')
                                                                    END !LUiMo
