*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE LUPR (TYPE,NBLINE,LENV,UENV,MCAD,MCAL,MCAU,EPSIL
     &                ,MCD,MCL,MCU,RST,CST,IRCODE)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : A.S.Bonnet (Septembre 1989) puis D.Martin (Juin 1991)
* Derniere modification : D.Martin (19 mars 2008)
* Version 2
*  
* Factorisation L.U d'une matrice stockee 'Profil'.
*  L est une matrice triangulaire inferieure a diagonale unite
*  U est une matrice triangulaire superieure
*  En sortie L et U sont stockee selon le meme Profil, et le cas
*  echeant dans le meme tableau que la matrice initiale.
*  
*-- Arguments de la Liste d'Entree --
* TYPE   type REEL ou COMPLEXE de la matrice
* NBLINE nombre de lignes de la matrice
* LENV   tableau contenant l'enveloppe inferieure stricte avec
*        LENV(I-1) adresse dans la partie triangulaire inferieure compactee
*        du 1er coeff. non nul de la ligne I, et LENV(I)=LENV(I-1) si le 1er 
*        coeff. de la ligne I+1 est le coeff. diagonal, et LENV(NBLINE)-1 
*        est la taille de la partie triangulaire inferieure stricte compactee.
* UENV   tableau contenant l'enveloppe inferieure stricte avec
*        UENV(I-1) adresse dans la partie triangulaire superieure compactee
*        du 1er coeff. non nul de la colonne I, et UENV(I)=UENV(I-1) si le 1er 
*        coeff. de la colonne I+1 est le coeff. diagonal, et UENV(NBLINE)-1 
*        est la taille de la partie triangulaire superieure stricte compactee.
* MCAD   adresse de la diagonale de la matrice dans le super-tab. du bon type
* MCAL   adresse de la partie triangulaire inf. dans le super-tab. du bon type
* MCAU   adresse de la partie triangulaire sup. dans le super-tab. du bon type
* EPSIL  seuil en dessous duquel la matrice n'est pas factorisable
*        i.e. un coeff. Di de la diagonale est tel que ABS(Di) < EPSIL
* MCD    adresse de la diagonale de la matrice factorisee dans RST ou CST
* MCL    adresse de la partie triangulaire inf. de la matrice factorisee
* MCU    adresse de la partie triangulaire sup. de la matrice factorisee
* RST    super-tableau version REELLE
* CST    super-tableau version COMPLEXE
* IRCODE code de retour en cas d'erreur
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPE
      INTEGER       NBLINE,LENV(*),UENV(*),MCAD,MCAL,MCAU,MCD,MCL,MCU
     &             ,IRCODE
      REAL          RST(*)
      COMPLEX       CST(*)
*
      INTEGER       MCDD,MCADD,MCLL,MCUU
     &             ,IADLNX,IAD,NBCOFU,LIG1,NBCOFL,KOL1,LIGNE
      REAL          RCOEFF
      COMPLEX       CCOEFF
      EQUIVALENCE  (CCOEFF,RCOEFF)
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*LUPr*')
      IRCODE=0
      LIGNE =1
      MCDD  =MCD+1
      MCADD =MCAD+1
      IADUNX=1
      IADLNX=1
      IF (TYPE(1:1).EQ.'R') THEN
*    
*        Cas d'une matrice a coefficients reels
*
         IF (ABS(RST(MCAD)).LT.EPSIL) GOTO 99991
         RST(MCD)=RST(MCAD)
         DO 21 LIGNE=2,NBLINE
*
*           Calcul des coeff. de la LIGNE-eme colonne de U :
*           Uki (k.le.i-1) est solution du systeme triangulaire inferieur:
*                 Somme(k.le.j-1) Ljk Uki + Uji = Aji ,  j=1,..,i-1
            IAD=IADUNX
            IADUNX=UENV(LIGNE)
            NBCOFU=IADUNX-IAD
            IF (NBCOFU.GT.0) THEN
               LIG1=LIGNE-NBCOFU
               CALL RSPRL1 (NBCOFU,LENV(LIG1),'R',MCL,'R',MCAU-1+IAD
     &                     ,'R',MCU-1+IAD,RST,CST)                      !syslin
            ENDIF
*
*           Calcul des coeff. de la LIGNE-eme ligne de L
*           Lik (k.le.i-1) est solution du systeme triangulaire inferieur:
*                 Somme(k.le.j-1) Lik Ukj = Aij ,  j=1,..,i-1
            IAD=IADLNX
            IADLNX=LENV(LIGNE)
            NBCOFL=IADLNX-IAD
            IF (NBCOFL.GT.0) THEN
               KOL1=LIGNE-NBCOFL
               CALL RSPRL (NBCOFL,UENV(KOL1),'R',MCD-1+KOL1,MCU
     &                    ,'R',MCAL-1+IAD,'R',MCL-1+IAD,RST,CST)        !syslin
            ENDIF
*
*           Calcul du coeff. diagonal de la LIGNE-eme ligne/colonne
*                  Uii=Aii-Somme(k.le.i-1) Lik Uki
            RCOEFF=0.
            MCLL=MCL-1+IADLNX
            MCUU=MCU-1+IADUNX
            DO 20 J=1,MIN(NBCOFU,NBCOFL)
               MCLL=MCLL-1
               MCUU=MCUU-1
20             RCOEFF=RCOEFF+RST(MCLL)*RST(MCUU)
            RST(MCDD)=RST(MCADD)-RCOEFF
            IF (ABS(RST(MCDD)).LT.EPSIL) GOTO 99992
            MCDD=MCDD+1
21          MCADD=MCADD+1

      ELSE
*    
*        Cas d'une matrice a coefficients complexes
*
         IF (ABS(CST(MCAD)).LT.EPSIL) GOTO 99993
         CST(MCD)=CST(MCAD)
         DO 41 LIGNE=2,NBLINE
            IAD=IADUNX
            IADUNX=UENV(LIGNE)
            NBCOFU=IADUNX-IAD
            IF (NBCOFU.GT.0) THEN
               LIG1=LIGNE-NBCOFU
               CALL RSPRL1 (NBCOFU,LENV(LIG1),'C',MCL,'C',MCAU-1+IAD
     &                     ,'C',MCU-1+IAD,RST,CST)                      !syslin
            ENDIF
            IAD=IADLNX
            IADLNX=LENV(LIGNE)
            NBCOFL=IADLNX-IAD
            IF (NBCOFL.GT.0) THEN
               KOL1=LIGNE-NBCOFL
               CALL RSPRL  (NBCOFL,UENV(KOL1),'C',MCD-1+KOL1,MCU 
     &                     ,'C',MCAL-1+IAD,'C',MCL-1+IAD,RST,CST)       !syslin
            ENDIF
            CCOEFF=0.
            MCLL=MCL-1+IADLNX
            MCUU=MCU-1+IADUNX
            DO 40 J=1,MIN(NBCOFU,NBCOFL)
               MCLL=MCLL-1
               MCUU=MCUU-1
40             CCOEFF=CCOEFF+CST(MCLL)*CST(MCUU)
            CST(MCDD)=CST(MCADD)-CCOEFF
            IF (ABS(CST(MCDD)).LT.EPSIL) GOTO 99994
            MCDD=MCDD+1
41          MCADD=MCADD+1
      ENDIF
      CALL PRFXMJ (-1,'*LUPr*')
      RETURN
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
99991 WRITE(*,*) '        Coeff. diagonal de la ligne 1',RST(MCAD)
      IRCODE=1
      RETURN
99992 WRITE(*,*) '        Coeff. diagonal de la ligne ',LIGNE,RST(MCDD)
      IRCODE=LIGNE
      RETURN
99993 WRITE(*,*) '        Coeff. diagonal de la ligne 1',CST(MCAD)
      IRCODE=1
      RETURN
99994 WRITE(*,*) '        Coeff. diagonal de la ligne ',LIGNE,CST(MCDD)
      IRCODE=LIGNE
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
*-- File history
* Version 3 : D.Martin (2.avr.2010)
* - bug fixed in comment "Uii = Aii-Somme(k.le.i-1) Lik Uki"
* Version 2 : D.Martin (19 mars 2008)
* - ajout des messages d'erreurs
* Version 1 : D.Martin (23 avril 1998)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                                                                    END !LUPr
