*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE RSMAFA (NMSTOK,MCSTOK,TYFACT,TYMATR,MCMATR
     &                  ,NBSCMB,TYSCMB,MCSCMB,TYSOLU,MCSOLU
     &                  ,IST,RST,CST,IRCODE)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin (Fevrier 1999)  
* Derniere modification : D.Martin (27 Fevrier 1999)
* Version 1.0.0
*
* Resolution d'un systeme lineaire reel ou complexe dont la matrice est 
*   i) stockee sous forme 'BiMorse' et factorisee 
*      (L1.U, L1.D.U1, L.LT ou L.L*, ou L.D.LT ou L1.D.L1*)
*  ii) stockee sous forme 'Morse-L' et factorisee 
*      (L1.U, L1.D.U1) 
* iii) stockee sous forme 'Profil' et factorisee 
*      (L1.U, L1.D.U1, L.LT ou L.L*, ou L.D.LT ou L1.D.L1*) et
*
* ou
* - L  (resp. U ) est une matrice triang. inf. (resp. sup.)
* - L1 (resp. U1) est une matrice triang. inf. (resp. sup.) a diagonale unite
* - D  est une matrice diagonale
* - M* (resp. MT) represente l'adjointe (resp. la transposee) de la matrice M
*
*-- Arguments d'entree -- 
* NMSTOK nom du type de stockage de la matrice. 
* MCSTOK adresse du tableau de stockage de la matrice (le cas echeant).
* TYFACT type de factorisation de la matrice
* TYMATR type (reel ou complexe) de la matrice
* MCMATR adresse de la matrice dans le super-tableau ad hoc
* NBSCMB nombre de seconds membres
* TYSCMB type (reel ou complexe) du  vecteur second membre
* MCSCMB adresse du second membre
* TYSOLU type (reel ou complexe) du vecteur solution
* MCSCMB adresse du vecteur solution
* IST,RST,CST super-tableau considere entier, reel ou complexe
* IRCODE code de retour en cas d'erreur
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*) NMSTOK,TYFACT,TYMATR,TYSCMB,TYSOLU
      INTEGER       IST(*),MCSTOK,MCMATR,NBSCMB,MCSCMB,MCSOLU,IRCODE
      REAL          RST(*)
      COMPLEX       CST(*)
*
      INTEGER       NBLIGN,NBPLAL,MCSTAL,MCSTIL,LGL,MCL
     &             ,NBCOLO,NBPLAU,MCSTAU,MCSTIU,LGU
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*RsMafa*')
*
      IRCODE=0
      IF (NMSTOK(1:4).EQ.'&BMO') THEN
*
         CALL EXBMOR (MCSTOK,IST(MCSTOK),NBLIGN,NBPLAL,MCSTAL,MCSTIL,LGL
     &               ,NBCOLO,NBPLAU,MCSTAU,MCSTIU,LGU)                  !sdexplo
         MCL=MCMATR+NBLIGN
         IF (TYFACT(1:3).EQ.'L.U') THEN
*
*           Matrice factorisee L1.U
*
*           Systeme triangulaire inferieur a diagonale unite L1.x=b
            CALL RSBML1 (NBLIGN,IST(MCSTAL),IST(MCSTIL),TYMATR 
     &                  ,MCL,TYSCMB,MCSCMB,TYSOLU,MCSOLU,RST,CST)
*           Systeme triangulaire superieur U.x=b
            MCL=MCL+LGL
            CALL RSBMU  (NBLIGN,IST(MCSTAU),IST(MCSTIU),TYMATR,MCMATR    
     &                  ,MCL,TYSOLU,MCSOLU,TYSOLU,MCSOLU,RST,CST)
         ELSEIF (TYFACT(1:5).EQ.'L.D.U') THEN
*
*           Matrice factorisee L1.D.U1
*
*           Systeme triangulaire inferieur a diagonale unite L1.x=b
            CALL RSBML1 (NBLIGN,IST(MCSTAL),IST(MCSTIL),TYMATR           
     &                  ,MCL,TYSCMB,MCSCMB,TYSOLU,MCSOLU,RST,CST)
*           Systeme diagonal D.x=b
            CALL RSD    (NBLIGN,TYMATR,MCMATR                            
     &                  ,TYSOLU,MCSOLU,TYSOLU,MCSOLU,RST,CST)
*           Systeme triangulaire superieur a diagonale unite U1.x=b
            MCL=MCL+LGL
            CALL RSBMU1 (NBLIGN,IST(MCSTAU),IST(MCSTIU),TYMATR
     &                  ,MCL,TYSOLU,MCSOLU,TYSOLU,MCSOLU,RST,CST)
         ELSEIF (TYFACT(1:4).EQ.'L.LT') THEN
*
*           Matrice factorisee L.LT
*
*           Systeme triangulaire inferieur L.x=b
            CALL RSBML  (NBLIGN,IST(MCSTAL),IST(MCSTIL),TYMATR,MCMATR    
     &                  ,MCL,TYSCMB,MCSCMB,TYSOLU,MCSOLU,RST,CST)
*           Systeme triangulaire superieur U.x=b
            CALL RSBMU  (NBLIGN,IST(MCSTAL),IST(MCSTIL),TYMATR,MCMATR
     &                  ,MCL,TYSOLU,MCSOLU,TYSOLU,MCSOLU,RST,CST)
         ELSEIF (TYFACT(1:6).EQ.'L.D.LT') THEN
*
*           Matrice factorisee L1.D.L1T
*
*           Systeme triangulaire inferieur a diagonale unite L1.x=b
            CALL RSBML1 (NBLIGN,IST(MCSTAL),IST(MCSTIL),TYMATR
     &                  ,MCL,TYSCMB,MCSCMB,TYSOLU,MCSOLU,RST,CST)
*           Systeme diagonal D.x=b
            CALL RSD    (NBLIGN,TYMATR,MCMATR
     &                  ,TYSOLU,MCSOLU,TYSOLU,MCSOLU,RST,CST)
*           Systeme triangulaire superieur a diagonale unite L1T.x=b
            CALL RSBMU1 (NBLIGN,IST(MCSTAL),IST(MCSTIL),TYMATR
     &                  ,MCL,TYSOLU,MCSOLU,TYSOLU,MCSOLU,RST,CST)
         ELSEIF (TYFACT(1:4).EQ.'L.L*') THEN
*
*           Matrice factorisee L.L*
*
*           Systeme triangulaire inferieur L.x=b
            CALL RSBML  (NBLIGN,IST(MCSTAL),IST(MCSTIL),TYMATR,MCMATR
     &                  ,MCL,TYSCMB,MCSCMB,TYSOLU,MCSOLU,RST,CST)
*           Systeme triangulaire superieur L*.x=b
            CALL RSBMAL (NBLIGN,IST(MCSTAL),IST(MCSTIL),TYMATR,MCMATR
     &                  ,MCL,TYSOLU,MCSOLU,TYSOLU,MCSOLU,RST,CST)
         ELSEIF (TYFACT(1:6).EQ.'L.D.L*') THEN
*
*           Matrice factorisee L1.D.L1*
*
*           Systeme triangulaire inferieur L1.x=b
            CALL RSBML1 (NBLIGN,IST(MCSTAL),IST(MCSTIL),TYMATR
     &                  ,MCL,TYSCMB,MCSCMB,TYSOLU,MCSOLU,RST,CST)
*           Systeme diagonal D.x=b
            CALL RSD    (NBLIGN,TYMATR,MCMATR
     &                  ,TYSOLU,MCSOLU,TYSOLU,MCSOLU,RST,CST)
*           Systeme triangulaire superieur L1*.x=b
            CALL RSBAL1 (NBLIGN,IST(MCSTAL),IST(MCSTIL),TYMATR
     &                  ,MCL,TYSOLU,MCSOLU,TYSOLU,MCSOLU,RST,CST)
         ELSE
            IRCODE=1
         ENDIF
*
      ELSEIF (NMSTOK(1:4).EQ.'&PLA') THEN
         CALL EXMORS (MCSTOK,IST(MCSTOK),NBLIGN,NBCOLO,NBPLAL,MCSTAL
     &               ,MCSTIL,LGL)                                       !sdexplo
         IF (TYFACT(1:3).EQ.'L.U') THEN
*
*           Matrice morse factorisee L1.U
*
*           Systeme triangulaire inferieur a diagonale unite L1.x=b
            CALL RSMOL1 (NBLIGN,IST(MCSTAL),IST(MCSTIL),TYMATR,MCMATR
     &                  ,TYSCMB,MCSCMB,TYSOLU,MCSOLU,RST,CST)
*           Resolution du systeme triangulaire superieur U.x=b
            CALL RSMOU  (NBLIGN,IST(MCSTAL),IST(MCSTIL),TYMATR,MCMATR
     &                  ,TYSOLU,MCSOLU,TYSOLU,MCSOLU,RST,CST)
         ELSE
            IRCODE=1
         ENDIF
*
      ELSEIF (NMSTOK(1:4).EQ.'&PRO') THEN
         CALL EXPROF (MCSTOK,IST(MCSTOK),NBLIGN,MCSTAL,LGL
     &                                  ,NBCOLO,MCSTAU,LGU)             !sdexplo
         MCL=MCMATR+NBLIGN
         IF (TYFACT(1:3).EQ.'L.U') THEN
*
*           Matrice factorisee L1.U
*
*           Systeme triangulaire inferieur a diagonale unite L1.x=b
            CALL RSPRL1 (NBLIGN,IST(MCSTAL),TYMATR
     &                  ,MCL,TYSCMB,MCSCMB,TYSOLU,MCSOLU,RST,CST)
*           Resolution du systeme triangulaire superieur U.x=b
            MCL=MCL+LGL
            CALL RSPRU (NBLIGN,IST(MCSTAU),TYMATR,MCMATR          
     &                 ,MCL,TYSOLU,MCSOLU,TYSOLU,MCSOLU,RST,CST)
         ELSEIF (TYFACT(1:5).EQ.'L.D.U') THEN
*
*           Matrice factorisee L1.D.U1
*
*           Systeme triangulaire inferieur a diagonale unite L1.x=b
            CALL RSPRL1 (NBLIGN,IST(MCSTAL),TYMATR
     &                  ,MCL,TYSCMB,MCSCMB,TYSOLU,MCSOLU,RST,CST)
*           Systeme diagonal D.x=b
            CALL RSD    (NBLIGN,TYMATR,MCMATR
     &                  ,TYSOLU,MCSOLU,TYSOLU,MCSOLU,RST,CST)
*           Systeme triangulaire superieur a diagonale unite U1.x=b
            MCL=MCL+LGL
            CALL RSPRU1 (NBLIGN,IST(MCSTAU),TYMATR
     &                  ,MCL,TYSOLU,MCSOLU,TYSOLU,MCSOLU,RST,CST) 
         ELSEIF (TYFACT(1:4).EQ.'L.LT') THEN
*
*           Matrice factorisee L.LT
*
*           Systeme triangulaire inferieur L.x=b
            CALL RSPRL  (NBLIGN,IST(MCSTAL),TYMATR,MCMATR
     &                  ,MCL,TYSCMB,MCSCMB,TYSOLU,MCSOLU,RST,CST)
*           Systeme triangulaire superieur tL.x=b
            CALL RSPRU  (NBLIGN,IST(MCSTAL),TYMATR,MCMATR
     &                  ,MCL,TYSOLU,MCSOLU,TYSOLU,MCSOLU,RST,CST)
         ELSEIF (TYFACT(1:6).EQ.'L.D.LT') THEN
*
*           Matrice factorisee L1.D.L1T
*
*           Systeme triangulaire inferieur a diagonale unite L1.x=b
            CALL RSPRL1 (NBLIGN,IST(MCSTAL),TYMATR
     &                  ,MCL,TYSCMB,MCSCMB,TYSOLU,MCSOLU,RST,CST)
*           Systeme diagonal D.x=b
            CALL RSD    (NBLIGN,TYMATR,MCMATR                
     &                  ,TYSOLU,MCSOLU,TYSOLU,MCSOLU,RST,CST)
*           Systeme triangulaire superieur a diag.unite tL1.x=b
            CALL RSPRU1 (NBLIGN,IST(MCSTAL),TYMATR                 
     &                  ,MCL,TYSOLU,MCSOLU,TYSOLU,MCSOLU,RST,CST)
         ELSEIF (TYFACT(1:4).EQ.'L.L*') THEN
*
*           Matrice factorisee L.L*
*
*           Systeme triangulaire inferieur L.x=b
            CALL RSPRL  (NBLIGN,IST(MCSTAL),TYMATR,MCMATR
     &                  ,MCL,TYSCMB,MCSCMB,TYSOLU,MCSOLU,RST,CST)
*           Systeme triangulaire superieur L*.x=b
            CALL RSPRAL (NBLIGN,IST(MCSTAL),TYMATR,MCMATR
     &                  ,MCL,TYSOLU,MCSOLU,TYSOLU,MCSOLU,RST,CST)
         ELSEIF (TYFACT(1:6).EQ.'L.D.L*') THEN
*
*           Matrice factorisee L1.D.L1*
*
*           Systeme triangulaire inferieur a diagonale unite L1.x=b
            CALL RSPRL1 (NBLIGN,IST(MCSTAL),TYMATR
     &                  ,MCL,TYSCMB,MCSCMB,TYSOLU,MCSOLU,RST,CST)
*           Systeme diagonal D.x=b
            CALL RSD    (NBLIGN,TYMATR,MCMATR
     &                  ,TYSOLU,MCSOLU,TYSOLU,MCSOLU,RST,CST)
*           Systeme triangulaire superieur a diag.unite L1*.x=b
            CALL RSPAL1 (NBLIGN,IST(MCSTAL),TYMATR
     &                  ,MCL,TYSOLU,MCSOLU,TYSOLU,MCSOLU,RST,CST)
         ELSE
            IRCODE=1
         ENDIF
*
      ENDIF
      CALL PRFXMJ (-1,'*RsMafa*')    
                                                                    END !RsMafa
