*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE GAULOB (NPOINS,POINTS,POIDS)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : L.Goudenege (3 avril 2009)
* Derniere modification : L.Goudenege & D.Martin (15 decembre 2009)
* Version 1
*
* Formule de Gauss-Lobatto a n points, exacte pour P_{2n-3}(0,1).
* n est strictement superieur a 2
*
*-- Variables d'entree --
* NPOINS nombre de points de la formule de quadrature complete (n)
*-- Variables en sortie --
* POINTS tableau des points de quadrature (voir ci-dessous)
* POIDS  tableau des poids du schema de quadrature
*
* ATTENTION : Programme sortant les points de Gauss-Lobatto sur [-1,1].
* De plus, il sort les points positifs ou nuls ranges dans l'ordre decroissant.
* Pour un appel NPOINS = n, la sortie contient (n-1)/2 points
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      INTEGER NPOINS
      DOUBLE PRECISION POINTS(*),POIDS(*)
*
      CHARACTER*3 ERRSTR
      INTEGER     NHALF,J,N1,K,ITERE,ITEMAX
      DOUBLE PRECISION ROOTNW,ROOTNN,PN,PN1,PN2,PPN,PPN1,PPN2,PPPN
      DOUBLE PRECISION PI,TOL,TWKM1X,TWOX
*
*     Poids et points de quadrature de Gauss-Lobatto sur [0,1]
*     On sort (n+1)/2 points (seuls les points strictement au dessous de 0.5).
*     -----------------------------------------------------------------
      ITERE = 0
      ITEMAX= 1000
      IF (NPOINS.GT.2) THEN
         PI = 4.*ATAN(1.D0) 
*        ZERO NUMERIQUE  = 1.110223024625157E-016
*        Jusqu'a 20, ça marche bien. Apres ca marche pas des fois.
*        K = 22,23,28,29,32,33,34,35,37,38,39,40,41,43,44,45,46,47,48,49,50
*        Autres Zeros Numeriques -->> 2.22044604925031308E-016
*                                     3.33066907387546962E-016
*                                     4.44089209850062616E-016
*                                     5.55111512312578270E-016
*        TOL = ZERO NUMERIQUE + 0.000000000000001*1D-16
         TOL = 1.110223024625158*1D-16
         IF (NPOINS.GT.21) THEN
            TOL=5.55111512312578270E-016
         ENDIF
         NHALF = ((NPOINS+1)/2)
         ROOTNW = 0D0
         ROOTNN = 0D0
         N1 = NPOINS-1
         DO 1 J = 1,NHALF-1
*           Point de depart pour l'algorithme de Newton
            ROOTNW = COS (PI*(J+0.5D0)/(NPOINS+0.5D0))
10          PN = 1
            PN1 = 0
            PN2 = 0
            TWOX = ROOTNW + ROOTNW
            TWKM1X = ROOTNW
              
*           Calcul de P_{n-3}, P_{n-2}, P_{n-1} par reccurence.
*           k*P_k(x) = (2*k-1)*x P_{k-1}(x) - (k-1) P_{k-2}(x) , k <= n+1
            DO 100 K = 1,N1
              PN2 = PN1
              PN1 = PN
              PN = ( TWKM1X * PN1 - (K-1.) * PN2 ) / K
              TWKM1X = TWKM1X + TWOX
100         CONTINUE
              
*           Calcul de (1-x^2)*P'_{n-1} and (1-x^2)*P'_{n-2} pour x_newto : 
*           (1-x^2) P'_k(x) = - k x P_k(x) + k P_{k-1}(x)
            PPN =      N1*(PN1 - ROOTNW*PN)
            PPN1 = (N1-1.)*(PN2 - ROOTNW*PN1)
*
*           Calcul de (1-x^2)^2*P''_{n-1} pour x=x_newto
*           (1-x^2)*P''_k(x) = (2-k) x P'_k(x) + k { P'_{k-1}(x) - P_k(x) }
            PPPN = ( (2.-N1)*ROOTNW*PPN + N1*PPN1)
     &             / (1.-ROOTNW*ROOTNW) - N1*PN
*           Update de l'itere de Newton
            ROOTNN = ROOTNW
            ROOTNW = ROOTNN - PPN / PPPN
*           On boucle tant que la precision n'est pas atteinte.  
            IF (ABS(ROOTNW - ROOTNN).GT.TOL) THEN
               IF (ITERE.LT.ITEMAX) THEN 
                  ITERE = ITERE +1
                  GOTO 10
               ELSE
                  WRITE(*,*)' Lors du calcul du point numero ',J
                  WRITE(*,*)' DIFFERENCE = ',ABS(ROOTNW - ROOTNN)
                  WRITE(*,*)' Boucle while infinie'
                  WRITE(*,*)' ->>> Break du while.'
*              CALL BAISE('Points de Gauss-Lobatto non calculables...')
               ENDIF              
            ENDIF
*           Pour les hauts degrés, il y a bug ici, car le premier point
*           est souvent négatif pour les degrés impairs, alors
*           que ce devrait tout simplement être le point 0.
            IF (ROOTNW.LT.0) ROOTNW = 0.D0
            POINTS(J) = ROOTNW
1        CONTINUE
      ELSE
         CALL ENCLER (NPOINS,ERRSTR)
         CALL BAISE  ('Calcul des points de Gauss-Lobatto '
     &              //'non prevue pour N = '//ERRSTR//' .')
      ENDIF
999   FORMAT(' ',A9,' ',1PD22.16)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* File history
* Version 1 : D.Martin & L.Goudenege (15 decembre 2009)
* - version incorporee dans Melina
* Version 0 : L.Goudenege (3 avril 2009)
* - cas general non limite en nombre de points (cf version C++)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      END                                                               !GauLob

