*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      INTEGER FUNCTION I1MACH(I)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Authors  fox, p. a., hall, a. d., schryer, n. l., (bell labs)
* Last update : D.Martin (Jan 7, 2004)
*
* Purpose : returns integer machine dependent constants description
**  * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*   these machine constant routines must be activated for
*   a particular environment.
**  * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
*  I1MACH can be used to obtain machine-dependent parameters
*  for the local machine environment.  it is a function
*  subroutine with one (input) argument, and can be called
*  as follows, for example
*                    k = I1MACH(i)
*  where i=1,...,16.  the (OUTPUT) value of k above is
*  determined by the (input) value of i.  the results for
*  various values of i are discussed below.
*
*  I/O unit numbers.
*    I1MACH( 1) = the standard input unit.
*    I1MACH( 2) = the standard OUTPUT unit.
*    I1MACH( 3) = the standard punch unit.
*    I1MACH( 4) = the standard error message unit.
*  Words.
*    I1MACH( 5) = the number of bits per integer storage unit.
*    I1MACH( 6) = the number of characters per integer storage unit.
*  Integers
*    assume integers are represented in the s-digit, base-a form
*               sign ( x(s-1)*a**(s-1) + ... + x(1)*a + x(0) )
*               where 0 .le. x(i) .lt. a for i=0,...,s-1.
*    I1MACH( 7) = a, the base.
*    I1MACH( 8) = s, the number of base-a digits.
*    I1MACH( 9) = a**s - 1, the largest magnitude.
*  Floating-point numbers
*    assume floating-point numbers are represented in the t-digit,
*    base-b form
*               sign (b**e)*( (x(1)/b) + ... + (x(t)/b**t) )
*               where 0 .le. x(i) .lt. b for i=1,...,t,
*               0 .lt. x(1), and emin .le. e .le. emax.
*    I1MACH(10) = b, the base.
*  Single-precision
*    I1MACH(11) = t, the number of base-b digits.
*    I1MACH(12) = emin, the smallest exponent e.
*    I1MACH(13) = emax, the largest exponent e.
*  Double-precision
*    I1MACH(14) = t, the number of base-b digits.
*    I1MACH(15) = emin, the smallest exponent e.
*    I1MACH(16) = emax, the largest exponent e.
*
*  References fox p.a., hall a.d., schryer n.l., *framework for a
*               portable library*, acm transactions on mathematical
*               software, vol. 4, no. 2, june 1978, pp. 177-188.
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      INTEGER I
*
      INTEGER IMACH(16)
      CHARACTER OSARCH*32,NAMEMA*32,F77CMP*16,AUTODP*3
      COMMON/OS   /OSARCH,NAMEMA,F77CMP,AUTODP
*-----------------------------------------------------------------------
*
      IMACH( 1) = 5
      IMACH( 2) = 6
      IMACH( 3) = 5
      IMACH( 4) = 6
*
      IF (OSARCH(1:3).EQ.'OSF') THEN
*
*        Machine constants for Dec_alpha (added by D.Martin, IRMAR, feb. 96)
*
         IMACH( 5) = 32
         IMACH( 6) = 4
         IMACH( 7) = 2
         IMACH( 8) = 31
         IMACH( 9) = 2147483647
         IMACH(10) = 2
         IMACH(11) = 24
         IMACH(12) = -125
         IMACH(13) = 128
         IMACH(14) = 53
         IMACH(15) = -1021
         IMACH(16) = 1024
*
      ELSEIF (OSARCH(1:3).EQ.'Sun'.OR.OSARCH(1:4).EQ.'IRIX') THEN
*
*        Machine constants for SunOS (added by E.Luneville, ENSTA, Jan. 1991,
*                                and corrected by D.Martin, IRMAR, Feb. 1993)
*                          and SGI   (added by O.DeBayser , ENSTA, Feb. 1996)
*
         IMACH( 5) = 32
         IMACH( 6) = 4
         IMACH( 7) = 2
         IMACH( 8) = 31
         IMACH( 9) = 2147483647
         IMACH(10) = 2
         IMACH(11) = 24
         IMACH(12) = -125
         IMACH(13) = 127
         IMACH(14) = 53
         IMACH(15) = -125
         IMACH(16) = 127
*
      ELSE
*
*        Default (or rather laziness, same as above)
*
         IMACH( 5) = 32
         IMACH( 6) = 4
         IMACH( 7) = 2
         IMACH( 8) = 31
         IMACH( 9) = 2147483647
         IMACH(10) = 2
         IMACH(11) = 24
         IMACH(12) = -125
         IMACH(13) = 127
         IMACH(14) = 53
         IMACH(15) = -125
         IMACH(16) = 127
*
      ENDIF
*
      I1MACH=IMACH(I)
      RETURN
      END
