*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      REAL FUNCTION R1MACH(I)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Authors  fox, p. a., hall, a. d., schryer, n. l., (bell labs)
* Last update : D.Martin (Jan 7, 2004)
*
* Purpose : returns single precision machine dependent constants description
**  * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*   these machine constant routines must be activated for
*   a particular environment.
**  * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*
*    R1MACH can be used to obtain machine-dependent parameters
*    for the local machine environment.  it is a function
*    subroutine with one (input) argument, and can be called
*    as follows, for example
*                      a = R1MACH(i)
*    where i=1,...,5.  the (output) value of a above is
*    determined by the (input) value of i.  the results for
*    various values of i are discussed below.
*
*  Single-precision machine constants
*    R1MACH(1) = b**(emin-1), the smallest positive magnitude.
*                (see I1mach(10), I1mach(12))
*    R1MACH(2) = b**emax*(1 - b**(-t)), the largest magnitude.
*                (see I1mach(10), I1mach(11), I1mach(13))
*    R1MACH(3) = b**(-t), the smallest relative spacing.
*    R1MACH(4) = b**(1-t), the largest relative spacing.
*    R1MACH(5) = log10(b)
*
*  Where possible, octal or hexadecimal constants have been used
*  to specify the constants exactly which has in some cases
*  required the use of equivalent integer arrays.
*
*  References  fox, p.a., hall, a.d., schryer, n.l., *framework for
*                a portable library*, acm transactions on mathe-
*                matical software, vol. 4, no. 2, june 1978,
*                pp. 177-188.
*  Routines called (none)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      INTEGER I
*
      INTEGER          SMALL,LARGE,RIGHT,DIVER
      DOUBLE PRECISION LOG10
      REAL             RMACH(5)
      EQUIVALENCE     (RMACH(1),SMALL),(RMACH(2),LARGE)
     &               ,(RMACH(3),RIGHT),(RMACH(4),DIVER)
      CHARACTER OSARCH*32,NAMEMA*32,F77CMP*16,AUTODP*3
      COMMON/OS   /OSARCH,NAMEMA,F77CMP,AUTODP

*-----------------------------------------------------------------------
*
      IF (OSARCH(1:3).EQ.'OSF') THEN
*
*        Machine constants for Dec_alpha (added by D.Martin, Feb. 1995)
         SMALL = 8388608
         LARGE = 2130706431
         RIGHT = 864026624
         DIVER = 872415232
         LOG10 =.3010300099849700928D0
*
      ELSEIF (OSARCH(1:4).EQ.'IRIX') THEN
*
*        Machine constants for SGI (added by O.DeBayser, ENSTA, Feb. 1996)
*
         SMALL = 8388608
         LARGE = 2139095039
         RIGHT = 864026624
         DIVER = 872415232
         LOG10 =.3010300099849700900D0
*
      ELSEIF (OSARCH(1:3).EQ.'Sun') THEN
*
*        Machine constants for Sun (added by D.Martin, ENSTA, Feb. 1993)
*
         SMALL = 8388608
         LARGE = 2139095039
         RIGHT = 864026624
         DIVER = 872415232
         LOG10 =.3010299956639811952
*
      ELSE
*
*        Machine constants by default
*
         SMALL = 8388608
         LARGE = 2139095039
         RIGHT = 864026624
         DIVER = 872415232
         LOG10 =.3010299956639811952D0
      ENDIF
      RMACH(5)=LOG10
*-----------------------------------------------------------------------
      R1MACH = RMACH(I)
      END

