*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE ITEINV (NMMATA,NVMATA,NMMATB,NVMATB,VALPRO
     &                  ,NMVECP,NVVECP,SHIFT,OPTION)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : F.Mahe (9 janvier 1992)
* Derniere modification D.Martin (7 Janvier 2004)
* Version 1.1
* 
* Methode d'iteration inverse pour determiner la plus petite valeur propre
* et le vecteur propre normalise dans le probleme generalise
*                              A V = Lambda B V
*
* Algorithme d'iteration inverse
*     Vk = (A-Shift*I)^{-1} Ukm1
*     Uk = Vk/tvmax(Vk)
*     a convergence: 
*     tvmax(Vk) = + grande val.propre en module de (A-Shift*I)^{-1}
*               = 1/(plus petite valeur propre en module de (A-Shift*I))
*               = 1/((valeur propre de A la + proche de Shift) - Shift)
*                      
*-- Arguments d'entree --
* NMMATA nom et niveau de la matrice [A] necessairement factorisee
* NVMATA son niveau
* NMMATB nom de la matrice [B]
* NVMATB son niveau
* NMVECP nom du vecteur initial en entree, du vecteur propre calcule en sortie
* NVVECP son niveau
* SHIFT  shift de selection de la valeur propre
* OPTION chaine de caracteres definissant les valeurs des parametres optionnels
*        Elle est soit blanche (' '), soit structuree sous la forme d'articles
*        de la forme <ITEM=ITEMVA>, separes par des virgules ou ITEM vaut
*        l'une des chaines
*               'Epsilon', 'Maximum_Iterations', 'Impression'
*        et ITEMVA est le nom d'une donnee declaree.
*        Les valeurs des donnees correspondent resectivment aux variables
*        - EPSIL  seuil de d'arret des iterations de l'algorithme (defaut 1.E-4)
*        - ITMAX  nombre maximum d'iterations (defaut ~Nblign/10)
*        - NIVIMP niveau d'impression des calculs (defaut 0)
*
*-- Arguments de sortie --
* VALPRO valeur propre recherchee
*
* Sont utilisees les constantes de nom: 'factor.1', 'factor.2'.
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      CHARACTER*(*)  NMMATA,NMMATB,NMVECP,OPTION
      INTEGER        NVMATA,NVMATB,NVVECP,NUIT
      REAL           VALPRO,SHIFT
*
      INCLUDE 'CONTEX'
*
      CHARACTER      PRFXAS*8,ERCODE*120,CARAC,TYPE*8
      REAL           EPSIL,KR,VALNI,RPSA,RPSM,RMAXA,RMAXM,UNUSED
      INTEGER        NIVIMP,ITMAX,NBITER,IADRR,IAD,IBID,NBLIGN
     &              ,NCATR1,NCATR2
      COMPLEX        KC
      COMMON/FORMAH/ERCODE
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*Iteinv*')
*
*--   Valeurs des parametres optionnels
      EPSIL = 1.E-4
      UNUSED= 0.
      ITMAX = 0
      NIVIMP= 0
      IF (OPTION.NE.' ') CALL VPOPTI (OPTION,EPSIL,UNUSED,ITMAX,NIVIMP)
*
      IF (NIVIMP.GE.0.AND.IMPPAL.GT.0) THEN
         CALL PRNTRM (NMMATA,NVMATA,ERCODE,NCATR1)
         CALL PRNTRM (NMMATB,NVMATB,ERCODE(NCATR1+1:),NCATR2)
         WRITE (IMPPAL,10000) PRFXAS (0),ERCODE(1:NCATR1)
     &                       ,ERCODE(NCATR1+1:NCATR1+NCATR2)
      ENDIF
*
      IF (ITMAX.LE.0) THEN
         CALL TBARTI (NMVECP,NVVECP,NBLIGN)
         ITMAX=MAX (10,1+NBLIGN/10)
      ENDIF
*
*     Normalisation initiale.
*
      CALL MXTERM (NMVECP,NVVECP,IADRR,KR,KC)
      CALL PUTCST ('factor.1','REEL',IBID,1./KR,KC,CARAC)
      CALL AXTERM (NMVECP,NVVECP,'factor.1','VVUKm1',1,-1)
*
*     Decomposition 'LDtL' de [A]
*
      CALL FALDLT (NMMATA,NVMATA,'APROFI',1,-1)
*
*     Recherche d'un vecteur propre, iteration.
*
      DO 100 NBITER=1,ITMAX
*
*        Calcul du second membre: [B]Ukm1.
*
         CALL MATVEC (NMMATB,NVMATB,'VVUKm1',1,'BxUKm1',1,-1)
*
*        Resolution de [A]Vk = [M]Uk-1
*
         CALL FASV   ('APROFI',1,'BxUKm1',1,NMVECP,NVVECP,-1)
         CALL GETERM (NMVECP,NVVECP,IADRR,KR,KC)
         IF (ABS (KR).LT.1E-20) WRITE (*,10011) PRFXAS (0),KR,NBITER
*        si cela pose pb. dans la suite il suffit de changer de vect. initial
         VALPRO=1./KR
*
*        Test d'arret : nmax[Ukm1/tvmax(Vk)-ValPro*Vk/tvmax(Vk)] < EPSIL
*
*        car a la convergence Vk=Ukm1/ValPro et tvmax(Vk) est un facteur
*        de normalisation du test.
*
         CALL MXTERM (NMVECP,NVVECP,IADRR,KR,KC)
         CALL PUTCST ('factor.1','REEL',IBID,1./KR,KC,CARAC)
         CALL PUTCST ('factor.2','REEL',IBID,-VALPRO/KR,KC,CARAC)
         CALL CLTERM ('VVUKm1',1,'factor.1',NMVECP,NVVECP
     &                ,'factor.2','VVUKm1',1,-1)
         CALL MXTERM ('VVUKm1',1,IBID,VALNI,KC)
         VALNI=ABS (VALNI)
*
*        Normalisation pour iteration suivante
*
         CALL AXTERM (NMVECP,NVVECP,'factor.1','VVUKm1',1,-1)
*
         IF (VALNI.GT.EPSIL) THEN
*
            IF (NIVIMP.GE.6.AND.IMPPAL.GT.0) 
     &         WRITE (IMPPAL,10006) NBITER,VALNI,VALPRO+SHIFT
*
         ELSE
*
*            Calcul de la valeur propre par quotient de Rayleigh
*
*            ValPro = ([A]*VecPro, VecPro) / ([B]*VecPro, VecPro)
*
             CALL MATVEC (NMMATA,NVMATA,'VVUKm1',1,'AVecPr',1,-1)
             CALL MATVEC (NMMATB,NVMATB,'VVUKm1',1,'BVecPr',1,-1)
             CALL XSTERM ('S','AVecPr',1,'VVUKm1',1,TYPE,RPSA,KC)
             CALL XSTERM ('S','BVecPr',1,'VVUKm1',1,TYPE,RPSM,KC)
*
             VALPRO=RPSA/RPSM
*
             IF (NIVIMP.GE.4.AND.IMPPAL.GT.0) 
     &          WRITE (IMPPAL,10004) VALPRO,VALPRO+SHIFT
*
*            Test en norme 'max' de la convergence (verification):
*
*               max | [A]*VecPro - ValPro*[M]*VecPro | < EPSIL
*
             CALL PUTCST ('factor.2','REEL',IBID,-VALPRO,KC,CARAC)
             CALL CLTERM ('AVecPr',1,' ','BVecPr',1,'factor.2'
     &                   ,NMVECP,NVVECP,-1)
             CALL MXTERM (NMVECP,NVVECP,IAD,VALNI,KC)
             VALNI=ABS (VALNI)
             IF (VALNI.LE.EPSIL) GOTO 200
*
*           Impression de max(|A|) et max(|M|) pour verification
*
            IF (NIVIMP.GE.4.AND.IMPPAL.GT.0) WRITE (IMPPAL,10005) VALNI
         ENDIF
*
100   CONTINUE
*
*     Affectation des parametres de sorties
*
200   CALL T1TERM ('VVUKm1',1,'=',NMVECP,NVVECP,-1)
      VALPRO=VALPRO+SHIFT
      IF (NIVIMP.GE.1.AND.IMPPAL.GT.0)
     &   WRITE (IMPPAL,10010) NBITER,VALNI,VALPRO
*
      CALL PRFXMJ (-1,'*IteInv*')
*-----------------------------------------------------------------------
10000 FORMAT (/T2,A8,' Element propre (Lambda,V) de AV=Lambda BV ou A='
     &,A,' et B=',A)
10004 FORMAT (T11,' ValPro-Shift =',E14.8,' ValPro = ',E14.8)
10005 FORMAT (T11,'max |[A]*(VecPro) - ValPro*[M]*(VecPro)| = ',E14.8)
10006 FORMAT (T11,'Iter #',I3,' Valni=',E14.8,' Valpro=',E14.8)
10011 FORMAT (/T2,A8,' Warning ValPro=(',E14.8,') a '
     &,' l''iteration ',I4)
10010 FORMAT(T11,'-> Nombre d''iterations ',I6,' , Residu =',E10.4
     &      /T11,'-> Lambda =',(T22,5E11.5,:/))
                                                                    END !IteInv
