*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE ORTOGM (N,P,TYPE,PDEB,PFIN,MCX,MCMX,MCQ,MCR
     &                  ,TYMATR,MCMATR,ITYSYM,MCSTOK,NMSTOK
     &                  ,NBMAVC,IST,RST,CST)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : C. Hazard (Fevrier 1989), D.Martin (Mai 1999)
* Derniere modification : D.Martin (8 Juin 1999)
*
* Orthonormalisation des colonnes d'une matrice avec ou sans construction de
* la transformation ( algorithme "Twice Is Enough" )
* [ relativement au produit scalaire defini par la matrice definie positive M :
*   ||x||_M= SQRT (Mx,x) ]
*
* Ce sous-programme orthonormalise les colonnes PDEB a PFIN de la matrice 
* de vecteurs (propres) colonnes X, les colonnes 1 a PDEB-1 etant supposees 
* deja orthonormales. 
* Il construit la matrice R (triangulaire superieure PxP) de la transformation:
*    X = Q R    avec    Q* [M] Q = I
* celle-ci n'est retournee a l'adresse MCR seulement si MCR.gt.0,
* Il retourne la matrice Q orthogonale [relativement a M]
* Il utilise la methode Gram-Schmidt dans un algorithme "twice is enough": 
* l'orthogonalisation est repetee lorsqu'une colonne de X voit sa norme reduite
* d'un facteur superieur a COEF lors de la premiere orthogonalisation 
* (ici COEF=10). 
* Attention, l'algorithme ne permet pas necessairement de detecter une colonne 
* qui serait "presque" une combinaison lineaire des autres (voir la reference
* pour les conditions assurees par l'algorithme).
*
* Bibliographie : Parlett, The Symmetric Eigenvalue Problem, 
*                 Prentice-Hall, 1980, chap. 6, page 98
*-- Arguments --
* N     nombre de lignes de la matrice des vecteurs colonnes X
* P     nombre de colonnes de la matrice des vecteurs colonnes X
* TYPE  type ('REEL' ou 'COMPLEXE') des vecteurs
* PDEB  premiere colonne de la matrice a orthonormaliser
* PFIN  derniere colonne de la matrice a orthonormaliser
*       = nombre de colonnes de la matrice X
*       = nombre de lignes de la matrice carree triangulaire superieure R
* MCX   adresse de la matrice X de dimension (N,P)
*       les colonnes du bloc (1:N,1:PFIN) de la matrice X donnees en entree
*       sont othonormalisees, et le resultat est range dans le bloc analogue
*       de Q en sortie.
*       Seules les colonnes PDEB a PFIN sont susceptibles d'etre modifiees,
*       les colonnes 1 a PDEB-1 etant supposees orthonormees en entree.
* MCMX  adresse du tableau pour le calcul des vecteurs M*X
*       (DOIT etre egal a MCX si MCMATR=0)
* MCQ   adresse de la matrice Q (coincide avec X si MCQ=MCX).
* MCR   adresse de la matrice R
*       en sortie le bloc (1:P,PDEB:PFIN) de la matrice R contient la
*       transformation produite par l'orthonormalisation (matrice triangulaire
*       superieure). 
*       Si lors de la seconde orthogonalisation, une colonne de X voit sa 
*       norme reduite a nouveau d'un facteur superieur a COEF, l'element
*       diagonal de R correspondant et la colonne de X sont mis a 0.
* Parametres de definition de la matrice M
* TYMATR type ('REEL' ou 'COMPLEXE') de la matrice B (le cas echeant).
* MCMATR adresse de la matrice B (le cas echeant).
* ITYSYM indice de type de symetrie de la matrice B (le cas echeant).
* MCSTOK adresse des tableaux de pointeurs de stockage de B(le cas echeant)
* NMSTOK nom du tableau (et type) de stockage de B (le cas echeant).
* NBMAVC nombre de multiplications matrice vecteur
* IST,RST,CST super-tableau version entiere, reelle ou complexe
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) TYPE,TYMATR,NMSTOK
      INTEGER       N,P,PDEB,PFIN,MCX,MCMX,MCQ,MCR,MCMATR,ITYSYM,MCSTOK
     &             ,NBMAVC,IST(*)
      REAL          RST(*)
      COMPLEX       CST(*)
*
      INTEGER       MCRDEB,MCRK,MCR1,MCQDEB,MCQK,MCQI,MCMXDB,MCMXI,MCMXK
     &             ,NORTHO,K,I
      REAL          S,T,COEF2
      COMPLEX       C
*-----------------------------------------------------------------------
      COEF2=100.
*
      IF (MCMATR.LE.0.AND.MCX.NE.MCMX) THEN
         CALL BAISE  ('*Ortogm* Pas de matrice M et MCX.NE.MCMX !')
      ENDIF
      IF (MCQ.NE.MCX) 
     &   CALL TDANST (N*P,TYPE,MCX,IST,RST,CST,TYPE,MCQ,IST,RST,CST)
*
      MCRDEB=MCR +(PDEB-1)*P
      MCQDEB=MCQ +(PDEB-1)*N
      MCMXDB=MCMX+(PDEB-1)*N
*     adresse du 1er coeff. de la colonne PDEB de la matrice triang. sup. R
*     et la matrice orthogonale Q
      MCRK=MCRDEB
      MCQK=MCQDEB
      MCMXK=MCMXDB
      DO 10 K=PDEB,PFIN
         MCR1=MCRK
*        numero de l'orthogonalisation en cours
         NORTHO=0
*        orthogonalisation de la colonne K / colonnes precedentes
1        NORTHO=NORTHO+1
         T=0.
         MCMXI=MCMX
         MCQI =MCQ
         DO 3 I=1,K-1
*           produit scalaire des colonnes I et K
*           [ le cas echeant produit scalaire defini par M ]
            CALL TSCALT (N,TYPE,MCMXI,RST,CST,TYPE,MCQK,RST,CST,C)
            S=REAL(C)
            T=T+S*S
*           mise a jour de la colonne K de X
            CALL TPLUAT (N,TYPE,MCQK,RST,CST,'REEL',1,IST,-S,CST
     &                    ,TYPE,MCQI,RST,CST,TYPE,MCQK,RST,CST)
            MCMXI=MCMXI+N
            MCQI =MCQI+N
*           remplissage de la matrice R
            IF (MCR.GT.0.AND.NORTHO.EQ.1) THEN
               RST(MCR1)=S
               MCR1=MCR1+1
            ENDIF
3        CONTINUE
*        carre de la norme de la nouvelle colonne K
*        [ le cas echeant relativement au produit scalaire defini par M ]
         IF (MCMATR.GT.0) THEN
            CALL MATMPC (N,N,NMSTOK,MCSTOK,ITYSYM,TYMATR,MCMATR
     &                  ,1,TYPE,MCQK,TYPE,MCMXK,IST,RST,CST)
            NBMAVC=NBMAVC+1
         ENDIF
         CALL TSCALT (N,TYPE,MCMXK,RST,CST,TYPE,MCQK,RST,CST,C)
         S=REAL(C)
*        carre de la norme de l'ancienne colonne K
         T=S+T
         IF (S.LE.T/COEF2) THEN
            IF (NORTHO.EQ.1) GOTO 1
            IF (NORTHO.EQ.2) S=0.
         ENDIF
*        normalisation
         S=SQRT(S)
         IF (MCR.GT.0) RST(MCRK+K-1)=S
         IF (S.NE.0.) S=1./S
         CALL TEQAT  (N,TYPE,MCQK,RST,CST,'REEL',1,IST,S,CST
     &                 ,TYPE,MCQK,RST,CST)
         IF (MCMATR.GT.0)
     &   CALL TEQAT  (N,TYPE,MCMXK,RST,CST,'REEL',1,IST,S,CST
     &                 ,TYPE,MCMXK,RST,CST)
         MCRK=MCRK+P
         MCQK=MCQK+N
         MCMXK=MCMXK+N
10    CONTINUE
      END
