*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE QLIMRS (N,D,E,TR,EPSMAC,IERR)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : C.Hazard (Juin 1989), F.Mahe, D.Martin (Janvier 1997)
* Derniere modification : D.Martin (16 Avril 1999)
*
* Calcul de toutes les valeurs propres et tous les vecteurs propres d'une 
* matrice reelle symetrique tridiagonale.
*                            Methode "QL implicite".
* ==> tire de EISPACK (subroutine ImtQl2)
* ==> reference: Wilkinson & Reinsch (Linear Algebra, 1971, Contrib. II/4)
*
* Ce sous-programme est principalement destine a etre utilise apres TriDrs qui 
* reduit une matrice reelle pleine symetrique sous forme tridiagonale 
* et fournit la transformation correspondante (matrice TR).
*-- Arguments --
* N       dimension de la matrice tridiagonale a traiter
*         (definit la dimension utile des tableau D, E et TR).
* D(*)  - en entree, D contient la diagonale de la matrice tridiagonale
*       - en sortie, D contient les valeurs propres de cette matrice 
*         rangees par ordre de valeurs absolues decroissantes.
* E(*)    en entree, E(2:N) contient la sous-(ou sur-)diagonale de la matrice
*         tridiagonale.
* TR(N,*) pour obtenir les vecteurs propres de la matrice tridiagonale,
*         TR doit contenir en entree la matrice identite; 
*         pour obtenir les vecteurs propres de la matrice pleine,
*         TR(1:N,1:N) doit contenir en entree la matrice de la transformation
*         orthogonale contruite par la procedure TriDrs; 
*       - en sortie, il contiendra les vecteurs propres normes de la matrice.
* EPSMAC  ("epsilon machine") est l'inf des reels r tels que 1+r>r
*         il depend de la machine utilisee.
* IERR    si plus de 30 iterations sont necessaires pour determiner une 
*         valeur propre, le calcul est interrompu et IERR porte le numero
*         de cette valeur propre. Les valeurs propres et vecteurs propres
*         d'indices 1 a IERR-1 sont corrects mais les valeurs propres ne 
*         sont pas reordonnees. Si toutes les valeurs propres sont calculees
*         en moins de 30 iterations, IERR est egal a 0 en sortie.
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      INTEGER N,IERR
      REAL D(*),E(*),TR(N,*),EPSMAC
*
      INTEGER I,J,K,L,M,II
      REAL B,C,F,G,P,R,S
*-----------------------------------------------------------------------
      IERR=0
      IF (N.EQ.1) RETURN
      DO 100 I=2,N
         E(I-1)=E(I)
100   CONTINUE
      E(N)=0.
*
*     algorithme QL
*
      DO 240 L=1,N
         J=0
*        recherche d'un element sub-diagonal negligeable
105      DO 110 M=L,N
            IF (M.EQ.N) GOTO 120
            IF (ABS(E(M)).LE.EPSMAC*(ABS(D(M))+ABS(D(M+1)))) GOTO 120
110      CONTINUE
120      P=D(L)
         IF (M.EQ.L) GOTO 240
         IF (J.EQ.30) THEN
*           sortie sans convergence (apres 30 iterations)
            IERR=L
            RETURN
         ENDIF
         J=J+1
*        construction du shift 
         G=(D(L+1)-P)/(2.*E(L))
         R=SQRT(G*G+1.)
         G=D(M)-P+E(L)/(G+SIGN(R,G))
         S=1.
         C=1.
         P=0.
*        iteration elementaire
         DO 200 II=1,M-L
            I=M-II
            F=S*E(I)
            B=C*E(I)
            IF (ABS(F).LT.ABS(G)) THEN
               S=F/G
               R=SQRT(S*S+1.)
               E(I+1)=G*R
               C=1./R
               S=C*S
            ELSE
               C=G/F
               R=SQRT(C*C+1.)
               E(I+1)=F*R
               S=1./R
               C=C*S
            ENDIF
            G=D(I+1)-P
            R=(D(I)-G)*S+2.*C*B
            P=S*R
            D(I+1)=G+P
            G=C*R-B
*
            DO 180 K=1,N
               F=TR(K,I+1)
               TR(K,I+1)=S*TR(K,I)+C*F
               TR(K,I)  =C*TR(K,I)-S*F
180         CONTINUE
200      CONTINUE
         D(L)=D(L)-P
         E(L)=G
         E(M)=0.
         GOTO 105
240   CONTINUE
*     fin de l'algorithme QL
*
*     Reordonner les valeurs et vecteurs propres
*
      DO 300 II=2,N
         I=II-1
         K=I
         P=D(I)
         DO 260 J=II,N
            IF (ABS(D(J)).GT.ABS(P)) THEN
              K=J
              P=D(J)
            ENDIF
260      CONTINUE
         IF (K.NE.I) THEN
            D(K)=D(I)
            D(I)=P
            DO 280 J=1,N
               P      =TR(J,I)
               TR(J,I)=TR(J,K)
               TR(J,K)=P
280         CONTINUE
         ENDIF
300   CONTINUE
      END
