*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE TRIDRS (N,A,D,E,TR)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : C.Hazard (Juin 1989), F.Mahe, D.Martin (Janvier 1997)
* Derniere modification : D.Martin (25 Mai 1999)
*
* Reduction d'une matrice pleine reelle symetrique a la forme tridiagonale 
* (avec accumulation de la transformation)     
*
* ==> tire de EISPACK (subroutine TreD2)
* ==> reference : Wilkinson & Reinsch (Linear Algebra, 1971, contrib. II/2)
*
* Cette reduction s'effectue a l'aide de transformations unitaires 
* (methode de Householder). Ce sous-programme est principalement destine 
* a etre utilise avant le sous-programme QlImrs ou tout autre sous-programme 
* calculant TOUTES les valeurs propres et TOUS les vecteurs propres 
* d'une matrice tridiagonale reelle symetrique.
*
*-- Arguments --
* N      nombre de lignes/colonnes de la matrice A.
* A(N,*) seule la partie triangulaire inferieure de A est utilisee.
* D(*)   en sortie, D contient la diagonale de la matrice tridiagonale.
* E(*)   en sortie, E(2:N) contient la sous-diagonale de la matrice 
*        tridiagonale (E(1) est toujours nul).
* TR(N,*) en sortie, TR est la matrice de la transformation orthogonale 
*        produite dans la reduction de A sous forme tridiagonale.
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      INTEGER N
      REAL    A(N,*),D(*),E(*),TR(N,*)
*
      INTEGER I,J,K,IM1
      REAL F,G,H,HH,SCALE,TRIK
*-----------------------------------------------------------------------
*     copie de A dans TR
      DO 1 I=1,N
      DO 1 J=1,I
         TR(I,J)=A(I,J)
1     CONTINUE
*
      DO 300 I=N,3,-1
         IM1=I-1
*
         SCALE=0.
         DO 120 J=1,IM1
            SCALE=SCALE+ABS(TR(I,J))
120      CONTINUE
*
         IF (SCALE.NE.0.) THEN
            H=0.
            DO 150 K=1,IM1
               TRIK=TR(I,K)/SCALE
               TR(I,K)=TRIK
               H=H+TRIK*TRIK
150         CONTINUE
            F= TR(I,IM1)
            G=-SIGN(SQRT(H),F)
            E(I)=SCALE*G
            H=H-F*G
            TR(I,IM1)=F-G
*
            F=0.
            DO 240 J=1,IM1
               TR(J,I)=TR(I,J)/H
               G=0.
               DO 180 K=1,J
                  G=G+TR(J,K)*TR(I,K)
180            CONTINUE
               DO 200 K=J+1,IM1
                  G=G+TR(K,J)*TR(I,K)
200            CONTINUE
               E(J)=G/H
               F=F+E(J)*TR(I,J)
240         CONTINUE
*
            HH=F/(H+H)
            DO 260 J=1,IM1
               F=TR(I,J)
               G=E(J)-HH*F
               E(J)=G
               DO 260 K=1,J
                  TR(J,K)=TR(J,K)-F*E(K)-G*TR(I,K)
260         CONTINUE
            D(I)=H
         ELSE 
            E(I)=0.
            D(I)=0.
         ENDIF
*
300   CONTINUE
*
      IF (N.GT.1) THEN
        D(2)=0.
        E(2)=TR(2,1)
      ENDIF
      D(1)=0.
      E(1)=0.
*
*     accumulation de la transformation
*
      DO 500 I=1,N
         IM1=I-1
         IF (D(I).NE.0.) THEN
            DO 370 J=1,IM1
               G=0.
               DO 340 K=1,IM1
                  G=G+TR(I,K)*TR(K,J)
340            CONTINUE
               DO 360 K=1,IM1
                  TR(K,J)=TR(K,J)-G*TR(K,I)
360            CONTINUE
370         CONTINUE
         ENDIF
         D(I)=TR(I,I)
         TR(I,I)=1.
         DO 400 J=1,IM1
            TR(I,J)=0.
            TR(J,I)=0.
400      CONTINUE
500   CONTINUE
*
      END
