*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE VP1CHB (P,NBVPCA,P2,VP,NCONV,CCONV,NPARA
     &                  ,EPSIL,PRECIS,OLDPRE,IERR,NIVIMP,IMPFCH)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : C.Hazard (Juin 1989), F.Mahe, D.Martin (Fevrier 1997)
* Derniere modification : D.Martin  (3 Mai 1999)
*
* Calculs preliminaires a la boucle d'acceleration de Chebyshev (VpMisr).
* Estimation de nombre d'iterations. 
*
*-- Arguments --
* P      dimension du sous-espace de travail (tableau VP)
* NBVPCA nombre de valeurs propres deja calculees
* P2     les vecteurs de numero NBVPCA+1 a P2 forment le prochain groupe
*        de vecteurs propres qui doit converger
* VP     tableau contenant les valeurs propres
* NCONV  nombre d'iterations dans la boucle de Chebyshev
* CCONV  coefficient de controle de la convergence, ajuste en cours d'execution 
*        en comparant la vitesse de convergence constatee avec la vitesse de 
*        convergence theorique (qui depend de la valeur propre suivant VP(P)
*        (non calculee) dont VP(P)/CCONV est une approximation grossiere)
* NPARA  estimation du nombre maximum d'iterations entre 2 orthonormalisations.
* EPSIL  parametre du test d'acceptation des vecteurs propres. Le test impose 
*        que le Sinus de l'angle entre un groupe de vecteurs propres calcule 
*        et le sous-espace invariant correspondant est inferieur a EPSIL.
* PRECIS precision utilisee pour determiner le prochain groupe de
*        vecteurs propres qui doit converger. Initialise a .2
* OLDPRE derniere estimation d'erreur calculee, ancien PRECIS.
* IERR   parametre de controle de l'execution de l'algorithme (voir VpMisr).
* IMPFCH numero d'unite logique du fichier d'impression
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      INTEGER P,NBVPCA,P2,NCONV,NPARA,IERR,NIVIMP,IMPFCH
      REAL    VP(*),EPSIL,PRECIS,OLDPRE,CCONV
*
      INTEGER K
      REAL    SOM,DIST
*-----------------------------------------------------------------------
*     Estimation du nombre d'iterations maximum entre 2 orthonormalisations
      SOM  =ABS (VP(NBVPCA+1)/VP(P))
      SOM  =LOG (SOM+SQRT(SOM*SOM-1.))
      NPARA=MIN(2*INT (1.5/SOM)+2,100)
*
*     ajustement de la valeur de CCONV
      IF (OLDPRE.NE.0..AND.PRECIS.LT..2) THEN
         CCONV=VP(P)/VP(P2)*EXP( LOG (OLDPRE/PRECIS)/(NCONV+1))
         CCONV=MIN (MAX (1.,CCONV),10.)
      ENDIF
*
*     estimation du nombre d'iterations necessaires pour obtenir
*     la convergence du prochain groupe de vecteurs
      SOM = ABS(VP(P2)/VP(P)*CCONV)
      IF (PRECIS.GE..2) THEN
*        si l'angle estime entre le groupe NBVPCA+1 -- P2 et le sous-espace
*        invariant correspondant est .gt. 10 degres, le calcul de NCONV
*        est considere comme non significatif, on fixe NCONV=4
         IF (NIVIMP.GE.1.AND.IMPFCH.GT.0) WRITE (IMPFCH,10002)
         NCONV=4
      ELSE
         IF (NIVIMP.GE.1.AND.IMPFCH.GT.0)
     &   WRITE (IMPFCH,10001) NBVPCA+1,P2,ASIN(PRECIS)/3.1416*180.
     &                       ,SOM,CCONV
         SOM =LOG (SOM+SQRT(SOM*SOM-1.))
         DIST=COSH (NPARA*SOM)
         K   =INT (LOG (PRECIS/EPSIL)/LOG (DIST))
         IF (K.NE.0) THEN
            DIST=PRECIS/EPSIL/(DIST**K)
         ELSE
            DIST=PRECIS/EPSIL
         ENDIF
         DIST =LOG (DIST+SQRT(DIST*DIST-1.))
         NCONV=K*NPARA+INT (DIST/SOM)
      ENDIF
*-----------------------------------------------------------------------
10001 FORMAT (T6,'Estimation du prochain groupe a converger : '
     &,I2,' a ',I2
     &,/,T6,'angle estime avec le sous-espace invariant : ',E10.4,
     &  ' (degres)'
     &,/,T6,'facteur de convergence pour une iteration  : ',E10.4
     &,/,T6,'coefficient de controle du fact. de conv.  : ',E10.4)
10002 FORMAT (T6,'Erreur calculee non significative ...')
      END
