*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE VP2CHB (CHOIX,N,NBUVEC,CHEB1,CHEB2,NPARA
     &                  ,TYMATA,MCMATA,ITYSYA,MCSTOA,NMSTOA
     &                  ,TYMATB,MCMATB,ITYSYB,MCSTOB,NMSTOB
     &                  ,TYVEC,MCX,MCY,MCZ,MCZAUX
     &                  ,NBOPEL,NBITER,NBMAVC,NBRESO,IST,RST,CST)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : C.Hazard (Juin 1989) & D.Martin (Juin 1999)
* Derniere modification : D.Martin (10 Juin 1999)
*
* Acceleration de Chebyschev de la methode d'iteration de sous-espace
*
*-- Arguments --
* CHOIX  parametre de choix de l'operation elementaire (cf VpMISE)
* N      definit la dimension du probleme
* NBVECT nombre de vecteurs operandes/resultats
* CHEB1  coefficient du developpement des polynomes de Chebyschev
* CHEB2  coefficient du developpement des polynomes de Chebyschev
* NPARA  estimation du nombre maximum d'iterations entre 2 orthonormalisations
* TYMATA type (relle ou complexe) de la matrice A.
* MCMATA adresse de la matrice A.
* ITYSYA indice de type de symetrie de la matrice A.
* MCSTOA adresse des tableaux de pointeurs de stockage de la matrice A
* NMSTOA nom du tableau (et type) de stockage de A.
* TYMATB type (relle ou complexe) de la matrice B (le cas echeant).
* MCMATB adresse de la matrice B (le cas echeant).
* ITYSYB indice de type de symetrie de la matrice B (le cas echeant).
* MCSTOB adresse des tableaux de pointeurs de stockage de B(le cas echeant)
* NMSTOB nom du tableau (et type) de stockage de B (le cas echeant).
* TYVEC  type de la matrice des vecteur colonnes X, Y, Z.
* MCX    adresse des vecteurs X operandes de l'operation elementaire
* MCY    adresse des vecteurs Y resultats de l'operation elementaire
* MCZ    adresse des vecteurs Z resultats de l'operation elementaire
* MCZAUX adresse des vecteurs auxiliaires de travail 
* NBITER numero de l'iteration courante
* NBOPEL nombre d'operations elementaires
* NBMAVC cumul du nombre de multiplications matrice x vecteur
* NBRESO cumul du nombre de resolutions de systeme lineaire
* IST,RST,CST super-tableau version entiere, reelle ou complexe
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) CHOIX,NMSTOA,NMSTOB,TYMATA,TYMATB,TYVEC
      INTEGER       N,NBUVEC,NPARA,MCX,MCY,MCZ,MCZAUX
     &             ,MCMATA,MCSTOA,ITYSYA,MCMATB,MCSTOB,ITYSYB
     &             ,NBOPEL,NBITER,NBMAVC,NBRESO
     &             ,IST(*)
      REAL          CHEB1,CHEB2,RST(*)
      COMPLEX       CST(*)
*
      INTEGER       K,LGUVEC
*-----------------------------------------------------------------------
      LGUVEC=N*NBUVEC
*
*     Z = Opel (A,X) (vecteurs de Nbvpca+1 a P)
*
      CALL VPOPEL (CHOIX,N,NBUVEC,TYMATA,MCMATA,ITYSYA,MCSTOA,NMSTOA
     &            ,TYMATB,MCMATB,ITYSYB,MCSTOB,NMSTOB
     &            ,TYVEC,MCX,MCZ,MCZAUX,NBMAVC,NBRESO,IST,RST,CST)
      NBOPEL=NBOPEL+NBUVEC
*
      IF (NPARA.EQ.1) THEN
         NBITER=NBITER+1
*
*        X = Z
*
         CALL TDANST (LGUVEC,TYVEC,MCZ,IST,RST,CST
     &                      ,TYVEC,MCX,IST,RST,CST)
      ELSE
         NBITER=NBITER+NPARA
*
*        Y = CHEB1 * Z
*
         CALL TEQAT  (LGUVEC,TYVEC,MCZ,RST,CST,'REEL',1,IST,CHEB1,CST
     &                      ,TYVEC,MCY,RST,CST)
*
         DO 430 K=1,NPARA/2-1
*
*           Z = Opel (A,Y)
*
            CALL VPOPEL (CHOIX,N,NBUVEC
     &                  ,TYMATA,MCMATA,ITYSYA,MCSTOA,NMSTOA
     &                  ,TYMATB,MCMATB,ITYSYB,MCSTOB,NMSTOB
     &                  ,TYVEC,MCY,MCZ,MCZAUX,NBMAVC,NBRESO,IST,RST,CST)
*
*           X = CHEB2 * Z - X
*
            CALL TAMOIT (LGUVEC,TYVEC,MCZ,RST,CST
     &                         ,'RE',1,IST,CHEB2,CST
     &                         ,TYVEC,MCX,RST,CST,TYVEC,MCX,RST,CST)
*
*           Z = Opel (A,X)
*
            CALL VPOPEL (CHOIX,N,NBUVEC
     &                  ,TYMATA,MCMATA,ITYSYA,MCSTOA,NMSTOA
     &                  ,TYMATB,MCMATB,ITYSYB,MCSTOB,NMSTOB
     &                  ,TYVEC,MCX,MCZ,MCZAUX,NBMAVC,NBRESO,IST,RST,CST)
*
*           Y = CHEB2 * Z - Y
*
            CALL TAMOIT (LGUVEC,TYVEC,MCZ,RST,CST
     &                         ,'RE',1,IST,CHEB2,CST
     &                         ,TYVEC,MCY,RST,CST,TYVEC,MCY,RST,CST)
*
            NBOPEL=NBOPEL+2*NBUVEC
430      CONTINUE
*
*        Z = Opel (A,Y)
*
         CALL VPOPEL (CHOIX,N,NBUVEC
     &               ,TYMATA,MCMATA,ITYSYA,MCSTOA,NMSTOA
     &               ,TYMATB,MCMATB,ITYSYB,MCSTOB,NMSTOB
     &               ,TYVEC,MCY,MCZ,MCZAUX,NBMAVC,NBRESO,IST,RST,CST)
         NBOPEL=NBOPEL+NBUVEC
*
*        X = CHEB2 * Z - X
*
         CALL TAMOIT (LGUVEC,TYVEC,MCZ,RST,CST
     &                      ,'RE',1,IST,CHEB2,CST
     &                      ,TYVEC,MCX,RST,CST,TYVEC,MCX,RST,CST)
      ENDIF
      END
