*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE VPMISE (CHOIX,NMMATA,NVMATA,NMMATB,NVMATB
     &                  ,NMVALP,NVVALP,NMVECP,NVVECP,NMESVP,NVESVP
     &                  ,NBVPDS,NBVPCA,OPTION)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : C.Hazard (1989), F.Mahe, D.Martin (Novembre 1996)
* Derniere modification : D.Martin  (7 Janvier 2004)
* Version 1.1
*  
* Calcul de valeurs et vecteurs propres (generalises) par la methode d'iteration
* de sous-espaces pour des matrices reelles symetriques :
*                              A X = [B] X D
* ou D matrice diagonale contient les valeurs propres (generalisees)
*    X contient les vecteurs propres correspondants en colonne
*  
*-- Arguments d'entree -- 
* CHOIX  parametre de choix de l'operation elementaire (cf VpOpel)
*  si NMMATB = ' ' (probleme non generalise)
*     ' '      : plus grandes valeurs propres Lambda de A 
*                calcul de Y = A*X
*     'L.LT'   : plus petites valeurs propres de A, A factorisee L.Lt
*                calcul de Y = A^{-1}*X avec A = L.L^{t}
*  sinon (probleme generalise)
*     'L.LT(A)': plus petites valeurs propres generalisees de A,B :
*                A X = lambda B X, A factorisee L.Lt
*                calcul de Y = L^{-1} B L^{-t} * X avec A = L.L^{t}
*     'L.LT(B)': plus grandes valeurs propres generalisees de A,B :
*                A X = lambda B X, B factorisee L.Lt
*                calcul de Y = L^{-1} A L * X  avec B = L.L^{t}
* NMMATA nom du tableau contenant la matrice A
* NVMATA son niveau
* NMMATB nom du tableau contenant la matrice B dans le cas generalise
*        (Poser NMMATB=' ' pour le probleme non generalise: A x = lambda x)
* NVMATB son niveau (entier quelconque si NMMATB=' ')
* NMVALP nom du tableau contenant les valeurs propres lambda calculees
*        suivies des erreurs sur les quelques valeurs propres suivantes
* NVVALP son niveau
* NMVECP nom du tableau contenant les vecteurs propres (ranges par colonnes)
*        en entree ils sont soit deja calcules (NBVPCA.gt.0) 
*        soit initialises a l'aide de VpINIT par exemple (NBVPCA=0).
*        En sortie les vecteurs propres calcules.
* NVVECP son niveau
* NMESVP nom du tableau contenant les "bornes d'erreur residuelle" sur 
*        les premieres valeurs propres deja calculees (NBVPCA) en entree et 
*        completees en sortie.
* NVESVP son niveau
* NBVPDS nombre de valeurs propres desirees
* NBVPCA nombre de valeurs propres deja calculees en entree et 
*        effectivement calculees en sortie
* OPTION chaine de caracteres definissant les valeurs des parametres optionnels
*        Elle est soit blanche (' '), soit structuree sous la forme d'articles 
*        de la forme <ITEM=ITEMVA>, separes par des virgules ou ITEM vaut 
*        l'une des chaines
*               'Epsilon', 'Saut', 'Maximum_Iterations', 'Impression'
*        et ITEMVA est le nom d'une donnee declaree.
*        Les valeurs des donnees correspondent resectivment aux variables
*        - EPSIL  seuil de d'arret des iterations de l'algorithme (defaut 1.E-4)
*        - SAUT   (defaut 10.)
*        - ITMAX  nombre maximum d'iterations (defaut Nblign/10)
*        - NIVIMP niveau d'impression des calculs (defaut 0)
*
*-- Remarque --
* Les tableaux NMVALP, NMVECP et NMESVP sont supposes deja crees et leur nombre
* de colonnes (LGVALP) doit etre strictement plus grand que NBVPDS.
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) CHOIX,NMMATA,NMMATB,NMVALP,NMVECP,NMESVP,OPTION
      INTEGER       NVMATA,NVMATB,NVVALP,NVVECP,NBVPDS,NBVPCA
     &             ,NIVIMP,NVESVP
*
      INCLUDE 'ALLOC'
      INCLUDE 'CONTEX'
*
      INTEGER       INDSYM,INTTYP
      INTEGER       ITMAX,MCVECP,MCESVP,MCVALP,MCHPP,MCTRAN,MCSDIA
     &             ,ITYSYA,NVSTOA,MCMATA,MCSTOA
     &             ,ITYSYB,NVSTOB,MCMATB,MCSTOB
     &             ,MCY,MCZ,MCAUX,ITREEL,LGVALP,PARENT,NCATR1,NCATR2
     &             ,NBLIGN,LGVECY,LGVECZ,LGVAUX,ITWORK,LGTBAU
     &             ,NBBCLE,NBITER,NBOPEL,NBMAVC,NBRESO,IERR,NBVPDC
      REAL          EPSIL,SAUT,RESIDU
      COMPLEX       CESIDU
      CHARACTER     PRFXAS*8,ERCODE*120,INVERS*9
      CHARACTER*8   TYMATA,TYMATB,TYVALP,TYVECP
      CHARACTER*6   NMSTOA,NMSTOB,NMTYKA,NMTYKB,NMFACA,NMFACB
      CHARACTER*14  NMSYMA,NMSYMB
      COMMON/FORMAH/ERCODE
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*VpMise*')
*
*--   Valeurs des parametres optionnels
      EPSIL = 1.E-4
      SAUT  = 10.
      ITMAX = 0
      NIVIMP= 0
      IF (OPTION.NE.' ') CALL VPOPTI (OPTION,EPSIL,SAUT,ITMAX,NIVIMP)
*
      IF (NIVIMP.GE.0.AND.IMPPAL.GT.0) THEN
         CALL PRNTRM (NMMATA,NVMATA,ERCODE,NCATR1)
         IF (NMMATB(1:1).NE.' ') THEN
            CALL PRNTRM (NMMATB,NVMATB,ERCODE(NCATR1+1:),NCATR2)
            WRITE (IMPPAL,10000) PRFXAS (0),NBVPDS
     &                 ,ERCODE(1:NCATR1),ERCODE(NCATR1+1:NCATR1+NCATR2)
         ELSE
            WRITE (IMPPAL,10006) PRFXAS (0),NBVPDS,ERCODE(1:NCATR1)
         ENDIF
      ENDIF
*
      CALL TBRR1  (ERCODE,NMVALP,NVVALP,LGVALP)
      IF (LGVALP.LE.NBVPDS) GOTO 99990
*
*--   Caracteristiques des matrices
      CALL VPPREP (NMMATA,NVMATA,NMMATB,NVMATB,NMVALP,NVVALP
     &            ,NMVECP,NVVECP,LGVALP
     &            ,TYMATA,NMSTOA,NVSTOA,NMSYMA,NMTYKA,NMFACA
     &            ,TYMATB,NMSTOB,NVSTOB,NMSYMB,NMTYKB,NMFACB
     &            ,TYVALP,TYVECP,NBLIGN,NIVIMP)
*
      IF (ITMAX.LE.0) ITMAX=MAX (10,1+NBLIGN/10)
*
      IF (TYMATA(1:4).NE.'REEL') CALL ERTERM (4,NMMATA,NVMATA)
      IF (NMSYMA(1:4).NE.'SYME') CALL WATERM (2,NMMATA,NVMATA)
      ITYSYA=INDSYM(NMSYMA)
      PARENT=1
      IF (NMMATB(1:1).NE.' ') THEN
        IF (TYMATB(1:4).NE.'REEL') CALL ERTERM (4,NMMATB,NVMATB)
        IF (NMSYMB(1:4).NE.'SYME') CALL WATERM (2,NMMATB,NVMATB)
        ITYSYB=INDSYM(NMSYMB)
         IF (CHOIX(1:1).EQ.' ') THEN
            PARENT=0
         ELSEIF (CHOIX(1:1).EQ.'L') THEN
            PARENT=INDEX (CHOIX,'(')
            IF (PARENT.GT.0) THEN
               IF (CHOIX(PARENT+1:PARENT+1).EQ.'A'
     &             .AND.NMFACA(1:1).EQ.' ') PARENT=0
               IF (CHOIX(PARENT+1:PARENT+1).EQ.'B'
     &             .AND.NMFACB(1:1).EQ.' ') PARENT=0
            ENDIF
         ELSE
            PARENT=0
         ENDIF
      ELSE
         IF (CHOIX(1:1).EQ.' ') THEN
            IF (NMFACA(1:1).NE.' ') PARENT=0
         ELSE
            IF (NMFACA(1:1).EQ.' ') PARENT=0
         ENDIF
      ENDIF
      IF (PARENT.LE.0) THEN
         ERCODE=CHOIX
         CALL BAISE  ('Choix : '//ERCODE(1:LEN(CHOIX))//' non prevu,'
     &               //' ou incompatible avec les matrices.')
      ENDIF
      IF (NMFACA(1:1).NE.' ') INVERS='(inverse)'
*
      CALL TBRR3  (ERCODE,NMMATA,NVMATA,MCMATA
     &                   ,NMVECP,NVVECP,MCVECP
     &                   ,NMESVP,NVESVP,MCESVP)
      MCMATB=0
      MCSTOB=1
      MCSTOA=1
      LGVECY=NBLIGN*LGVALP
      LGVECZ=LGVECY
      LGVAUX=0
      IF (NMSTOA(1:5).NE.'PLEIN') 
     &   CALL TBRR1  (ERCODE,NMSTOA,NVSTOA,MCSTOA)
      IF (NMMATB(1:1).NE.' ') THEN
         CALL TBRR1  (ERCODE,NMMATB,NVMATB,MCMATB)
         IF (NMSTOB(1:5).NE.'PLEIN') 
     &      CALL TBRR1  (ERCODE,NMSTOB,NVSTOB,MCSTOB)
         LGVAUX=LGVECY
      ENDIF
*
*     Creation des vecteurs de travail
*
      ITREEL=INTTYP ('REEL')
      ITWORK=INTTYP (TYVECP)
      CALL TBCREE ('&WORK$',0,ITWORK,LGVALP*(2*LGVALP+1),'c')
      LGTBAU=LGVALP*(2*LGVALP+1)
      ITWORK=INTTYP (TYVECP)
      CALL TBCREE ('&WORK$',1,ITWORK,LGVECY+LGVECZ+LGVAUX,'c')
      LGTBAU=LGTBAU+LGVECY+LGVECZ+LGVAUX
      WRITE (IMPPAL,10002) LGVALP
*
*     Adresses des vecteurs de travail
*
      CALL TBAR6  (ERCODE,NMMATA,NVMATA,MCMATA,NMVALP,NVVALP,MCVALP
     &                   ,NMVECP,NVVECP,MCVECP,NMESVP,NVESVP,MCESVP
     &                   ,'&WORK$',0,MCHPP,'&WORK$',1,MCY)
      MCTRAN=MCHPP +LGVALP*LGVALP
      MCSDIA=MCTRAN+LGVALP*LGVALP
      MCZ   =MCY+LGVECY
      MCAUX =MCZ
      IF (NMSTOA(1:5).NE.'PLEIN') 
     &   CALL TBAR1  (ERCODE,NMSTOA,NVSTOA,MCSTOA)
      IF (NMMATB(1:1).NE.' ') THEN
         CALL TBAR1  (ERCODE,NMMATB,NVMATB,MCMATB)
         IF (NMSTOB(1:5).NE.'PLEIN') 
     &      CALL TBAR1  (ERCODE,NMSTOB,NVSTOB,MCSTOB)
         MCAUX=MCZ+LGVECZ
      ENDIF
*
      NBVPDC=NBVPDS
      CALL VPMISR (CHOIX,NBLIGN,LGVALP,TYVECP,MCVECP,MCVALP,RST(MCESVP)
     &            ,NBVPDC,NBVPCA,EPSIL,SAUT,ITMAX,IERR,EPSMAC
     &            ,MCY,MCZ,MCAUX,MCHPP,MCTRAN,MCSDIA
     &            ,TYMATA,MCMATA,ITYSYA,MCSTOA,NMSTOA
     &            ,TYMATB,MCMATB,ITYSYB,MCSTOB,NMSTOB
     &            ,NBBCLE,NBITER,NBOPEL,NBMAVC,NBRESO,INVERS
     &            ,IST,RST,CST,NIVIMP,IMPSDR)
*
      CALL TSCALT (NBVPDS,'R',MCESVP,RST,CST,'R',MCESVP,RST,CST,CESIDU)
      RESIDU=SQRT(REAL(CESIDU))
*  
*     Destruction des tableaux temporaires de travail
      CALL TBTUER ('&WORK$',0)
      CALL TBTUER ('&WORK$',1)
      IF (NMMATB(1:1).NE.' ') THEN
         IF (NMSTOB(1:5).NE.'PLEIN') CALL TBSAVE (NMSTOB,NVSTOB)
         CALL TBSAVE (NMMATB,NVMATB)
      ENDIF
      IF (NMSTOA(1:5).NE.'PLEIN') CALL TBSAVE (NMSTOA,NVSTOA)
      CALL TBSAVE (NMMATA,NVMATA)
*-----Impressions
      IF (NIVIMP.GE.0.AND.IMPPAL.GT.0) THEN
         WRITE (IMPPAL,10001) NBVPCA,NBVPDS
      ENDIF
      IF (NIVIMP.GE.0.AND.IMPSDR.GT.0) THEN
         WRITE (IMPSDR,10005) '*VpMise*'
         WRITE (IMPSDR,10001) NBVPCA,NBVPDS
         WRITE (IMPSDR,10002) LGVALP,NBLIGN,LGTBAU
         WRITE (IMPSDR,10003) NBOPEL,NBMAVC,NBRESO
         IF (IERR.NE.0) THEN
            WRITE (IMPSDR,10004) IERR
         ENDIF
      ENDIF
*
      IF (IERR.EQ.2) THEN
         WRITE (*,11002) LGVALP,SAUT
         IF (NIVIMP.GE.0.AND.IMPSDR.GT.0) 
     &      WRITE (IMPSDR,11002) LGVALP,SAUT
      ELSEIF (IERR.EQ.10) THEN
         WRITE (*,11010) NBITER,RESIDU,EPSIL
         IF (NIVIMP.GE.0.AND.IMPSDR.GT.0) 
     &      WRITE (IMPSDR,11010) NBITER,RESIDU,EPSIL
      ELSEIF (IERR.EQ.11) THEN
         WRITE (*,11011) ITMAX
         IF (NIVIMP.GE.0.AND.IMPSDR.GT.0) WRITE (IMPSDR,11011) ITMAX
      ELSEIF (IERR.EQ.100) THEN
         WRITE (*,11100)
         IF (NIVIMP.GE.0.AND.IMPSDR.GT.0) WRITE (IMPSDR,11100)
      ELSEIF (IERR.EQ.100) THEN
         WRITE (*,11000)
         IF (NIVIMP.GE.0.AND.IMPSDR.GT.0) WRITE (IMPSDR,11000)
      ELSEIF (NIVIMP.GE.0.AND.IMPPAL.GT.0) THEN
         WRITE (IMPPAL,10010) NBITER,RESIDU,(RST(MCVALP+K),K=0,NBVPCA-1)
      ENDIF
*
      CALL TBSAVE (NMVALP,NVVALP)
      CALL TBSAVE (NMVECP,NVVECP)
      CALL TBSAVE (NMESVP,NVESVP)
*
      CALL PRFXMJ (-1,'*VpMise*')
      RETURN
*-----------------------------------------------------------------------
10000 FORMAT(/T2,A8,' ',I2,' Elements propres (Lambda,V) de AV=Lambda B'
     &,'V ou A=',A,' et B=',A)
10006 FORMAT(/T2,A8,' ',I2,' Elements propres (Lambda,V) de AV=Lambda V'
     &,' ou A=',A)
10001 FORMAT(T11,'-> Nb. vecteurs propres acceptes       ',I3,' /',I3)
10002 FORMAT(T11,'-> Dimension du sous-espace invariant  ',I3,:
     &      /T11,'-> Dimension des vecteurs propres   ',I6
     &      /T11,'-> Taille des tableaux auxiliaires',I8)
10003 FORMAT(T11,'-> Nb. operations elementaires       ',I5
     &/      T11,'-> Nb. multiplications matr. x vect. ',I5
     &/      T11,'-> Nb. resolutions de systemes lin.  ',I5)
10004 FORMAT(T11,'-> Code d''erreur                      ',I4)
10010 FORMAT(T11,'-> Nombre d''iterations ',I6,' , Residu =',E10.4
     &      /T11,'-> Lambda =',(T22,5E11.5,:/))
10005 FORMAT (/T2,A8,' Fin d''execution')
*
11002 FORMAT(/T5,';( Le dernier groupe de vecteurs propres n''a pu'
     &,' etre accepte;'
     &/T5,';( essayer d''augmenter la dimension de l''espace'
     &,' de travail (actuellement ',I4,')'
     &/T5,';( ou de diminuer la valeur du parametre Saut'
     &,' (actuellement ',F5.2,').')
11010 FORMAT(T11,';( Nombre maximum (',I4,') d''iterations atteint ;('
     &/T11,';(   Residu =',E11.4,'  Epsilon=',E11.4,' ;('
     &/T11,';( ',42('!'),' ;(')
11011 FORMAT (T5,';( L''estimation de la vitesse de convergence montre'
     &,' que le prochain groupe'
     &/T5,';( de vecteurs propres ne pouvait etre atteint en ITMAX= '
     &,I5,' iterations. On s''est arrete.')
11100 FORMAT (T5,';( Une valeurs propres trouvee nulle dans'
     &,' une iteration de Rayleigh-Ritz.'
     &/T5,';( Algorithme interrompu.')
11000 FORMAT (T2,';( Probleme de calcul de valeurs propres.'
     &/T5,';( Algorithme interrompu.')
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
99990 CALL ENCLER (LGVALP,ERCODE(1:3))
      CALL ENCLER (NBVPDS,ERCODE(4:6))
      CALL BAISE  ('La dimension du sous-espace de travail DIMEST='
     &//ERCODE(1:3)//' doit etre superieure au nombre de'
     &//' valeurs propres desire NBVPDS='//ERCODE(4:6)//'.')
                                                                    END !VpMise
