*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE VPMISR (CHOIX,N,P,TYVECX,MCVECP,MCVALP,EPSVP
     &                  ,NBVPDS,NBVPCA,EPSIL,SAUT,ITMAX,IERR,EPSMAC
     &                  ,MCY,MCZ,MCAUX,MCHPP,MCTRAN,MCSDIA
     &                  ,TYMATA,MCMATA,ITYSYA,MCSTOA,NMSTOA
     &                  ,TYMATB,MCMATB,ITYSYB,MCSTOB,NMSTOB
     &                  ,NBBCLE,NBITER,NBOPEL,NBMAVC,NBRESO,INVERS
     &                  ,IST,RST,CST,NIVIMP,IMPFCH)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : C.Hazard (Juin 1989) & F.Mahe, D.Martin (Decembre 1996)
* Derniere modification : D.Martin (9 Juin 2000)
* Version 1.0.1
*
* Calcul de quelques vecteurs propres et valeurs propres dominantes 
* d'une matrice ou de son inverse (pour les val. propres de plus petit module)
* par la methode d'iteration de sous-espaces.
* Cas des matrices reelles symetriques pour le probleme generalise:
*
*                               A X = [B] X D.
*
* ou D matrice diagonale contient les valeurs propres (generalisees)
*    X contient les vecteurs propres correspondant en colonne.
*
*-- Arguments --
* CHOIX  parametre de choix de l'operation elementaire (voir VpMise).
* N      definit la dimension du probleme a traiter (ordre des matrices du 
*        probleme aux valeurs propres); 
*        c'est le nombre de lignes des tableaux X, Y et Z
* P      dimension du sous-espace de travail;
*        c'est le nombre de colonnes des tableaux X, Y et Z
*        et un majorant strict du nombre de valeurs propres desire.
* MCVECP adresse dans RST du tableau X contenant les vecteurs propres.
*       - en entree, les P colonnes de X contiennent les vecteurs qui engendrent
*        le sous-espace invariant
*       - en sortie, les NBVPCA premieres colonnes de X contiennent les
*        vecteurs propres acceptes
* MCVALP adresse du tableau, note ci-apres VP, contenant les valeurs propres.
*       - en entree, si NBVPCA est non nul, les NBVPCA premiers elements de VP
*        contiennent les valeurs propres deja calculees. 
*       - en sortie, les NBVPCA premiers elements de VP sont les valeurs propres
*        correspondant aux vecteurs propres acceptes.
* EPSVP  vecteur de meme dimension que VP contenant en sortie les "bornes 
*        d'erreur residuelle" calculees par le sous-programme. Si NBVPCA 
*        est non nul, comme pour VP, EPSVP doit contenir en entree les valeurs 
*        calculees precedemment.
* NBVPDS en entree, NBVPDS est le nombre de vecteurs propres desires (NBVPDS 
*        doit etre strictement inferieur a P).
* NBVPCA - en entree, NBVPCA est le nombre de vecteurs propres deja acceptes 
*        (dans un calcul precedent, par exemple dans le cas ou une precedente 
*        execution de VPMISE a permis de calculer les premieres valeurs propres 
*        et vecteurs propres correspondants et on a augmente P pour une
*        nouvelle execution). Si NBVPCA .gt. 0 les NBVPCA premieres colonnes
*        de X contiennent ces vecteurs propres et les NBVPCA premiers elements 
*        de VP, les valeurs propres correspondantes.
*       - en sortie, NBVPCA est le nombre de vecteurs propres acceptes 
* EPSIL  parametre du test d'acceptation des vecteurs propres
*        (cf. VPSECV & VP1CHB)
* SAUT   permet de caracteriser le "niveau d'isolement" des P-1 valeurs propres 
*        dominantes par rapport au reste du spectre. Si le saut (VP(P-1)-VP(P))
*        est inferieur a EPSIL*SAUT*VP(P), on considere que cet isolement est
*        insuffisant pour pouvoir conclure a la convergence du "dernier" groupe 
*        de vecteurs propres (associe a un groupe de valeurs propres comprenant
*        VP(P-1) ). IERR est fixe a 2 . Pour obtenir ce dernier groupe de 
*        vecteurs, il faudra effectuer une nouvelle execution en ayant soit 
*        diminue le parametre SAUT, soit (ce qui parait plus judicieux)
*        augmente la dimension du sous-espace de travail (P).
*        Par defaut (si SAUT=0. en entree) SAUT prend la valeur 10.
* ITMAX  nombre maximum d'iterations.
* IERR   parametre de controle de l'execution du sous-programme:
*        IERR = 0 , aucune remarque particuliere sur l'execution
*        IERR = 1 , le nombre de vecteurs propres desires (NBVPDS) a ete 
*        augmente au cours de l'execution afin de pouvoir accepter les 
*        vecteurs propres associes au dernier groupe isole de valeurs propres.
*        IERR = 2 , le dernier groupe de vecteurs propres ne peut etre 
*        accepte compte tenu de la valeur du parametre SAUT (voir les 
*        commentaires concernant ce parametre).
*        IERR = 10 , le nombre maximum d'iterations (ITMAX) a ete atteint.
*        IERR = 11 , l'estimation de la vitesse de convergence a montre
*        que le prochain groupe de vecteurs propres ne pouvait converger
*        en ITMAX iterations. On s'est arrete avant de les effectuer.
*        IERR = 100 , une des valeurs propres calculees dans une iteration 
*        de Rayleigh-Ritz est (presque) nulle; l'execution a ete interrompue.
*        IERR = 1000 , dans une iteration de Rayleigh-Ritz, le calcul des 
*        valeurs propres s'est mal passe; l'execution a ete interrompue.
* EPSMAC ("epsilon machine") l'inf. des reels r (dependant de la machine ...) 
*        tel que  1+r .gt. 1
* MCY    adresse d'un tableau de travail de memes dimension et type que X
* MCZ    adresse d'un tableau de travail de memes dimension et type que X
* MCAUX  adresse d'un tableau de travail de memes dimension et type que X
* MCHPP  adresse de la matrice de Householder de dimension (1:P,1:P).
* MCTRAN adresse des tableaux de travail de dimension (1:P,1:P).
* MCSDIA vecteur de dimension P utilise localement pour la sous-diagonale
*        de la matrice HPP tridiagonalisee.
* TYMATA type (reel ou complexe) de la matrice A
* MCMATA adresse de la matrice A
* ITYSYA indicateur de symetrie de la matrice A
* MCSTOA adresse des tableaux de pointeurs de stockage.
* NMSTOA nom du tableau (type) de stockage de A.
* TYMATB type (reel ou complexe) de la matrice B
* MCMATB aresse de la matrice B (le cas echeant).
* ITYSYB indicateur de symetrie de la matrice B
* MCSTOB adresse des tableaux de pointeurs de stockage.
* NMSTOB nom du tableau (type) de stockage de B.
* NBBCLE nombre de boucle elementaires
* NBITER nombre d'iterations de Rayleigh-Ritz, dont acceleration de Chebyschev
* NBOPEL nombre d'operations elementaires
* NBMAVC nombre de multiplications matrice vecteur
* NBRESO nombre de resolutions de systeme lineaire
* INVERS ='(inverse)' indique que l'on calcule les inverses des valeurs propres
* IST,RST,CST super-tableau version entiere, reelle, complexe
* NIVIMP controle les ecritures eventuelles sur l'execution du sous-programme:
*         NIVIMP = 0 : pas d'ecriture
*         NIVIMP > 0 : resume de l'execution; les ecritures sont prevues 
*        jusqu'a un sous-espace de travail de dimension SOULIGX (P). 
*        Lors de l'ecriture des valeurs propres calculees, celles qui corres-
*        pondent a des vecteurs propres acceptes sont soulignees et
*        un / separe les differents groupes acceptes.
* IMPFCH numero d'unite logique du fichier d'impression
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) CHOIX,TYVECX,TYMATA,NMSTOA,TYMATB,NMSTOB
      CHARACTER     INVERS*9
      INTEGER  N,P,NBVPDS,NBVPCA,ITMAX,IERR,NIVIMP,IMPFCH
     &        ,MCVECP,MCVALP,MCY,MCZ,MCAUX,MCHPP,MCTRAN,MCSDIA
     &        ,ITYSYA,MCMATA,MCSTOA,ITYSYB,MCMATB,MCSTOB
     &        ,NBBCLE,NBITER,NBOPEL,NBMAVC,NBRESO,IST(*)
      REAL     EPSIL,SAUT,EPSMAC,EPSVP(*),RST(*)
      COMPLEX  CST(*)
*
      INTEGER P2,NCONV,NPARA,NREST,NUVP,IRND,NBCCAL,LGUVEC,NBUVEC
     &       ,MCTRDI,MCTR1,MCHPPD,MCHPP1,MCHPPN,MCUAUX
     &       ,MCVEC1,MCUVEC,MCUVE1,MCUVEN,MCY1,MCUY,MCUY1,MCUYN,MCUZ
     &       ,MCVP,KLADRS,I,J,K,NBVPC1,SOULIX,PARENT,IRCODE
     &       ,MCBOPE,MCBORT
      PARAMETER (SOULIX=20)
      REAL    PRECIS,OLDPRE,CCONV,CHEB1,CHEB2,VPMMIN
      COMPLEX C
      CHARACTER AST
      CHARACTER*12 SOULIG(SOULIX)
*-----------------------------------------------------------------------
      MCBOPE=MCMATB
      MCBORT=0
*     Valeur par defaut: matrices symetriques reelles.
      TYVECX='REEL'
      TYMATA=TYVECX
      TYMATB=TYVECX
*     Initialisation
      DO 1000 I=1,20
         SOULIG(I)=' '
1000  CONTINUE
      IERR=0
*     Numero de la boucle elementaire en cours, de l'iteration en cours,
*     nombre d'operations elementaires effectuees
      NBITER=0
      NBBCLE=0
      NBOPEL=0
      NBMAVC=0
      NBRESO=0
*     Coefficients de controle de la convergence (cf Vp1Chb) 
      CCONV=10.
      NCONV =0
      OLDPRE=0.
*
      IF (NIVIMP.GE.1.AND.IMPFCH.GT.0) 
     &   WRITE (IMPFCH,10000) '*VpMisr*',EPSIL,SAUT,ITMAX
*     Adresse relative de travail dans les matrices X, Y, Z
*     = nombre de coefficients de X deja calcules et
*     calcul des adresses utiles des matrices X, Y et Z
      NBUVEC=P-NBVPCA
      LGUVEC=N*NBUVEC
      NBCCAL=N*NBVPCA
      MCUVEC=MCVECP+NBCCAL
      MCUY  =MCY+NBCCAL
      MCUZ  =MCZ+NBCCAL
      MCUAUX=MCAUX+NBCCAL
      NBVPC1=NBVPCA+1
*
*     Orthonormalisation des vecteurs X initiaux
*
      CALL ORTOGS (N,TYVECX,1,P,MCVECP,MCVECP,0,RST,CST)
*
*     Iteration "0" : controle des vecteurs X initiaux
*
      CALL VPOPEL (CHOIX,N,NBUVEC
     &            ,TYMATA,MCMATA,ITYSYA,MCSTOA,NMSTOA
     &            ,TYMATB,MCBOPE,ITYSYB,MCSTOB,NMSTOB
     &            ,TYVECX,MCUVEC,MCUY,MCUAUX,NBMAVC,NBRESO,IST,RST,CST)
      NBOPEL=NBOPEL+NBUVEC
*     Copie des derniers vecteurs de Y = Opel (A,[B],X) dans X.
      CALL TDANST (LGUVEC,TYVECX,MCUY,IST,RST,CST
     &                   ,TYVECX,MCUVEC,IST,RST,CST)
*
*     Orthonormalisation des vecteurs de X ( X_{Nbvpca+1} a X_{p} )
*
      CALL ORTOGS (N,TYVECX,NBVPC1,P,MCVECP,MCVECP,MCTRAN
     &            ,RST,CST)
*
*     Controle d'un eventuel mauvais choix initial de X
      IRND=123456
*     On va retrouver le meme que le premier si IRND=0: A REVOIR
      MCTRDI=MCTRAN+NBVPCA*(P+1)
      MCUVE1=MCUVEC
      DO 10 NUVP=NBVPC1,P
         IF (RST(MCTRDI).EQ.0.) THEN
            CALL TBRAND (IRND,N,TYVECX,MCUVE1,IST,RST,CST)
            NBITER=-1
         ENDIF
         MCTRDI=MCTRDI+P+1
         MCUVE1=MCUVE1+N
10    CONTINUE
      IF (NBITER.EQ.-1) THEN
         CALL ORTOGS (N,TYVECX,NBVPC1,P,MCVECP,MCVECP,0
     &               ,RST,CST)
         NBITER=0
      ENDIF
      CALL TAZERO (P*P,TYVECX,MCHPP,AST,IST,RST,CST)
*
*-----Boucle elementaire de l'algorithme
*
1     NBBCLE=NBBCLE+1
*
*--- 1) Iteration de RAYLEIGH-RITZ
*
      NBITER=NBITER+1
      IF (NIVIMP.GE.1.AND.IMPFCH.GT.0)WRITE (IMPFCH,10001)NBBCLE,NBITER
*
*     Y = OPEL (A,[B],X)
*
      CALL VPOPEL (CHOIX,N,NBUVEC
     &            ,TYMATA,MCMATA,ITYSYA,MCSTOA,NMSTOA
     &            ,TYMATB,MCBOPE,ITYSYB,MCSTOB,NMSTOB
     &            ,TYVECX,MCUVEC,MCUY,MCUAUX,NBMAVC,NBRESO,IST,RST,CST)
      NBOPEL=NBOPEL+NBUVEC
*        write(8,*) 'X av Hpp'
*        write(8,12005) (RST(KZ),KZ=MCUVEC,MCUVEC-1+N*P)
*        write(8,*) 'Y av Hpp'
*        write(8,12005) (RST(KZ),KZ=MCUY,MCUY-1+N*P)
*
*     Calcul du 'quotient de Rayleigh' matriciel de X : H = X^{t}*Y
*     (partie triangulaire inferieure)
*
      MCHPPD=MCHPP
      MCVP  =MCVALP
      MCVEC1=MCVECP
      MCUVEN=MCUVEC
      MCUYN =MCUY
      DO 110 J=1,NBVPCA
         MCHPPN=MCHPPD+NBVPCA-J
*        affectation du coeff diagonal, on conserve les elements propres
*        deja calcules
         RST(MCHPPD)=RST(MCVP)
         DO 107 MCHPP1=MCHPPD+1,MCHPPN
             RST(MCHPP1)=0.
107      CONTINUE
         MCHPP1=MCHPPN+1
         MCUY1 =MCUYN
         DO 109 I=NBVPC1,P
*           produit scalaire des colonnes J de X et I de Y.
            CALL TSCALT (N,TYVECX,MCVEC1,RST,CST
     &                    ,TYVECX,MCUY1,RST,CST,C)
            RST(MCHPP1)=REAL(C)
            MCUY1 =MCUY1+N
            MCHPP1=MCHPP1+1
109      CONTINUE
         MCVEC1=MCVEC1+N
         MCVP  =MCVP+1
         MCHPPD=MCHPP1+J
110   CONTINUE
      DO 120 J=NBVPC1,P
         MCHPP1=MCHPPD
         MCVEC1=MCUVEN
         DO 119 I=J,P
*           produit scalaire des colonnes I de X et J de Y.
            CALL TSCALT (N,TYVECX,MCVEC1,RST,CST
     &                    ,TYVECX,MCUYN,RST,CST,C)
            RST(MCHPP1)=REAL(C)
            MCVEC1     =MCVEC1+N
            MCHPP1     =MCHPP1+1
119      CONTINUE
         MCUVEN=MCUVEN+N
         MCUYN =MCUYN+N
         MCHPPD=MCHPP1+J
120   CONTINUE
*      write(8,*) 'Hpp Nbiter,Nbvpca',NBITER,NBVPCA
*      write(8,12005) (RST(KZ),KZ=MCHPP,MCHPP-1+P*P)
*
*     Calcul des P valeurs et vecteurs propres de H
*
      CALL TRIDRS (P,RST(MCHPP),RST(MCVALP),RST(MCSDIA),RST(MCTRAN))
*
      CALL QLIMRS (P,RST(MCVALP),RST(MCSDIA),RST(MCTRAN),EPSMAC,K)
*
*     Si le calcul des valeurs propres s'est mal passe ...
      IF (K .NE. 0) THEN
         IERR=1000
         RETURN
      ENDIF
*
*     Controler que toutes les valeurs propres sont non nulles
      CALL TMINVA (P,TYVECX,MCVALP,IST,RST,CST,KLADRS,I,VPMMIN,C)
      IF (ABS(VPMMIN).LE.EPSMAC) THEN
         IERR=100
         RETURN
      ENDIF
*
*     Sauvegarde Z = X
      CALL TDANST (N*P,TYVECX,MCVECP,IST,RST,CST
     &                ,TYVECX,MCZ,IST,RST,CST)
*
*     Calcul des images des vecteurs de Ritz
*
*     On remplit les premieres colonnes de Y=lambda*X pour les
*     valeurs propres deja calculees pour faciliter le calcul suivant.
*     On a Y = OPEL (A,[B],X) pour les dernieres colonnes.
*
      MCVEC1=MCVECP      
      MCY1  =MCY
      MCVP  =MCVALP
      DO 141 NUVP=1,NBVPCA
         CALL TEQAT (N,TYVECX,MCVEC1,RST,CST
     &                ,TYVECX,MCVP,IST,RST,CST
     &                ,TYVECX,MCY1,RST,CST)
         MCVP  =MCVP+1
         MCVEC1=MCVEC1+N
         MCY1  =MCY1+N
141   CONTINUE
*
*     X = Y * TRAN
      MCTR1 =MCTRAN+P*NBVPCA
      MCUVE1=MCUVEC
      CALL TAZERO (LGUVEC,TYVECX,MCUVE1,AST,IST,RST,CST)
      DO 150 NUVP=NBVPC1,P
         CALL PLVC   (N,P,TYVECX,MCY,TYVECX,MCTR1,TYVECX,MCUVE1,RST,CST)
         MCUVE1=MCUVE1+N
         MCTR1 =MCTR1+P
150   CONTINUE
*
*---  2) Calcul des quantites de controle de la convergence
*
*     Calcul de  Y = X-Z*TRAN*VP (Y = Z*TRAN & Y = X-Y*VP)
*     et de la norme des colonnes de la matrice residuelle ||X_{n+1}-X_n||
*
      MCTR1 =MCTRAN+P*NBVPCA
      MCUVE1=MCUVEC
      MCUY1 =MCUY
      MCVP  =MCVALP+NBVPCA
      CALL TAZERO  (LGUVEC,TYVECX,MCUY1,AST,IST,RST,CST)
      DO 230 NUVP=NBVPC1,P
         CALL PLVC   (N,P,TYVECX,MCZ,TYVECX,MCTR1,TYVECX,MCUY1,RST,CST)
         MCTR1 =MCTR1+P
         CALL TPLUAT (N,TYVECX,MCUVE1,RST,CST
     &                 ,TYVECX,1,IST,-RST(MCVP),CST
     &                 ,TYVECX,MCUY1,RST,CST,TYVECX,MCUY1,RST,CST)
         MCUVE1=MCUVE1+N
         MCVP  =MCVP+1
         CALL TSCALT (N,TYVECX,MCUY1,RST,CST
     &                 ,TYVECX,MCUY1,RST,CST,C)
         MCUY1 =MCUY1+N
         EPSVP(NUVP)=SQRT (REAL (C))
230   CONTINUE
*     et orthonormalisation des vecteurs propres
      CALL ORTOGS (N,TYVECX,NBVPC1,P,MCVECP,MCVECP,0
     &            ,RST,CST)
*
*---  3) Tests de convergence des vecteurs propres
*
      CALL VPSECV (P,NBVPCA,P2,RST(MCVALP),EPSVP,NBVPDS,SOULIX,SOULIG
     &            ,SAUT,EPSIL,PRECIS,OLDPRE,IERR,NBITER
     &            ,INVERS,NIVIMP,IMPFCH)
*
*     Le nombre de vecteurs propres deja calculees peut avoir change.
*     Mise a jour des adresses utiles des matrices X, Y, Z.
*
      NBUVEC=P-NBVPCA
      LGUVEC=N*NBUVEC
      NBCCAL=N*NBVPCA
      MCUVEC=MCVECP+NBCCAL
      MCUY  =MCY+NBCCAL
      MCUZ  =MCZ+NBCCAL
      MCUAUX=MCAUX+NBCCAL
      NBVPC1=NBVPCA+1
*
      IF (NBVPCA.EQ.NBVPDS.OR.IERR.EQ.2) GOTO 900
*     si le nombre d'iterations autorise est atteint ...
      IF (NBITER.GE.ITMAX) THEN
         IERR=10
         GOTO 900
      ENDIF
*
*---  4) Boucle d'acceleration de Chebyschev
*
*     Preliminaires ..
*
      IF (NIVIMP.GE.1.AND.IMPFCH.GT.0) WRITE (IMPFCH,10003) NBITER
      CALL VP1CHB (P,NBVPCA,P2,RST(MCVALP),NCONV,CCONV,NPARA
     &            ,EPSIL,PRECIS,OLDPRE,IERR,NIVIMP,IMPFCH)
*
*     Si la convergence ne peut a coup sur etre obtenue en ITMAX
*     iterations, on s'arrete la ...
      IF (NBITER+NCONV.GE.ITMAX+5) THEN
         IERR=11
         IF (NIVIMP.GE.1.AND.IMPFCH.GT.0) 
     &      WRITE (IMPFCH,10010) NBITER+NCONV
         GOTO 900
      ELSE
         NCONV=MIN (ITMAX-NBITER-1,NCONV)
         IF (NIVIMP.GE.1.AND.IMPFCH.GT.0)
     &      WRITE (IMPFCH,10011) NCONV,NPARA
         IF (NCONV.LE.0) GOTO 1
      ENDIF
*
      OLDPRE=PRECIS
      CHEB2=2./ABS(RST(MCVALP+P-1))
      CHEB1=.51*CHEB2
      NREST=NCONV
400   NPARA=MIN (NPARA,NREST)
      IF (NPARA.EQ.0)  GOTO 1
*
*     Acceleration de Chebyschev et re-orthonormalisation
*
      CALL VP2CHB (CHOIX,N,NBUVEC,CHEB1,CHEB2,NPARA
     &            ,TYMATA,MCMATA,ITYSYA,MCSTOA,NMSTOA
     &            ,TYMATB,MCBOPE,ITYSYB,MCSTOB,NMSTOB
     &            ,TYVECX,MCUVEC,MCUY,MCUZ,MCUAUX
     &            ,NBOPEL,NBITER,NBMAVC,NBRESO,IST,RST,CST)
*
      CALL ORTOGS (N,TYVECX,NBVPC1,P,MCVECP,MCVECP,0
     &            ,RST,CST)
*
      NREST=NREST-NPARA
      GOTO 400
*
*-----Fin de la boucle elementaire
*
*     Calcul des vecteurs de Ritz ( X = Z * TRAN )
*
900   MCTR1=MCTRAN
      MCVEC1=MCVECP
      CALL TAZERO (N*P,TYVECX,MCVEC1,AST,IST,RST,CST)
      DO 910 NUVP=1,P
         CALL PLVC   (N,P,TYVECX,MCZ,TYVECX,MCTR1,TYVECX,MCVEC1,RST,CST)
         MCVEC1=MCVEC1+N
         MCTR1=MCTR1+P
910   CONTINUE
*
*     Calcul si necessaire des inverses des valeurs propres
*
      IF (INVERS(1:1).NE.' ') THEN
         DO 920 MCVP=MCVALP,MCVALP-1+NBVPCA
            RST(MCVP)=1./RST(MCVP)
920      CONTINUE
      ENDIF
*
*     Retour si necessaire aux vrais vecteurs propres et normalisation
*
      IF (MCBOPE.GT.0.AND.CHOIX(1:1).EQ.'L') THEN
         PARENT=INDEX (CHOIX,'(')
         MCVEC1=MCVECP
         IF (CHOIX(PARENT+1:PARENT+1).EQ.'A') THEN
            DO 930 NUVP=1,NBVPCA
               CALL RSMAFU (NMSTOA,MCSTOA,CHOIX,TYMATA,MCMATA
     &                     ,1,TYVECX,MCVEC1,TYVECX,MCVEC1
     &                     ,IST,RST,CST,IRCODE)
               CALL TSCALT (N,TYVECX,MCVEC1,RST,CST
     &                       ,TYVECX,MCVEC1,RST,CST,C)
               CALL TEQAT  (N,TYVECX,MCVEC1,RST,CST
     &                       ,TYVECX,1,IST,1./SQRT(REAL(C)),C
     &                       ,TYVECX,MCVEC1,RST,CST)
               MCVEC1=MCVEC1+N
930         CONTINUE
         ENDIF
      ENDIF
*
      IF (NIVIMP.GE.3.AND.IMPFCH.GT.0) THEN
         MCVEC1=MCVECP
         WRITE (IMPFCH,12006)
         DO 950 NUVP=1,NBVPCA
            WRITE (IMPFCH,12004) NUVP,RST(MCVALP-1+NUVP)
            WRITE (IMPFCH,12005) (RST(MCUVEC),MCUVEC=MCVEC1,MCVEC1+N-1)
            MCVEC1=MCVEC1+N
950      CONTINUE
      ENDIF
*-----------------------------------------------------------------------
10000 FORMAT (/T2,A8,' Resume d''execution ('
     &,' Epsilon=',E10.5,' Saut=',E10.5
     &,' Maximum d''iterations=',I4,' )')
10001 FORMAT (/T2,'Boucle elementaire numero ',I3,' ',50('-')
     &/T4,'Rayleigh-Ritz a l''iteration ',I3)
10003 FORMAT (T4,'Boucle de Chebyshev a l''iteration ',I3)
10010 FORMAT (T8,'La convergence theorique necessiterait ',I6,
     &   ' iterations .... on s''arrete la.')
10011 FORMAT (T6,'Nombre d''iterations dans la boucle      : ',I3
     &     ,/,T6,'Nombre max. d''iterations entre 2 ortho. : ',I3)
12002 FORMAT (10(' ',G11.5))
12004 FORMAT (T2,'Vecteur propre # ',I3,' Valeur propre =',G11.5)
12005 FORMAT (10(' ',E11.5))
12006 FORMAT (/T2,120('#'))
      END
