*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE VPOPEL (CHOIX,N,NBVECT
     &                  ,TYMATA,MCMATA,ITYSYA,MCSTOA,NMSTOA
     &                  ,TYMATB,MCMATB,ITYSYB,MCSTOB,NMSTOB
     &                  ,TYVECX,MCX,MCY,MCAUX,NBMAVC,NBRESO,IST,RST,CST)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : F.Mahe, D.Martin (Janvier 1997)
* Derniere modification : D.Martin (15 Juin 1999)
* Version 1
*  
* Operation elementaire pour le calcul d'elements propres par la methode
* d'iteration de sous-espaces: 
* multiplication matrice vecteurs ou resolution de systemes lineaires.
*  
*-- Arguments d'entree -- 
* CHOIX  parametre de choix de l'operation elementaire
*  si NMMATB = ' ' (probleme non generalise)
*     ' '      : plus grandes valeurs propres Lambda de A 
*                calcul de Y = A*X
*     'L.LT'   : plus petites valeurs propres de A, A factorisee L.Lt
*                calcul de Y = A^{-1}*X avec A = L.L^{t}
*  sinon (probleme generalise)
*     'L.LT(A)': plus petites valeurs propres generalisees de A,B :
*                A X = lambda B X, A factorisee L.Lt
*                calcul de Y = L^{-1} B L^{-t} * X avec A = L.L^{t}
*     'L.LT(B)': plus grandes valeurs propres generalisees de A,B :
*                A X = lambda B X, B factorisee L.Lt
*                calcul de Y = L^{-1} A L * X  avec B = L.L^{t}
* N      definit la dimension du probleme
* NBVECT nombre de vecteurs operandes/resultats
* TYMATA type (relle ou complexe) de la matrice A.
* MCMATA adresse de la matrice A de taille N*N.
* ITYSYA indice de type de symetrie de la matrice A.
* MCSTOA adresse des tableaux de pointeurs de stockage de la matrice A
* NMSTOA nom du tableau (et type) de stockage de A.
* TYMATB type (relle ou complexe) de la matrice B (le cas echeant).
* MCMATB adresse de la matrice B de taille N*N (le cas echeant).
* ITYSYB indice de type de symetrie de la matrice B (le cas echeant).
* MCSTOB adresse des tableaux de pointeurs de stockage de B(le cas echeant)
* NMSTOB nom du tableau (et type) de stockage de B (le cas echeant).
* TYVECX type de la matrice des vecteur colonnes X et Y.
* MCX    adresse des vecteurs X operandes de l'operation elementaire
* MCY    adresse des vecteurs Y resultats de l'operation elementaire,
* MCAUX  adresse d'un tableau de meme longueur que X et Y (N*NBVECT)
*        (peut coincider avec MCY si MCMATB=0)
* NBMAVC cumul du nombre de multiplications matrice x vecteur
* NBRESO cumul du nombre de resolutions de systeme lineaire
* IST,RST,CST super-tableau version entiere, reelle ou complexe
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) CHOIX,NMSTOA,NMSTOB,TYMATA,TYMATB,TYVECX
      INTEGER N,NBVECT,MCMATA,MCSTOA,ITYSYA,MCMATB,MCSTOB,ITYSYB
     &       ,MCX,MCY,MCAUX,NBMAVC,NBRESO,IST(*)
      REAL    RST(*)
      COMPLEX CST(*)
*
      INTEGER MCX1,MCY1,MCAUX1,PARENT,IRCODE,NUV
      CHARACTER ERRORC*16
*-----------------------------------------------------------------------
      MCX1=MCX
      MCY1=MCY
      MCAUX1=MCAUX
      IRCODE=0
      IF (MCMATB.EQ.0) THEN
*
*        Probleme de valeurs propres  A X = Lambda X
*        (Matrice B a adresse nulle)
*
         IF (CHOIX(1:1).NE.' ') THEN
*
*           Plus petites valeurs propres 
*           Calcul de Y = A^{-1} * X ou A est factorisee
*
            IF (CHOIX(1:1).EQ.'L') THEN
               DO 101 NUV=1,NBVECT
               CALL RSMAFA (NMSTOA,MCSTOA,CHOIX,TYMATA,MCMATA
     &                     ,1,TYVECX,MCX1,TYVECX,MCY1
     &                     ,IST,RST,CST,IRCODE)
               MCX1=MCX1+N
               MCY1=MCY1+N
101            CONTINUE
               NBRESO=NBRESO+NBVECT
            ENDIF
*
         ELSE
*
*           Plus grandes valeurs propres 
*           Calcul de Y = A * X
*
            CALL MATMPC (N,N,NMSTOA,MCSTOA,ITYSYA,TYMATA,MCMATA
     &                  ,NBVECT,TYVECX,MCX,TYVECX,MCY
     &                  ,IST,RST,CST)
            NBMAVC=NBMAVC+NBVECT
         ENDIF
*
      ELSE
*
*        Probleme de valeurs propres generalise A X = Lambda B X
*        CHOIX est du type 'L.LT(A)', L.LT(B)', ...
*
         PARENT=INDEX (CHOIX,'(')
         IF (CHOIX(PARENT+1:PARENT+1).EQ.'A') THEN
*
*           Plus petites valeurs propres
*           Calcul de Y = L^{-1} B L^{-t} * X avec A = L.L^{t}
*
            IF (CHOIX(1:1).EQ.'L') THEN
*              Calcul de AUX = L^{-t} * X
               DO 301 NUV=1,NBVECT
               CALL RSMAFU (NMSTOA,MCSTOA,CHOIX,TYMATA,MCMATA
     &                     ,1,TYVECX,MCX1,TYVECX,MCAUX1
     &                     ,IST,RST,CST,IRCODE)
               MCX1=MCX1+N
               MCAUX1=MCAUX1+N
301            CONTINUE
            ENDIF
*           Calcul de Y = B * AUX
            CALL MATMPC (N,N,NMSTOB,MCSTOB,ITYSYB,TYMATB,MCMATB
     &                  ,NBVECT,TYVECX,MCAUX,TYVECX,MCY
     &                  ,IST,RST,CST)
            NBMAVC=NBMAVC+NBVECT
            IF (CHOIX(1:1).EQ.'L') THEN
*              Calcul de Y = L^{-1} Y
               DO 302 NUV=1,NBVECT
               CALL RSMAFL (NMSTOA,MCSTOA,CHOIX,TYMATA,MCMATA
     &                     ,1,TYVECX,MCY1,TYVECX,MCY1
     &                     ,IST,RST,CST,IRCODE)
               MCY1=MCY1+N
302            CONTINUE
               NBRESO=NBRESO+NBVECT
            ENDIF
*
         ELSEIF (CHOIX(PARENT+1:PARENT+1).EQ.'B') THEN
*
*           Plus grandes valeurs propres :
*           Calcul de Y = L^{-1} A L * X  avec B = L.L^{t}
*
            IF (CHOIX(1:1).EQ.'L') THEN
*              Calcul de AUX = L * X
               DO 201 NUV=1,NBVECT
               CALL RSMAFL (NMSTOB,MCSTOB,CHOIX,TYMATB,MCMATB
     &                     ,1,TYVECX,MCX1,TYVECX,MCAUX1
     &                     ,IST,RST,CST,IRCODE)
               MCX1=MCX1+N
               MCAUX1=MCAUX1+N
201            CONTINUE
            ENDIF
*           Calcul de Y = A * AUX
            CALL MATMPC (N,N,NMSTOA,MCSTOA,ITYSYA,TYMATA,MCMATA
     &                  ,NBVECT,TYVECX,MCAUX,TYVECX,MCY
     &                  ,IST,RST,CST)
            NBMAVC=NBMAVC+NBVECT
            IF (CHOIX(1:1).EQ.'L') THEN
*              Calcul de Y = L^{-1} Y
               DO 202 NUV=1,NBVECT
               CALL RSMAFU (NMSTOB,MCSTOB,CHOIX,TYMATB,MCMATB
     &                     ,1,TYVECX,MCY1,TYVECX,MCY1
     &                     ,IST,RST,CST,IRCODE)
               MCY1=MCY1+N
202            CONTINUE
               NBRESO=NBRESO+NBVECT
            ENDIF
         ENDIF
      ENDIF
      IF (IRCODE.EQ.1) GOTO 99901
      RETURN
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
99901 ERRORC=CHOIX
      CALL BAISE  ('*VpOpel* Choix : '//ERRORC//' non prevu.')
                                                                    END !VpOpel
