*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE VPSECV (P,NBVPCA,P2,VP,EPSVP,NBVPDS,SOULIX,SOULIG
     &                  ,SAUT,EPSIL,PRECIS,OLDPRE,IERR,NUITER
     &                  ,INVERS,NIVIMP,IMPFCH)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : C.Hazard (Juin 1989), F.Mahe, D.Martin (Fevrier 1997)
* Derniere modification : D.Martin (28 Mai 1999)
*
* Tests de convergence des vecteurs propres dans la methode d'iterations
* de sous-espaces 
*
*-- Arguments --
* P      dimension du sous-espace invariant
* NBVPCA nombre de valeurs propres deja calculees.
* P2     en sortie, les vecteurs de numero NBVPCA+1 a P2 forment le prochain
*        groupe de vecteurs propres qui doit converger.
* VP     tableau contenant les valeurs propres.
* EPSVP  vecteur de meme dimension que VP contenant les "bornes  d'erreur
*        residuelle".
* NBVPDS nombre de vecteurs propres desires (peut etre augmente!).
* EPSIL  parametre du test d'acceptation des vecteurs propres. 
*        Le test impose que le Sinus de l'angle entre un groupe de vecteurs
*        propres calcule et le sous-espace invariant correspondant est inferieur
*        a EPSIL.
* SOULIX nombre maximum de v.p. acceptees soulignees
* SOULIG chaines de caractere pour souligner les v.p. acceptees
* SAUT   permet de caracteriser le "niveau d'isolement" des P-1 valeurs propres 
*        dominantes par rapport au reste du spectre. 
*        Si le saut (VP(P-1)-VP(P)) est inferieur a EPSIL*SAUT*VP(P), 
*        on considere que cet isolement est insuffisant pour pouvoir conclure 
*        a la convergence du "dernier" groupe de vecteurs propres 
*       (associe a un groupe de valeurs propres comprenant VP(P-1) ). 
*        Pour obtenir ce dernier groupe de vecteurs, il faudra effectuer 
*        une nouvelle execution soit en diminuant le parametre SAUT, 
*        soit (ce qui parait plus judicieux) en augmentant la dimension 
*        du sous-espace de travail (P).
* PRECIS precision utilisee pour determiner le prochain groupe de
*        vecteurs propres qui doit converger. Initialise a .2
* OLDPRE derniere estimation d'erreur calculee, ancien PRECIS.
* IERR   parametre de controle de l'execution de l'algorithme (voir VpMisr)
* NUITER numero de l'iteration courante
* INVERS '(inverse)' lors du calcul des plus petites valeurs propres
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CHARACTER*(*) SOULIG(*),INVERS*9
      INTEGER P,P2,NBVPCA,NBVPDS,SOULIX,IERR,NUITER,NIVIMP,IMPFCH
      REAL    VP(*),EPSVP(*),EPSIL,SAUT,PRECIS,OLDPRE
*
      INTEGER       NGROUP,P1,NUVP,I,PW,NBW,K
      REAL          ANGLE,EPS2,DIST
*-----------------------------------------------------------------------
*     preliminaires
300   NGROUP= 0
*     PRECIS precision utilisee pour determiner le prochain groupe de
*     vecteurs propres qui doit converger.
      PRECIS=.2
      P2    = NBVPDS
      EPS2  = 0.
*     estimation du "saut a droite"
      IF (NBVPCA.EQ.0) THEN
         DIST = ABS(VP(1))
      ELSE
         DIST = ABS(VP(NBVPCA))-EPSVP(NBVPCA)-ABS(VP(NBVPCA+1))
      ENDIF
*
*     Boucle sur les groupes de vecteurs propres
310   NGROUP= NGROUP+1
      P1    = NBVPCA+NGROUP
*     Estimation de l'erreur pour le groupe  NBVPCA+1,NBVPCA+NGROUP
      EPS2 = EPS2+EPSVP(P1)*EPSVP(P1)
*     Estimation du saut "a gauche"
      ANGLE = DIST
      DO 320 NUVP = P1+1,P
         ANGLE = MIN ( ANGLE,ABS(VP(P1))-ABS(VP(NUVP))-EPSVP(NUVP) )
320   CONTINUE
      IF (ANGLE.GT.0) THEN
         ANGLE = SQRT(EPS2)/ANGLE
      ELSE
         ANGLE = 1.
      ENDIF
*
*     Test proprement dit ...
      IF (ANGLE.LT.PRECIS) THEN
*        Cas ou la precision est meilleure qu'avec le precedent groupe
         PRECIS= ANGLE
         P2    = P1
         IF (NIVIMP.GE.1.AND.IMPFCH.GT.0) THEN
            IF (NBVPCA+1.EQ.P1) THEN
             WRITE (IMPFCH,12004) P1,PRECIS,EPSIL,NBVPCA,NUITER
            ELSE
             WRITE (IMPFCH,12003) NBVPCA+1,P1,PRECIS,EPSIL,NBVPCA,NUITER
            ENDIF
         ENDIF
         IF (PRECIS.LE.EPSIL) THEN
*           Acceptation du groupe NBVPCA+1 a NBVPCA+NGROUP
            IF (NIVIMP.GE.1.AND.IMPFCH.GT.0) THEN
               DO 350 I=NBVPCA+1,MIN(SOULIX,P1)
                  SOULIG(I) = '------------'
350            CONTINUE
               IF (P1 .LE. SOULIX) SOULIG(P1)(12:12) = '/'
            ENDIF
            NBVPCA= NBVPCA + NGROUP
            IF (NBVPCA.GT.NBVPDS) THEN
               IERR  = 1
               NBVPDS= NBVPCA
            ENDIF
            OLDPRE= 0.
*           on passe a un nouveau test ...
            IF (NBVPCA.LT.NBVPDS)  GOTO 300
         ENDIF
      ENDIF
*     on augmente le groupe de 1 et on refait le test
      IF ((NBVPCA.LT.NBVPDS).AND.(P1+1.LT.P)) GOTO 310
*
*     En fonction des resultats des tests ...
      IF (NIVIMP.GE.1.AND.IMPFCH.GT.0) THEN
         NBW=P/10+1
         PW=1
         WRITE (IMPFCH,12001) 'Valeurs propres '//INVERS
         DO 360 I=1,NBW
            WRITE(IMPFCH,12002) (VP(K),K=PW,MIN(P,PW+9))
            IF (PW.LT.SOULIX.AND.SOULIG(PW)(1:1).EQ.'-') 
     &      WRITE(IMPFCH,12010) (SOULIG(K),K=PW,MIN(NBVPCA,PW+9))
            PW  = PW+10
360      CONTINUE
         WRITE (IMPFCH,12001) 'Bornes d''erreur'
         WRITE (IMPFCH,12002) (EPSVP(I),I=1,P)
      ENDIF
*
*     -- si on a obtenu ce qu'on cherchait...
      IF (NBVPCA.GE.NBVPDS) RETURN
*     -- si on ne peut pas accepter le dernier groupe de vect. propres
      IF (P2.EQ.P-1) THEN
         DIST = (ABS(VP(P2))-ABS(VP(P))+EPSVP(P))/ABS(VP(P))
         IF (DIST.LT.EPSIL*SAUT) IERR = 2
      ENDIF
*-----------------------------------------------------------------------
12001 FORMAT (T6,A,' : ')
12002 FORMAT ((T6,10(' ',G11.5),:))
12003 FORMAT (T6,'Precision Vecteurs',I3,' a ',I3,' : ',G11.5
     &,' ( Epsil=',G11.5,', NbVpca=',I3,', Iteration',I4,' )')
12004 FORMAT (T6,'Precision Vecteur',I3,' : ',G11.5
     &,' ( Epsil=',G11.5,', NbVpca=',I3,', Iteration',I4,' )')
12010 FORMAT(T6,10A12,:)
      END
