      SUBROUTINE INIPRJ (XMIN,XMAX,YMIN,YMAX,ZMIN,ZMAX,VALPRJ)

C- Fonction : Calcul des parametres necessaires a la projection.
C- 	Inspire de `pgdemo8.f` une demo fournie dans la distribution
C-		plplot-5.2
C-			From: Philip Palmer <philip@maths.qmw.ac.uk>
C-			Date: Mon, 26 Nov 90 12:27:22 GMT
C-
C-			This program plots a 3d surface using PGSURF.
C-
C- Entree/sortie :
C-    VALPRJ : Tableau contenant les parametres de projection 3D en 2D
C-
C- Entree :
C-    XMIN,XMAX,YMIN,YMAX,ZMIN,ZMAX : Min et max des X,Y
C-    VALPRJ(1) = PHI : angle de projection horizontal
C-    VALPRJ(2) = THETA : angle de projection vertical 
C-    VALPRJ(3) = ZOOM : agrandissement / reduction de l'objet 
C-
C- Sortie :
C-		VALPRJ(4) = Cosinus Phi  
C-		VALPRJ(5) = Sinus Phi
C-		VALPRJ(6) = Cosinus Theta
C-		VALPRJ(7) = Sinus Theta
C-		VALPRJ(8) = DeltaX : recentrage en X de l'objet sur le viewport
C-		VALPRJ(9) = DeltaY : recentrage en Y de l'objet sur le viewport
C-
C- Local
C-		DEG2RD : degres en radians
C-    XC,YC,ZC : isocentre de [X-Y-Z]
C-    XP,YP : projection 2D de [XC,YC,ZC]

C- Derniere modif : 24 FEV 2003

      REAL XMIN,XMAX,YMIN,YMAX,ZMIN,ZMAX
		REAL VALPRJ(*),DEG2RD,XC,YC,ZC,XP,YP

      PARAMETER (DEG2RD = 0.0174533)

      VALPRJ(4) = COS(VALPRJ(1) * DEG2RD)
      VALPRJ(5) = SIN(VALPRJ(1) * DEG2RD)
      VALPRJ(6) = COS(VALPRJ(2) * DEG2RD)
      VALPRJ(7) = SIN(VALPRJ(2) * DEG2RD)
      
C- On calcule la projection du milieu de faon  calculer
C- un DELTAX et DELTAY permettant de centrer le dessin
      XC = (XMIN + XMAX) / 2
      YC = (YMIN + YMAX) / 2
      ZC = (ZMIN + ZMAX) / 2
      
C- On initialise VALPRJ(8),VALPRJ(9)  0.0      
      VALPRJ(8) = 0.0
      VALPRJ(9) = 0.0
      CALL PROJEC (XC,YC,ZC,VALPRJ,XP,YP)
      VALPRJ(8) = XC - XP
      VALPRJ(9) = YC - YP

      END
