*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE NUMDOM (NDMDOM,NDMDOE,NBELDO,NXLIST,ADDOMK,NUDOMK
     &                  ,NBNECO,NVNUMC,NBNELI,NVNUML,NIVALL,IMPFCH)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin & O.deBayser (Avril 1988) 
* Derniere modification : D.Martin (1 Decembre 1999)
* Version 1.0.0
*  
* Creation des tableaux de numerotation des noeuds et d.l. d'un domaine
* de calcul (procedures NUNEDO et NUDLDO)
* et des rangs des noeuds locaux dans la numerotation des noeuds d'un domaine
* de calcul (procedure RGNEDO)
*  
*-- Arguments d'entree --
* NDMDOM dimension (des constituants) du domaine geometrique
* NDMDOE dimension des elements supports des constituants du domaine geometrique
* NBELDO nombre d'elements (de couples element-face) du domaine
* NXLIST adresse pour l'exploitation de la structure #ISTEL
* ADDOMK adresse relative dans #TERDO pour le domaine en cours
* IMPFCH numero d'unite logique d'impression
*-- Arguments d'entree/sortie
* NVNUMC niveau de la numerotation en colonne des noeuds du domaine
* NVNUML niveau de la numerotation en ligne des noeuds du domaine
* NIVALL niveau commun des structures de donnees du domaine
*-- Arguments de sortie --
* NBNECO nombre de noeuds du domaine pour l'interpolation en colonne
* NBNELI nombre de noeuds du domaine pour l'interpolation en ligne
*  
*-- Structures remplies --
*--> #GNEDO <-- qui contient les numerotations des noeuds en colonne ou ligne
*    ======
*   Adresse  |   Interpretation
*  
*   1        | IDLHRM indicateur d'interpolation d'Hermite
*   2        | NBNECO (nombre de noeuds en colonne ou ligne sur le domaine)
*   3=IANECO | NGNECO(1)      (numero du premier noeud en colonne ou ligne)
*   ........ |  ...... 
*   NBNECO+2 | NGNECO(NBNECO) (numero du dernier noeud en colonne ou ligne)
*  
*               Puis en cas d'interpolation d'Hermite en colonne ou ligne :
*   NBNECO+3 | NBDLCO (nombre de d.l. de l'interpolation en colonne ou ligne) 
*   NBNECO+4 | NUCONE(1) (numero du 1er d.l. du 1er noeud en colonne ou ligne)
*   ........ |  ...... 
* 2*NBNECO+3 | NUCONE(NBNECO) (num. du 1er d.l. du dernier noeud en colonne)  
*  
*--> #RGNDO <-- qui contient IRGNEC (et eventuellement IRGNEL), tableau(x) des 
*    ======    rangs des noeuds en colonne (en ligne) dans la numerotation 
*              globale des noeuds de l'interpolation en colonne (en ligne)
*
*   Adresse  |    Interpretation
*   1        | adresse des tableaux IRGNEL pour tous les elements  
*              (=2 si coincidence des numerotations en colonne et ligne)
*   2        | nombre de noeuds en colonne pour le 1er element (NBNEUC)
*   3        | 'IRGNEC(I,L)' Rang du noeud I de l'element L
*   .....
*   3+NBNEUC | Nombre de noeuds en colonne pour le 2eme element
*      etc.    jusqu'au dernier element du domaine 
*  
*    Ensuite si les numerotations des noeuds en ligne et colonne different 
*    la meme structure se repete pour les 'IRGNEL' en igne a partir de 
*    l'adresse figurant dans la premiere case du tableau .
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      INTEGER  NDMDOM,NDMDOE,NBELDO,NXLIST,ADDOMK,NUDOMK,NBNECO,NVNUMC
     &        ,NBNELI,NVNUML,NIVALL,IMPFCH
*
      INCLUDE 'ALLOC'
      INCLUDE 'CONTEX'
*
      INTEGER       KLNIVE
      INTEGER       NUDOG,IGEOME,INDMIX,NUSCHQ,INDEXC,INDEXL
     &             ,IDLHRC,INTRPC,NBNECX,NBNEEX,NBDLEX,NBNEFX
     &             ,NBDLFX,KVNUMC
     &             ,IDLHRL,INTRPL,NBNELX,NBNEEY,NBDLEY,NBNEFY
     &             ,NBDLFY,KVNUML,KBTRDO,NIVIMP
      INTEGER       ADDLDA,ADNEDU,ADIRGD,ADNDLI,LGNUFA,MCNUFA
      INTEGER       LGGNDO,MCTRDO,MCMDOM,MCDOMK,MCLIST,MCBLOK,MCNEEL
      INTEGER       LGNECO,LGNELI,LGNEDO,MCNEDO,LGREEL,LGGOCO,LGGOLI
     &             ,MCGOCO,MCGOLI,INUTI1,INUTI2
      INTEGER       NBDLLX
      CHARACTER     ERCODE*120 
      COMMON/FORMAH/ERCODE 
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*NumDom*')
*
      ERCODE(1:1)='!'
      CALL TBRR1  (ERCODE,'#RGNDO',NUDOMK,LGGNDO)
*
      IF (ERCODE(1:1).NE.'C') THEN
*
*        Si la structure #RGNDO existe, on est deja passe par la ...
*
         CALL TBRR1  (ERCODE,'#GNEDO',NVNUMC,MCNEDO)
         CALL TBAR1  (ERCODE,'#GNEDO',NVNUMC,MCNEDO)
         NBNECO=IST(MCNEDO+1)  
         NBNELI=NBNECO 
         IF (NVNUML.NE.NVNUMC) THEN
            CALL TBRR1  (ERCODE,'#GNEDO',NVNUML,MCNEDO)
            CALL TBAR1  (ERCODE,'#GNEDO',NVNUML,MCNEDO)
            NBNELI=IST(MCNEDO+1)  
         ENDIF
*
      ELSE
*
         IF (NIVALL.LE.0) NIVALL=KLNIVE ()                              !sdexplo
*
*        Sinon on la cree; attention les structures de numerotations
*        #GNEDO pour le domaine en cours peuvent avoir ete heritees
*        d'un autre domaine de calcul sur le meme domaine geometrique
*
         CALL TBAR1  (ERCODE,'#TERDO',1,MCTRDO)
         MCDOMK=MCTRDO+ADDOMK
         CALL GETDOM (IST(MCDOMK),NUDOG,NUDOMK,IGEOME,INDMIX,NUSCHQ
     &               ,INDEXC,IDLHRC,INTRPC,NBNECX,NBNEEX,NBDLEX,NBNEFX
     &               ,NBDLFX,KVNUMC
     &               ,INDEXL,IDLHRL,INTRPL,NBNELX,NBNEEY,NBDLEY,NBNEFY
     &               ,NBDLFY,KVNUML,INUTI1,INUTI2,KBTRDO,NIVIMP)        !sdexplo
*  
*        Calcul des adresses relatives des tableaux de numerotations locales
*        et de la longueur du tableau de travail les contenant
*        (tableaux locaux NUNEFA, NUDLFA utilises dans la procedure NUNEDO, 
*        et NEDUDL, IRGNDL, NDLINN utilises dans la procedure NUDLDO)
*
         ADDLDA=0 
         ADNEDU=0 
         IF (NDMDOM.LT.NDIM) THEN  
            ADDLDA=MAX(NBNEFX,NBNEFY)
            ADNEDU=ADDLDA 
            IF (IDLHRC.GT.0.OR.IDLHRL.GT.0) 
     &         ADNEDU=ADDLDA+MAX(NBDLFX,NBDLFY) 
         ENDIF
         ADIRGD=ADNEDU
         ADNDLI=ADNEDU
         LGNUFA=ADNEDU
         IF (IDLHRC.GT.0.OR.IDLHRL.GT.0) THEN 
            ADIRGD=ADNEDU+MAX(NBDLEX,NBDLEY)
            ADNDLI=ADIRGD+MAX(NBDLEX,NBDLEY) 
            LGNUFA=ADNDLI+MAX(NBNEEX,NBNEEY)  
         ENDIF
         IF (LGNUFA.GT.0) 
     &      CALL TBCREE ('$WORK$',0,1,LGNUFA,'c')
*
         MCBLOK=1
         MCNUFA=1
*  
*        Creation du tableau des rangs des noeuds du domaine par element
*
         LGGOLI=0
         LGGNDO=NBNECX+1  
         IF (INDEXL.NE.INDEXC.OR.INTRPC.NE.INTRPL)
     &      LGGNDO=LGGNDO+NBNELX+1  
         LGGNDO=NBELDO*LGGNDO+1
         CALL TBCREE ('#RGNDO',NUDOMK,1,LGGNDO,'c')
*
*        Tableau de numerotation globale pour l'interpolation en colonne
*
         CALL TBRR1  (ERCODE,'#GNEEL',INTRPC,MCNEEL)
         NBDLLX=NBNEEX
         IF (INTRPC.EQ.NIVENG) NBDLLX=NBPTEX
*  
*        Tableau de la numerotation en noeuds en colonne du domaine
*
         LGNECO=0
         IF (NVNUMC.NE.NDFNUM) THEN
            CALL TBRR1  (ERCODE,'#GNEDO',NVNUMC,MCNEDO)
*
            IF (LGNUFA.GT.0) CALL TBAR1  (ERCODE,'$WORK$',0,MCNUFA)
            IF (NBBBLK.GE.2) CALL TBAR1  (ERCODE,'#LOKEL',1,MCBLOK)
            CALL TBAR4  (ERCODE,'#OMDOM',1,MCMDOM
     &                         ,'#GNEDO',NVNUMC,MCNEDO
     &                         ,'#ISTEL',NUDOG ,MCLIST
     &                         ,'#GNEEL',INTRPC,MCNEEL)
            NBNECO=IST(MCNEDO+1)
         ELSE
            NVNUMC=NIVALL
            LGNEDO=NBNECX*NBELDO+2
            IF (IDLHRC.GT.0) LGNEDO=LGNEDO+(NBNECX*NBELDO+1)
            CALL ENCLER (NUDOMK,ERCODE(1:2))                            !utilite
            CALL TBCRSU ('#GNEDO',NVNUMC,1,LGNEDO,LGREEL,'c')
*
            IF (LGNUFA.GT.0) CALL TBAR1  (ERCODE,'$WORK$',0,MCNUFA)
            IF (NBBBLK.GE.2) CALL TBAR1  (ERCODE,'#LOKEL',1,MCBLOK)
            CALL TBAR4  (ERCODE,'#OMDOM',1,MCMDOM
     &                         ,'#GNEDO',NVNUMC,MCNEDO
     &                         ,'#ISTEL',NUDOG ,MCLIST
     &                         ,'#GNEEL',INTRPC,MCNEEL)
            LGNEDO=LGREEL
            LGNECO=LGNEDO-2
            IF (IDLHRC.GT.0) LGNECO=(LGNEDO-1)/2-1
            NBNECO=0
            CALL NUNEDO (NUDOMK,NDMDOM,NDMDOE,NBELDO,INDEXC,INTRPC
     &                  ,IST(MCBLOK),IST(MCLIST+NXLIST),NBDLLX
     &                  ,IST(MCNEEL),LGNECO,IST(MCNUFA)
     &                  ,IST(MCNUFA+ADDLDA),NBNECO,IST(MCNEDO+2)
     &                  ,0,IMPFCH)                                      !caldom
            LGNECO=NBNECO+2
            IST(MCNEDO)=IDLHRC
            IST(MCNEDO+1)=NBNECO 
            IF (IDLHRC.GT.0) THEN
*
               CALL NUDLDO (NDMDOM,NDMDOE,NBELDO,INDEXC,INTRPC
     &                     ,IST(MCBLOK),IST(MCLIST+NXLIST),NBDLLX
     &                     ,IST(MCNEEL),IST(MCNUFA),IST(MCNUFA+ADDLDA)
     &                     ,IST(MCNUFA+ADNEDU),IST(MCNUFA+ADIRGD)
     &                     ,IST(MCNUFA+ADNDLI),NBNECO,IST(MCNEDO+2)
     &                     ,0,IMPFCH)                                   !caldom
               LGNECO=LGNECO+1+NBNECO
            ENDIF
            IF (NIVIMP.GT.0)  
     &      CALL PRNEDO ('*Numdom*','colonne',IST(MCNEDO),IMPFCH)       !prsd
         ENDIF
*  
*        Tableau des rangs des noeuds du domaine par element pour 
*        l'interpolation en colonne
*
         CALL TBAR1  (ERCODE,'#RGNDO',NUDOMK,MCGOCO)
         IST(MCGOCO)=2
         LGGOCO=LGGNDO
         CALL RGNEDO (NUDOMK,NDMDOM,NDMDOE,NBELDO,INDEXC,INTRPC
     &               ,IST(MCBLOK),IST(MCLIST+NXLIST),NBDLLX,IST(MCNEEL)
     &               ,IST(MCNUFA),IST(MCNUFA+ADDLDA),NBNECO
     &               ,IST(MCNEDO+2),IST(MCGOCO+1),LGGOCO,0,IMPFCH)      !caldom
         IF (NIVIMP.GT.0)  CALL PRRGDO (' ','colonne',NBELDO
     &                                 ,IST(MCGOCO+1),NIVIMP,IMPFCH)    !prsd
         IF (LGNECO.GT.0) CALL TBAJST ('#GNEDO',NVNUMC,LGNECO)
         CALL TBSAVE ('#GNEEL',INTRPC)
*
         LGNELI=0
         IF (INDEXL.EQ.INDEXC.AND.INTRPL.EQ.INTRPC) THEN
            NVNUML=NVNUMC
            NBNELI=NBNECO
         ELSE
*
*           Tableau de numerotation globale pour l'interpolation en ligne
*
            CALL TBRR1  (ERCODE,'#GNEEL',INTRPL,MCNEEL)
            NBDLLX=NBNEEY
            IF (INTRPL.EQ.NIVENG) NBDLLX=NBPTEX
*  
*           Tableau de la numerotation en noeuds en ligne du domaine
*
            IF (NVNUML.NE.NDFNUM) THEN
               CALL TBRR1  (ERCODE,'#GNEDO',NVNUML,MCNEDO)
*
               IF (LGNUFA.GT.0) CALL TBAR1  (ERCODE,'$WORK$',0,MCNUFA)
               IF (NBBBLK.GE.2) CALL TBAR1  (ERCODE,'#LOKEL',1,MCBLOK)
               CALL TBAR4  (ERCODE,'#OMDOM',1,MCMDOM
     &                            ,'#GNEDO',NVNUML,MCNEDO
     &                            ,'#ISTEL',NUDOG ,MCLIST
     &                            ,'#GNEEL',INTRPL,MCNEEL)
               NBNELI=IST(MCNEDO+1)
            ELSE
               NIVALL=KLNIVE ()                                         !sdexplo
               NVNUML=NIVALL 
               LGNEDO=NBNELX*NBELDO+2
               IF (IDLHRL.GT.0) LGNEDO=LGNEDO+NBNELX*NBELDO+1 
               CALL ENCLER (NUDOMK,ERCODE(1:2))                         !utilite
               CALL TBCRSU ('#GNEDO',NVNUML,1,LGNEDO,LGREEL,'c')
*
               IF (LGNUFA.GT.0) CALL TBAR1  (ERCODE,'$WORK$',0,MCNUFA)
               IF (NBBBLK.GE.2) CALL TBAR1  (ERCODE,'#LOKEL',1,MCBLOK)
               CALL TBAR4  (ERCODE,'#OMDOM',1,MCMDOM
     &                            ,'#GNEDO',NVNUML,MCNEDO
     &                            ,'#ISTEL',NUDOG ,MCLIST
     &                            ,'#GNEEL',INTRPL,MCNEEL)
               LGNEDO=LGREEL
               LGNELI=LGNEDO-2
               IF (IDLHRL.GT.0) LGNELI=(LGNEDO-1)/2-1
               NBNELI=0 
               CALL NUNEDO (NUDOMK,NDMDOM,NDMDOE,NBELDO,INDEXL,INTRPL
     &                     ,IST(MCBLOK)
     &                     ,IST(MCLIST+NXLIST),NBDLLX,IST(MCNEEL),LGNELI
     &                     ,IST(MCNUFA),IST(MCNUFA+ADDLDA),NBNELI
     &                     ,IST(MCNEDO+2),0,IMPFCH)                     !caldom
               LGNELI=2+NBNELI
               IST(MCNEDO)=IDLHRL
               IST(MCNEDO+1)=NBNELI 
               IF (IDLHRL.GT.0) THEN 
*
                  CALL NUDLDO (NDMDOM,NDMDOE,NBELDO,INDEXL,INTRPL
     &                        ,IST(MCBLOK),IST(MCLIST+NXLIST),NBDLLX
     &                        ,IST(MCNEEL),IST(MCNUFA)
     &                        ,IST(MCNUFA+ADDLDA),IST(MCNUFA+ADNEDU)
     &                        ,IST(MCNUFA+ADIRGD),IST(MCNUFA+ADNDLI)
     &                        ,NBNELI,IST(MCNEDO+2),0,IMPFCH)           !caldom
                  LGNELI=LGNELI+1+NBNELI
               ENDIF
               IF (NIVIMP.GT.0) 
     &         CALL PRNEDO ('*Numdom*','ligne',IST(MCNEDO),IMPFCH)      !prsd
            ENDIF
*
            CALL TBAR1  (ERCODE,'#RGNDO',NUDOMK,MCGOCO)
            LGGOLI=LGGNDO-LGGOCO+1
            IST(MCGOCO)=LGGOCO+1
            MCGOLI=MCGOCO+LGGOCO-1
*
            CALL RGNEDO (NUDOMK,NDMDOM,NDMDOE,NBELDO,INDEXL,INTRPL
     &                  ,IST(MCBLOK),IST(MCLIST+NXLIST),NBDLLX
     &                  ,IST(MCNEEL),IST(MCNUFA),IST(MCNUFA+ADDLDA)
     &                  ,NBNELI,IST(MCNEDO+2),IST(MCGOLI+1)
     &                  ,LGGOLI,0,IMPFCH)                               !caldom
            LGGOLI=LGGOLI-1
            IF (NIVIMP.GT.0) CALL PRRGDO (' ','ligne',NBELDO 
     &                                   ,IST(MCGOLI+1),NIVIMP,IMPFCH)  !prsd
            IF (LGNELI.GT.0) CALL TBAJST ('#GNEDO',NVNUML,LGNELI)
            CALL TBSAVE ('#GNEEL',INTRPL)
         ENDIF
*  
         IF (LGGNDO.GT.LGGOCO+LGGOLI)
     &      CALL TBAJST ('#RGNDO',NUDOMK,LGGOCO+LGGOLI)
         IF (LGNUFA.GT.0) CALL TBTUER ('$WORK$',0)
      ENDIF
*
      CALL PRFXMJ (-1,'*NumDom*')
*-----------------------------------------------------------------------  
                                                                    END !Numdom
