*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE EXSTEL (NDMDOM,NDMDOE,NXLIST,LISTEL,BLOKEL,NUTGEO
     &                  ,INTERP,NUEL  ,NUTYAV,NUTELT,LAGHER,KCLASS
     &                  ,NBNEEL,NBDLEL,NUF   ,NUFAAV,NUTYFA
     &                  ,NBNEFA,NBDLFA,NUNEFA,NUDLFA,NIVIMP,IMPFCH)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteurs : D.Martin & O.DeBayser (Mai 1988)
* Derniere modification : D.Martin (05 juillet 2005)
* Version 2
*
* Exploitation des tableaux LISTEL & BLOKEL pour trouver les attributs
* d'un constituant d'un domaine (Numero d'element, de face, de type ...)
*  
*-- Arguments d'entree -- 
* NDMDOM dimension (des constituants du) domaine geometrique
* NDMDOE dimension des elements supports des constituants du domaine
* LISTEL tableau des numeros d'elements du domaine si NDMDOM=NDMDOE
*        tableau des couples (No d'element,No de face ou d'arete), sinon  
* BLOKEL tableau des numeros de type des elements du maillage par blocs
* INTERP indice d'interpolation des elements  
* NUTYAV numero de type de l'element precedent
* NUFAAV numero de face ou de l'arete de l'element precedent (le cas echeant)
* NIVIMP parametre d'impression
* IMPFCH numero d'unite logique du fichier d'impression des messages
*-- Arguments d'entree modifie en sortie --
* NXLIST rang dans LISTEL de l'element ou du couple (element,face) precedent
*-- Arguments de sortie --  
* NUTGEO numero de type geometrique de l'element en cours
* NUEL   numero de l'element dans le maillage 
* NUTELT numero de type de l'element en cours
* LAGHER indicateur d'interpolation d'Hermite
* KCLASS classe de l'element (regularite de l'element : classe C^0, C^1, ...)
* NBNEEL nombre de noeuds de l'element
* NBDLEL nombre de degre de liberte de l'element  
* NUF    numero de face ou de l'arete de l'element courant (le cas echeant)  
* NUTYFA numero de type de la face
* NBNEFA nombre de noeuds sur la face (le cas echeant, par defaut = NBNEEL)
* NBDLFA nombre de D.L. sur la face   ( - - - - - - - - - - - - - = NBDLEL)
* NUNEFA tableau de correspondance entre la numerotation locale sur la face
*        ou l'arete et la numerotation locale dans l'element (le cas echeant)
* NUDLFA idem pour les d.l. (elements d'Hermite)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      IMPLICIT NONE
      INTEGER NDMDOM,NDMDOE,NXLIST,NUTGEO,INTERP,NUEL  ,NUTYAV,NUTELT
     &       ,LAGHER,KCLASS,NBNEEL,NBDLEL,NUF   ,NUFAAV,NUTYFA,NBNEFA
     &       ,NBDLFA,NIVIMP,IMPFCH
      INTEGER BLOKEL(*),LISTEL(*),NUNEFA(*),NUDLFA(*)
*
      CHARACTER PRFXAS*8
      INTEGER   NBSOMT,NBARET,NBFACE,NBNEAX,NBDLAX,NBNEFX,NBDLFX
*-----------------------------------------------------------------------
      CALL PRFXMJ (1,'*Exstel*')
*
*     Numero de l'element
      NXLIST=NXLIST+1
      NUEL=LISTEL(NXLIST)
      NUF=0
      IF (NDMDOM.LT.NDMDOE) THEN
*        Numero de face/arete/point de dimension NDMDOM dans les elements
*        de dimension NDMDOE
         NXLIST=NXLIST+1
         NUF=LISTEL(NXLIST)
      ENDIF
*     Type de l'element de numero NUEL
      CALL EXBLOK (NUEL,BLOKEL,NUTGEO)                                  !sdexplo
      CALL PRFXMJ (-1,'*Exstel*')  
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      ENTRY EXSTEW (NDMDOM,INTERP,NUTGEO,NUEL,NUTYAV,NUTELT
     &             ,LAGHER,KCLASS,NBNEEL,NBDLEL,NUF ,NUFAAV,NUTYFA
     &             ,NBNEFA,NBDLFA,NUNEFA,NUDLFA,NIVIMP,IMPFCH)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      CALL PRFXMJ (1,'*Exstel*')
*
      NBNEAX=0
      NBDLAX=0
      NBNEFX=0
      NBDLFX=0
      NUTELT=INTERP+100*(NUTGEO/100)
*
      IF (NUTYAV.NE.NUTELT) THEN
*
         CALL ELEMNB (NUTELT,LAGHER,KCLASS,NBSOMT,NBARET,NBFACE
     &               ,NBNEEL,NBDLEL,NBNEAX,NBDLAX,NBNEFX,NBDLFX)        !ef3d
         NBNEFA=NBNEEL
         NBDLFA=NBDLEL
         NUTYFA=NUTELT
*
         IF (NIVIMP.GT.0)
     &   WRITE (IMPFCH,10000) PRFXAS(2),PRFXAS(1),PRFXAS(0)
     &                       ,' Element',NUEL,NUTELT,NBNEEL,NBDLEL
      ENDIF
*
*     Attributs de la face de dimension NDMDOM (le cas echeant)
*
      IF (NUF.NE.0.AND.(NUTYAV.NE.NUTELT.OR.NUFAAV.NE.NUF)) THEN
*
         CALL ELEMFA (NUTELT,NDMDOM,NUF,NUTYFA,NBNEFA,NUNEFA
     &               ,NBDLFA,NUDLFA)                                    !ef3d
*
         IF (NIVIMP.GT.0) THEN
            IF (NDMDOM.EQ.2) THEN
               WRITE(IMPFCH,10000) ' ',PRFXAS(1),PRFXAS(0)
     &                            ,'    Face',NUF,NUTYFA,NBNEFA,NBDLFA
            ELSEIF (NDMDOM.EQ.1) THEN
               WRITE(IMPFCH,10000) ' ',PRFXAS(1),PRFXAS(0)
     &                            ,'   Arete',NUF,NUTYFA,NBNEFA,NBDLFA
            ELSEIF (NDMDOM.EQ.0) THEN
               WRITE(IMPFCH,10000) ' ',PRFXAS(1),PRFXAS(0)
     &                            ,'   Point',NUF,NUTYFA,NBNEFA,NBDLFA
            ENDIF
         ENDIF
*
      ENDIF
      CALL PRFXMJ (-1,'*Exstel*')
*-----------------------------------------------------------------------
10000 FORMAT (T2,A8,2('>',A8),A,:,I6,', Type ',I5,', Nb. noeuds '
     &,I3,', Nb. d.l.   ',I3)
                                                                    END !exstel
