*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      SUBROUTINE EXBMOR (MCINIT,BMORSE
     &                  ,NBLIGN,NBPLAL,MCPLAL,MCPLIL,LGMORL
     &                  ,NBCOLO,NBPLAU,MCPLAU,MCPLIU,LGMORU)
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
* Auteur : D.Martin (Octobre 1991)
* Derniere modification : D.Martin (14 Octobre 1991)
*  
* Exploitation d'une structure &BMORS : Cette procedure fournit
* les nombre de lignes, de plages en ligne, la longueur de la
* partie triangulaire inferieure Morse par ligne et les adresses
* relatives ou absolues des pointeurs de compactage morse ligne
* et les memes parametres pour le compactage morse colonne de la
* partie triangulaire superieure Morse par colonne
*  
*-- Argument d'entree --  
* MCINIT adresse de la structure &BMORS (absolue si on cherche
*          des adresses absolues , egale a 1 pour des adresses relatives.
* BMORSE structure &BMORS a son adresse initiale.
*
*-- Arguments de sortie --
* NBLIGN nombre de lignes
* NBPLAL nombre de plages en ligne pour la partie triangulaire inferieure
* MCPLAL adresse absolue ou relative du tab. des numero de derniere plage
*         pour la partie triangulaire inferieure
* MCPLIL idem pour les tableaux des numeros de debut de plage et adresses 
*         des coeff. de fin de plage
* LCMORL nombre de coefficients de la partie triangulaire inferieure
* NBCOLO,NBPLAU,MCPLAU,MCPLIU,LGMORU: idem pour la part. triang. superieure
*
*-- Structure &BMORS -- 
*  
* Adresse  -> Contenu 
*  
* 1        -> Adresse de NBCOLO (nombre de colonne) dans BMORSE
* 2        -> NBLIGN nombre de lignes 
* 3        -> MCPLAL adrs. du tab. des numeros de dernieres plages en ligne
* 2+NBLIGN -> MCPLIL adrs. du tab. des numeros de debut de plage et des
*                    adresses des coeff. de fin des plages en ligne
* 2+NBLIGN+2*NBPLAL-1 -> longueur de la partie triangulaire inferieure
*                     = adresse du dernier coeff. de la derniere plage en ligne
* Puis en cas de structure non symetrique (BMORSE(1) > 2)
* 2+NBLIGN+2*NBPLAL   -> NBCOLO nombre de colonnes
* 3+NBLIGN+2+NBPLAL   -> MCPLAU adrs. du tab. des numeros de dernieres plages
*                               en colonne et adresses des coeff. de fin des
*                               plages en colonne
* 3+NBLIGN+2*NBPLAL+NBCOLO-1 -> MCPLIU adrs. du tab. des numeros de debut de
*                               plage en colonne et adresses des coeff. de fin  
*                               des plages en colonne
* 2+NBLIGN+2*NBPLAL+NBCOLO+2*NBPLAU-1 -> longueur de la partie triang. sup.
*                   = adresse du dernier coeff. de la derniere plage en colonne
*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      INTEGER MCINIT,BMORSE(*),NBLIGN,NBPLAL,MCPLAL,MCPLIL,LGMORL
     &                        ,NBCOLO,NBPLAU,MCPLAU,MCPLIU,LGMORU
*
      INTEGER IANEXT
*-----------------------------------------------------------------------
*     Partie triangulaire inferieure
      NBLIGN=BMORSE(2)
      MCPLAL=3+MCINIT-1  
      MCPLIL=2+NBLIGN
      NBPLAL=BMORSE(MCPLIL-1)
      IANEXT=MCPLIL+2*NBPLAL-2
      MCPLIL=MCPLIL+MCINIT-1
      LGMORL=BMORSE(IANEXT+1)
*
      IF (BMORSE(1).EQ.2) IANEXT=0
*
*     Partie triangulaire superieure
      NBCOLO=BMORSE(2+IANEXT) 
      MCPLAU=3+MCINIT-1+IANEXT
      MCPLIU=2+NBCOLO+IANEXT
      NBPLAU=BMORSE(MCPLIU-1)
      IANEXT=MCPLIU+2*NBPLAU-2
      MCPLIU=MCPLIU+MCINIT-1 
      LGMORU=BMORSE(IANEXT+1)
                                                                    END !ExBMor
